/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2015 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/******************************************************************************
* File: decode_encode_sample.c
*     Decode and encode a JPEG image.
*
* - $Rev: 35 $
* - $Date: 2017-08-30T17:43:26+09:00 $
* - Device(s)    : RZ/A1H, RZ/A1M, RZ/A1LU
* - Tool-Chain   : -
* - OS           : -
* - H/W Platform : Genmai board
* - Description  : -
* - Operation    : None.
* - Limitations  : None.
******************************************************************************/
/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include  <string.h>
#include  "r_typedefs.h"
#include  "clib_drivers.h"
#include  "r_jcu_api.h"
#include  "jcu_sample.h"
#include  "Image/JCU_SampleImage.h"
#include  "VRAM.h"
#ifndef  R_OSPL_NDEBUG
#include  <stdio.h>  /* printf */
#endif


/******************************************************************************
Macro definitions
******************************************************************************/
#define  OUTPUT_PIXEL_FORMAT          (JCU_OUTPUT_YCbCr422)
/* JCU_OUTPUT_RGB565(=2), JCU_OUTPUT_ARGB8888(=1), JCU_OUTPUT_YCbCr422(=0) */
#define  FRAME_SIZE_MUL               (4u)  /* ARGB8888 is 4, not RGB565 is 2 */
#define  FRAME_WIDTH                  (800u)
#define  FRAME_HEIGHT                 (480u)
#define  ALPHA_VAL_MAX                (0xFF)
#define  NUM_FF_JPEG_HEADER_LETTER_1  (0xFFu)
#define  NUM_D8_JPEG_HEADER_LETTER_2  (0xD8u)
#define  QUANTIZATION_TABLE_SIZE      (64u)
#define  HUFFMAN_TABLE_DC_SIZE        (28u)
#define  HUFFMAN_TABLE_AC_SIZE        (178u)
#define  TEMPORARY_MEM_MUL            (2u)


/******************************************************************************
Typedef definitions
******************************************************************************/

/***********************************************************************
* Structure: sample_config_t
************************************************************************/
typedef struct st_sample_config_t  sample_config_t;
struct st_sample_config_t
{
    jcu_decode_format_t  output_pixel_format;
    /* JCU_OUTPUT_RGB565(=2), JCU_OUTPUT_ARGB8888(=1), JCU_OUTPUT_YCbCr422(=0) */

    const uint8_t*  address_of_image;
};


/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/

/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/

/******************************************************************************
Private global variables and functions
******************************************************************************/

/***********************************************************************
* Function: R_JCU_SampleDecodeEncode
*    Decodes and encode JPEG data.
*
* Arguments:
*    argc - The argument of main function. Ignored.
*    argv - The argument of main function. Ignored.
*
* Returns:
*    Error Code. 0=No Error.
************************************************************************/
errnum_t  R_JCU_SampleDecodeEncode( int32_t const  argc,  char_t * const* const  argv )
{
    errnum_t                  e;
    jcu_errorcode_t           jcu_error;
    jcu_decode_param_t        decode;
    jcu_buffer_param_t        buffer;
    bool_t                    is_JCU_initialized = false;
    uintptr_t                 physical_address;
    uint8_t*                  memory_address;
    size_t                    frame_size;
    size_t                    memory_size;
    uintptr_t                 physical_address_of_RAW;
    uintptr_t                 physical_address_of_JPEG;
    uint8_t*                  memory_address_of_JPEG;
    size_t const              size_of_image = sizeof(g_jcu_sample_image);
    void*                     rv;
    sample_config_t           config;

    /* encode test */
    jcu_encode_param_t        encode;
    size_t                    encoded_size;
    const uint32_t            Encode_Width      = 800u;
    const uint32_t            Encode_Height     = 480u;

    /*[QuantizationTable_Y]*/
    /* Quality is IJG75 calculated by "QuantizationTable_Generator.xls" */
    /* ->SEC M1.10.1 Table data. */
    const uint8_t  QuantizationTable_Y[QUANTIZATION_TABLE_SIZE] =
    {
        8, 6, 5, 8, 12, 20, 26, 31,
        6, 6, 7, 10, 13, 29, 30, 28,
        7, 7, 8, 12, 20, 29, 35, 28,
        7, 9, 11, 15, 26, 44, 40, 31,
        9, 11, 19, 28, 34, 55, 52, 39,
        12, 18, 28, 32, 41, 52, 57, 46,
        25, 32, 39, 44, 52, 61, 60, 51,
        36, 46, 48, 49, 56, 50, 52, 50
    };
    /* <-SEC M1.10.1 */

    /*[QuantizationTable_C]*/
    /* Quality is IJG75 calculated by "QuantizationTable_Generator.xls" */
    /* ->SEC M1.10.1 Table data. */
    const uint8_t  QuantizationTable_C[QUANTIZATION_TABLE_SIZE] =
    {
        9, 9, 12, 24, 50, 50, 50, 50,
        9, 11, 13, 33, 50, 50, 50, 50,
        12, 13, 28, 50, 50, 50, 50, 50,
        24, 33, 50, 50, 50, 50, 50, 50,
        50, 50, 50, 50, 50, 50, 50, 50,
        50, 50, 50, 50, 50, 50, 50, 50,
        50, 50, 50, 50, 50, 50, 50, 50,
        50, 50, 50, 50, 50, 50, 50, 50
    };
    /* <-SEC M1.10.1 */

    /*[HuffmanTable_Y_DC]*/
    /* Example written in ITU-T T81 specification */
    /* ->SEC M1.10.1 Table data. */
    const uint8_t  HuffmanTable_Y_DC[HUFFMAN_TABLE_DC_SIZE] =
    {
        0x00, 0x01, 0x05, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A, 0x0B
    };
    /* <-SEC M1.10.1 */


    /*[HuffmanTable_C_DC]*/
    /* Example written in ITU-T T81 specification */
    /* ->SEC M1.10.1 Table data. */
    const uint8_t  HuffmanTable_C_DC[HUFFMAN_TABLE_DC_SIZE] =
    {
        0x00, 0x03, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A, 0x0B
    };
    /* <-SEC M1.10.1 */


    /*[HuffmanTable_Y_AC]*/
    /* Example written in ITU-T T81 specification */
    /* ->SEC M1.10.1 Table data. */
    const uint8_t  HuffmanTable_Y_AC[HUFFMAN_TABLE_AC_SIZE] =
    {
        0x00, 0x02, 0x01, 0x03, 0x03, 0x02, 0x04, 0x03, 0x05, 0x05, 0x04, 0x04, 0x00, 0x00, 0x01, 0x7D,
        0x01, 0x02, 0x03, 0x00, 0x04, 0x11, 0x05, 0x12, 0x21, 0x31, 0x41, 0x06, 0x13, 0x51, 0x61, 0x07,
        0x22, 0x71, 0x14, 0x32, 0x81, 0x91, 0xA1, 0x08, 0x23, 0x42, 0xB1, 0xC1, 0x15, 0x52, 0xD1, 0xF0,
        0x24, 0x33, 0x62, 0x72, 0x82, 0x09, 0x0A, 0x16, 0x17, 0x18, 0x19, 0x1A, 0x25, 0x26, 0x27, 0x28,
        0x29, 0x2A, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3A, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48, 0x49,
        0x4A, 0x53, 0x54, 0x55, 0x56, 0x57, 0x58, 0x59, 0x5A, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68, 0x69,
        0x6A, 0x73, 0x74, 0x75, 0x76, 0x77, 0x78, 0x79, 0x7A, 0x83, 0x84, 0x85, 0x86, 0x87, 0x88, 0x89,
        0x8A, 0x92, 0x93, 0x94, 0x95, 0x96, 0x97, 0x98, 0x99, 0x9A, 0xA2, 0xA3, 0xA4, 0xA5, 0xA6, 0xA7,
        0xA8, 0xA9, 0xAA, 0xB2, 0xB3, 0xB4, 0xB5, 0xB6, 0xB7, 0xB8, 0xB9, 0xBA, 0xC2, 0xC3, 0xC4, 0xC5,
        0xC6, 0xC7, 0xC8, 0xC9, 0xCA, 0xD2, 0xD3, 0xD4, 0xD5, 0xD6, 0xD7, 0xD8, 0xD9, 0xDA, 0xE1, 0xE2,
        0xE3, 0xE4, 0xE5, 0xE6, 0xE7, 0xE8, 0xE9, 0xEA, 0xF1, 0xF2, 0xF3, 0xF4, 0xF5, 0xF6, 0xF7, 0xF8,
        0xF9, 0xFA
    };
    /* <-SEC M1.10.1 */


    /*[HuffmanTable_C_AC]*/
    /* Example written in ITU-T T81 specification */
    /* ->SEC M1.10.1 Table data. */
    const uint8_t  HuffmanTable_C_AC[HUFFMAN_TABLE_AC_SIZE] =
    {
        0x00, 0x02, 0x01, 0x02, 0x04, 0x04, 0x03, 0x04, 0x07, 0x05, 0x04, 0x04, 0x00, 0x01, 0x02, 0x77,
        0x00, 0x01, 0x02, 0x03, 0x11, 0x04, 0x05, 0x21, 0x31, 0x06, 0x12, 0x41, 0x51, 0x07, 0x61, 0x71,
        0x13, 0x22, 0x32, 0x81, 0x08, 0x14, 0x42, 0x91, 0xA1, 0xB1, 0xC1, 0x09, 0x23, 0x33, 0x52, 0xF0,
        0x15, 0x62, 0x72, 0xD1, 0x0A, 0x16, 0x24, 0x34, 0xE1, 0x25, 0xF1, 0x17, 0x18, 0x19, 0x1A, 0x26,
        0x27, 0x28, 0x29, 0x2A, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3A, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48,
        0x49, 0x4A, 0x53, 0x54, 0x55, 0x56, 0x57, 0x58, 0x59, 0x5A, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68,
        0x69, 0x6A, 0x73, 0x74, 0x75, 0x76, 0x77, 0x78, 0x79, 0x7A, 0x82, 0x83, 0x84, 0x85, 0x86, 0x87,
        0x88, 0x89, 0x8A, 0x92, 0x93, 0x94, 0x95, 0x96, 0x97, 0x98, 0x99, 0x9A, 0xA2, 0xA3, 0xA4, 0xA5,
        0xA6, 0xA7, 0xA8, 0xA9, 0xAA, 0xB2, 0xB3, 0xB4, 0xB5, 0xB6, 0xB7, 0xB8, 0xB9, 0xBA, 0xC2, 0xC3,
        0xC4, 0xC5, 0xC6, 0xC7, 0xC8, 0xC9, 0xCA, 0xD2, 0xD3, 0xD4, 0xD5, 0xD6, 0xD7, 0xD8, 0xD9, 0xDA,
        0xE2, 0xE3, 0xE4, 0xE5, 0xE6, 0xE7, 0xE8, 0xE9, 0xEA, 0xF2, 0xF3, 0xF4, 0xF5, 0xF6, 0xF7, 0xF8,
        0xF9, 0xFA
    };
    /* <-SEC M1.10.1 */


    R_UNREFERENCED_VARIABLE_2( argc, argv );

    config.output_pixel_format = OUTPUT_PIXEL_FORMAT;
    config.address_of_image = g_jcu_sample_image;

    IF ( ((uint32_t)(config.address_of_image[0]) != NUM_FF_JPEG_HEADER_LETTER_1 ) ||
         ((uint32_t)(config.address_of_image[1]) != NUM_D8_JPEG_HEADER_LETTER_2) )
    {
        e = E_OTHERS;  /* JPEG data is not in ROM */
        goto  fin;
    }

    e= R_SAMPLE_GetVRAM( &memory_address, &memory_size );
    IF(e!=0)
    {
        goto fin;
    }
    e= R_OSPL_ToPhysicalAddress( memory_address, &physical_address );
    IF(e!=0)
    {
        goto fin;
    }


    frame_size = FRAME_WIDTH * FRAME_SIZE_MUL * FRAME_HEIGHT;

    physical_address_of_RAW = R_Ceil_8u( physical_address );

    physical_address_of_JPEG = R_Ceil_8u( physical_address_of_RAW + frame_size );
    /* ->MISRA 11.3 */ /* ->SEC R2.7.1 */
    memory_address_of_JPEG = (void*) R_Ceil_8u( (uintptr_t)memory_address + frame_size );
    /* <-MISRA 11.3 */ /* <-SEC R2.7.1 */
    IF ( ( memory_size - frame_size ) < size_of_image )
    {
        e=E_OTHERS;
        goto fin;
    }
    rv= memcpy( memory_address_of_JPEG, config.address_of_image, size_of_image );
    R_UNREFERENCED_VARIABLE( rv );  /* QAC 3200 : This is not error information */

    R_OSPL_MEMORY_Flush( R_OSPL_FLUSH_WRITEBACK_INVALIDATE );


    /* JPEG decode */
    jcu_error = R_JCU_Initialize( NULL );
    IF ( jcu_error != JCU_ERROR_OK )
    {
        e=E_OTHERS;
        goto fin;
    }
    is_JCU_initialized = true;

    jcu_error = R_JCU_SelectCodec( JCU_DECODE );
    IF ( jcu_error != JCU_ERROR_OK )
    {
        e=E_OTHERS;
        goto fin;
    }

    buffer.source.swapSetting       = JCU_SWAP_JPEG_NONE;
    /* ->MISRA 11.3 */ /* ->MISRA 11.4 */ /* ->SEC R2.7.1 */
    buffer.source.address           = (uint32_t*) physical_address_of_JPEG;
    buffer.destination.address      = (uint32_t*) physical_address_of_RAW;
    /* <-MISRA 11.3 */ /* <-MISRA 11.4 */ /* <-SEC R2.7.1 */
    buffer.lineOffset               = (int16_t) FRAME_WIDTH;
    decode.decodeFormat             = config.output_pixel_format;

    if ( config.output_pixel_format == JCU_OUTPUT_YCbCr422 )
    {
        buffer.destination.swapSetting  = JCU_SWAP_CB_Y0_CR_Y1_PIXEL0123;
        decode.outputCbCrOffset = JCU_CBCR_OFFSET_128;
    }
    else if ( config.output_pixel_format == JCU_OUTPUT_RGB565 )
    {
        buffer.destination.swapSetting  = JCU_SWAP_RGB565_PIXEL0123;
        decode.outputCbCrOffset = JCU_CBCR_OFFSET_0;
    }
    else
    {
        IF ( config.output_pixel_format != JCU_OUTPUT_ARGB8888 )
        {
            e=E_OTHERS;
            goto fin;
        }
        buffer.destination.swapSetting  = JCU_SWAP_ARGB8888_PIXEL01;
        decode.outputCbCrOffset = JCU_CBCR_OFFSET_0;
    }

    decode.alpha                    = ALPHA_VAL_MAX;
    decode.horizontalSubSampling    = JCU_SUB_SAMPLING_1_1;
    decode.verticalSubSampling      = JCU_SUB_SAMPLING_1_1;
    jcu_error = R_JCU_SetDecodeParam( &decode, &buffer );
    IF ( jcu_error != JCU_ERROR_OK )
    {
        e=E_OTHERS;
        goto fin;
    }
    jcu_error = R_JCU_Start();
    IF ( jcu_error != JCU_ERROR_OK )
    {
        e=E_OTHERS;
        goto fin;
    }


    /* Start to encode */

    jcu_error = R_JCU_SelectCodec( JCU_ENCODE );
    IF ( jcu_error != JCU_ERROR_OK )
    {
        e=E_OTHERS;
        goto fin;
    }

    /* Set tables */
    jcu_error = R_JCU_SetQuantizationTable( JCU_TABLE_NO_0, QuantizationTable_Y );
    IF ( jcu_error != JCU_ERROR_OK )
    {
        e=E_OTHERS;
        goto fin;
    }
    jcu_error = R_JCU_SetQuantizationTable( JCU_TABLE_NO_1, QuantizationTable_C );
    IF ( jcu_error != JCU_ERROR_OK )
    {
        e=E_OTHERS;
        goto fin;
    }
    jcu_error = R_JCU_SetHuffmanTable( JCU_TABLE_NO_0, JCU_HUFFMAN_DC, HuffmanTable_Y_DC );
    IF ( jcu_error != JCU_ERROR_OK )
    {
        e=E_OTHERS;
        goto fin;
    }
    jcu_error = R_JCU_SetHuffmanTable( JCU_TABLE_NO_1, JCU_HUFFMAN_DC, HuffmanTable_C_DC );
    IF ( jcu_error != JCU_ERROR_OK )
    {
        e=E_OTHERS;
        goto fin;
    }
    jcu_error = R_JCU_SetHuffmanTable( JCU_TABLE_NO_0, JCU_HUFFMAN_AC, HuffmanTable_Y_AC );
    IF ( jcu_error != JCU_ERROR_OK )
    {
        e=E_OTHERS;
        goto fin;
    }
    jcu_error = R_JCU_SetHuffmanTable( JCU_TABLE_NO_1, JCU_HUFFMAN_AC, HuffmanTable_C_AC );
    IF ( jcu_error != JCU_ERROR_OK )
    {
        e=E_OTHERS;
        goto fin;
    }

    /* JPEG encode */
    buffer.source.swapSetting       = JCU_SWAP_CB_Y0_CR_Y1_PIXEL0123;
    /* ->MISRA 11.3 */ /* ->MISRA 11.4 */ /* ->SEC R2.7.1 */
    buffer.source.address           = (uint32_t*) physical_address_of_RAW;
    buffer.destination.swapSetting  = JCU_SWAP_JPEG_NONE;
    /* ->QAC 3305 : "memory_address_of_JPEG" is aligned by "R_Ceil_8u" function */
    buffer.destination.address      = (uint32_t*) memory_address_of_JPEG;
    /* <-QAC 3305 */
    /* <-MISRA 11.3 */ /* <-MISRA 11.4 */ /* <-SEC R2.7.1 */
    buffer.lineOffset               = (int16_t) FRAME_WIDTH;
    encode.encodeFormat = (jcu_jpeg_format_t) JCU_JPEG_YCbCr422;
    encode.QuantizationTable[ JCU_ELEMENT_Y  ] = JCU_TABLE_NO_0;
    encode.QuantizationTable[ JCU_ELEMENT_Cb ] = JCU_TABLE_NO_1;
    encode.QuantizationTable[ JCU_ELEMENT_Cr ] = JCU_TABLE_NO_1;
    encode.HuffmanTable[ JCU_ELEMENT_Y  ] = JCU_TABLE_NO_0;
    encode.HuffmanTable[ JCU_ELEMENT_Cb ] = JCU_TABLE_NO_1;
    encode.HuffmanTable[ JCU_ELEMENT_Cr ] = JCU_TABLE_NO_1;
    encode.DRI_value = 0;
    encode.width  = Encode_Width;
    encode.height = Encode_Height;
    encode.inputCbCrOffset = JCU_CBCR_OFFSET_128;
    jcu_error = R_JCU_SetEncodeParam( &encode, &buffer );
    IF ( jcu_error != JCU_ERROR_OK )
    {
        e=E_OTHERS;
        goto fin;
    }

    jcu_error = R_JCU_Start();
    IF ( jcu_error != JCU_ERROR_OK )
    {
        e=E_OTHERS;
        goto fin;
    }

    jcu_error = R_JCU_GetEncodedSize( &encoded_size );
    IF ( jcu_error != JCU_ERROR_OK )
    {
        e=E_OTHERS;
        goto fin;
    }

#ifndef R_OSPL_NDEBUG
    printf( "<DecodedRAW address=\"0x%08X\"/>\n", physical_address_of_RAW );
    printf( "<EncodedJPEG address=\"0x%08X\" size=\"0x%08X\"/>\n", memory_address_of_JPEG, encoded_size );
#endif

    e = 0;
fin:
    if ( is_JCU_initialized != false )
    {
        jcu_error = R_JCU_Terminate();
        IF ( (jcu_error != JCU_ERROR_OK) && (e == 0) )
        {
            e = E_OTHERS;
        }
    }
    R_DEBUG_BREAK_IF_ERROR();
    return  e;
}

