/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2015 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/******************************************************************************
* File: decode_sample.c
*     Decode a JPEG image.
*
* - $Rev: 35 $
* - $Date: 2017-08-30T17:41:32+09:00 $
* - Device(s)    : RZ/A1H, RZ/A1M, RZ/A1LU
* - Tool-Chain   : -
* - OS           : -
* - H/W Platform : Genmai board
* - Description  : -
* - Operation    : None.
* - Limitations  : None.
******************************************************************************/
/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include  <string.h>
#include  "r_typedefs.h"
#include  "clib_drivers.h"
#include  "r_jcu_api.h"
#include  "jcu_sample.h"
#include  "Image/JCU_SampleImage.h"
#include  "VRAM.h"
#ifndef  R_OSPL_NDEBUG
#include  <stdio.h>  /* printf */
#endif


/******************************************************************************
Macro definitions
******************************************************************************/
#define  OUTPUT_PIXEL_FORMAT          (JCU_OUTPUT_ARGB8888)
/* JCU_OUTPUT_RGB565(=2), JCU_OUTPUT_ARGB8888(=1), JCU_OUTPUT_YCbCr422(=0) */
#define  FRAME_SIZE_MUL               (4u)  /* ARGB8888 is 4, not RGB565 is 2 */
#define  FRAME_WIDTH                  (800u)
#define  FRAME_HEIGHT                 (480u)
#define  ALPHA_VAL_MAX                (0xFF)
#define  NUM_FF_JPEG_HEADER_LETTER_1  (0xFFu)
#define  NUM_D8_JPEG_HEADER_LETTER_2  (0xD8u)


/******************************************************************************
Typedef definitions
******************************************************************************/

/***********************************************************************
* Structure: sample_config_t
************************************************************************/
typedef struct st_sample_config_t  sample_config_t;
struct st_sample_config_t
{
    jcu_decode_format_t  output_pixel_format;
    /* JCU_OUTPUT_RGB565(=2), JCU_OUTPUT_ARGB8888(=1), JCU_OUTPUT_YCbCr422(=0) */

    const uint8_t*  address_of_image;
};


/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/

/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/

/******************************************************************************
Private global variables and functions
******************************************************************************/

/***********************************************************************
* Function: R_JCU_SampleDecode
*    Decodes JPEG data to a memory.
*
* Arguments:
*    argc - The argument of main function. Ignored.
*    argv - The argument of main function. Ignored.
*
* Returns:
*    Error Code. 0=No Error.
************************************************************************/
errnum_t  R_JCU_SampleDecode( int32_t const  argc, char_t * const* const  argv )
{
    errnum_t                  e;
    jcu_errorcode_t           jcu_error;
    jcu_decode_param_t        decode;
    jcu_buffer_param_t        buffer;
    bool_t                    is_JCU_initialized = false;
    uintptr_t                 physical_address;
    uint8_t*                  memory_address;
    size_t                    frame_size;
    size_t                    memory_size;
    uintptr_t                 physical_address_of_RAW;
    uintptr_t                 physical_address_of_JPEG;
    uint8_t*                  memory_address_of_JPEG;
    size_t const              size_of_image = sizeof(g_jcu_sample_image);
    void*                     rv;
    sample_config_t           config;


    R_UNREFERENCED_VARIABLE_2( argc, argv );

    config.output_pixel_format = OUTPUT_PIXEL_FORMAT;
    config.address_of_image = g_jcu_sample_image;

    IF ( ((uint32_t)(config.address_of_image[0]) != NUM_FF_JPEG_HEADER_LETTER_1 ) ||
         ((uint32_t)(config.address_of_image[1]) != NUM_D8_JPEG_HEADER_LETTER_2) )
    {
        e = E_OTHERS;  /* JPEG data is not in ROM */
        goto  fin;
    }

    e= R_SAMPLE_GetVRAM( &memory_address, &memory_size );
    IF(e!=0)
    {
        goto fin;
    }
    e= R_OSPL_ToPhysicalAddress( memory_address, &physical_address );
    IF(e!=0)
    {
        goto fin;
    }


    frame_size = FRAME_WIDTH * FRAME_SIZE_MUL * FRAME_HEIGHT;

    physical_address_of_RAW = R_Ceil_8u( physical_address );

    physical_address_of_JPEG = R_Ceil_8u( physical_address_of_RAW + frame_size );
    /* ->MISRA 11.3 */ /* ->SEC R2.7.1 */
    memory_address_of_JPEG = (void*) R_Ceil_8u( (uintptr_t)memory_address + frame_size );
    /* <-MISRA 11.3 */ /* <-SEC R2.7.1 */
    IF ( ( memory_size - frame_size ) < size_of_image )
    {
        e=E_OTHERS;
        goto fin;
    }
    rv= memcpy( memory_address_of_JPEG, config.address_of_image, size_of_image );
    R_UNREFERENCED_VARIABLE( rv );  /* QAC 3200 : This is not error information */

    R_OSPL_MEMORY_Flush( R_OSPL_FLUSH_WRITEBACK_INVALIDATE );


    /* JPEG decode */
    jcu_error = R_JCU_Initialize( NULL );
    IF ( jcu_error != JCU_ERROR_OK )
    {
        e=E_OTHERS;
        goto fin;
    }
    is_JCU_initialized = true;

    jcu_error = R_JCU_SelectCodec( JCU_DECODE );
    IF ( jcu_error != JCU_ERROR_OK )
    {
        e=E_OTHERS;
        goto fin;
    }

    buffer.source.swapSetting       = JCU_SWAP_JPEG_NONE;
    /* ->MISRA 11.3 */ /* ->MISRA 11.4 */ /* ->SEC R2.7.1 */
    buffer.source.address           = (uint32_t*) physical_address_of_JPEG;
    buffer.destination.address      = (uint32_t*) physical_address_of_RAW;
    /* <-MISRA 11.3 */ /* <-MISRA 11.4 */ /* <-SEC R2.7.1 */
    buffer.lineOffset               = (int16_t) FRAME_WIDTH;
    decode.decodeFormat             = config.output_pixel_format;

    if ( config.output_pixel_format == JCU_OUTPUT_YCbCr422 )
    {
        buffer.destination.swapSetting  = JCU_SWAP_CB_Y0_CR_Y1_PIXEL0123;
        decode.outputCbCrOffset = JCU_CBCR_OFFSET_128;
    }
    else if ( config.output_pixel_format == JCU_OUTPUT_RGB565 )
    {
        buffer.destination.swapSetting  = JCU_SWAP_RGB565_PIXEL0123;
        decode.outputCbCrOffset = JCU_CBCR_OFFSET_0;
    }
    else
    {
        IF ( config.output_pixel_format != JCU_OUTPUT_ARGB8888 )
        {
            e=E_OTHERS;
            goto fin;
        }
        buffer.destination.swapSetting  = JCU_SWAP_ARGB8888_PIXEL01;
        decode.outputCbCrOffset = JCU_CBCR_OFFSET_0;
    }

    decode.alpha                    = ALPHA_VAL_MAX;
    decode.horizontalSubSampling    = JCU_SUB_SAMPLING_1_1;
    decode.verticalSubSampling      = JCU_SUB_SAMPLING_1_1;
    jcu_error = R_JCU_SetDecodeParam( &decode, &buffer );
    IF ( jcu_error != JCU_ERROR_OK )
    {
        e=E_OTHERS;
        goto fin;
    }
    jcu_error = R_JCU_Start();
    IF ( jcu_error != JCU_ERROR_OK )
    {
        e=E_OTHERS;
        goto fin;
    }

#ifndef R_OSPL_NDEBUG
    printf( "<DecodedRAW address=\"0x%08X\"/>\n", physical_address_of_RAW );
#endif

    e = 0;
fin:
    if ( is_JCU_initialized != false )
    {
        jcu_error = R_JCU_Terminate();
        IF ( (jcu_error != JCU_ERROR_OK) && (e == 0) )
        {
            e = E_OTHERS;
        }
    }
    R_DEBUG_BREAK_IF_ERROR();
    return  e;
}

