/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2014 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/

/*******************************************************************************
*   File Name : display_sample.c
*        $Rev :                            $
*       $Date :                           #$
* Description : Display driver sample
*             :  Base source code is graphics_sample.c [version: 1.00]
******************************************************************************/
/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include    <stdio.h>
#include    <string.h>

#include    "cmsis_os.h"
#include    "r_typedefs.h"
#include    "Renesas_RZ_A1.h"
#include    "gic.h"

#include    "iodefine.h"
#include    "display_if.h"
#include    "displaywrapper_if.h"
#include    "lcd_if.h"


/******************************************************************************
Macro definitions
******************************************************************************/
/* VDC5_CHANNEL_0 or VDC5_CHANNEL_1 */
#ifndef __RZA1L
/* setup GRAPHICS_CHANNEL Channel */
#if (LCD_BOARD == LCD_BOARD_LCDKITB01)

#if (LCD_KITB01_CH == LCD_KITB01_CH_0)
#define GRAPHICS_CHANNEL      (VDC5_CHANNEL_0)
#elif (LCD_KITB01_CH == LCD_KITB01_CH_1)
#define GRAPHICS_CHANNEL      (VDC5_CHANNEL_1)
#else
#error LCD_KITB01_CH is incorrect setting
#endif /* LCD_KITB01_CH */

#elif (LCD_BOARD == LCD_BOARD_TITSC2007)
#define GRAPHICS_CHANNEL     VDC5_CHANNEL_1

#else
#error LCD_BOARD is incorrect setting
#endif /* LCD_BOARD */

#else
#define     GRAPHICS_CHANNEL            (VDC5_CHANNEL_0)

#endif

/*! Graphics format */
#define     GRAPHICS_FORMAT             (VDC5_GR_FORMAT_RGB565)
#define     MAIN_GRAPHICS_FORMAT_SIZE   (2u)

/*! Frame buffer stride: Frame buffer stride should be set to a multiple of 32 or 128
    in accordance with the frame buffer burst transfer mode. */
#ifndef __RZA1L
/* VDC5_CHANNEL_0 or VDC5_CHANNEL_1 */
#if (LCD_KITB01_CH == LCD_KITB01_CH_0)
#define     MAIN_FRAME_BUFFER_STRIDE    (((LCD_CH0_DISP_HW * MAIN_GRAPHICS_FORMAT_SIZE) + 31u) & ~31u)
#define     MAIN_FRAME_BUFFER_HEIGHT    (LCD_CH0_DISP_VW)
#else
#define     MAIN_FRAME_BUFFER_STRIDE    (((LCD_CH1_DISP_HW * MAIN_GRAPHICS_FORMAT_SIZE) + 31u) & ~31u)
#define     MAIN_FRAME_BUFFER_HEIGHT    (LCD_CH1_DISP_VW)
#endif
#else
#define     MAIN_FRAME_BUFFER_STRIDE    (((LCD_CH0_DISP_HW * MAIN_GRAPHICS_FORMAT_SIZE) + 31u) & ~31u)
#define     MAIN_FRAME_BUFFER_HEIGHT    (LCD_CH0_DISP_VW)
#define     SUB_FRAME_BUFFER_STRIDE     (((LCD_CH0_DISP_HW * SUB_GRAPHICS_FORMAT_SIZE) + 31u) & ~31u)
#endif

#define     VSYNC_WAIT_COUNT            (240)


/******************************************************************************
Typedef definitions
******************************************************************************/
typedef struct
{
    void      * buffer_1;
    void      * buffer_2;
    uint32_t    backbuffer_num;
} buffering_t;


/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/

/******************************************************************************
Exported global variables (to be accessed by other files)
******************************************************************************/

/******************************************************************************
Private global variables and functions
******************************************************************************/
/*! Interrupt parameter */
static vdc5_int_t           interrupt;
/*! Read data parameter */
static vdc5_read_chg_t      read_chg;
/*! Graphics display mode */
static vdc5_gr_disp_sel_t   gr_disp_sel[VDC5_GR_TYPE_NUM];
/*! Display area */
static vdc5_period_rect_t   period_rect_main;

static volatile int32_t vsync_count = VSYNC_WAIT_COUNT;

static buffering_t buffering_inf;

#define  IS_DISPLAY_SAMPLE_VRAM_USED  0  /* 1 or 0 */
#if IS_DISPLAY_SAMPLE_VRAM_USED
#define __CHANGE_FLAME 1
#ifdef      __CC_ARM
#pragma arm section zidata = "VRAM"
    static uint8_t FrameBuffer_1[MAIN_FRAME_BUFFER_STRIDE * MAIN_FRAME_BUFFER_HEIGHT];
#ifdef __CHANGE_FLAME
    static uint8_t FrameBuffer_2[MAIN_FRAME_BUFFER_STRIDE * MAIN_FRAME_BUFFER_HEIGHT];
#endif
#pragma arm section zidata
#else
    static uint8_t FrameBuffer_1[MAIN_FRAME_BUFFER_STRIDE * MAIN_FRAME_BUFFER_HEIGHT];
    static uint8_t FrameBuffer_2[MAIN_FRAME_BUFFER_STRIDE * MAIN_FRAME_BUFFER_HEIGHT];
#endif
#else
    static uint8_t FrameBuffer_1[1];
#endif

static void InitParameters (vdc5_channel_t ch);
static void SetInterrupt (vdc5_channel_t ch);
static void IntCallbackFunc (vdc5_int_type_t int_type);
static void WaitVsync (void);
static void SetGraphicsData (vdc5_gr_format_t gr_format, void  * fr_buff, uint16_t width, uint16_t height, uint32_t stride);
static void CreateFrameBuffer (void);
static void *GetBackBuffer (void);
static void SwapFrameBuffer (void);
static void Termination (vdc5_channel_t ch);

osSemaphoreDef( vdc5_vsync );
osSemaphoreId   vdc5_vsync;
/**************************************************************************//**
 * @brief       Initialize variables
 * @param[in]   ch                      : Channel
 * @retval      None
******************************************************************************/
static void InitParameters (const vdc5_channel_t ch)
{
    uint32_t index;

    /* Interrupt parameter */
    interrupt.type      = VDC5_INT_TYPE_VLINE;  /* Interrupt type */
    interrupt.callback  = 0;                    /* Callback function pointer */
    interrupt.line_num  = (uint16_t)0 ;         /* Line number */

    read_chg.gr_base        = NULL;         /* Frame buffer base address */
    read_chg.width_read_fb  = NULL;         /* Width of the image read from frame buffer */
    read_chg.gr_grc         = NULL;         /* Display area */
    read_chg.gr_disp_sel    = NULL;         /* Graphics display mode */

    for (index = 0; index < (uint32_t)VDC5_GR_TYPE_NUM; index++)
    {
        gr_disp_sel[index] = VDC5_DISPSEL_IGNORED;
    }

    if (ch == VDC5_CHANNEL_0)
     {
         period_rect_main.vs  = (uint16_t)LCD_CH0_DISP_VS;
         period_rect_main.vw  = (uint16_t)LCD_CH0_DISP_VW;
         period_rect_main.hs  = (uint16_t)LCD_CH0_DISP_HS;
         period_rect_main.hw  = (uint16_t)LCD_CH0_DISP_HW;
     }
     else
     {
#ifndef __RZA1L
         period_rect_main.vs  = (uint16_t)LCD_CH1_DISP_VS;
         period_rect_main.vw  = (uint16_t)LCD_CH1_DISP_VW;
         period_rect_main.hs  = (uint16_t)LCD_CH1_DISP_HS;
         period_rect_main.hw  = (uint16_t)LCD_CH1_DISP_HW;
#else
#endif
     }



}   /* End of function InitParameters() */

/**************************************************************************//**
 * @brief       Setup VDC5 interrupt
 * @param[in]   ch                      : Channel
 * @retval      None
******************************************************************************/
static void SetInterrupt (const vdc5_channel_t ch)
{

    uint32_t        unInhRet;
    VDC5_INTFUNC    func;
    IRQn_Type       int_id;

    int_id = (ch == VDC5_CHANNEL_0) ? GR3_VLINE0_IRQn : GR3_VLINE1_IRQn;

    /* disable LCD interrupt */
    GIC_DisableIRQ( int_id );

    /** define interrupt handler */
    func = AppDrv_Disp_GetInterruptFunc(ch, VDC5_INT_TYPE_VLINE);
    printf( "[DISP INT] ID = %d\n", (int_t)int_id );
    unInhRet = InterruptHandlerRegister( int_id, func );
    if( unInhRet != 0 )
    {
        printf( "[ERROR] interrupt handler is not registed\n" );
    }
    else
    {
        /* set interrupt priority */
        GIC_SetPriority( int_id, VDC5_INT_PRIORITY );

        /* enable LCD interrupt */
        GIC_EnableIRQ( int_id );
    }
}   /* End of function SetInterrupt() */

/**************************************************************************//**
 * @brief       VDC5 Interrupt callback function
 * @param[in]   int_type    : VDC5 interrupt type
 * @retval      None
******************************************************************************/
static void IntCallbackFunc (vdc5_int_type_t int_type)
{
    if (vsync_count > 0)
    {
        vsync_count--;
    }
    if(vsync_count == 0)
    {
        vsync_count = VSYNC_WAIT_COUNT;
        (void)osSemaphoreRelease( vdc5_vsync );
    }
}   /* End of function IntCallbackFunc() */
/**************************************************************************//**
 * @brief       Wait Vsync function
 * @param[in]   None
 * @retval      None
******************************************************************************/
static void WaitVsync (void)
{
    /* Wait VSync*/
    (void)osSemaphoreWait( vdc5_vsync, osWaitForever );
}   /* End of function WaitVsync() */
/**************************************************************************//**
 * @brief       Setup graphics
 * @param[in]   gr_format           : Format of the frame buffer read signal
 * @param[out]  fr_buff             : Frame buffer base address
 * @param[in]   width               : Width of the frame buffer to be read
 * @param[in]   height              : Height of the frame buffer to be read
 * @param[in]   stride              : Frame buffer line offset address (buffer stride)
 * @retval      None
******************************************************************************/
static void SetGraphicsData (
    const vdc5_gr_format_t  gr_format,
    void            * const fr_buff,
    const uint16_t          width,
    const uint16_t          height,
    const uint32_t          stride)
{
    /* Create fill color image */
    GRAPHICS_CreateFillColorImage(fr_buff, NULL, width, height, (uint16_t)stride, gr_format, (int32_t)-1);
}   /* End of function SetGraphicsData() */

/**************************************************************************//**
 * @brief       Create a frame buffer
 * @param[in]   void
 * @retval      None
******************************************************************************/
static void CreateFrameBuffer (void)
{
    buffering_inf.buffer_1          = (void *)FrameBuffer_1;
#ifdef __CHANGE_FLAME
    buffering_inf.buffer_2          = (void *)FrameBuffer_2;
#else
    buffering_inf.buffer_2          = (void *)FrameBuffer_1;
#endif
    buffering_inf.backbuffer_num    = 0u;
}   /* End of function CreateFrameBuffer() */

/**************************************************************************//**
 * @brief       Get back buffer address
 * @param[in]   void
 * @retval      Back buffer address
******************************************************************************/
static void * GetBackBuffer (void)
{
    if (buffering_inf.backbuffer_num == 0u)
    {
        return buffering_inf.buffer_1;
    }
    else
    {
        return buffering_inf.buffer_2;
    }
}   /* End of function GetBackBuffer() */

/**************************************************************************//**
 * @brief       Swap frame buffers
 * @param[in]   void
 * @retval      None
******************************************************************************/
static void SwapFrameBuffer (void)
{
    if (buffering_inf.backbuffer_num == 0u)
    {
        buffering_inf.backbuffer_num = (uint32_t)1u;
    }
    else
    {
        buffering_inf.backbuffer_num = (uint32_t)0u;
    }
}   /* End of function SwapFrameBuffer() */

/**************************************************************************//**
 * @brief       Termination
 * @param[in]   ch                      : Channel
 * @retval      None
******************************************************************************/
static void Termination (const vdc5_channel_t ch)
{
    vdc5_error_t error;

    /* Interrupt parameter */
    interrupt.callback = 0;         /* Clear callback function pointer */
    /* Set interrupt service routine */
    error = AppDrv_Disp_RegistCallbackFunc(ch, &interrupt);
    if (error != VDC5_OK)
    {
        return;
    }

    /* Stop surface */
    error = AppDrv_Disp_StopSurfaces(ch, VDC5_LAYER_ID_ALL);
    if (error != VDC5_OK)
    {
        return;
    }
    /* Destroy surface */
    error = AppDrv_Disp_DestroySurfaces(ch, VDC5_LAYER_ID_ALL);
    if (error != VDC5_OK)
    {
        return;
    }
    /* Termination */
    error = AppDrv_Disp_Term(ch);
    if (error != VDC5_OK)
    {
        return;
    }
}   /* End of function Termination() */

/**************************************************************************//**
 * @brief       Graphics display sample
 * @param[in]   void
 * @retval      None
******************************************************************************/
void Graphics_GraphicsSample (void)
{
    vdc5_error_t                error;
    void                      * framebuff;
    /* VDC5 channel (VDC5_CHANNEL_0 or VDC5_CHANNEL_1) */
    const vdc5_channel_t          ch                = GRAPHICS_CHANNEL;
    const vdc5_layer_id_t         layer_id_main     = VDC5_LAYER_ID_2_RD;
    const vdc5_graphics_type_t    gr_type_main      = VDC5_GR_TYPE_GR2;
    const vdc5_gr_format_t        gr_format_main    = GRAPHICS_FORMAT;
    int                     nLoop;

    InitParameters(ch);

    CreateFrameBuffer();

    SetInterrupt(ch);

    /* Initialization */
    error = AppDrv_Disp_Init(ch, VDC5_OFF, VDC5_OFF);
    if (error != VDC5_OK)
    {
        return;
    }

    framebuff = GetBackBuffer();
    SetGraphicsData(gr_format_main, framebuff, period_rect_main.hw, period_rect_main.vw, MAIN_FRAME_BUFFER_STRIDE);
    /* Create graphics surface */
    error = AppDrv_Disp_GraphicsCreateSurface(
                ch, layer_id_main, framebuff, MAIN_FRAME_BUFFER_STRIDE, gr_format_main,
                &period_rect_main, NULL
            );
    if (error != VDC5_OK)
    {
        return;
    }

    /* Interrupt parameter */
    interrupt.callback = IntCallbackFunc;       /* Callback function pointer */
    /* Set interrupt service routine */
    error = AppDrv_Disp_RegistCallbackFunc(ch, &interrupt);
    if (error != VDC5_OK)
    {
        return;
    }

    /* Start surface */
    gr_disp_sel[gr_type_main] = VDC5_DISPSEL_CURRENT;
    error = AppDrv_Disp_StartSurfaces(ch, layer_id_main, &gr_disp_sel[gr_type_main]);
    if (error != VDC5_OK)
    {
        return;
    }

    vdc5_vsync = osSemaphoreCreate( osSemaphore(vdc5_vsync), 0 );
    nLoop = 0;
    while (nLoop < 30)
    {
        nLoop ++;
        /* Wait Vsync */
        WaitVsync();

        SwapFrameBuffer();
        framebuff = GetBackBuffer();
        SetGraphicsData(gr_format_main, framebuff, period_rect_main.hw, period_rect_main.vw, MAIN_FRAME_BUFFER_STRIDE);
        /* Read data parameter */
        read_chg.gr_base = framebuff;       /* Frame buffer base address */
        /* Change read process */
        error = AppDrv_Disp_GraphicsChangeParam(ch, layer_id_main, &read_chg);
        if (error != VDC5_OK)
        {
            break;
        }
    }
    /* Quit */
    (void)osSemaphoreDelete(vdc5_vsync);
    Termination(ch);

}   /* End of function Graphics_GraphicsSample() */

/**************************************************************************//**
 * @brief
 * @param[in]   void
 * @retval      None
******************************************************************************/
void Graphics_VideoSample (void)
{
#ifndef __RZA1L
    GRAPHICS_VideoSample();
#else

#endif
}

