/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2017 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/******************************************************************************
* File: video_encode_decode_i_sample.c
*     Interlace video encode and decode.
*
* - $Rev: 35 $
* - $Date: 2017-08-30T18:31:26+09:00 $
* - Device(s)    : RZ/A1H, RZ/A1M, RZ/A1LU
* - Tool-Chain   : -
* - OS           : -
* - H/W Platform : Genmai board
* - Description  : -
* - Operation    : None.
* - Limitations  : None.
******************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include    <string.h>
#include    <stdio.h>
#include    "r_typedefs.h"
#include    "iodefine.h"
#include    "dev_drv.h"
#include    "r_ospl.h"
#if ! R_OSPL_IS_PREEMPTION
#include    "r_ospl_os_less.h"
#endif

#include    "r_vdc5.h"
#ifdef  USE_LIST_VDC5_PORT
#include    "r_vdc5_port.h"
#endif
#include    "graphics.h"
#include    "graphics_drv_wrapper.h"
#include    "video_decoder.h"
#include    "graphics_debug.h"
#include    "lcd_panel.h"

#include    "r_jcu_api.h"
#include    "jcu_sample.h"
#include    "clib_drivers.h"
#include    "VRAM.h"


/******************************************************************************
Macro definitions
******************************************************************************/
/*! Graphics format: ARGB8888, 4 bytes per pixel */
#define     GRAPHICS_FORMAT             (VDC5_GR_FORMAT_YCBCR422)
/*! Frame buffer stride: Frame buffer stride should be set to a multiple of 32 or 128
	in accordance with the frame buffer burst transfer mode. */
#define     FRAME_BUFFER_STRIDE         (((LCD_CH0_DISP_HW * 2u) + 31u) & ~31u)
#define     FRAME_BUFFER_HEIGHT         (LCD_CH0_DISP_VW / 2u)
#define     FRAME_BUFFER_SIZE           ( FRAME_BUFFER_STRIDE * FRAME_BUFFER_HEIGHT )

#define     VIDEO_FORMAT_TYPE           VIDEO_FORMAT_TYPE_NTSC
#define     VIDEO_FORMAT_TYPE_NTSC          1
#define     VIDEO_FORMAT_TYPE_PAL           2

#define     IMGCAP_SIZE_NTSC_HS         (122u * 2u)
#define     IMGCAP_SIZE_NTSC_HW         (720u * 2u)
#define     IMGCAP_SIZE_NTSC_VS         (16u)
#define     IMGCAP_SIZE_NTSC_VW         (480u/2u)

#define     IMGCAP_SIZE_PAL_HS          (132u * 2u)
#define     IMGCAP_SIZE_PAL_HW          (720u * 2u)
#define     IMGCAP_SIZE_PAL_VS          (19u)
#define     IMGCAP_SIZE_PAL_VW          (560u/2u)

/*! Video format: YCbCr422, 2 bytes per pixel */
#define     VIDEO_FORMAT                (VDC5_RES_MD_YCBCR422)
/*! Frame buffer stride: Frame buffer stride should be set to a multiple of 32 or 128
	in accordance with the frame buffer burst transfer mode. */
#define     VIDEO_BUFFER_STRIDE         (((LCD_CH0_DISP_HW * 2u) + 31u) & ~31u)
#if VIDEO_FORMAT_TYPE == VIDEO_FORMAT_TYPE_NTSC
#define     VIDEO_BUFFER_HEIGHT         (IMGCAP_SIZE_NTSC_VW)
#else
#define     VIDEO_BUFFER_HEIGHT         (IMGCAP_SIZE_PAL_VW)
#endif

#define     BUFFER_CNT                  (2u)
#define     LCD_CHANNEL_COUNT           (1)

#define  JPEG_HEIGHT            (( VIDEO_BUFFER_HEIGHT + 7u)&~7u)
#define  JPEG_BUFFER_SIZE       0x10000

#define  QUANTIZATION_TABLE_SIZE    64u
#define  HUFFMAN_TABLE_DC_SIZE      28u
#define  HUFFMAN_TABLE_AC_SIZE      178u

#define  EVENT_VIDEO_IN_VSYNC   0x0010
#define  EVENT_JPEG_ENCODED     0x0020
#define  EVENT_JPEG_DECODED     0x0040
#define  EVENT_VIDEO_OUT_VSYNC  0x0080
#define  EVENT_ALL              0x00F0

#ifndef  NDEBUG
#define  COMMAND_MAX_ARGLENGTH      (256)
#endif


/******************************************************************************
Typedef definitions
******************************************************************************/
typedef struct
{
    void      * buffer_1;
    void      * buffer_2;
    uint32_t    backbuffer_num;
} buffering_t;

/*[app_jcu_t]*/
typedef struct st_app_jcu_t  app_jcu_t;
struct st_app_jcu_t
{
    bool_t              IsInitialized;
    jcu_encode_param_t  JCU_Encode;
    jcu_buffer_param_t  JCU_EncodeBuffer;
    r_ospl_async_t      JCU_EncodeAsync;
    jcu_decode_param_t  JCU_Decode;
    jcu_buffer_param_t  JCU_DecodeBuffer;
    r_ospl_async_t      JCU_DecodeAsync;
};
static app_jcu_t  gs_app_jcu;


/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/

/******************************************************************************
Exported global variables (to be accessed by other files)
******************************************************************************/

/******************************************************************************
Private global variables and functions
******************************************************************************/
static void InitParameters(const vdc5_channel_t ch);
static void SetInterrupt(const vdc5_channel_t ch);
#if R_OSPL_INTERRUPT_HANDLER_IS != R_OSPL_INTERRUPT_HANDLER_IS_WITH_INT_SENSE
static void IntCallbackVector_GR3_VLINE( R_OSPL_INTERRUPT_ARGUMENTS );
static void IntCallbackVector_S0_VI_VSYNC( R_OSPL_INTERRUPT_ARGUMENTS );
static void (* gs_VDC5_ISR_GR3_VLINE)(uint32_t int_sense);
static void (* gs_VDC5_ISR_S0_VI_VSYNC)(uint32_t int_sense);
#endif
static r_ospl_thread_id_t    gs_message_loop_thread_id;
static bit_flags32_t         gs_video_in_event;
static bit_flags32_t         gs_video_out_event;
static void IntCallbackFunc(vdc5_int_type_t int_type);
static void SetVideoDecoder(const vdc5_channel_t ch, vdc5_period_rect_t * const cap_rect);
static void CreateFrameBuffer(void);
static void * GetBackBuffer(void);
static void SwapFrameBuffer(void);
static void Termination(const vdc5_channel_t ch);

static errnum_t      APP_JCU_SetUp(void);
static errnum_t      APP_JCU_Encode( uint32_t* SourceAddress );
static errnum_t      APP_JCU_Decode( uint32_t* DestinationAddress );
static void          APP_JCU_Terminate(void);

static vdc5_error_t GRPDRV_VideoCreateNonDispSurface (
    const vdc5_channel_t                ch,
    const vdc5_layer_id_t               layer_id,
    void                        * const framebuff,
    const uint32_t                      fb_stride,
    const vdc5_res_md_t                 res_md,
    const vdc5_period_rect_t    * const res,
    const uint16_t                buff_w_vw,
    const uint16_t                buff_w_hw);

static vdc5_error_t GRPDRV_GraphicsCreateSurfaceScale (
    const vdc5_channel_t                ch,
    const vdc5_layer_id_t               layer_id,
    void                        * const framebuff,
    const uint32_t                      fb_stride,
    const vdc5_gr_format_t              gr_format,
    const uint16_t                      buff_r_vw,
    const uint16_t                      buff_r_hw,
    const vdc5_period_rect_t    * const period_rect);


/*! Interrupt parameter */
static vdc5_int_t           interrupt;
/*! Read data parameter */
static vdc5_read_chg_t      read_chg;
/*! Graphics display mode */
static vdc5_gr_disp_sel_t   gr_disp_sel_tmp[VDC5_GR_TYPE_NUM];
/*! Display area */
static vdc5_period_rect_t   period_rect;

static const vdc5_channel_t  gs_VDC5_ch = VDC5_CHANNEL_0;
static void*    Videoframebuff;

static buffering_t buffering_inf;

#define  GS_IS_DECLARE_VRAM  0

#if GS_IS_DECLARE_VRAM
#ifdef  __CC_ARM
#pragma arm section zidata = "VRAM"
#endif
static uint8_t FrameBuffer_Video[BUFFER_CNT][VIDEO_BUFFER_STRIDE * VIDEO_BUFFER_HEIGHT];
static uint8_t FrameBuffer_1[FRAME_BUFFER_SIZE];
static uint8_t FrameBuffer_2[FRAME_BUFFER_SIZE];
static uint8_t JPEG_Buffer[1][JPEG_BUFFER_SIZE];
#ifdef  __CC_ARM
#pragma arm section zidata
#endif
#else
static uint8_t* FrameBuffer_Video[BUFFER_CNT];
static uint8_t* FrameBuffer_1;
static uint8_t* FrameBuffer_2;
static uint8_t* JPEG_Buffer[1];
#endif


/***********************************************************************
* Function: InitParameters
*    Initialize variables.
*
* Arguments:
*    ch - Channel
*
* Return Value:
*    None.
************************************************************************/
static void  InitParameters (const vdc5_channel_t ch)
{
    uint32_t index;

    gs_message_loop_thread_id = R_OSPL_THREAD_GetCurrentId();

    /* Interrupt parameter */
    interrupt.type      = VDC5_INT_TYPE_VLINE;  /* Interrupt type */
    interrupt.callback  = 0;                    /* Callback function pointer */
    interrupt.line_num  = (uint16_t)0 ;         /* Line number */

    read_chg.gr_base        = NULL;         /* Frame buffer base address */
    read_chg.width_read_fb  = NULL;         /* Width of the image read from frame buffer */
    read_chg.gr_grc         = NULL;         /* Display area */
    read_chg.gr_disp_sel    = NULL;         /* Graphics display mode */

    for (index = 0; index < (uint32_t)VDC5_GR_TYPE_NUM; index++)
    {
        gr_disp_sel_tmp[index] = VDC5_DISPSEL_IGNORED;
    }

    if (ch == VDC5_CHANNEL_0)
    {
        period_rect.vs  = (uint16_t)LCD_CH0_DISP_VS;
        period_rect.vw  = (uint16_t)LCD_CH0_DISP_VW;
        period_rect.hs  = (uint16_t)LCD_CH0_DISP_HS;
        period_rect.hw  = (uint16_t)LCD_CH0_DISP_HW;
    }
    else
    {
#if  LCD_CHANNEL_COUNT == 1
        GRAPHICS_DebugPrint(__FILE__, __LINE__, 1);
#else
        period_rect.vs  = (uint16_t)LCD_CH1_DISP_VS;
        period_rect.vw  = (uint16_t)LCD_CH1_DISP_VW;
        period_rect.hs  = (uint16_t)LCD_CH1_DISP_HS;
        period_rect.hw  = (uint16_t)LCD_CH1_DISP_HW;
#endif
    }

#if ! GS_IS_DECLARE_VRAM
    {
        errnum_t  e;
        uint8_t*  big_memory_address;
        size_t    big_memory_size;
        size_t    offset;
        size_t    i;

        e= R_SAMPLE_GetVRAM( &big_memory_address, &big_memory_size );
        IF(e)
        {
            goto fin;
        }

        offset = ( R_Ceil_32u( (uintptr_t) big_memory_address )
                   - (uintptr_t) big_memory_address );
        FrameBuffer_1 = big_memory_address;
        offset += FRAME_BUFFER_SIZE;
        FrameBuffer_2 = big_memory_address + offset;
        offset += FRAME_BUFFER_SIZE;
        FrameBuffer_Video[0] = big_memory_address + offset;
        offset += VIDEO_BUFFER_STRIDE * VIDEO_BUFFER_HEIGHT;
        FrameBuffer_Video[1] = big_memory_address + offset;
        offset += VIDEO_BUFFER_STRIDE * VIDEO_BUFFER_HEIGHT;
        JPEG_Buffer[0] = big_memory_address + offset;
        offset += JPEG_BUFFER_SIZE;
        IF ( offset > big_memory_size )
        {
            e=E_FEW_ARRAY;
            goto fin;
        }

        for ( i = 0;  i < offset;  i += 2 )
        {
            big_memory_address[ i + 0 ] = 0x00;
            big_memory_address[ i + 1 ] = 0x80;
        }
fin:
        R_DEBUG_BREAK_IF_ERROR();
    }
#endif
}


/***********************************************************************
* Function: SetInterrupt
*    Setup VDC5 interrupt.
*
* Arguments:
*    ch - Channel
*
* Return Value:
*    None.
************************************************************************/
static void SetInterrupt (const vdc5_channel_t ch)
{
#if R_OSPL_INTERRUPT_HANDLER_IS == R_OSPL_INTERRUPT_HANDLER_IS_WITH_INT_SENSE
    bsp_int_cb_t    func;
#endif
    bsp_int_src_t   int_id;
    errnum_t        e;
    int32_t         ret_code = DEVDRV_SUCCESS;
    bsp_int_err_t   ret_b;


    /* BSP_INT_SRC_GR3_VLINE# */
    if (ch == VDC5_CHANNEL_0)
    {
        int_id = BSP_INT_SRC_GR3_VLINE0;
    }
    else
    {
        int_id = BSP_INT_SRC_GR3_VLINE1;
    }

#if R_OSPL_INTERRUPT_HANDLER_IS == R_OSPL_INTERRUPT_HANDLER_IS_WITH_INT_SENSE
    func = R_VDC5_GetISR(ch, VDC5_INT_TYPE_VLINE);
    ret_b = R_BSP_InterruptWrite(int_id, func);
#else
    gs_VDC5_ISR_GR3_VLINE = R_VDC5_GetISR(ch, VDC5_INT_TYPE_VLINE);
    ret_b = R_BSP_InterruptWrite(int_id, IntCallbackVector_GR3_VLINE);
#endif
    if (ret_b != BSP_INT_SUCCESS)
    {
        GRAPHICS_DebugPrint(__FILE__, __LINE__, ret_b);
    }
    if (ret_b == DEVDRV_SUCCESS)
    {
        e = R_OSPL_SetInterruptPriority(int_id, VDC5_INT_PRIORITY);
        if (e != 0)
        {
            GRAPHICS_DebugPrint(__FILE__, __LINE__, e);
            ret_code = DEVDRV_ERROR;
        }
    }

    if (ret_code == DEVDRV_SUCCESS)
    {
        ret_b = R_BSP_InterruptControl(int_id, BSP_INT_CMD_INTERRUPT_ENABLE, FIT_NO_PTR);
        if (ret_b != BSP_INT_SUCCESS)
        {
            GRAPHICS_DebugPrint(__FILE__, __LINE__, ret_code);
            ret_code = DEVDRV_ERROR;
        }
    }


    /* BSP_INT_SRC_S0_VI_VSYNC# */
    if (ch == VDC5_CHANNEL_0)
    {
        int_id = BSP_INT_SRC_S0_VI_VSYNC0;
    }
    else
    {
        int_id = BSP_INT_SRC_S0_VI_VSYNC1;
    }

#if R_OSPL_INTERRUPT_HANDLER_IS == R_OSPL_INTERRUPT_HANDLER_IS_WITH_INT_SENSE
    func = R_VDC5_GetISR(ch, VDC5_INT_TYPE_S0_VI_VSYNC);
    ret_b = R_BSP_InterruptWrite(int_id, func);
#else
    gs_VDC5_ISR_S0_VI_VSYNC = R_VDC5_GetISR(ch, VDC5_INT_TYPE_S0_VI_VSYNC);
    ret_b = R_BSP_InterruptWrite(int_id, IntCallbackVector_S0_VI_VSYNC);
#endif
    if (ret_b != BSP_INT_SUCCESS)
    {
        GRAPHICS_DebugPrint(__FILE__, __LINE__, ret_b);
        ret_code = DEVDRV_ERROR;
    }

    if (ret_code == DEVDRV_SUCCESS)
    {
        e = R_OSPL_SetInterruptPriority(int_id, VDC5_INT_PRIORITY);
        if (e != 0)
        {
            GRAPHICS_DebugPrint(__FILE__, __LINE__, e);
            ret_code = DEVDRV_ERROR;
        }
    }

    if (ret_code == DEVDRV_SUCCESS)
    {
        ret_b = R_BSP_InterruptControl(int_id, BSP_INT_CMD_INTERRUPT_ENABLE, FIT_NO_PTR);
        if (ret_b != BSP_INT_SUCCESS)
        {
            GRAPHICS_DebugPrint(__FILE__, __LINE__, ret_b);
            ret_code = DEVDRV_ERROR;
        }
    }

}


/***********************************************************************
* Function: IntCallbackVector_GR3_VLINE
*    VDC5 Interrupt callback function.
*
* Arguments:
*    int_type - VDC5 interrupt type
*
* Return Value:
*    None.
************************************************************************/
#if R_OSPL_INTERRUPT_HANDLER_IS != R_OSPL_INTERRUPT_HANDLER_IS_WITH_INT_SENSE
static void  IntCallbackVector_GR3_VLINE( R_OSPL_INTERRUPT_ARGUMENTS )
{
    gs_VDC5_ISR_GR3_VLINE( 0 );
    R_OSPL_END_OF_INTERRUPT();
}
#endif


/***********************************************************************
* Function: IntCallbackVector_S0_VI_VSYNC
*    Wait for the specified number of times Vsync (VLINE) occurs.
*
* Arguments:
*    int_type - VDC5 interrupt type
*
* Return Value:
*    None.
************************************************************************/
#if R_OSPL_INTERRUPT_HANDLER_IS != R_OSPL_INTERRUPT_HANDLER_IS_WITH_INT_SENSE
static void  IntCallbackVector_S0_VI_VSYNC( R_OSPL_INTERRUPT_ARGUMENTS )
{
    gs_VDC5_ISR_S0_VI_VSYNC( 0 );
    R_OSPL_END_OF_INTERRUPT();
}
#endif


/***********************************************************************
* Function: IntCallbackFunc
*    VDC5 Interrupt callback function.
*
* Arguments:
*    int_type - VDC5 interrupt type
*
* Return Value:
*    None.
************************************************************************/
static void IntCallbackFunc (vdc5_int_type_t int_type)
{
    switch (int_type)
    {
    case VDC5_INT_TYPE_S0_VI_VSYNC:
        if ( IS_BIT_SET( VDC51.SC0_SCL1_WR7, 0x00000001u ) )
        {
            Videoframebuff = (void *)&FrameBuffer_Video[0][0];
        }
        else
        {
            Videoframebuff = (void *)&FrameBuffer_Video[1][0];
        }
        R_OSPL_EVENT_Set( gs_message_loop_thread_id,  gs_video_in_event );
        break;

    case VDC5_INT_TYPE_VLINE:
        R_OSPL_EVENT_Set( gs_message_loop_thread_id,  gs_video_out_event );
        break;

    default:
        break;
    }
}


/***********************************************************************
* Function: SetVideoDecoder
*    Setup video decoder.
*
* Arguments:
*    ch              - Channel
*    cap_rect        - (out) Image area to be captured
*
* Return Value:
*    None.
************************************************************************/
static void SetVideoDecoder (const vdc5_channel_t ch, vdc5_period_rect_t * const cap_rect)
{
    vdec_channel_t          channel;
    graphics_col_sys_t      col_sys;
    const vdec_adc_vinsel_t adc_vinsel = VDEC_ADC_VINSEL_VIN1;

    if (ch == VDC5_CHANNEL_0)
    {
        channel = VDEC_CHANNEL_0;
    }
    else
    {
        channel = VDEC_CHANNEL_1;
    }

    col_sys = GRAPHICS_VideoDecoderInit(adc_vinsel, channel);
    if (col_sys == GRPH_COL_SYS_UNKNOWN)
    {
        /* If not detected, ... */
        col_sys = GRPH_COL_SYS_NTSC_358;
    }

    if ((col_sys == GRPH_COL_SYS_NTSC_358) || (col_sys == GRPH_COL_SYS_NTSC_443))
    {
        /* Size of NTSC video image to be captured */
        cap_rect->vs = (uint16_t)IMGCAP_SIZE_NTSC_VS;
        cap_rect->vw = (uint16_t)IMGCAP_SIZE_NTSC_VW;
        cap_rect->hs = (uint16_t)IMGCAP_SIZE_NTSC_HS;
        cap_rect->hw = (uint16_t)IMGCAP_SIZE_NTSC_HW;

        if ( VIDEO_FORMAT_TYPE != VIDEO_FORMAT_TYPE_NTSC )
        {
            GRAPHICS_DebugPrint(__FILE__, __LINE__, (int32_t)VDC5_ERR_IF_CONDITION);
        }
    }
    else
    {
        /* Size of PAL video image to be captured */
        cap_rect->vs = (uint16_t)IMGCAP_SIZE_PAL_VS;
        cap_rect->vw = (uint16_t)IMGCAP_SIZE_PAL_VW;
        cap_rect->hs = (uint16_t)IMGCAP_SIZE_PAL_HS;
        cap_rect->hw = (uint16_t)IMGCAP_SIZE_PAL_HW;

        if ( VIDEO_FORMAT_TYPE != VIDEO_FORMAT_TYPE_PAL )
        {
            GRAPHICS_DebugPrint(__FILE__, __LINE__, (int32_t)VDC5_ERR_IF_CONDITION);
        }
    }
}


/***********************************************************************
* Function: CreateFrameBuffer
*    Create a frame buffer.
*
* Arguments:
*    None.
*
* Return Value:
*    None.
************************************************************************/
static void CreateFrameBuffer (void)
{
    buffering_inf.buffer_1          = (void *)FrameBuffer_1;
    buffering_inf.buffer_2          = (void *)FrameBuffer_2;
    buffering_inf.backbuffer_num    = 0u;
}


/***********************************************************************
* Function: GetBackBuffer
*    Get back buffer address.
*
* Arguments:
*    None.
*
* Return Value:
*    Back buffer address
************************************************************************/
static void * GetBackBuffer (void)
{
    return ((buffering_inf.backbuffer_num == 0u) ? buffering_inf.buffer_1 : buffering_inf.buffer_2);
}


/***********************************************************************
* Function: SwapFrameBuffer
*    Swap frame buffers.
*
* Arguments:
*    None.
*
* Return Value:
*    None.
************************************************************************/
static void SwapFrameBuffer (void)
{
    if (buffering_inf.backbuffer_num == 0u)
    {
        buffering_inf.backbuffer_num = (uint32_t)1u;
    }
    else
    {
        buffering_inf.backbuffer_num = (uint32_t)0u;
    }
}


/***********************************************************************
* Function: GRPDRV_VideoCreateNonDispSurface
*    Create video non disp surface.
*
* Arguments:
*    ch
*    layer_id
*    framebuff
*    fb_stride
*    res_md
*    res
*    period_rect
*
* Return Value:
*    Error Code, VDC5_OK=No Error
************************************************************************/
vdc5_error_t GRPDRV_VideoCreateNonDispSurface (
    const vdc5_channel_t                ch,
    const vdc5_layer_id_t               layer_id,
    void                        * const framebuff,
    const uint32_t                      fb_stride,
    const vdc5_res_md_t                 res_md,
    const vdc5_period_rect_t    * const res,
    const uint16_t                buff_w_vw,
    const uint16_t                buff_w_hw)
{
    vdc5_error_t            error;
    vdc5_write_t            write;
#if R_VDC5_VERSION >= 100  ||  ! defined( R_VDC5_VERSION )
    vdc5_scalingdown_rot_t* scldw_rot;
#else
    vdc5_sclingdown_rot_t*  scldw_rot;
#endif


    /* Scaling-down and rotation parameter */
#if R_VDC5_VERSION >= 100  ||  ! defined( R_VDC5_VERSION )
    scldw_rot = &write.scalingdown_rot;
#else
    scldw_rot = &write.sclingdown_rot;
#endif
    /* Image area to be captured */
    scldw_rot->res.vs   = (uint16_t)((uint32_t)res->vs - 1u);
    scldw_rot->res.vw   = res->vw;
    scldw_rot->res.hs   = res->hs;
    scldw_rot->res.hw   = res->hw;

    /* Write data parameter */
    scldw_rot->res_pfil_sel = VDC5_ON;              /* Prefilter mode select for brightness signals (on/off) */
    scldw_rot->res_out_vw   = buff_w_vw;            /* Number of valid lines in vertical direction
													   output by scaling-down control block */
    scldw_rot->res_out_hw   = buff_w_hw;            /* Number of valid horizontal pixels
													   output by scaling-down control block */
    scldw_rot->adj_sel      = VDC5_ON;              /* Measures to decrease the influence
													   by lack of last-input line (on/off) */
    scldw_rot->res_ds_wr_md = VDC5_WR_MD_NORMAL;    /* Frame buffer writing mode */
//    write.res_wrswa     = VDC5_WR_RD_WRSWA_NON;     /* Frame buffer swap setting */
    write.res_wrswa     = VDC5_WR_RD_WRSWA_16BIT;     /* Frame buffer swap setting */
    write.res_md        = res_md;                   /* Frame buffer video-signal writing format */
    write.res_bst_md    = VDC5_BST_MD_32BYTE;       /* Transfer burst length for frame buffer */
    write.res_inter     = VDC5_RES_INTER_INTERLACE; /* Field operating mode select */
    write.res_fs_rate   = VDC5_RES_FS_RATE_PER1;    /* Writing rate */
//    write.res_fs_rate   = VDC5_RES_FS_RATE_PER2;    /* Writing rate */
    write.res_fld_sel   = VDC5_RES_FLD_SEL_TOP;     /* Write field select */
    write.res_dth_on    = VDC5_ON;                  /* Dither correction on/off */
    write.base          = framebuff;                /* Frame buffer base address */
    write.ln_off        = fb_stride;                /* Frame buffer line offset address [byte] */
    write.flm_num       = (uint32_t)(BUFFER_CNT - 1u);/* Number of frames of buffer (res_flm_num + 1) */
    /* Frame buffer frame offset address */
    write.flm_off       = fb_stride * (uint32_t)scldw_rot->res_out_vw;
    write.btm_base      = NULL;                     /* Frame buffer base address for bottom */
    /* Write data control */
    error = R_VDC5_WriteDataControl(ch, layer_id, &write);
    if (error != VDC5_OK)
    {
        GRAPHICS_DebugPrint(__FILE__, __LINE__, (int32_t)error);
    }

    return error;
}


/***********************************************************************
* Function: GRPDRV_GraphicsCreateSurfaceScale
*    Create graphics surface scale.
*
* Arguments:
*    ch
*    layer_id
*    framebuff
*    fb_stride
*    gr_format
*    period_rect
*
* Return Value:
*    Error Code, VDC5_OK=No Error
************************************************************************/
vdc5_error_t GRPDRV_GraphicsCreateSurfaceScale (
    const vdc5_channel_t                ch,
    const vdc5_layer_id_t               layer_id,
    void                        * const framebuff,
    const uint32_t                      fb_stride,
    const vdc5_gr_format_t              gr_format,
    const uint16_t                      buff_r_vw,
    const uint16_t                      buff_r_hw,
    const vdc5_period_rect_t    * const period_rect)
{
    vdc5_error_t    error;
    vdc5_read_t     read;
    vdc5_width_read_fb_t width_read_fb;

    width_read_fb.in_hw = buff_r_hw;
    width_read_fb.in_vw = buff_r_vw;

    /* Read data parameter */
    read.gr_ln_off_dir  = VDC5_GR_LN_OFF_DIR_INC;   /* Line offset address direction of the frame buffer */
    read.gr_flm_sel     = VDC5_GR_FLM_SEL_FLM_NUM;  /* Selects a frame buffer address setting signal */
    read.gr_imr_flm_inv = VDC5_OFF;                 /* Frame buffer number for distortion correction */
    read.gr_bst_md      = VDC5_BST_MD_32BYTE;       /* Frame buffer burst transfer mode */
    read.gr_base        = framebuff;                /* Frame buffer base address */
    read.gr_ln_off      = fb_stride;                /* Frame buffer line offset address */
    read.width_read_fb  = &width_read_fb;           /* Width of the image read from frame buffer */
    read.adj_sel        = VDC5_OFF;                 /* Measures to decrease the influence
													   by folding pixels/lines (on/off) */
    read.gr_format      = gr_format;                /* Format of the frame buffer read signal */
    read.gr_ycc_swap    = VDC5_GR_YCCSWAP_CBY0CRY1; /* Controls swapping of data read from buffer
													   in the YCbCr422 format */
    read.gr_rdswa       = VDC5_WR_RD_WRSWA_16BIT;   /* Frame buffer swap setting */
    /* Display area */
    read.gr_grc.vs      = period_rect->vs;
    read.gr_grc.vw      = period_rect->vw;
    read.gr_grc.hs      = period_rect->hs;
    read.gr_grc.hw      = period_rect->hw;
    /* Read data control */
    error = R_VDC5_ReadDataControl(ch, layer_id, &read);
    if (error != VDC5_OK)
    {
        GRAPHICS_DebugPrint(__FILE__, __LINE__, (int32_t)error);
    }
    return error;
}


/***********************************************************************
* Function: APP_JCU_SetUp
*
* Arguments:
*    None.
*
* Return Value:
*    Error Code, 0=No Error
************************************************************************/
errnum_t  APP_JCU_SetUp()
{
    app_jcu_t*       self = &gs_app_jcu;
    errnum_t         e;
    jcu_errorcode_t  jcu_error;


    /* ... */
    jcu_error = R_JCU_Initialize( NULL );
    IF ( jcu_error != JCU_ERROR_OK )
    {
        e = E_OTHERS;
        goto  fin;
    }
    self->IsInitialized = true;


    /* Set JPEG encode parameters */
    self->JCU_EncodeBuffer.source.swapSetting       = JCU_SWAP_Y0_CB_Y1_CR_PIXEL2301;
    self->JCU_EncodeBuffer.source.address           = NULL;
    self->JCU_EncodeBuffer.destination.swapSetting  = JCU_SWAP_JPEG_NONE;
    self->JCU_EncodeBuffer.destination.address      = NULL;
    self->JCU_EncodeBuffer.lineOffset               = (int16_t) LCD_CH0_DISP_HW;
    self->JCU_Encode.encodeFormat = (jcu_jpeg_format_t) JCU_JPEG_YCbCr422;
    self->JCU_Encode.QuantizationTable[ JCU_ELEMENT_Y  ] = JCU_TABLE_NO_0;
    self->JCU_Encode.QuantizationTable[ JCU_ELEMENT_Cb ] = JCU_TABLE_NO_1;
    self->JCU_Encode.QuantizationTable[ JCU_ELEMENT_Cr ] = JCU_TABLE_NO_1;
    self->JCU_Encode.HuffmanTable[ JCU_ELEMENT_Y  ] = JCU_TABLE_NO_0;
    self->JCU_Encode.HuffmanTable[ JCU_ELEMENT_Cb ] = JCU_TABLE_NO_1;
    self->JCU_Encode.HuffmanTable[ JCU_ELEMENT_Cr ] = JCU_TABLE_NO_1;
    self->JCU_Encode.DRI_value = 0;
    self->JCU_Encode.width  = LCD_CH0_DISP_HW;
    self->JCU_Encode.height = JPEG_HEIGHT;
    self->JCU_Encode.inputCbCrOffset = JCU_CBCR_OFFSET_128;
    self->JCU_EncodeAsync.Flags        = R_F_OSPL_A_Thread | R_F_OSPL_A_EventValue;
    e= R_OSPL_EVENT_Allocate(
           &self->JCU_EncodeAsync.A_Thread,  gs_message_loop_thread_id,
           &self->JCU_EncodeAsync.A_EventValue,  EVENT_JPEG_ENCODED );
    IF(e)
    {
        goto fin;
    }

    /* Set JPEG decode parameters */
    self->JCU_DecodeBuffer.source.swapSetting       = JCU_SWAP_JPEG_NONE;
    self->JCU_DecodeBuffer.source.address           = NULL;
    self->JCU_DecodeBuffer.destination.address      = (uint32_t*) NULL;
    self->JCU_DecodeBuffer.lineOffset               = (int16_t) LCD_CH0_DISP_HW;
    self->JCU_DecodeBuffer.destination.swapSetting  = JCU_SWAP_Y0_CB_Y1_CR_PIXEL2301;
    self->JCU_Decode.decodeFormat             = JCU_OUTPUT_YCbCr422;
    self->JCU_Decode.outputCbCrOffset         = JCU_CBCR_OFFSET_128;
    self->JCU_Decode.alpha                    = 0xFF;
    self->JCU_Decode.horizontalSubSampling    = JCU_SUB_SAMPLING_1_1;
    self->JCU_Decode.verticalSubSampling      = JCU_SUB_SAMPLING_1_1;
    self->JCU_DecodeAsync.Flags = R_F_OSPL_A_Thread | R_F_OSPL_A_EventValue;
    e= R_OSPL_EVENT_Allocate(
           &self->JCU_DecodeAsync.A_Thread,  gs_message_loop_thread_id,
           &self->JCU_DecodeAsync.A_EventValue,  EVENT_JPEG_DECODED );
    IF(e)
    {
        goto fin;
    }

    e=0;
fin:
    return  e;
}


/***********************************************************************
* Function: APP_JCU_Encode
*
* Arguments:
*    in_SourceAddress - uint32_t*
*
* Return Value:
*    Error Code, 0=No Error
************************************************************************/
static errnum_t  APP_JCU_Encode( uint32_t*  in_SourceAddress )
{
    /* Constant: QuantizationTable_Y
    	Quality is IJG75 calculated by "QuantizationTable_Generator.xls" */
    /* ->SEC M1.10.1 Table data. */
    const uint8_t  QuantizationTable_Y[QUANTIZATION_TABLE_SIZE] =
    {
        8, 6, 5, 8, 12, 20, 26, 31,
        6, 6, 7, 10, 13, 29, 30, 28,
        7, 7, 8, 12, 20, 29, 35, 28,
        7, 9, 11, 15, 26, 44, 40, 31,
        9, 11, 19, 28, 34, 55, 52, 39,
        12, 18, 28, 32, 41, 52, 57, 46,
        25, 32, 39, 44, 52, 61, 60, 51,
        36, 46, 48, 49, 56, 50, 52, 50
    };
    /* <-SEC M1.10.1 */


    /* Constant: QuantizationTable_C
    	Quality is IJG75 calculated by "QuantizationTable_Generator.xls" */
    /* ->SEC M1.10.1 Table data. */
    const uint8_t  QuantizationTable_C[QUANTIZATION_TABLE_SIZE] =
    {
        9, 9, 12, 24, 50, 50, 50, 50,
        9, 11, 13, 33, 50, 50, 50, 50,
        12, 13, 28, 50, 50, 50, 50, 50,
        24, 33, 50, 50, 50, 50, 50, 50,
        50, 50, 50, 50, 50, 50, 50, 50,
        50, 50, 50, 50, 50, 50, 50, 50,
        50, 50, 50, 50, 50, 50, 50, 50,
        50, 50, 50, 50, 50, 50, 50, 50
    };
    /* <-SEC M1.10.1 */


    /* Constant: HuffmanTable_Y_DC
    	Example written in ITU-T T81 specification */
    /* ->SEC M1.10.1 Table data. */
    const uint8_t  HuffmanTable_Y_DC[HUFFMAN_TABLE_DC_SIZE] =
    {
        0x00, 0x01, 0x05, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A, 0x0B
    };
    /* <-SEC M1.10.1 */


    /* Constant: HuffmanTable_C_DC
    	Example written in ITU-T T81 specification */
    /* ->SEC M1.10.1 Table data. */
    const uint8_t  HuffmanTable_C_DC[HUFFMAN_TABLE_DC_SIZE] =
    {
        0x00, 0x03, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A, 0x0B
    };
    /* <-SEC M1.10.1 */


    /* Constant: HuffmanTable_Y_AC
    	Example written in ITU-T T81 specification */
    /* ->SEC M1.10.1 Table data. */
    const uint8_t  HuffmanTable_Y_AC[HUFFMAN_TABLE_AC_SIZE] =
    {
        0x00, 0x02, 0x01, 0x03, 0x03, 0x02, 0x04, 0x03, 0x05, 0x05, 0x04, 0x04, 0x00, 0x00, 0x01, 0x7D,
        0x01, 0x02, 0x03, 0x00, 0x04, 0x11, 0x05, 0x12, 0x21, 0x31, 0x41, 0x06, 0x13, 0x51, 0x61, 0x07,
        0x22, 0x71, 0x14, 0x32, 0x81, 0x91, 0xA1, 0x08, 0x23, 0x42, 0xB1, 0xC1, 0x15, 0x52, 0xD1, 0xF0,
        0x24, 0x33, 0x62, 0x72, 0x82, 0x09, 0x0A, 0x16, 0x17, 0x18, 0x19, 0x1A, 0x25, 0x26, 0x27, 0x28,
        0x29, 0x2A, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3A, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48, 0x49,
        0x4A, 0x53, 0x54, 0x55, 0x56, 0x57, 0x58, 0x59, 0x5A, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68, 0x69,
        0x6A, 0x73, 0x74, 0x75, 0x76, 0x77, 0x78, 0x79, 0x7A, 0x83, 0x84, 0x85, 0x86, 0x87, 0x88, 0x89,
        0x8A, 0x92, 0x93, 0x94, 0x95, 0x96, 0x97, 0x98, 0x99, 0x9A, 0xA2, 0xA3, 0xA4, 0xA5, 0xA6, 0xA7,
        0xA8, 0xA9, 0xAA, 0xB2, 0xB3, 0xB4, 0xB5, 0xB6, 0xB7, 0xB8, 0xB9, 0xBA, 0xC2, 0xC3, 0xC4, 0xC5,
        0xC6, 0xC7, 0xC8, 0xC9, 0xCA, 0xD2, 0xD3, 0xD4, 0xD5, 0xD6, 0xD7, 0xD8, 0xD9, 0xDA, 0xE1, 0xE2,
        0xE3, 0xE4, 0xE5, 0xE6, 0xE7, 0xE8, 0xE9, 0xEA, 0xF1, 0xF2, 0xF3, 0xF4, 0xF5, 0xF6, 0xF7, 0xF8,
        0xF9, 0xFA
    };
    /* <-SEC M1.10.1 */


    /* Constant: HuffmanTable_C_AC
    	Example written in ITU-T T81 specification */
    /* ->SEC M1.10.1 Table data. */
    const uint8_t  HuffmanTable_C_AC[HUFFMAN_TABLE_AC_SIZE] =
    {
        0x00, 0x02, 0x01, 0x02, 0x04, 0x04, 0x03, 0x04, 0x07, 0x05, 0x04, 0x04, 0x00, 0x01, 0x02, 0x77,
        0x00, 0x01, 0x02, 0x03, 0x11, 0x04, 0x05, 0x21, 0x31, 0x06, 0x12, 0x41, 0x51, 0x07, 0x61, 0x71,
        0x13, 0x22, 0x32, 0x81, 0x08, 0x14, 0x42, 0x91, 0xA1, 0xB1, 0xC1, 0x09, 0x23, 0x33, 0x52, 0xF0,
        0x15, 0x62, 0x72, 0xD1, 0x0A, 0x16, 0x24, 0x34, 0xE1, 0x25, 0xF1, 0x17, 0x18, 0x19, 0x1A, 0x26,
        0x27, 0x28, 0x29, 0x2A, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3A, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48,
        0x49, 0x4A, 0x53, 0x54, 0x55, 0x56, 0x57, 0x58, 0x59, 0x5A, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68,
        0x69, 0x6A, 0x73, 0x74, 0x75, 0x76, 0x77, 0x78, 0x79, 0x7A, 0x82, 0x83, 0x84, 0x85, 0x86, 0x87,
        0x88, 0x89, 0x8A, 0x92, 0x93, 0x94, 0x95, 0x96, 0x97, 0x98, 0x99, 0x9A, 0xA2, 0xA3, 0xA4, 0xA5,
        0xA6, 0xA7, 0xA8, 0xA9, 0xAA, 0xB2, 0xB3, 0xB4, 0xB5, 0xB6, 0xB7, 0xB8, 0xB9, 0xBA, 0xC2, 0xC3,
        0xC4, 0xC5, 0xC6, 0xC7, 0xC8, 0xC9, 0xCA, 0xD2, 0xD3, 0xD4, 0xD5, 0xD6, 0xD7, 0xD8, 0xD9, 0xDA,
        0xE2, 0xE3, 0xE4, 0xE5, 0xE6, 0xE7, 0xE8, 0xE9, 0xEA, 0xF2, 0xF3, 0xF4, 0xF5, 0xF6, 0xF7, 0xF8,
        0xF9, 0xFA
    };
    /* <-SEC M1.10.1 */

    app_jcu_t*       self = &gs_app_jcu;
    errnum_t         e;
    jcu_errorcode_t  jcu_error;

    jcu_error = R_JCU_SelectCodec( JCU_ENCODE );
    IF ( jcu_error != JCU_ERROR_OK )
    {
        e = E_OTHERS;
        goto  fin;
    }


    /* Set tables. Call these functions by every called R_JCU_SelectCodec(H/W Reset) */
    jcu_error = R_JCU_SetQuantizationTable( JCU_TABLE_NO_0, QuantizationTable_Y );
    IF( jcu_error != JCU_ERROR_OK )
    {
        e = E_OTHERS;
        goto  fin;
    }
    jcu_error = R_JCU_SetQuantizationTable( JCU_TABLE_NO_1, QuantizationTable_C );
    IF( jcu_error != JCU_ERROR_OK )
    {
        e = E_OTHERS;
        goto  fin;
    }
    jcu_error = R_JCU_SetHuffmanTable( JCU_TABLE_NO_0, JCU_HUFFMAN_DC, HuffmanTable_Y_DC );
    IF( jcu_error != JCU_ERROR_OK )
    {
        e = E_OTHERS;
        goto  fin;
    }
    jcu_error = R_JCU_SetHuffmanTable( JCU_TABLE_NO_1, JCU_HUFFMAN_DC, HuffmanTable_C_DC );
    IF( jcu_error != JCU_ERROR_OK )
    {
        e = E_OTHERS;
        goto  fin;
    }
    jcu_error = R_JCU_SetHuffmanTable( JCU_TABLE_NO_0, JCU_HUFFMAN_AC, HuffmanTable_Y_AC );
    IF( jcu_error != JCU_ERROR_OK )
    {
        e = E_OTHERS;
        goto  fin;
    }
    jcu_error = R_JCU_SetHuffmanTable( JCU_TABLE_NO_1, JCU_HUFFMAN_AC, HuffmanTable_C_AC );
    IF( jcu_error != JCU_ERROR_OK )
    {
        e = E_OTHERS;
        goto  fin;
    }


    /* Call "R_JCU_SetEncodeParam" */
    self->JCU_EncodeBuffer.source.address = in_SourceAddress;
    self->JCU_EncodeBuffer.destination.address = (uint32_t*) JPEG_Buffer[0];

    jcu_error = R_JCU_SetEncodeParam( &self->JCU_Encode, &self->JCU_EncodeBuffer );
    IF ( jcu_error != JCU_ERROR_OK )
    {
        e = E_OTHERS;
        goto  fin;
    }


    /* Call "R_JCU_StartAsync" */
    R_OSPL_EVENT_Clear( gs_message_loop_thread_id, self->JCU_EncodeAsync.A_EventValue );

    jcu_error = R_JCU_StartAsync( &self->JCU_EncodeAsync );
    IF ( jcu_error != JCU_ERROR_OK )
    {
        e = E_OTHERS;
        goto  fin;
    }

    e=0;
fin:
    return  e;
}


/***********************************************************************
* Function: APP_JCU_Decode
*
* Arguments:
*    in_DestinationAddress - uint32_t*
*
* Return Value:
*    Error Code, 0=No Error
************************************************************************/
static errnum_t  APP_JCU_Decode( uint32_t*  in_DestinationAddress )
{
    app_jcu_t*       self = &gs_app_jcu;
    errnum_t         e;
    jcu_errorcode_t  jcu_error;

    jcu_error = R_JCU_SelectCodec( JCU_DECODE );
    IF ( jcu_error != JCU_ERROR_OK )
    {
        e = E_OTHERS;
        goto  fin;
    }


    /* Call "R_JCU_SetDecodeParam" */
    self->JCU_DecodeBuffer.source.address = (uint32_t*) JPEG_Buffer[0];
    self->JCU_DecodeBuffer.destination.address = in_DestinationAddress;

    jcu_error = R_JCU_SetDecodeParam( &self->JCU_Decode, &self->JCU_DecodeBuffer );
    IF ( jcu_error != JCU_ERROR_OK )
    {
        e = E_OTHERS;
        goto  fin;
    }


    /* Call "R_JCU_StartAsync" */
    R_OSPL_EVENT_Clear( gs_message_loop_thread_id, self->JCU_DecodeAsync.A_EventValue );

    jcu_error = R_JCU_StartAsync( &self->JCU_DecodeAsync );
    IF ( jcu_error != JCU_ERROR_OK )
    {
        e = E_OTHERS;
        goto  fin;
    }

    e=0;
fin:
    return  e;
}


/***********************************************************************
* Function: APP_JCU_Terminate
*
* Arguments:
*    None.
*
* Return Value:
*    None.
************************************************************************/
static void  APP_JCU_Terminate(void)
{
    app_jcu_t*  self = &gs_app_jcu;

    if ( self->IsInitialized )
    {
        R_JCU_Terminate();

        R_OSPL_EVENT_Free( &self->JCU_EncodeAsync.A_Thread,  &self->JCU_EncodeAsync.A_EventValue );
        R_OSPL_EVENT_Free( &self->JCU_DecodeAsync.A_Thread,  &self->JCU_DecodeAsync.A_EventValue );

        self->IsInitialized = false;
    }
}


/***********************************************************************
* Function: Termination
*
* Arguments:
*    ch - Channel
*
* Return Value:
*    None.
************************************************************************/
static void Termination (const vdc5_channel_t ch)
{
    vdc5_error_t error;

    /* Interrupt parameter */
    interrupt.callback = 0;         /* Clear callback function pointer */
    /* Set interrupt service routine */
    error = R_VDC5_CallbackISR(ch, &interrupt);
    if (error != VDC5_OK)
    {
        GRAPHICS_DebugPrint(__FILE__, __LINE__, (int32_t)error);
    }

    if (error == VDC5_OK)
    {
        /* Stop surface */
        error = GRPDRV_StopSurfaces(ch);
    }
    if (error == VDC5_OK)
    {
        /* Destroy surface */
        error = GRPDRV_DestroySurfaces(ch);
    }
    if (error == VDC5_OK)
    {
        /* Termination */
        error = GRPDRV_Term(ch);
    }
}


/***********************************************************************
* Function: R_JCU_SampleVideoEncodeAndShowI
*    Main.
*
* Arguments:
*    argc - 0 = infinite loop, 1 = not infinite loop
*    argv - not used
*
* Return Value:
*    None.
************************************************************************/
void R_JCU_SampleVideoEncodeAndShowI( int32_t const  argc,  char_t * const* const  argv )
{
    enum { frame_num_max = 5 * 60 };

    vdc5_error_t                error;
    void                      * framebuff;
    vdc5_period_rect_t          res_tmp;
    errnum_t                    e;
    jcu_errorcode_t             jcu_error;
    bool_t                      is_JCU_running = false;
    int_fast32_t                frame_num;
    int_fast32_t                frame_num_plus_value = +1;
    size_t                      encoded_size;
    /* VDC5 channel (VDC5_CHANNEL_0 or VDC5_CHANNEL_1) */
    const vdc5_channel_t        ch              = gs_VDC5_ch;
    const vdc5_layer_id_t       layer_vd_id     = VDC5_LAYER_ID_0_WR;
    const vdc5_layer_id_t       layer_gr_id     = VDC5_LAYER_ID_1_RD;
    const vdc5_graphics_type_t  gr_type_vd      = VDC5_GR_TYPE_GR0;
    const vdc5_graphics_type_t  gr_type_gr      = VDC5_GR_TYPE_GR1;
    const vdc5_res_md_t         res_md_tmp      = VIDEO_FORMAT;
    const vdc5_gr_format_t      gr_format_tmp   = GRAPHICS_FORMAT;
    /* Video input channel */
    const vdc5_channel_t        ch_input        = VDC5_CHANNEL_0;

    /* R_OSPL_SET_BREAK_ERROR_ID( 1 ); */

    printf( "R_JCU_SampleVideoEncodeAndShow Ver 0.11\n" );

    InitParameters(ch);

    CreateFrameBuffer();
    R_OSPL_MEMORY_Flush( R_OSPL_FLUSH_WRITEBACK_INVALIDATE );

    SetInterrupt(ch);

    SetVideoDecoder(ch_input, &res_tmp);

    /* Initialization
    	Video input channel 0: Used
    	Video input channel 1: Not used
     */
    error = GRPDRV_Init(ch, VDC5_ON, VDC5_OFF);

    if (error == VDC5_OK)
    {
        /* Create video surface */
        error = GRPDRV_VideoCreateNonDispSurface(
                    ch,
                    layer_vd_id,
                    FrameBuffer_Video[0],
                    (uint32_t)VIDEO_BUFFER_STRIDE,      /* 2 bytes per pixel */
                    res_md_tmp,
                    &res_tmp,
                    res_tmp.vw,            /* input 240  line  -> output 240 line non scale */
                    LCD_CH0_DISP_HW );     /* input 1440 pixel -> output 800 pixel scale    */
        /* Uses &FrameBuffer_Video[1], too */
    }

    if (error == VDC5_OK)
    {
        period_rect.vw = (uint16_t)FRAME_BUFFER_HEIGHT * 2;

        framebuff = GetBackBuffer();
        /* Create graphics surface */
        error = GRPDRV_GraphicsCreateSurfaceScale(
                    ch,
                    layer_gr_id,
                    framebuff,
                    FRAME_BUFFER_STRIDE,
                    gr_format_tmp,
                    res_tmp.vw,            /* read 240 line -> disp 480 line scale */
                    LCD_CH0_DISP_HW,       /* read 800 pixel-> dips 800 non scale  */
                    &period_rect);
    }

    if (error == VDC5_OK)
    {
        /* Event */
        R_OSPL_EVENT_Allocate( &gs_message_loop_thread_id,  gs_message_loop_thread_id,
                               &gs_video_in_event,  EVENT_VIDEO_IN_VSYNC );
        R_OSPL_EVENT_Allocate( &gs_message_loop_thread_id,  gs_message_loop_thread_id,
                               &gs_video_out_event,  EVENT_VIDEO_OUT_VSYNC );

        /* Interrupt parameter */
        interrupt.type     = VDC5_INT_TYPE_VLINE;  /* Interrupt type */
        interrupt.callback = &IntCallbackFunc;     /* Callback function pointer */
        interrupt.line_num = (uint16_t)0 ;         /* Line number */
        /* Set interrupt service routine */
        error = R_VDC5_CallbackISR(ch, &interrupt);
        if (error != VDC5_OK)
        {
            GRAPHICS_DebugPrint(__FILE__, __LINE__, (int32_t)error);
        }

        /* Interrupt parameter */
        interrupt.type      = VDC5_INT_TYPE_S0_VI_VSYNC;  /* Interrupt type */
        interrupt.callback  = &IntCallbackFunc;   /* Callback function pointer */
        interrupt.line_num  = (uint16_t)0 ;               /* Line number */
        /* Set interrupt service routine */
        error = R_VDC5_CallbackISR(ch, &interrupt);
        if (error != VDC5_OK)
        {
            GRAPHICS_DebugPrint(__FILE__, __LINE__, (int32_t)error);
        }

    }

    if (error == VDC5_OK)
    {
        gr_disp_sel_tmp[gr_type_vd] = VDC5_DISPSEL_LOWER;
        gr_disp_sel_tmp[gr_type_gr] = VDC5_DISPSEL_LOWER;

        /* Start surface */
        error = GRPDRV_StartSurfaces(ch, gr_disp_sel_tmp);
    }

    if (error == VDC5_OK)
    {
        e= APP_JCU_SetUp();
        IF(e!=0)
        {
            error = VDC5_ERR_PARAM_UNDEFINED;
        }
    }

    if (error == VDC5_OK)
    {
        R_OSPL_EVENT_Clear( gs_message_loop_thread_id, EVENT_ALL );
#if ! R_OSPL_IS_PREEMPTION  &&  ! JCU_MULTI_THREAD  &&  R_OSPL_CPU_LOAD
        R_OSPL_IDLE_Start_CPU_Load( 1000 );
#endif

        for ( frame_num = 0;  frame_num < frame_num_max;  /* frame_num += 1 is written after here. */ )
        {
            bit_flags32_t  got_flags;

#if  R_OSPL_IS_PREEMPTION  ||  ! JCU_MULTI_THREAD
            e= R_OSPL_EVENT_Wait( R_OSPL_ANY_FLAG, &got_flags, R_OSPL_INFINITE );
            IF (e != 0)
            {
                GRAPHICS_DebugPrint(__FILE__, __LINE__, (int32_t)error);
                break;
            }
#else
            for /*ever*/ (;;)
            {
                e= R_JCU_OPERATOR_Dispatch( R_JCU_GetOperatorThreadId() );
                IF(e)
                {
                    break;
                }
                e= R_OSPL_EVENT_Wait( R_OSPL_ANY_FLAG, &got_flags, 0 );
                IF(e)
                {
                    break;
                }
                if ( got_flags != R_OSPL_TIMEOUT )
                {
                    break;
                }
            }
            IF (e != 0)
            {
                GRAPHICS_DebugPrint(__FILE__, __LINE__, (int32_t)error);
                break;
            }
#endif

            if ( IS_BIT_SET( got_flags, EVENT_VIDEO_IN_VSYNC )  &&  ! is_JCU_running )
            {
                e= APP_JCU_Encode( (uint32_t*) Videoframebuff );
                IF (e != 0)
                {
                    GRAPHICS_DebugPrint(__FILE__, __LINE__, (int32_t)error);
                    break;
                }
                is_JCU_running = true;
            }

            if ( IS_BIT_SET( got_flags, EVENT_JPEG_ENCODED ) )
            {
                is_JCU_running = false;
                IF ( gs_app_jcu.JCU_EncodeAsync.ReturnValue != 0 )
                {
                    GRAPHICS_DebugPrint(__FILE__, __LINE__, (int32_t)error);
                    break;
                }

                jcu_error = R_JCU_GetEncodedSize( &encoded_size );
                IF ( jcu_error != JCU_ERROR_OK )
                {
                    GRAPHICS_DebugPrint(__FILE__, __LINE__, (int32_t)jcu_error);
                    break;
                }


                framebuff = GetBackBuffer();

                e= APP_JCU_Decode( framebuff );
                IF (e != 0)
                {
                    GRAPHICS_DebugPrint(__FILE__, __LINE__, (int32_t)error);
                    break;
                }
                is_JCU_running = true;
            }

            if ( IS_BIT_SET( got_flags, EVENT_JPEG_DECODED ) )
            {
                is_JCU_running = false;
                IF ( gs_app_jcu.JCU_DecodeAsync.ReturnValue != 0 )
                {
                    GRAPHICS_DebugPrint(__FILE__, __LINE__, (int32_t)error);
                    break;
                }


                /* Read data parameter */
                read_chg.gr_base = GetBackBuffer();       /* Frame buffer base address */
                /* Change read process */
                error = R_VDC5_ChangeReadProcess(ch, layer_gr_id, &read_chg);
                IF (error != VDC5_OK)
                {
                    GRAPHICS_DebugPrint(__FILE__, __LINE__, (int32_t)error);
                    break;
                }


                SwapFrameBuffer();


                if ( frame_num == 0 )
                {
#ifndef NDEBUG
                    printf( "<DecodedJPEG address=\"0x%08X\" size=\"0x%08X\"/>\n", JPEG_Buffer[0], encoded_size );
#endif
                }
                frame_num += frame_num_plus_value;

                if ( argc != 0 )
                {
                    frame_num_plus_value = 0;
                }
            }
        }
    }
    /* Quit */
    R_OSPL_EVENT_Free( &gs_message_loop_thread_id,  &gs_video_in_event );
    R_OSPL_EVENT_Free( &gs_message_loop_thread_id,  &gs_video_out_event );
    APP_JCU_Terminate();
    Termination(ch);

    R_DEBUG_BREAK_IF_ERROR();
}


