/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2014 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/**************************************************************************//**
* @file         video_sample.c
* @version      1.00
* $Rev: 1234 $
* $Date:: 2014-10-24 14:52:15 +0900#$
* @brief        VDC5 driver video sample
******************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include    <string.h>

#include    "cmsis_os.h"
#include    "r_typedefs.h"
#include    "Renesas_RZ_A1.h"
#include    "gic.h"

#include    "iodefine.h"
#include    "dev_drv.h"
#include    "Display_if.h"

#include    "displaywrapper_if.h"
#include    "lcd_if.h"




/******************************************************************************
Macro definitions
******************************************************************************/
/* VDC5_CHANNEL_0 or VDC5_CHANNEL_1 */
#ifndef __RZA1L
/* setup GRAPHICS_CHANNEL Channel */
#if (LCD_BOARD == LCD_BOARD_LCDKITB01)

#if (LCD_KITB01_CH == LCD_KITB01_CH_0)
#define GRAPHICS_CHANNEL      (VDC5_CHANNEL_0)
#elif (LCD_KITB01_CH == LCD_KITB01_CH_1)
#define GRAPHICS_CHANNEL      (VDC5_CHANNEL_1)
#else
#error LCD_KITB01_CH is incorrect setting
#endif /* LCD_KITB01_CH */

#elif (LCD_BOARD == LCD_BOARD_TITSC2007)
#define GRAPHICS_CHANNEL     VDC5_CHANNEL_1

#else
#error LCD_BOARD is incorrect setting
#endif /* LCD_BOARD */

#else
#define     GRAPHICS_CHANNEL            (VDC5_CHANNEL_0)

#endif


/*! Graphics format: ARGB8888, 4 bytes per pixel */
#define     GRAPHICS_FORMAT             (VDC5_GR_FORMAT_ARGB8888)
/*! Frame buffer stride: Frame buffer stride should be set to a multiple of 32 or 128
    in accordance with the frame buffer burst transfer mode. */
#ifndef __RZA1L
/* VDC5_CHANNEL_0 or VDC5_CHANNEL_1 */
#if (LCD_KITB01_CH == LCD_KITB01_CH_0)
#define     FRAME_BUFFER_STRIDE         (((LCD_CH0_DISP_HW * 4u) + 31u) & ~31u)
#define     FRAME_BUFFER_HEIGHT         (LCD_CH0_DISP_VW / 2u)
#else
#define     FRAME_BUFFER_STRIDE         (((LCD_CH1_DISP_HW * 4u) + 31u) & ~31u)
#define     FRAME_BUFFER_HEIGHT         (LCD_CH1_DISP_VW / 2u)
#endif
#else
#define     FRAME_BUFFER_STRIDE         (((LCD_CH0_DISP_HW * 4u) + 31u) & ~31u)
#define     FRAME_BUFFER_HEIGHT         (LCD_CH0_DISP_VW / 2u)
#endif

#define     VSYNC_WAIT_COUNT            (240)

#define     IMGCAP_SIZE_NTSC_HS         (122u * 2u)
#define     IMGCAP_SIZE_NTSC_HW         (720u * 2u)
#define     IMGCAP_SIZE_NTSC_VS         (16u)
#define     IMGCAP_SIZE_NTSC_VW         (480u/2u)

#define     IMGCAP_SIZE_PAL_HS          (132u * 2u)
#define     IMGCAP_SIZE_PAL_HW          (720u * 2u)
#define     IMGCAP_SIZE_PAL_VS          (19u)
#define     IMGCAP_SIZE_PAL_VW          (560u/2u)

/*! Video format: YCbCr422, 2 bytes per pixel */
#define     VIDEO_FORMAT                (VDC5_RES_MD_YCBCR422)
/*! Frame buffer stride: Frame buffer stride should be set to a multiple of 32 or 128
    in accordance with the frame buffer burst transfer mode. */
#ifndef __RZA1L
#if (LCD_KITB01_CH == LCD_KITB01_CH_0)
#define     VIDEO_BUFFER_STRIDE         (((LCD_CH0_DISP_HW * 2u) + 31u) & ~31u)
#else
#define     VIDEO_BUFFER_STRIDE         (((LCD_CH1_DISP_HW * 2u) + 31u) & ~31u)
#endif
#else
#define     VIDEO_BUFFER_STRIDE         (((LCD_CH0_DISP_HW * 2u) + 31u) & ~31u)
#endif

#define     VIDEO_BUFFER_HEIGHT         (IMGCAP_SIZE_PAL_VW)

#define     BUFFER_CNT                  (2u)

/******************************************************************************
Typedef definitions
******************************************************************************/
typedef struct
{
    void      * buffer_1;
    void      * buffer_2;
    uint32_t    backbuffer_num;
} buffering_t;


/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/

/******************************************************************************
Exported global variables (to be accessed by other files)
******************************************************************************/

/******************************************************************************
Private global variables and functions
******************************************************************************/
static void InitParameters(const vdc5_channel_t ch);
static void SetInterrupt(const vdc5_channel_t ch);
static void IntCallbackFunc(vdc5_int_type_t int_type);
static void WaitVsync(void);
static void SetGraphicsData(
    const vdc5_gr_format_t  gr_format,
    void            * const fr_buff,
    const uint16_t          width,
    const uint16_t          height,
    const uint32_t          stride);
static void SetVideoDecoder(const vdc5_channel_t ch, vdc5_period_rect_t * const cap_rect);
static void CreateFrameBuffer(void);
static void * GetBackBuffer(void);
static void SwapFrameBuffer(void);
static void Termination(const vdc5_channel_t ch);

/*! Interrupt parameter */
static vdc5_int_t           interrupt;
/*! Read data parameter */
static vdc5_read_chg_t      read_chg;
/*! Graphics display mode */
static vdc5_gr_disp_sel_t   gr_disp_sel_tmp[VDC5_GR_TYPE_NUM];
/*! Display area */
static vdc5_period_rect_t   period_rect;

static volatile int32_t vsync_count = VSYNC_WAIT_COUNT;

static buffering_t buffering_inf;

#define  IS_VIDEO_SAMPLE_VRAM_USED  0  /* 1 or 0 */
#if IS_VIDEO_SAMPLE_VRAM_USED
#if     defined(__CC_ARM)
#pragma arm section zidata = "VRAM"
static uint8_t FrameBuffer_Video[VIDEO_BUFFER_STRIDE * VIDEO_BUFFER_HEIGHT * BUFFER_CNT];
static uint8_t FrameBuffer_1[FRAME_BUFFER_STRIDE * FRAME_BUFFER_HEIGHT];
static uint8_t FrameBuffer_2[FRAME_BUFFER_STRIDE * FRAME_BUFFER_HEIGHT];
#pragma arm section zidata
#elif   defined(__ICCARM__)
static uint8_t FrameBuffer_Video[VIDEO_BUFFER_STRIDE * VIDEO_BUFFER_HEIGHT * BUFFER_CNT] @ "VRAM";
static uint8_t FrameBuffer_1[FRAME_BUFFER_STRIDE * FRAME_BUFFER_HEIGHT] @ "VRAM";
static uint8_t FrameBuffer_2[FRAME_BUFFER_STRIDE * FRAME_BUFFER_HEIGHT] @ "VRAM";
#elif   defined(__GNUC__)
static uint8_t FrameBuffer_Video[VIDEO_BUFFER_STRIDE * VIDEO_BUFFER_HEIGHT * BUFFER_CNT] __attribute__ ((section("VRAM")));
static uint8_t FrameBuffer_1[FRAME_BUFFER_STRIDE * FRAME_BUFFER_HEIGHT] __attribute__ ((section("VRAM")));
static uint8_t FrameBuffer_2[FRAME_BUFFER_STRIDE * FRAME_BUFFER_HEIGHT] __attribute__ ((section("VRAM")));
#else
static uint8_t FrameBuffer_Video[VIDEO_BUFFER_STRIDE * VIDEO_BUFFER_HEIGHT * BUFFER_CNT];
static uint8_t FrameBuffer_1[FRAME_BUFFER_STRIDE * FRAME_BUFFER_HEIGHT];
static uint8_t FrameBuffer_2[FRAME_BUFFER_STRIDE * FRAME_BUFFER_HEIGHT];
#endif
#else
static uint8_t FrameBuffer_Video[1];
static uint8_t FrameBuffer_1[1];
static uint8_t FrameBuffer_2[1];
#endif


osSemaphoreDef( vdec_vsync );
osSemaphoreId   vdec_vsync;
/**************************************************************************//**
 * @brief       Initialize variables
 * @param[in]   ch                      : Channel
 * @retval      None
******************************************************************************/
static void InitParameters (const vdc5_channel_t ch)
{
    uint32_t index;

    /* Interrupt parameter */
    interrupt.type      = VDC5_INT_TYPE_VLINE;  /* Interrupt type */
    interrupt.callback  = 0;                    /* Callback function pointer */
    interrupt.line_num  = (uint16_t)0 ;         /* Line number */

    read_chg.gr_base        = NULL;         /* Frame buffer base address */
    read_chg.width_read_fb  = NULL;         /* Width of the image read from frame buffer */
    read_chg.gr_grc         = NULL;         /* Display area */
    read_chg.gr_disp_sel    = NULL;         /* Graphics display mode */

    for (index = 0; index < (uint32_t)VDC5_GR_TYPE_NUM; index++)
    {
        gr_disp_sel_tmp[index] = VDC5_DISPSEL_IGNORED;
    }

    if (ch == VDC5_CHANNEL_0)
    {
        period_rect.vs  = (uint16_t)LCD_CH0_DISP_VS;
        period_rect.vw  = (uint16_t)LCD_CH0_DISP_VW;
        period_rect.hs  = (uint16_t)LCD_CH0_DISP_HS;
        period_rect.hw  = (uint16_t)LCD_CH0_DISP_HW;
    }
    else
    {
        period_rect.vs  = (uint16_t)LCD_CH1_DISP_VS;
        period_rect.vw  = (uint16_t)LCD_CH1_DISP_VW;
        period_rect.hs  = (uint16_t)LCD_CH1_DISP_HS;
        period_rect.hw  = (uint16_t)LCD_CH1_DISP_HW;
    }
}   /* End of function InitParameters() */

/**************************************************************************//**
 * @brief       Setup VDC5 interrupt
 * @param[in]   ch                      : Channel
 * @retval      None
******************************************************************************/
static void SetInterrupt (const vdc5_channel_t ch)
{


    void         (* func)(uint32_t int_sense);
    IRQn_Type       int_id;

    int_id = (ch == VDC5_CHANNEL_0) ? GR3_VLINE0_IRQn : GR3_VLINE1_IRQn;
    /* disable LCD interrupt */
    GIC_DisableIRQ( int_id );
    func = R_VDC5_GetISR(ch, VDC5_INT_TYPE_VLINE);
    (void)InterruptHandlerRegister( int_id, func );
    /* set interrupt priority */
    GIC_SetPriority( int_id, VDC5_INT_PRIORITY );
    /* enable LCD interrupt */
    GIC_EnableIRQ( int_id );
    
}   /* End of function SetInterrupt() */

/**************************************************************************//**
 * @brief       VDC5 Interrupt callback function
 * @param[in]   int_type    : VDC5 interrupt type
 * @retval      None
******************************************************************************/
static void IntCallbackFunc (vdc5_int_type_t int_type)
{
    if (vsync_count > 0)
    {
        vsync_count--;
    }
    if(vsync_count == 0)
    {
        vsync_count = VSYNC_WAIT_COUNT;
        (void)osSemaphoreRelease( vdec_vsync );
    }
}   /* End of function IntCallbackFunc() */

/**************************************************************************//**
 * @brief       Wait for the specified number of times Vsync (VLINE) occurs
 * @param[in]   None
 * @retval      None
******************************************************************************/
static void WaitVsync (void)
{

    /* Wait VSync*/
    (void)osSemaphoreWait( vdec_vsync, osWaitForever );
}   /* End of function WaitVsync() */

/**************************************************************************//**
 * @brief       Setup graphics
 * @param[in]   gr_format           : Format of the frame buffer read signal
 * @param[out]  fr_buff             : Frame buffer base address
 * @param[in]   width               : Width of the frame buffer to be read
 * @param[in]   height              : Height of the frame buffer to be read
 * @param[in]   stride              : Frame buffer line offset address (buffer stride)
 * @retval      None
******************************************************************************/
static void SetGraphicsData (
    const vdc5_gr_format_t  gr_format,
    void            * const fr_buff,
    const uint16_t          width,
    const uint16_t          height,
    const uint32_t          stride)
{
    /* Create fill color image */
    GRAPHICS_CreateFillColorImage(fr_buff, NULL, width, height, (uint16_t)stride, gr_format, (int32_t)-1);
}   /* End of function SetGraphicsData() */

/**************************************************************************//**
 * @brief       Setup video decoder
 * @param[in]   ch              : Channel
 * @param[out]  cap_rect        : Image area to be captured
 * @retval      None
******************************************************************************/
static void SetVideoDecoder (const vdc5_channel_t ch, vdc5_period_rect_t * const cap_rect)
{
    vdec_channel_t          channel;
    graphics_col_sys_t      col_sys;
    const vdec_adc_vinsel_t adc_vinsel = VDEC_ADC_VINSEL_VIN1;

    if (ch == VDC5_CHANNEL_0)
    {
        channel = VDEC_CHANNEL_0;
    }
    else
    {
        channel = VDEC_CHANNEL_1;
    }

    col_sys = GRAPHICS_VideoDecoderInit(adc_vinsel, channel);
    if (col_sys == GRPH_COL_SYS_UNKNOWN)
    {   /* If not detected, ... */
        col_sys = GRPH_COL_SYS_NTSC_358;
    }

    if ((col_sys == GRPH_COL_SYS_NTSC_358) || (col_sys == GRPH_COL_SYS_NTSC_443))
    {   /* Size of NTSC video image to be captured */
        cap_rect->vs = (uint16_t)IMGCAP_SIZE_NTSC_VS;
        cap_rect->vw = (uint16_t)IMGCAP_SIZE_NTSC_VW;
        cap_rect->hs = (uint16_t)IMGCAP_SIZE_NTSC_HS;
        cap_rect->hw = (uint16_t)IMGCAP_SIZE_NTSC_HW;
    }
    else
    {   /* Size of PAL video image to be captured */
        cap_rect->vs = (uint16_t)IMGCAP_SIZE_PAL_VS;
        cap_rect->vw = (uint16_t)IMGCAP_SIZE_PAL_VW;
        cap_rect->hs = (uint16_t)IMGCAP_SIZE_PAL_HS;
        cap_rect->hw = (uint16_t)IMGCAP_SIZE_PAL_HW;
    }
}   /* End of function SetVideoDecoder() */

/**************************************************************************//**
 * @brief       Create a frame buffer
 * @param[in]   void
 * @retval      None
******************************************************************************/
static void CreateFrameBuffer (void)
{
    buffering_inf.buffer_1          = (void *)FrameBuffer_1;
    buffering_inf.buffer_2          = (void *)FrameBuffer_2;
    buffering_inf.backbuffer_num    = 0u;
}   /* End of function CreateFrameBuffer() */

/**************************************************************************//**
 * @brief       Get back buffer address
 * @param[in]   void
 * @retval      Back buffer address
******************************************************************************/
static void * GetBackBuffer (void)
{
    return ((buffering_inf.backbuffer_num == 0u) ? buffering_inf.buffer_1 : buffering_inf.buffer_2);
}   /* End of function GetBackBuffer() */

/**************************************************************************//**
 * @brief       Swap frame buffers
 * @param[in]   void
 * @retval      None
******************************************************************************/
static void SwapFrameBuffer (void)
{
    if (buffering_inf.backbuffer_num == 0u)
    {
        buffering_inf.backbuffer_num = (uint32_t)1u;
    }
    else
    {
        buffering_inf.backbuffer_num = (uint32_t)0u;
    }
}   /* End of function SwapFrameBuffer() */

/**************************************************************************//**
 * @brief       Termination
 * @param[in]   ch                      : Channel
 * @retval      None
******************************************************************************/
static void Termination (const vdc5_channel_t ch)
{
    vdc5_error_t error;

    /* Interrupt parameter */
    interrupt.callback = 0;         /* Clear callback function pointer */
    /* Set interrupt service routine */
    error = R_VDC5_CallbackISR(ch, &interrupt);
    if (error != VDC5_OK)
    {
        GRAPHICS_DebugPrint(__FILE__, __LINE__, (int32_t)error);
    }

    if (error == VDC5_OK)
    {
        /* Stop surface */
        error = GRPDRV_StopSurfaces(ch);
    }
    if (error == VDC5_OK)
    {
        /* Destroy surface */
        error = GRPDRV_DestroySurfaces(ch);
    }
    if (error == VDC5_OK)
    {
        /* Termination */
        error = GRPDRV_Term(ch);
    }
}   /* End of function Termination() */

/**************************************************************************//**
 * @brief       Video display sample
 * @param[in]   void
 * @retval      None
******************************************************************************/
void GRAPHICS_VideoSample (void)
{
    vdc5_error_t                error;
    void                      * framebuff;
    vdc5_period_rect_t          res_tmp;
    const vdc5_channel_t        ch              = GRAPHICS_CHANNEL;
    const vdc5_layer_id_t       layer_vd_id     = VDC5_LAYER_ID_0_WR;
    const vdc5_layer_id_t       layer_gr_id     = VDC5_LAYER_ID_2_RD;
    const vdc5_graphics_type_t  gr_type_vd      = VDC5_GR_TYPE_GR0;
    const vdc5_graphics_type_t  gr_type_gr      = VDC5_GR_TYPE_GR2;
    const vdc5_res_md_t         res_md_tmp      = VIDEO_FORMAT;
    const vdc5_gr_format_t      gr_format_tmp   = GRAPHICS_FORMAT;
    /* Video input channel
        If layer_vd_id is VDC5_LAYER_ID_0_WR, ch_input    
    */
    const vdc5_channel_t        ch_input        = GRAPHICS_CHANNEL;
    int   nLoop;

    InitParameters(ch);

    CreateFrameBuffer();

    SetInterrupt(ch);

    SetVideoDecoder(ch_input, &res_tmp);

    /* Initialization
        Video input channel 0: Used
        Video input channel 1: Not used
     */
    if (ch == VDC5_CHANNEL_0)
    {
        error = GRPDRV_Init(ch, VDC5_ON, VDC5_OFF);
    }
    else
    {
        error = GRPDRV_Init(ch, VDC5_OFF, VDC5_ON);
    }

    if (error == VDC5_OK)
    {
        /* Create video surface */
        error = GRPDRV_VideoCreateSurface(
                    ch,
                    layer_vd_id,
                    FrameBuffer_Video,
                    (uint32_t)VIDEO_BUFFER_STRIDE,      /* 2 bytes per pixel */
                    res_md_tmp,
                    &res_tmp,
                    &period_rect);
    }

    if (error == VDC5_OK)
    {
        period_rect.vw = (uint16_t)FRAME_BUFFER_HEIGHT;

        framebuff = GetBackBuffer();
        SetGraphicsData(gr_format_tmp, framebuff, period_rect.hw, period_rect.vw, FRAME_BUFFER_STRIDE);
        /* Create graphics surface */
        error = GRPDRV_GraphicsCreateSurface(
                    ch,
                    layer_gr_id,
                    framebuff,
                    FRAME_BUFFER_STRIDE,
                    gr_format_tmp,
                    &period_rect);
    }

    if (error == VDC5_OK)
    {
        /* Interrupt parameter */
        interrupt.callback = &IntCallbackFunc;      /* Callback function pointer */
        /* Set interrupt service routine */
        error = R_VDC5_CallbackISR(ch, &interrupt);
        if (error != VDC5_OK)
        {
            GRAPHICS_DebugPrint(__FILE__, __LINE__, (int32_t)error);
        }
    }

    if (error == VDC5_OK)
    {
        gr_disp_sel_tmp[gr_type_vd] = VDC5_DISPSEL_LOWER;
        gr_disp_sel_tmp[gr_type_gr] = VDC5_DISPSEL_BLEND;
        /* Start surface */
        error = GRPDRV_StartSurfaces(ch, gr_disp_sel_tmp);
    }

    if (error == VDC5_OK)
    {
        vdec_vsync = osSemaphoreCreate( osSemaphore(vdec_vsync), 0 );
        nLoop = 0;
        while (nLoop < 30)
        {
            nLoop ++;
            /* Wait Vsync */
            WaitVsync();

            SwapFrameBuffer();

            framebuff = GetBackBuffer();
            SetGraphicsData(gr_format_tmp, framebuff, period_rect.hw, period_rect.vw, FRAME_BUFFER_STRIDE);
            /* Read data parameter */
            read_chg.gr_base = framebuff;       /* Frame buffer base address */
            /* Change read process */
            error = R_VDC5_ChangeReadProcess(ch, layer_gr_id, &read_chg);
            if (error != VDC5_OK)
            {
                GRAPHICS_DebugPrint(__FILE__, __LINE__, (int32_t)error);
                break;
            }
        }
        (void)osSemaphoreDelete(vdec_vsync);
    }
    /* Quit */
    Termination(ch);

}   /* End of function GRAPHICS_VideoSample() */

