/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2015 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/******************************************************************************
* File: window_surfaces.h
*    Window Surfaces API
*
* - $Version: $ (=WINDOW_SURFACES_VERSION)
* - $ModuleRevision: {Masters}\WindowSurfaces\RZ_NoEx\10_RStyle $
* - $Rev: $
* - $Date: 2017-07-21T19:22:49+09:00 $
******************************************************************************/

#ifndef  WINDOW_SURFACES_H
#define  WINDOW_SURFACES_H

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include  "window_surfaces_typedef.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/******************************************************************************
Typedef definitions
******************************************************************************/
/* In "frame_buffer_typedef.h" */

/******************************************************************************
Macro definitions
******************************************************************************/
/* In "frame_buffer_typedef.h" */

/******************************************************************************
Variable Externs
******************************************************************************/
/* In "frame_buffer_typedef.h" */

/******************************************************************************
Functions Prototypes
******************************************************************************/

/***********************************************************************
* Function: R_WINDOW_SURFACES_InitConst
*    R_WINDOW_SURFACES_InitConst
*
* Arguments:
*    self - window_surfaces_t
*
* Return Value:
*    None
************************************************************************/
void  R_WINDOW_SURFACES_InitConst( window_surfaces_t* const  self );


/***********************************************************************
* Function: R_WINDOW_SURFACES_Initialize
*    R_WINDOW_SURFACES_Initialize
*
* Arguments:
*    self          - window_surfaces_t
*    in_out_config - window_surfaces_config_t
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_WINDOW_SURFACES_Initialize( window_surfaces_t* const  self,
                                        window_surfaces_config_t* in_out_config );


/***********************************************************************
* Function: R_WINDOW_SURFACES_Finalize
*    R_WINDOW_SURFACES_Finalize
*
* Arguments:
*    self - window_surfaces_t
*    e   - Errors that have occurred. No error = 0
*
* Return Value:
*    Error code or e, 0 = successful and input e=0
************************************************************************/
errnum_t  R_WINDOW_SURFACES_Finalize( window_surfaces_t* const  self, errnum_t e );


/***********************************************************************
* Function: R_WINDOW_SURFACES_Dispose
*    Terminate display unit.
*
* Arguments:
*    e   - Errors that have occurred. No error = 0
*
* Return Value:
*    Error code or e, 0 = successful and input e=0
************************************************************************/
errnum_t  R_WINDOW_SURFACES_Dispose( window_surfaces_t* const  self,  errnum_t e );


/***********************************************************************
* Function: R_WINDOW_SURFACES_GetLayerFrameBuffer
*    Get <frame_buffer_t>* of specified layer number
*
* Arguments:
*    self             - window_surfaces_t
*    layer_num        - layer_num
*    out_frame_buffer - out_frame_buffer
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_WINDOW_SURFACES_GetLayerFrameBuffer( const window_surfaces_t* const  self,
        int_fast32_t const  layer_num,  frame_buffer_t** const  out_frame_buffer );


/***********************************************************************
* Function: R_WINDOW_SURFACES_SwapBuffers
*    R_WINDOW_SURFACES_SwapBuffers
*
* Arguments:
*    self         - window_surfaces_t
*    layer_num    - layer_num
*    null_context - NULL
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_WINDOW_SURFACES_SwapBuffers( window_surfaces_t* const  self,
        int_fast32_t const  layer_num,  const void* const  null_context );


/***********************************************************************
* Function: R_WINDOW_SURFACES_SwapBuffersStart
*    Asynchronous function of <R_WINDOW_SURFACES_SwapBuffers>
*
* Arguments:
*    self      - window_surfaces_t
*    layer_num - layer_num
*    async     - r_ospl_async_t
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_WINDOW_SURFACES_SwapBuffersStart( window_surfaces_t* const  self,
        int_fast32_t const  layer_num,  r_ospl_async_t* async );


/***********************************************************************
* Function: R_WINDOW_SURFACES_WaitForVSync
*    Wait for V-Sync
*
* Arguments:
*    swap_interval          - If 1=60fps Then 2=30fps, 3=20fps
*    is_1_v_sync_at_minimum - false = It is possible to return soon
*
* Return Value:
*    Error Code. 0=No Error.
************************************************************************/
errnum_t  R_WINDOW_SURFACES_WaitForVSync( window_surfaces_t* const  self,
        int_fast32_t const  swap_interval,  bool_t const  is_1_v_sync_at_minimum );


/***********************************************************************
* Function: R_WINDOW_SURFACES_AllocOffscreenStack
*    Allocate offscreen from stack
*
* Arguments:
*    self                - window_surfaces_t
*    in_out_frame_buffer - frame_buffer_t
*
* Return Value:
*    Error code, 0=No error
*
* Description:
*    (input) ->stride, ->height, ->buffer_count
*    (output) ->buffer_address[(all)]
************************************************************************/
errnum_t  R_WINDOW_SURFACES_AllocOffscreenStack( window_surfaces_t* const  self,
        frame_buffer_t* const  in_out_frame_buffer );


/***********************************************************************
* Function: R_WINDOW_SURFACES_FreeOffscreenStack
*    Free offscreen to stack
*
* Arguments:
*    self         - window_surfaces_t
*    frame_buffer - frame_buffer_t
*
* Return Value:
*    Error code, 0=No error
*
* Description:
*    If frame_buffer->buffer_count == 0, do nothing.
************************************************************************/
errnum_t  R_WINDOW_SURFACES_FreeOffscreenStack( window_surfaces_t* const  self,
        const frame_buffer_t* const  frame_buffer );


/***********************************************************************
* Class: vram_stack_t
*    Shared VRAM stack.
************************************************************************/


/***********************************************************************
* Function: R_VRAM_STACK_Initialize
*    R_VRAM_STACK_Initialize
*
* Arguments:
*    self          - vram_stack_t
*    in_out_Config - vram_stack_config_t
*
* Return Value:
*    Error Code. 0=No Error.
************************************************************************/
errnum_t  R_VRAM_STACK_Initialize( vram_stack_t* self,  vram_stack_config_t* in_out_Config );


/***********************************************************************
* Function: R_VRAM_STACK_AllocateMemory
*    R_VRAM_STACK_AllocateMemory
*
* Arguments:
*    self              - vram_stack_t
*    out_Address       - (out) Address
*    in_AllocatingSize - AllocatingSize
*
* Return Value:
*    Error Code. 0=No Error.
************************************************************************/
errnum_t  R_VRAM_STACK_AllocateMemory( vram_stack_t* self,
                                       void* /* byte** */  out_Address,  size_t  in_AllocatingSize );


/***********************************************************************
* Function: R_VRAM_STACK_FreeMemory
*    R_VRAM_STACK_FreeMemory
*
* Arguments:
*    self             - vram_stack_t
*    in_out_Address   - (in) AllocatedAddress, (out) NULL
*    in_AllocatedSize - AllocatedSize
*
* Return Value:
*    Error Code. 0=No Error.
************************************************************************/
errnum_t  R_VRAM_STACK_FreeMemory( vram_stack_t* self,
                                   void* /* byte** */  in_out_Address,  size_t  in_AllocatedSize );


#ifdef __cplusplus
}  /* extern "C" */
#endif /* __cplusplus */

#endif  /* WINDOW_SURFACES_H */
