/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2015 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/******************************************************************************
* File: window_surfaces_ex.h
*    Window Surfaces API
*
* - $Version: $ (=WINDOW_SURFACES_VERSION)
* - $ModuleRevision: {Masters}\WindowSurfaces\RZ_NoEx\10_RStyle $
* - $Rev: $
* - $Date: 2017-06-09T14:02:44+09:00 $
******************************************************************************/

#ifndef  WINDOW_SURFACES_EX_H
#define  WINDOW_SURFACES_EX_H

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include  "window_surfaces_ex_typedef.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/******************************************************************************
Typedef definitions
******************************************************************************/
/* In "frame_buffer_typedef.h" */

/******************************************************************************
Macro definitions
******************************************************************************/
/* In "frame_buffer_typedef.h" */

/******************************************************************************
Variable Externs
******************************************************************************/
/* In "frame_buffer_typedef.h" */

/******************************************************************************
Functions Prototypes
******************************************************************************/


/***********************************************************************
* Class: window_surfaces_t
*    Extra methods of <window_surfaces_t>
************************************************************************/

/***********************************************************************
* Function: R_WINDOW_SURFACES_DoMessageLoop
*    Emulate message loop function.
*
* Arguments:
*    self - window_surfaces_t
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_WINDOW_SURFACES_DoMessageLoop( window_surfaces_t* self );


/***********************************************************************
* Function: R_WINDOW_SURFACES_AccessLayerAttributes
*    Set or get by <layer_attributes_t>
*
* Arguments:
*    self              - window_surfaces_t
*    in_out_Attributes - <layer_attributes_t>
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_WINDOW_SURFACES_AccessLayerAttributes( window_surfaces_t* self,
        layer_attributes_t* in_out_Attributes );


/***********************************************************************
* Class: vram_ex_stack_t
*    VRAM stack in external RAM
************************************************************************/

/***********************************************************************
* Function: R_VRAM_EX_STACK_Initialize
*    R_VRAM_EX_STACK_Initialize
*
* Arguments:
*    self       - vram_ex_stack_t
*    NullConfig - NULL
*
* Return Value:
*    Error Code. 0=No Error.
************************************************************************/
#ifdef  USE_LIST_SDRAM
errnum_t  R_VRAM_EX_STACK_Initialize( vram_ex_stack_t* self, void* NullConfig );
#endif


/***********************************************************************
* Function: R_VRAM_EX_STACK_Alloc
*    Allocate VRAM from external RAM
*
* Arguments:
*    self               - vram_ex_stack_t
*    in_out_FrameBuffer - frame_buffer_t
*
* Return Value:
*    Error code, 0=No error
*
* Description:
*    (input) ->stride, ->height, ->buffer_count
*    (output) ->buffer_address[(all)]
************************************************************************/
#ifdef  USE_LIST_SDRAM
errnum_t  R_VRAM_EX_STACK_Alloc( vram_ex_stack_t* self, frame_buffer_t* in_out_FrameBuffer );
#endif


/***********************************************************************
* Function: R_VRAM_EX_STACK_Free
*    Free VRAM to external RAM
*
* Arguments:
*    self         - vram_ex_stack_t
*    frame_buffer - frame_buffer_t
*
* Return Value:
*    Error code, 0=No error
*
* Description:
*    If frame_buffer->buffer_count == 0, do nothing.
************************************************************************/
#ifdef  USE_LIST_SDRAM
errnum_t  R_VRAM_EX_STACK_Free( vram_ex_stack_t* self, frame_buffer_t* frame_buffer );
#endif


#ifdef __cplusplus
}  /* extern "C" */
#endif /* __cplusplus */

#endif  /* WINDOW_SURFACES_EX_H */
