/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2017 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/******************************************************************************
* File: vsync_pl.c
*    Porting Layer of V-Sync
*
* - $Version: $ (=WINDOW_SURFACES_VERSION)
* - $ModuleRevision: {Masters}\WindowSurfaces\RZ_NoEx\10_RStyle $
* - $Rev: $
* - $Date: 2017-03-14T19:39:34+09:00 $
******************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
*******************************************************************************/
#include  "r_ospl.h"
#include  "r_vdc5.h"
#include  "vsync.h"
#include  "vsync_pl.h"

#if ! R_OSPL_IS_PREEMPTION
#include  "dev_drv.h"
#endif

/******************************************************************************
Typedef definitions
******************************************************************************/


/***********************************************************************
* Type: r_v_sync_pl_channel_t
************************************************************************/
typedef struct st_r_v_sync_pl_channel_t  r_v_sync_pl_channel_t;
struct st_r_v_sync_pl_channel_t
{
    const r_ospl_caller_t*  InterruptCallbackCaller;
    bool_t                  IsVDC5_Callback;
};


/***********************************************************************
* Variable: gs_v_sync_pl_channel
************************************************************************/
static r_v_sync_pl_channel_t  gs_v_sync_pl_channel[ R_V_SYNC_CHANNEL_COUNT ];


/***********************************************************************
* Type: gs_cint_interrupt_t
************************************************************************/
#if  R_OSPL_INTERRUPT_HANDLER_IS == R_OSPL_INTERRUPT_HANDLER_IS_WITH_INT_SENSE
typedef bsp_int_cb_t  gs_cint_interrupt_t;
#else
typedef void (* gs_cint_interrupt_t )( uint32_t int_type );
#endif


/***********************************************************************
* Type: gs_vdc_interrupt_t
************************************************************************/
typedef void (* gs_vdc_interrupt_t )( vdc5_int_type_t int_type );


/******************************************************************************
Macro definitions
******************************************************************************/

/***********************************************************************
* Macro: GS_VSYNC_INTERRUPT_PRIORITY
*    GS_VSYNC_INTERRUPT_PRIORITY
************************************************************************/
enum { GS_VSYNC_INTERRUPT_PRIORITY = 25 };


/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/

/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/

/******************************************************************************
Private global variables and functions
******************************************************************************/
static void  R_V_SYNC_IRQ_HandlerN( int_fast32_t const  in_ChannelNum );
static void  R_V_SYNC_IRQ_Handler0( vdc5_int_type_t const  int_type );
static void  R_V_SYNC_IRQ_Handler1( vdc5_int_type_t const  int_type );
#if  R_OSPL_INTERRUPT_HANDLER_IS != R_OSPL_INTERRUPT_HANDLER_IS_WITH_INT_SENSE
static gs_cint_interrupt_t  gs_cint_vdc5_interrupt_handler[2];
static void  R_V_SYNC_IRQ_HandlerRoot0(void);
static void  R_V_SYNC_IRQ_HandlerRoot1(void);
#endif


/***********************************************************************
* Variable: gs_array_of_i_context
*    Table of interrupt line to channel number
************************************************************************/
static const r_ospl_interrupt_t  gs_array_of_i_context[ R_V_SYNC_CHANNEL_COUNT ] =
{
    { BSP_INT_SRC_GR3_VLINE0,  0 },
    { BSP_INT_SRC_GR3_VLINE1,  1 }
};


/***********************************************************************
* Implement: R_V_SYNC_SetDefaultAsync
************************************************************************/
void  R_V_SYNC_SetDefaultAsync( r_ospl_async_t* const  ref_Async,  r_ospl_async_type_t  in_AsyncType )
{
    R_UNREFERENCED_VARIABLE( in_AsyncType );

    IF_DQ( ref_Async == NULL )
    {
        goto fin;
    }

    R_OSPL_ASYNC_SetDefaultPreset( ref_Async );

    if ( IS_BIT_NOT_SET( ref_Async->Flags, R_F_OSPL_A_Thread ) )
    {
        ref_Async->A_Thread = R_OSPL_THREAD_NULL;
    }

    if ( IS_BIT_NOT_SET( ref_Async->Flags, R_F_OSPL_A_EventValue ) )
    {
        ref_Async->A_EventValue = R_OSPL_UNUSED_FLAG;
    }

    if ( IS_BIT_NOT_SET( ref_Async->Flags, R_F_OSPL_I_Thread ) )
    {
        ref_Async->I_Thread = R_OSPL_THREAD_NULL;
    }

    if ( IS_BIT_NOT_SET( ref_Async->Flags, R_F_OSPL_I_EventValue ) )
    {
        ref_Async->I_EventValue = R_OSPL_I_FLAG;
    }

    if ( IS_BIT_NOT_SET( ref_Async->Flags, R_F_OSPL_InterruptCallback ) )
    {
        ref_Async->InterruptCallback = (r_ospl_callback_t)&( R_V_SYNC_OnInterruptDefault );
    }
    /* MISRA 16.9 */

    ref_Async->Flags = R_F_OSPL_A_Thread | R_F_OSPL_A_EventValue |
                       R_F_OSPL_I_Thread | R_F_OSPL_I_EventValue |
                       R_F_OSPL_InterruptCallback | R_F_OSPL_Delegate;

fin:
    return;
}


/***********************************************************************
* Implement: R_V_SYNC_OnInitialize
************************************************************************/
errnum_t  R_V_SYNC_OnInitialize( int_fast32_t const  in_ChannelNum )
{
#if  R_OSPL_INTERRUPT_HANDLER_IS == R_OSPL_INTERRUPT_HANDLER_IS_WITH_INT_SENSE
    bsp_int_cb_t    handler;
#endif
    errnum_t       e;
    vdc5_error_t   er;
    bsp_int_err_t  eb;
    bsp_int_src_t  int_id;
    vdc5_int_t     interrupt;
    r_v_sync_pl_channel_t* const     self = &gs_v_sync_pl_channel[ in_ChannelNum ];
    const r_ospl_interrupt_t* const  i_context = &gs_array_of_i_context[ in_ChannelNum ];

    /*[gs_IRQHandlers]*/
    static const gs_vdc_interrupt_t  gs_IRQHandlers[ R_V_SYNC_CHANNEL_COUNT ] =
    {
        &( R_V_SYNC_IRQ_Handler0 ), /* MISRA 16.9 */
        &( R_V_SYNC_IRQ_Handler1 )
    };

#if  R_OSPL_INTERRUPT_HANDLER_IS != R_OSPL_INTERRUPT_HANDLER_IS_WITH_INT_SENSE
    /*[gs_IRQHandlerRoots]*/
    static const bsp_int_cb_t  gs_IRQHandlerRoots[ R_V_SYNC_CHANNEL_COUNT ] =
    {
        &( R_V_SYNC_IRQ_HandlerRoot0 ), /* MISRA 16.9 */
        &( R_V_SYNC_IRQ_HandlerRoot1 )
    };
#endif


    IF ( (in_ChannelNum < 0) || (in_ChannelNum >= R_V_SYNC_CHANNEL_COUNT) )
    {
        e=E_OTHERS;
        goto fin;
    }

    self->InterruptCallbackCaller = NULL;

    int_id  = i_context->IRQ_Num;
#if  R_OSPL_INTERRUPT_HANDLER_IS == R_OSPL_INTERRUPT_HANDLER_IS_WITH_INT_SENSE
    handler = R_VDC5_GetISR( (vdc5_channel_t) in_ChannelNum, VDC5_INT_TYPE_VLINE );
    eb= R_BSP_InterruptWrite( int_id, handler );
#else
    gs_cint_vdc5_interrupt_handler[in_ChannelNum] = R_VDC5_GetISR(
                (vdc5_channel_t) in_ChannelNum, VDC5_INT_TYPE_VLINE );
    eb= R_BSP_InterruptWrite( int_id,  (bsp_int_cb_t) gs_IRQHandlerRoots[in_ChannelNum] );
#endif
    IF ( eb != BSP_INT_SUCCESS )
    {
        e=E_OTHERS;
        goto fin;
    }

    e= R_OSPL_SetInterruptPriority( int_id, GS_VSYNC_INTERRUPT_PRIORITY );
    IF ( e != 0 )
    {
        goto fin;
    }

    interrupt.type     = VDC5_INT_TYPE_VLINE;
    interrupt.callback = gs_IRQHandlers[ in_ChannelNum ];
    interrupt.line_num = (uint16_t) 0;

    er = R_VDC5_CallbackISR( (vdc5_channel_t) in_ChannelNum, &interrupt );
    IF ( er != VDC5_OK )
    {
        e=E_OTHERS;
        goto fin;
    }

    self->IsVDC5_Callback = true;

    e=0;
fin:
    return  e;
}


/***********************************************************************
* Implement: R_V_SYNC_OnFinalize
************************************************************************/
errnum_t  R_V_SYNC_OnFinalize( int_fast32_t const  in_ChannelNum,  errnum_t e )
{
    r_v_sync_pl_channel_t* const  self = &gs_v_sync_pl_channel[ in_ChannelNum ];

    if ( (in_ChannelNum >= 0) && (in_ChannelNum < R_V_SYNC_CHANNEL_COUNT) )
    {
        if ( IS( self->IsVDC5_Callback ) )
        {
            vdc5_error_t  er;
            vdc5_int_t    interrupt;

            interrupt.type     = VDC5_INT_TYPE_VLINE;
            interrupt.callback = NULL;
            interrupt.line_num = (uint16_t) 0;

            er = R_VDC5_CallbackISR( (vdc5_channel_t) in_ChannelNum, &interrupt );
            IF ( (er != VDC5_OK) && (e == 0) )
            {
                e=E_OTHERS;
            }

            self->IsVDC5_Callback = false;
        }
    }

    return  e;
}


/***********************************************************************
* Implement: R_V_SYNC_SetInterruptCallbackCaller
************************************************************************/
errnum_t  R_V_SYNC_SetInterruptCallbackCaller( int_fast32_t const  in_ChannelNum,
        const r_ospl_caller_t* const  Caller )
{
    r_v_sync_pl_channel_t* const  self = &gs_v_sync_pl_channel[ in_ChannelNum ];

    self->InterruptCallbackCaller = Caller;

    return  0;
}


/***********************************************************************
* Implement: R_V_SYNC_OnEnableInterrupt
************************************************************************/
void  R_V_SYNC_OnEnableInterrupt( int_fast32_t const  in_ChannelNum,
                                  r_v_sync_interrupt_lines_t const  Enables )
{
    bsp_int_err_t  eb;

    if ( IS_BIT_SET( Enables, R_V_SYNC_INTERRUPT_LINE_V_LINE ) )
    {
        bsp_int_src_t const  num_of_IRQ = gs_array_of_i_context[ in_ChannelNum ].IRQ_Num;

        eb= R_BSP_InterruptControl( num_of_IRQ, BSP_INT_CMD_INTERRUPT_ENABLE, FIT_NO_PTR );
        ASSERT_D( eb == 0,  R_NOOP() );
        R_UNREFERENCED_VARIABLE( eb ); /* for Release configuration */
    }
}


/***********************************************************************
* Implement: R_V_SYNC_OnDisableInterrupt
************************************************************************/
void  R_V_SYNC_OnDisableInterrupt( int_fast32_t const  in_ChannelNum,
                                   r_v_sync_interrupt_lines_t const  Disables1 )
{
    bsp_int_err_t  eb;

    if ( IS_BIT_SET( Disables1, R_V_SYNC_INTERRUPT_LINE_V_LINE ) )
    {
        bsp_int_src_t const  num_of_IRQ = gs_array_of_i_context[ in_ChannelNum ].IRQ_Num;

        eb= R_BSP_InterruptControl( num_of_IRQ, BSP_INT_CMD_INTERRUPT_DISABLE, FIT_NO_PTR );
        ASSERT_D( eb == 0,  R_NOOP() );
#if  R_OSPL_ERROR_BREAK
        if ( eb != 0 )
        {
            R_OSPL_CLEAR_ERROR();
        }
#endif
        R_UNREFERENCED_VARIABLE( eb ); /* for Release configuration */
    }
}


/***********************************************************************
* Implement: R_V_SYNC_OnInterruptDefault
************************************************************************/
errnum_t  R_V_SYNC_OnInterruptDefault( const r_ospl_interrupt_t* const  InterruptSource,
                                       const r_ospl_caller_t* const  Caller )
{
    R_UNREFERENCED_VARIABLE( Caller );

    return  R_V_SYNC_OnInterrupting( InterruptSource );
}


/***********************************************************************
* Function: R_V_SYNC_IRQ_HandlerN
*    Interrupt service routine for all channels
*
* Arguments:
*   in_ChannelNum - ChannelNum
*
* Return Value:
*    None
************************************************************************/
static void  R_V_SYNC_IRQ_HandlerN( int_fast32_t const  in_ChannelNum )
{
    r_v_sync_pl_channel_t* const     self = &gs_v_sync_pl_channel[ in_ChannelNum ];
    const r_ospl_interrupt_t* const  i_context = &gs_array_of_i_context[ in_ChannelNum ];

    /* V-Sync interrupt always be enabled. Because interrupt status must not be suspended. */

    if ( self->InterruptCallbackCaller != NULL )
    {
        R_OSPL_CallInterruptCallback( self->InterruptCallbackCaller, i_context );
    }
}


/***********************************************************************
* Function: R_V_SYNC_IRQ_Handler0
*    Interrupt service routine for INTC channel 0. gs_vdc_interrupt_t.
*
* Arguments:
*    int_type - ignored
*
* Return Value:
*    None
************************************************************************/
static void  R_V_SYNC_IRQ_Handler0( vdc5_int_type_t const  int_type )
{
    R_UNREFERENCED_VARIABLE( int_type );
    R_V_SYNC_IRQ_HandlerN( 0 );
}


/***********************************************************************
* Function: R_V_SYNC_IRQ_Handler1
*    Interrupt service routine for INTC channel 1. gs_vdc_interrupt_t.
*
* Arguments:
*    int_type - ignored
*
* Return Value:
*    None
************************************************************************/
static void  R_V_SYNC_IRQ_Handler1( vdc5_int_type_t const  int_type )
{
    R_UNREFERENCED_VARIABLE( int_type );
    R_V_SYNC_IRQ_HandlerN( 1 );
}


/***********************************************************************
* Function: R_V_SYNC_IRQ_HandlerRoot0
*    Interrupt service routine for RTX channel 0
*
* Arguments:
*    None
*
* Return Value:
*    None
************************************************************************/
#if  R_OSPL_INTERRUPT_HANDLER_IS == R_OSPL_INTERRUPT_HANDLER_IS_FOR_RTX
static void  R_V_SYNC_IRQ_HandlerRoot0( R_OSPL_INTERRUPT_ARGUMENTS )
{
    gs_cint_vdc5_interrupt_handler[0]( 0 );
    R_OSPL_END_OF_LEVEL_INTERRUPT();
}
#endif


/***********************************************************************
* Function: R_V_SYNC_IRQ_HandlerRoot1
*    Interrupt service routine for RTX channel 1
*
* Arguments:
*    None
*
* Return Value:
*    None
************************************************************************/
#if  R_OSPL_INTERRUPT_HANDLER_IS == R_OSPL_INTERRUPT_HANDLER_IS_FOR_RTX
static void  R_V_SYNC_IRQ_HandlerRoot1( R_OSPL_INTERRUPT_ARGUMENTS )
{
    gs_cint_vdc5_interrupt_handler[1]( 0 );
    R_OSPL_END_OF_LEVEL_INTERRUPT();
}
#endif


