/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2017 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/******************************************************************************
* File: jcu_mt_pl.c
*    JPEG Codec Unit (JCU) Sample Driver. Internal Thread Porting Layer.
*
* - $Version: $ (=JCU_VERSION)
* - $ModuleRevision: {Masters}\JCU\RZ\48_RStyle2 $
* - $Rev: $
* - $Date: 2017-12-25T18:35:51+09:00 $
******************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include  "use_list.h"
#include  "r_jcu_user.h"
#include  "r_jcu_api.h"
#include  "../src/r_jcu_pl.h"
#include  "../src/r_jcu_local.h"
#include  "r_ospl.h"
#ifdef  USE_LIST_FREE_RTOS
#include  "FreeRTOS.h"
#include  "task.h"
#endif
#ifdef  USE_LIST_CMSIS
#include  "cmsis_os.h"
#endif

#if  JCU_MULTI_THREAD


/******************************************************************************
Typedef definitions
******************************************************************************/

/******************************************************************************
Macro definitions
******************************************************************************/

/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/

/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/
/* Section: Internal Global */

/******************************************************************************
Private global variables and functions
******************************************************************************/

/***********************************************************************
* Variable: gs_OperatorConfig
************************************************************************/
static jcu_operator_config_t  gs_OperatorConfig;


/***********************************************************************
* Variable: gs_ContextMemory
************************************************************************/
static  jcu_context_t     gs_ContextMemoryArray[4];  /* Variable: gs_ContextMemoryArray */
static  R_OSPL_TABLE_DEF( gs_ContextMemory,  R_COUNT_OF( gs_ContextMemoryArray ),  R_OSPL_TABLE_T_LOCK );


/***********************************************************************
* Variable: gs_ThreadMemory
************************************************************************/
static  jcu_thread_t      gs_ThreadMemoryArray[4];  /* Variable: gs_ThreadMemoryArray */
static  R_OSPL_TABLE_DEF( gs_ThreadMemory,  R_COUNT_OF( gs_ThreadMemoryArray ),  R_OSPL_TABLE_T_LOCK );


/***********************************************************************
* Variable: gs_O_ThreadId
************************************************************************/
r_ospl_thread_id_t  gs_O_ThreadId = R_OSPL_THREAD_NULL;
#if ! R_OSPL_IS_PREEMPTION
R_OSPL_THREAD_DEF( gs_O_Thread_Def );  /* Variable: gs_O_Thread_Def */
#elif  defined( osCMSIS )
osThreadDef( R_JCU_OPERATOR_Thread,  osPriorityNormal,  1,  0 );
#endif


/***********************************************************************
* Variable: gs_Queue
************************************************************************/
static R_OSPL_QUEUE_DEF( gs_Queue,  3,  jcu_message_t );


/***********************************************************************
* Variable: gs_ThreadsLock
************************************************************************/
static BSP_CFG_USER_LOCKING_TYPE  gs_ThreadsLock;


/***********************************************************************
* Function: R_JCU_CreateOperatorThread
*    R_JCU_CreateOperatorThread.
*
* Arguments:
*    None
*
* Return Value:
*    Error code, 0=No error
*
* Description:
*    This code is executed in NOT operator thread.
************************************************************************/
errnum_t  R_JCU_CreateOperatorThread()
{
    errnum_t  e, ee;

    jcu_contexts_config_t  contexts_config;
    r_ospl_thread_id_t     caller_thread = R_OSPL_THREAD_GetCurrentId();
    r_ospl_async_t*        ready_event = &gs_OperatorConfig.ReadyOrFinishedEvent;

    ready_event->A_EventValue = 0;  /* For fin block */

    if ( gs_O_ThreadId == R_OSPL_THREAD_NULL )
    {


        /* Initialize contexts. */
        contexts_config.Flags =
            F_JCU_ThreadMemory |
            F_JCU_ThreadMemoryArray;
        contexts_config.ThreadMemory = R_OSPL_TABLE( gs_ThreadMemory );
        contexts_config.ThreadMemoryArray = gs_ThreadMemoryArray;

        e= R_JCU_CONTEXTS_Initialize( &contexts_config );
        IF(e)
        {
            goto fin;
        }


        /* Initialize operator thread. */
        gs_OperatorConfig.Flags =
            F_JCU_MultiThreadQueue | F_JCU_ReadyOrFinishedEvent;
        if ( gs_OperatorConfig.MultiThreadQueue == R_OSPL_QUEUE_NULL )
        {
            e= R_OSPL_QUEUE_Create( &gs_OperatorConfig.MultiThreadQueue,  R_OSPL_QUEUE( gs_Queue ) );
            IF(e)
            {
                goto fin;
            }
        }
        ready_event->Flags = R_F_OSPL_A_Thread | R_F_OSPL_A_EventValue;
        e= R_OSPL_EVENT_Allocate(
               /*Set*/ &ready_event->A_Thread,      R_OSPL_THREAD_GetCurrentId(),
               /*Set*/ &ready_event->A_EventValue,  R_OSPL_UNUSED_FLAG );
        IF(e)
        {
            goto fin;
        }


#if  !  R_OSPL_IS_PREEMPTION

        e= R_OSPL_THREAD_Create( R_OSPL_THREAD( gs_O_Thread_Def ),
                                 &gs_OperatorConfig,  /*Set*/ &gs_O_ThreadId );
        IF(e)
        {
            goto fin;
        }
        R_JCU_OPERATOR_Step0( gs_O_ThreadId );

        e= R_JCU_OPERATOR_Dispatch( gs_O_ThreadId );
        IF(e)
        {
            goto fin;
        }

#elif  defined( tskKERNEL_VERSION_NUMBER )
        {
            BaseType_t  er;  /* An error code */

            er= xTaskCreate( R_JCU_OPERATOR_Thread,  "R_JCU_OPERATOR_Thread",  1000,
                             &gs_OperatorConfig,  tskIDLE_PRIORITY + 1,  NULL );
            IF(er!=pdPASS)
            {
                goto fin;
            }
        }
#elif  defined( osCMSIS )
        gs_O_ThreadId = osThreadCreate( osThread( R_JCU_OPERATOR_Thread ),
                                        &gs_OperatorConfig );
        IF ( gs_O_ThreadId == NULL )
        {
            e=E_OTHERS;
            goto fin;
        }
#else
#error  Not implemented to call other OS API
#endif


        e= R_OSPL_EVENT_Wait( ready_event->A_EventValue,  NULL,  R_OSPL_INFINITE );
        IF(e)
        {
            goto fin;
        }
    }

    e=0;
fin:
    if ( ready_event != NULL )
    {
        ee= R_OSPL_EVENT_Free( &ready_event->A_Thread,  &ready_event->A_EventValue );
        e= R_OSPL_MergeErrNum( e, ee );
    }
    return  e;
}


/***********************************************************************
* Function: R_JCU_DestroyOperatorThread
*    R_JCU_DestroyOperatorThread.
*
* Arguments:
*    None
*
* Return Value:
*    Error code, 0=No error
*
* Description:
*    This code is executed in NOT operator thread.
************************************************************************/
errnum_t  R_JCU_DestroyOperatorThread()
{
    errnum_t  e, ee;

    r_ospl_thread_id_t  operator_thread = R_JCU_GetOperatorThreadId();
    r_ospl_thread_id_t  caller_thread = R_OSPL_THREAD_GetCurrentId();
    r_ospl_async_t*     finished_event = &gs_OperatorConfig.ReadyOrFinishedEvent;


    e=0;

    if ( gs_O_ThreadId != R_OSPL_THREAD_NULL )
    {
        finished_event->Flags = R_F_OSPL_A_Thread | R_F_OSPL_A_EventValue;
        ee= R_OSPL_EVENT_Allocate(
                /*Set*/ &finished_event->A_Thread,      R_OSPL_THREAD_GetCurrentId(),
                /*Set*/ &finished_event->A_EventValue,  R_OSPL_UNUSED_FLAG );
        IF(ee)
        {
            R_OSPL_RaiseUnrecoverable( ee );
        }


        ee= R_JCU_OPERATOR_FinalizeStart( finished_event );
        e= R_OSPL_MergeErrNum( e, ee );

#if  !  R_OSPL_IS_PREEMPTION
        for (;;)
        {
            bit_flags32_t  got_flags;

            ee= R_JCU_OPERATOR_Dispatch( gs_O_ThreadId );
            e= R_OSPL_MergeErrNum( e, ee );

            ee= R_OSPL_EVENT_Wait( finished_event->A_EventValue,  &got_flags,  R_OSPL_INFINITE_PSEUDO );
            e= R_OSPL_MergeErrNum( e, ee );

            if ( IS_BIT_SET( got_flags,  finished_event->A_EventValue ) )
            {
                break;
            }
        }
#else
        ee= R_OSPL_EVENT_Wait( finished_event->A_EventValue,  NULL,  R_OSPL_INFINITE );
        e= R_OSPL_MergeErrNum( e, ee );
#endif

        ee= R_OSPL_EVENT_Free( &finished_event->A_Thread,  &finished_event->A_EventValue );
        e= R_OSPL_MergeErrNum( e, ee );

#if  !  R_OSPL_IS_PREEMPTION
        ee= R_OSPL_THREAD_Destroy( &gs_O_ThreadId );
        e= R_OSPL_MergeErrNum( e, ee );

#elif  defined( tskKERNEL_VERSION_NUMBER )
        /* "vTaskDelete" is called from "THREAD_Thread" function. */
        gs_O_ThreadId = R_OSPL_THREAD_NULL;

#elif  defined( osCMSIS )
        /* It is not necessary to call "osThreadTerminate". */
        gs_O_ThreadId = R_OSPL_THREAD_NULL;
#else
#error  Not implemented to call other OS API
        gs_O_ThreadId = R_OSPL_THREAD_NULL;
#endif
    }

    return  e;
}


/***********************************************************************
* Function: R_JCU_OnAllocateContext
*    Allocates a memory area of F-context.
*
* Arguments:
*    out_AddressOfContext - jcu_context_t**
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_JCU_OnAllocateContext( jcu_context_t**  out_AddressOfContext )
{
    errnum_t      e;
    int_fast32_t  index;

    e= R_OSPL_TABLE_GetIndex( R_OSPL_TABLE( gs_ContextMemory ),  NULL,
                              &index,  R_OSPL_ALLOCATE_IF_EXIST_OR_IF_NOT );
    IF(e)
    {
        goto fin;
    }

    *out_AddressOfContext = &gs_ContextMemoryArray[ index ];

    e=0;
fin:
    return  e;
}


/***********************************************************************
* Function: R_JCU_OnFreeContext
*    Allocates a memory area of F-context.
*
* Arguments:
*    in_out_AddressOfContext - (in) freeing address, (out) NULL
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_JCU_OnFreeContext( jcu_context_t**  in_out_AddressOfContext )
{
    int_fast32_t  index;

    index = *in_out_AddressOfContext - &gs_ContextMemoryArray[0];

    R_OSPL_TABLE_FreeByIndex( R_OSPL_TABLE( gs_ContextMemory ),  index );

    *in_out_AddressOfContext = NULL;

    return  0;
}


#endif  /* JCU_MULTI_THREAD */
