/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2016 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/***********************************************************************
* File: locking.h
*    Lock related FIT BSP. Functions.
*
* - $Version: $ (=R_OSPL_VERSION)
* - $ModuleRevision: {Masters}\OSPL\ALL_Driver_parent\27_RStyled $
* - $Rev: $
* - $Date: 2017-08-29T10:34:42+09:00 $
************************************************************************/

#ifndef LOCKING_H
#define LOCKING_H


/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include  "r_ospl_config.h"
#if  BSP_CFG_USER_LOCKING_ENABLED
#include  "locking_user.h"
#endif
#include  "r_typedefs.h"
#include  "locking_typedef.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/******************************************************************************
Typedef definitions
******************************************************************************/
/* In "locking_typedef.h" */

/******************************************************************************
Macro definitions
******************************************************************************/
/* In "locking_typedef.h" */

/******************************************************************************
Variable Externs
******************************************************************************/
/* In "locking_typedef.h" */

/******************************************************************************
Functions Prototypes
******************************************************************************/

/***********************************************************************
* Function: R_BSP_HardwareLock
*    Attempts to reserve a hardware peripheral lock.
*
* Arguments:
*    HardwareIndex - Index of lock to acquire from the hardware lock array.
*
* Return Value:
*    true = Successful, false = Failure
************************************************************************/
#if  R_OSPL_C_LOCK_CODE
bool_t  R_BSP_HardwareLock( mcu_lock_t const  HardwareIndex );
#endif


/***********************************************************************
* Function: R_BSP_HardwareUnlock
*    Releases a hardware peripheral lock.
*
* Arguments:
*    HardwareIndex - Index of lock to release from the hardware lock array.
*
* Return Value:
*    true = Successful, false = Failure
************************************************************************/
#if  R_OSPL_C_LOCK_CODE
bool_t  R_BSP_HardwareUnlock( mcu_lock_t const  HardwareIndex );
#endif


/***********************************************************************
* Function: R_BSP_SoftwareLock
*    Attempts to reserve a lock.
*
* Arguments:
*    LockObject - Pointer to lock structure with lock to try and acquire.
*
* Return Value:
*    true = Successful, false = Failure
************************************************************************/
#if  R_OSPL_C_LOCK_CODE
bool_t  R_BSP_SoftwareLock( BSP_CFG_USER_LOCKING_TYPE* const  LockObject );
#endif


/***********************************************************************
* Function: R_BSP_SoftwareUnlock
*    Releases a lock.
*
* Arguments:
*    LockObject - Pointer to lock structure with lock to release.
*
* Return Value:
*    true = Successful, false = Failure
************************************************************************/
#if  R_OSPL_C_LOCK_CODE
bool_t  R_BSP_SoftwareUnlock( BSP_CFG_USER_LOCKING_TYPE* const  LockObject );
#endif


/***********************************************************************
* End of File:
************************************************************************/
#ifdef __cplusplus
}  /* extern "C" */
#endif /* __cplusplus */

#endif /* LOCKING_H */

