/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2016 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/***********************************************************************
* File: mcu_interrupts.h
*    Interrupt related FIT BSP. Functions.
*
* - $Version: $ (=R_OSPL_VERSION)
* - $ModuleRevision: {Masters}\OSPL\ALL_Driver_parent\27_RStyled $
* - $Rev: $
* - $Date: 2017-08-29T10:34:42+09:00 $
************************************************************************/

#ifndef MCU_INTERRUPTS_H
#define MCU_INTERRUPTS_H


/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include  "r_ospl_config.h"
#include  "mcu_interrupts_typedef.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/******************************************************************************
Typedef definitions
******************************************************************************/
/* In "mcu_interrupts_typedef.h" */

/******************************************************************************
Macro definitions
******************************************************************************/
/* In "mcu_interrupts_typedef.h" */

/******************************************************************************
Variable Externs
******************************************************************************/
/* In "mcu_interrupts_typedef.h" */

/******************************************************************************
Functions Prototypes
******************************************************************************/

/***********************************************************************
* Function: R_BSP_InterruptWrite
*    Registers a interrupt handler
*
* Arguments:
*    in_IRQ_Num  - Interrupt request number
*    in_Callback - The function as interrupt handler
*
* Return Value:
*    Error code.  If there is no error, the return value is BSP_INT_SUCCESS.
************************************************************************/
#if  R_OSPL_BASE_INTERRUPT_CODE
bsp_int_err_t  R_BSP_InterruptWrite( bsp_int_src_t const  in_IRQ_Num,  bsp_int_cb_t const  in_Callback );
#endif


/***********************************************************************
* Function: R_BSP_InterruptRead
*    Returns registered interrupt handler
*
* Arguments:
*    in_IRQ_Num   - Interrupt request number
*    out_Callback - Output: the function as interrupt handler
*
* Return Value:
*    Error code.  If there is no error, the return value is BSP_INT_SUCCESS.
************************************************************************/
#if  R_OSPL_BASE_INTERRUPT_CODE
bsp_int_err_t  R_BSP_InterruptRead( bsp_int_src_t const  in_IRQ_Num,  bsp_int_cb_t* const  out_Callback );
#endif


/***********************************************************************
* Function: R_BSP_InterruptControl
*    Controls related to the interrupt
*
* Arguments:
*    in_IRQ_Num - Interrupt request number
*    in_Command - Control command. See <bsp_int_cmd_t>
*    NotUsed    - Depend on "Command" argument
*
* Return Value:
*    Error code.  If there is no error, the return value is BSP_INT_SUCCESS.
************************************************************************/
#if  R_OSPL_BASE_INTERRUPT_CODE
bsp_int_err_t  R_BSP_InterruptControl( bsp_int_src_t const  in_IRQ_Num,  bsp_int_cmd_t const  in_Command,
                                       void* const  NotUsed );
#endif


/***********************************************************************
* End of File:
************************************************************************/
#ifdef __cplusplus
}  /* extern "C" */
#endif /* __cplusplus */

#endif /* MCU_INTERRUPTS_H */

