/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2016 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/***********************************************************************
* File: mcu_interrupts_typedef.h
*    Interrupt related FIT BSP. Data types.
*
* - $Version: $ (=R_OSPL_VERSION)
* - $ModuleRevision: {Masters}\OSPL\ALL_Driver_parent\27_RStyled $
* - $Rev: $
* - $Date: 2017-08-29T10:34:42+09:00 $
************************************************************************/

#ifndef MCU_INTERRUPTS_TYPEDEF_H
#define MCU_INTERRUPTS_TYPEDEF_H


/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include  "use_list.h"
#include  "r_ospl_config.h"
#include  "r_typedefs.h"
#ifdef  R_OSPL_FOR_RTX
#include  "r_multi_compiler_typedef.h"
#endif
#ifdef  USE_LIST_MBED
#include  "MBRZA1H.h"
#endif

#ifdef  USE_LIST_RZA1H_BSP
#ifdef  __ICCARM__
#ifdef  __cplusplus
/* intrinsic function "__get_PSR" called from "cmsis_iar.h" is not declared in C++. */
#include  <stdint.h>
#include  "r_ospl_typedef.h"
extern "C" void  R_OSPL_RaiseUnrecoverable( errnum_t const  e );
static uint32_t __get_PSR(void)
{
    R_OSPL_RaiseUnrecoverable( 1 );
    return 0;
}
#endif
#endif
#include  "Renesas_RZ_A1.h"
#endif

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/******************************************************************************
Typedef definitions
******************************************************************************/

/***********************************************************************
* Enumeration: bsp_int_err_t
*    Error code defined by FIT BSP.
*
*    : BSP_INT_SUCCESS                    - 0
*    : BSP_INT_ERR_NO_REGISTERED_CALLBACK - 0x2101
*    : BSP_INT_ERR_INVALID_ARG            - 1
*    : BSP_INT_ERR_UNSUPPORTED            - 15
************************************************************************/
#if  R_OSPL_BASE_INTERRUPT_CODE
typedef enum {
    BSP_INT_SUCCESS = 0,
    BSP_INT_ERR_NO_REGISTERED_CALLBACK = 0x2101,
    BSP_INT_ERR_INVALID_ARG = 1,
    BSP_INT_ERR_UNSUPPORTED = 15
}
                          bsp_int_err_t;
#endif


/***********************************************************************
* Function: bsp_int_cb_t
*    Type of callback function from interrupt.
*
* Arguments:
*    int_sense - uint32_t
*
* Return Value:
*    None
************************************************************************/
#if  R_OSPL_BASE_INTERRUPT_CODE
#if  R_OSPL_INTERRUPT_HANDLER_IS == R_OSPL_INTERRUPT_HANDLER_IS_WITH_INT_SENSE
typedef void  (* bsp_int_cb_t )( uint32_t const  int_sense );
#else
typedef void  (* bsp_int_cb_t )(void);
#endif
#endif


/***********************************************************************
* Enumeration: bsp_int_cmd_t
*    Control command related to the interrupt.
*
*    : BSP_INT_CMD_INTERRUPT_ENABLE  - 0
*    : BSP_INT_CMD_INTERRUPT_DISABLE - 1
************************************************************************/
#if  R_OSPL_BASE_INTERRUPT_CODE
typedef enum
{
    BSP_INT_CMD_INTERRUPT_ENABLE,
    BSP_INT_CMD_INTERRUPT_DISABLE
} bsp_int_cmd_t;
#endif


/******************************************************************************
Macro definitions
******************************************************************************/

/***********************************************************************
* Constant: FIT_NO_PTR
*    None as pointer type
************************************************************************/
#define  FIT_NO_PTR  ( (void*) 0 )


/***********************************************************************
* Constant: FIT_NO_FUNC
*    None as function pointer type
************************************************************************/
#define  FIT_NO_FUNC ( (void*) 0 )


/***********************************************************************
* Type: bsp_int_src_t
*    Type of BSP_INT_SRC_* macro.
************************************************************************/
#if  R_OSPL_BASE_INTERRUPT_CODE
#if  defined( R_OSPL_FOR_RTX )
typedef IRQn_Type  bsp_int_src_t;
#elif  defined( R_OSPL_FOR_RZ_A1_INTC ) || defined( R_OSPL_FOR_RZ_A1_INTC_RSK )
typedef uint16_t  bsp_int_src_t;
#else
#error
#endif
#endif


/***********************************************************************
* Macros: BSP_INT_SRC_*
*    Interrupt number.
************************************************************************/
#if  R_OSPL_BASE_INTERRUPT_CODE
#define BSP_INT_SRC_SW0             ((bsp_int_src_t)(0))     /* GIC software interrupt             */
#define BSP_INT_SRC_SW1             ((bsp_int_src_t)(1))     /*                                    */
#define BSP_INT_SRC_SW2             ((bsp_int_src_t)(2))     /*                                    */
#define BSP_INT_SRC_SW3             ((bsp_int_src_t)(3))     /*                                    */
#define BSP_INT_SRC_SW4             ((bsp_int_src_t)(4))     /*                                    */
#define BSP_INT_SRC_SW5             ((bsp_int_src_t)(5))     /*                                    */
#define BSP_INT_SRC_SW6             ((bsp_int_src_t)(6))     /*                                    */
#define BSP_INT_SRC_SW7             ((bsp_int_src_t)(7))     /*                                    */
#define BSP_INT_SRC_SW8             ((bsp_int_src_t)(8))     /*                                    */
#define BSP_INT_SRC_SW9             ((bsp_int_src_t)(9))     /*                                    */
#define BSP_INT_SRC_SW10            ((bsp_int_src_t)(10))    /*                                    */
#define BSP_INT_SRC_SW11            ((bsp_int_src_t)(11))    /*                                    */
#define BSP_INT_SRC_SW12            ((bsp_int_src_t)(12))    /*                                    */
#define BSP_INT_SRC_SW13            ((bsp_int_src_t)(13))    /*                                    */
#define BSP_INT_SRC_SW14            ((bsp_int_src_t)(14))    /*                                    */
#define BSP_INT_SRC_SW15            ((bsp_int_src_t)(15))    /*                                    */
#define BSP_INT_SRC_PMUIRQ0         ((bsp_int_src_t)(16))    /* CPU                                */
#define BSP_INT_SRC_COMMRX0         ((bsp_int_src_t)(17))    /*                                    */
#define BSP_INT_SRC_COMMTX0         ((bsp_int_src_t)(18))    /*                                    */
#define BSP_INT_SRC_CTIIRQ0         ((bsp_int_src_t)(19))    /*                                    */
#define BSP_INT_SRC_IRQ0            ((bsp_int_src_t)(32))    /* IRQ                                */
#define BSP_INT_SRC_IRQ1            ((bsp_int_src_t)(33))    /*                                    */
#define BSP_INT_SRC_IRQ2            ((bsp_int_src_t)(34))    /*                                    */
#define BSP_INT_SRC_IRQ3            ((bsp_int_src_t)(35))    /*                                    */
#define BSP_INT_SRC_IRQ4            ((bsp_int_src_t)(36))    /*                                    */
#define BSP_INT_SRC_IRQ5            ((bsp_int_src_t)(37))    /*                                    */
#define BSP_INT_SRC_IRQ6            ((bsp_int_src_t)(38))    /*                                    */
#define BSP_INT_SRC_IRQ7            ((bsp_int_src_t)(39))    /*                                    */
#define BSP_INT_SRC_PL310ERR        ((bsp_int_src_t)(40))    /* Level 2 cache                      */
#define BSP_INT_SRC_DMAINT0         ((bsp_int_src_t)(41))    /* Direct memory access controller    */
#define BSP_INT_SRC_DMAINT1         ((bsp_int_src_t)(42))    /*                                    */
#define BSP_INT_SRC_DMAINT2         ((bsp_int_src_t)(43))    /*                                    */
#define BSP_INT_SRC_DMAINT3         ((bsp_int_src_t)(44))    /*                                    */
#define BSP_INT_SRC_DMAINT4         ((bsp_int_src_t)(45))    /*                                    */
#define BSP_INT_SRC_DMAINT5         ((bsp_int_src_t)(46))    /*                                    */
#define BSP_INT_SRC_DMAINT6         ((bsp_int_src_t)(47))    /*                                    */
#define BSP_INT_SRC_DMAINT7         ((bsp_int_src_t)(48))    /*                                    */
#define BSP_INT_SRC_DMAINT8         ((bsp_int_src_t)(49))    /*                                    */
#define BSP_INT_SRC_DMAINT9         ((bsp_int_src_t)(50))    /*                                    */
#define BSP_INT_SRC_DMAINT10        ((bsp_int_src_t)(51))    /*                                    */
#define BSP_INT_SRC_DMAINT11        ((bsp_int_src_t)(52))    /*                                    */
#define BSP_INT_SRC_DMAINT12        ((bsp_int_src_t)(53))    /*                                    */
#define BSP_INT_SRC_DMAINT13        ((bsp_int_src_t)(54))    /*                                    */
#define BSP_INT_SRC_DMAINT14        ((bsp_int_src_t)(55))    /*                                    */
#define BSP_INT_SRC_DMAINT15        ((bsp_int_src_t)(56))    /*                                    */
#define BSP_INT_SRC_DMAERR          ((bsp_int_src_t)(57))    /*                                    */
#define BSP_INT_SRC_USBI0           ((bsp_int_src_t)(73))    /* USB 2.0 host/function module       */
#define BSP_INT_SRC_USBI1           ((bsp_int_src_t)(74))    /*                                    */
#define BSP_INT_SRC_S0_VI_VSYNC0    ((bsp_int_src_t)(75))    /* Video display controller 5         */
#define BSP_INT_SRC_S0_LO_VSYNC0    ((bsp_int_src_t)(76))    /*                                    */
#define BSP_INT_SRC_S0_VSYNCERR0    ((bsp_int_src_t)(77))    /*                                    */
#define BSP_INT_SRC_GR3_VLINE0      ((bsp_int_src_t)(78))    /*                                    */
#define BSP_INT_SRC_S0_VFIELD0      ((bsp_int_src_t)(79))    /*                                    */
#define BSP_INT_SRC_IV1_VBUFERR0    ((bsp_int_src_t)(80))    /*                                    */
#define BSP_INT_SRC_IV3_VBUFERR0    ((bsp_int_src_t)(81))    /*                                    */
#define BSP_INT_SRC_IV5_VBUFERR0    ((bsp_int_src_t)(82))    /*                                    */
#define BSP_INT_SRC_IV6_VBUFERR0    ((bsp_int_src_t)(83))    /*                                    */
#define BSP_INT_SRC_S0_WLINE0       ((bsp_int_src_t)(84))    /*                                    */
#define BSP_INT_SRC_S1_VI_VSYNC0    ((bsp_int_src_t)(85))    /*                                    */
#define BSP_INT_SRC_S1_LO_VSYNC0    ((bsp_int_src_t)(86))    /*                                    */
#define BSP_INT_SRC_S1_VSYNCERR0    ((bsp_int_src_t)(87))    /*                                    */
#define BSP_INT_SRC_S1_VFIELD0      ((bsp_int_src_t)(88))    /*                                    */
#define BSP_INT_SRC_IV2_VBUFERR0    ((bsp_int_src_t)(89))    /*                                    */
#define BSP_INT_SRC_IV4_VBUFERR0    ((bsp_int_src_t)(90))    /*                                    */
#define BSP_INT_SRC_S1_WLINE0       ((bsp_int_src_t)(91))    /*                                    */
#define BSP_INT_SRC_OIR_VI_VSYNC0   ((bsp_int_src_t)(92))    /*                                    */
#define BSP_INT_SRC_OIR_LO_VSYNC0   ((bsp_int_src_t)(93))    /*                                    */
#define BSP_INT_SRC_OIR_VSYNCERR0   ((bsp_int_src_t)(94))    /*                                    */
#define BSP_INT_SRC_OIR_VFIELD0     ((bsp_int_src_t)(95))    /*                                    */
#define BSP_INT_SRC_IV7_VBUFERR0    ((bsp_int_src_t)(96))    /*                                    */
#define BSP_INT_SRC_IV8_VBUFERR0    ((bsp_int_src_t)(97))    /*                                    */
#define BSP_INT_SRC_OIR_WLINE0      ((bsp_int_src_t)(98))    /*                                    */
#define BSP_INT_SRC_S0_VI_VSYNC1    ((bsp_int_src_t)(99))    /*                                    */
#define BSP_INT_SRC_S0_LO_VSYNC1    ((bsp_int_src_t)(100))   /*                                    */
#define BSP_INT_SRC_S0_VSYNCERR1    ((bsp_int_src_t)(101))   /*                                    */
#define BSP_INT_SRC_GR3_VLINE1      ((bsp_int_src_t)(102))   /*                                    */
#define BSP_INT_SRC_S0_VFIELD1      ((bsp_int_src_t)(103))   /*                                    */
#define BSP_INT_SRC_IV1_VBUFERR1    ((bsp_int_src_t)(104))   /*                                    */
#define BSP_INT_SRC_IV3_VBUFERR1    ((bsp_int_src_t)(105))   /*                                    */
#define BSP_INT_SRC_IV5_VBUFERR1    ((bsp_int_src_t)(106))   /*                                    */
#define BSP_INT_SRC_IV6_VBUFERR1    ((bsp_int_src_t)(107))   /*                                    */
#define BSP_INT_SRC_S0_WLINE1       ((bsp_int_src_t)(108))   /*                                    */
#define BSP_INT_SRC_S1_VI_VSYNC1    ((bsp_int_src_t)(109))   /*                                    */
#define BSP_INT_SRC_S1_LO_VSYNC1    ((bsp_int_src_t)(110))   /*                                    */
#define BSP_INT_SRC_S1_VSYNCERR1    ((bsp_int_src_t)(111))   /*                                    */
#define BSP_INT_SRC_S1_VFIELD1      ((bsp_int_src_t)(112))   /*                                    */
#define BSP_INT_SRC_IV2_VBUFERR1    ((bsp_int_src_t)(113))   /*                                    */
#define BSP_INT_SRC_IV4_VBUFERR1    ((bsp_int_src_t)(114))   /*                                    */
#define BSP_INT_SRC_S1_WLINE1       ((bsp_int_src_t)(115))   /*                                    */
#define BSP_INT_SRC_OIR_VI_VSYNC1   ((bsp_int_src_t)(116))   /*                                    */
#define BSP_INT_SRC_OIR_LO_VSYNC1   ((bsp_int_src_t)(117))   /*                                    */
#define BSP_INT_SRC_OIR_VLINE1      ((bsp_int_src_t)(118))   /*                                    */
#define BSP_INT_SRC_OIR_VFIELD1     ((bsp_int_src_t)(119))   /*                                    */
#define BSP_INT_SRC_IV7_VBUFERR1    ((bsp_int_src_t)(120))   /*                                    */
#define BSP_INT_SRC_IV8_VBUFERR1    ((bsp_int_src_t)(121))   /*                                    */
#define BSP_INT_SRC_OIR_WLINE1      ((bsp_int_src_t)(122))   /*                                    */
#define BSP_INT_SRC_IMRDI           ((bsp_int_src_t)(123))   /* Image renderer                     */
#define BSP_INT_SRC_IMR2I0          ((bsp_int_src_t)(124))   /*                                    */
#define BSP_INT_SRC_IMR2I1          ((bsp_int_src_t)(125))   /*                                    */
#define BSP_INT_SRC_JEDI            ((bsp_int_src_t)(126))   /* JPEG Codec unit                    */
#define BSP_INT_SRC_JDTI            ((bsp_int_src_t)(127))   /*                                    */
#define BSP_INT_SRC_CMP0            ((bsp_int_src_t)(128))   /* Display out comparison unit        */
#define BSP_INT_SRC_CMP1            ((bsp_int_src_t)(129))   /*                                    */
#define BSP_INT_SRC_INT0            ((bsp_int_src_t)(130))   /* OpenVG-Compliant Renesas graphics processor */
#define BSP_INT_SRC_INT1            ((bsp_int_src_t)(131))   /*                                    */
#define BSP_INT_SRC_INT2            ((bsp_int_src_t)(132))   /*                                    */
#define BSP_INT_SRC_INT3            ((bsp_int_src_t)(133))   /*                                    */
#define BSP_INT_SRC_OSTM0TINT       ((bsp_int_src_t)(134))   /* OS timer                           */
#define BSP_INT_SRC_OSTM1TINT       ((bsp_int_src_t)(135))   /*                                    */
#define BSP_INT_SRC_CMI             ((bsp_int_src_t)(136))   /* Bus state controller               */
#define BSP_INT_SRC_WTOUT           ((bsp_int_src_t)(137))   /*                                    */
#define BSP_INT_SRC_ITI             ((bsp_int_src_t)(138))   /* Watchdog timer                     */
#define BSP_INT_SRC_TGI0A           ((bsp_int_src_t)(139))   /* Multi-function timer pulse unit 2  */
#define BSP_INT_SRC_TGI0B           ((bsp_int_src_t)(140))   /*                                    */
#define BSP_INT_SRC_TGI0C           ((bsp_int_src_t)(141))   /*                                    */
#define BSP_INT_SRC_TGI0D           ((bsp_int_src_t)(142))   /*                                    */
#define BSP_INT_SRC_TGI0V           ((bsp_int_src_t)(143))   /*                                    */
#define BSP_INT_SRC_TGI0E           ((bsp_int_src_t)(144))   /*                                    */
#define BSP_INT_SRC_TGI0F           ((bsp_int_src_t)(145))   /*                                    */
#define BSP_INT_SRC_TGI1A           ((bsp_int_src_t)(146))   /*                                    */
#define BSP_INT_SRC_TGI1B           ((bsp_int_src_t)(147))   /*                                    */
#define BSP_INT_SRC_TGI1V           ((bsp_int_src_t)(148))   /*                                    */
#define BSP_INT_SRC_TGI1U           ((bsp_int_src_t)(149))   /*                                    */
#define BSP_INT_SRC_TGI2A           ((bsp_int_src_t)(150))   /*                                    */
#define BSP_INT_SRC_TGI2B           ((bsp_int_src_t)(151))   /*                                    */
#define BSP_INT_SRC_TGI2V           ((bsp_int_src_t)(152))   /*                                    */
#define BSP_INT_SRC_TGI2U           ((bsp_int_src_t)(153))   /*                                    */
#define BSP_INT_SRC_TGI3A           ((bsp_int_src_t)(154))   /*                                    */
#define BSP_INT_SRC_TGI3B           ((bsp_int_src_t)(155))   /*                                    */
#define BSP_INT_SRC_TGI3C           ((bsp_int_src_t)(156))   /*                                    */
#define BSP_INT_SRC_TGI3D           ((bsp_int_src_t)(157))   /*                                    */
#define BSP_INT_SRC_TGI3V           ((bsp_int_src_t)(158))   /*                                    */
#define BSP_INT_SRC_TGI4A           ((bsp_int_src_t)(159))   /*                                    */
#define BSP_INT_SRC_TGI4B           ((bsp_int_src_t)(160))   /*                                    */
#define BSP_INT_SRC_TGI4C           ((bsp_int_src_t)(161))   /*                                    */
#define BSP_INT_SRC_TGI4D           ((bsp_int_src_t)(162))   /*                                    */
#define BSP_INT_SRC_TGI4V           ((bsp_int_src_t)(163))   /*                                    */
#define BSP_INT_SRC_CMI1            ((bsp_int_src_t)(164))   /* Motor control PWM timer            */
#define BSP_INT_SRC_CMI2            ((bsp_int_src_t)(165))   /*                                    */
#define BSP_INT_SRC_SGDEI0          ((bsp_int_src_t)(166))   /* Sound generator                    */
#define BSP_INT_SRC_SGDEI1          ((bsp_int_src_t)(167))   /*                                    */
#define BSP_INT_SRC_SGDEI2          ((bsp_int_src_t)(168))   /*                                    */
#define BSP_INT_SRC_SGDEI3          ((bsp_int_src_t)(169))   /*                                    */
#define BSP_INT_SRC_ADI             ((bsp_int_src_t)(170))   /* 12bit A/D converter                */
#define BSP_INT_SRC_LMTI            ((bsp_int_src_t)(171))   /*                                    */
#define BSP_INT_SRC_SSII0           ((bsp_int_src_t)(172))   /* Serial sound interface             */
#define BSP_INT_SRC_SSIRXI0         ((bsp_int_src_t)(173))   /*                                    */
#define BSP_INT_SRC_SSITXI0         ((bsp_int_src_t)(174))   /*                                    */
#define BSP_INT_SRC_SSII1           ((bsp_int_src_t)(175))   /*                                    */
#define BSP_INT_SRC_SSIRXI1         ((bsp_int_src_t)(176))   /*                                    */
#define BSP_INT_SRC_SSITXI1         ((bsp_int_src_t)(177))   /*                                    */
#define BSP_INT_SRC_SSII2           ((bsp_int_src_t)(178))   /*                                    */
#define BSP_INT_SRC_SSIRTI2         ((bsp_int_src_t)(179))   /*                                    */
#define BSP_INT_SRC_SSII3           ((bsp_int_src_t)(180))   /*                                    */
#define BSP_INT_SRC_SSIRXI3         ((bsp_int_src_t)(181))   /*                                    */
#define BSP_INT_SRC_SSITXI3         ((bsp_int_src_t)(182))   /*                                    */
#define BSP_INT_SRC_SSII4           ((bsp_int_src_t)(183))   /*                                    */
#define BSP_INT_SRC_SSIRTI4         ((bsp_int_src_t)(184))   /*                                    */
#define BSP_INT_SRC_SSII5           ((bsp_int_src_t)(185))   /*                                    */
#define BSP_INT_SRC_SSIRXI5         ((bsp_int_src_t)(186))   /*                                    */
#define BSP_INT_SRC_SSITXI5         ((bsp_int_src_t)(187))   /*                                    */
#define BSP_INT_SRC_SPDIFI          ((bsp_int_src_t)(188))   /* Renesas SPDIF interface            */
#define BSP_INT_SRC_INTIICTEI0      ((bsp_int_src_t)(189))   /* I2C interface                      */
#define BSP_INT_SRC_INTIICRI0       ((bsp_int_src_t)(190))   /*                                    */
#define BSP_INT_SRC_INTIICTI0       ((bsp_int_src_t)(191))   /*                                    */
#define BSP_INT_SRC_INTIICSPI0      ((bsp_int_src_t)(192))   /*                                    */
#define BSP_INT_SRC_INTIICSTI0      ((bsp_int_src_t)(193))   /*                                    */
#define BSP_INT_SRC_INTIICNAKI0     ((bsp_int_src_t)(194))   /*                                    */
#define BSP_INT_SRC_INTIICALI0      ((bsp_int_src_t)(195))   /*                                    */
#define BSP_INT_SRC_INTIICTMOI0     ((bsp_int_src_t)(196))   /*                                    */
#define BSP_INT_SRC_INTIICTEI1      ((bsp_int_src_t)(197))   /*                                    */
#define BSP_INT_SRC_INTIICRI1       ((bsp_int_src_t)(198))   /*                                    */
#define BSP_INT_SRC_INTIICTI1       ((bsp_int_src_t)(199))   /*                                    */
#define BSP_INT_SRC_INTIICSPI1      ((bsp_int_src_t)(200))   /*                                    */
#define BSP_INT_SRC_INTIICSTI1      ((bsp_int_src_t)(201))   /*                                    */
#define BSP_INT_SRC_INTIICNAKI1     ((bsp_int_src_t)(202))   /*                                    */
#define BSP_INT_SRC_INTIICALI1      ((bsp_int_src_t)(203))   /*                                    */
#define BSP_INT_SRC_INTIICTMOI1     ((bsp_int_src_t)(204))   /*                                    */
#define BSP_INT_SRC_INTIICTEI2      ((bsp_int_src_t)(205))   /*                                    */
#define BSP_INT_SRC_INTIICRI2       ((bsp_int_src_t)(206))   /*                                    */
#define BSP_INT_SRC_INTIICTI2       ((bsp_int_src_t)(207))   /*                                    */
#define BSP_INT_SRC_INTIICSPI2      ((bsp_int_src_t)(208))   /*                                    */
#define BSP_INT_SRC_INTIICSTI2      ((bsp_int_src_t)(209))   /*                                    */
#define BSP_INT_SRC_INTIICNAKI2     ((bsp_int_src_t)(210))   /*                                    */
#define BSP_INT_SRC_INTIICALI2      ((bsp_int_src_t)(211))   /*                                    */
#define BSP_INT_SRC_INTIICTMOI2     ((bsp_int_src_t)(212))   /*                                    */
#define BSP_INT_SRC_INTIICTEI3      ((bsp_int_src_t)(213))   /*                                    */
#define BSP_INT_SRC_INTIICRI3       ((bsp_int_src_t)(214))   /*                                    */
#define BSP_INT_SRC_INTIICTI3       ((bsp_int_src_t)(215))   /*                                    */
#define BSP_INT_SRC_INTIICSPI3      ((bsp_int_src_t)(216))   /*                                    */
#define BSP_INT_SRC_INTIICSTI3      ((bsp_int_src_t)(217))   /*                                    */
#define BSP_INT_SRC_INTIICNAKI3     ((bsp_int_src_t)(218))   /*                                    */
#define BSP_INT_SRC_INTIICALI3      ((bsp_int_src_t)(219))   /*                                    */
#define BSP_INT_SRC_INTIICTMOI3     ((bsp_int_src_t)(220))   /*                                    */
#define BSP_INT_SRC_BRI0            ((bsp_int_src_t)(221))   /* Serial Communication Interface with FIFO */
#define BSP_INT_SRC_ERI0            ((bsp_int_src_t)(222))   /*                                    */
#define BSP_INT_SRC_RXI0            ((bsp_int_src_t)(223))   /*                                    */
#define BSP_INT_SRC_TXI0            ((bsp_int_src_t)(224))   /*                                    */
#define BSP_INT_SRC_BRI1            ((bsp_int_src_t)(225))   /*                                    */
#define BSP_INT_SRC_ERI1            ((bsp_int_src_t)(226))   /*                                    */
#define BSP_INT_SRC_RXI1            ((bsp_int_src_t)(227))   /*                                    */
#define BSP_INT_SRC_TXI1            ((bsp_int_src_t)(228))   /*                                    */
#define BSP_INT_SRC_BRI2            ((bsp_int_src_t)(229))   /*                                    */
#define BSP_INT_SRC_ERI2            ((bsp_int_src_t)(230))   /*                                    */
#define BSP_INT_SRC_RXI2            ((bsp_int_src_t)(231))   /*                                    */
#define BSP_INT_SRC_TXI2            ((bsp_int_src_t)(232))   /*                                    */
#define BSP_INT_SRC_BRI3            ((bsp_int_src_t)(233))   /*                                    */
#define BSP_INT_SRC_ERI3            ((bsp_int_src_t)(234))   /*                                    */
#define BSP_INT_SRC_RXI3            ((bsp_int_src_t)(235))   /*                                    */
#define BSP_INT_SRC_TXI3            ((bsp_int_src_t)(236))   /*                                    */
#define BSP_INT_SRC_BRI4            ((bsp_int_src_t)(237))   /*                                    */
#define BSP_INT_SRC_ERI4            ((bsp_int_src_t)(238))   /*                                    */
#define BSP_INT_SRC_RXI4            ((bsp_int_src_t)(239))   /*                                    */
#define BSP_INT_SRC_TXI4            ((bsp_int_src_t)(240))   /*                                    */
#define BSP_INT_SRC_BRI5            ((bsp_int_src_t)(241))   /*                                    */
#define BSP_INT_SRC_ERI5            ((bsp_int_src_t)(242))   /*                                    */
#define BSP_INT_SRC_RXI5            ((bsp_int_src_t)(243))   /*                                    */
#define BSP_INT_SRC_TXI5            ((bsp_int_src_t)(244))   /*                                    */
#define BSP_INT_SRC_BRI6            ((bsp_int_src_t)(245))   /*                                    */
#define BSP_INT_SRC_ERI6            ((bsp_int_src_t)(246))   /*                                    */
#define BSP_INT_SRC_RXI6            ((bsp_int_src_t)(247))   /*                                    */
#define BSP_INT_SRC_TXI6            ((bsp_int_src_t)(248))   /*                                    */
#define BSP_INT_SRC_BRI7            ((bsp_int_src_t)(249))   /*                                    */
#define BSP_INT_SRC_ERI7            ((bsp_int_src_t)(250))   /*                                    */
#define BSP_INT_SRC_RXI7            ((bsp_int_src_t)(251))   /*                                    */
#define BSP_INT_SRC_TXI7            ((bsp_int_src_t)(252))   /*                                    */
#define BSP_INT_SRC_INTRCANGERR     ((bsp_int_src_t)(253))   /* CAN interface                      */
#define BSP_INT_SRC_INTRCANGRECC    ((bsp_int_src_t)(254))   /*                                    */
#define BSP_INT_SRC_INTRCAN0REC     ((bsp_int_src_t)(255))   /*                                    */
#define BSP_INT_SRC_INTRCAN0ERR     ((bsp_int_src_t)(256))   /*                                    */
#define BSP_INT_SRC_INTRCAN0TRX     ((bsp_int_src_t)(257))   /*                                    */
#define BSP_INT_SRC_INTRCAN1REC     ((bsp_int_src_t)(258))   /*                                    */
#define BSP_INT_SRC_INTRCAN1ERR     ((bsp_int_src_t)(259))   /*                                    */
#define BSP_INT_SRC_INTRCAN1TRX     ((bsp_int_src_t)(260))   /*                                    */
#define BSP_INT_SRC_INTRCAN2REC     ((bsp_int_src_t)(261))   /*                                    */
#define BSP_INT_SRC_INTRCAN2ERR     ((bsp_int_src_t)(262))   /*                                    */
#define BSP_INT_SRC_INTRCAN2TRX     ((bsp_int_src_t)(263))   /*                                    */
#define BSP_INT_SRC_INTRCAN3REC     ((bsp_int_src_t)(264))   /*                                    */
#define BSP_INT_SRC_INTRCAN3ERR     ((bsp_int_src_t)(265))   /*                                    */
#define BSP_INT_SRC_INTRCAN3TRX     ((bsp_int_src_t)(266))   /*                                    */
#define BSP_INT_SRC_INTRCAN4REC     ((bsp_int_src_t)(267))   /*                                    */
#define BSP_INT_SRC_INTRCAN4ERR     ((bsp_int_src_t)(268))   /*                                    */
#define BSP_INT_SRC_INTRCAN4TRX     ((bsp_int_src_t)(269))   /*                                    */
#define BSP_INT_SRC_SPEI0           ((bsp_int_src_t)(270))   /* Renesas serial peripheral interface */
#define BSP_INT_SRC_SPRI0           ((bsp_int_src_t)(271))   /*                                    */
#define BSP_INT_SRC_SPTI0           ((bsp_int_src_t)(272))   /*                                    */
#define BSP_INT_SRC_SPEI1           ((bsp_int_src_t)(273))   /*                                    */
#define BSP_INT_SRC_SPRI1           ((bsp_int_src_t)(274))   /*                                    */
#define BSP_INT_SRC_SPTI1           ((bsp_int_src_t)(275))   /*                                    */
#define BSP_INT_SRC_SPEI2           ((bsp_int_src_t)(276))   /*                                    */
#define BSP_INT_SRC_SPRI2           ((bsp_int_src_t)(277))   /*                                    */
#define BSP_INT_SRC_SPTI2           ((bsp_int_src_t)(278))   /*                                    */
#define BSP_INT_SRC_SPEI3           ((bsp_int_src_t)(279))   /*                                    */
#define BSP_INT_SRC_SPRI3           ((bsp_int_src_t)(280))   /*                                    */
#define BSP_INT_SRC_SPTI3           ((bsp_int_src_t)(281))   /*                                    */
#define BSP_INT_SRC_SPEI4           ((bsp_int_src_t)(282))   /*                                    */
#define BSP_INT_SRC_SPRI4           ((bsp_int_src_t)(283))   /*                                    */
#define BSP_INT_SRC_SPTI4           ((bsp_int_src_t)(284))   /*                                    */
#define BSP_INT_SRC_IEBBTD          ((bsp_int_src_t)(285))   /* IEBusTM controller                 */
#define BSP_INT_SRC_IEBBTERR        ((bsp_int_src_t)(286))   /*                                    */
#define BSP_INT_SRC_IEBBTSTA        ((bsp_int_src_t)(287))   /*                                    */
#define BSP_INT_SRC_IEBBTV          ((bsp_int_src_t)(288))   /*                                    */
#define BSP_INT_SRC_ISY             ((bsp_int_src_t)(289))   /* CD-ROM decoder                     */
#define BSP_INT_SRC_IERR            ((bsp_int_src_t)(290))   /*                                    */
#define BSP_INT_SRC_ITARG           ((bsp_int_src_t)(291))   /*                                    */
#define BSP_INT_SRC_ISEC            ((bsp_int_src_t)(292))   /*                                    */
#define BSP_INT_SRC_IBUF            ((bsp_int_src_t)(293))   /*                                    */
#define BSP_INT_SRC_IREADY          ((bsp_int_src_t)(294))   /*                                    */
#define BSP_INT_SRC_FLSTE           ((bsp_int_src_t)(295))   /* NAND Flash memory controller       */
#define BSP_INT_SRC_FLTENDI         ((bsp_int_src_t)(296))   /*                                    */
#define BSP_INT_SRC_FLTREQ0I        ((bsp_int_src_t)(297))   /*                                    */
#define BSP_INT_SRC_FLTREQ1I        ((bsp_int_src_t)(298))   /*                                    */
#define BSP_INT_SRC_MMC0            ((bsp_int_src_t)(299))   /* MMC Host interface                 */
#define BSP_INT_SRC_MMC1            ((bsp_int_src_t)(300))   /*                                    */
#define BSP_INT_SRC_MMC2            ((bsp_int_src_t)(301))   /*                                    */
#define BSP_INT_SRC_SDHI0_3         ((bsp_int_src_t)(302))   /* SD Host interface                  */
#define BSP_INT_SRC_SDHI0_0         ((bsp_int_src_t)(303))   /*                                    */
#define BSP_INT_SRC_SDHI0_1         ((bsp_int_src_t)(304))   /*                                    */
#define BSP_INT_SRC_SDHI1_3         ((bsp_int_src_t)(305))   /*                                    */
#define BSP_INT_SRC_SDHI1_0         ((bsp_int_src_t)(306))   /*                                    */
#define BSP_INT_SRC_SDHI1_1         ((bsp_int_src_t)(307))   /*                                    */
#define BSP_INT_SRC_ARM             ((bsp_int_src_t)(308))   /* Real time clock                    */
#define BSP_INT_SRC_PRD             ((bsp_int_src_t)(309))   /*                                    */
#define BSP_INT_SRC_CUP             ((bsp_int_src_t)(310))   /*                                    */
#define BSP_INT_SRC_SCUAI0          ((bsp_int_src_t)(311))   /* SCUX                               */
#define BSP_INT_SRC_SCUAI1          ((bsp_int_src_t)(312))   /*                                    */
#define BSP_INT_SRC_SCUFDI0         ((bsp_int_src_t)(313))   /*                                    */
#define BSP_INT_SRC_SCUFDI1         ((bsp_int_src_t)(314))   /*                                    */
#define BSP_INT_SRC_SCUFDI2         ((bsp_int_src_t)(315))   /*                                    */
#define BSP_INT_SRC_SCUFDI3         ((bsp_int_src_t)(316))   /*                                    */
#define BSP_INT_SRC_SCUFUI0         ((bsp_int_src_t)(317))   /*                                    */
#define BSP_INT_SRC_SCUFUI1         ((bsp_int_src_t)(318))   /*                                    */
#define BSP_INT_SRC_SCUFUI2         ((bsp_int_src_t)(319))   /*                                    */
#define BSP_INT_SRC_SCUFUI3         ((bsp_int_src_t)(320))   /*                                    */
#define BSP_INT_SRC_SCUDVI0         ((bsp_int_src_t)(321))   /*                                    */
#define BSP_INT_SRC_SCUDVI1         ((bsp_int_src_t)(322))   /*                                    */
#define BSP_INT_SRC_SCUDVI2         ((bsp_int_src_t)(323))   /*                                    */
#define BSP_INT_SRC_SCUDVI3         ((bsp_int_src_t)(324))   /*                                    */
#define BSP_INT_SRC_MLB_CINT        ((bsp_int_src_t)(325))   /* Media local bus                    */
#define BSP_INT_SRC_MLB_SINT        ((bsp_int_src_t)(326))   /*                                    */
#define BSP_INT_SRC_DRC0            ((bsp_int_src_t)(327))   /* Dynamic range compalator           */
#define BSP_INT_SRC_DRC1            ((bsp_int_src_t)(328))   /*                                    */
#define BSP_INT_SRC_LINI0_INT_T     ((bsp_int_src_t)(331))   /* LIN/UART interface                 */
#define BSP_INT_SRC_LINI0_INT_R     ((bsp_int_src_t)(332))   /*                                    */
#define BSP_INT_SRC_LINI0_INT_S     ((bsp_int_src_t)(333))   /*                                    */
#define BSP_INT_SRC_LINI0_INT_M     ((bsp_int_src_t)(334))   /*                                    */
#define BSP_INT_SRC_LINI1_INT_T     ((bsp_int_src_t)(335))   /*                                    */
#define BSP_INT_SRC_LINI1_INT_R     ((bsp_int_src_t)(336))   /*                                    */
#define BSP_INT_SRC_LINI1_INT_S     ((bsp_int_src_t)(337))   /*                                    */
#define BSP_INT_SRC_LINI1_INT_M     ((bsp_int_src_t)(338))   /*                                    */
#define BSP_INT_SRC_SCI_ERI0        ((bsp_int_src_t)(347))   /* Serial communication interface     */
#define BSP_INT_SRC_SCI_RXI0        ((bsp_int_src_t)(348))   /*                                    */
#define BSP_INT_SRC_SCI_TXI0        ((bsp_int_src_t)(349))   /*                                    */
#define BSP_INT_SRC_SCI_TEI0        ((bsp_int_src_t)(350))   /*                                    */
#define BSP_INT_SRC_SCI_ERI1        ((bsp_int_src_t)(351))   /*                                    */
#define BSP_INT_SRC_SCI_RXI1        ((bsp_int_src_t)(352))   /*                                    */
#define BSP_INT_SRC_SCI_TXI1        ((bsp_int_src_t)(353))   /*                                    */
#define BSP_INT_SRC_SCI_TEI1        ((bsp_int_src_t)(354))   /*                                    */
#define BSP_INT_SRC_AVBI_DATA       ((bsp_int_src_t)(355))   /* EthernetAVB                        */
#define BSP_INT_SRC_AVBI_ERROR      ((bsp_int_src_t)(356))   /*                                    */
#define BSP_INT_SRC_AVBI_MANAGE     ((bsp_int_src_t)(357))   /*                                    */
#define BSP_INT_SRC_AVBI_MAC        ((bsp_int_src_t)(358))   /*                                    */
#define BSP_INT_SRC_ETHERI          ((bsp_int_src_t)(359))   /* Ethernet controller                */
#define BSP_INT_SRC_CEUI            ((bsp_int_src_t)(364))   /* Capture engine unit                */
#define BSP_INT_SRC_H2XMLB_ERRINT   ((bsp_int_src_t)(381))   /* Internal bus                       */
#define BSP_INT_SRC_H2XIC1_ERRINT   ((bsp_int_src_t)(382))   /*                                    */
#define BSP_INT_SRC_X2HPERI1_ERRINT ((bsp_int_src_t)(383))   /*                                    */
#define BSP_INT_SRC_X2HPERI2_ERRINT ((bsp_int_src_t)(384))   /*                                    */
#define BSP_INT_SRC_X2HPERI34_ERRINT ((bsp_int_src_t)(385))  /*                                    */
#define BSP_INT_SRC_X2HPERI5_ERRINT ((bsp_int_src_t)(386))   /*                                    */
#define BSP_INT_SRC_X2HPERI67_ERRINT ((bsp_int_src_t)(387))  /*                                    */
#define BSP_INT_SRC_X2HDBGR_ERRINT  ((bsp_int_src_t)(388))   /*                                    */
#define BSP_INT_SRC_X2HBSC_ERRINT   ((bsp_int_src_t)(389))   /*                                    */
#define BSP_INT_SRC_X2HSPI1_ERRINT  ((bsp_int_src_t)(390))   /*                                    */
#define BSP_INT_SRC_X2HSPI2_ERRINT  ((bsp_int_src_t)(391))   /*                                    */
#define BSP_INT_SRC_PRRI            ((bsp_int_src_t)(392))   /*                                    */
#define BSP_INT_SRC_IFEI0           ((bsp_int_src_t)(393))   /* Pixel format converter             */
#define BSP_INT_SRC_OFFI0           ((bsp_int_src_t)(394))   /*                                    */
#define BSP_INT_SRC_PFVEI0          ((bsp_int_src_t)(395))   /*                                    */
#define BSP_INT_SRC_IFEI1           ((bsp_int_src_t)(396))   /*                                    */
#define BSP_INT_SRC_OFFI1           ((bsp_int_src_t)(397))   /*                                    */
#define BSP_INT_SRC_PFVEI1          ((bsp_int_src_t)(398))   /*                                    */
#define BSP_INT_SRC_TINT0           ((bsp_int_src_t)(416))   /* Terminal interrupts                */
#define BSP_INT_SRC_TINT1           ((bsp_int_src_t)(417))   /*                                    */
#define BSP_INT_SRC_TINT2           ((bsp_int_src_t)(418))   /*                                    */
#define BSP_INT_SRC_TINT3           ((bsp_int_src_t)(419))   /*                                    */
#define BSP_INT_SRC_TINT4           ((bsp_int_src_t)(420))   /*                                    */
#define BSP_INT_SRC_TINT5           ((bsp_int_src_t)(421))   /*                                    */
#define BSP_INT_SRC_TINT6           ((bsp_int_src_t)(422))   /*                                    */
#define BSP_INT_SRC_TINT7           ((bsp_int_src_t)(423))   /*                                    */
#define BSP_INT_SRC_TINT8           ((bsp_int_src_t)(424))   /*                                    */
#define BSP_INT_SRC_TINT9           ((bsp_int_src_t)(425))   /*                                    */
#define BSP_INT_SRC_TINT10          ((bsp_int_src_t)(426))   /*                                    */
#define BSP_INT_SRC_TINT11          ((bsp_int_src_t)(427))   /*                                    */
#define BSP_INT_SRC_TINT12          ((bsp_int_src_t)(428))   /*                                    */
#define BSP_INT_SRC_TINT13          ((bsp_int_src_t)(429))   /*                                    */
#define BSP_INT_SRC_TINT14          ((bsp_int_src_t)(430))   /*                                    */
#define BSP_INT_SRC_TINT15          ((bsp_int_src_t)(431))   /*                                    */
#define BSP_INT_SRC_TINT16          ((bsp_int_src_t)(432))   /*                                    */
#define BSP_INT_SRC_TINT17          ((bsp_int_src_t)(433))   /*                                    */
#define BSP_INT_SRC_TINT18          ((bsp_int_src_t)(434))   /*                                    */
#define BSP_INT_SRC_TINT19          ((bsp_int_src_t)(435))   /*                                    */
#define BSP_INT_SRC_TINT20          ((bsp_int_src_t)(436))   /*                                    */
#define BSP_INT_SRC_TINT21          ((bsp_int_src_t)(437))   /*                                    */
#define BSP_INT_SRC_TINT22          ((bsp_int_src_t)(438))   /*                                    */
#define BSP_INT_SRC_TINT23          ((bsp_int_src_t)(439))   /*                                    */
#define BSP_INT_SRC_TINT24          ((bsp_int_src_t)(440))   /*                                    */
#define BSP_INT_SRC_TINT25          ((bsp_int_src_t)(441))   /*                                    */
#define BSP_INT_SRC_TINT26          ((bsp_int_src_t)(442))   /*                                    */
#define BSP_INT_SRC_TINT27          ((bsp_int_src_t)(443))   /*                                    */
#define BSP_INT_SRC_TINT28          ((bsp_int_src_t)(444))   /*                                    */
#define BSP_INT_SRC_TINT29          ((bsp_int_src_t)(445))   /*                                    */
#define BSP_INT_SRC_TINT30          ((bsp_int_src_t)(446))   /*                                    */
#define BSP_INT_SRC_TINT31          ((bsp_int_src_t)(447))   /*                                    */
#define BSP_INT_SRC_TINT32          ((bsp_int_src_t)(448))   /*                                    */
#define BSP_INT_SRC_TINT33          ((bsp_int_src_t)(449))   /*                                    */
#define BSP_INT_SRC_TINT34          ((bsp_int_src_t)(450))   /*                                    */
#define BSP_INT_SRC_TINT35          ((bsp_int_src_t)(451))   /*                                    */
#define BSP_INT_SRC_TINT36          ((bsp_int_src_t)(452))   /*                                    */
#define BSP_INT_SRC_TINT37          ((bsp_int_src_t)(453))   /*                                    */
#define BSP_INT_SRC_TINT38          ((bsp_int_src_t)(454))   /*                                    */
#define BSP_INT_SRC_TINT39          ((bsp_int_src_t)(455))   /*                                    */
#define BSP_INT_SRC_TINT40          ((bsp_int_src_t)(456))   /*                                    */
#define BSP_INT_SRC_TINT41          ((bsp_int_src_t)(457))   /*                                    */
#define BSP_INT_SRC_TINT42          ((bsp_int_src_t)(458))   /*                                    */
#define BSP_INT_SRC_TINT43          ((bsp_int_src_t)(459))   /*                                    */
#define BSP_INT_SRC_TINT44          ((bsp_int_src_t)(460))   /*                                    */
#define BSP_INT_SRC_TINT45          ((bsp_int_src_t)(461))   /*                                    */
#define BSP_INT_SRC_TINT46          ((bsp_int_src_t)(462))   /*                                    */
#define BSP_INT_SRC_TINT47          ((bsp_int_src_t)(463))   /*                                    */
#define BSP_INT_SRC_TINT48          ((bsp_int_src_t)(464))   /*                                    */
#define BSP_INT_SRC_TINT49          ((bsp_int_src_t)(465))   /*                                    */
#define BSP_INT_SRC_TINT50          ((bsp_int_src_t)(466))   /*                                    */
#define BSP_INT_SRC_TINT51          ((bsp_int_src_t)(467))   /*                                    */
#define BSP_INT_SRC_TINT52          ((bsp_int_src_t)(468))   /*                                    */
#define BSP_INT_SRC_TINT53          ((bsp_int_src_t)(469))   /*                                    */
#define BSP_INT_SRC_TINT54          ((bsp_int_src_t)(470))   /*                                    */
#define BSP_INT_SRC_TINT55          ((bsp_int_src_t)(471))   /*                                    */
#define BSP_INT_SRC_TINT56          ((bsp_int_src_t)(472))   /*                                    */
#define BSP_INT_SRC_TINT57          ((bsp_int_src_t)(473))   /*                                    */
#define BSP_INT_SRC_TINT58          ((bsp_int_src_t)(474))   /*                                    */
#define BSP_INT_SRC_TINT59          ((bsp_int_src_t)(475))   /*                                    */
#define BSP_INT_SRC_TINT60          ((bsp_int_src_t)(476))   /*                                    */
#define BSP_INT_SRC_TINT61          ((bsp_int_src_t)(477))   /*                                    */
#define BSP_INT_SRC_TINT62          ((bsp_int_src_t)(478))   /*                                    */
#define BSP_INT_SRC_TINT63          ((bsp_int_src_t)(479))   /*                                    */
#define BSP_INT_SRC_TINT64          ((bsp_int_src_t)(480))   /*                                    */
#define BSP_INT_SRC_TINT65          ((bsp_int_src_t)(481))   /*                                    */
#define BSP_INT_SRC_TINT66          ((bsp_int_src_t)(482))   /*                                    */
#define BSP_INT_SRC_TINT67          ((bsp_int_src_t)(483))   /*                                    */
#define BSP_INT_SRC_TINT68          ((bsp_int_src_t)(484))   /*                                    */
#define BSP_INT_SRC_TINT69          ((bsp_int_src_t)(485))   /*                                    */
#define BSP_INT_SRC_TINT70          ((bsp_int_src_t)(486))   /*                                    */
#define BSP_INT_SRC_TINT71          ((bsp_int_src_t)(487))   /*                                    */
#define BSP_INT_SRC_TINT72          ((bsp_int_src_t)(488))   /*                                    */
#define BSP_INT_SRC_TINT73          ((bsp_int_src_t)(489))   /*                                    */
#define BSP_INT_SRC_TINT74          ((bsp_int_src_t)(490))   /*                                    */
#define BSP_INT_SRC_TINT75          ((bsp_int_src_t)(491))   /*                                    */
#define BSP_INT_SRC_TINT76          ((bsp_int_src_t)(492))   /*                                    */
#define BSP_INT_SRC_TINT77          ((bsp_int_src_t)(493))   /*                                    */
#define BSP_INT_SRC_TINT78          ((bsp_int_src_t)(494))   /*                                    */
#define BSP_INT_SRC_TINT79          ((bsp_int_src_t)(495))   /*                                    */
#define BSP_INT_SRC_TINT80          ((bsp_int_src_t)(496))   /*                                    */
#define BSP_INT_SRC_TINT81          ((bsp_int_src_t)(497))   /*                                    */
#define BSP_INT_SRC_TINT82          ((bsp_int_src_t)(498))   /*                                    */
#define BSP_INT_SRC_TINT83          ((bsp_int_src_t)(499))   /*                                    */
#define BSP_INT_SRC_TINT84          ((bsp_int_src_t)(500))   /*                                    */
#define BSP_INT_SRC_TINT85          ((bsp_int_src_t)(501))   /*                                    */
#define BSP_INT_SRC_TINT86          ((bsp_int_src_t)(502))   /*                                    */
#define BSP_INT_SRC_TINT87          ((bsp_int_src_t)(503))   /*                                    */
#define BSP_INT_SRC_TINT88          ((bsp_int_src_t)(504))   /*                                    */
#define BSP_INT_SRC_TINT89          ((bsp_int_src_t)(505))   /*                                    */
#define BSP_INT_SRC_TINT90          ((bsp_int_src_t)(506))   /*                                    */
#define BSP_INT_SRC_TINT91          ((bsp_int_src_t)(507))   /*                                    */
#define BSP_INT_SRC_TINT92          ((bsp_int_src_t)(508))   /*                                    */
#define BSP_INT_SRC_TINT93          ((bsp_int_src_t)(509))   /*                                    */
#define BSP_INT_SRC_TINT94          ((bsp_int_src_t)(510))   /*                                    */
#define BSP_INT_SRC_TINT95          ((bsp_int_src_t)(511))   /*                                    */
#define BSP_INT_SRC_TINT96          ((bsp_int_src_t)(512))   /*                                    */
#define BSP_INT_SRC_TINT97          ((bsp_int_src_t)(513))   /*                                    */
#define BSP_INT_SRC_TINT98          ((bsp_int_src_t)(514))   /*                                    */
#define BSP_INT_SRC_TINT99          ((bsp_int_src_t)(515))   /*                                    */
#define BSP_INT_SRC_TINT100         ((bsp_int_src_t)(516))   /*                                    */
#define BSP_INT_SRC_TINT101         ((bsp_int_src_t)(517))   /*                                    */
#define BSP_INT_SRC_TINT102         ((bsp_int_src_t)(518))   /*                                    */
#define BSP_INT_SRC_TINT103         ((bsp_int_src_t)(519))   /*                                    */
#define BSP_INT_SRC_TINT104         ((bsp_int_src_t)(520))   /*                                    */
#define BSP_INT_SRC_TINT105         ((bsp_int_src_t)(521))   /*                                    */
#define BSP_INT_SRC_TINT106         ((bsp_int_src_t)(522))   /*                                    */
#define BSP_INT_SRC_TINT107         ((bsp_int_src_t)(523))   /*                                    */
#define BSP_INT_SRC_TINT108         ((bsp_int_src_t)(524))   /*                                    */
#define BSP_INT_SRC_TINT109         ((bsp_int_src_t)(525))   /*                                    */
#define BSP_INT_SRC_TINT110         ((bsp_int_src_t)(526))   /*                                    */
#define BSP_INT_SRC_TINT111         ((bsp_int_src_t)(527))   /*                                    */
#define BSP_INT_SRC_TINT112         ((bsp_int_src_t)(528))   /*                                    */
#define BSP_INT_SRC_TINT113         ((bsp_int_src_t)(529))   /*                                    */
#define BSP_INT_SRC_TINT114         ((bsp_int_src_t)(530))   /*                                    */
#define BSP_INT_SRC_TINT115         ((bsp_int_src_t)(531))   /*                                    */
#define BSP_INT_SRC_TINT116         ((bsp_int_src_t)(532))   /*                                    */
#define BSP_INT_SRC_TINT117         ((bsp_int_src_t)(533))   /*                                    */
#define BSP_INT_SRC_TINT118         ((bsp_int_src_t)(534))   /*                                    */
#define BSP_INT_SRC_TINT119         ((bsp_int_src_t)(535))   /*                                    */
#define BSP_INT_SRC_TINT120         ((bsp_int_src_t)(536))   /*                                    */
#define BSP_INT_SRC_TINT121         ((bsp_int_src_t)(537))   /*                                    */
#define BSP_INT_SRC_TINT122         ((bsp_int_src_t)(538))   /*                                    */
#define BSP_INT_SRC_TINT123         ((bsp_int_src_t)(539))   /*                                    */
#define BSP_INT_SRC_TINT124         ((bsp_int_src_t)(540))   /*                                    */
#define BSP_INT_SRC_TINT125         ((bsp_int_src_t)(541))   /*                                    */
#define BSP_INT_SRC_TINT126         ((bsp_int_src_t)(542))   /*                                    */
#define BSP_INT_SRC_TINT127         ((bsp_int_src_t)(543))   /*                                    */
#define BSP_INT_SRC_TINT128         ((bsp_int_src_t)(544))   /*                                    */
#define BSP_INT_SRC_TINT129         ((bsp_int_src_t)(545))   /*                                    */
#define BSP_INT_SRC_TINT130         ((bsp_int_src_t)(546))   /*                                    */
#define BSP_INT_SRC_TINT131         ((bsp_int_src_t)(547))   /*                                    */
#define BSP_INT_SRC_TINT132         ((bsp_int_src_t)(548))   /*                                    */
#define BSP_INT_SRC_TINT133         ((bsp_int_src_t)(549))   /*                                    */
#define BSP_INT_SRC_TINT134         ((bsp_int_src_t)(550))   /*                                    */
#define BSP_INT_SRC_TINT135         ((bsp_int_src_t)(551))   /*                                    */
#define BSP_INT_SRC_TINT136         ((bsp_int_src_t)(552))   /*                                    */
#define BSP_INT_SRC_TINT137         ((bsp_int_src_t)(553))   /*                                    */
#define BSP_INT_SRC_TINT138         ((bsp_int_src_t)(554))   /*                                    */
#define BSP_INT_SRC_TINT139         ((bsp_int_src_t)(555))   /*                                    */
#define BSP_INT_SRC_TINT140         ((bsp_int_src_t)(556))   /*                                    */
#define BSP_INT_SRC_TINT141         ((bsp_int_src_t)(557))   /*                                    */
#define BSP_INT_SRC_TINT142         ((bsp_int_src_t)(558))   /*                                    */
#define BSP_INT_SRC_TINT143         ((bsp_int_src_t)(559))   /*                                    */
#define BSP_INT_SRC_TINT144         ((bsp_int_src_t)(560))   /*                                    */
#define BSP_INT_SRC_TINT145         ((bsp_int_src_t)(561))   /*                                    */
#define BSP_INT_SRC_TINT146         ((bsp_int_src_t)(562))   /*                                    */
#define BSP_INT_SRC_TINT147         ((bsp_int_src_t)(563))   /*                                    */
#define BSP_INT_SRC_TINT148         ((bsp_int_src_t)(564))   /*                                    */
#define BSP_INT_SRC_TINT149         ((bsp_int_src_t)(565))   /*                                    */
#define BSP_INT_SRC_TINT150         ((bsp_int_src_t)(566))   /*                                    */
#define BSP_INT_SRC_TINT151         ((bsp_int_src_t)(567))   /*                                    */
#define BSP_INT_SRC_TINT152         ((bsp_int_src_t)(568))   /*                                    */
#define BSP_INT_SRC_TINT153         ((bsp_int_src_t)(569))   /*                                    */
#define BSP_INT_SRC_TINT154         ((bsp_int_src_t)(570))   /*                                    */
#define BSP_INT_SRC_TINT155         ((bsp_int_src_t)(571))   /*                                    */
#define BSP_INT_SRC_TINT156         ((bsp_int_src_t)(572))   /*                                    */
#define BSP_INT_SRC_TINT157         ((bsp_int_src_t)(573))   /*                                    */
#define BSP_INT_SRC_TINT158         ((bsp_int_src_t)(574))   /*                                    */
#define BSP_INT_SRC_TINT159         ((bsp_int_src_t)(575))   /*                                    */
#define BSP_INT_SRC_TINT160         ((bsp_int_src_t)(576))   /*                                    */
#define BSP_INT_SRC_TINT161         ((bsp_int_src_t)(577))   /*                                    */
#define BSP_INT_SRC_TINT162         ((bsp_int_src_t)(578))   /*                                    */
#define BSP_INT_SRC_TINT163         ((bsp_int_src_t)(579))   /*                                    */
#define BSP_INT_SRC_TINT164         ((bsp_int_src_t)(580))   /*                                    */
#define BSP_INT_SRC_TINT165         ((bsp_int_src_t)(581))   /*                                    */
#define BSP_INT_SRC_TINT166         ((bsp_int_src_t)(582))   /*                                    */
#define BSP_INT_SRC_TINT167         ((bsp_int_src_t)(583))   /*                                    */
#define BSP_INT_SRC_TINT168         ((bsp_int_src_t)(584))   /*                                    */
#define BSP_INT_SRC_TINT169         ((bsp_int_src_t)(585))   /*                                    */
#define BSP_INT_SRC_TINT170         ((bsp_int_src_t)(586))   /*                                    */

#ifdef  R_OSPL_FOR_RTX
INLINE IRQn_Type  R_CAST_bsp_int_src_t_to_IRQn_Type( bsp_int_src_t const  Value )
{
    return  (IRQn_Type) Value;
}
#endif

#endif


/******************************************************************************
Variable Externs
******************************************************************************/

/******************************************************************************
Functions Prototypes
******************************************************************************/
/* In "mcu_interrupts.h" */


/***********************************************************************
* End of File:
************************************************************************/
#ifdef __cplusplus
}  /* extern "C" */
#endif /* __cplusplus */

#endif /* MCU_INTERRUPTS_TYPEDEF_H */

