/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2016 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/***********************************************************************
* File: locking.c
*    Lock related FIT BSP. For RTX, RZ/A1 BSP, OS less.
*
* - $Version: $ (=R_OSPL_VERSION)
* - $ModuleRevision: {Masters}\OSPL\ALL_Driver_parent\27_RStyled $
* - $Rev: $
* - $Date: 2017-08-29T10:34:43+09:00 $
************************************************************************/


/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include  "use_list.h"
#include  "r_ospl.h"
#include  "r_ospl_private.h"
#if R_OSPL_IS_PREEMPTION
#include  "r_ospl_RTOS_private.h"
#endif
#ifdef  USE_LIST_RZA1H_BSP
#include  "dma_if.h"  /* R_DMA_Alloc */
#endif
#ifdef  USE_LIST_FREE_RTOS
#include  "FreeRTOS.h"
#include  "semphr.h"
#endif


/******************************************************************************
Typedef definitions
******************************************************************************/

/******************************************************************************
Macro definitions
******************************************************************************/

/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/

/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/

/******************************************************************************
Private global variables and functions
******************************************************************************/

/***********************************************************************
* Variable: g_bsp_Locks
************************************************************************/
#if  R_OSPL_C_LOCK_CODE
BSP_CFG_USER_LOCKING_TYPE  g_bsp_Locks[ BSP_NUM_LOCKS ];
#endif


/***********************************************************************
* Implement: R_BSP_HardwareLock
************************************************************************/
#if  R_OSPL_C_LOCK_CODE
bool_t  R_BSP_HardwareLock( mcu_lock_t const  in_HardwareIndex )
{
#if BSP_CFG_USER_LOCKING_ENABLED
    return  BSP_CFG_USER_LOCKING_HW_LOCK_FUNCTION( in_HardwareIndex );
#else
    bool_t  is_success;


#ifdef  R_OSPL_VERSION_OF_RZ_A1_BSP
    if ( in_HardwareIndex >= BSP_LOCK_DMAC0  &&  in_HardwareIndex <= BSP_LOCK_DMAC15 )
    {
        int_fast32_t  channel_of_DMAC = in_HardwareIndex - BSP_LOCK_DMAC0;

        channel_of_DMAC = R_DMA_Alloc( channel_of_DMAC, NULL );
        IF ( channel_of_DMAC == -1 )
        {
            is_success = false;
            goto fin;
        }

        is_success = true;
        goto fin;
    }
#endif

    IF_D ( R_CUT_IF_ALWAYS_FALSE( in_HardwareIndex < 0u  ||)  in_HardwareIndex >= R_COUNT_OF( g_bsp_Locks ) )
    {
        is_success = false;
        goto fin;
    }

    is_success = R_BSP_SoftwareLock( &g_bsp_Locks[ in_HardwareIndex ] );
    IF ( ! is_success )
    {
        goto fin;
    }

    is_success = true;
fin:
    return  is_success;
#endif
}
#endif


/***********************************************************************
* Implement: R_OSPL_LockUnlockedChannel
************************************************************************/
#if  R_OSPL_C_LOCK_CODE
#if ! BSP_CFG_USER_LOCKING_ENABLED
errnum_t  R_OSPL_LockUnlockedChannel( int_fast32_t* out_ChannelNum,
                                      mcu_lock_t  in_HardwareIndexMin,  mcu_lock_t  in_HardwareIndexMax )
{
    errnum_t         e;
    mcu_lock_t       hardware_index;
    r_ospl_error_t*  err = NULL;


    err = R_OSPL_LockCurrentThreadError_Sub( R_OSPL_THREAD_GetCurrentId(), R_OSPL_ALLOCATE_IF_NOT );

#if R_OSPL_IS_PREEMPTION
    if ( R_OSPL_THREAD_GetCurrentId() == R_OSPL_THREAD_NULL )    /* If interrrupt context */
    {
        e = E_NOT_THREAD;
        goto fin;
    }
#endif


#ifdef  R_OSPL_VERSION_OF_RZ_A1_BSP
    if ( in_HardwareIndexMin == BSP_LOCK_DMAC0 )
    {
        int_fast32_t  channel_of_DMAC;

        channel_of_DMAC = R_DMA_Alloc( DMA_ALLOC_CH, NULL );
        IF ( channel_of_DMAC == -1 )
        {
            e = E_FEW_ARRAY;
            goto fin;
        }

        *out_ChannelNum = channel_of_DMAC;
        e = 0;
        goto fin;
    }
#endif


    for ( hardware_index = in_HardwareIndexMin;  hardware_index <= in_HardwareIndexMax;
            hardware_index += 1 )
    {
        r_ospl_c_lock_t*  lock = &g_bsp_Locks[ hardware_index ];

        if ( ! lock->IsLocked )
        {
            lock->IsLocked = true;
            break;
        }
    }
    IF_C ( hardware_index > in_HardwareIndexMax,  err )
    {
        e=E_FEW_ARRAY;
        goto fin;
    }


    *out_ChannelNum = hardware_index - in_HardwareIndexMin;


    e=0;
fin:
    R_OSPL_UnlockCurrentThreadError( &err );
    return  e;
}
#endif
#endif


/***********************************************************************
* Implement: R_BSP_HardwareUnlock
************************************************************************/
#if  R_OSPL_C_LOCK_CODE
bool_t  R_BSP_HardwareUnlock( mcu_lock_t const  in_HardwareIndex )
{
#if BSP_CFG_USER_LOCKING_ENABLED
    return  BSP_CFG_USER_LOCKING_HW_UNLOCK_FUNCTION( in_HardwareIndex );
#else
    bool_t  is_success;


#ifdef  R_OSPL_VERSION_OF_RZ_A1_BSP
    if ( in_HardwareIndex >= BSP_LOCK_DMAC0  &&  in_HardwareIndex <= BSP_LOCK_DMAC15 )
    {
        int_fast32_t  channel_of_DMAC = in_HardwareIndex - BSP_LOCK_DMAC0;
        int_fast32_t  err2;

        err2 = R_DMA_Free( channel_of_DMAC, NULL );
        IF ( err2 != ESUCCESS )
        {
            is_success = false;
            goto fin;
        }

        is_success = true;
        goto fin;
    }
#endif


    IF_D ( R_CUT_IF_ALWAYS_FALSE( in_HardwareIndex < 0u  ||)  in_HardwareIndex >= R_COUNT_OF( g_bsp_Locks ) )
    {
        is_success = false;
        R_OSPL_RaiseUnrecoverable( E_FEW_ARRAY );
        goto fin;
    }

    is_success = R_BSP_SoftwareUnlock( &g_bsp_Locks[ in_HardwareIndex ] );
    IF ( ! is_success )
    {
        goto fin;
    }

    is_success = true;
fin:
    return  is_success;
#endif
}
#endif


/***********************************************************************
* Implement: R_BSP_SoftwareLock
************************************************************************/
#if  R_OSPL_C_LOCK_CODE
bool_t  R_BSP_SoftwareLock( BSP_CFG_USER_LOCKING_TYPE* const  in_LockObject )
{
#if BSP_CFG_USER_LOCKING_ENABLED
    return  BSP_CFG_USER_LOCKING_SW_LOCK_FUNCTION( in_LockObject );
#else
    errnum_t  e;
    r_ospl_error_t*  err = NULL;

    err = R_OSPL_LockCurrentThreadError_Sub( R_OSPL_THREAD_GetCurrentId(), R_OSPL_ALLOCATE_IF_NOT );


    e= R_OSPL_C_LOCK_Lock_Sub( in_LockObject, err );
    if ( e == E_ACCESS_DENIED )
    {
        R_OSPL_CLEAR_ERROR_Sub( err );
        goto fin;
    }
    IF(e)
    {
        goto fin;
    }

    e=0;
fin:
    R_OSPL_UnlockCurrentThreadError( &err );
    return  (bool_t)( e == 0 );
#endif
}
#endif


/***********************************************************************
* Implement: R_BSP_SoftwareUnlock
************************************************************************/
#if  R_OSPL_C_LOCK_CODE
bool_t  R_BSP_SoftwareUnlock( BSP_CFG_USER_LOCKING_TYPE* const  in_LockObject )
{
#if BSP_CFG_USER_LOCKING_ENABLED
    return  BSP_CFG_USER_LOCKING_SW_UNLOCK_FUNCTION( in_LockObject );
#else
    errnum_t  e;
    r_ospl_error_t*  err = NULL;

    err = R_OSPL_LockCurrentThreadError_Sub( R_OSPL_THREAD_GetCurrentId(), R_OSPL_ALLOCATE_IF_NOT );


    e= R_OSPL_C_LOCK_Unlock_Sub( in_LockObject, err );
    if ( e == E_ACCESS_DENIED )
    {
        R_OSPL_CLEAR_ERROR_Sub( err );
        goto fin;
    }
    IF(e)
    {
        goto fin;
    }

    e=0;
fin:
    R_OSPL_UnlockCurrentThreadError( &err );
    return  (bool_t)( e == 0 );
#endif
}
#endif


