/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2016 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/***********************************************************************
* File: mcu_interrupts.c
*    Interrupt related FIT BSP.
*
* - $Version: $ (=R_OSPL_VERSION)
* - $ModuleRevision: {Masters}\OSPL\ALL_Driver_parent\27_RStyled $
* - $Rev: $
* - $Date: 2017-08-29T10:34:43+09:00 $
************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include  "r_ospl.h"
#include  "mcu_interrupts.h"
#if  defined( R_OSPL_FOR_RZ_A1_INTC )
#include  "devdrv_intc.h"
#elif  defined( R_OSPL_FOR_RZ_A1_INTC_RSK )
#include  "intc.h"
#endif


/******************************************************************************
Macro definitions
******************************************************************************/

/***********************************************************************
* Macro: GS_INTERRUPT_DEBUG
*    0 or 1
************************************************************************/
#define  GS_INTERRUPT_DEBUG  0


/******************************************************************************
Typedef definitions
******************************************************************************/

/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/

/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/

/******************************************************************************
Private global variables and functions
******************************************************************************/


/***********************************************************************
* Implement: R_BSP_InterruptWrite
************************************************************************/
#if  R_OSPL_BASE_INTERRUPT_CODE
bsp_int_err_t  R_BSP_InterruptWrite( bsp_int_src_t const  in_IRQ_Num,  bsp_int_cb_t  in_Callback )
{
#if  defined( R_OSPL_FOR_RZ_A1_INTC )  ||  defined( R_OSPL_FOR_RZ_A1_INTC_RSK )
    int32_t        er;
    bsp_int_err_t  eb;

#if  GS_INTERRUPT_DEBUG
    R_D_AddToIntLog( 0x71000000 + in_IRQ_Num );
#endif

    if ( in_Callback == FIT_NO_FUNC )
    {
        in_Callback = &( R_OSPL_OnInterruptForUnregistered );  /* MISRA 16.9 */
    }

#if  defined( R_OSPL_FOR_RZ_A1_INTC )
    er= R_INTC_RegistIntFunc( in_IRQ_Num, in_Callback );
    IF ( er != 0 )
    {
        eb=BSP_INT_ERR_INVALID_ARG;
        goto fin;
    }
#elif  defined( R_OSPL_FOR_RZ_A1_INTC_RSK )
    er= R_INTC_Regist_Int_Func( in_IRQ_Num, in_Callback );
    IF ( er != 0 )
    {
        eb=BSP_INT_ERR_INVALID_ARG;
        goto fin;
    }
#endif

    eb = BSP_INT_SUCCESS;
fin:
    return  eb;


#elif  defined( R_OSPL_FOR_RTX )
    bsp_int_err_t  eb;
    uint32_t       er;
    IRQn_Type      irq;  /* QAC 3441 */

#if  GS_INTERRUPT_DEBUG
    R_D_AddToIntLog( 0x71000000 + in_IRQ_Num );
#endif

    if ( in_Callback == FIT_NO_FUNC )
    {
        irq = R_CAST_bsp_int_src_t_to_IRQn_Type( in_IRQ_Num );
        er= InterruptHandlerUnregister( irq );
        IF ( er != 0u )
        {
            eb=BSP_INT_ERR_INVALID_ARG;
            goto fin;
        }
    }
    else
    {
        irq = R_CAST_bsp_int_src_t_to_IRQn_Type( in_IRQ_Num );  /* QAC 3441 */
        er= InterruptHandlerRegister( irq, in_Callback );
        IF ( er != 0u )
        {
            eb=BSP_INT_ERR_INVALID_ARG;
            goto fin;
        }
    }

    eb = BSP_INT_SUCCESS;
fin:
    return  eb;

#else
#error
#endif
}
#endif


/***********************************************************************
* Implement: R_BSP_InterruptRead
************************************************************************/
#if  R_OSPL_BASE_INTERRUPT_CODE
bsp_int_err_t  R_BSP_InterruptRead( bsp_int_src_t const  in_IRQ_Num,  bsp_int_cb_t* const  out_Callback )
{
    R_UNREFERENCED_VARIABLE_2( in_IRQ_Num, out_Callback );
    return  BSP_INT_ERR_UNSUPPORTED;
}
#endif


/***********************************************************************
* Implement: R_OSPL_OnInterruptForUnregistered
************************************************************************/
#if  R_OSPL_BASE_INTERRUPT_CODE
void  R_OSPL_OnInterruptForUnregistered( uint32_t const  int_sense )
{
    R_UNREFERENCED_VARIABLE( int_sense );
    R_OSPL_RaiseUnrecoverable( E_OTHERS );
}
#endif


/***********************************************************************
* Implement: R_BSP_InterruptControl
************************************************************************/
#if  R_OSPL_BASE_INTERRUPT_CODE
bsp_int_err_t  R_BSP_InterruptControl( bsp_int_src_t const  in_IRQ_Num,  bsp_int_cmd_t const  in_Command,
                                       void* const  in_NotUsed )
{
#if  defined( R_OSPL_FOR_RZ_A1_INTC )  ||  defined( R_OSPL_FOR_RZ_A1_INTC_RSK )
    int32_t        er;
    bsp_int_err_t  eb;

    R_UNREFERENCED_VARIABLE( in_NotUsed );
    R_IT_WILL_BE_NOT_CONST( in_NotUsed );

    switch ( in_Command )
    {
    case  BSP_INT_CMD_INTERRUPT_ENABLE:

#if  GS_INTERRUPT_DEBUG
        R_D_AddToIntLog( 0x71100000 + in_IRQ_Num );
#endif

        er= R_INTC_Enable( in_IRQ_Num );
        IF ( er != 0 )
        {
            eb=BSP_INT_ERR_INVALID_ARG;
            goto fin;
        }
        break;

    default:
        IF_DS( in_Command != BSP_INT_CMD_INTERRUPT_DISABLE )
        {
            eb=BSP_INT_ERR_INVALID_ARG;
            goto fin;
        }

#if  GS_INTERRUPT_DEBUG
        R_D_AddToIntLog( 0x71200000 + in_IRQ_Num );
#endif

        er= R_INTC_Disable( in_IRQ_Num );
        IF ( er != 0 )
        {
            eb=BSP_INT_ERR_INVALID_ARG;
            goto fin;
        }
        break;

    }

    eb = BSP_INT_SUCCESS;
fin:
    return  eb;


#elif  defined( R_OSPL_FOR_RTX )
    bsp_int_err_t  eb;
    IRQn_Type      irq;  /* QAC 3441 */

    R_UNREFERENCED_VARIABLE( in_NotUsed );
    R_IT_WILL_BE_NOT_CONST( in_NotUsed );

    switch ( in_Command )
    {
    case  BSP_INT_CMD_INTERRUPT_ENABLE:

#if  GS_INTERRUPT_DEBUG
        R_D_AddToIntLog( 0x71100000 + in_IRQ_Num );
#endif

        irq = R_CAST_bsp_int_src_t_to_IRQn_Type( in_IRQ_Num );
        GIC_EnableIRQ( irq );
        break;

    default:
        IF_DS( in_Command != BSP_INT_CMD_INTERRUPT_DISABLE )
        {
            eb=BSP_INT_ERR_INVALID_ARG;
            goto fin;
        }

#if  GS_INTERRUPT_DEBUG
        R_D_AddToIntLog( 0x71200000 + in_IRQ_Num );
#endif

        irq = R_CAST_bsp_int_src_t_to_IRQn_Type( in_IRQ_Num );
        GIC_DisableIRQ( irq );
        break;
    }

    eb = BSP_INT_SUCCESS;
#if !defined(NDEBUG) || defined(__QAC_ARM_H__)
fin:
#endif
    return  eb;

#else
#error
#endif
}
#endif


/***********************************************************************
* Implement: R_OSPL_SetInterruptPriority
************************************************************************/
#if  R_OSPL_BASE_INTERRUPT_CODE
errnum_t  R_OSPL_SetInterruptPriority( bsp_int_src_t const  in_IRQ_Num,  int_fast32_t const  in_Priority )
{
#if  defined( R_OSPL_FOR_RZ_A1_INTC )  ||  defined( R_OSPL_FOR_RZ_A1_INTC_RSK )
    errnum_t  e;
    int32_t  er;

    IF ( (in_Priority < 0) || (in_Priority > UINT8_MAX) )
    {
        e=E_OTHERS;
        goto fin;
    }

#if  defined( R_OSPL_FOR_RZ_A1_INTC )
    er= R_INTC_SetPriority( in_IRQ_Num, (uint8_t) in_Priority );
    IF ( er != 0 )
    {
        e=E_OTHERS;
        goto fin;
    }
#elif  defined( R_OSPL_FOR_RZ_A1_INTC_RSK )
    er= R_INTC_Set_Priority( in_IRQ_Num, (uint8_t) in_Priority );
    IF ( er != 0 )
    {
        e=E_OTHERS;
        goto fin;
    }
#endif

    e=0;
fin:
    return  e;


#elif  defined( R_OSPL_FOR_RTX )

    GIC_SetPriority( in_IRQ_Num, (uint32_t) in_Priority );
    return  0;

#else
#error
#endif
}
#endif


