/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2016 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/***********************************************************************
* File: r_ospl_FreeRTOS.c
*    OS Porting Layer API for Coretex-A9 FreeRTOS
*
* - $Version: $ (=R_OSPL_VERSION)
* - $ModuleRevision: {Masters}\OSPL\ALL_Driver_parent\27_RStyled $
* - $Rev: $
* - $Date: 2017-08-29T10:34:43+09:00 $
************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include  "use_list.h"
#if  defined( USE_LIST_FREE_RTOS )
#include  "FreeRTOS.h"
#include  "task.h"
#include  "event_groups.h"
#include  "queue.h"
#include  "semphr.h"
#include  "timers.h"
#endif
#if  defined( USE_LIST_RZ_A1_INIT )  ||  defined( USE_LIST_FREE_RTOS )
#include  "cache.h"
#endif
#include  "r_ospl.h"
#include  "r_ospl_os_less_private.h"
#include  "r_ospl_private.h"
#if  R_OSPL_IS_PREEMPTION
#include  "r_ospl_RTOS_private.h"
#endif
#if R_OSPL_DEBUG_TOOL
#include  <stdio.h>
#endif

#ifdef  tskKERNEL_VERSION_NUMBER
#ifdef  R_OSPL_VERSION


/******************************************************************************
Typedef definitions
******************************************************************************/

/******************************************************************************
Macro definitions
******************************************************************************/

/***********************************************************************
* Macro: OS_ERROR_SIGNAL
*    CMSIS-RTOS defined immediate value
************************************************************************/
#define  OS_ERROR_SIGNAL  0x80000000


/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/
extern uint32_t  ulPortInterruptNesting;

/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/

/******************************************************************************
Private global variables and functions
******************************************************************************/


/***********************************************************************
* Variable: gs_ospl_mutex
************************************************************************/
static SemaphoreHandle_t  gs_ospl_mutex;


/***********************************************************************
* Implement: R_OSPL_InitializeIfNot
************************************************************************/
#if  R_OSPL_VERSION <= 96
errnum_t  R_OSPL_InitializeIfNot(void)
{
    return  0;
}
#endif


/***********************************************************************
* Implement: R_OSPL_Initialize
************************************************************************/
#if  R_OSPL_VERSION <= 96
errnum_t  R_OSPL_Initialize( const void* const  in_NullConfig )
{
    R_UNREFERENCED_VARIABLE( in_NullConfig );
    return  0;
}
#endif


/***********************************************************************
* Implement: R_OSPL_THREAD_GetCurrentId
************************************************************************/
#if  R_OSPL_THREAD_CODE
r_ospl_thread_id_t  R_OSPL_THREAD_GetCurrentId(void)
{
    bool_t  is_interrupt;

    enum { mode_usr = 0x10,  mode_sys = 0x1F,  mode_svc = 0x13 };

    switch ( __get_CPSR() & 0x1F )
    {

    case  mode_usr:
    case  mode_sys:
        is_interrupt = false;
        break;

    case  mode_svc:
        if ( ulPortInterruptNesting == 0 )
        {
            is_interrupt = false;
        }
        else
        {
            is_interrupt = true;
        }
        break;

    default:
        is_interrupt = true;
        break;
    }

    if ( is_interrupt )
    {
        return  R_OSPL_THREAD_NULL;
    }
    else
    {
        return  xTaskGetCurrentTaskHandle();
    }
}
#endif


/***********************************************************************
* Implement: R_OSPL_EVENT_Set
************************************************************************/
#if  R_OSPL_EVENT_CODE
void  R_OSPL_EVENT_Set( r_ospl_thread_id_t const  in_ThreadId,  bit_flags32_t const  in_SetFlags )
{
    int32_t  ret;
#if  R_OSPL_EVENT_OBJECT_CODE
    r_ospl_event_object_id_t  event_object;
#endif
#if  R_OSPL_EVENT_GROUP_CODE
    r_ospl_event_group_id_t  current_event_group;
#endif
    BaseType_t  higher = pdFALSE;  /* Is HigherPriorityTaskWoken */

    if ( in_ThreadId == R_OSPL_THREAD_NULL )
    {
        goto fin;
    }

#if  R_OSPL_EVENT_WATCH
    if ( g_DebugVar[ R_OSPL_EVENT_WATCH ] == NULL  ||
            g_DebugVar[ R_OSPL_EVENT_WATCH ] == (int_fast32_t) in_ThreadId )
    {
        R_D_AddToIntLog( 0x70200000 + (int_fast32_t)( in_SetFlags & R_OSPL_EVENT_ALL_BITS ) );
        R_D_AddToIntLog( (int_fast32_t) (uintptr_t) R_OSPL_THREAD_GetCurrentId() );
        R_D_AddToIntLog( (int_fast32_t) (uintptr_t) in_ThreadId );
    }
#endif
#if  R_OSPL_DETECT_BAD_EVENT
    R_OSPL_EVENT_SetForDebug_Sub( in_ThreadId, in_SetFlags );
#endif

#if  R_OSPL_EVENT_OBJECT_CODE  &&  ( R_OSPL_ERROR_BREAK  ||  R_OSPL_TLS_ERROR_CODE )
    event_object = R_OSPL_EVENT_GetEventObject_Sub( in_ThreadId, in_SetFlags );
    if ( event_object == R_OSPL_EVENT_OBJECT_NULL )  /* If Event Flag */
#endif
    {
#if  R_OSPL_EVENT_GROUP_CODE
        {
            r_ospl_error_t*  err = NULL;

            if ( err == NULL )
            {
                err = R_OSPL_LockCurrentThreadError_Sub( in_ThreadId,  R_OSPL_ALLOCATE_IF_NOT );
            }
            current_event_group = err->EnabledEventGroup;
            R_OSPL_UnlockCurrentThreadError( &err );
            IF ( current_event_group == NULL )
            {
                R_OSPL_RaiseUnrecoverable( E_OTHERS );
                goto fin;
            }
            /* Please set R_OSPL_EVENT_MUST_ALLOCATE = 1, if R_OSPL_EVENT_GROUP_CODE == 1. */
        }
#endif


#if  R_OSPL_EVENT_GROUP_CODE
        if ( R_OSPL_THREAD_GetCurrentId() != R_OSPL_THREAD_NULL )
        {

            xEventGroupSetBits( current_event_group,
                                (EventBits_t)( in_SetFlags & R_OSPL_EVENT_ALL_BITS ) );
        }
        else
        {
            xEventGroupSetBitsFromISR( current_event_group,
                                       (EventBits_t)( in_SetFlags & R_OSPL_EVENT_ALL_BITS ),  &higher );
        }
#endif


    }
#if  R_OSPL_EVENT_OBJECT_CODE  &&  ( R_OSPL_ERROR_BREAK  ||  R_OSPL_TLS_ERROR_CODE )
    else    /* If Event Object */
    {
        BaseType_t  b;

        if ( R_OSPL_THREAD_GetCurrentId() != R_OSPL_THREAD_NULL )
        {

            b= xSemaphoreGive( event_object );
            ASSERT_D( b,  R_NOOP() );
            R_UNREFERENCED_VARIABLE( b );  /* for Release configuration */

        }
        else
        {
            b= xSemaphoreGiveFromISR( event_object,  &higher );
            ASSERT_D( b,  R_NOOP() );
            R_UNREFERENCED_VARIABLE( b );  /* for Release configuration */
        }
    }
#endif

fin:
    portYIELD_FROM_ISR( higher );
    return;
}
#endif


/***********************************************************************
* Implement: R_OSPL_EVENT_Clear
************************************************************************/
#if  R_OSPL_EVENT_CODE
void  R_OSPL_EVENT_Clear( r_ospl_thread_id_t const  in_ThreadId,  bit_flags32_t const  in_ClearFlags1 )
{
    int32_t  ret;
#if  R_OSPL_EVENT_OBJECT_CODE
    r_ospl_event_object_id_t  event_object;
#endif
#if  R_OSPL_EVENT_GROUP_CODE
    r_ospl_event_group_id_t  current_event_group;
#endif
#if  R_OSPL_EVENT_OBJECT_CODE  &&  ( R_OSPL_ERROR_BREAK  ||  R_OSPL_TLS_ERROR_CODE )
    BaseType_t  higher = pdFALSE;  /* Is HigherPriorityTaskWoken */
#endif

    if ( in_ThreadId == R_OSPL_THREAD_NULL )
    {
        goto fin;
    }
    ASSERT_D( R_OSPL_THREAD_GetCurrentId() != R_OSPL_THREAD_NULL,
              R_OSPL_RaiseUnrecoverable( E_NOT_THREAD ); goto fin );
    /* Error: This function can not be called from interrupt context. */
    /* Because portable with RTX */

#if  R_OSPL_EVENT_WATCH
    if ( g_DebugVar[ R_OSPL_EVENT_WATCH ] == NULL  ||
            g_DebugVar[ R_OSPL_EVENT_WATCH ] == (int_fast32_t) in_ThreadId )
    {
        R_D_AddToIntLog( 0x70C00000 + (int_fast32_t)( in_ClearFlags1 & R_OSPL_EVENT_ALL_BITS ) );
        R_D_AddToIntLog( (int_fast32_t) (uintptr_t) R_OSPL_THREAD_GetCurrentId() );
        R_D_AddToIntLog( (int_fast32_t) (uintptr_t) in_ThreadId );
    }
#endif

#if  R_OSPL_EVENT_OBJECT_CODE  &&  ( R_OSPL_ERROR_BREAK  ||  R_OSPL_TLS_ERROR_CODE )
    event_object = R_OSPL_EVENT_GetEventObject_Sub( in_ThreadId, in_ClearFlags1 );
    if ( event_object == R_OSPL_EVENT_OBJECT_NULL )  /* If Event Flag */
#endif
    {
        if ( in_ClearFlags1 != 0 )    /* For avoiding error in osSignalClear */
        {

#if  R_OSPL_EVENT_GROUP_CODE
            {
                r_ospl_error_t*  err = NULL;

                if ( err == NULL )
                {
                    err = R_OSPL_LockCurrentThreadError_Sub( in_ThreadId,  R_OSPL_ALLOCATE_IF_NOT );
                }
                current_event_group = err->EnabledEventGroup;
                R_OSPL_UnlockCurrentThreadError( &err );
                if ( current_event_group != NULL )
                {
                    if ( R_OSPL_THREAD_GetCurrentId() != R_OSPL_THREAD_NULL )
                    {


                        xEventGroupClearBits( current_event_group,
                        (EventBits_t)( in_ClearFlags1 & R_OSPL_EVENT_ALL_BITS ) );
                        /* "& R_OSPL_EVENT_ALL_BITS" is for avoiding error in osSignalClear */


                    }
                    else
                    {
                        xEventGroupClearBitsFromISR( current_event_group,
                                                     (EventBits_t)( in_ClearFlags1 & R_OSPL_EVENT_ALL_BITS ) );
                    }
                }
                else {
                    /* Clear all bits when event group was created. */
                }
            }
#endif
        }
    }
#if  R_OSPL_EVENT_OBJECT_CODE  &&  ( R_OSPL_ERROR_BREAK  ||  R_OSPL_TLS_ERROR_CODE )
    else    /* If Event Object */
    {
        BaseType_t  b;

        if ( R_OSPL_THREAD_GetCurrentId() != R_OSPL_THREAD_NULL )
        {

            b= xSemaphoreTake( event_object,  0 );  /* Change to empty state. */
            R_UNREFERENCED_VARIABLE( b );  /* Don't care previous state. */

        }
        else
        {
            b= xSemaphoreTakeFromISR( event_object,  &higher );  /* Change to empty state. */
            R_UNREFERENCED_VARIABLE( b );
        }
    }
#endif

fin:
#if  R_OSPL_EVENT_OBJECT_CODE  &&  ( R_OSPL_ERROR_BREAK  ||  R_OSPL_TLS_ERROR_CODE )
    portYIELD_FROM_ISR( higher );
#endif
    return;
}
#endif


/***********************************************************************
* Implement: R_OSPL_EVENT_Wait
************************************************************************/
#if  R_OSPL_EVENT_CODE
errnum_t  R_OSPL_EVENT_Wait( bit_flags32_t const  in_WaigingFlags,  bit_flags32_t* const  out_GotFlags,
                             uint32_t const  in_Timeout_msec )
{
    errnum_t  e;
#if  R_OSPL_EVENT_OBJECT_CODE  ||  R_OSPL_EVENT_WATCH  ||  R_OSPL_DETECT_BAD_EVENT  ||  R_OSPL_EVENT_GROUP_CODE
    r_ospl_thread_id_t const  current_thread = (r_ospl_thread_id_t) R_OSPL_THREAD_GetCurrentId();
#endif
#if  R_OSPL_EVENT_GROUP_CODE
    r_ospl_event_group_id_t  current_event_group;
#endif
#if  R_OSPL_EVENT_OBJECT_CODE
    r_ospl_event_object_id_t  event_object;
#endif
#if  R_OSPL_DETECT_BAD_EVENT
    r_ospl_event_status_t*  status = NULL;
#endif
#if  R_OSPL_DETECT_BAD_EVENT  ||  R_OSPL_EVENT_GROUP_CODE
    r_ospl_error_t*  err = NULL;
#endif
    EventBits_t  got_flags = 0x00000000u;


    R_STATIC_ASSERT( R_OSPL_INFINITE == portMAX_DELAY, "" );

#if  R_OSPL_EVENT_WATCH
    if ( in_Timeout_msec > 0 )
    {
        R_D_AddToIntLog( 0x70BE0000 + (int_fast32_t) in_WaigingFlags );
        R_D_AddToIntLog( (int_fast32_t) (uintptr_t) current_thread );

        {
            static int tc;
            if ( R_D_Counter( &tc, 0, NULL ) )
            {
                R_DEBUG_BREAK();
            }
        }
    }
#endif

    ASSERT_R( R_OSPL_THREAD_GetCurrentId() != R_OSPL_THREAD_NULL,  e=E_NOT_THREAD;  goto fin );

#if  R_OSPL_EVENT_GROUP_CODE
    if ( err == NULL )
    {
        err = R_OSPL_LockCurrentThreadError_Sub( current_thread,  R_OSPL_ALLOCATE_IF_NOT );
    }
    current_event_group = err->EnabledEventGroup;
    R_OSPL_UnlockCurrentThreadError( &err );
    IF ( current_event_group == NULL )
    {
        e=E_STATE;
        goto fin;
    }
#endif

#if  R_OSPL_EVENT_OBJECT_CODE
    event_object = R_OSPL_EVENT_GetEventObject_Sub( current_thread,  R_OSPL_EVENT_OBJECT_FLAG );

    if ( event_object != R_OSPL_EVENT_OBJECT_NULL )
    {
        ASSERT_D( in_WaigingFlags == R_OSPL_ANY_FLAG  ||
                  IS_BIT_SET( in_WaigingFlags, R_OSPL_EVENT_OBJECT_FLAG ),
                  R_NOOP() );
    }
#endif

#if  R_OSPL_DETECT_BAD_EVENT
    e= R_OSPL_EVENT_GetStatus( current_thread,  &status );
    IF(e!=0)
    {
        goto fin;
    }

    if ( in_Timeout_msec != 0 )
    {
        ASSERT_R( status->AllocatedEvents != 0u,  R_OSPL_BAD_EVENT_ERROR() );
        /* Error: Any events are not allocated. */
    }
#endif


#if  R_OSPL_EVENT_OBJECT_CODE
    if ( event_object == R_OSPL_EVENT_OBJECT_NULL )  /* If Event Flag */
#endif
    {
        EventBits_t  current_bits;
        EventBits_t  bits_to_wait_for;
        BaseType_t   is_waiting_for_all;

        if ( in_WaigingFlags == R_OSPL_ANY_FLAG )
        {
            bits_to_wait_for = R_OSPL_EVENT_ALL_BITS;
            is_waiting_for_all = false;
        }
        else if ( IS_BIT_SET( in_WaigingFlags,  R_OSPL_OR_FLAG ) )
        {
            bits_to_wait_for = (EventBits_t)( in_WaigingFlags & R_OSPL_EVENT_ALL_BITS );
            is_waiting_for_all = false;
        }
        else
        {
            bits_to_wait_for = (EventBits_t)( in_WaigingFlags & R_OSPL_EVENT_ALL_BITS );
            is_waiting_for_all = true;
        }

#if  R_OSPL_EVENT_GROUP_CODE

        current_bits = xEventGroupWaitBits( current_event_group,  bits_to_wait_for,
                                            pdTRUE,  is_waiting_for_all,  in_Timeout_msec / portTICK_RATE_MS );

#endif

        if ( IS( is_waiting_for_all ) )
        {
            if ( IS_ALL_BITS_SET( current_bits,  bits_to_wait_for ) )
            {
                got_flags = bits_to_wait_for;
            }
            else
            {
                got_flags = R_OSPL_TIMEOUT;
            }
        }
        else
        {
            if ( IS_ANY_BITS_SET( current_bits,  bits_to_wait_for ) )
            {
                got_flags = current_bits & bits_to_wait_for;
            }
            else
            {
                got_flags = R_OSPL_TIMEOUT;
            }
        }
    }

#if  R_OSPL_EVENT_OBJECT_CODE
    else    /* If Event Object */
    {
        BaseType_t  b;


        ASSERT_R( IS_BIT_NOT_SET( in_WaigingFlags,  R_OSPL_OR_FLAG ),  e=E_OTHERS; goto fin );
        /* Error: It is not possible to specify both event object and R_OSPL_OR_FLAG.*/
        /* Error: It is not possible to specify both event object and R_OSPL_OR_FLAG. */


        b= xSemaphoreTake( event_object,  in_Timeout_msec / portTICK_RATE_MS );


        if ( ! b )
        {
            got_flags = R_OSPL_TIMEOUT;
        }
        else
        {
            got_flags = R_OSPL_EVENT_OBJECT_FLAG;
        }
    }
#endif


#if  R_OSPL_DETECT_BAD_EVENT  ||  R_OSPL_EVENT_GROUP_CODE
    if ( err == NULL )
    {
        err = R_OSPL_LockCurrentThreadError_Sub( current_thread,  R_OSPL_ALLOCATE_IF_NOT );
    }
#endif

#if  R_OSPL_DETECT_BAD_EVENT
    {
        uint32_t  got_flags_2 = got_flags & ( R_OSPL_EVENT_OBJECT_FLAG | R_OSPL_EVENT_ALL_BITS );

        ASSERT_RC( IS_ALL_BITS_SET( status->AllocatedEvents,  got_flags_2 ),  err,  R_OSPL_BAD_EVENT_ERROR() );

        err->EventStatus.UnexpectedEvents |= got_flags_2;
    }
#endif

#if  R_OSPL_EVENT_GROUP_CODE
    IF_C ( in_Timeout_msec > 0  &&  got_flags == R_OSPL_TIMEOUT,  err )
    {
        e=E_TIME_OUT;
        goto fin;
    }
#endif

    e=0;
fin:
#if  R_OSPL_DETECT_BAD_EVENT  ||  R_OSPL_EVENT_GROUP_CODE
    R_OSPL_UnlockCurrentThreadError( &err );
#endif
#if  R_OSPL_EVENT_WATCH
    if ( in_Timeout_msec > 0 )
    {
        R_D_AddToIntLog( 0x70AF0000 );
        R_D_AddToIntLog( (int_fast32_t) (uintptr_t) current_thread );
#if  R_OSPL_EVENT_GROUP_CODE
        R_D_AddToIntLog( (int_fast32_t) (uintptr_t) current_event_group );
#endif
        R_D_AddToIntLog( (int_fast32_t) got_flags );
    }
#endif
    if ( out_GotFlags != NULL )
    {
        *out_GotFlags = got_flags;
    }
    return  e;
}
#endif


/***********************************************************************
* Implement: R_OSPL_EVENT_GROUP_Create
************************************************************************/
#if  R_OSPL_EVENT_CODE
#if  R_OSPL_EVENT_GROUP_CODE
errnum_t  R_OSPL_EVENT_GROUP_Create( volatile r_ospl_event_group_id_t*  out_EventGroupId,
                                     r_ospl_thread_id_t  in_ThreadId )
{
    errnum_t  e;

    R_UNREFERENCED_VARIABLE( in_ThreadId );
    ASSERT_R( R_OSPL_THREAD_GetCurrentId() != R_OSPL_THREAD_NULL,  e=E_NOT_THREAD;  goto fin );

    *out_EventGroupId = xEventGroupCreate();

    e=0;
fin:
    return  e;
}
#endif
#endif


/***********************************************************************
* Implement: R_OSPL_EVENT_GROUP_Delete
************************************************************************/
#if  R_OSPL_EVENT_CODE
#if  R_OSPL_EVENT_GROUP_CODE
errnum_t  R_OSPL_EVENT_GROUP_Delete( volatile r_ospl_event_group_id_t  in_EventGroupId,
                                     r_ospl_thread_id_t  in_ThreadId )
{
    errnum_t  e;

    R_UNREFERENCED_VARIABLE( in_ThreadId );
    ASSERT_R( R_OSPL_THREAD_GetCurrentId() != R_OSPL_THREAD_NULL,  e=E_NOT_THREAD;  goto fin );

    vEventGroupDelete( in_EventGroupId );

    e=0;
fin:
    return  e;
}
#endif
#endif


/***********************************************************************
* Implement: R_OSPL_MEMORY_Flush
************************************************************************/
#if  R_OSPL_MEMORY_CONTROL_CODE
void  R_OSPL_MEMORY_Flush( r_ospl_flush_t const  in_FlushType )
{
    if ( in_FlushType == R_OSPL_FLUSH_WRITEBACK_INVALIDATE )
    {
#if 0
        printf( "L1Flush\n" );
#endif


#if  defined( R_OSPL_FOR_RZ_A1_CACHE )
        L1_D_CacheWritebackFlushAll();
#elif  defined( R_OSPL_FOR_RTX )
        __v7_clean_inv_dcache_all();
#else
#error
#endif


    }
    else if ( in_FlushType == R_OSPL_FLUSH_WRITEBACK_INVALIDATE_2ND )
    {
        ASSERT_D( false,  R_NOOP() );
    }
    else
    {
        ASSERT_D( false,  R_NOOP() );
    }
}
#endif


/***********************************************************************
* Function: R_OSPL_Is1bitOnly_Fast32_Sub
*    R_OSPL_Is1bitOnly_Fast32_Sub
************************************************************************/
#if  R_OSPL_BASIC_TYPES_CODE
#ifndef  R_OSPL_NDEBUG
static bool_t  R_OSPL_Is1bitOnly_Fast32_Sub( uint_fast32_t in_Value )
{
    if ( (in_Value & 0x0000FFFFu) == 0 )
    {
        in_Value >>= 16;
    }
    if ( (in_Value & 0x000000FFu) == 0 )
    {
        in_Value >>=  8;
    }
    if ( (in_Value & 0x0000000Fu) == 0 )
    {
        in_Value >>=  4;
    }
    if ( (in_Value & 0x00000003u) == 0 )
    {
        in_Value >>=  2;
    }
    if ( (in_Value & 0x00000001u) == 0 )
    {
        in_Value >>=  1;
    }
    return  ( in_Value == 1 );
}
#endif
#endif


/***********************************************************************
* Implement: R_OSPL_MEMORY_RangeFlush
************************************************************************/
#if  R_OSPL_MEMORY_CONTROL_CODE
errnum_t  R_OSPL_MEMORY_RangeFlush( r_ospl_flush_t const  in_FlushType,
                                    const void* const  in_StartAddress,  size_t const  in_Length )
{
    errnum_t   e;
    size_t     cache_line_size;
    size_t     cache_line_mask;
    uintptr_t  start;
    uintptr_t  over;

    ASSERT_R( in_FlushType == R_OSPL_FLUSH_INVALIDATE,  e=E_BAD_COMMAND_ID; goto fin );

    cache_line_size = R_OSPL_MEMORY_GetCacheLineSize();
    cache_line_mask = cache_line_size - 1u;
    ASSERT_D( R_OSPL_Is1bitOnly_Fast32_Sub( cache_line_size ), e=E_OTHERS; goto fin );

    /* ->MISRA 11.3 */ /* ->SEC R2.7.1 */
    ASSERT_R( ( (uintptr_t) in_StartAddress & cache_line_mask ) == 0u, e=E_OTHERS; goto fin );
    ASSERT_R( ( in_Length & cache_line_mask ) == 0u, e=E_OTHERS; goto fin );

    start = (uintptr_t) in_StartAddress;
    over  = ( (uintptr_t) in_StartAddress + in_Length ) - 1u;
    /* <-MISRA 11.3 */ /* <-SEC R2.7.1 */

    R_OSPL_MEMORY_RangeFlush_Sub( start, over, cache_line_size );

    e=0;
fin:
    return  e;
}
#endif


/***********************************************************************
* Implement: R_OSPL_MEMORY_GetSpecification
************************************************************************/
#if  R_OSPL_MEMORY_CONTROL_CODE
void  R_OSPL_MEMORY_GetSpecification( r_ospl_memory_spec_t* const  out_MemorySpec )
{
    IF_DQ( out_MemorySpec == NULL )
    {
        goto fin;
    }

    out_MemorySpec->CacheLineSize = R_OSPL_MEMORY_GetCacheLineSize();

fin:
    return;
}
#endif


/***********************************************************************
* Implement: R_OSPL_Delay
************************************************************************/
#if  R_OSPL_DELAY_CODE
errnum_t  R_OSPL_Delay( uint32_t const  in_DelayTime_msec )
{
    errnum_t  e;
    uint32_t  plus_time;

    bool_t const  is_overflow = ( in_DelayTime_msec > R_OSPL_MAX_TIME_OUT );

    ASSERT_D( ! is_overflow, R_NOOP() );
    /* "R_OSPL_MAX_TIME_OUT" is not full of "TickType_t" for portability */
    /* RTX 5.16: If delay_parameter = 100000, "osDelay" waits 65534 */

    IF ( R_OSPL_THREAD_GetCurrentId() == R_OSPL_THREAD_NULL )
    {
        e = E_NOT_THREAD;
        R_OSPL_RaiseUnrecoverable( e );
        goto fin;
    }


    if ( in_DelayTime_msec >= 1 )
    {
        plus_time = 1;
        /* Wait equal or longer than "in_DelayTime_msec". */
        /* Because "xTimeToWake = xTickCount + xTicksToDelay" in implementation of "vTaskDelay" */
    }
    else
    {
        plus_time = 0;
    }

    vTaskDelay( in_DelayTime_msec / portTICK_RATE_MS + plus_time );


    IF ( IS( is_overflow ) )
    {
        e=E_TIME_OUT;
        goto fin;
    }

    e=0;
fin:
    return  e;
}
#endif


/***********************************************************************
* Implement: R_OSPL_QUEUE_Create
************************************************************************/
#if  R_OSPL_QUEUE_CODE
errnum_t  R_OSPL_QUEUE_Create( r_ospl_queue_id_t* out_self, r_ospl_queue_def_t* in_QueueDefine )
{
    errnum_t  e;
    r_ospl_queue_def_t*  self = in_QueueDefine;

    self->Queue = NULL;
    self->Semaphore = NULL;

    R_OSPL_TABLE_InitConst( self->Table,  self->Table->Area,
                            (size_t) self->PublicStatus.MaxCount * sizeof( r_ospl_table_block_t ),
                            R_OSPL_TABLE_I_LOCK );

    ASSERT_R( R_OSPL_THREAD_GetCurrentId() != R_OSPL_THREAD_NULL,  e=E_NOT_THREAD;  goto fin );


    self->Queue = xQueueCreate( (UBaseType_t) self->PublicStatus.MaxCount,  sizeof(void*) );
    IF ( self->Queue == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }


    self->Semaphore = xSemaphoreCreateCounting(
                          (UBaseType_t) self->PublicStatus.MaxCount,
                          (UBaseType_t) self->PublicStatus.MaxCount );
    IF ( self->Semaphore == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }


    self->PublicStatus.UsedCount = 0;

    *out_self = self;

    e=0;
fin:
    if ( e != 0 )    /* Rollback */
    {
        if ( self->Semaphore != NULL )
        {
            vSemaphoreDelete( self->Semaphore );
            self->Semaphore = NULL;
        }
        if ( self->Queue != NULL )
        {
            vQueueDelete( self->Queue );
            self->Queue = NULL;
        }
    }
    return  e;
}
#endif


/***********************************************************************
* Implement: R_OSPL_QUEUE_GetStatus
************************************************************************/
#if  R_OSPL_QUEUE_CODE
errnum_t  R_OSPL_QUEUE_GetStatus( r_ospl_queue_id_t  self,  const r_ospl_queue_status_t** out_Status )
{
    *out_Status = &self->PublicStatus;
    return  0;
}
#endif


/***********************************************************************
* Implement: R_OSPL_QUEUE_Allocate
************************************************************************/
#if  R_OSPL_QUEUE_CODE
errnum_t  R_OSPL_QUEUE_Allocate( r_ospl_queue_id_t  self,  void* out_Address,  uint32_t  in_Timeout_msec )
{
    errnum_t  e;
    bool_t    was_all_enabled = false;

    BaseType_t    b;
    int_fast32_t  i;
    BaseType_t    higher = pdFALSE;  /* Is HigherPriorityTaskWoken */


    if ( R_OSPL_THREAD_GetCurrentId() != R_OSPL_THREAD_NULL )
    {

        b= xSemaphoreTake( self->Semaphore,  in_Timeout_msec );

        IF ( ! b )
        {
            if ( in_Timeout_msec == 0u )
            {
                *(void**)out_Address = NULL;
                e = 0;
            }
            else
            {
                e = E_TIME_OUT;
            }
            goto fin;
        }
    }
    else
    {
        ASSERT_R( in_Timeout_msec == 0,  e=E_NOT_THREAD; goto fin );

        b= xSemaphoreTakeFromISR( self->Semaphore,  &higher );
        if ( ! b )
        {
            *(void**)out_Address = NULL;
            e=0;
            goto fin;
        }
    }


    e= R_OSPL_TABLE_GetIndex( self->Table,  NULL,
                              &i,  R_OSPL_ALLOCATE_IF_EXIST_OR_IF_NOT );
    IF(e)
    {
        goto fin;
    }
    *(void**) out_Address = &self->MemoryArray[ (size_t) i * self->ElementSize ];


    was_all_enabled = R_OSPL_DisableAllInterrupt();
    self->PublicStatus.UsedCount += 1;

    e=0;
fin:
    if ( was_all_enabled )
    {
        R_OSPL_EnableAllInterrupt();
    }
    portYIELD_FROM_ISR( higher );
    return  e;
}
#endif


/***********************************************************************
* Implement: R_OSPL_QUEUE_Put
************************************************************************/
#if  R_OSPL_QUEUE_CODE
errnum_t  R_OSPL_QUEUE_Put( r_ospl_queue_id_t  self,  void* in_Address )
{
    errnum_t  e;
    BaseType_t  b;


    if ( R_OSPL_THREAD_GetCurrentId() != R_OSPL_THREAD_NULL )
    {

        b= xQueueSendToBack( self->Queue,  &in_Address,  0 );
    }
    else
    {
        b= xQueueSendToBackFromISR( self->Queue,  &in_Address,  0 );
    }
    IF ( ! b )
    {
        e=E_OTHERS;
        goto fin;
    }


    e=0;
fin:
    return  e;
}
#endif


/***********************************************************************
* Implement: R_OSPL_QUEUE_Get
************************************************************************/
#if  R_OSPL_QUEUE_CODE
errnum_t  R_OSPL_QUEUE_Get( r_ospl_queue_id_t  self,  void* out_Address,  uint32_t  in_Timeout_msec )
{
    errnum_t  e;
    BaseType_t  b;
    BaseType_t  higher = pdFALSE;  /* Is HigherPriorityTaskWoken */


    if ( R_OSPL_THREAD_GetCurrentId() != R_OSPL_THREAD_NULL )
    {

        b= xQueueReceive( self->Queue,  out_Address,  in_Timeout_msec );

        if ( b == errQUEUE_EMPTY )    /* Attention - (errQUEUE_EMPTY in code) == (pdFALSE in document) */
        {
            if ( in_Timeout_msec == 0 )
            {
                *(void**) out_Address = NULL;
                b = pdTRUE;
            }
        }
    }
    else
    {
        ASSERT_R( in_Timeout_msec == 0,  e=E_NOT_THREAD; goto fin );

        b= xQueueReceiveFromISR( self->Queue,  out_Address,  &higher );
    }
    IF ( ! b )
    {
        e=E_OTHERS;
        goto fin;
    }


    e=0;
fin:
    portYIELD_FROM_ISR( higher );
    return  e;
}
#endif


/***********************************************************************
* Implement: R_OSPL_QUEUE_Free
************************************************************************/
#if  R_OSPL_QUEUE_CODE
errnum_t  R_OSPL_QUEUE_Free( r_ospl_queue_id_t  self,  void* in_Address )
{
    errnum_t  e;
    bool_t    was_all_enabled = false;

    int_fast32_t  index = ( (byte_t*) in_Address - (byte_t*) &self->MemoryArray[ 0 ] ) / (int_fast32_t) self->ElementSize;
    BaseType_t    b;
    BaseType_t    higher = pdFALSE;  /* Is HigherPriorityTaskWoken */

    e = 0;

    R_OSPL_TABLE_FreeByIndex( self->Table,  index );
    was_all_enabled = R_OSPL_DisableAllInterrupt();

    self->PublicStatus.UsedCount -= 1;
    if ( was_all_enabled )
    {
        R_OSPL_EnableAllInterrupt();
    }


    if ( R_OSPL_THREAD_GetCurrentId() != R_OSPL_THREAD_NULL )
    {

        b= xSemaphoreGive( self->Semaphore );
    }
    else
    {
        b= xSemaphoreGiveFromISR( self->Semaphore,  &higher );
    }
    IF(!b)
    {
        e= R_OSPL_MergeErrNum( e, E_OTHERS );
    }

    portYIELD_FROM_ISR( higher );
    return  e;
}
#endif


/***********************************************************************
* Variable: gs_QueueAsSingletonLockConfig
************************************************************************/
#if  R_OSPL_QUEUE_CODE
static R_OSPL_QUEUE_DEF( gs_QueueAsSingletonLockConfig,  1,  int_fast32_t );
#endif


/***********************************************************************
* Implement: R_OSPL_GetQueueAsSingletonLock
************************************************************************/
#if  R_OSPL_QUEUE_CODE
r_ospl_queue_id_t  R_OSPL_GetQueueAsSingletonLock(void)
{
    static r_ospl_queue_id_t  gs_QueueAsSingletonLock = R_OSPL_QUEUE_NULL;

    if ( gs_QueueAsSingletonLock == R_OSPL_QUEUE_NULL )
    {

        ASSERT_D( R_OSPL_THREAD_GetCurrentId() != R_OSPL_THREAD_NULL,  R_NOOP() );
        R_OSPL_Start_T_Lock();
        /* This area must be thread safe. */
        if ( gs_QueueAsSingletonLock == R_OSPL_QUEUE_NULL )
        {

            R_OSPL_QUEUE_Create( &gs_QueueAsSingletonLock,
                                 R_OSPL_QUEUE( gs_QueueAsSingletonLockConfig ) );
        }

        R_OSPL_End_T_Lock();
    }
    return  gs_QueueAsSingletonLock;
}
#endif


/***********************************************************************
* Implement: R_OSPL_Start_T_Lock
*    The function callbacked from OSPL internal, when T-Lock started
*
* Arguments:
*    None
*
* Return Value:
*    Error Code. 0=No Error.
*
* InternalDescription:
*    See. <R_OSPL_LockCurrentThreadError_Sub>.
************************************************************************/
#if  R_OSPL_INTERNAL_T_LOCK_CODE
#if  R_OSPL_IS_PREEMPTION
errnum_t  R_OSPL_Start_T_Lock(void)
{
    errnum_t    e;
    BaseType_t  b;


#if  R_OSPL_T_LOCK_WATCH
    R_D_AddToIntLog( (int_fast32_t) 0xDDDD7101 );
    R_D_AddToIntLog( (int_fast32_t)(uintptr_t) R_OSPL_THREAD_GetCurrentId() );
#endif

    if ( R_OSPL_THREAD_GetCurrentId() == R_OSPL_THREAD_NULL )
    {
        R_OSPL_RaiseUnrecoverable( E_NOT_THREAD );
    }

    if ( gs_ospl_mutex == NULL )
    {

        gs_ospl_mutex = xSemaphoreCreateBinary();
        if ( gs_ospl_mutex == NULL )
        {
            e=E_OTHERS;
            goto fin;
        }
        b= xSemaphoreGive( gs_ospl_mutex );
        if(!b)
        {
            e=E_OTHERS;
            goto fin;
        }
    }


    b= xSemaphoreTake( gs_ospl_mutex,  portMAX_DELAY );
    if (!b)
    {
        e=E_OTHERS;
        goto fin;
    }

    e=0;
fin:
#if  R_OSPL_T_LOCK_WATCH
    R_D_AddToIntLog( (int_fast32_t) 0xDDDD7102 );
    R_D_AddToIntLog( (int_fast32_t) (uintptr_t) R_OSPL_THREAD_GetCurrentId() );
#endif
    return  e;
}
#endif
#endif


/***********************************************************************
* Implement: R_OSPL_End_T_Lock
*    The function callbacked from OSPL internal, when T-Lock ended
*
* Arguments:
*    None
*
* Return Value:
*    None
*
* InternalDescription:
*    See. <R_OSPL_UnlockCurrentThreadError>.
************************************************************************/
#if  R_OSPL_INTERNAL_T_LOCK_CODE
#if  R_OSPL_IS_PREEMPTION
void  R_OSPL_End_T_Lock(void)
{
#if  R_OSPL_T_LOCK_WATCH
    R_D_AddToIntLog( (int_fast32_t) 0xDDDD710F );
    R_D_AddToIntLog( (int_fast32_t)(uintptr_t) R_OSPL_THREAD_GetCurrentId() );
#endif

    if ( R_OSPL_THREAD_GetCurrentId() == R_OSPL_THREAD_NULL )
    {
        R_OSPL_RaiseUnrecoverable( E_NOT_THREAD );
    }

    if ( gs_ospl_mutex != NULL )
    {
        BaseType_t  b;

        b= xSemaphoreGive( gs_ospl_mutex );
        ASSERT_R( b,  R_OSPL_RaiseUnrecoverable( E_OTHERS ) );
    }
}
#endif
#endif


#endif  /* R_OSPL_VERSION */
#endif  /* tskKERNEL_VERSION_NUMBER */
