/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2017 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/***********************************************************************
* File: r_ospl_RTX.c
*    OS Porting Layer API for RTX
*
* - $Version: $ (=R_OSPL_VERSION)
* - $ModuleRevision: {Masters}\OSPL\ALL_Driver_parent\27_RStyled $
* - $Rev: $
* - $Date: 2017-08-21T14:17:21+09:00 $
************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include  "use_list.h"
#include  "r_ospl.h"
#include  "r_ospl_os_less_private.h"
#include  "r_ospl_private.h"
#if  defined( USE_LIST_RTX )
#include  "pl310.h"  /* 2nd cache */
#if  R_OSPL_IS_PREEMPTION
#include  "cmsis_os.h"
#include  "gic.h"
#include  "r_ospl_RTOS_private.h"
#endif
#endif

#ifdef  osCMSIS_RTX
#ifdef  R_OSPL_VERSION


/******************************************************************************
Typedef definitions
******************************************************************************/

/******************************************************************************
Macro definitions
******************************************************************************/

/***********************************************************************
* Define: GS_IS_CHECK_TO_ENABLE_ALL_INTERRUPT_USED
*    0 or 1.  RTX 4.80 = 1.
************************************************************************/
#define  GS_IS_CHECK_TO_ENABLE_ALL_INTERRUPT_USED  1


/***********************************************************************
* Macro: OS_ERROR_SIGNAL
*    CMSIS-RTOS defined immediate value
************************************************************************/
#define  OS_ERROR_SIGNAL  0x80000000


/***********************************************************************
* Macro: GS_TO_UNSIGNED_A
*    TO_UNSIGNED for osWaitForever.
************************************************************************/
#if  osCMSIS_RTX >= ((4<<16)|80)
#define  GS_TO_UNSIGNED_A( x )  ( x )
#else
#define  GS_TO_UNSIGNED_A( x )  TO_UNSIGNED( x )
#endif


/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/

/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/

/******************************************************************************
Private global variables and functions
******************************************************************************/


/***********************************************************************
* Variable: gs_ospl_mutex
************************************************************************/
static osMutexId   gs_ospl_mutex;
static osMutexDef( gs_ospl_mutex );


/***********************************************************************
* Function: GS_CHECK_TO_ENABLE_ALL_INTERRUPT
*
* Description:
*    RTX API can be called, when all interrupts (IRQ) was enabled.
************************************************************************/
STATIC_INLINE  void  GS_CHECK_TO_ENABLE_ALL_INTERRUPT()
{
#if  GS_IS_CHECK_TO_ENABLE_ALL_INTERRUPT_USED
    bool_t  is_enabled = R_OSPL_GetIsAllInterruptEnabled();

    if ( ! is_enabled )
    {
        R_OSPL_RaiseUnrecoverable( E_OTHERS );
    }
    /* Error: This area requests interrupt is enabled.  Please, call "R_OSPL_EnableAllInterrupt". */
#endif
}

/***********************************************************************
* Implement: R_OSPL_InitializeIfNot
************************************************************************/
#if  R_OSPL_VERSION <= 96
errnum_t  R_OSPL_InitializeIfNot(void)
{
    return  0;
}
#endif


/***********************************************************************
* Implement: R_OSPL_Initialize
************************************************************************/
#if  R_OSPL_VERSION <= 96
errnum_t  R_OSPL_Initialize( const void* const  in_NullConfig )
{
    R_UNREFERENCED_VARIABLE( in_NullConfig );
    return  0;
}
#endif


/***********************************************************************
* Implement: R_OSPL_THREAD_GetCurrentId
************************************************************************/
#if  R_OSPL_THREAD_CODE
r_ospl_thread_id_t  R_OSPL_THREAD_GetCurrentId(void)
{
    GS_CHECK_TO_ENABLE_ALL_INTERRUPT();

    return  osThreadGetId();
}
#endif


/***********************************************************************
* Implement: R_OSPL_EVENT_Set
************************************************************************/
#if  R_OSPL_EVENT_CODE
void  R_OSPL_EVENT_Set( r_ospl_thread_id_t const  in_ThreadId,  bit_flags32_t const  in_SetFlags )
{
    int32_t  ret;
#if  R_OSPL_EVENT_OBJECT_CODE
    r_ospl_event_object_id_t  event_object;
#endif
#if  R_OSPL_EVENT_GROUP_CODE
    r_ospl_event_group_id_t  current_event_group;
#endif

    if ( in_ThreadId == R_OSPL_THREAD_NULL )
    {
        goto fin;
    }

#if  R_OSPL_EVENT_WATCH
    if ( g_DebugVar[ R_OSPL_EVENT_WATCH ] == NULL  ||
            g_DebugVar[ R_OSPL_EVENT_WATCH ] == (int_fast32_t) in_ThreadId )
    {
        R_D_AddToIntLog( 0x70200000 + (int_fast32_t)( in_SetFlags & R_OSPL_EVENT_ALL_BITS ) );
        R_D_AddToIntLog( (int_fast32_t) (uintptr_t) R_OSPL_THREAD_GetCurrentId() );
        R_D_AddToIntLog( (int_fast32_t) (uintptr_t) in_ThreadId );
    }
#endif
#if  R_OSPL_DETECT_BAD_EVENT
    R_OSPL_EVENT_SetForDebug_Sub( in_ThreadId, in_SetFlags );
#endif

    GS_CHECK_TO_ENABLE_ALL_INTERRUPT();

#if  R_OSPL_EVENT_OBJECT_CODE  &&  ( R_OSPL_ERROR_BREAK  ||  R_OSPL_TLS_ERROR_CODE )
    event_object = R_OSPL_EVENT_GetEventObject_Sub( in_ThreadId, in_SetFlags );
    if ( event_object == R_OSPL_EVENT_OBJECT_NULL )  /* If Event Flag */
#endif
    {
#if  R_OSPL_EVENT_GROUP_CODE
        {
            r_ospl_error_t*  err = NULL;

            if ( err == NULL )
            {
                err = R_OSPL_LockCurrentThreadError_Sub( in_ThreadId,  R_OSPL_ALLOCATE_IF_NOT );
            }
            current_event_group = err->EnabledEventGroup;
            R_OSPL_UnlockCurrentThreadError( &err );
            IF ( current_event_group == NULL )
            {
                R_OSPL_RaiseUnrecoverable( E_OTHERS );
                goto fin;
            }
        }
#endif

        ret = osSignalSet( (osThreadId) in_ThreadId,
                           (int32_t)( in_SetFlags & R_OSPL_EVENT_ALL_BITS ) );
        ASSERT_D( (ret & OS_ERROR_SIGNAL) == 0,  R_NOOP() );
        R_UNREFERENCED_VARIABLE( ret );  /* for Release configuration */
    }
#if  R_OSPL_EVENT_OBJECT_CODE  &&  ( R_OSPL_ERROR_BREAK  ||  R_OSPL_TLS_ERROR_CODE )
    else    /* If Event Object */
    {
        osSemaphoreId  object = (osSemaphoreId) event_object;
        osStatus       es;


        es= osSemaphoreRelease( object );
        ASSERT_D( es == osOK,  R_NOOP() );
        R_UNREFERENCED_VARIABLE( es );  /* for Release configuration */
    }
#endif

fin:
    return;
}
#endif


/***********************************************************************
* Implement: R_OSPL_EVENT_Clear
************************************************************************/
#if  R_OSPL_EVENT_CODE
void  R_OSPL_EVENT_Clear( r_ospl_thread_id_t const  in_ThreadId,  bit_flags32_t const  in_ClearFlags1 )
{
    int32_t  ret;
#if  R_OSPL_EVENT_OBJECT_CODE
    r_ospl_event_object_id_t  event_object;
#endif
#if  R_OSPL_EVENT_GROUP_CODE
    r_ospl_event_group_id_t  current_event_group;
#endif

    if ( in_ThreadId == R_OSPL_THREAD_NULL )
    {
        goto fin;
    }
    ASSERT_D( R_OSPL_THREAD_GetCurrentId() != R_OSPL_THREAD_NULL,
              R_OSPL_RaiseUnrecoverable( E_NOT_THREAD ); goto fin );
    /* Error: This function can not be called from interrupt context. */
    /* Because portable with RTX */

#if  R_OSPL_EVENT_WATCH
    if ( g_DebugVar[ R_OSPL_EVENT_WATCH ] == NULL  ||
            g_DebugVar[ R_OSPL_EVENT_WATCH ] == (int_fast32_t) in_ThreadId )
    {
        R_D_AddToIntLog( 0x70C00000 + (int_fast32_t)( in_ClearFlags1 & R_OSPL_EVENT_ALL_BITS ) );
        R_D_AddToIntLog( (int_fast32_t) (uintptr_t) R_OSPL_THREAD_GetCurrentId() );
        R_D_AddToIntLog( (int_fast32_t) (uintptr_t) in_ThreadId );
    }
#endif

    GS_CHECK_TO_ENABLE_ALL_INTERRUPT();

#if  R_OSPL_EVENT_OBJECT_CODE  &&  ( R_OSPL_ERROR_BREAK  ||  R_OSPL_TLS_ERROR_CODE )
    event_object = R_OSPL_EVENT_GetEventObject_Sub( in_ThreadId, in_ClearFlags1 );
    if ( event_object == R_OSPL_EVENT_OBJECT_NULL )  /* If Event Flag */
#endif
    {
        if ( in_ClearFlags1 != 0 )    /* For avoiding error in osSignalClear */
        {
            ret = osSignalClear( (osThreadId) in_ThreadId,
                                 (int32_t)( in_ClearFlags1 & R_OSPL_EVENT_ALL_BITS ) );
            /* "& R_OSPL_EVENT_ALL_BITS" is for avoiding error in osSignalClear */
            ASSERT_D( (ret & OS_ERROR_SIGNAL) == 0,  R_NOOP() );
            R_UNREFERENCED_VARIABLE( ret );  /* for Release configuration */
        }
    }
#if  R_OSPL_EVENT_OBJECT_CODE  &&  ( R_OSPL_ERROR_BREAK  ||  R_OSPL_TLS_ERROR_CODE )
    else    /* If Event Object */
    {
        osSemaphoreId  object = (osSemaphoreId) event_object;
        int32_t        ei;


        ei= osSemaphoreWait( object, 0 );
        ASSERT_D( ei != -1,  R_NOOP() );
        R_UNREFERENCED_VARIABLE( ei );  /* for Release configuration */
    }
#endif

fin:
    return;
}
#endif


/***********************************************************************
* Implement: R_OSPL_EVENT_Get
************************************************************************/
#if  R_OSPL_EVENT_CODE
#if ( ( ! defined( osCMSIS )  ||  osCMSIS <= 0x10001 ) &&  R_OSPL_VERSION < 85 ) ||  R_OSPL_SUPPORT_EVENT_GET
bit_flags32_t  R_OSPL_EVENT_Get( r_ospl_thread_id_t const  in_ThreadId )
{
    int32_t  ret;

    if ( in_ThreadId == R_OSPL_THREAD_NULL )
    {
        ret = 0;
    }
    else
    {
        GS_CHECK_TO_ENABLE_ALL_INTERRUPT();

        ret = osSignalGet( (osThreadId) in_ThreadId );
        ASSERT_D( (ret & OS_ERROR_SIGNAL) == 0,  R_NOOP() );
    }

    return  (bit_flags32_t) ret;
}
#endif
#endif


/***********************************************************************
* Implement: R_OSPL_EVENT_Wait
************************************************************************/
#if  R_OSPL_EVENT_CODE
errnum_t  R_OSPL_EVENT_Wait( bit_flags32_t  in_WaigingFlags,  bit_flags32_t* const  out_GotFlags,
                             uint32_t const  in_Timeout_msec )
{
    errnum_t  e;
    osEvent   event;
#if  R_OSPL_EVENT_OBJECT_CODE  ||  R_OSPL_EVENT_WATCH  ||  R_OSPL_DETECT_BAD_EVENT  ||  R_OSPL_EVENT_GROUP_CODE
    r_ospl_thread_id_t const  current_thread = (r_ospl_thread_id_t) R_OSPL_THREAD_GetCurrentId();
#endif
#if  R_OSPL_EVENT_GROUP_CODE
    r_ospl_event_group_id_t  current_event_group;
#endif
#if  R_OSPL_EVENT_OBJECT_CODE
    r_ospl_event_object_id_t    event_object;
#endif
#if  R_OSPL_DETECT_BAD_EVENT
    r_ospl_event_status_t*  status = NULL;
#endif
#if  R_OSPL_DETECT_BAD_EVENT  ||  R_OSPL_EVENT_GROUP_CODE
    r_ospl_error_t*         err = NULL;
#endif
    bit_flags32_t  or_flag;
    bit_flags32_t  got_flags = 0x00000000u;


    R_STATIC_ASSERT( R_OSPL_INFINITE == GS_TO_UNSIGNED_A( osWaitForever ), "" );
    GS_CHECK_TO_ENABLE_ALL_INTERRUPT();

#if  R_OSPL_EVENT_WATCH
    if ( in_Timeout_msec > 0 )
    {
        R_D_AddToIntLog( 0x70BE0000 + (int_fast32_t) in_WaigingFlags );
        R_D_AddToIntLog( (int_fast32_t) (uintptr_t) current_thread );

        {
            static int tc;
            if ( R_D_Counter( &tc, 0, NULL ) )
            {
                R_DEBUG_BREAK();
            }
        }
    }
#endif

    ASSERT_R( R_OSPL_THREAD_GetCurrentId() != R_OSPL_THREAD_NULL,  e=E_NOT_THREAD;  goto fin );

#if  R_OSPL_EVENT_GROUP_CODE
    if ( err == NULL )
    {
        err = R_OSPL_LockCurrentThreadError_Sub( current_thread,  R_OSPL_ALLOCATE_IF_NOT );
    }
    current_event_group = err->EnabledEventGroup;
    R_OSPL_UnlockCurrentThreadError( &err );
    IF ( current_event_group == NULL )
    {
        e=E_STATE;
        goto fin;
    }
#endif

#if  R_OSPL_EVENT_OBJECT_CODE
    event_object = R_OSPL_EVENT_GetEventObject_Sub( current_thread,  R_OSPL_EVENT_OBJECT_FLAG );

    if ( event_object != R_OSPL_EVENT_OBJECT_NULL )
    {
        ASSERT_D( in_WaigingFlags == R_OSPL_ANY_FLAG  ||
                  IS_BIT_SET( in_WaigingFlags, R_OSPL_EVENT_OBJECT_FLAG ),
                  R_NOOP() );
    }
#endif

#if  R_OSPL_DETECT_BAD_EVENT
    e= R_OSPL_EVENT_GetStatus( current_thread,  &status );
    IF(e!=0)
    {
        goto fin;
    }

    if ( in_Timeout_msec != 0 )
    {
        ASSERT_R( status->AllocatedEvents != 0u,  R_OSPL_BAD_EVENT_ERROR() );
        /* Error: Any events are not allocated. */
    }
#endif


    if ( IS_BIT_NOT_SET( in_WaigingFlags,  R_OSPL_OR_FLAG ) )
    {
        or_flag = 0u;
        in_WaigingFlags &= R_OSPL_EVENT_ALL_BITS;
    }
    else
    {
        or_flag = in_WaigingFlags & R_OSPL_EVENT_ALL_BITS;
        in_WaigingFlags = 0u;  /* Waiting any bits. */

#if  R_OSPL_EVENT_OBJECT_CODE
        ASSERT_R( event_object == R_OSPL_EVENT_OBJECT_NULL,  e=E_OTHERS; goto fin );
        /* Error: It is not possible to specify both event object and R_OSPL_OR_FLAG.*/
        /* Error: It is not possible to specify both event object and R_OSPL_OR_FLAG. */
#endif
    }


#if  R_OSPL_EVENT_OBJECT_CODE
    if ( event_object == R_OSPL_EVENT_OBJECT_NULL )  /* If Event Flag */
#endif
    {
        event = osSignalWait( (int32_t)( in_WaigingFlags ), in_Timeout_msec );


        if ( (event.status == osOK) || (event.status == osEventTimeout) )
        {
            got_flags = R_OSPL_TIMEOUT;
            IF ( event.status == osEventTimeout )
            {
                e=E_TIME_OUT;
                goto fin;
            }
        }
        else
        {
            ASSERT_R( event.status == osEventSignal,  e=E_OTHERS;  goto fin );

            got_flags = (bit_flags32_t) event.value.signals;
        }


        if ( or_flag != 0u )
        {
            bit_flags32_t  ignored_flags = got_flags & ~ or_flag;
#if  ! ( R_OSPL_EVENT_OBJECT_CODE  ||  R_OSPL_EVENT_WATCH  ||  R_OSPL_DETECT_BAD_EVENT  ||  R_OSPL_EVENT_GROUP_CODE )
            r_ospl_thread_id_t const  current_thread = (r_ospl_thread_id_t) R_OSPL_THREAD_GetCurrentId();
#endif

            int32_t  ret = osSignalSet( current_thread,  (int32_t)( ignored_flags ) );
            ASSERT_D( (ret & OS_ERROR_SIGNAL) == 0,  R_NOOP() );
            R_UNREFERENCED_VARIABLE( ret );  /* for Release configuration */

            got_flags &= or_flag;
        }
    }

#if  R_OSPL_EVENT_OBJECT_CODE
    else    /* If Event Object */
    {
        osSemaphoreId  object = (osSemaphoreId) event_object;
        int32_t        ei;


        ei= osSemaphoreWait( object,  in_Timeout_msec );
        IF ( ei == -1 )
        {
            e=E_OTHERS;
            goto fin;
        }


        if ( ei == 0 )
        {
            got_flags = R_OSPL_TIMEOUT;
        }
        else
        {
            got_flags = R_OSPL_EVENT_OBJECT_FLAG;
        }
    }
#endif


#if  R_OSPL_DETECT_BAD_EVENT  ||  R_OSPL_EVENT_GROUP_CODE
    if ( err == NULL )
    {
        err = R_OSPL_LockCurrentThreadError_Sub( current_thread,  R_OSPL_ALLOCATE_IF_NOT );
    }
#endif

#if  R_OSPL_DETECT_BAD_EVENT
    {
        uint32_t  got_flags_2 = got_flags & ( R_OSPL_EVENT_OBJECT_FLAG | R_OSPL_EVENT_ALL_BITS );

        ASSERT_RC( IS_ALL_BITS_SET( status->AllocatedEvents,  got_flags_2 ),  err,  R_OSPL_BAD_EVENT_ERROR() );

        err->EventStatus.UnexpectedEvents |= got_flags_2;
    }
#endif

#if  R_OSPL_EVENT_GROUP_CODE
    IF_C ( in_Timeout_msec > 0  &&  got_flags == R_OSPL_TIMEOUT,  err )
    {
        e=E_TIME_OUT;
        goto fin;
    }
#endif

    e=0;
fin:
#if  R_OSPL_DETECT_BAD_EVENT  ||  R_OSPL_EVENT_GROUP_CODE
    R_OSPL_UnlockCurrentThreadError( &err );
#endif
#if  R_OSPL_EVENT_WATCH
    if ( in_Timeout_msec > 0 )
    {
        R_D_AddToIntLog( 0x70AF0000 );
        R_D_AddToIntLog( (int_fast32_t) (uintptr_t) current_thread );
#if  R_OSPL_EVENT_GROUP_CODE
        R_D_AddToIntLog( (int_fast32_t) (uintptr_t) current_event_group );
#endif
        R_D_AddToIntLog( (int_fast32_t) got_flags );
    }
#endif
    if ( out_GotFlags != NULL )
    {
        *out_GotFlags = got_flags;
    }
    return  e;
}
#endif


/***********************************************************************
* Implement: R_OSPL_MEMORY_Flush
************************************************************************/
#if  R_OSPL_MEMORY_CONTROL_CODE
void  R_OSPL_MEMORY_Flush( r_ospl_flush_t const  in_FlushType )
{
    if ( in_FlushType == R_OSPL_FLUSH_WRITEBACK_INVALIDATE )
    {
#if 0
        printf( "L1Flush\n" );
#endif


#if  defined( R_OSPL_FOR_RZ_A1_CACHE )
        L1_D_CacheWritebackFlushAll();
#elif  defined( R_OSPL_FOR_RTX )
#if  defined( __ICCARM__ )
        __v7_all_cache(2);
#else
        __v7_clean_inv_dcache_all();
#endif
#else
#error
#endif


    }
    else if ( in_FlushType == R_OSPL_FLUSH_WRITEBACK_INVALIDATE_2ND )
    {
#if 0
        printf( "PL310Flush\n" );
#endif

        PL310_CleanInvAllByWay();
    }
    else
    {
        ASSERT_D( false,  R_NOOP() );
    }
}
#endif


/***********************************************************************
* Function: R_OSPL_Is1bitOnly_Fast32_Sub
*    R_OSPL_Is1bitOnly_Fast32_Sub
************************************************************************/
#if  R_OSPL_BASIC_TYPES_CODE
#ifndef  R_OSPL_NDEBUG
static bool_t  R_OSPL_Is1bitOnly_Fast32_Sub( uint_fast32_t in_Value )
{
    if ( (in_Value & 0x0000FFFFu) == 0 )
    {
        in_Value >>= 16;
    }
    if ( (in_Value & 0x000000FFu) == 0 )
    {
        in_Value >>=  8;
    }
    if ( (in_Value & 0x0000000Fu) == 0 )
    {
        in_Value >>=  4;
    }
    if ( (in_Value & 0x00000003u) == 0 )
    {
        in_Value >>=  2;
    }
    if ( (in_Value & 0x00000001u) == 0 )
    {
        in_Value >>=  1;
    }
    return  ( in_Value == 1 );
}
#endif
#endif


/***********************************************************************
* Implement: R_OSPL_MEMORY_RangeFlush
************************************************************************/
#if  R_OSPL_MEMORY_CONTROL_CODE
errnum_t  R_OSPL_MEMORY_RangeFlush( r_ospl_flush_t const  in_FlushType,
                                    const void* const  in_StartAddress,  size_t const  in_Length )
{
    errnum_t   e;
    size_t     cache_line_size;
    size_t     cache_line_mask;
    uintptr_t  start;
    uintptr_t  over;

    ASSERT_R( in_FlushType == R_OSPL_FLUSH_INVALIDATE,  e=E_BAD_COMMAND_ID; goto fin );

    cache_line_size = R_OSPL_MEMORY_GetCacheLineSize();
    cache_line_mask = cache_line_size - 1u;
    ASSERT_D( R_OSPL_Is1bitOnly_Fast32_Sub( cache_line_size ), e=E_OTHERS; goto fin );

    /* ->MISRA 11.3 */ /* ->SEC R2.7.1 */
    ASSERT_R( ( (uintptr_t) in_StartAddress & cache_line_mask ) == 0u, e=E_OTHERS; goto fin );
    ASSERT_R( ( in_Length & cache_line_mask ) == 0u, e=E_OTHERS; goto fin );

    start = (uintptr_t) in_StartAddress;
    over  = ( (uintptr_t) in_StartAddress + in_Length ) - 1u;
    /* <-MISRA 11.3 */ /* <-SEC R2.7.1 */

    R_OSPL_MEMORY_RangeFlush_Sub( start, over, cache_line_size );

    e=0;
fin:
    return  e;
}
#endif


/***********************************************************************
* Implement: R_OSPL_MEMORY_GetSpecification
************************************************************************/
#if  R_OSPL_MEMORY_CONTROL_CODE
void  R_OSPL_MEMORY_GetSpecification( r_ospl_memory_spec_t* const  out_MemorySpec )
{
    IF_DQ( out_MemorySpec == NULL )
    {
        goto fin;
    }

    out_MemorySpec->CacheLineSize = R_OSPL_MEMORY_GetCacheLineSize();

fin:
    return;
}
#endif


/***********************************************************************
* Implement: R_OSPL_Delay
************************************************************************/
#if  R_OSPL_DELAY_CODE
errnum_t  R_OSPL_Delay( uint32_t const  in_DelayTime_msec )
{
    errnum_t  e;
    osStatus  rs;
    bool_t const    is_overflow = ( in_DelayTime_msec > R_OSPL_MAX_TIME_OUT );
    uint32_t const  delay_parameter = in_DelayTime_msec + 1u;

    ASSERT_D( ! is_overflow, R_NOOP() );
    /* RTX 5.16: If delay_parameter = 100000, "osDelay" waits 65534 */
    GS_CHECK_TO_ENABLE_ALL_INTERRUPT();

    rs= osDelay( delay_parameter );
    IF ( rs == osErrorISR )
    {
        e=E_NOT_THREAD;
        R_OSPL_RaiseUnrecoverable( e );
        goto fin;
    }
    IF ( IS( is_overflow ) )
    {
        e=E_TIME_OUT;
        goto fin;
    }

    IF (
        (rs != osOK) &&           /* for delay_parameter == 0 */
        (rs != osEventTimeout) )  /* for delay_parameter != 0 */
    {
        e=E_OTHERS;
        goto fin;
    }

    e=0;
fin:
    return  e;
}
#endif


/***********************************************************************
* Implement: R_OSPL_QUEUE_Create
************************************************************************/
#if  R_OSPL_QUEUE_CODE
errnum_t  R_OSPL_QUEUE_Create( r_ospl_queue_id_t* out_self, r_ospl_queue_def_t* in_QueueDefine )
{
    errnum_t  e;
    r_ospl_queue_def_t*  self = in_QueueDefine;

    GS_CHECK_TO_ENABLE_ALL_INTERRUPT();

    self->MailQId = osMailCreate( (osMailQDef_t*) in_QueueDefine->MailQDef, NULL );
    IF ( self->MailQId == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }
    self->PublicStatus.UsedCount = 0;

    *out_self = self;

    e=0;
fin:
    return  e;
}
#endif


/***********************************************************************
* Implement: R_OSPL_QUEUE_GetStatus
************************************************************************/
#if  R_OSPL_QUEUE_CODE
errnum_t  R_OSPL_QUEUE_GetStatus( r_ospl_queue_id_t  self,  const r_ospl_queue_status_t** out_Status )
{
    *out_Status = &self->PublicStatus;
    return  0;
}
#endif


/***********************************************************************
* Implement: R_OSPL_QUEUE_Allocate
************************************************************************/
#if  R_OSPL_QUEUE_CODE
errnum_t  R_OSPL_QUEUE_Allocate( r_ospl_queue_id_t  self,  void* out_Address,  uint32_t in_Timeout_msec )
{
    errnum_t  e;
    void*     address;
    bool_t    was_all_enabled = false;

    GS_CHECK_TO_ENABLE_ALL_INTERRUPT();

    address = osMailAlloc( self->MailQId,  in_Timeout_msec );
    *(void**) out_Address = address;
    IF ( address == NULL  &&  in_Timeout_msec > 0 )
    {
        if ( R_OSPL_THREAD_GetCurrentId() == R_OSPL_THREAD_NULL )
        {
            e=E_NOT_THREAD;
        }
        else
        {
            e=E_TIME_OUT;
        }
        goto fin;
    }

    if ( address != NULL )
    {
        was_all_enabled = R_OSPL_DisableAllInterrupt();
        self->PublicStatus.UsedCount += 1;
    }

    e=0;
fin:
    if ( was_all_enabled )
    {
        R_OSPL_EnableAllInterrupt();
    }
    return  e;
}
#endif


/***********************************************************************
* Implement: R_OSPL_QUEUE_Put
************************************************************************/
#if  R_OSPL_QUEUE_CODE
errnum_t  R_OSPL_QUEUE_Put( r_ospl_queue_id_t  self,  void* in_Address )
{
    errnum_t  e;
    osStatus  status;

    GS_CHECK_TO_ENABLE_ALL_INTERRUPT();

    status = osMailPut( self->MailQId,  in_Address );
    IF ( status != osOK )
    {
        e=E_OTHERS;
        goto fin;
    }

    e=0;
fin:
    return  e;
}
#endif


/***********************************************************************
* Implement: R_OSPL_QUEUE_Get
************************************************************************/
#if  R_OSPL_QUEUE_CODE
errnum_t  R_OSPL_QUEUE_Get( r_ospl_queue_id_t  self,  void* out_Address,  uint32_t in_Timeout_msec )
{
    errnum_t  e;
    osEvent   event;

    GS_CHECK_TO_ENABLE_ALL_INTERRUPT();

    event = osMailGet( self->MailQId,  in_Timeout_msec );

    *(void**) out_Address = NULL;

    if ( event.status != osOK )
    {
        IF ( event.status != osEventMail )
        {
            if ( event.status == osEventTimeout )
            {
                e = E_TIME_OUT;
            }
            else if ( event.status == osErrorParameter )
            {
                if ( R_OSPL_THREAD_GetCurrentId() == R_OSPL_THREAD_NULL )
                {
                    e = E_NOT_THREAD;
                }
                else
                {
                    e = E_OTHERS;
                }
            }
            else
            {
                e = E_OTHERS;
            }
            goto fin;
        }
        else
        {
            *(void**) out_Address = event.value.p;
        }
    }

    e=0;
fin:
    return  e;
}
#endif


/***********************************************************************
* Implement: R_OSPL_QUEUE_Free
************************************************************************/
#if  R_OSPL_QUEUE_CODE
errnum_t  R_OSPL_QUEUE_Free( r_ospl_queue_id_t  self,  void* in_Address )
{
    errnum_t  e;
    osStatus  status;
    bool_t    was_all_enabled = false;

    GS_CHECK_TO_ENABLE_ALL_INTERRUPT();

    status = osMailFree( self->MailQId,  in_Address );
    IF ( status != osOK )
    {
        e=E_OTHERS;
        goto fin;
    }

    was_all_enabled = R_OSPL_DisableAllInterrupt();
    self->PublicStatus.UsedCount -= 1;

    e=0;
fin:
    if ( was_all_enabled )
    {
        R_OSPL_EnableAllInterrupt();
    }
    return  e;
}
#endif


/***********************************************************************
* Variable: gs_QueueAsSingletonLockConfig
************************************************************************/
#if  R_OSPL_QUEUE_CODE
static R_OSPL_QUEUE_DEF( gs_QueueAsSingletonLockConfig,  1,  int_fast32_t );
#endif


/***********************************************************************
* Implement: R_OSPL_GetQueueAsSingletonLock
************************************************************************/
#if  R_OSPL_QUEUE_CODE
r_ospl_queue_id_t  R_OSPL_GetQueueAsSingletonLock(void)
{
    static r_ospl_queue_id_t  gs_QueueAsSingletonLock = R_OSPL_QUEUE_NULL;

    if ( gs_QueueAsSingletonLock == R_OSPL_QUEUE_NULL )
    {

        ASSERT_D( R_OSPL_THREAD_GetCurrentId() != R_OSPL_THREAD_NULL,  R_NOOP() );
        R_OSPL_Start_T_Lock();
        /* This area must be thread safe. */
        if ( gs_QueueAsSingletonLock == R_OSPL_QUEUE_NULL )
        {

            R_OSPL_QUEUE_Create( &gs_QueueAsSingletonLock,
                                 R_OSPL_QUEUE( gs_QueueAsSingletonLockConfig ) );
        }

        R_OSPL_End_T_Lock();
    }
    return  gs_QueueAsSingletonLock;
}
#endif


/***********************************************************************
* Implement: R_OSPL_Start_T_Lock
*    The function callbacked from OSPL internal, when T-Lock started
*
* Arguments:
*    None
*
* Return Value:
*    Error Code. 0=No Error.
*
* InternalDescription:
*    See. <R_OSPL_LockCurrentThreadError_Sub>.
************************************************************************/
#if  R_OSPL_INTERNAL_T_LOCK_CODE
#if  R_OSPL_IS_PREEMPTION
errnum_t  R_OSPL_Start_T_Lock(void)
{
    errnum_t  e;
    osStatus  es;


#if  R_OSPL_T_LOCK_WATCH
    R_D_AddToIntLog( (int_fast32_t) 0xDDDD7101 );
    R_D_AddToIntLog( (int_fast32_t)(uintptr_t) R_OSPL_THREAD_GetCurrentId() );
#endif
    GS_CHECK_TO_ENABLE_ALL_INTERRUPT();

    if ( R_OSPL_THREAD_GetCurrentId() == R_OSPL_THREAD_NULL )
    {
        R_OSPL_RaiseUnrecoverable( E_NOT_THREAD );
    }

    if ( gs_ospl_mutex == NULL )
    {

        gs_ospl_mutex = osMutexCreate( osMutex( gs_ospl_mutex ) );
        if ( gs_ospl_mutex == NULL )
        {
            e=E_OTHERS;
            goto fin;
        }
    }

    es= osMutexWait( gs_ospl_mutex,  GS_TO_UNSIGNED_A( osWaitForever ) );
    if ( es == osErrorISR )
    {
        es = osOK;
    }
    if ( es != osOK )
    {
        e=E_OTHERS;
        goto fin;
    }

    e=0;
fin:
#if  R_OSPL_T_LOCK_WATCH
    R_D_AddToIntLog( (int_fast32_t) 0xDDDD7102 );
    R_D_AddToIntLog( (int_fast32_t) (uintptr_t) R_OSPL_THREAD_GetCurrentId() );
#endif
    return  e;
}
#endif
#endif


/***********************************************************************
* Implement: R_OSPL_End_T_Lock
*    The function callbacked from OSPL internal, when T-Lock ended
*
* Arguments:
*    None
*
* Return Value:
*    None
*
* InternalDescription:
*    See. <R_OSPL_UnlockCurrentThreadError>.
************************************************************************/
#if  R_OSPL_INTERNAL_T_LOCK_CODE
#if  R_OSPL_IS_PREEMPTION
void  R_OSPL_End_T_Lock(void)
{
#if  R_OSPL_T_LOCK_WATCH
    R_D_AddToIntLog( (int_fast32_t) 0xDDDD710F );
    R_D_AddToIntLog( (int_fast32_t)(uintptr_t) R_OSPL_THREAD_GetCurrentId() );
#endif
    GS_CHECK_TO_ENABLE_ALL_INTERRUPT();

    if ( R_OSPL_THREAD_GetCurrentId() == R_OSPL_THREAD_NULL )
    {
        R_OSPL_RaiseUnrecoverable( E_NOT_THREAD );
    }

    if ( gs_ospl_mutex != NULL )
    {
        osStatus  rs;

        rs= osMutexRelease( gs_ospl_mutex );
        if ( rs == osErrorISR )
        {
            rs = osOK;
        }
        ASSERT_R( rs == osOK,  R_OSPL_RaiseUnrecoverable( E_OTHERS ) );
    }
}
#endif
#endif


#endif  /* R_OSPL_VERSION */
#endif  /* osCMSIS_RTX */
