;/*******************************************************************************
;* DISCLAIMER
;* This software is supplied by Renesas Electronics Corporation and is only
;* intended for use with Renesas products. No other uses are authorized. This
;* software is owned by Renesas Electronics Corporation and is protected under
;* all applicable laws, including copyright laws.
;* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
;* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
;* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
;* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
;* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
;* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
;* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
;* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
;* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
;* Renesas reserves the right, without notice, to make changes to this software
;* and to discontinue the availability of this software. By using this software,
;* you agree to the additional terms and conditions found by accessing the
;* following link:
;* http://www.renesas.com/disclaimer
;* Copyright (C) 2012 - 2017 Renesas Electronics Corporation. All rights reserved.
;*******************************************************************************/
;/*******************************************************************************
;* File: r_ospl_asm_armcc.S
;*    OS Porting Layer API for ARM assembler
;*
;* - $Version: $ (=R_OSPL_VERSION)
;* - $ModuleRevision: {Masters}\OSPL\ALL_Driver_parent\27_RStyled $
;* - $Rev: $
;* - $Date: 2017-05-09T11:22:50+09:00 $
;******************************************************************************/


#ifdef  __CC_ARM

	AREA    |.text|, CODE, READONLY

		;//    ALL: If C language preprocessor was not supported, please except
		;//         to assemble this file by changing file extension.
		;//    ARM: --cpreproc option of analyzes C language preprocessor.
		;//    GNU: -x assembler-with-cpp option and .S (upper case) file extension
		;//         analyzes C language preprocessor.
		;//         If GNU did not suppported .S (upper case), please change
		;//         extention of files in order to enable/disable compile.


;/******************************************************************************
;* Implement: R_OSPL_MEMORY_GetCacheLineSize
;*
;* Arguments:
;*    None
;*
;* Return Value:
;*    None
;******************************************************************************/

	EXPORT  R_OSPL_MEMORY_GetCacheLineSize

R_OSPL_MEMORY_GetCacheLineSize    FUNCTION

	ARM

	MRC     p15, 1, r0, c0, c0, 0   ;// r0 = p15(CCSIDR);    // CCSIDR: Cache Size ID Registers
	AND     r0, r0, #7              ;// r0 = r0 & 7;         // CCSIDR.LineSize: log2(cache_line_byte)-4
	ADD     r0, r0, #4              ;// r0 += 4;             //
	MOV     r1, #1                  ;// r0 = 1 << r0;        // [r0] cache_line_byte
	MOV     r0, r1, LSL r0          ;    :

	BX     lr

	ENDFUNC


;/******************************************************************************
;* Implement: R_OSPL_MEMORY_RangeFlush_Sub
;*
;* Arguments:
;*    r0 - Start address
;*    r1 - Over address (Last + 1)
;*    r2 - Cache line size
;*
;* Return Value:
;*    None
;******************************************************************************/

	EXPORT  R_OSPL_MEMORY_RangeFlush_Sub

R_OSPL_MEMORY_RangeFlush_Sub    FUNCTION

	ARM

Loop
	CMP     r0, r1                  ;// for ( r0 = r0;  r0 < r1;  r0 += r2 ) {
	BGE     Fin2                    ;//  :
	MCR     p15, 0, r0, c7, c6, 1   ;//     p15(DCIMVAC) = r0;  // Invalidate
	ADD     r0, r0, r2              ;// }
	B       Loop                    ;//  :
Fin2
	DSB
	BX     lr

	ENDFUNC


;/******************************************************************************
;* Implement: R_OSPL_GET_STACK_POINTER
;*
;* Arguments:
;*    None
;*
;* Return Value:
;*    r0 - Value of stack pointer
;******************************************************************************/

	EXPORT  R_OSPL_GET_STACK_POINTER

R_OSPL_GET_STACK_POINTER    FUNCTION

	ARM

	MOV    r0, sp
	BX     lr

	ENDFUNC


#endif
#ifdef  __IAR_SYSTEMS_ASM__
	section    .text : CODE    ;* Suppress warning 17
#endif
#ifdef __GNUC__
#define  END  .END  ;* Change END directive from for ARM and IAR to for GNU.
#endif
	END
