/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2015 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/***********************************************************************
* File: r_ospl_os_less.c
*    OS Porting Layer API for RZ/A1 OS less.
*
* - $Version: $ (=R_OSPL_VERSION)
* - $ModuleRevision: {Masters}\OSPL\ALL_DriverForRelease\24_RStyled $
* - $Rev: $
* - $Date: 2017-08-08T20:03:47+09:00 $
************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include  "r_ospl_os_less.h"
#include  "r_ospl_os_less_private.h"
#include  "r_ospl_private.h"
#include  "iodefine.h"
#if  defined( R_OSPL_FOR_RZ_A1_OS_LESS )  &&  R_OSPL_FOR_RZ_A1_OS_LESS >= 101
#include  "cache.h"
#endif
#if  defined( USE_LIST_RZA1H_OS_LESS )
#include  "devdrv_intc.h"  /* INTC */
#elif  defined( USE_LIST_RZA1H_OS_LESS_RSK )
#include  "intc.h"
#endif
#ifndef R_OSPL_NDEBUG
#include  <stdio.h>
#include  <string.h>
#endif


#if  ! R_OSPL_IS_PREEMPTION

/******************************************************************************
Typedef definitions
******************************************************************************/

/******************************************************************************
Macro definitions
******************************************************************************/

/***********************************************************************
* Define: R_OSPL_WAIT_OWNER_DEBUG
*    Debug tool. 0 or 1
************************************************************************/
#define  R_OSPL_WAIT_OWNER_DEBUG  0


/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/

/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/

/******************************************************************************
Private global variables and functions
******************************************************************************/

static errnum_t  R_OSPL_TIMEOUT_Start_Sub( uint32_t const  Timeout_msec );
static bool_t  R_OSPL_TIMEOUT_GetIsLoop_Sub( bool_t* const  out_IsTimeout,
        bool_t* const  out_IsOverMaxTimeout );

STATIC_INLINE uint32_t  R_OSPL_GetFTimerMaxCountToTimeoutMaxCount(
    const r_ospl_ftimer_spec_t* const  in_TimerSpec );

errnum_t  R_OSPL_BLOCK_PrintList( r_ospl_block_t* in_ListFirst,  char_t* in_Name,
                                  r_ospl_block_t*  in_BlockArray,  int_fast32_t  in_MaxCount );


/***********************************************************************
* Variable: gs_main_thread
*    Main thread object as <r_ospl_thread_def_t>
************************************************************************/
static r_ospl_thread_def_t  gs_main_thread =
{
    0,     /* .Flags */
    NULL,  /* .Argument */
    0,     /* .Status */
    0,     /* .Timeout_msec */
    0      /* .TimeoutCount */
};


/***********************************************************************
* Variable: gs_master
*    Instance of <r_ospl_master_t>
************************************************************************/
static r_ospl_master_t  gs_master =
{
    &gs_main_thread,            /* .CurrentThread */
    &( R_OSPL_OnIdleDefault ),  /* .IdleCallback */ /* MISRA 16.9 */
    false,                      /* .IsFreeRunTimerSpec */
    {0},                        /* .FreeRunTimerSpec */
    0                           /* .MaxOneTimeoutTime */
};


/***********************************************************************
* Implement: R_OSPL_GetPrivateContext
************************************************************************/
r_ospl_master_t*  R_OSPL_GetPrivateContext(void)
{
    return  &gs_master;
}


/***********************************************************************
* Implement: R_OSPL_InitializeIfNot
************************************************************************/
errnum_t  R_OSPL_InitializeIfNot(void)
{
    return  0;
}


/***********************************************************************
* Implement: R_OSPL_Initialize
************************************************************************/
errnum_t  R_OSPL_Initialize( const void* const  in_NullConfig )
{
    R_UNREFERENCED_VARIABLE( in_NullConfig );
    return  0;
}


/***********************************************************************
* Function: R_OSPL_GetFTimerMaxCountToTimeoutMaxCount
*    R_OSPL_GetFTimerMaxCountToTimeoutMaxCount
************************************************************************/
STATIC_INLINE uint32_t  R_OSPL_GetFTimerMaxCountToTimeoutMaxCount(
    const r_ospl_ftimer_spec_t* const  in_TimerSpec )
{
    uint32_t  time_out_max_count;
    enum { num_2 = 2 };

    IF_DQ( in_TimerSpec == NULL )
    {
        time_out_max_count = 0;
    }
    else
    {
        time_out_max_count = in_TimerSpec->MaxCount / num_2;
    }
    return  time_out_max_count;
}


/***********************************************************************
* Implement: R_OSPL_THREAD_Create
************************************************************************/
#if  R_OSPL_THREAD_CODE
errnum_t  R_OSPL_THREAD_Create( r_ospl_thread_def_t* const  in_ThreadDef, void* const  in_Argument,
                                r_ospl_thread_id_t* const  out_ThreadId )
{
    errnum_t  e;
    r_ospl_thread_def_t* const  self = in_ThreadDef;

    IF_DQ( out_ThreadId == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }
    IF_DQ( self == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }

    *out_ThreadId = self;

    ASSERT_R( IS_BIT_NOT_SET( self->Status, R_OSPL_IS_THREAD_USED ),  e=E_STATE; goto fin );

    self->Status =
        R_OSPL_IS_THREAD_USED |
        /* R_OSPL_IS_EXIT_IF_WAIT | */
        /* R_OSPL_IS_WAITING | */
        0;
    self->Argument = in_Argument;
    self->Timeout_msec = R_OSPL_MAX_TIME_OUT;

    self->Flags = 0;

    e=0;
fin:
    return  e;
}
#endif


/***********************************************************************
* Implement: R_OSPL_THREAD_Destroy
************************************************************************/
#if  R_OSPL_THREAD_CODE
errnum_t  R_OSPL_THREAD_Destroy( r_ospl_thread_id_t* const  in_out_ThreadId )
{
    errnum_t  e;

    r_ospl_thread_def_t*  self;

    IF_DQ( in_out_ThreadId == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }

    self = *in_out_ThreadId;
    if ( self != R_OSPL_THREAD_NULL )
    {
        self->Flags = 0u;
        self->Argument = NULL;
        self->Status = 0;
        self->Timeout_msec = R_OSPL_MAX_TIME_OUT;
        self->TimeoutCount = 0;
        self->Delegate = NULL;
        *in_out_ThreadId = R_OSPL_THREAD_NULL;
    }

    e=0;
fin:
    return  e;
}
#endif



/***********************************************************************
* Implement: R_OSPL_THREAD_GetArgument
************************************************************************/
#if  R_OSPL_THREAD_CODE
void*  R_OSPL_THREAD_GetArgument(void)
{
    void*  return_value;

    IF_DQ( gs_master.CurrentThread == NULL )
    {
        return_value = NULL;
        goto fin;
    }

    return_value = gs_master.CurrentThread->Argument;

fin:
    return  return_value;
}
#endif


/***********************************************************************
* Implement: R_OSPL_THREAD_SetCurrentId
************************************************************************/
#if  R_OSPL_THREAD_CODE
void  R_OSPL_THREAD_SetCurrentId( r_ospl_thread_id_t const  in_ThreadId )
{
    gs_master.CurrentThread = (r_ospl_thread_def_t*) in_ThreadId;
}
#endif


/***********************************************************************
* Implement: R_OSPL_THREAD_GetCurrentId
************************************************************************/
#if  R_OSPL_THREAD_CODE
r_ospl_thread_id_t  R_OSPL_THREAD_GetCurrentId(void)
{
    return  (r_ospl_thread_id_t) gs_master.CurrentThread;
}
#endif


/***********************************************************************
* Implement: R_OSPL_THREAD_GetMainId
************************************************************************/
#if  R_OSPL_THREAD_CODE
r_ospl_thread_id_t  R_OSPL_THREAD_GetMainId(void)
{
    return  (r_ospl_thread_id_t) &gs_main_thread;
}
#endif


/***********************************************************************
* Implement: R_OSPL_THREAD_SetOnWait
************************************************************************/
#if  R_OSPL_THREAD_CODE
errnum_t  R_OSPL_THREAD_SetOnWait( r_ospl_wait_t const  in_OnWait )
{
    errnum_t  e;
    r_ospl_thread_def_t* const  self = gs_master.CurrentThread;

    IF ( self == NULL )
    {
        e=E_STATE;
        goto fin;
    }

    switch ( in_OnWait )
    {
    case R_OSPL_WAIT_POLLING:
        self->Status &= ~ (bit_flags_fast32_t) R_OSPL_IS_EXIT_IF_WAIT;
        break;
    default:
        IF_DS( in_OnWait != R_OSPL_WAIT_PM_THREAD )
        {
            e=E_OTHERS;
            goto fin;
        }

        self->Status |= R_OSPL_IS_EXIT_IF_WAIT;
        break;
    }

    e=0;
fin:
    return  e;
}
#endif


/***********************************************************************
* Implement: R_OSPL_THREAD_GetOnWait
************************************************************************/
#if  R_OSPL_THREAD_CODE
r_ospl_wait_t  R_OSPL_THREAD_GetOnWait(void)
{
    r_ospl_wait_t  return_value;
    r_ospl_thread_def_t* const  self = gs_master.CurrentThread;

    if ( self == NULL )
    {
        return_value = R_OSPL_WAIT_POLLING;
    }
    else if ( IS_BIT_NOT_SET( self->Status, R_OSPL_IS_EXIT_IF_WAIT ) )
    {
        return_value = R_OSPL_WAIT_POLLING;
    }
    else
    {
        return_value = R_OSPL_WAIT_PM_THREAD;
    }

    return  return_value;
}
#endif


/***********************************************************************
* Implement: R_OSPL_THREAD_GetIsWaiting
************************************************************************/
#if  R_OSPL_THREAD_CODE
bool_t  R_OSPL_THREAD_GetIsWaiting(void)
{
    bool_t  is_waiting;
    r_ospl_thread_def_t* const  self = gs_master.CurrentThread;

    ASSERT_D( self != NULL, R_NOOP() );

    if ( self == NULL )
    {
        is_waiting = false;
    }
    else
    {
        is_waiting = IS_ANY_BITS_SET( self->Status, R_OSPL_IS_WAITING | R_OSPL_IS_WAITING_BY_0 );
    }
    return  is_waiting;
}
#endif


/***********************************************************************
* Implement: R_OSPL_THREAD_ExitWaiting
************************************************************************/
#if  R_OSPL_THREAD_CODE
void  R_OSPL_THREAD_ExitWaiting(void)
{
    r_ospl_thread_def_t* const  self = gs_master.CurrentThread;

    ASSERT_D( self != NULL, R_NOOP() );

    if ( self != NULL )
    {
        self->Status &= ~ (bit_flags_fast32_t) ( R_OSPL_IS_WAITING | R_OSPL_IS_WAITING_BY_0 );
    }
}
#endif


/***********************************************************************
* Implement: R_OSPL_THREAD_SetDelegate
************************************************************************/
#if  R_OSPL_THREAD_CODE
void  R_OSPL_THREAD_SetDelegate( r_ospl_thread_id_t const  in_ThreadId,  void*  in_Delegate )
{
    r_ospl_thread_def_t* const  self = (r_ospl_thread_def_t*) in_ThreadId;

    self->Delegate = in_Delegate;
}
#endif


/***********************************************************************
* Implement: R_OSPL_THREAD_GetDelegate
************************************************************************/
#if  R_OSPL_THREAD_CODE
void*  R_OSPL_THREAD_GetDelegate( r_ospl_thread_id_t const  in_ThreadId )
{
    r_ospl_thread_def_t* const  self = (r_ospl_thread_def_t*) in_ThreadId;

    return  self->Delegate;
}
#endif


/***********************************************************************
* Implement: R_OSPL_EVENT_Set
************************************************************************/
void  R_OSPL_EVENT_Set( r_ospl_thread_id_t const  in_ThreadId,  r_ospl_event_flags_t const  in_SetFlags )
{
    bool_t                      was_all_enabled = false;  /* QAC 3197 */
    r_ospl_thread_def_t* const  target_thread = (r_ospl_thread_def_t*) in_ThreadId;
#if  R_OSPL_EVENT_OBJECT_CODE
    r_ospl_event_object_id_t    event_object;
#endif

    if ( target_thread == R_OSPL_THREAD_NULL )
    {
        goto fin;
    }

    was_all_enabled = R_OSPL_DisableAllInterrupt();

#if  R_OSPL_EVENT_WATCH
    if ( g_DebugVar[ R_OSPL_EVENT_WATCH ] == NULL  ||
            g_DebugVar[ R_OSPL_EVENT_WATCH ] == (int_fast32_t) in_ThreadId )
    {
        R_D_AddToIntLog( 0x70200000 + ( (int_fast32_t)( in_SetFlags & R_OSPL_EVENT_ALL_BITS ) ) );
        R_D_AddToIntLog( (int_fast32_t) (uintptr_t) R_OSPL_THREAD_GetCurrentId() );
        R_D_AddToIntLog( (int_fast32_t) (uintptr_t) in_ThreadId );
    }
#endif
#if  R_OSPL_DETECT_BAD_EVENT
    R_OSPL_EVENT_SetForDebug_Sub( in_ThreadId, in_SetFlags );
#endif

#if  R_OSPL_EVENT_OBJECT_CODE  &&  ( R_OSPL_ERROR_BREAK  ||  R_OSPL_TLS_ERROR_CODE )
    event_object = R_OSPL_EVENT_GetEventObject_Sub( in_ThreadId, in_SetFlags );
    if ( event_object == R_OSPL_EVENT_OBJECT_NULL )  /* If Event Flag */
#endif
    {

        target_thread->Flags |= ( in_SetFlags & R_OSPL_EVENT_ALL_BITS );

    }
#if  R_OSPL_EVENT_OBJECT_CODE  &&  ( R_OSPL_ERROR_BREAK  ||  R_OSPL_TLS_ERROR_CODE )
    else    /* If Event Object */
    {
        r_ospl_event_object_def_t*  object = (r_ospl_event_object_def_t*) event_object;


        object->IsSignaled = true;
    }
#endif


fin:
    if ( IS( was_all_enabled ) )
    {
        R_OSPL_EnableAllInterrupt();
    }
    return;
}


/***********************************************************************
* Implement: R_OSPL_EVENT_Clear
************************************************************************/
void  R_OSPL_EVENT_Clear( r_ospl_thread_id_t const  in_ThreadId,  r_ospl_event_flags_t const  in_ClearFlags1 )
{
    bool_t                      was_all_enabled = false;  /* QAC 3197 */
    r_ospl_thread_def_t* const  target_thread = (r_ospl_thread_def_t*) in_ThreadId;
#if  R_OSPL_EVENT_OBJECT_CODE
    r_ospl_event_object_id_t    event_object;
#endif

    if ( in_ThreadId == R_OSPL_THREAD_NULL )
    {
        goto fin;
    }
    ASSERT_D( R_OSPL_THREAD_GetCurrentId() != R_OSPL_THREAD_NULL,
              R_OSPL_RaiseUnrecoverable( E_NOT_THREAD ); goto fin );
    /* Error: This function can not be called from interrupt context. */
    /* Because portable with RTX */

    was_all_enabled = R_OSPL_DisableAllInterrupt();

#if  R_OSPL_EVENT_WATCH
    if ( g_DebugVar[ R_OSPL_EVENT_WATCH ] == NULL  ||
            g_DebugVar[ R_OSPL_EVENT_WATCH ] == (int_fast32_t) in_ThreadId )
    {
        R_D_AddToIntLog( 0x70C00000 + ( (int_fast32_t)( in_ClearFlags1 & R_OSPL_EVENT_ALL_BITS ) ) );
        R_D_AddToIntLog( (int_fast32_t) (uintptr_t) R_OSPL_THREAD_GetCurrentId() );
        R_D_AddToIntLog( (int_fast32_t) (uintptr_t) in_ThreadId );
    }
#endif

#if  R_OSPL_EVENT_OBJECT_CODE
    event_object = R_OSPL_EVENT_GetEventObject_Sub( in_ThreadId, in_ClearFlags1 );
    if ( event_object == R_OSPL_EVENT_OBJECT_NULL )  /* If Event Flag */
#endif
    {

        target_thread->Flags &= ~( in_ClearFlags1 & R_OSPL_EVENT_ALL_BITS );

    }
#if  R_OSPL_EVENT_OBJECT_CODE
    else    /* If Event Object */
    {
        r_ospl_event_object_def_t*  object = (r_ospl_event_object_def_t*) event_object;


        object->IsSignaled = false;
    }
#endif


fin:
    if ( IS( was_all_enabled ) )
    {
        R_OSPL_EnableAllInterrupt();
    }
    return;
}


/***********************************************************************
* Implement: R_OSPL_EVENT_Get
************************************************************************/
#if ( ( ! defined( osCMSIS )  ||  osCMSIS <= 0x10001 ) &&  R_OSPL_VERSION < 85 ) ||  R_OSPL_SUPPORT_EVENT_GET
bit_flags32_t  R_OSPL_EVENT_Get( r_ospl_thread_id_t const  in_ThreadId )
{
    bit_flags32_t  return_value;
    r_ospl_thread_def_t* const  target_thread = (r_ospl_thread_def_t*) in_ThreadId;

    if ( target_thread == R_OSPL_THREAD_NULL )
    {
        return_value = 0;
        goto fin;
    }

    return_value = target_thread->Flags;

fin:
    return  return_value;
}
#endif


/***********************************************************************
* Implement: R_OSPL_EVENT_Wait
************************************************************************/
errnum_t  R_OSPL_EVENT_Wait( bit_flags32_t  in_WaigingFlags,  bit_flags32_t* const  out_GotFlags,
                             uint32_t const  in_Timeout_msec )
{
    errnum_t  e;
#if R_OSPL_CPU_LOAD
    bool_t    is_idle_started = false;
#endif
    bool_t    is_over_timeout_max = false;
    bool_t    is_timeout;
    r_ospl_thread_def_t* const  current_thread = (r_ospl_thread_def_t*) R_OSPL_THREAD_GetCurrentId();
#if  R_OSPL_EVENT_OBJECT_CODE
    r_ospl_event_object_id_t    event_object;
#endif
#if  R_OSPL_DETECT_BAD_EVENT  &&  R_OSPL_TLS_EVENT_CODE
    r_ospl_event_status_t*  status = NULL;
    r_ospl_error_t*         err = NULL;
#endif
    bit_flags32_t  or_flag;
    bit_flags32_t  got_flags = 0x00000000u;

    IF_DQ( current_thread == R_OSPL_THREAD_NULL )
    {
        e=E_OTHERS;
        goto fin;
    }
    R_STATIC_ASSERT( R_OSPL_TIMEOUT != 0,  "" );


#if R_OSPL_CPU_LOAD
    {
        bool_t  was_all_enabled;

        was_all_enabled = R_OSPL_DisableAllInterrupt();

        gs_master.IdleCallback( R_OSPL_IDLE_START );

        if ( IS( was_all_enabled ) )
        {
            R_OSPL_EnableAllInterrupt();
        }

        is_idle_started = true;
    }
#endif

#if  R_OSPL_EVENT_WATCH
    if ( g_DebugVar[ R_OSPL_EVENT_WATCH ] == NULL  ||
            g_DebugVar[ R_OSPL_EVENT_WATCH ] == (int_fast32_t) current_thread )
    {
        bool_t  is_log;

#if  R_OSPL_IS_PREEMPTION
        is_log = ( in_Timeout_msec > 0 );
#else
        is_log = ( in_Timeout_msec > 0 ) ||
                 IS_BIT_NOT_SET( current_thread->Status,  R_OSPL_IS_WAITING_BY_0 );
#endif

        if ( IS( is_log ) )
        {
            R_D_AddToIntLog( 0x70BE0000 + ( (int_fast32_t)( in_WaigingFlags & R_OSPL_EVENT_ALL_BITS ) ) );
            R_D_AddToIntLog( (int_fast32_t) (uintptr_t) R_OSPL_THREAD_GetCurrentId() );

            {
                static int tc;
                if ( R_D_Counter( &tc, 0, NULL ) )
                {
                    R_DEBUG_BREAK();
                }
            }
        }
    }
#endif

#if  R_OSPL_EVENT_OBJECT_CODE
    event_object = R_OSPL_EVENT_GetEventObject_Sub( current_thread,  R_OSPL_EVENT_OBJECT_FLAG );
#endif

#if  R_OSPL_DETECT_BAD_EVENT  &&  R_OSPL_TLS_EVENT_CODE
    e= R_OSPL_EVENT_GetStatus( current_thread,  &status );
    IF(e!=0)
    {
        goto fin;
    }
    if ( in_Timeout_msec != 0 )
    {
        ASSERT_R( status->AllocatedEvents != 0u,  R_OSPL_BAD_EVENT_ERROR() );
        /* Error: Any events are not allocated. */
    }
#endif


    e= R_OSPL_TIMEOUT_Start_Sub( in_Timeout_msec );
    IF(e!=0)
    {
        goto fin;
    }


    or_flag = in_WaigingFlags & R_OSPL_OR_FLAG;
    in_WaigingFlags &= R_OSPL_EVENT_ALL_BITS;


    for (;;)
    {
        bool_t    is_exit;
        uint32_t  thread_flags = current_thread->Flags;


#if  R_OSPL_EVENT_OBJECT_CODE
        if ( event_object == R_OSPL_EVENT_OBJECT_NULL )  /* If Event Flag */
#endif
        {
            if ( in_WaigingFlags == R_OSPL_ANY_FLAG )
            {
                is_exit = IS_ANY_BITS_SET(
                              (bit_flags_fast32_t) thread_flags,
                              R_OSPL_EVENT_ALL_BITS );
            }
            else if ( or_flag != 0 )
            {
                is_exit = IS_ANY_BITS_SET(
                              (bit_flags_fast32_t) thread_flags,
                              (bit_flags_fast32_t) in_WaigingFlags );
            }
            else
            {
                is_exit = IS_ALL_BITS_SET(
                              (bit_flags_fast32_t) thread_flags,
                              (bit_flags_fast32_t) in_WaigingFlags );
            }
            if ( IS( is_exit ) )
            {
                bool_t  was_all_enabled;

                was_all_enabled = R_OSPL_DisableAllInterrupt();

#if R_OSPL_CPU_LOAD
                gs_master.IdleCallback( R_OSPL_IDLE_END );
                is_idle_started = false;
#endif


                if ( in_WaigingFlags == R_OSPL_ANY_FLAG )
                {
                    got_flags = current_thread->Flags;
                    current_thread->Flags = 0u;
                }
                else if ( or_flag != 0 )
                {
                    got_flags = current_thread->Flags & in_WaigingFlags;
                    current_thread->Flags &= ~ (uint32_t) in_WaigingFlags;
                }
                else
                {
                    got_flags = in_WaigingFlags;
                    current_thread->Flags &= ~ (uint32_t) in_WaigingFlags;
                }


                if ( IS( was_all_enabled ) )
                {
                    R_OSPL_EnableAllInterrupt();
                }
            }
        }

#if  R_OSPL_EVENT_OBJECT_CODE
        else    /* If Event Object */
        {
            r_ospl_event_object_def_t*  object = (r_ospl_event_object_def_t*) event_object;


            is_exit = IS( object->IsSignaled );


            if ( IS( is_exit ) )
            {
                bool_t  was_all_enabled;

                was_all_enabled = R_OSPL_DisableAllInterrupt();

#if R_OSPL_CPU_LOAD
                gs_master.IdleCallback( R_OSPL_IDLE_END );
                is_idle_started = false;
#endif


                got_flags = R_OSPL_EVENT_OBJECT_FLAG;
                object->IsSignaled = false;


                if ( IS( was_all_enabled ) )
                {
                    R_OSPL_EnableAllInterrupt();
                }
            }
        }
#endif


        if ( ! is_exit )
        {
            is_exit = ! R_OSPL_TIMEOUT_GetIsLoop_Sub( &is_timeout, &is_over_timeout_max );
            if ( IS( is_timeout ) )
            {

                got_flags = R_OSPL_TIMEOUT;

                IF ( in_Timeout_msec != 0u )
                {
                    e=E_TIME_OUT;
                    goto fin;
                }
                current_thread->Status |= R_OSPL_IS_WAITING_BY_0;
            }
        }
        if ( IS( is_exit ) )
        {
            break;    /* MISRA 14.6 */
        }


#if R_OSPL_CPU_LOAD
        gs_master.IdleCallback( R_OSPL_IDLING );
#endif
    }


#if  R_OSPL_DETECT_BAD_EVENT  &&  R_OSPL_TLS_EVENT_CODE
    {
        uint32_t  got_flags_2 = got_flags & ( R_OSPL_EVENT_OBJECT_FLAG | R_OSPL_EVENT_ALL_BITS );

        ASSERT_R( IS_ALL_BITS_SET( status->AllocatedEvents,  got_flags_2 ),  R_OSPL_BAD_EVENT_ERROR() );

        err = R_OSPL_LockCurrentThreadError_Sub( current_thread, R_OSPL_ALLOCATE_IF_NOT );
        err->EventStatus.UnexpectedEvents |= got_flags_2;
    }
#endif


    e=0;
fin:
#if  R_OSPL_DETECT_BAD_EVENT  &&  R_OSPL_TLS_EVENT_CODE
    R_OSPL_UnlockCurrentThreadError( &err );
#endif
#if  R_OSPL_EVENT_WATCH
    if ( g_DebugVar[ R_OSPL_EVENT_WATCH ] == NULL  ||
            g_DebugVar[ R_OSPL_EVENT_WATCH ] == (int_fast32_t) current_thread )
    {
        bool_t  is_log;

#if  R_OSPL_IS_PREEMPTION
        is_log = ( in_Timeout_msec > 0 );
#else
        is_log = ( in_Timeout_msec > 0 ) ||
                 IS_BIT_NOT_SET( current_thread->Status,  R_OSPL_IS_WAITING_BY_0 );
#endif

        if ( IS( is_log ) )
        {
            R_D_AddToIntLog( 0x70AF0000 );
            R_D_AddToIntLog( (int_fast32_t) (uintptr_t) R_OSPL_THREAD_GetCurrentId() );
            if ( current_thread == R_OSPL_THREAD_NULL )
            {
                R_D_AddToIntLog( 0 );
            }
            else
            {
                R_D_AddToIntLog( (int_fast32_t) current_thread->Flags );
            }
            R_D_AddToIntLog( (int_fast32_t) got_flags );
        }
    }
#endif
#if R_OSPL_CPU_LOAD
    if ( IS( is_idle_started ) )
    {
        gs_master.IdleCallback( R_OSPL_IDLE_END );
    }
#endif
    IF ( IS( is_over_timeout_max ) )
    {
        e= R_OSPL_MergeErrNum( e, E_TIME_OUT );
    }
    if ( out_GotFlags != NULL )
    {
        *out_GotFlags = got_flags;
    }
    return  e;
}


/***********************************************************************
* Implement: R_OSPL_EVENT_OBJECT_Create
************************************************************************/
#if  R_OSPL_EVENT_OBJECT_CODE
errnum_t  R_OSPL_EVENT_OBJECT_Create( r_ospl_event_object_id_t* const  out_EventObject,
                                      r_ospl_event_object_def_t* const  in_EventObjectDef )
{
    errnum_t  e;
    r_ospl_event_object_def_t* const  self = in_EventObjectDef;

    IF_DQ( out_EventObject == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }
    IF_DQ( self == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }

    *out_EventObject = self;

    self->IsSignaled = false;

    e=0;
fin:
    return  e;
}
#endif  /* R_OSPL_EVENT_OBJECT_CODE */


/***********************************************************************
* Implement: R_OSPL_MEMORY_Flush
************************************************************************/
void  R_OSPL_MEMORY_Flush( r_ospl_flush_t const  in_FlushType )
{
    if ( in_FlushType == R_OSPL_FLUSH_WRITEBACK_INVALIDATE )
    {
#if 0
        printf( "L1Flush\n" );
#endif

#if  defined( R_OSPL_FOR_RZ_A1_OS_LESS )  &&  R_OSPL_FOR_RZ_A1_OS_LESS >= 101
        L1_D_CacheWritebackFlushAll();
#elif  defined( USE_LIST_RZA1H_OS_LESS_RSK )
        R_OSPL_MEMORY_Flush_Sub();
#endif
    }
    else if ( in_FlushType == R_OSPL_FLUSH_WRITEBACK_INVALIDATE_2ND )
    {
#if 0
        printf( "L2Flush\n" );
#endif

        ASSERT_D( false,  R_NOOP() ); /* It is necessary to flush */
        /* If "AWCACHE" and "ARCACHE" of peripherals were set to cache writeback. */
    }
    else
    {
        ASSERT_D( false,  R_NOOP() );
    }
}


/***********************************************************************
* Implement: R_OSPL_MEMORY_RangeFlush
************************************************************************/
errnum_t  R_OSPL_MEMORY_RangeFlush( r_ospl_flush_t const  in_FlushType,
                                    const void* const  in_StartAddress,  size_t const  in_Length )
{
    errnum_t   e;
    size_t     cache_line_size;
    size_t     cache_line_mask;
    uintptr_t  start;
    uintptr_t  over;

    ASSERT_R( in_FlushType == R_OSPL_FLUSH_INVALIDATE,  e=E_BAD_COMMAND_ID; goto fin );

    cache_line_size = R_OSPL_MEMORY_GetCacheLineSize();
    cache_line_mask = cache_line_size - 1u;
    ASSERT_D( R_OSPL_IsSetBitsCount1( cache_line_size ), e=E_OTHERS; goto fin );

    /* ->MISRA 11.3 */ /* ->SEC R2.7.1 */
    ASSERT_R( ( (uintptr_t) in_StartAddress & cache_line_mask ) == 0u, e=E_OTHERS; goto fin );
    ASSERT_R( ( in_Length & cache_line_mask ) == 0u, e=E_OTHERS; goto fin );

    start = (uintptr_t) in_StartAddress;
    over  = ( (uintptr_t) in_StartAddress + in_Length ) - 1u;
    /* <-MISRA 11.3 */ /* <-SEC R2.7.1 */

    R_OSPL_MEMORY_RangeFlush_Sub( start, over, cache_line_size );

    e=0;
fin:
    return  e;
}


/***********************************************************************
* Implement: R_OSPL_MEMORY_GetSpecification
************************************************************************/
void  R_OSPL_MEMORY_GetSpecification( r_ospl_memory_spec_t* const  out_MemorySpec )
{
    IF_DQ( out_MemorySpec == NULL )
    {
        goto fin;
    }

    out_MemorySpec->CacheLineSize = R_OSPL_MEMORY_GetCacheLineSize();

fin:
    return;
}


/***********************************************************************
* Implement: R_OSPL_Delay
************************************************************************/
errnum_t  R_OSPL_Delay( uint32_t const  in_DelayTime_msec )
{
    errnum_t  e;
#if R_OSPL_CPU_LOAD
    bool_t    is_idle_started = false;
#endif
    bool_t    is_over_timeout_max = false;
    bool_t    is_timeout;

    IF ( gs_master.CurrentThread == R_OSPL_THREAD_NULL )    /* Interrupt */
    {
        e = E_NOT_THREAD;
        R_OSPL_RaiseUnrecoverable( e );
        goto fin;
    }

    e= R_OSPL_TIMEOUT_Start_Sub( in_DelayTime_msec );
    IF(e!=0)
    {
        goto fin;
    }

#if R_OSPL_CPU_LOAD
    gs_master.IdleCallback( R_OSPL_IDLE_START );
    is_idle_started = true;
#endif


    while ( IS( R_OSPL_TIMEOUT_GetIsLoop_Sub( &is_timeout, &is_over_timeout_max ) ) )
    {
#if R_OSPL_CPU_LOAD
        gs_master.IdleCallback( R_OSPL_IDLING );
#endif
    }

    e=0;
fin:
#if R_OSPL_CPU_LOAD
    if ( IS( is_idle_started ) )
    {
        gs_master.IdleCallback( R_OSPL_IDLE_END );
    }
#endif
    IF ( IS( is_over_timeout_max ) )
    {
        e= R_OSPL_MergeErrNum( e, E_TIME_OUT );
    }
    return  e;
}


/***********************************************************************
* Function: R_OSPL_TIMEOUT_Start_Sub
*
* Description:
*    "R_OSPL_TIMEOUT_GetIsLoop_Sub" returns whether time out or not.
************************************************************************/
static errnum_t  R_OSPL_TIMEOUT_Start_Sub( uint32_t const  in_Timeout_msec )
{
    errnum_t  e;
    r_ospl_thread_def_t* const  current_thread = gs_master.CurrentThread;

    IF ( current_thread == R_OSPL_THREAD_NULL )    /* If interrupt context */
    {
        e = E_ACCESS_DENIED;
        R_OSPL_RaiseUnrecoverable( e );
        goto fin;
    }

    current_thread->Status &= ~ (bit_flags_fast32_t) R_OSPL_IS_WAITING_BY_0;

    if ( IS_BIT_NOT_SET( current_thread->Status, R_OSPL_IS_WAITING ) )
    {
        if ( in_Timeout_msec == R_OSPL_INFINITE )
        {
            current_thread->Status |= R_OSPL_IS_TIME_OUT_INFINITE;
            current_thread->Timeout_msec = in_Timeout_msec;
        }

        /* Set "TimeoutCount" */
        else
        {
            r_ospl_ftimer_spec_t* const  ts = &gs_master.FreeRunTimerSpec;
            uint32_t  timeout_msec_2;
            uint32_t  timeout_count;
            uint32_t  now;

            current_thread->Status &= ~ (bit_flags_fast32_t) R_OSPL_IS_TIME_OUT_INFINITE;

            if ( ! gs_master.IsFreeRunTimerSpec )
            {
                e= R_OSPL_FTIMER_InitializeIfNot( ts );
                IF(e!=0)
                {
                    goto fin;
                }
                gs_master.IsFreeRunTimerSpec = true;

                if ( ts->msec_Numerator >= ts->msec_Denominator )
                {
                    gs_master.MaxOneTimeoutTime = R_OSPL_MAX_TIME_OUT;
                }
                else
                {
                    uint32_t const  timeout_max_count =
                        R_OSPL_GetFTimerMaxCountToTimeoutMaxCount( ts );
                    /* SEC R3.6.2 */

                    gs_master.MaxOneTimeoutTime = R_OSPL_FTIMER_CountToTime(
                                                      ts, timeout_max_count );
                }
            }


            /* Set "timeout_time" */
            if ( in_Timeout_msec <= R_OSPL_MAX_TIME_OUT )
            {
                timeout_msec_2 = in_Timeout_msec;
            }
            else
            {
                ASSERT_D( false, R_NOOP() );  /* Break, if debug configuration */

                current_thread->Status |= R_OSPL_IS_OVER_MAX_TIME_OUT;
                timeout_msec_2 = R_OSPL_MAX_TIME_OUT;
            }


            /* Set "timeout_count" */
            if ( timeout_msec_2 < gs_master.MaxOneTimeoutTime )
            {
                timeout_count = R_OSPL_FTIMER_TimeToCount( ts, timeout_msec_2 );
            }
            else
            {
                current_thread->Status |= R_OSPL_IS_CASCADE_TIME_OUT;

                timeout_count = R_OSPL_GetFTimerMaxCountToTimeoutMaxCount( ts );
            }


            /* ... */
            now = R_OSPL_FTIMER_Get();
            /* This code should be set after ASSERT_D of "in_Timeout_msec <= R_OSPL_MAX_TIME_OUT" */
            current_thread->Timeout_msec = in_Timeout_msec;
            current_thread->TimeoutCount = now + timeout_count;
        }
    }
    else
    {
        current_thread->Status &= ~ (bit_flags_fast32_t) R_OSPL_IS_WAITING;
        ASSERT_R( current_thread->Timeout_msec == in_Timeout_msec,  e=E_STATE; goto fin );
        /* Caller may be changed. Please call a waiting function or "R_OSPL_THREAD_ExitWaiting" */
        /* Caller may be found by "R_OSPL_WAIT_OWNER_DEBUG" */
    }

    e=0;
fin:
    return  e;
}


/***********************************************************************
* Function: R_OSPL_TIMEOUT_GetIsLoop_Sub
*
* Return Value:
*    Whether time out is not occurred. (not = true)
************************************************************************/
static bool_t  R_OSPL_TIMEOUT_GetIsLoop_Sub( bool_t* const  out_IsTimeout,
        bool_t* const  out_IsOverMaxTimeout )
{
    errnum_t  e;
    bool_t    is_loop = false;
    r_ospl_thread_def_t* const  current_thread = gs_master.CurrentThread;

    IF_DQ( current_thread == R_OSPL_THREAD_NULL )
    {
        goto fin;
    }
    IF_DQ( out_IsTimeout == NULL )
    {
        goto fin;
    }
    IF_DQ( out_IsOverMaxTimeout == NULL )
    {
        goto fin;
    }

    *out_IsTimeout = false;
    *out_IsOverMaxTimeout = false;


    if ( IS_BIT_SET( current_thread->Status, R_OSPL_IS_TIME_OUT_INFINITE ) )
    {
        is_loop = true;
    }
    else
    {
        bool_t    is_past;
        uint32_t  timeout_count;
        uint32_t  now;

        now = R_OSPL_FTIMER_Get();

        e= R_OSPL_FTIMER_IsPast( &gs_master.FreeRunTimerSpec,
                                 now, current_thread->TimeoutCount, &is_past );
        IF(e!=0)
        {
            goto fin;
        }

        if ( IS( is_past ) )
        {
            if ( IS_BIT_SET( current_thread->Status, R_OSPL_IS_CASCADE_TIME_OUT ) )
            {
                /* Set next "TimeoutCount" */
                current_thread->Timeout_msec -= gs_master.MaxOneTimeoutTime;
                if ( current_thread->Timeout_msec <= gs_master.MaxOneTimeoutTime )
                {
                    timeout_count = R_OSPL_FTIMER_TimeToCount(
                                        &gs_master.FreeRunTimerSpec, current_thread->Timeout_msec );
                    current_thread->Status &= ~R_OSPL_IS_CASCADE_TIME_OUT;
                }
                else
                {
                    timeout_count = R_OSPL_GetFTimerMaxCountToTimeoutMaxCount(
                                        &gs_master.FreeRunTimerSpec ) ;
                }
                current_thread->TimeoutCount += timeout_count;

                is_loop = true;
            }
            else
            {
                *out_IsTimeout = true;

                if ( IS_BIT_SET( current_thread->Status, R_OSPL_IS_OVER_MAX_TIME_OUT ) )
                {
                    *out_IsOverMaxTimeout = true;
                }
                current_thread->Status &= ~ (bit_flags_fast32_t) R_OSPL_IS_OVER_MAX_TIME_OUT;

                is_loop = false;
            }
        }
        else
        {
            is_loop = true;
        }

        if ( IS( is_loop ) )
        {
            if ( IS_BIT_SET( current_thread->Status, R_OSPL_IS_EXIT_IF_WAIT ) )
            {
                current_thread->Status |= R_OSPL_IS_WAITING;
                is_loop = false;

#if R_OSPL_WAIT_OWNER_DEBUG
                {
                    static int tc;

                    if ( R_D_Counter( &tc, 0, NULL ) )
                    {
                        R_DEBUG_BREAK();
                        /* This is for finding previous waited thread */
                    }

                    R_D_AddToIntLog( 0x71000000 + tc );
                    R_D_AddToIntLog( (uintptr_t) current_thread );
                }
#endif
            }
        }
    }
fin:
    return  is_loop;
}


/***********************************************************************
* Implement: R_OSPL_QUEUE_Create
************************************************************************/
errnum_t  R_OSPL_QUEUE_Create( r_ospl_queue_id_t*  out_self, r_ospl_queue_def_t* in_QueueDefine )
{
    errnum_t  e;
    int_fast32_t       i;
    int_fast32_t       i_last;
    r_ospl_queue_id_t  self = in_QueueDefine;


    /* Set "self->BlockArray" */
    i_last = self->PublicStatus.MaxCount - 1;
    for ( i = 0;  i < i_last;  i += 1 )
    {
        self->BlockArray[i].Next = &self->BlockArray[i+1];
    }
    self->BlockArray[i_last].Next = NULL;

    /* ... */
    self->NotUsedBlocksList = &self->BlockArray[0];
    self->PutBlocksFirst = NULL;
    self->PutBlocksLast = NULL;
    self->PublicStatus.UsedCount = 0;

    *out_self = self;

    e=0;
    /*fin:*/
    return  e;
}


/***********************************************************************
* Implement: R_OSPL_QUEUE_GetStatus
************************************************************************/
errnum_t  R_OSPL_QUEUE_GetStatus( r_ospl_queue_id_t  self,  const r_ospl_queue_status_t** out_Status )
{
    *out_Status = &self->PublicStatus;
    return  0;
}


/***********************************************************************
* Implement: R_OSPL_QUEUE_Allocate
************************************************************************/
errnum_t  R_OSPL_QUEUE_Allocate( r_ospl_queue_id_t  self,  void* out_Address,  uint32_t in_Timeout_msec )
{
    errnum_t  e;
#if R_OSPL_CPU_LOAD
    bool_t    is_idle_started = false;
#endif
    bool_t    is_over_timeout_max = false;
    bool_t    is_timeout;
    r_ospl_thread_def_t* const  current_thread = (r_ospl_thread_def_t*) R_OSPL_THREAD_GetCurrentId();

    int_fast32_t     index_of_block;
    void*            allocated_address = NULL;


    if ( gs_master.CurrentThread == R_OSPL_THREAD_NULL )
    {
        ASSERT_R( in_Timeout_msec == 0,  e=E_NOT_THREAD; goto fin );
    }
    else
    {
        e= R_OSPL_TIMEOUT_Start_Sub( in_Timeout_msec );
        IF(e!=0)
        {
            goto fin;
        }
    }


#if R_OSPL_CPU_LOAD
    {
        bool_t  was_all_enabled;

        was_all_enabled = R_OSPL_DisableAllInterrupt();

        gs_master.IdleCallback( R_OSPL_IDLE_START );

        if ( IS( was_all_enabled ) )
        {
            R_OSPL_EnableAllInterrupt();
        }

        is_idle_started = true;
    }
#endif

#if  R_OSPL_EVENT_WATCH
    if ( in_Timeout_msec > 0 )
    {
        R_D_AddToIntLog( 0x70BF0051 );
        R_D_AddToIntLog( (uintptr_t) R_OSPL_THREAD_GetCurrentId() );

        {
            static int tc;
            if ( R_D_Counter( &tc, 0, NULL ) )
            {
                R_DEBUG_BREAK();
            }
        }
    }
#endif

    for (;;)
    {
        bool_t  is_exit;

        is_exit = ( self->NotUsedBlocksList != NULL );

        if ( IS( is_exit ) )
        {
            bool_t  was_all_enabled;

            was_all_enabled = R_OSPL_DisableAllInterrupt();

#if R_OSPL_CPU_LOAD
            gs_master.IdleCallback( R_OSPL_IDLE_END );
            is_idle_started = false;
#endif


            /* Set "*out_Address" */
            index_of_block = self->NotUsedBlocksList - self->BlockArray;
            allocated_address = self->MemoryArray + index_of_block * self->ElementSize;

            /* ... */
            self->NotUsedBlocksList = self->NotUsedBlocksList->Next;
            self->PublicStatus.UsedCount += 1;

#ifndef R_OSPL_NDEBUG
            self->BlockArray[ index_of_block ].Next = NULL;
#endif

            if ( IS( was_all_enabled ) )
            {
                R_OSPL_EnableAllInterrupt();
            }
        }
        else
        {
            if ( gs_master.CurrentThread == R_OSPL_THREAD_NULL )    /* Interrupt */
            {
                is_exit = true;
                e=E_NOT_THREAD;
            }
            else
            {
                is_exit = ! R_OSPL_TIMEOUT_GetIsLoop_Sub( &is_timeout, &is_over_timeout_max );
                if ( IS( is_timeout ) )
                {

                    *(void**) out_Address = NULL;

                    IF ( in_Timeout_msec != 0u )
                    {
                        e=E_TIME_OUT;
                        goto fin;
                    }
                    current_thread->Status |= R_OSPL_IS_WAITING_BY_0;
                }
            }
        }
        if ( IS( is_exit ) )
        {
            break;    /* MISRA 14.6 */
        }


#if R_OSPL_CPU_LOAD
        gs_master.IdleCallback( R_OSPL_IDLING );
#endif
    }

    e=0;
fin:
#if  R_OSPL_EVENT_WATCH
    if ( in_Timeout_msec > 0 )
    {
        R_D_AddToIntLog( 0x70AF0051 );
        R_D_AddToIntLog( (uintptr_t) R_OSPL_THREAD_GetCurrentId() );
    }
#endif
#if R_OSPL_CPU_LOAD
    if ( IS( is_idle_started ) )
    {
        gs_master.IdleCallback( R_OSPL_IDLE_END );
    }
#endif
    IF ( IS( is_over_timeout_max ) )
    {
        e= R_OSPL_MergeErrNum( e, E_TIME_OUT );
    }
    IF_D ( out_Address == NULL )
    {
        e= R_OSPL_MergeErrNum( e, E_OTHERS );
    }
    else
    {
        *(void**) out_Address = allocated_address;
    }
    return  e;
}


/***********************************************************************
* Implement: R_OSPL_QUEUE_Put
************************************************************************/
errnum_t  R_OSPL_QUEUE_Put( r_ospl_queue_id_t  self,  void* in_Address )
{
    errnum_t         e;
    int_fast32_t     index_of_block;
    r_ospl_block_t*  block;
    bool_t           was_all_enabled = false;

    IF_DQ( self == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }
    IF_DQ( in_Address == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }

    was_all_enabled = R_OSPL_DisableAllInterrupt();


    index_of_block = ( (uint8_t*) in_Address - self->MemoryArray ) / self->ElementSize;
    block = &self->BlockArray[ index_of_block ];

#ifndef R_OSPL_NDEBUG
    ASSERT_D( block->Next == NULL,  e=E_STATE; goto fin );
#endif


    /* Set "self->PutBlocksFirst", "self->PutBlocksLast" */
    block->Next = NULL;
    if ( self->PutBlocksFirst == NULL )
    {
        self->PutBlocksLast = block;
    }
    else
    {
        self->PutBlocksFirst->Next = block;
    }
    self->PutBlocksFirst = block;

    e=0;
fin:
    if ( was_all_enabled )
    {
        R_OSPL_EnableAllInterrupt();
    }
    return  e;
}


/***********************************************************************
* Implement: R_OSPL_QUEUE_Get
************************************************************************/
errnum_t  R_OSPL_QUEUE_Get( r_ospl_queue_id_t  self,  void* out_Address,  uint32_t in_Timeout_msec )
{
    errnum_t  e;
#if R_OSPL_CPU_LOAD
    bool_t    is_idle_started = false;
#endif
    bool_t    is_over_timeout_max = false;
    bool_t    is_timeout;
    r_ospl_thread_def_t* const  current_thread = (r_ospl_thread_def_t*) R_OSPL_THREAD_GetCurrentId();

    void*            got_address = NULL;
    r_ospl_block_t*  got_block;
    r_ospl_block_t*  next_block;

    IF_DQ( self == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }
    IF_DQ( out_Address == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }


    if ( gs_master.CurrentThread == R_OSPL_THREAD_NULL )
    {
        ASSERT_R( in_Timeout_msec == 0,  e=E_NOT_THREAD; goto fin );
    }
    else
    {
        e= R_OSPL_TIMEOUT_Start_Sub( in_Timeout_msec );
        IF(e!=0)
        {
            goto fin;
        }
    }


#if R_OSPL_CPU_LOAD
    {
        bool_t  was_all_enabled;

        was_all_enabled = R_OSPL_DisableAllInterrupt();

        gs_master.IdleCallback( R_OSPL_IDLE_START );

        if ( IS( was_all_enabled ) )
        {
            R_OSPL_EnableAllInterrupt();
        }

        is_idle_started = true;
    }
#endif

#if  R_OSPL_EVENT_WATCH
    if ( in_Timeout_msec > 0 )
    {
        R_D_AddToIntLog( 0x70BF0051 );
        R_D_AddToIntLog( (uintptr_t) R_OSPL_THREAD_GetCurrentId() );

        {
            static int tc;
            if ( R_D_Counter( &tc, 0, NULL ) )
            {
                R_DEBUG_BREAK();
            }
        }
    }
#endif

    for (;;)
    {
        bool_t  is_exit;

        is_exit = ( self->PutBlocksLast != NULL );

        if ( IS( is_exit ) )
        {
            bool_t        was_all_enabled;
            int_fast32_t  index_of_block;

            was_all_enabled = R_OSPL_DisableAllInterrupt();

#if R_OSPL_CPU_LOAD
            gs_master.IdleCallback( R_OSPL_IDLE_END );
            is_idle_started = false;
#endif


            /* Set "got_address" */
            got_block = self->PutBlocksLast;
            index_of_block = ( got_block - self->BlockArray );
            got_address = self->MemoryArray + index_of_block * self->ElementSize;

            next_block = got_block->Next;
            self->PutBlocksLast = next_block;
            if ( next_block == NULL )
            {
                self->PutBlocksFirst = NULL;
            }

#ifndef R_OSPL_NDEBUG
            got_block->Next = NULL;
#endif

            if ( IS( was_all_enabled ) )
            {
                R_OSPL_EnableAllInterrupt();
            }
        }
        else
        {
            if ( gs_master.CurrentThread == R_OSPL_THREAD_NULL )    /* Interrupt */
            {
                is_exit = true;
                e=E_NOT_THREAD;
            }
            else
            {
                is_exit = ! R_OSPL_TIMEOUT_GetIsLoop_Sub( &is_timeout, &is_over_timeout_max );
                if ( IS( is_timeout ) )
                {

                    *(void**) out_Address = NULL;

                    IF ( in_Timeout_msec != 0u )
                    {
                        e=E_TIME_OUT;
                        goto fin;
                    }
                    current_thread->Status |= R_OSPL_IS_WAITING_BY_0;
                }
            }
        }
        if ( IS( is_exit ) )
        {
            break;    /* MISRA 14.6 */
        }


#if R_OSPL_CPU_LOAD
        gs_master.IdleCallback( R_OSPL_IDLING );
#endif
    }

    e=0;
fin:
#if  R_OSPL_EVENT_WATCH
    if ( in_Timeout_msec > 0 )
    {
        R_D_AddToIntLog( 0x70AF0054 );
        R_D_AddToIntLog( (uintptr_t) R_OSPL_THREAD_GetCurrentId() );
    }
#endif
#if R_OSPL_CPU_LOAD
    if ( IS( is_idle_started ) )
    {
        gs_master.IdleCallback( R_OSPL_IDLE_END );
    }
#endif
    IF ( IS( is_over_timeout_max ) )
    {
        e= R_OSPL_MergeErrNum( e, E_TIME_OUT );
    }
    IF_D ( out_Address == NULL )
    {
        e= R_OSPL_MergeErrNum( e, E_OTHERS );
    }
    else
    {
        *(void**) out_Address = got_address;
    }
    return  e;
}


/***********************************************************************
* Implement: R_OSPL_QUEUE_Free
************************************************************************/
errnum_t  R_OSPL_QUEUE_Free( r_ospl_queue_id_t  self,  void* in_Address )
{
    errnum_t         e;
    r_ospl_block_t*  freeing_block;
    int_fast32_t     index_of_block;
    bool_t           was_all_enabled = false;

    IF_DQ( self == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }

    was_all_enabled = R_OSPL_DisableAllInterrupt();


    index_of_block = ( (uint8_t*) in_Address - self->MemoryArray ) / self->ElementSize;
    freeing_block = &self->BlockArray[ index_of_block ];

#ifndef R_OSPL_NDEBUG
    ASSERT_D( freeing_block->Next == NULL,  e=E_STATE; goto fin );
#endif

    freeing_block->Next = self->NotUsedBlocksList;
    self->NotUsedBlocksList = freeing_block;

    self->PublicStatus.UsedCount -= 1;

    e=0;
fin:
    if ( was_all_enabled )
    {
        R_OSPL_EnableAllInterrupt();
    }
    return  e;
}


/***********************************************************************
* Implement: R_OSPL_QUEUE_Print
************************************************************************/
#ifndef  R_OSPL_NDEBUG
errnum_t  R_OSPL_QUEUE_Print( r_ospl_queue_id_t  self )
{
    errnum_t  e;

    printf( "\n" );
    printf( "r_ospl_queue_id_t  0x%08X\n", (uintptr_t) self );
    if ( self == NULL )
    {
        e=0;
        goto fin;
    }

    printf( "    .MemoryArray = 0x%08X\n", (uintptr_t) self->MemoryArray );
    printf( "    .ElementSize = 0x%08X\n", self->ElementSize );
    printf( "    .BlockArray  = 0x%08X\n", (uintptr_t) self->BlockArray );
    e= R_OSPL_BLOCK_PrintList( self->NotUsedBlocksList, "NotUsedBlocksList",
                               self->BlockArray, self->PublicStatus.MaxCount );
    IF(e)
    {
        goto fin;
    }

    printf( "    .PutBlocksFirst = 0x%08X\n", (uintptr_t) self->PutBlocksFirst );
    if ( self->PutBlocksFirst != NULL )
    {
        ASSERT_R( self->PutBlocksFirst->Next == NULL,  e=E_OTHERS; goto fin );
    }
    e= R_OSPL_BLOCK_PrintList( self->PutBlocksLast, "PutBlocksLast",
                               self->BlockArray, self->PublicStatus.MaxCount );
    IF(e)
    {
        goto fin;
    }
    printf( "    .PublicStatus.UsedCount = %d\n", self->PublicStatus.UsedCount );
    printf( "    .PublicStatus.MaxCount  = %d\n", self->PublicStatus.MaxCount );

    e=0;
fin:
    return  e;
}
#endif  /* R_OSPL_NDEBUG */


/***********************************************************************
* Variable: gs_QueueAsSingletonLockConfig
************************************************************************/
#if  R_OSPL_QUEUE_CODE
static R_OSPL_QUEUE_DEF( gs_QueueAsSingletonLockConfig,  1,  int_fast32_t );
#endif


/***********************************************************************
* Implement: R_OSPL_GetQueueAsSingletonLock
************************************************************************/
#if  R_OSPL_QUEUE_CODE
r_ospl_queue_id_t  R_OSPL_GetQueueAsSingletonLock(void)
{
    static r_ospl_queue_id_t  gs_QueueAsSingletonLock = R_OSPL_QUEUE_NULL;

    if ( gs_QueueAsSingletonLock == R_OSPL_QUEUE_NULL )
    {
        bool_t  was_all_enabled = false;

        ASSERT_D( R_OSPL_THREAD_GetCurrentId() != R_OSPL_THREAD_NULL,  R_NOOP() );
        was_all_enabled = R_OSPL_DisableAllInterrupt();
        /* This area must be thread safe. */

        R_OSPL_QUEUE_Create( &gs_QueueAsSingletonLock,
                             R_OSPL_QUEUE( gs_QueueAsSingletonLockConfig ) );

        if ( was_all_enabled )
        {
            R_OSPL_EnableAllInterrupt();
        }
    }
    return  gs_QueueAsSingletonLock;
}
#endif


/***********************************************************************
* Function: R_OSPL_BLOCK_PrintList
*    Print a block.
************************************************************************/
#ifndef  R_OSPL_NDEBUG
errnum_t  R_OSPL_BLOCK_PrintList( r_ospl_block_t* in_ListFirst,  char_t* in_Name,
                                  r_ospl_block_t*  in_BlockArray,  int_fast32_t  in_MaxCount )
{
    int_fast32_t     i;
    r_ospl_block_t*  block;
    char_t*          a_string;

    i = 0;
    a_string = "";
    for ( block = in_ListFirst;  block != NULL;  block = block->Next )
    {
        i += 1;
        if ( i >= in_MaxCount  &&  block->Next != NULL )
        {
            a_string = "over ";
            break;
        }
    }
    printf( "    .%s.Length = %s%d\n", in_Name, a_string, i );
    i = 0;
    for ( block = in_ListFirst;  block != NULL;  block = block->Next )
    {
        printf( "    .%s[%d] = [%d]0x%08X\n", in_Name, i, block - in_BlockArray, (uintptr_t) block );
        i += 1;
        if ( i >= in_MaxCount  &&  block->Next != NULL )
        {
            printf( "       :\n" );
            break;
        }
    }

    return  0;
}
#endif  /* R_OSPL_NDEBUG */


/***********************************************************************
* Implement: R_OSPL_Start_T_Lock
*    The function callbacked from OSPL internal, when T-Lock started
*
* Arguments:
*    None
*
* Return Value:
*    Error Code. 0=No Error.
*
* InternalDescription:
*    See. <R_OSPL_LockCurrentThreadError_Sub>.
************************************************************************/
#if R_OSPL_IS_PREEMPTION
errnum_t  R_OSPL_Start_T_Lock(void)
{
    errnum_t  e;
    osStatus  es;

    static osMutexDef( gs_ospl_mutex );


#if  GS_DEBUG_T_LOCK
    R_D_AddToIntLog( 0xDDDD7101 );
    R_D_AddToIntLog( (uintptr_t) R_OSPL_THREAD_GetCurrentId() );
#endif


    if ( gs_ospl_mutex == NULL )
    {
        gs_ospl_mutex = osMutexCreate( osMutex( gs_ospl_mutex ) );
        if ( gs_ospl_mutex == NULL )
        {
            e=E_OTHERS;
            goto fin;
        }
    }

    es= osMutexWait( gs_ospl_mutex, TO_UNSIGNED( osWaitForever ) );
    if ( es == osErrorISR )
    {
        es = osOK;
    }
    if ( es != osOK )
    {
        e=E_OTHERS;
        goto fin;
    }

    e=0;
fin:
#if  GS_DEBUG_T_LOCK
    R_D_AddToIntLog( 0xDDDD7102 );
    R_D_AddToIntLog( (uintptr_t) R_OSPL_THREAD_GetCurrentId() );
#endif

    return  e;
}
#endif


/***********************************************************************
* Implement: R_OSPL_End_T_Lock
*    The function callbacked from OSPL internal, when T-Lock ended
*
* Arguments:
*    None
*
* Return Value:
*    None
*
* InternalDescription:
*    See. <R_OSPL_UnlockCurrentThreadError>.
************************************************************************/
#if R_OSPL_IS_PREEMPTION
void  R_OSPL_End_T_Lock(void)
{
#if  GS_DEBUG_T_LOCK
    R_D_AddToIntLog( 0xDDDD710F );
    R_D_AddToIntLog( (uintptr_t) R_OSPL_THREAD_GetCurrentId() );
#endif


    if ( gs_ospl_mutex != NULL )
    {
        osStatus  rs;

        rs= osMutexRelease( gs_ospl_mutex );
        if ( rs == osErrorISR )
        {
            rs = osOK;
        }
        ASSERT_R( rs == osOK,  R_OSPL_RaiseUnrecoverable( E_OTHERS ) );
    }
}
#endif


#endif  /* R_OSPL_IS_PREEMPTION */
