/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2015 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/***********************************************************************
* File: r_dmac_rm_inline.h
*    Direct Memory Access Controller (DMAC) - Register Mode Sample Driver.
*
* - $Version: $ (=PFV_VERSION)
* - $ModuleRevision: {Masters}\PFV_DMAC\PFV_DMAC\23_RStyled $
* - $Rev: $
* - $Date: 2017-08-24T18:32:45+09:00 $
************************************************************************/

#ifndef R_DMAC_RM_INLINE_H
#define R_DMAC_RM_INLINE_H

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include  "r_typedefs.h"
#include  "r_dmac_rm.h"


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/***********************************************************************
* Function: DMAC_RM_STATIC_GetRegisterSet
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
STATIC_INLINE errnum_t  DMAC_RM_STATIC_GetRegisterSet( int_fast32_t const  dmac_channel,
        struct st_dmac_n** const  out_reg_set )
{
    errnum_t  e;

    /* ->MISRA 11.3 */ /* ->MISRA 11.4 */ /* ->MISRA 11.5 */ /* ->SEC R2.7.1 */ /* ->SEC R2.7.2 */
    /* ->QAC 0312 */
    static struct st_dmac_n*  reg_set_array[ DMACn_COUNT ] =
                DMACn_ADDRESS_LIST;
    /* <-MISRA 11.3 */ /* <-MISRA 11.4 */ /* <-MISRA 11.5 */ /* <-SEC R2.7.1 */ /* <-SEC R2.7.2 */
    /* <-QAC 0312 */

#if DMAC_PARAMETER_CHECK
    IF ( dmac_channel < DMAC_CHANNEL_MIN )
{
        e = E_OTHERS;
        goto fin;
    }
    IF ( dmac_channel > DMAC_CHANNEL_MAX )
    {
        e = E_OTHERS;
        goto fin;
    }
#endif

    IF_DQ( out_reg_set == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }

    *out_reg_set = reg_set_array[ dmac_channel ];

    e = 0;
fin:
    return  e;
}


/***********************************************************************
* Function: DMAC_RM_STATIC_GetSharedRegisterSet
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
STATIC_INLINE errnum_t  DMAC_RM_STATIC_GetSharedRegisterSet( int_fast32_t const  dmac_channel,
        struct st_dmaccommon_n** const  out_reg_set )
{
    errnum_t  e;

    /* ->MISRA 11.3 */ /* ->MISRA 11.4 */ /* ->MISRA 11.5 */ /* ->SEC R2.7.1 */ /* ->SEC R2.7.2 */
    /* ->QAC 0312 */
    static struct st_dmaccommon_n*  shared_reg_set_array[ DMACnn_COUNT ] =
                DMACnn_ADDRESS_LIST;
    /* <-MISRA 11.3 */ /* <-MISRA 11.4 */ /* <-MISRA 11.5 */ /* <-SEC R2.7.1 */ /* <-SEC R2.7.2 */
    /* <-QAC 0312 */

    IF_DQ( out_reg_set == NULL )
{
        e=E_OTHERS;
        goto fin;
    }

    *out_reg_set = (struct st_dmaccommon_n*) NULL;  /* DUMMY_INITIAL_VALUE */

#if DMAC_PARAMETER_CHECK
    IF ( dmac_channel < DMAC_CHANNEL_MIN )
    {
        e = E_OTHERS;
        goto fin;
    }
    IF ( dmac_channel > DMAC_CHANNEL_MAX )
    {
        e = E_OTHERS;
        goto fin;
    }
#endif

    {
        enum { count_in_register = ( DMACn_COUNT / DMACnn_COUNT ) };

        *out_reg_set = shared_reg_set_array[ dmac_channel / count_in_register ];
    }

    e = 0;
fin:
    return  e;
}


#ifdef __cplusplus
}  /* End of extern "C" */
#endif /* __cplusplus */

#endif
