/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2015 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/***********************************************************************
* File: r_pfv.c
*    Pixel Format Converter (PFV) Sample Driver.
*
* - $Version: $ (=PFV_VERSION)
* - $ModuleRevision: {Masters}\PFV_DMAC\PFV_DMAC\23_RStyled $
* - $Rev: $
* - $Date: 2017-08-24T18:32:46+09:00 $
************************************************************************/

#ifndef R_PFV_H
#define R_PFV_H

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include  "r_pfv_user.h"
#include  "r_ospl.h"
#include  "r_dmac_rm_typedef.h"
#include  "r_pfv_typedef.h"


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/******************************************************************************
Macro definitions
******************************************************************************/
/* In "r_pfv_typedef.h" */

/******************************************************************************
Typedef definitions
******************************************************************************/
/* In "r_pfv_typedef.h" */

/******************************************************************************
Variable Externs
******************************************************************************/
/* In "r_pfv_typedef.h" */

/******************************************************************************
Functions Prototypes
******************************************************************************/
/* "const" is for QAC 3227 */

errnum_t  R_PFV_DMAC_Initialize( int_fast32_t const  pfv_channel,
                                 pfv_dmac_config_t* const  in_out_config );
errnum_t  R_PFV_DMAC_Terminate( int_fast32_t const  pfv_channel );
errnum_t  R_PFV_DMAC_SetIOFormat( int_fast32_t const  pfv_channel,
                                  pfv_io_format_t* const  in_out_io_format );
errnum_t  R_PFV_DMAC_GetIOFormat( int_fast32_t const  pfv_channel,
                                  pfv_io_format_t* const  out_io_format );
errnum_t  R_PFV_DMAC_ResetImageColorMatrix( int_fast32_t const  pfv_channel,
        pfv_color_matrix_mode_t const  mode );
errnum_t  R_PFV_DMAC_SetImageColorMatrix( int_fast32_t const  pfv_channel,
        const pfv_color_matrix_t* const  offset_and_matrix );
errnum_t  R_PFV_DMAC_GetImageColorMatrix( int_fast32_t const  pfv_channel,
        pfv_color_matrix_t* const  out_offset_and_matrix );
errnum_t  R_PFV_DMAC_Transfer( int_fast32_t const  pfv_channel,
                               pfv_transfer_config_t* const  in_out_config );
errnum_t  R_PFV_DMAC_TransferAsync( int_fast32_t const  pfv_channel,
                                    pfv_transfer_config_t* const  in_out_config,  r_ospl_async_t* const  async );
errnum_t  R_PFV_DMAC_StopTransfer( int_fast32_t const  pfv_channel );

errnum_t  R_PFV_DMAC_OnInterrupting( const r_ospl_interrupt_t* const  InterruptSource );
errnum_t  R_PFV_DMAC_OnInterrupted( int_fast32_t const  pfv_channel );

errnum_t  R_PFV_Initialize( int_fast32_t const  pfv_channel,
                            pfv_config_t*  in_out_config );
errnum_t  R_PFV_Terminate( int_fast32_t const  pfv_channel );
errnum_t  R_PFV_SetIOFormat( int_fast32_t const  pfv_channel,
                             pfv_io_format_t* const  in_out_io_format );
errnum_t  R_PFV_GetIOFormat( int_fast32_t const  pfv_channel,
                             pfv_io_format_t* const  out_io_format );
errnum_t  R_PFV_ResetImageColorMatrix( int_fast32_t const  pfv_channel,
                                       pfv_color_matrix_mode_t const  mode );
errnum_t  R_PFV_SetImageColorMatrix( int_fast32_t const  pfv_channel,
                                     const pfv_color_matrix_t* const  offset_and_matrix );
errnum_t  R_PFV_GetImageColorMatrix( int_fast32_t const  pfv_channel,
                                     pfv_color_matrix_t* const  out_offset_and_matrix );
errnum_t  R_PFV_GetFilledByteInInputFIFO( int_fast32_t const  pfv_channel,
        int_fast32_t* const  out_filled_byte );
errnum_t  R_PFV_WritePixelDataViaPIO( int_fast32_t const  pfv_channel,
                                      uint32_t const  pixel_data,  int_fast32_t const  bit_count );
errnum_t  R_PFV_GetFilledByteInOutputFIFO( int_fast32_t const  pfv_channel,
        int_fast32_t* const  out_filled_byte );
errnum_t  R_PFV_ReadPixelDataViaPIO( int_fast32_t const  pfv_channel,
                                     uint32_t* const  out_pixel_data,  int_fast32_t const  bit_count );

errnum_t  R_PFV_STATIC_GetColorMatrixMode( pfv_format_t const  input_format,
        pfv_format_t const  output_format,
        pfv_color_matrix_mode_t* const  out_color_matrix_mode );

errnum_t  R_PFV_GetAsyncStatus( int_fast32_t const  pfv_channel,
                                const pfv_async_status_t** const  out_Status );
#ifndef R_OSPL_NDEBUG
errnum_t  R_PFV_STATIC_PrintRegisters( int_fast32_t  pfv_channel );
#endif

/* For interrupt callback */
errnum_t  R_PFV_OnInterrupting( const r_ospl_interrupt_t* const  InterruptSource );
errnum_t  R_PFV_OnInterrupted( int_fast32_t const  pfv_channel );


#ifdef __cplusplus
}  /* End of extern "C" */
#endif /* __cplusplus */

#endif
