/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2015 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/***********************************************************************
* File: r_dmac_rm_pl.c
*    Direct Memory Access Controller (DMAC) - Register Mode Porting Layer.
*
* - $Version: $ (=PFV_VERSION)
* - $ModuleRevision: {Masters}\PFV_DMAC\PFV_DMAC\23_RStyled $
* - $Rev: $
* - $Date: 2017-08-24T18:32:46+09:00 $
************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include  "r_ospl_config.h"
#include  "r_dmac_rm.h"
#include  "r_dmac_rm_pl.h"


/******************************************************************************
Typedef definitions
******************************************************************************/

/***********************************************************************
* Structure: r_dmac_rm_pl_channel_t
************************************************************************/
typedef struct st_r_dmac_rm_pl_channel_t  r_dmac_rm_pl_channel_t;
struct st_r_dmac_rm_pl_channel_t
{
    const r_ospl_caller_t*         InterruptCallbackCaller;
    const dmac_rm_async_status_t*  Status;
};


/***********************************************************************
* Variable: gs_dmac_rm_channel
************************************************************************/
static r_dmac_rm_pl_channel_t  gs_dmac_rm_pl_channel[ R_DMAC_RM_CHANNEL_COUNT ];


/***********************************************************************
* Structure: r_dmac_rm_pl_t
************************************************************************/
typedef struct st_r_dmac_rm_pl_t  r_dmac_rm_pl_t;
struct st_r_dmac_rm_pl_t
{
    bit_flags_fast32_t  ErrorInterruptEnables;  /* Bit number = channel number */
};


/***********************************************************************
* Variable: gs_dmac_rm_pl
************************************************************************/
static r_dmac_rm_pl_t  gs_dmac_rm_pl;


/******************************************************************************
Macro definitions
******************************************************************************/

/***********************************************************************
* Constant: R_DMAC_RM_INTERRUPT_PRIORITY
************************************************************************/
enum { R_DMAC_RM_INTERRUPT_PRIORITY = 7 };


/***********************************************************************
* Constant: GS_UNKNOWN_CHANNEL_NUM
************************************************************************/
enum { GS_UNKNOWN_CHANNEL_NUM = -1 };


/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/

/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/

/******************************************************************************
Private global variables and functions
******************************************************************************/
static void  R_DMAC_RM_IRQ_Handler0( R_OSPL_INTERRUPT_ARGUMENTS );
static void  R_DMAC_RM_IRQ_Handler1( R_OSPL_INTERRUPT_ARGUMENTS );
static void  R_DMAC_RM_IRQ_Handler2( R_OSPL_INTERRUPT_ARGUMENTS );
static void  R_DMAC_RM_IRQ_Handler3( R_OSPL_INTERRUPT_ARGUMENTS );
static void  R_DMAC_RM_IRQ_Handler4( R_OSPL_INTERRUPT_ARGUMENTS );
static void  R_DMAC_RM_IRQ_Handler5( R_OSPL_INTERRUPT_ARGUMENTS );
static void  R_DMAC_RM_IRQ_Handler6( R_OSPL_INTERRUPT_ARGUMENTS );
static void  R_DMAC_RM_IRQ_Handler7( R_OSPL_INTERRUPT_ARGUMENTS );
static void  R_DMAC_RM_IRQ_Handler8( R_OSPL_INTERRUPT_ARGUMENTS );
static void  R_DMAC_RM_IRQ_Handler9( R_OSPL_INTERRUPT_ARGUMENTS );
static void  R_DMAC_RM_IRQ_Handler10( R_OSPL_INTERRUPT_ARGUMENTS );
static void  R_DMAC_RM_IRQ_Handler11( R_OSPL_INTERRUPT_ARGUMENTS );
static void  R_DMAC_RM_IRQ_Handler12( R_OSPL_INTERRUPT_ARGUMENTS );
static void  R_DMAC_RM_IRQ_Handler13( R_OSPL_INTERRUPT_ARGUMENTS );
static void  R_DMAC_RM_IRQ_Handler14( R_OSPL_INTERRUPT_ARGUMENTS );
static void  R_DMAC_RM_IRQ_Handler15( R_OSPL_INTERRUPT_ARGUMENTS );


/***********************************************************************
* Variable: gs_IRQHandlers
************************************************************************/
static bsp_int_cb_t  gs_IRQHandlers[ R_DMAC_RM_CHANNEL_COUNT ] =
{
    &( R_DMAC_RM_IRQ_Handler0 ), /* MISRA 16.9 */
    &( R_DMAC_RM_IRQ_Handler1 ),
    &( R_DMAC_RM_IRQ_Handler2 ),
    &( R_DMAC_RM_IRQ_Handler3 ),
    &( R_DMAC_RM_IRQ_Handler4 ),
    &( R_DMAC_RM_IRQ_Handler5 ),
    &( R_DMAC_RM_IRQ_Handler6 ),
    &( R_DMAC_RM_IRQ_Handler7 ),
    &( R_DMAC_RM_IRQ_Handler8 ),
    &( R_DMAC_RM_IRQ_Handler9 ),
    &( R_DMAC_RM_IRQ_Handler10 ),
    &( R_DMAC_RM_IRQ_Handler11 ),
    &( R_DMAC_RM_IRQ_Handler12 ),
    &( R_DMAC_RM_IRQ_Handler13 ),
    &( R_DMAC_RM_IRQ_Handler14 ),
    &( R_DMAC_RM_IRQ_Handler15 )
};


/***********************************************************************
* Variable: gs_array_of_i_context
************************************************************************/
static const r_ospl_interrupt_t  gs_array_of_i_context[ R_DMAC_RM_CHANNEL_COUNT ] =
{
    { BSP_INT_SRC_DMAINT0,   0 },
    { BSP_INT_SRC_DMAINT1,   1 },
    { BSP_INT_SRC_DMAINT2,   2 },
    { BSP_INT_SRC_DMAINT3,   3 },
    { BSP_INT_SRC_DMAINT4,   4 },
    { BSP_INT_SRC_DMAINT5,   5 },
    { BSP_INT_SRC_DMAINT6,   6 },
    { BSP_INT_SRC_DMAINT7,   7 },
    { BSP_INT_SRC_DMAINT8,   8 },
    { BSP_INT_SRC_DMAINT9,   9 },
    { BSP_INT_SRC_DMAINT10, 10 },
    { BSP_INT_SRC_DMAINT11, 11 },
    { BSP_INT_SRC_DMAINT12, 12 },
    { BSP_INT_SRC_DMAINT13, 13 },
    { BSP_INT_SRC_DMAINT14, 14 },
    { BSP_INT_SRC_DMAINT15, 15 }
};


/***********************************************************************
* Variable: gs_array_of_error_i_context
************************************************************************/
static const r_ospl_interrupt_t  gs_array_of_error_i_context =
{
    BSP_INT_SRC_DMAERR,  GS_UNKNOWN_CHANNEL_NUM
};


/***********************************************************************
* Function: R_DMAC_RM_SetDefaultAsync
*
* Return Value:
*    None
************************************************************************/
void  R_DMAC_RM_SetDefaultAsync( r_ospl_async_t*  ref_Async,  r_ospl_async_type_t  in_AsyncType )
{
    IF_DQ( ref_Async == NULL )
    {
        goto fin;
    }

    R_OSPL_ASYNC_SetDefaultPreset( ref_Async );

    if ( IS_BIT_NOT_SET( ref_Async->Flags, R_F_OSPL_A_Thread ) )
    {
        ref_Async->A_Thread = R_OSPL_THREAD_NULL;
    }

    if ( IS_BIT_NOT_SET( ref_Async->Flags, R_F_OSPL_A_EventValue ) )
    {
        if ( in_AsyncType == R_OSPL_ASYNC_TYPE_NORMAL )
        {
            ref_Async->A_EventValue = R_OSPL_A_FLAG;
        }
        else
        {
            ref_Async->A_EventValue = R_OSPL_FINAL_A_FLAG;
        }
    }
    else
    {
        ASSERT_D( IS_BIT_SET( ref_Async->Flags, R_F_OSPL_A_Thread ), R_NOOP() );
    }

    if ( IS_BIT_NOT_SET( ref_Async->Flags, R_F_OSPL_I_Thread ) )
    {
        ref_Async->I_Thread = R_OSPL_THREAD_NULL;
    }

    if ( IS_BIT_NOT_SET( ref_Async->Flags, R_F_OSPL_I_EventValue ) )
    {
        if ( ref_Async->I_Thread == R_OSPL_THREAD_NULL )
        {
            ref_Async->I_EventValue = 0;
        }
        else
        {
            ref_Async->I_EventValue = R_OSPL_I_FLAG;
        }
    }
    else
    {
        ASSERT_D( IS_BIT_SET( ref_Async->Flags, R_F_OSPL_I_Thread ), R_NOOP() );
    }

    if ( IS_BIT_NOT_SET( ref_Async->Flags, R_F_OSPL_InterruptCallback ) )
    {
        ref_Async->InterruptCallback = &( R_DMAC_RM_OnInterruptDefault );    /* MISRA 16.9 */
    }

    ref_Async->Flags = R_F_OSPL_A_Thread | R_F_OSPL_A_EventValue |
                       R_F_OSPL_I_Thread | R_F_OSPL_I_EventValue |
                       R_F_OSPL_InterruptCallback | R_F_OSPL_Delegate;
fin:
    return;
}


/***********************************************************************
* Function: R_DMAC_RM_LockChannel
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_DMAC_RM_LockChannel( int_fast32_t const  dmac_channel, int_fast32_t* out_ChannelNum )
{
    return  R_OSPL_LockChannel( dmac_channel, out_ChannelNum,
                                BSP_LOCK_DMAC0, BSP_LOCK_DMAC15 );
}


/***********************************************************************
* Function: R_DMAC_RM_UnlockChannel
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_DMAC_RM_UnlockChannel( int_fast32_t const  dmac_channel, errnum_t e )
{
    return  R_OSPL_UnlockChannel( dmac_channel, e,
                                  BSP_LOCK_DMAC0, BSP_LOCK_DMAC15 );
}


/***********************************************************************
* Function: R_DMAC_RM_OnInitialize
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_DMAC_RM_OnInitialize( int_fast32_t ChannelNum )
{
    errnum_t       e;
    bsp_int_err_t  eb;
    bsp_int_src_t  num_of_IRQ = gs_array_of_i_context[ ChannelNum ].IRQ_Num;
    r_dmac_rm_pl_channel_t*  self = &gs_dmac_rm_pl_channel[ ChannelNum ];

    IF ( ChannelNum < 0  ||  ChannelNum >= R_DMAC_RM_CHANNEL_COUNT )
    {
        e=E_OTHERS;
        goto fin;
    }

    self->InterruptCallbackCaller = NULL;
    self->Status = NULL;

    eb= R_BSP_InterruptWrite( num_of_IRQ, gs_IRQHandlers[ ChannelNum ] );
    IF ( eb != BSP_INT_SUCCESS )
    {
        e=E_OTHERS;
        goto fin;
    }

    e= R_OSPL_SetInterruptPriority( num_of_IRQ, R_DMAC_RM_INTERRUPT_PRIORITY );
    IF ( e != 0 )
    {
        goto fin;
    }

    e=0;
fin:
    return  e;
}


/***********************************************************************
* Function: R_DMAC_RM_OnFinalize
*
* Return Value:
*    Error code, 0=No error and e=0
************************************************************************/
errnum_t  R_DMAC_RM_OnFinalize( int_fast32_t ChannelNum, errnum_t e )
{
    return  e;
}


/***********************************************************************
* Function: R_DMAC_RM_SetInterruptCallbackCaller
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_DMAC_RM_SetInterruptCallbackCaller( int_fast32_t ChannelNum,
        const r_ospl_caller_t* Caller )
{
    r_dmac_rm_pl_channel_t*  self = &gs_dmac_rm_pl_channel[ ChannelNum ];

    self->InterruptCallbackCaller = Caller;

    return  0;
}


/***********************************************************************
* Function: R_DMAC_RM_OnEnableInterrupt
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
void  R_DMAC_RM_OnEnableInterrupt( int_fast32_t ChannelNum, dmac_interrupt_lines_t Enables )
{
    bsp_int_err_t  eb;

    if ( IS_BIT_SET( Enables, DMAC_INTERRUPT_LINE_TRANSFER_END ) )
    {
        bsp_int_src_t  num_of_IRQ = gs_array_of_i_context[ ChannelNum ].IRQ_Num;

        eb= R_BSP_InterruptControl( num_of_IRQ, BSP_INT_CMD_INTERRUPT_ENABLE, FIT_NO_PTR );
        ASSERT_D( eb == 0,  R_NOOP() );
        R_UNREFERENCED_VARIABLE( eb ); /* for Release configuration */
    }

    if ( IS_BIT_SET( Enables, DMAC_INTERRUPT_LINE_ERROR ) )
    {
        bsp_int_src_t  num_of_IRQ;

        gs_dmac_rm_pl.ErrorInterruptEnables |= ( 1u << ChannelNum );

        num_of_IRQ = gs_array_of_error_i_context.IRQ_Num;
        eb= R_BSP_InterruptControl( num_of_IRQ, BSP_INT_CMD_INTERRUPT_ENABLE, FIT_NO_PTR );
        ASSERT_D( eb == 0,  R_NOOP() );
        R_UNREFERENCED_VARIABLE( eb ); /* for Release configuration */
    }
}


/***********************************************************************
* Function: R_DMAC_RM_OnDisableInterrupt
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
void  R_DMAC_RM_OnDisableInterrupt( int_fast32_t ChannelNum, dmac_interrupt_lines_t Disables1 )
{
    bsp_int_err_t  eb;

    if ( IS_BIT_SET( Disables1, DMAC_INTERRUPT_LINE_TRANSFER_END ) )
    {
        bsp_int_src_t   num_of_IRQ = gs_array_of_i_context[ ChannelNum ].IRQ_Num;

        eb= R_BSP_InterruptControl( num_of_IRQ, BSP_INT_CMD_INTERRUPT_DISABLE, FIT_NO_PTR );
        ASSERT_D( eb == 0,  R_NOOP() );
        R_UNREFERENCED_VARIABLE( eb ); /* for Release configuration */
    }

    if ( IS_BIT_SET( Disables1, DMAC_INTERRUPT_LINE_ERROR ) )
    {
        bsp_int_src_t  num_of_IRQ;

        gs_dmac_rm_pl.ErrorInterruptEnables &= ~( 1u << ChannelNum );

        if ( gs_dmac_rm_pl.ErrorInterruptEnables == 0 )
        {
            num_of_IRQ = gs_array_of_error_i_context.IRQ_Num;
            eb= R_BSP_InterruptControl( num_of_IRQ, BSP_INT_CMD_INTERRUPT_DISABLE, FIT_NO_PTR );
            ASSERT_D( eb == 0,  R_NOOP() );
            R_UNREFERENCED_VARIABLE( eb ); /* for Release configuration */
        }
    }
}


/***********************************************************************
* Function: R_DMAC_RM_OnInterruptDefault
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_DMAC_RM_OnInterruptDefault( const r_ospl_interrupt_t* InterruptSource,
                                        const r_ospl_caller_t* Caller )
{
    errnum_t  e;
    errnum_t  ee;
    r_ospl_async_t*  async;

    async = Caller->Async;

    e = 0;

    ee= R_DMAC_RM_OnInterrupting( InterruptSource );
    IF ( ee != 0  &&  e == 0 )
    {
        e = ee;
    }

    if ( async->I_Thread == R_OSPL_THREAD_NULL )
    {
        ee= R_DMAC_RM_OnInterrupted( R_OSPL_CALLER_GetRootChannelNum( Caller ) );
        IF ( ee != 0  &&  e == 0 )
        {
            e = ee;
        }
    }
    else
    {
        R_OSPL_EVENT_Set( async->I_Thread, async->I_EventValue );
    }

    return  e;
}


/***********************************************************************
* Function: R_DMAC_RM_IRQ_HandlerN
*
* Return Value:
*    None
************************************************************************/
void  R_DMAC_RM_IRQ_HandlerN( int_fast32_t ChannelNum )
{
    r_dmac_rm_pl_channel_t*  self = &gs_dmac_rm_pl_channel[ ChannelNum ];
    const r_ospl_interrupt_t*       i_context = &gs_array_of_i_context[ ChannelNum ];

    R_OSPL_CallInterruptCallback( self->InterruptCallbackCaller, i_context );
}


/***********************************************************************
* Function: R_DMAC_RM_IRQ_Handler0
*
* Return Value:
*    None
************************************************************************/
static void  R_DMAC_RM_IRQ_Handler0( R_OSPL_INTERRUPT_ARGUMENTS )
{
    R_DMAC_RM_IRQ_HandlerN( 0 );
    R_OSPL_END_OF_EDGE_INTERRUPT( BSP_INT_SRC_DMAINT0 );
}


/***********************************************************************
* Function: R_DMAC_RM_IRQ_Handler1
*
* Return Value:
*    None
************************************************************************/
static void  R_DMAC_RM_IRQ_Handler1( R_OSPL_INTERRUPT_ARGUMENTS )
{
    R_DMAC_RM_IRQ_HandlerN( 1 );
    R_OSPL_END_OF_EDGE_INTERRUPT( BSP_INT_SRC_DMAINT1 );
}


/***********************************************************************
* Function: R_DMAC_RM_IRQ_Handler2
*
* Return Value:
*    None
************************************************************************/
static void  R_DMAC_RM_IRQ_Handler2( R_OSPL_INTERRUPT_ARGUMENTS )
{
    R_DMAC_RM_IRQ_HandlerN( 2 );
    R_OSPL_END_OF_EDGE_INTERRUPT( BSP_INT_SRC_DMAINT2 );
}


/***********************************************************************
* Function: R_DMAC_RM_IRQ_Handler3
*
* Return Value:
*    None
************************************************************************/
static void  R_DMAC_RM_IRQ_Handler3( R_OSPL_INTERRUPT_ARGUMENTS )
{
    R_DMAC_RM_IRQ_HandlerN( 3 );
    R_OSPL_END_OF_EDGE_INTERRUPT( BSP_INT_SRC_DMAINT3 );
}


/***********************************************************************
* Function: R_DMAC_RM_IRQ_Handler4
*
* Return Value:
*    None
************************************************************************/
static void  R_DMAC_RM_IRQ_Handler4( R_OSPL_INTERRUPT_ARGUMENTS )
{
    R_DMAC_RM_IRQ_HandlerN( 4 );
    R_OSPL_END_OF_EDGE_INTERRUPT( BSP_INT_SRC_DMAINT4 );
}


/***********************************************************************
* Function: R_DMAC_RM_IRQ_Handler5
*
* Return Value:
*    None
************************************************************************/
static void  R_DMAC_RM_IRQ_Handler5( R_OSPL_INTERRUPT_ARGUMENTS )
{
    R_DMAC_RM_IRQ_HandlerN( 5 );
    R_OSPL_END_OF_EDGE_INTERRUPT( BSP_INT_SRC_DMAINT5 );
}


/***********************************************************************
* Function: R_DMAC_RM_IRQ_Handler6
*
* Return Value:
*    None
************************************************************************/
static void  R_DMAC_RM_IRQ_Handler6( R_OSPL_INTERRUPT_ARGUMENTS )
{
    R_DMAC_RM_IRQ_HandlerN( 6 );
    R_OSPL_END_OF_EDGE_INTERRUPT( BSP_INT_SRC_DMAINT6 );
}


/***********************************************************************
* Function: R_DMAC_RM_IRQ_Handler7
*
* Return Value:
*    None
************************************************************************/
static void  R_DMAC_RM_IRQ_Handler7( R_OSPL_INTERRUPT_ARGUMENTS )
{
    R_DMAC_RM_IRQ_HandlerN( 7 );
    R_OSPL_END_OF_EDGE_INTERRUPT( BSP_INT_SRC_DMAINT7 );
}


/***********************************************************************
* Function: R_DMAC_RM_IRQ_Handler8
*
* Return Value:
*    None
************************************************************************/
static void  R_DMAC_RM_IRQ_Handler8( R_OSPL_INTERRUPT_ARGUMENTS )
{
    R_DMAC_RM_IRQ_HandlerN( 8 );
    R_OSPL_END_OF_EDGE_INTERRUPT( BSP_INT_SRC_DMAINT8 );
}


/***********************************************************************
* Function: R_DMAC_RM_IRQ_Handler9
*
* Return Value:
*    None
************************************************************************/
static void  R_DMAC_RM_IRQ_Handler9( R_OSPL_INTERRUPT_ARGUMENTS )
{
    R_DMAC_RM_IRQ_HandlerN( 9 );
    R_OSPL_END_OF_EDGE_INTERRUPT( BSP_INT_SRC_DMAINT9 );
}


/***********************************************************************
* Function: R_DMAC_RM_IRQ_Handler10
*
* Return Value:
*    None
************************************************************************/
static void  R_DMAC_RM_IRQ_Handler10( R_OSPL_INTERRUPT_ARGUMENTS )
{
    R_DMAC_RM_IRQ_HandlerN( 10 );
    R_OSPL_END_OF_EDGE_INTERRUPT( BSP_INT_SRC_DMAINT10 );
}


/***********************************************************************
* Function: R_DMAC_RM_IRQ_Handler11
*
* Return Value:
*    None
************************************************************************/
static void  R_DMAC_RM_IRQ_Handler11( R_OSPL_INTERRUPT_ARGUMENTS )
{
    R_DMAC_RM_IRQ_HandlerN( 11 );
    R_OSPL_END_OF_EDGE_INTERRUPT( BSP_INT_SRC_DMAINT11 );
}


/***********************************************************************
* Function: R_DMAC_RM_IRQ_Handler12
*
* Return Value:
*    None
************************************************************************/
static void  R_DMAC_RM_IRQ_Handler12( R_OSPL_INTERRUPT_ARGUMENTS )
{
    R_DMAC_RM_IRQ_HandlerN( 12 );
    R_OSPL_END_OF_EDGE_INTERRUPT( BSP_INT_SRC_DMAINT12 );
}


/***********************************************************************
* Function: R_DMAC_RM_IRQ_Handler13
*
* Return Value:
*    None
************************************************************************/
static void  R_DMAC_RM_IRQ_Handler13( R_OSPL_INTERRUPT_ARGUMENTS )
{
    R_DMAC_RM_IRQ_HandlerN( 13 );
    R_OSPL_END_OF_EDGE_INTERRUPT( BSP_INT_SRC_DMAINT13 );
}


/***********************************************************************
* Function: R_DMAC_RM_IRQ_Handler14
*
* Return Value:
*    None
************************************************************************/
static void  R_DMAC_RM_IRQ_Handler14( R_OSPL_INTERRUPT_ARGUMENTS )
{
    R_DMAC_RM_IRQ_HandlerN( 14 );
    R_OSPL_END_OF_EDGE_INTERRUPT( BSP_INT_SRC_DMAINT14 );
}


/***********************************************************************
* Function: R_DMAC_RM_IRQ_Handler15
*
* Return Value:
*    None
************************************************************************/
static void  R_DMAC_RM_IRQ_Handler15( R_OSPL_INTERRUPT_ARGUMENTS )
{
    R_DMAC_RM_IRQ_HandlerN( 15 );
    R_OSPL_END_OF_EDGE_INTERRUPT( BSP_INT_SRC_DMAINT15 );
}


