/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2015 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/***********************************************************************
* File: r_dmac_lm.c
*    Direct Memory Access Controller (DMAC) - Link Mode Sample Driver.
*
* - $Version: $ (=PFV_VERSION)
* - $ModuleRevision: {Masters}\PFV_DMAC\PFV_DMAC\23_RStyled $
* - $Rev: $
* - $Date: 2017-08-24T18:32:46+09:00 $
************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include  "r_typedefs.h"
#include  "r_ospl.h"
#include  "iobitmasks/dmac_rm_iobitmask.h"
#include  "r_dmac_lm.h"
#include  "r_dmac_rm_pl.h"
#include  "r_dmac_rm_internal.h"
#include  "r_dmac_rm_inline.h"


/******************************************************************************
Typedef definitions
******************************************************************************/

/******************************************************************************
Macro definitions
******************************************************************************/

/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/

/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/

/******************************************************************************
Private global variables and functions
******************************************************************************/

/***********************************************************************
* Function: R_DMAC_LM_DESCRIPTOR_Initialize
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_DMAC_LM_DESCRIPTOR_Initialize( dmac_lm_descriptor_t* self,
        int_fast32_t const  dmac_channel,
        bit_flags32_t /*<dmac_lm_header_t>*/  header,
        dmac_register_mode_config_t*  config,
        uintptr_t  next_descriptor_physical_address )
{
    errnum_t  e;
    bit_flags_fast32_t  config_flags;


    IF_DQ( self == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }
    IF_DQ( config == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }

    config_flags = config->flags;


    /* Set to memory */
    /* Call "R_OSPL_MEMORY_Flush( R_OSPL_FLUSH_WRITEBACK_INVALIDATE );" at application */
    R_OSPL_MEMORY_Barrier();


    /* Check must flags */
    {
        static const bit_flags_fast32_t  must_flags =
            F_DMAC_NEXT0_SOURCE_PHYSICAL_ADDRESS |
            F_DMAC_NEXT0_DESTINATION_PHYSICAL_ADDRESS |
            F_DMAC_NEXT0_TRANSFER_BYTE;

        IF ( IS_ANY_BITS_NOT_SET( config_flags, must_flags ) )
        {
            e=E_OTHERS;
            goto fin;
        }
    }

    if ( IS_BIT_SET( config_flags, F_DMAC_REQUEST_RESOURCE_MID ) )
    {
        static const bit_flags_fast32_t  must_flags_rid =
            F_DMAC_REQUEST_RESOURCE_MID |
            F_DMAC_REQUEST_RESOURCE_RID |
            F_DMAC_TRANSFER_MODE_TM |
            F_DMAC_IS_DMA_ACK_OUTPUT_MODE_AM |
            F_DMAC_REQUEST_EDGE_OR_LEVEL_LVL |
            F_DMAC_IS_REQUEST_RAISING_OR_HIGH_HIEN |
            F_DMAC_IS_REQUEST_FALLING_OR_LOW_LOEN |
            F_DMAC_REQUEST_DIRECTION_REQD;

        IF ( IS_ANY_BITS_NOT_SET( config_flags, must_flags_rid ) )
        {
            e = E_OTHERS;
            goto  fin;
        }
    }


    /* Check not supported flags */
    {
        static const bit_flags_fast32_t  except_flags =
            F_DMAC_NEXT1_SOURCE_PHYSICAL_ADDRESS |
            F_DMAC_NEXT1_DESTINATION_PHYSICAL_ADDRESS |
            F_DMAC_NEXT1_TRANSFER_BYTE;

        IF ( IS_ANY_BITS_SET( config_flags, except_flags ) )
        {
            e=E_OTHERS;
            goto fin;
        }
    }


    /* Set default config */
    R_DMAC_RM_SetDefaultConfig_Sub( config );


    /* Set except for related with "CHCFG_n", "CHITVL_n", "CHEXT_n" register */
    self->header = header;
    self->source_physical_address      = (uintptr_t) config->next0_source_physical_address;
    self->destination_physical_address = (uintptr_t) config->next0_destination_physical_address;
    self->transfer_byte                = (uintptr_t) config->next0_transfer_byte;
    self->next_descriptor_link_address = next_descriptor_physical_address;


    /* Set related with "CHCFG_n", "CHITVL_n", "CHEXT_n" register */
    {
        uint32_t  value_of_CHCFG_n;
        uint32_t  value_of_CHITVL_n;
        uint32_t  value_of_CHEXT_n;

        e= R_DMAC_RM_GetRegisterValues_Sub( config, dmac_channel,
                                            &value_of_CHCFG_n, &value_of_CHITVL_n, &value_of_CHEXT_n );
        IF(e)
        {
            goto fin;
        }

        self->CHCFG_n  = value_of_CHCFG_n;
        self->CHITVL_n = value_of_CHITVL_n;
        self->CHEXT_n  = value_of_CHEXT_n;
    }


    /* Check not supported value */
    if ( IS_BIT_NOT_SET( header, F_DMAC_LM_LINK_END ) )
    {
        ASSERT_R( config->is_disable_next_end_interrupt,  e=E_OTHERS; goto fin );
        /* Because DMAC_RM interrupt callback function resets the DMAC. */
    }


    e=0;
fin:
    return  e;
}


/***********************************************************************
* Function: R_DMAC_RM_TransferAsLinkMode
*    See "R_DMAC_RM_TransferAsLinkModeStart"
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_DMAC_RM_TransferAsLinkMode( int_fast32_t const  dmac_channel,
                                        dmac_register_mode_config_t*  a_config,
                                        uintptr_t /*<dmac_lm_descriptor_t*>*/ const  first_descriptor_physical_address,
                                        dmac_lm_descriptor_config_t*  descriptor_config )
{
    errnum_t         e;
    errnum_t         ee;
    r_ospl_async_t   async;
    bit_flags32_t    got_flags;


    async.A_EventValue = 0;  /* For fin block */

    async.Flags = R_F_OSPL_A_Thread | R_F_OSPL_A_EventValue;
    e= R_OSPL_EVENT_Allocate(
           &async.A_Thread,  R_OSPL_THREAD_GetCurrentId(),
           &async.A_EventValue,  R_OSPL_UNUSED_FLAG );
    IF(e)
    {
        goto fin;
    }

    e= R_DMAC_RM_StartAsLinkMode( dmac_channel, a_config,
                                  first_descriptor_physical_address,  descriptor_config,  &async );
    IF(e)
    {
        goto fin;
    }

    e= R_OSPL_EVENT_Wait( async.A_EventValue, &got_flags, R_OSPL_INFINITE );
    IF(e)
    {
        goto fin;
    }

    e=0;
fin:
    ee= R_OSPL_EVENT_Free( &async.A_Thread, &async.A_EventValue );
    e= R_OSPL_MergeErrNum( e, ee );
    return  e;
}


/******************************************************************************
* Function: R_DMAC_RM_StartAsLinkMode
*
* Description:
*    "a_config" - Input is following variables only.
*    - a_config->flags
*    - a_config->request_resource_MID
*    - a_config->request_resource_RID
*
*    "descriptor_config" can be set NULL.
*
* Return Value:
*    Error code, 0=No error
******************************************************************************/
errnum_t  R_DMAC_RM_StartAsLinkMode( int_fast32_t const  dmac_channel,
                                     dmac_register_mode_config_t*  a_config,
                                     uintptr_t /*<dmac_lm_descriptor_t*>*/ const  first_descriptor_physical_address,
                                     dmac_lm_descriptor_config_t*  descriptor_config,
                                     r_ospl_async_t* const         Async )
{
    errnum_t            e;
    uint32_t            value;
    bool_t              was_enabled = false;
#if IODEFINE_H_VERSION >= 60
#ifdef __GNUC__
    struct st_dmac_n*                  regs_DMAC = NULL;
    /* NULL is avoid for warning that "-Wmaybe-uninitialized". */
#else
    struct st_dmac_n*                  regs_DMAC;
#endif
    struct st_dmars_mm*                regs_res_DMAC;
#else
    volatile struct st_dmac_n*         regs_DMAC;
    volatile struct st_dmars_mm*       regs_res_DMAC;
#endif
    dmac_lm_descriptor_config_t  descriptor_config_for_NULL;
    const dmars_mask_shift_t*    mask_shift_DMARS;
    r_dmac_rm_channel_t* const  self = &R_DMAC_RM_STATIC_GetChanelContexts()[ dmac_channel ];


    IF_DQ( a_config == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }
    IF_DQ( Async == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }

    ASSERT_R( self->async_status.State == R_OSPL_RUNNABLE,  e=E_STATE; goto fin );

    if ( descriptor_config == NULL )
    {
        descriptor_config = &descriptor_config_for_NULL;
        descriptor_config->flags = 0;
    }


    R_DMAC_RM_SetDefaultAsync( Async, R_OSPL_ASYNC_TYPE_NORMAL );
    Async->ReturnValue = 0;

    was_enabled = self->i_lock_v_table->Lock( self->i_lock );


    /* Set to memory */
    /* Call "R_OSPL_MEMORY_Flush( R_OSPL_FLUSH_WRITEBACK_INVALIDATE );" at application */
    R_OSPL_MEMORY_Barrier();


    /* Set "regs_DMAC" */
    e= DMAC_RM_STATIC_GetRegisterSet( dmac_channel, &regs_DMAC );
    IF(e!=0)
    {
        goto fin;
    }
    IF_DQ( regs_DMAC == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }

    /* Set "regs_res_DMAC" */
    e= DMAC_RM_STATIC_GetResouceRegisterSet( dmac_channel, &regs_res_DMAC, &mask_shift_DMARS );
    IF(e!=0)
    {
        goto fin;
    }
    IF_DQ( regs_res_DMAC == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }
    IF_DQ( mask_shift_DMARS == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }


    /* Set default config */
    if ( IS_BIT_NOT_SET( descriptor_config->flags, F_DMAC_AXI_CACHE_ATTRIBUTE ) )
    {
        descriptor_config->AXI_cache_attribute = R_OSPL_AXI_CACHE_ZERO;
    }

    if ( IS_BIT_NOT_SET( descriptor_config->flags, F_DMAC_AXI_PROTECTION ) )
    {
        descriptor_config->AXI_protection = R_OSPL_AXI_SECURE;
    }


    /* Write to "CHCFG_n" register */

    /* Set to "DMS" bit */
    R_DMAC_RM_SET_REGISTER_BIT_FIELD( &regs_DMAC->CHCFG_n, CHCFG_n, DMS,
                                      DMAC_T__DMS_LINK_MODE );  /* Other bits are set by descriptor */


    /* Write to "DCTRL_0_7", "DCTRL_8_15 register : for accessing to descriptor */
    {
        dmac_shared_config_t  shared;

        shared.flags = F_DMAC_LINK_WRITE_BACK_CACHE | F_DMAC_LINK_DESCRIPTOR_CACHE |
                       F_DMAC_LINK_WRITE_BACK_PROTECTION | F_DMAC_LINK_DESCRIPTOR_PROTECTION;
        shared.link_write_back_cache      = descriptor_config->AXI_cache_attribute;
        shared.link_write_back_protection = descriptor_config->AXI_protection;
        shared.link_descriptor_cache      = descriptor_config->AXI_cache_attribute;
        shared.link_descriptor_protection = descriptor_config->AXI_protection;

        e= R_DMAC_RM_SetSharedConfiguration( dmac_channel, &shared );
        IF(e)
        {
            goto fin;
        }
    }


    /* Write to "DMARSn" register */
    if ( IS_BIT_SET( a_config->flags, F_DMAC_REQUEST_RESOURCE_MID ) )
    {
        value = regs_res_DMAC->DMARS;
        e= R_DMAC_RM_GetDMARS_Values_Sub( a_config, mask_shift_DMARS, &value );
        IF(e)
        {
            goto fin;
        }
        regs_res_DMAC->DMARS = value;
    }


    /* Write to "NXLA_n" register */
    regs_DMAC->NXLA_n = first_descriptor_physical_address;


    /* Attach "Async" to interrupt */
    /* ->MISRA 11.4 : Not too big "enum" is same bit count as "int_t" */ /* ->SEC R2.7.1 */
    R_OSPL_CALLER_Initialize( &self->interrupt_callback_caller,
                              Async, (volatile int_t*) &self->async_status.State, R_OSPL_INTERRUPTING,
                              self->i_lock, self->i_lock_v_table );
    /* <-MISRA 11.4 */ /* <-SEC R2.7.1 */
    e= R_DMAC_RM_SetInterruptCallbackCaller( dmac_channel,
            &self->interrupt_callback_caller );
    IF(e!=0)
    {
        goto fin;
    }
    R_OSPL_FLAG32_Set( &self->async_status.InterruptEnables, DMAC_INTERRUPT_LINE_TRANSFER_END );


    /* Clear event flags */
    R_OSPL_EVENT_Clear( Async->A_Thread, Async->A_EventValue );
    R_OSPL_EVENT_Clear( Async->I_Thread, Async->I_EventValue );


    self->async_status.State = R_OSPL_RUNNING;


    /* Software reset DMA */
    R_DMAC_RM_SET_REGISTER_BIT_FIELD( &regs_DMAC->CHCTRL_n, CHCTRL_n, SWRST, TO_UNSIGNED( true ) );
#if  DMAC_PARAMETER_RESET_DELAY
    R_OSPL_Delay( 1 );
#endif


    /* Start DMA transfer */
    R_DMAC_RM_SET_REGISTER_BIT_FIELD( &regs_DMAC->CHCTRL_n, CHCTRL_n, SETEN, TO_UNSIGNED( true ) );


    /* Start DMA transfer by auto request mode */
    if ( IS_BIT_NOT_SET( a_config->flags, F_DMAC_REQUEST_RESOURCE_MID ) )
    {
        R_DMAC_RM_SET_REGISTER_BIT_FIELD( &regs_DMAC->CHCTRL_n, CHCTRL_n, STG, TO_UNSIGNED( true ) );
    }


    e = 0;
fin:
    if ( IS( was_enabled ) )
    {
        self->i_lock_v_table->Unlock( self->i_lock );
    }
    return  e;
}


/***********************************************************************
* Function: R_DMAC_RM_StopAsLinkMode
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_DMAC_RM_StopAsLinkMode( int_fast32_t const  dmac_channel )
{
    return  R_DMAC_RM_StopAsRegisterMode( dmac_channel );
}


