/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2016 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/***********************************************************************
* File: ospl_config.h
*    Configuration of OSPL.
*
* - $Version: $ (=R_OSPL_VERSION)
* - $ModuleRevision: {Masters}\OSPL\ALL_DriverForRelease\24_RStyled $
* - $Rev: $
* - $Date: 2017-07-21T18:38:25+09:00 $
************************************************************************/

#ifndef R_OSPL_CONFIG_H
#define R_OSPL_CONFIG_H


/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include  "use_list.h"
#include  "platform_config.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/******************************************************************************
Typedef definitions
******************************************************************************/

/******************************************************************************
Macro definitions
******************************************************************************/

/***********************************************************************
* Define: R_OSPL_NDEBUG
*    Debug configuration or Release configuration.
*
* Description:
*    The value can be changed by OSPL user, if it was supported.
*    Define (=Release) or not define (=Debug).
*    This is same as "NDEBUG" of standard library.
*    The system can run with the debug configuration OSPL and the release
*    configuration application.
*    If the library (compiled binary) called the debug configuration OSPL,
*    compile the OSPL source with debug configuration.
************************************************************************/
#ifndef  R_OSPL_NDEBUG_DEFINED

#define  R_OSPL_NDEBUG

#define  R_OSPL_NDEBUG_DEFINED
#endif


/***********************************************************************
* Constant: R_OSPL_VERSION
*    Version number of OSPL
*
* Description:
*    The value can be changed by OSPL supplier.
*    101 = version 1.01.
*    Hundreds place is version number of OSPL specification.
*    Tens place and one's place are minor version number in specified OS and board.
************************************************************************/
#define  R_OSPL_VERSION  160


/***********************************************************************
* Constant: R_OSPL_VERSION_STRING
*    String of version number of OSPL.
*
* Description:
*    The value can be changed by OSPL supplier.
************************************************************************/
#define  R_OSPL_VERSION_STRING  "1.60"


/***********************************************************************
* Constant: R_OSPL_IS_PREEMPTION
*    Whether preemptive RTOS or not.
*
* Description:
*    The value can be changed by OSPL supplier.
*    The value is 1 or 0.
*    This value is 0, if the environment was OS less.
*    It is necessary to configure to pseudo multithreading, if this value was 0.
************************************************************************/
#define  R_OSPL_IS_PREEMPTION   BSP_CFG_RTOS_USED


/***********************************************************************
* Define: R_OSPL_FOR_FREE_RTOS
*
* Description:
*    The value can be changed by OSPL provider.
*    This macro is for header files of OSPL.
*    The value is version number.
************************************************************************/
#ifdef  USE_LIST_FREE_RTOS
#define  R_OSPL_FOR_FREE_RTOS  80102
#endif


/***********************************************************************
* Define: R_OSPL_FOR_RTX
*
* Description:
*    The value can be changed by OSPL provider.
*    This macro is for header files of OSPL.
*    The value is version number.
************************************************************************/
#ifdef  USE_LIST_RTX
#define  R_OSPL_FOR_RTX  ((4<<16)|74)
#endif


/***********************************************************************
* Define: R_OSPL_FOR_MBED
*
* Description:
*    The value can be changed by OSPL provider.
*    This macro is for header files of OSPL.
************************************************************************/
#ifdef  USE_LIST_MBED
#define  R_OSPL_FOR_MBED
#endif


/***********************************************************************
* Define: R_OSPL_FOR_RZ_A1_BSP
*
* Description:
*    The value can be changed by OSPL provider.
*    This macro is for header files of OSPL.
*    The value is version number.
************************************************************************/
#if  defined( USE_LIST_RZA1H_BSP )
#define  R_OSPL_FOR_RZ_A1_BSP  203
#endif


/***********************************************************************
* Define: R_OSPL_FOR_RZ_A1_OS_LESS
*
* Description:
*    The value can be changed by OSPL provider.
*    This macro is for header files of OSPL.
*    The value is version number.
************************************************************************/
#if  defined( USE_LIST_RZA1H_OS_LESS )
#define  R_OSPL_FOR_RZ_A1_OS_LESS  101
#endif


/***********************************************************************
* Define: R_OSPL_FOR_RZ_A1_OS_LESS_RSK
*
* Description:
*    The value can be changed by OSPL provider.
*    This macro is for header files of OSPL.
*    The value is version number.
************************************************************************/
#if  defined( USE_LIST_RZA1H_OS_LESS_RSK )
#define  R_OSPL_FOR_RZ_A1_OS_LESS_RSK  200
#endif


/***********************************************************************
* Define: R_OSPL_FOR_RZ_A1_INTC
*
* Description:
*    The value can be changed by OSPL provider.
*    This macro is for header files of OSPL.
*    The value is version number.
************************************************************************/
#if  defined( USE_LIST_RZA1H_OS_LESS ) || defined( USE_LIST_FREE_RTOS )
#define  R_OSPL_FOR_RZ_A1_INTC  101
#endif


/***********************************************************************
* Define: R_OSPL_FOR_RZ_A1_INTC_RSK
*
* Description:
*    The value can be changed by OSPL provider.
*    This macro is for header files of OSPL.
*    The value is version number.
************************************************************************/
#if  defined( USE_LIST_RZA1H_OS_LESS_RSK )
#define  R_OSPL_FOR_RZ_A1_INTC_RSK  200
#endif


/***********************************************************************
* Define: R_OSPL_FOR_RZ_A1_CACHE
*
* Description:
*    The value can be changed by OSPL provider.
*    This macro is for header files of OSPL.
*    The value is version number.
************************************************************************/
#if  defined( USE_LIST_RZA1H_OS_LESS ) || defined( USE_LIST_FREE_RTOS )
#define  R_OSPL_FOR_RZ_A1_CACHE  101
#endif


/***********************************************************************
* Define: R_OSPL_HEAP_DEBUG_FILL
*    Fills in allocated or freed heap memory.
*
* Description:
*    The value can be changed by OSPL user, if it was supported.
*    The value is 1 or 0.
************************************************************************/
#ifdef  R_OSPL_FOR_FREE_RTOS
#ifndef  R_OSPL_HEAP_DEBUG_FILL
#ifndef  R_OSPL_NDEBUG
#define  R_OSPL_HEAP_DEBUG_FILL  1
#else
#define  R_OSPL_HEAP_DEBUG_FILL  0
#endif
#endif
#endif


/***********************************************************************
* Define: R_OSPL_ERROR_HANDLING_CODE
*    Whether code of error break and error handling is enabled.
*
* Description:
*    The value can be changed by OSPL user, if it was supported.
*    The value is 1 or 0.
*    Sub options are <R_OSPL_ERROR_BREAK>, <R_OSPL_STACK_CHECK_CODE>,
*    <R_OSPL_TLS_ERROR_CODE>, <R_OSPL_DEBUG_THREAD_COUNT>,
*    <R_OSPL_DEBUG_EVENT_OBJECT_COUNT>, <R_OSPL_DQ>.
************************************************************************/
#ifndef  R_OSPL_ERROR_HANDLING_CODE
#define  R_OSPL_ERROR_HANDLING_CODE  1
#endif


/***********************************************************************
* Define: R_OSPL_ERROR_BREAK
*    Whether it is supported to break, when error was raised.
*
* Description:
*    The value can be changed by OSPL user, if it was supported.
*    The value is 1 or 0.
************************************************************************/
#if  R_OSPL_ERROR_HANDLING_CODE
#ifndef  R_OSPL_ERROR_BREAK
#if  defined( R_OSPL_NDEBUG )
#define  R_OSPL_ERROR_BREAK  0
#else
#define  R_OSPL_ERROR_BREAK  1
#endif
#endif
#endif


/***********************************************************************
* Define: R_OSPL_PRINTF
*    Printf output device from OSPL by OSPL user.
*
* Description:
*    The value of "R_OSPL_PRINTF" can be changed to the following values.
*    : R_OSPL_PRINTF_ENABLED    - 1
*    : R_OSPL_PRINTF_DISABLED   - 0
*    : R_OSPL_PRINTF_TO_INT_LOG - 2
************************************************************************/
#ifndef  R_OSPL_PRINTF
#ifdef  R_OSPL_NDEBUG
#define  R_OSPL_PRINTF  R_OSPL_PRINTF_DISABLED
#else
#define  R_OSPL_PRINTF  R_OSPL_PRINTF_ENABLED
#endif
#endif
#define  R_OSPL_PRINTF_ENABLED     1
#define  R_OSPL_PRINTF_DISABLED    0
#define  R_OSPL_PRINTF_TO_INT_LOG  2


/***********************************************************************
* Define: R_OSPL_EASY_ERROR_CHECK_CODE
*    Whether it is supported easy error check
*
* Description:
*    The value can be changed by OSPL user.
*    The value is 1 or 0.
************************************************************************/
#define  R_OSPL_EASY_ERROR_CHECK_CODE  1


/***********************************************************************
* Define: R_OSPL_TLS_ERROR_CODE
*    Whether it is supported that error code is stored in thread local storage
*
* Description:
*    The value can be changed by OSPL user, if it was supported.
*    The value is 1 or 0.
*    This value must be 1 by some application or some library with OSPL.
************************************************************************/
#if  R_OSPL_ERROR_HANDLING_CODE
#ifndef  R_OSPL_TLS_ERROR_CODE
#define  R_OSPL_TLS_ERROR_CODE  1
#endif
#endif


/***********************************************************************
* Define: R_OSPL_STACK_CHECK_CODE
*    Whether it is supported stack check
*
* Description:
*    The value can be changed by OSPL user, if it was supported.
*    The value is 1 or 0.
*    This value must be 1 by some application or some library with OSPL.
*    OSPL checks it, when "IF" macro was called.
*    RTX checks it, when context was switched, even if any setting.
*    FreeRTOS checks it, when context was switched, if "configCHECK_FOR_STACK_OVERFLOW = 1".
************************************************************************/
#if  R_OSPL_ERROR_HANDLING_CODE
#ifndef  R_OSPL_STACK_CHECK_CODE
#ifndef R_OSPL_NDEBUG
#define  R_OSPL_STACK_CHECK_CODE  1
#else
#define  R_OSPL_STACK_CHECK_CODE  0
#endif
#endif
#endif


/***********************************************************************
* Define: R_OSPL_FIT_BSP_CODE
*    Whether code of FIT BSP convertible API is enabled.
*
* Description:
*    The value can be changed by OSPL user, if it was supported.
*    The value is 1 or 0.
************************************************************************/
#ifndef  R_OSPL_FIT_BSP_CODE
#define  R_OSPL_FIT_BSP_CODE  1
#endif


/***********************************************************************
* Define: R_OSPL_EVENT_CODE
*    Whether code of thread attached event flags is enabled.
*
* Description:
*    The value can be changed by OSPL user, if it was supported.
*    The value is 1 or 0.
*    Sub options are <R_OSPL_EVENT_WATCH>, <R_OSPL_TLS_EVENT_CODE>, <R_OSPL_DETECT_BAD_EVENT>,
*    <R_OSPL_EVENT_MUST_ALLOCATE>, <R_OSPL_ALL_EVENT_ALLOCATE>, <R_OSPL_EVENT_GROUP_CODE>,
*    <R_OSPL_EVENT_OBJECT_CODE>.
************************************************************************/
#ifndef  R_OSPL_EVENT_CODE
#define  R_OSPL_EVENT_CODE  1
#endif


/***********************************************************************
* Define: R_OSPL_TLS_EVENT_CODE
*    Whether it is enabled to manage event using.
*
* Description:
*    The value can be changed by OSPL user, if it was supported.
*    The value is 1 or 0.
*    This value must be 1 by some application or some library with OSPL.
************************************************************************/
#if  R_OSPL_EVENT_CODE
#ifndef  R_OSPL_TLS_EVENT_CODE
#if  R_OSPL_TLS_ERROR_CODE
#define  R_OSPL_TLS_EVENT_CODE  1
#else
#define  R_OSPL_TLS_EVENT_CODE  0
#endif
#endif
#endif


/***********************************************************************
* Define: R_OSPL_DETECT_BAD_EVENT
*    Whether it is enabled detecting bad event.
*
* Description:
*    The value can be changed by OSPL user, if it was supported.
*    The value is 1 or 0.
*    This value must be 1 by some application or some library with OSPL.
************************************************************************/
#if  R_OSPL_EVENT_CODE
#ifndef  R_OSPL_DETECT_BAD_EVENT
#if  R_OSPL_ERROR_BREAK  &&  R_OSPL_TLS_EVENT_CODE
#define  R_OSPL_DETECT_BAD_EVENT  1
#else
#define  R_OSPL_DETECT_BAD_EVENT  0
#endif
#endif
#endif


/***********************************************************************
* Define: R_OSPL_EVENT_GROUP_CODE
*    Whether it is enabled event group.
*
* Description:
*    The value can be changed by OSPL supplier.
*    The value is 1 or 0.
*    This value must be 1 by some application or some library with OSPL.
************************************************************************/
#if  R_OSPL_EVENT_CODE
#ifndef  R_OSPL_EVENT_GROUP_CODE
#ifdef  R_OSPL_FOR_FREE_RTOS
#define  R_OSPL_EVENT_GROUP_CODE  1
#else
#define  R_OSPL_EVENT_GROUP_CODE  0
#endif
#endif
#endif

#if  R_OSPL_EVENT_GROUP_CODE
#if  ! R_OSPL_TLS_ERROR_CODE
#error
#endif
#endif


/***********************************************************************
* Define: R_OSPL_EVENT_MUST_ALLOCATE
*    Whether event must be allocated.
*
* Description:
*    The value can be changed by OSPL user, if it was supported.
*    The value must be 1, if R_OSPL_EVENT_GROUP_CODE == 1.
*    The value is 1 or 0.
************************************************************************/
#if  R_OSPL_EVENT_CODE
#if  R_OSPL_EVENT_GROUP_CODE
#ifndef  R_OSPL_EVENT_MUST_ALLOCATE
#define  R_OSPL_EVENT_MUST_ALLOCATE  1
#elif  R_OSPL_EVENT_MUST_ALLOCATE != 1
#error
#endif
#elif  R_OSPL_DETECT_BAD_EVENT
#ifndef  R_OSPL_EVENT_MUST_ALLOCATE
#define  R_OSPL_EVENT_MUST_ALLOCATE  1
#endif
#endif
#endif


/***********************************************************************
* Define: R_OSPL_ALL_EVENT_ALLOCATE
*    Whether all events can be allocated, when check bits are not used.
*
* Description:
*    The value can be changed by OSPL supplier.
*    The value is 1 or 0.
************************************************************************/
#if  R_OSPL_EVENT_CODE
#ifndef  R_OSPL_ALL_EVENT_ALLOCATE
#ifdef  R_OSPL_FOR_FREE_RTOS
#define  R_OSPL_ALL_EVENT_ALLOCATE  1
#else
#define  R_OSPL_ALL_EVENT_ALLOCATE  0
#endif
#endif
#endif


/***********************************************************************
* Define: R_OSPL_BASE_INTERRUPT_CODE
*    Whether code of base interrupt is enabled.
*
* Description:
*    The value can be changed by OSPL user, if it was supported.
*    The value is 1 or 0.
************************************************************************/
#ifndef  R_OSPL_BASE_INTERRUPT_CODE
#define  R_OSPL_BASE_INTERRUPT_CODE  1
#endif


/***********************************************************************
* Define: R_OSPL_INTERRUPT_HANDLER_IS
*    Whether interrupt handler is for RZ/A1 INTC.
*
* Description:
*    The value can be changed by OSPL supplier.
*
*    : R_OSPL_INTERRUPT_HANDLER_IS_GENERAL        - 0
*    : R_OSPL_INTERRUPT_HANDLER_IS_WITH_INT_SENSE - 2
*    : R_OSPL_INTERRUPT_HANDLER_IS_FOR_RTX        - 3
*
* R_OSPL_INTERRUPT_HANDLER_IS_GENERAL:
*    Interrupt handler does not have any arguments.
*    Interrupt handler does not return any values.
*    No API have to be called at last of interrupt handler.
*
* R_OSPL_INTERRUPT_HANDLER_IS_WITH_INT_SENSE:
*    With int_sence argument in interrupt handler.
*
* R_OSPL_INTERRUPT_HANDLER_IS_FOR_RTX:
*    GIC_EndInterrupt must be called at last of interrupt handler.
************************************************************************/
#if  defined( USE_LIST_RZA1H_OS_LESS ) || defined( USE_LIST_RZA1H_OS_LESS_RSK ) || defined( USE_LIST_FREE_RTOS )
#define  R_OSPL_INTERRUPT_HANDLER_IS  R_OSPL_INTERRUPT_HANDLER_IS_WITH_INT_SENSE
#elif  defined( USE_LIST_RTX )
#define  R_OSPL_INTERRUPT_HANDLER_IS  R_OSPL_INTERRUPT_HANDLER_IS_FOR_RTX
#else
#define  R_OSPL_INTERRUPT_HANDLER_IS  R_OSPL_INTERRUPT_HANDLER_IS_GENERAL
#endif
#define  R_OSPL_INTERRUPT_HANDLER_IS_GENERAL         0
#define  R_OSPL_INTERRUPT_HANDLER_IS_WITH_INT_SENSE  2
#define  R_OSPL_INTERRUPT_HANDLER_IS_FOR_RTX         3


/***********************************************************************
* Define: R_OSPL_INTERRUPT_ENTRY_CODE
*    Whether code of interrupt entry point is enabled.
*
* Description:
*    The value can be changed by OSPL user, if it was supported.
*    The value is 1 or 0.
************************************************************************/
#ifndef  R_OSPL_INTERRUPT_ENTRY_CODE
#define  R_OSPL_INTERRUPT_ENTRY_CODE  1
#endif

#if  R_OSPL_INTERRUPT_ENTRY_CODE
#if  ! R_OSPL_EVENT_CODE
#error
#endif
#endif


/***********************************************************************
* Define: R_OSPL_DELAY_CODE
*    Whether code of delay specified time is enabled.
*
* Description:
*    The value can be changed by OSPL user, if it was supported.
*    The value is 1 or 0.
************************************************************************/
#ifndef  R_OSPL_DELAY_CODE
#define  R_OSPL_DELAY_CODE  1
#endif


/***********************************************************************
* Define: R_OSPL_FREE_RUN_TIMER_CODE
*    Whether code of delay specified time is enabled.
*
* Description:
*    The value can be changed by OSPL user, if it was supported.
*    The value is 1 or 0.
************************************************************************/
#ifndef  R_OSPL_FREE_RUN_TIMER_CODE
#define  R_OSPL_FREE_RUN_TIMER_CODE  1
#endif


/***********************************************************************
* Define: R_OSPL_THREAD_CODE
*    Whether code of current thread and pseudo multi thread is enabled.
*
* Description:
*    The value can be changed by OSPL user, if it was supported.
*    The value is 1 or 0.
*    Sub options are <R_OSPL_WAIT_OWNER_DEBUG>, <R_OSPL_CPU_LOAD>.
************************************************************************/
#ifndef  R_OSPL_THREAD_CODE
#define  R_OSPL_THREAD_CODE  1
#endif


/***********************************************************************
* Define: R_OSPL_QUEUE_CODE
*    Whether code of message queue is enabled.
*
* Description:
*    The value can be changed by OSPL user, if it was supported.
*    The value is 1 or 0.
************************************************************************/
#ifndef  R_OSPL_QUEUE_CODE
#define  R_OSPL_QUEUE_CODE  1
#endif


/***********************************************************************
* Define: R_OSPL_C_LOCK_CODE
*    Whether code of locking to context and thread is enabled.
*
* Description:
*    The value can be changed by OSPL user, if it was supported.
*    The value is 1 or 0.
************************************************************************/
#ifndef  R_OSPL_C_LOCK_CODE
#define  R_OSPL_C_LOCK_CODE  1
#endif


/***********************************************************************
* Define: R_OSPL_I_LOCK_CODE
*    Whether code of locking interrupt is enabled.
*
* Description:
*    The value can be changed by OSPL user, if it was supported.
*    The value is 1 or 0.
************************************************************************/
#ifndef  R_OSPL_I_LOCK_CODE
#define  R_OSPL_I_LOCK_CODE  1
#endif


/***********************************************************************
* Define: R_OSPL_INTERNAL_T_LOCK_CODE
*    Whether code of internal thread lock is enabled.
*
* Description:
*    The value can be changed by OSPL user, if it was supported.
*    The value is 1 or 0.
************************************************************************/
#ifndef  R_OSPL_INTERNAL_T_LOCK_CODE
#define  R_OSPL_INTERNAL_T_LOCK_CODE  1
#endif


/***********************************************************************
* Define: R_OSPL_MEMORY_CONTROL_CODE
*    Whether code of memory address and flushing memory cache is enabled.
*
* Description:
*    The value can be changed by OSPL user, if it was supported.
*    The value is 1 or 0.
************************************************************************/
#ifndef  R_OSPL_MEMORY_CONTROL_CODE
#define  R_OSPL_MEMORY_CONTROL_CODE  1
#endif


/***********************************************************************
* Define: R_OSPL_TABLE_CODE
*    Whether code of table of array index is enabled.
*
* Description:
*    The value can be changed by OSPL user, if it was supported.
*    The value is 1 or 0.
************************************************************************/
#ifndef  R_OSPL_TABLE_CODE
#define  R_OSPL_TABLE_CODE  1
#endif


/***********************************************************************
* Define: R_OSPL_BASE_ERROR_HANDLING_CODE
*    Whether code of base error handling is enabled.
*
* Description:
*    The value can be changed by OSPL user, if it was supported.
*    The value is 1 or 0.
************************************************************************/
#ifndef  R_OSPL_BASE_ERROR_HANDLING_CODE
#define  R_OSPL_BASE_ERROR_HANDLING_CODE  1
#endif


/***********************************************************************
* Define: R_OSPL_STATIC_ANALYZING_CODE
*    Whether code of static assertion and disabling bad static analyzing is enabled.
*
* Description:
*    The value can be changed by OSPL user, if it was supported.
*    The value is 1 or 0.
************************************************************************/
#ifndef  R_OSPL_STATIC_ANALYZING_CODE
#define  R_OSPL_STATIC_ANALYZING_CODE  1
#endif


/***********************************************************************
* Define: R_OSPL_COMPILER_SPECIFIC_CODE
*    Whether code of abstruct of compiler specific syntax and embedded function is enabled.
*
* Description:
*    The value can be changed by OSPL user, if it was supported.
*    The value is 1 or 0.
************************************************************************/
#ifndef  R_OSPL_COMPILER_SPECIFIC_CODE
#define  R_OSPL_COMPILER_SPECIFIC_CODE  1
#endif


/***********************************************************************
* Define: R_OSPL_BASIC_TYPES_CODE
*    Whether code of basic types like bit flags and so on is enabled.
*
* Description:
*    The value can be changed by OSPL user, if it was supported.
*    The value is 1 or 0.
************************************************************************/
#ifndef  R_OSPL_BASIC_TYPES_CODE
#define  R_OSPL_BASIC_TYPES_CODE  1
#endif


/***********************************************************************
* Define: R_OSPL_REGISTER_ACCESS_CODE
*    Whether code of accessing to register is enabled.
*
* Description:
*    The value can be changed by OSPL user, if it was supported.
*    The value is 1 or 0.
************************************************************************/
#ifndef  R_OSPL_REGISTER_ACCESS_CODE
#define  R_OSPL_REGISTER_ACCESS_CODE  1
#endif


/***********************************************************************
* Define: R_OSPL_EVENT_OBJECT_CODE
*    Whether it is enabled event object.
*
* Description:
*    This specification is removed.
*    The value can be changed by OSPL user, if it was supported.
*    The value is 1 or 0.
*    This value must be 1 by some application or some library with OSPL.
************************************************************************/
#define  R_OSPL_EVENT_OBJECT_CODE  1
#ifndef  R_OSPL_EVENT_OBJECT_CODE
#if  R_OSPL_EVENT_CODE
#if  R_OSPL_VERSION <= 96
#define  R_OSPL_EVENT_OBJECT_CODE  1
#else
#define  R_OSPL_EVENT_OBJECT_CODE  0
#endif
#endif
#endif

#if  R_OSPL_EVENT_OBJECT_CODE
#if  ! R_OSPL_TLS_ERROR_CODE
#error
#endif
#endif


/***********************************************************************
* Define: R_OSPL_CPU_LOAD
*    Whether it is supported to measure CPU load (for OS less)
*
* Description:
*    The value can be changed, if <R_OSPL_IS_PREEMPTION> was 0.
*    The value is 1 or 0.
************************************************************************/
#if  R_OSPL_THREAD_CODE
#ifndef  R_OSPL_CPU_LOAD
#if  ! R_OSPL_IS_PREEMPTION
#define  R_OSPL_CPU_LOAD  1
#else
#define  R_OSPL_CPU_LOAD  0
#endif
#endif
#endif


/***********************************************************************
* Define: R_OSPL_LIBRARY_MAKING
*    Whether current project makes library.
*
* Description:
*    The value can be changed.
*    The value is 1 or 0.
************************************************************************/
#ifndef  R_OSPL_LIBRARY_MAKING
#define  R_OSPL_LIBRARY_MAKING  0
#endif


/***********************************************************************
* Constant: R_BOOL_IS_SIGNED
*    Whether compiler defines that bool_t type is signed.
*
* Description:
*    The value can be changed.
*    The value is 1 or 0.
************************************************************************/
#if  R_OSPL_BASIC_TYPES_CODE
#define  R_BOOL_IS_SIGNED  1
#endif


/***********************************************************************
* Define: R_OSPL_SUPPORT_EVENT_GET
*    Whether event flag function supports getting event.
*
* Description:
*    It is recommended to not support this for portability.
*    The value is 1 or 0.
************************************************************************/
#if  R_OSPL_EVENT_CODE
#define  R_OSPL_SUPPORT_EVENT_GET  0
#endif


/***********************************************************************
* Define: R_OSPL_BIT_FIELD_ACCESS_MACRO
*    Whether bit field access function is provided as macro or function.
*
* Description:
*    The value can be changed.
*    The value is 1 (=macro) or 0 (=function).
************************************************************************/
#if  R_OSPL_BASIC_TYPES_CODE
#define  R_OSPL_BIT_FIELD_ACCESS_MACRO  1
#endif


/***********************************************************************
* Define: R_OSPL_FTIMER_IS
*    Which channel of timer to use.
*
*    : R_OSPL_FTIMER_IS_OSTM0    - OSTM0
*    : R_OSPL_FTIMER_IS_OSTM1    - OSTM1
*    : R_OSPL_FTIMER_IS_MTU2_1_2 - MTU2 ch1 and ch2
*
* Description:
*    The value can be changed.
*    The symbol of value is depend on target timer.
************************************************************************/
#if  R_OSPL_FREE_RUN_TIMER_CODE

#if  defined( R_OSPL_FOR_RZ_A1_OS_LESS ) || defined( R_OSPL_FOR_FREE_RTOS ) || \
			defined( R_OSPL_FOR_RZ_A1_OS_LESS_RSK )
#define  R_OSPL_FTIMER_IS        R_OSPL_FTIMER_IS_OSTM1
#elif  defined( R_OSPL_FOR_RTX )
#define  R_OSPL_FTIMER_IS        R_OSPL_FTIMER_IS_MTU2_1_2
#else
#error
#endif

#define  R_OSPL_FTIMER_IS_OSTM0     0
#define  R_OSPL_FTIMER_IS_OSTM1     1
#define  R_OSPL_FTIMER_IS_MTU2_1_2  2
#define  R_OSPL_FTIMER_IS_NOT_USED  9

#ifdef  R_OSPL_FOR_RTX
#if R_OSPL_FTIMER_IS == R_OSPL_FTIMER_IS_OSTM0
#error  OSTM0 is used by RTX
#endif
#endif

#ifdef  R_OSPL_FOR_FREE_RTOS
#if R_OSPL_FTIMER_IS == R_OSPL_FTIMER_IS_OSTM0
#error  OSTM0 is used by FreeRTOS. See "vConfigureTickInterrupt".
#endif

#if R_OSPL_FTIMER_IS == R_OSPL_FTIMER_IS_MTU2_1_2
#error  MTU2 is used by FreeRTOS. See "init_switch_debounce_timer".
#endif
#endif
#endif


/******************************************************************************
Variable Externs
******************************************************************************/

/******************************************************************************
Functions Prototypes
******************************************************************************/

/***********************************************************************
* End of File:
************************************************************************/
#ifdef __cplusplus
}  /* extern "C" */
#endif /* __cplusplus */

#endif /* R_OSPL_H */

