/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2015 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/***********************************************************************
* File: r_pfv_typedef.c
*    Pixel Format Converter (PFV) Sample Driver.
*
* - $Version: $ (=PFV_VERSION)
* - $ModuleRevision: {Masters}\PFV_DMAC\PFV_DMAC\23_RStyled $
* - $Rev: $
* - $Date: 2017-08-24T18:32:46+09:00 $
************************************************************************/

#ifndef R_PFV_TYPEDEF_H
#define R_PFV_TYPEDEF_H

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include  "r_pfv_user.h"


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/******************************************************************************
Macro definitions
******************************************************************************/

/***********************************************************************
* Macro: PFV_VERSION
************************************************************************/
#define  PFV_VERSION  103  /*[PFV_VERSION]*/


/***********************************************************************
* Macro: PFV_VERSION_STRING
************************************************************************/
#define  PFV_VERSION_STRING  "1.03"


/***********************************************************************
* Constant: PFV_INPUT_FIFO_FULL_BYTE
*    PFV.PFVIFSR
************************************************************************/
enum {
    PFV_INPUT_FIFO_FULL_BYTE  = 32
};


/***********************************************************************
* Constant: PFV_OUTPUT_FIFO_FULL_BYTE
*    PFV.PFVOFSR
************************************************************************/
enum {  PFV_OUTPUT_FIFO_FULL_BYTE = 32 };


/***********************************************************************
* Constants:
*
*    : PFV_CHANNEL_MIN   - .
*    : PFV_CHANNEL_MAX   - .
*    : PFV_CHANNEL_COUNT - .
************************************************************************/
enum
{
    PFV_CHANNEL_MIN = 0,
    PFV_CHANNEL_MAX = 1,
    PFV_CHANNEL_COUNT = PFV_CHANNEL_MAX - PFV_CHANNEL_MIN + 1
};
R_STATIC_ASSERT_GLOBAL( PFV_CHANNEL_MIN == 0, "" );
/* Range check of minimun value is been comment out, because QAC of MISRA-C:2004 Rule 13.7 */


/******************************************************************************
Typedef definitions
******************************************************************************/

/***********************************************************************
* Enumeration: pfv_format_t
************************************************************************/
typedef enum
{
    /* Value of "pfv_config_t::input_format", "pfv_config_t::output_format" */
    /* Value of "IFMT", "OFMT" */
    PFV_RGB888    = 0,  /* I/O bit count = 32. Byte per pixel = 4 */
    PFV_ARGB8888  = 0,  /* I/O bit count = 32. Byte per pixel = 4 */
    PFV_RGB565    = 1,  /* I/O bit count = 16. Byte per pixel = 2 */
    PFV_YCbCr422  = 3   /* I/O bit count = 32. Byte per pixel = 2 */
} pfv_format_t;


/***********************************************************************
* Enumeration: pfv_swap_t
*
* Description:
*    Value of "pfv_config_t::input_swap", "pfv_config_t::output_swap".
*    Value of "DINSWAP32", "DOUTSWAP32" : format of "PFVID", "PFVOD" :
*    (47.3.2) input output data format.
************************************************************************/
typedef enum
{

    /* for "PFV_RGB888", "PFV_ARGB8888" */
    PFV_SWAP_ARGB8888  = 0,
    PFV_SWAP_RABG8888  = 1,
    PFV_SWAP_GBAR8888  = 2,
    PFV_SWAP_BGRA8888  = 3,

    /* for "PFV_RGB565" */
    PFV_SWAP_RGB565_PIXEL01  = 0,
    PFV_SWAP_RGB565_PIXEL10  = 2,

    /* for "PFV_YCbCr422" : PFV endian 32bit */
    PFV_SWAP_YCbCr422_CbY0CrY1 = 0,
    PFV_SWAP_YCbCr422_Y0CbY1Cr = 1,
    PFV_SWAP_YCbCr422_CrY1CbY0 = 2,
    PFV_SWAP_YCbCr422_Y1CrY0Cb = 3,

    /* for "PFV_YCbCr422" : Byte order */
    PFV_SWAP_YCbCr422_Y1_Cr_Y0_Cb = 0,
    PFV_SWAP_YCbCr422_Cr_Y1_Cb_Y0 = 1,
    PFV_SWAP_YCbCr422_Y0_Cb_Y1_Cr = 2,
    PFV_SWAP_YCbCr422_Cb_Y0_Cr_Y1 = 3  /* UYVY, Standard YUV422(U=Cb,V=Cr) */
} pfv_swap_t;


/***********************************************************************
* Enumeration: pfv_color_matrix_mode_t
************************************************************************/
typedef enum
{
    PFV_GBR_TO_GBR     = 0,  /* GBR is same as RGB */
    PFV_GBR_TO_YCbCr   = 1,
    PFV_YCbCr_TO_GBR   = 2,
    PFV_YCbCr_TO_YCbCr = 3,
    PFV_MATRIX_MODE_COUNT = 4
} pfv_color_matrix_mode_t;


/***********************************************************************
* Enumeration: pfv_dc_offset_index_t
*    Array index number of "pfv_color_matrix_t::dc_offset".
************************************************************************/
typedef enum
{
    PFV_DC_OFFSET_Y_OR_GREEN,
    PFV_DC_OFFSET_BLUE,
    PFV_DC_OFFSET_RED,
    PFV_DC_OFFSET_ELEMENT_COUNT
} pfv_dc_offset_index_t;


/***********************************************************************
* Enumeration: pfv_matrix_multiply_index_t
*    Array index number of "pfv_color_matrix_t::matrix_multiply_256".
************************************************************************/
typedef enum
{
    PFV_COLOR_MATRIX_GG = 0,
    PFV_COLOR_MATRIX_GB = 1,  /* GB = output is Green, input is Blue */
    PFV_COLOR_MATRIX_GR = 2,
    PFV_COLOR_MATRIX_BG = 3,
    PFV_COLOR_MATRIX_BB = 4,
    PFV_COLOR_MATRIX_BR = 5,
    PFV_COLOR_MATRIX_RG = 6,
    PFV_COLOR_MATRIX_RB = 7,
    PFV_COLOR_MATRIX_RR = 8,
    PFV_COLOR_MATRIX_ELEMENT_COUNT = 9,
    PFV_COLOR_MATRIX_YY = 0
} pfv_matrix_multiply_index_t;


/***********************************************************************
* Structure: pfv_color_matrix_t
*    Compatible with "vdc5_color_matrix_t" type in VDC5 driver.
************************************************************************/
typedef struct st_pfv_color_matrix_t  pfv_color_matrix_t;
struct st_pfv_color_matrix_t
{
    int32_t                  dummy;
    pfv_color_matrix_mode_t  mode;    /* Check at "pfv_io_format_t::input_format" and "output_format" */
    uint16_t                 dc_offset_plus_128[ PFV_DC_OFFSET_ELEMENT_COUNT ];
    int16_t                  matrix_multiply_256[ PFV_COLOR_MATRIX_ELEMENT_COUNT ];
};


/***********************************************************************
* Structure: pfv_reset_color_matrix_t
************************************************************************/
typedef struct st_pfv_reset_color_matrix_t  pfv_reset_color_matrix_t;
struct st_pfv_reset_color_matrix_t
{
    uint16_t  dc_offset_plus_128[ PFV_DC_OFFSET_ELEMENT_COUNT ];
    int16_t   matrix_multiply_256[ PFV_COLOR_MATRIX_ELEMENT_COUNT ];
};


/***********************************************************************
* Structure: pfv_io_format_t
************************************************************************/
typedef struct st_pfv_io_format_t  pfv_io_format_t;
struct st_pfv_io_format_t
{
    bit_flags_fast32_t   flags;

    pfv_format_t  input_format;   /* PFVCR register - IFMT bitfield */
    pfv_swap_t    input_swap;     /* PFVCR register - DINSWAP32 bitfield */
    pfv_format_t  output_format;  /* PFVCR register - OFMT bitfield */
    pfv_swap_t    output_swap;    /* PFVCR register - DOUTSWAP32 bitfield */
    int_fast32_t  output_alpha;   /* PFVACR register */
    bool_t        is_dither;      /* PFVCR register - DTH_ON bitfield */
    int_fast32_t  image_width;    /* for dither only, PFVSZR register - PFVSZX bitfield */
    int_fast32_t  image_height;   /* for dither only, PFVSZR register - PFVSZY bitfield */
};

enum   /*bit_flags_fast32_t*/
{
    F_PFV_INPUT_FORMAT  = 0x0001u,  /* If not set, not changed. Default is PFV_RGB888 */
    F_PFV_INPUT_SWAP    = 0x0002u,  /* Default is standard byte order */
    F_PFV_OUTPUT_FORMAT = 0x0004u,  /* If not set, not changed. Default is PFV_RGB888 */
    F_PFV_OUTPUT_SWAP   = 0x0008u,  /* Default is standard byte order */
    F_PFV_OUTPUT_ALPHA  = 0x0010u,  /* If not set, not changed. Default is 255 */
    F_PFV_IS_DITHER     = 0x0020u,  /* If not set, not changed. Default is true */
    F_PFV_IMAGE_WIDTH   = 0x0040u,  /* If not set, not changed. Default is 0 */
    F_PFV_IMAGE_HEIGHT  = 0x0080u   /* If not set, not changed. Default is 0 */
};


/***********************************************************************
* Enumeration: pfv_idtrg_t
*    Value of PFV.PFVICR.IDTRG.
************************************************************************/
typedef enum
{
    PFV_IDTRG_SPACED_2_BYTE  = 0,
    PFV_IDTRG_SPACED_4_BYTE  = 1,
    PFV_IDTRG_SPACED_16_BYTE = 2,
    PFV_IDTRG_SPACED_32_BYTE = 3,
    PFV_IDTRG_COUNT          = 4
} pfv_idtrg_t;


/***********************************************************************
* Enumeration: pfv_odtrg_t
*    Value of PFV.PFVICR.ODTRG.
************************************************************************/
typedef enum
{
    PFV_ODTRG_FILLED_2_BYTE  = 0,
    PFV_ODTRG_FILLED_4_BYTE  = 1,
    PFV_ODTRG_FILLED_16_BYTE = 2,
    PFV_ODTRG_FILLED_32_BYTE = 3,
    PFV_ODTRG_COUNT          = 4
} pfv_odtrg_t;


/***********************************************************************
* Enumeration: errnum_t
*    Part of values as <errnum_t>.
************************************************************************/
/*(errnum_t)*/
#ifndef  E_CATEGORY_PFV  /* Overwritable */
#define  E_CATEGORY_PFV  E_CATEGORY_PFV
enum { E_CATEGORY_PFV = 0x00004700 };
#endif

enum { E_PFV_HW_ERROR   = E_CATEGORY_PFV | 0x01 }; /* 0x4701 = 18177 */


/***********************************************************************
* Structure: pfv_dma_bit_count_t
************************************************************************/
#ifdef  DMAC_BIT_COUNT_T_DEFINED
typedef struct st_pfv_dma_bit_count_t  pfv_dma_bit_count_t;
struct st_pfv_dma_bit_count_t
{
    dmac_bit_count_t  input_buffer_DMA_bit_count;
    dmac_bit_count_t  input_PFV_DMA_bit_count;
    pfv_idtrg_t       input_trigger_byte_count;

    dmac_bit_count_t  output_PFV_DMA_bit_count;
    dmac_bit_count_t  output_buffer_DMA_bit_count;
    pfv_odtrg_t       output_trigger_byte_count;
};
#endif


/***********************************************************************
* Type: pfv_get_dma_bit_count_func_t
************************************************************************/
typedef errnum_t  (* pfv_get_dma_bit_count_func_t )(
    uintptr_t const  input_buffer_start_address,
    size_t    const  input_buffer_size,
    uintptr_t const  output_buffer_start_address,
    size_t    const  output_buffer_size,
    pfv_dma_bit_count_t* const  out );


/***********************************************************************
* Structure: pfv_dmac_config_t
************************************************************************/
typedef struct st_pfv_dmac_config_t  pfv_dmac_config_t;
struct st_pfv_dmac_config_t
{
    bit_flags_fast32_t  flags;  /* 0 | F_PFV_DMAC_INPUT_DMAC_CHANNEL | ... */
    int_fast32_t        input_DMAC_channel;
    int_fast32_t        output_DMAC_channel;

    const pfv_reset_color_matrix_t*  reset_color_matrixes;  /* Array element count is PFV_MATRIX_MODE_COUNT */
    pfv_get_dma_bit_count_func_t     get_dma_bit_count_func;
};

enum   /*bit_flags_fast32_t*/
{
    F_PFV_DMAC_INPUT_DMAC_CHANNEL    = 0x01u,
    F_PFV_DMAC_OUTPUT_DMAC_CHANNEL   = 0x02u,
    F_PFV_DMAC_RESET_COLOR_MATRIXES  = 0x04u, /* Default is defined in "r_pfv_pl.c" */
    F_PFV_DMAC_GET_DMA_BIT_COUNT     = 0x08u  /* Default is defined in "r_pfv_pl.c" */
};


/***********************************************************************
* Structure: pfv_transfer_config_t
************************************************************************/
typedef struct st_pfv_transfer_config_t  pfv_transfer_config_t;
struct st_pfv_transfer_config_t
{
    bit_flags_fast32_t  flags;  /* 0 | F_PFV_TRANSFER_INPUT_BUFFER_ADDRESS | ... */

    uintptr_t   input_buffer_address;
    size_t      input_buffer_size;
    uintptr_t   output_buffer_address;
    size_t      output_buffer_size;
    int_fast32_t  buffer_width;
    int_fast32_t  buffer_height;

    int_fast32_t                  interval_count_of_DMA;
    r_ospl_axi_cache_attribute_t  source_AXI_cache_attribute;
    r_ospl_axi_protection_t       source_AXI_protection;
    r_ospl_axi_cache_attribute_t  destination_AXI_cache_attribute;
    r_ospl_axi_protection_t       destination_AXI_protection;
};
/*bit_flags_fast32_t*/
#define  F_PFV_TRANSFER_INPUT_BUFFER_ADDRESS   0x0001u  /* Default is not used DMAC, used PIO */
#define  F_PFV_TRANSFER_INPUT_BUFFER_SIZE      0x0002u  /* Must, if F_INPUT_BUFFER_ADDRESS was set */
#define  F_PFV_TRANSFER_OUTPUT_BUFFER_ADDRESS  0x0004u  /* Default is not used DMAC, used PIO */
#define  F_PFV_TRANSFER_OUTPUT_BUFFER_SIZE     0x0008u  /* Must, if F_OUTPUT_BUFFER_ADDRESS was set */
#define  F_PFV_TRANSFER_BUFFER_WIDTH           0x0010u  /* Default is used only buffer size. Must */
/* if dither was used */
#define  F_PFV_TRANSFER_BUFFER_HEIGHT          0x0020u  /* Must, if F_BUFFER_WIDTH was set */

#define  F_PFV_TRANSFER_INTERVAL_COUNT_OF_DMA            0x0040u  /* Default is 0 */
#define  F_PFV_TRANSFER_SOURCE_AXI_CACHE_ATTRIBUTE       0x0100u  /* Default is R_OSPL_AXI_CACHE_ZERO */
#define  F_PFV_TRANSFER_SOURCE_AXI_PROTECTION            0x0200u  /* Default is R_OSPL_AXI_PROTECTION_ZERO */
#define  F_PFV_TRANSFER_DESTINATION_AXI_CACHE_ATTRIBUTE  0x0400u  /* Default is R_OSPL_AXI_CACHE_ZERO */
#define  F_PFV_TRANSFER_DESTINATION_AXI_PROTECTION       0x0800u  /* Default is R_OSPL_AXI_PROTECTION_ZERO */
/* #define is longer symbol than QAC rule */


/***********************************************************************
* Structure: pfv_config_t
************************************************************************/
typedef struct st_pfv_config_t  pfv_config_t;
struct st_pfv_config_t
{
    bit_flags_fast32_t  flags;  /* 0 | F_PFV_RESET_COLOR_MATRIXES | ... */

    const pfv_reset_color_matrix_t*  reset_color_matrixes;  /* Array element count is PFV_MATRIX_MODE_COUNT */
};
enum { /*bit_flags_fast32_t*/ F_PFV_RESET_COLOR_MATRIXES  = 0x01 }; /* Default is defined in "r_pfv_pl.c" */


/***********************************************************************
* Structure: pfv_async_status_t
************************************************************************/
typedef r_ospl_async_status_t  pfv_async_status_t;


/***********************************************************************
* Constants: pfv_interrupt_line_t
*    Value of r_ospl_interrupt_t::Type.
*
*    : PFV_INTERRUPT_LINE_PFVEIn - PFVEIn.
*    : PFV_INTERRUPT_LINE_IFEIn  - IFEIn.
*    : PFV_INTERRUPT_LINE_OFFIn  - OFFIn.
************************************************************************/
#define  PFV_INTERRUPT_LINE_PFVEIn  1u
#define  PFV_INTERRUPT_LINE_IFEIn   2u
#define  PFV_INTERRUPT_LINE_OFFIn   4u


/***********************************************************************
* Type: pfv_interrupt_lines_t
************************************************************************/
typedef bit_flags_fast32_t /*<pfv_interrupt_line_t>*/  pfv_interrupt_lines_t;


/***********************************************************************
* Constant: PFV_INTERRUPT_LINE_ALL
************************************************************************/
#define  PFV_INTERRUPT_LINE_ALL  \
	( PFV_INTERRUPT_LINE_PFVEIn | PFV_INTERRUPT_LINE_IFEIn | PFV_INTERRUPT_LINE_OFFIn )


/***********************************************************************
* Enumeration: pfv_dmac_interrupt_unit_t
************************************************************************/
typedef enum
{
    PFV_INTERRUPT_UNIT_PFV   = 1,
    PFV_INTERRUPT_UNIT_DMAC  = 2
} pfv_dmac_interrupt_unit_t;


/******************************************************************************
Variable Externs
******************************************************************************/

/******************************************************************************
Functions Prototypes
******************************************************************************/
/* In "r_pfv.h" */


#ifdef __cplusplus
}  /* End of extern "C" */
#endif /* __cplusplus */

#endif
