/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2015 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/***********************************************************************
* File: r_pfv.c
*    Pixel Format Converter (PFV) Sample Driver.
*
* - $Version: $ (=PFV_VERSION)
* - $ModuleRevision: {Masters}\PFV_DMAC\PFV_DMAC\23_RStyled $
* - $Rev: $
* - $Date: 2017-08-24T18:32:46+09:00 $
************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include  "r_typedefs.h"
#include  "dev_drv.h"
#include  "iodefine.h"
#include  "clib_drivers.h"
#include  "iobitmasks/pfv_iobitmask.h"
#include  "r_dmac_rm.h"
#include  "r_dmac_rm_pl.h"
#include  "r_pfv.h"
#include  "r_pfv_pl.h"
#include  "r_pfv_internal.h"
#ifndef R_OSPL_NDEBUG
#include  <stdio.h>
#endif


/******************************************************************************
Typedef definitions
******************************************************************************/
typedef struct st_r_pfv_dmac_i_lock_t  r_pfv_dmac_i_lock_t;


/***********************************************************************
* Structure: pfv_dmac_t
*    PFV with DMAC.
************************************************************************/
typedef struct st_pfv_dmac_t  pfv_dmac_t;
struct st_pfv_dmac_t
{
    bool_t                                  is_i_lock_master;
    r_pfv_dmac_i_lock_t*                    i_lock;
    const r_ospl_i_lock_vtable_t*           i_lock_v_table;

    pfv_async_status_t                      async_status;
    r_ospl_caller_t                         interrupt_callback_caller;

    bool_t                                  is_used_input_DMAC;
    int_fast32_t                            channel_of_input_DMAC;
    r_ospl_async_t                          async_of_input_DMAC;
    const dmac_rm_async_status_t*    async_status_of_input_DMAC;
    bool_t                                  is_i_lock_master_of_input_DMAC;

    bool_t                                  is_used_output_DMAC;
    int_fast32_t                            channel_of_output_DMAC;
    r_ospl_async_t                          async_of_output_DMAC;
    const dmac_rm_async_status_t*    async_status_of_output_DMAC;
    bool_t                                  is_i_lock_master_of_output_DMAC;

    pfv_get_dma_bit_count_func_t            get_dma_bit_count_func;
};


/***********************************************************************
* Structure: pfv_dmac_contexts_t
*    Contexts of <pfv_dmac_t>.
************************************************************************/
typedef struct st_pfv_dmac_contexts_t  pfv_dmac_contexts_t;
struct st_pfv_dmac_contexts_t
{
    pfv_dmac_t  channel[ PFV_CHANNEL_COUNT ];
};


/***********************************************************************
* Structure: pfv_t
*    PFV driver without DMAC.
************************************************************************/
typedef struct st_pfv_t  pfv_t;
struct st_pfv_t
{
    bool_t    is_initialized;  /* Initial value is false */

    const pfv_reset_color_matrix_t*  reset_color_matrixes;  /* Array element count is PFV_MATRIX_MODE_COUNT */

    bool_t           is_enabled_interrupt;
    r_ospl_flag32_t  interrupt_enables;
};


/***********************************************************************
* Structure: pfv_contexts_t
*    Contexts of <pfv_t>.
************************************************************************/
typedef struct st_pfv_contexts_t  pfv_contexts_t;
struct st_pfv_contexts_t
{
    pfv_t  channel[ PFV_CHANNEL_COUNT ];
};


/***********************************************************************
* Structure: r_pfv_dmac_i_lock_t
*    I-Lock for <pfv_dmac_t>.
************************************************************************/
struct st_r_pfv_dmac_i_lock_t
{
    bool_t                     is_lock;
    int_fast32_t               channel_num_PFV;
    int_fast32_t               channel_num_input_DMAC;
    int_fast32_t               channel_num_output_DMAC;
    pfv_interrupt_lines_t      enables_PFV;
    dmac_interrupt_lines_t     enables_InputDMAC;
    dmac_interrupt_lines_t     enables_OutputDMAC;
    bool_t                     is_requested_finalize;
};


/***********************************************************************
* Variable: gs_pfv_dmac_i_lock
************************************************************************/
static r_pfv_dmac_i_lock_t  gs_pfv_dmac_i_lock[ PFV_CHANNEL_COUNT ];


/******************************************************************************
Macro definitions
******************************************************************************/
R_STATIC_ASSERT_GLOBAL( PFV_CHANNEL_COUNT == PFV_COUNT, "" );


/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/

/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/

/******************************************************************************
Private global variables and functions
******************************************************************************/

STATIC_INLINE errnum_t  pfv_format_t_Is_YCbCr( pfv_format_t const  format,  bool_t* const  out_is_YCbCr );
STATIC_INLINE errnum_t  PFV_CheckIO_BitCount( pfv_format_t const  format,  int_fast32_t const  io_bit_count );
static void  R_PFV_DMAC_ExitRunning_Sub( int_fast32_t const  pfv_channel,  pfv_dmac_t* const  self );


/*[gs_pfv_dmac_contexts] PFV DMAC contexts */
static pfv_dmac_contexts_t  gs_pfv_dmac_contexts;

/*[gs_pfv_contexts] PFV contexts */
static pfv_contexts_t  gs_pfv_contexts;


/*[PFV_PFV_MTX_MD_GBR_TO_GBR] "PFV_MTX_MD" bits value : GBR to GBR */
enum { PFV_PFV_MTX_MD_GBR_TO_GBR   = 0 };

/*[PFV_PFV_MTX_MD_GBR_TO_YCBCR] "PFV_MTX_MD" bits value : GBR to YCbCr */
enum { PFV_PFV_MTX_MD_GBR_TO_YCBCR = 1 };

/*[PFV_PFV_MTX_MD_YCBCR_TO_GBR] "PFV_MTX_MD" bits value : YCbCr to GBR */
enum { PFV_PFV_MTX_MD_YCBCR_TO_GBR = 2 };

/*[PFV_PFV_MTX_MD_YCBCR_TO_YCBCR] "PFV_MTX_MD" bits value : YCbCr to YCbCr */
enum { PFV_PFV_MTX_MD_YCBCR_TO_YCBCR = 3 };



/*[PFV_DC_OFFSET_UINT_MIN] DC offset + 128 min value */
enum { PFV_DC_OFFSET_UINT_MIN       = 0 };

/*[PFV_DC_OFFSET_UINT_MAX] DC offset + 128 max value */
enum { PFV_DC_OFFSET_UINT_MAX       = 255 };

/*[PFV_MATRIX_MULTIPLY_INT_MIN] Multiply * 256 min value */
enum { PFV_MATRIX_MULTIPLY_INT_MIN  = -1024 };

/*[PFV_MATRIX_MULTIPLY_INT_MAX] Multiply * 256 max value */
enum { PFV_MATRIX_MULTIPLY_INT_MAX  =  1023 };


/*[gs_I_LockVTable]*/
static void   R_PFV_DMAC_I_LOCK_Reset( r_pfv_dmac_i_lock_t* const  self );
static void   R_PFV_DMAC_I_LOCK_Finalize( int_fast32_t const  pfv_channel );
static bool_t R_PFV_DMAC_I_LOCK_Lock( void* const  self_ );
static void   R_PFV_DMAC_I_LOCK_Unlock( void* const  self_ );
static void   R_PFV_DMAC_I_LOCK_RequestFinalize( void* const  self_ );
static int_fast32_t  R_PFV_DMAC_I_LOCK_GetRootChannelNum( const void* const  self_ );


/******************************************************************************
* Function Name: [pfv_format_t_Is_YCbCr]
* Description  :
* Arguments    : pfv_format_t  format;
* Arguments    : bool_t*       out_is_YCbCr;
* Return Value : Error code, 0=No error
******************************************************************************/
STATIC_INLINE errnum_t  pfv_format_t_Is_YCbCr( pfv_format_t const  format,  bool_t* const  out_is_YCbCr )
{
    errnum_t  e;
#if ! defined( R_OSPL_NDEBUG ) || defined( __QAC_ARM_H__ )
    enum { /*pfv_format_t*/  min_format = PFV_RGB888 };
    enum { /*pfv_format_t*/  max_format = PFV_YCbCr422 };
#endif
    enum { /*pfv_format_t*/  start_YCbCr = PFV_YCbCr422 };
    IF_S  ( format < (pfv_format_t) min_format )
    {
        e=E_OTHERS;
        goto fin;
    }
    IF_DS ( format > (pfv_format_t) max_format )
    {
        e=E_OTHERS;
        goto fin;
    }

    IF_DQ( out_is_YCbCr == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }

    *out_is_YCbCr = ( format >= (pfv_format_t) start_YCbCr );

    e = 0;
fin:
    return  e;
}


/***********************************************************************
* Function: PFV_CheckIO_BitCount
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
STATIC_INLINE errnum_t  PFV_CheckIO_BitCount( pfv_format_t const  format,  int_fast32_t const  io_bit_count )
{
    errnum_t  e;
    enum { /*int_fast32_t*/  num_32 = 32 };
    enum { /*int_fast32_t*/  num_16 = 16 };

    if ( format == PFV_RGB565 )
    {
        IF ( (io_bit_count != num_32) && (io_bit_count != num_16) )
        {
            e=E_OTHERS;
            goto fin;
        }
    }
    else
    {
#if PFV_PARAMETER_CHECK
        IF_DS ( (format != PFV_ARGB8888) && (format != PFV_YCbCr422) )
        {
            return  E_OTHERS;
        }
#endif

        IF ( io_bit_count != num_32 )
        {
            e=E_OTHERS;
            goto fin;
        }
    }

    e = 0;
fin:
    return  e;
}


/***********************************************************************
* Function: pfv_format_t_GetBytePerPixel
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
STATIC_INLINE errnum_t  pfv_format_t_GetBytePerPixel( pfv_format_t const  format,
        int_fast32_t* const  out_byte_per_pixel );  /* MISRA 8.1 Advice */

STATIC_INLINE errnum_t  pfv_format_t_GetBytePerPixel( pfv_format_t const  format,
        int_fast32_t* const  out_byte_per_pixel )
{
    errnum_t  e;

    enum { /*int_fast32_t*/  num_2 = 2 };
    enum { /*int_fast32_t*/  num_4 = 4 };
    IF_DQ( out_byte_per_pixel == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }


    if ( (format == PFV_RGB565) || (format == PFV_YCbCr422) )
    {
        *out_byte_per_pixel = num_2;
    }
    else
    {
#if PFV_PARAMETER_CHECK
        IF_DS ( format != PFV_ARGB8888 )
        {
            return  E_OTHERS;
        }
#endif

        *out_byte_per_pixel = num_4;
    }

    e = 0;
fin:
    return  e;
}


/***********************************************************************
* Function: R_PFV_DMAC_Initialize
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_PFV_DMAC_Initialize( int_fast32_t const  pfv_channel,
                                 pfv_dmac_config_t* const  in_out_config )
{
    errnum_t      e;
    pfv_dmac_t*   self;
    pfv_config_t  sub_config;

    /*[gs_i_lock_v_table_PFV]*/
    static const r_ospl_i_lock_vtable_t  gs_i_lock_v_table_PFV =
    {
        &( R_PFV_DMAC_I_LOCK_Lock ),  /* MISRA 16.9 */
        &( R_PFV_DMAC_I_LOCK_Unlock ),
        &( R_PFV_DMAC_I_LOCK_RequestFinalize ),
        &( R_PFV_DMAC_I_LOCK_GetRootChannelNum )
    };

    /*[gs_i_lock_v_table_output_DMAC]*/
    static const r_ospl_i_lock_vtable_t  gs_i_lock_v_table_output_DMAC =
    {
        &( R_OSPL_I_LOCK_LockStub ),  /* MISRA 16.9 */
        &( R_OSPL_I_LOCK_UnlockStub ),
        &( R_OSPL_I_LOCK_RequestFinalizeStub ),
        &( R_PFV_DMAC_I_LOCK_GetRootChannelNum )
    };

    /*[gs_i_lock_v_table_input_DMAC]*/
    static const r_ospl_i_lock_vtable_t  gs_i_lock_v_table_input_DMAC =
    {
        &( R_OSPL_I_LOCK_LockStub ),  /* MISRA 16.9 */
        &( R_OSPL_I_LOCK_UnlockStub ),
        &( R_OSPL_I_LOCK_RequestFinalizeStub ),
        &( R_PFV_DMAC_I_LOCK_GetRootChannelNum )
    };


#if PFV_PARAMETER_CHECK
    IF ( in_out_config == NULL )
    {
        e = E_OTHERS;
        goto fin;
    }
#endif

    R_IT_WILL_BE_NOT_CONST( in_out_config );


    IF ( (pfv_channel < PFV_CHANNEL_MIN) || (pfv_channel > PFV_CHANNEL_MAX) )
    {
        e = E_OTHERS;
        goto fin;
    }

    self = &gs_pfv_dmac_contexts.channel[ pfv_channel ];


    if ( self->i_lock == NULL )
    {
        r_pfv_dmac_i_lock_t* const  i_lock_ =
            &gs_pfv_dmac_i_lock[ pfv_channel ]; /* _ is for MISRA 5.6 */

        i_lock_->channel_num_PFV = pfv_channel;

        self->is_i_lock_master = R_PFV_DMAC_I_LOCK_Replace(
                                     pfv_channel,
                                     i_lock_,  &gs_i_lock_v_table_PFV );
    }

    if ( IS_BIT_SET( in_out_config->flags, F_PFV_DMAC_INPUT_DMAC_CHANNEL ) )
    {
        self->is_i_lock_master_of_input_DMAC = R_DMAC_RM_I_LOCK_Replace(
                in_out_config->input_DMAC_channel,
                self->i_lock,  &gs_i_lock_v_table_input_DMAC );

        ASSERT_D( self->is_i_lock_master_of_input_DMAC == self->is_i_lock_master,
                  e=E_OTHERS; goto fin );

        self->i_lock->channel_num_input_DMAC = in_out_config->input_DMAC_channel;
    }

    if ( IS_BIT_SET( in_out_config->flags, F_PFV_DMAC_OUTPUT_DMAC_CHANNEL ) )
    {
        self->is_i_lock_master_of_output_DMAC = R_DMAC_RM_I_LOCK_Replace(
                in_out_config->output_DMAC_channel,
                self->i_lock,  &gs_i_lock_v_table_output_DMAC );

        ASSERT_D( self->is_i_lock_master_of_output_DMAC == self->is_i_lock_master,
                  e=E_OTHERS; goto fin );

        self->i_lock->channel_num_output_DMAC = in_out_config->output_DMAC_channel;
    }

    if ( IS( self->is_i_lock_master ) )
    {
        R_PFV_DMAC_I_LOCK_Reset( self->i_lock );
    }
    ASSERT_R( self->async_status.State == R_OSPL_UNINITIALIZED,  e=E_STATE; goto fin );
    R_OSPL_FLAG32_InitConst( &self->async_status.InterruptFlags );
    R_OSPL_FLAG32_InitConst( &self->async_status.CancelFlags );


    e= R_Userdef_PFV_DMAC_SetDefaultConfig( in_out_config );
    IF ( e != 0 )
    {
        goto fin;
    }


    /* Initialize PFV */
    {
        sub_config.flags = 0;
        sub_config.reset_color_matrixes = in_out_config->reset_color_matrixes;
        sub_config.flags |= F_PFV_RESET_COLOR_MATRIXES;

        e= R_PFV_Initialize( pfv_channel, &sub_config );
        /* Calls "R_Userdef_PFV_OnInitialize" */
        IF ( e != 0 )
        {
            goto fin;
        }
    }


    /* Initialize input DMAC */
    if ( IS_BIT_SET( in_out_config->flags, F_PFV_DMAC_INPUT_DMAC_CHANNEL ) )
    {
        e= R_DMAC_RM_Initialize( in_out_config->input_DMAC_channel );
        IF ( e != 0 )
        {
            goto fin;
        }
        self->channel_of_input_DMAC = in_out_config->input_DMAC_channel;
        self->is_used_input_DMAC = true;

        e= R_DMAC_RM_GetAsyncStatus( in_out_config->input_DMAC_channel,
                                     &self->async_status_of_input_DMAC );
        IF ( e != 0 )
        {
            goto fin;
        }
    }
    else
    {
        self->is_used_input_DMAC = false;
    }


    /* Initialize output DMAC */
    if ( IS_BIT_SET( in_out_config->flags, F_PFV_DMAC_OUTPUT_DMAC_CHANNEL ) )
    {
        e= R_DMAC_RM_Initialize( in_out_config->output_DMAC_channel );
        IF ( e != 0 )
        {
            goto fin;
        }
        self->channel_of_output_DMAC = in_out_config->output_DMAC_channel;
        self->is_used_output_DMAC = true;

        e= R_DMAC_RM_GetAsyncStatus( in_out_config->output_DMAC_channel,
                                     &self->async_status_of_output_DMAC );
        IF ( e != 0 )
        {
            goto fin;
        }
    }
    else
    {
        self->is_used_output_DMAC = false;
    }


    self->get_dma_bit_count_func = in_out_config->get_dma_bit_count_func;


    self->async_status.State = R_OSPL_RUNNABLE;


    e = 0;
fin:
    if ( e != 0 )
    {
        errnum_t  ee;

        ee= R_PFV_DMAC_Terminate( pfv_channel );
        R_UNREFERENCED_VARIABLE( ee );
    }
    return  e;
}


/***********************************************************************
* Function: R_PFV_DMAC_Terminate
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_PFV_DMAC_Terminate( int_fast32_t const  pfv_channel )
{
    errnum_t     e;
    errnum_t     ee;
    bool_t       was_enabled; /* = false; */ /* QAC 3197 */
    pfv_dmac_t*  self;
    bool_t       b;

#if PFV_PARAMETER_CHECK
    IF ( (pfv_channel < PFV_CHANNEL_MIN) || (pfv_channel > PFV_CHANNEL_MAX) )
    {
        e = E_OTHERS;
        goto  fin;
    }
#endif


    self = &gs_pfv_dmac_contexts.channel[ pfv_channel ];
    if ( self->async_status.State == R_OSPL_UNINITIALIZED )
    {
        e = 0;
        goto  fin;
    }


    e = 0;
    R_AVOID_UNSAFE_ALWAYS_WARNING( e );


    was_enabled = self->i_lock_v_table->Lock( self->i_lock );


    if ( IS( self->is_used_input_DMAC ) )
    {
        e= R_DMAC_RM_Finalize( self->channel_of_input_DMAC, e );
    }
    if ( IS( self->is_used_output_DMAC ) )
    {
        e= R_DMAC_RM_Finalize( self->channel_of_output_DMAC, e );
    }

    ee= R_PFV_Terminate( pfv_channel );
    IF ( (ee != 0) && (e == 0) )
    {
        e = ee;
    }


    self->async_status.State = R_OSPL_UNINITIALIZED;


    if ( IS( self->is_i_lock_master_of_input_DMAC ) )
    {
        b= R_DMAC_RM_I_LOCK_Replace(
               self->channel_of_input_DMAC, NULL, NULL );
        R_UNREFERENCED_VARIABLE( b );  /* QAC 3200 : This is not error information */
    }
    if ( IS( self->is_i_lock_master_of_output_DMAC ) )
    {
        b= R_DMAC_RM_I_LOCK_Replace(
               self->channel_of_output_DMAC, NULL, NULL );
        R_UNREFERENCED_VARIABLE( b );  /* QAC 3200 : This is not error information */
    }

    if ( IS( was_enabled ) )
    {
        self->i_lock_v_table->Unlock( self->i_lock );

        if ( IS( self->is_i_lock_master ) )
        {
            if ( IS( gs_pfv_dmac_i_lock[ pfv_channel ].is_requested_finalize ) )
            {
                R_PFV_DMAC_I_LOCK_Finalize( pfv_channel );
            }
        }
    }

fin:
    return  e;
}


/***********************************************************************
* Function: R_PFV_DMAC_SetIOFormat
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_PFV_DMAC_SetIOFormat( int_fast32_t const  pfv_channel,
                                  pfv_io_format_t* const  in_out_io_format )
{
    errnum_t     e;
    pfv_dmac_t*  self;
    bool_t       was_enabled = false;

#if PFV_PARAMETER_CHECK
    IF ( (pfv_channel < PFV_CHANNEL_MIN) || (pfv_channel > PFV_CHANNEL_MAX) )
    {
        e = E_OTHERS;
        goto  fin;
    }
#endif

    self = &gs_pfv_dmac_contexts.channel[ pfv_channel ];

    was_enabled = self->i_lock_v_table->Lock( self->i_lock );
    ASSERT_R( self->async_status.State == R_OSPL_RUNNABLE,  e=E_STATE; goto fin );


    e= R_PFV_SetIOFormat( pfv_channel, in_out_io_format );
    IF(e!=0)
    {
        goto fin;
    }


    e=0;
fin:
    if ( IS( was_enabled ) )
    {
        /* ->QAC 3353 : "self" is always assigned, if "was_enabled" is true */
        IF_DQ( self == NULL ) {}  /* QAC 3353 raises SEC R3.2.2 */
        else
        {
            self->i_lock_v_table->Unlock( self->i_lock );
        }
        /* <-QAC 3353 */
    }
    return  e;
}


/***********************************************************************
* Function: R_PFV_DMAC_GetIOFormat
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_PFV_DMAC_GetIOFormat( int_fast32_t const  pfv_channel,
                                  pfv_io_format_t* const  out_io_format )
{
    errnum_t     e;
    pfv_dmac_t*  self;
    bool_t       was_enabled = false;

#if PFV_PARAMETER_CHECK
    IF ( (pfv_channel < PFV_CHANNEL_MIN) || (pfv_channel > PFV_CHANNEL_MAX) )
    {
        e = E_OTHERS;
        goto  fin;
    }
#endif

    self = &gs_pfv_dmac_contexts.channel[ pfv_channel ];
    was_enabled = self->i_lock_v_table->Lock( self->i_lock );
    ASSERT_R( self->async_status.State == R_OSPL_RUNNABLE,  e=E_STATE; goto fin );


    e= R_PFV_GetIOFormat( pfv_channel, out_io_format );
    IF(e!=0)
    {
        goto fin;
    }


    e=0;
fin:
    if ( IS( was_enabled ) )
    {
        /* ->QAC 3353 : "self" is always assigned, if "was_enabled" is true */
        IF_DQ( self == NULL ) {}  /* QAC 3353 raises SEC R3.2.2 */
        else
        {
            self->i_lock_v_table->Unlock( self->i_lock );
        }
        /* <-QAC 3353 */
    }
    return  e;
}


/***********************************************************************
* Function: R_PFV_DMAC_ResetImageColorMatrix
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_PFV_DMAC_ResetImageColorMatrix( int_fast32_t const  pfv_channel,
        pfv_color_matrix_mode_t const  mode )
{
    errnum_t     e;
    pfv_dmac_t*  self;
    bool_t       was_enabled = false;

#if PFV_PARAMETER_CHECK
    IF ( (pfv_channel < PFV_CHANNEL_MIN) || (pfv_channel > PFV_CHANNEL_MAX) )
    {
        e = E_OTHERS;
        goto  fin;
    }
#endif

    self = &gs_pfv_dmac_contexts.channel[ pfv_channel ];
    was_enabled = self->i_lock_v_table->Lock( self->i_lock );
    ASSERT_R( self->async_status.State == R_OSPL_RUNNABLE,  e=E_STATE; goto fin );


    e= R_PFV_ResetImageColorMatrix( pfv_channel, mode );
    IF(e!=0)
    {
        goto fin;
    }


    e=0;
fin:
    if ( IS( was_enabled ) )
    {
        /* ->QAC 3353 : "self" is always assigned, if "was_enabled" is true */
        IF_DQ( self == NULL ) {}  /* QAC 3353 raises SEC R3.2.2 */
        else
        {
            self->i_lock_v_table->Unlock( self->i_lock );
        }
        /* <-QAC 3353 */
    }
    return  e;
}


/***********************************************************************
* Function: R_PFV_DMAC_SetImageColorMatrix
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_PFV_DMAC_SetImageColorMatrix( int_fast32_t const  pfv_channel,
        const pfv_color_matrix_t* const  offset_and_matrix )
{
    errnum_t     e;
    pfv_dmac_t*  self;
    bool_t       was_enabled = false;

#if PFV_PARAMETER_CHECK
    IF ( (pfv_channel < PFV_CHANNEL_MIN) || (pfv_channel > PFV_CHANNEL_MAX) )
    {
        e = E_OTHERS;
        goto  fin;
    }
#endif

    self = &gs_pfv_dmac_contexts.channel[ pfv_channel ];
    was_enabled = self->i_lock_v_table->Lock( self->i_lock );
    ASSERT_R( self->async_status.State == R_OSPL_RUNNABLE,  e=E_STATE; goto fin );


    e= R_PFV_SetImageColorMatrix( pfv_channel, offset_and_matrix );
    IF(e!=0)
    {
        goto fin;
    }


    e=0;
fin:
    if ( IS( was_enabled ) )
    {
        /* ->QAC 3353 : "self" is always assigned, if "was_enabled" is true */
        IF_DQ( self == NULL ) {}  /* QAC 3353 raises SEC R3.2.2 */
        else
        {
            self->i_lock_v_table->Unlock( self->i_lock );
        }
        /* <-QAC 3353 */
    }
    return  e;
}


/***********************************************************************
* Function: R_PFV_DMAC_GetImageColorMatrix
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_PFV_DMAC_GetImageColorMatrix( int_fast32_t const  pfv_channel,
        pfv_color_matrix_t* const  out_offset_and_matrix )
{
    errnum_t     e;
    pfv_dmac_t*  self;
    bool_t       was_enabled = false;

#if PFV_PARAMETER_CHECK
    IF ( (pfv_channel < PFV_CHANNEL_MIN) || (pfv_channel > PFV_CHANNEL_MAX) )
    {
        e = E_OTHERS;
        goto  fin;
    }
#endif

    self = &gs_pfv_dmac_contexts.channel[ pfv_channel ];
    was_enabled = self->i_lock_v_table->Lock( self->i_lock );
    ASSERT_R( self->async_status.State == R_OSPL_RUNNABLE,  e=E_STATE; goto fin );


    e= R_PFV_GetImageColorMatrix( pfv_channel, out_offset_and_matrix );
    IF(e!=0)
    {
        goto fin;
    }


    e=0;
fin:
    if ( IS( was_enabled ) )
    {
        /* ->QAC 3353 : "self" is always assigned, if "was_enabled" is true */
        IF_DQ( self == NULL ) {}  /* QAC 3353 raises SEC R3.2.2 */
        else
        {
            self->i_lock_v_table->Unlock( self->i_lock );
        }
        /* <-QAC 3353 */
    }
    return  e;
}


/***********************************************************************
* Function: R_PFV_DMAC_Transfer
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_PFV_DMAC_Transfer( int_fast32_t const  pfv_channel,
                               pfv_transfer_config_t* const  in_out_config )
{
    errnum_t        e;
    errnum_t        ee;
    r_ospl_async_t  async;
    bit_flags32_t   got_flags;

    async.A_EventValue = 0;  /* For fin block */

    async.Flags = R_F_OSPL_A_Thread | R_F_OSPL_A_EventValue;
    e= R_OSPL_EVENT_Allocate(
           &async.A_Thread,  R_OSPL_THREAD_GetCurrentId(),
           &async.A_EventValue,  R_OSPL_UNUSED_FLAG );
    IF(e)
    {
        goto fin;
    }

    e= R_PFV_DMAC_TransferAsync( pfv_channel, in_out_config, &async );
    IF(e!=0)
    {
        goto fin;
    }

    e= R_OSPL_EVENT_Wait( async.A_EventValue, &got_flags, R_OSPL_INFINITE );
    IF(e!=0)
    {
        goto fin;
    }

    e= async.ReturnValue;
    IF(e!=0)
    {
        goto fin;
    }

    e=0;
fin:
    ee= R_OSPL_EVENT_Free( &async.A_Thread, &async.A_EventValue );
    e= R_OSPL_MergeErrNum( e, ee );
    return  e;
}


/***********************************************************************
* Function: R_PFV_DMAC_TransferAsync
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_PFV_DMAC_TransferAsync( int_fast32_t const  pfv_channel,
                                    pfv_transfer_config_t* const  in_out_config,  r_ospl_async_t* const  async )
{
    errnum_t                   e;
    pfv_dmac_t*                self;
    pfv_t*                     pfv;
    bit_flags_fast32_t         config_flags;
    uint32_t                   input_transfer_byte;
    uint32_t                   output_transfer_byte;
    int_fast32_t               byte_per_pixel;
    uint32_t                   value;
    uint32_t                   sub_value;
    bool_t                     was_enabled = false;
    volatile struct st_pfv*    reg;  /* register */
    pfv_io_format_t            io_format;
    pfv_dma_bit_count_t        bit_count_DMA;

    enum { /*int8_t*/  num_0x6C = 0x6C };
    enum { /*int8_t*/  num_0x6D = 0x6D };
    static const int8_t  array_of_MID[ PFV_CHANNEL_COUNT ] = { num_0x6C, num_0x6D };
    /* DMAC request resource MID/RID for PFV# */
    enum { /*int_fast32_t*/  num_2_RID = 2 };


    IF_DQ( in_out_config == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }
    IF_DQ( async == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }

    R_Userdef_PFV_SetDefaultAsync( async, R_OSPL_ASYNC_TYPE_NORMAL );
    async->ReturnValue = 0;

    e= R_PFV_STATIC_GetRegisterSet( pfv_channel, &reg );
    IF(e!=0)
    {
        goto fin;
    }
    IF_DQ( reg == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }

    self = &gs_pfv_dmac_contexts.channel[ pfv_channel ];
    pfv = &gs_pfv_contexts.channel[ pfv_channel ];
    config_flags = in_out_config->flags;

    was_enabled = self->i_lock_v_table->Lock( self->i_lock );

#if PFV_PARAMETER_CHECK
    if ( ! self->is_used_input_DMAC )
    {
        IF ( IS_BIT_SET( config_flags, F_PFV_TRANSFER_INPUT_BUFFER_ADDRESS ) )
        {
            e = E_OTHERS;
            goto  fin;
        }
    }

    if ( ! self->is_used_output_DMAC )
    {
        IF ( IS_BIT_SET( config_flags, F_PFV_TRANSFER_OUTPUT_BUFFER_ADDRESS ) )
        {
            e = E_OTHERS;
            goto  fin;
        }
    }
#endif


    ASSERT_R( self->async_status.State == R_OSPL_RUNNABLE,  e=E_STATE; goto fin );


    e= R_PFV_GetIOFormat( pfv_channel, &io_format );
    IF ( e != 0 )
    {
        goto fin;
    }


    /* Check "config_flags" */
#if PFV_PARAMETER_CHECK
    if ( IS_BIT_SET( config_flags, F_PFV_TRANSFER_INPUT_BUFFER_ADDRESS ) )
    {
        IF ( IS_ANY_BITS_NOT_SET( config_flags,
                                  F_PFV_TRANSFER_INPUT_BUFFER_ADDRESS | F_PFV_TRANSFER_INPUT_BUFFER_SIZE ) )
        {
            e = E_OTHERS;
            goto  fin;
        }
    }

    if ( IS_BIT_SET( config_flags, F_PFV_TRANSFER_OUTPUT_BUFFER_ADDRESS ) )
    {
        IF ( IS_ANY_BITS_NOT_SET( config_flags,
                                  F_PFV_TRANSFER_OUTPUT_BUFFER_ADDRESS | F_PFV_TRANSFER_OUTPUT_BUFFER_SIZE ) )
        {
            e = E_OTHERS;
            goto  fin;
        }
    }

    if ( IS_BIT_SET( config_flags, F_PFV_TRANSFER_BUFFER_WIDTH ) )
    {
        IF ( IS_ANY_BITS_NOT_SET( config_flags,
                                  F_PFV_TRANSFER_BUFFER_WIDTH | F_PFV_TRANSFER_BUFFER_HEIGHT ) )
        {
            e = E_OTHERS;
            goto  fin;
        }
    }
    else
    {
        if ( io_format.output_format == PFV_RGB565 )
        {
            IF ( IS( io_format.is_dither ) )
            {
                e = E_OTHERS;  /* No F_PFV_TRANSFER_BUFFER_WIDTH error */
                goto  fin;
            }
        }
    }
#endif


    /* Set "input_transfer_byte", "output_transfer_byte" */
    if ( IS_BIT_SET( config_flags, F_PFV_TRANSFER_BUFFER_WIDTH ) )
    {
        e= pfv_format_t_GetBytePerPixel( io_format.input_format, &byte_per_pixel );
        IF ( e != 0 )
        {
            goto fin;
        }
        input_transfer_byte = (uint32_t)(
                                  in_out_config->buffer_width * in_out_config->buffer_height * byte_per_pixel );

        IF ( in_out_config->input_buffer_size < input_transfer_byte )
        {
            e = E_OTHERS;  /* Few "input_transfer_byte" */
            goto  fin;
        }


        e= pfv_format_t_GetBytePerPixel( io_format.output_format, &byte_per_pixel );
        IF ( e != 0 )
        {
            goto fin;
        }
        output_transfer_byte = (uint32_t)(
                                   in_out_config->buffer_width * in_out_config->buffer_height * byte_per_pixel );

        IF ( in_out_config->output_buffer_size < output_transfer_byte )
        {
            e = E_OTHERS;  /* Few "output_buffer_size" */
            goto  fin;
        }

        in_out_config->output_buffer_size = output_transfer_byte;
    }
    else
    {
        if ( IS_BIT_SET( config_flags, F_PFV_TRANSFER_INPUT_BUFFER_ADDRESS ) )
        {
            input_transfer_byte = in_out_config->input_buffer_size;
        }
        else
        {
            input_transfer_byte = 0;
        }

        if ( IS_BIT_SET( config_flags, F_PFV_TRANSFER_OUTPUT_BUFFER_ADDRESS ) )
        {
            output_transfer_byte = in_out_config->output_buffer_size;
        }
        else
        {
            output_transfer_byte = 0;
        }
    }


    /* Set "bit_count_DMA" */
    e= self->get_dma_bit_count_func(
           in_out_config->input_buffer_address,   input_transfer_byte,
           in_out_config->output_buffer_address,  output_transfer_byte,
           &bit_count_DMA );
    IF ( e != 0 )
    {
        goto  fin;
    }


    /* Write "PFVICR" register */
    {
        value = reg->PFVICR;


        R_PFV_SET_REGISTER_BIT_FIELD( &value, PFVICR, PFVEEN, TO_UNSIGNED( true ) );
        R_PFV_SET_REGISTER_BIT_FIELD( &value, PFVICR, IFEN,   TO_UNSIGNED( true ) );
        R_PFV_SET_REGISTER_BIT_FIELD( &value, PFVICR, OFEN,   TO_UNSIGNED( true ) );


        /* Set value of "IDTRG" bitfield */
        if ( input_transfer_byte > 0u )
        {
            sub_value = bit_count_DMA.input_trigger_byte_count;
#if PFV_PARAMETER_CHECK
            IF ( R_CUT_IF_ALWAYS_FALSE( (sub_value < 0u) ||) (sub_value >= PFV_IDTRG_COUNT) )
            {
                e = E_OTHERS;
                goto  fin;
            }
#endif
            R_PFV_SET_REGISTER_BIT_FIELD( &value, PFVICR, IDTRG, sub_value );
        }

        /* Set value of "ODTRG" bitfield */
        if ( output_transfer_byte > 0u )
        {
            sub_value = bit_count_DMA.output_trigger_byte_count;
#if PFV_PARAMETER_CHECK
            IF ( R_CUT_IF_ALWAYS_FALSE( (sub_value < 0u) ||) (sub_value >= PFV_ODTRG_COUNT) )
            {
                e = E_OTHERS;
                goto  fin;
            }
#endif
            R_PFV_SET_REGISTER_BIT_FIELD( &value, PFVICR, ODTRG, sub_value );
        }

        /* Write "PFVICR" register */
        reg->PFVICR = value;
    }


    /* Write "PFVSZR" register */
    {
        /* Read "value" */
        value = reg->PFVSZR;

        /* Set value of "PFVSZX" bitfield */
        if ( IS_BIT_SET( config_flags, F_PFV_TRANSFER_BUFFER_WIDTH ) )
        {
            sub_value = (uint32_t) in_out_config->buffer_width;
#if PFV_PARAMETER_CHECK
            IF ( IS( R_PFV_IS_OVERFLOW_BIT_FIELD( PFVSZR, PFVSZX, sub_value ) ) )
            {
                e = E_OTHERS;
                goto  fin;
            }
#endif
            R_PFV_SET_REGISTER_BIT_FIELD( &value, PFVSZR, PFVSZX, sub_value );
        }
        else
        {
            in_out_config->buffer_width = (int_fast32_t)
                                          R_PFV_GET_REGISTER_BIT_FIELD( value, PFVSZR, PFVSZX );
        }

        /* Set value of "PFVSZY" bitfield */
        if ( IS_BIT_SET( config_flags, F_PFV_TRANSFER_BUFFER_HEIGHT ) )
        {
            sub_value = (uint32_t) in_out_config->buffer_height;
#if PFV_PARAMETER_CHECK
            IF ( IS( R_PFV_IS_OVERFLOW_BIT_FIELD( PFVSZR, PFVSZY, sub_value ) ) )
            {
                e = E_OTHERS;
                goto  fin;
            }
#endif
            R_PFV_SET_REGISTER_BIT_FIELD( &value, PFVSZR, PFVSZY, sub_value );
        }
        else
        {
            in_out_config->buffer_height = (int_fast32_t)
                                           R_PFV_GET_REGISTER_BIT_FIELD( value, PFVSZR, PFVSZY );
        }

        /* Write "PFVSZR" register */
        reg->PFVSZR = value;
    }


    /* Clear the interrupt status */
    value = reg->PFVISR;  /* Read from the register for clear */
    reg->PFVISR = 0u;     /* Clear the interrupt status */
    R_OSPL_EVENT_Clear( async->A_Thread, async->A_EventValue );
    R_OSPL_EVENT_Clear( async->I_Thread, async->I_EventValue );


    /* ->MISRA 11.4 : Not too big "enum" is same bit count as "int" */ /* ->SEC R2.7.1 */
    R_OSPL_CALLER_Initialize( &self->interrupt_callback_caller,
                              async, (volatile int_t*) &self->async_status.State, R_OSPL_INTERRUPTING,
                              self->i_lock, self->i_lock_v_table );
    /* <-MISRA 11.4 */ /* <-SEC R2.7.1 */
    e= R_Userdef_PFV_SetInterruptCallbackCaller( pfv_channel,
            &self->interrupt_callback_caller );
    IF(e!=0)
    {
        goto fin;
    }
    R_OSPL_FLAG32_Set( &pfv->interrupt_enables, PFV_INTERRUPT_LINE_PFVEIn );


    /* Start input DMAC */
    if ( input_transfer_byte > 0u )
    {
        dmac_register_mode_config_t  config_DMAC;
        r_ospl_async_t* const        async_DMAC = &self->async_of_input_DMAC;

#if PFV_PARAMETER_CHECK
        IF ( in_out_config->input_buffer_size < input_transfer_byte )
        {
            e = E_OTHERS;
            goto  fin;
        }
#endif


        config_DMAC.flags =
            F_DMAC_NEXT0_SOURCE_PHYSICAL_ADDRESS |
            F_DMAC_NEXT0_DESTINATION_PHYSICAL_ADDRESS |
            F_DMAC_NEXT0_TRANSFER_BYTE |
            F_DMAC_IS_DESTINATION_ADDRESS_FIXED |
            F_DMAC_SOURCE_BIT_COUNT |
            F_DMAC_DESTINATION_BIT_COUNT |

            F_DMAC_REQUEST_RESOURCE_RID |
            F_DMAC_REQUEST_RESOURCE_MID |
            F_DMAC_TRANSFER_MODE_TM |
            F_DMAC_IS_DMA_ACK_OUTPUT_MODE_AM |
            F_DMAC_REQUEST_EDGE_OR_LEVEL_LVL |
            F_DMAC_IS_REQUEST_RAISING_OR_HIGH_HIEN |
            F_DMAC_IS_REQUEST_FALLING_OR_LOW_LOEN |
            F_DMAC_REQUEST_DIRECTION_REQD;

        /* Set address */
        config_DMAC.next0_source_physical_address = in_out_config->input_buffer_address;
        e= R_OSPL_ToPhysicalAddress( &reg->PFVID0,
                                     &config_DMAC.next0_destination_physical_address );
        IF(e!=0)
        {
            goto fin;
        }
        config_DMAC.next0_transfer_byte = (size_t) input_transfer_byte;
        config_DMAC.is_destination_address_fixed = true;
        config_DMAC.source_bit_count      = bit_count_DMA.input_buffer_DMA_bit_count;
        config_DMAC.destination_bit_count = bit_count_DMA.input_PFV_DMA_bit_count;

        /* Set request resource */
        R_STATIC_ASSERT( (int_fast32_t) R_COUNT_OF( array_of_MID ) == (int_fast32_t) PFV_CHANNEL_MAX + 1, "" );
        config_DMAC.request_resource_MID = array_of_MID[ pfv_channel ];
        config_DMAC.request_resource_RID = 1;
        config_DMAC.transfer_mode_TM = DMAC_SINGLE_TRANSFER_0;
        config_DMAC.is_DMA_ack_output_mode_AM = DMAC_ACK_BUS_CYCLE_MODE_010;
        config_DMAC.request_edge_or_level_LVL = DMAC_LEVEL_1;
        config_DMAC.is_request_raising_or_high_HIEN = true;
        config_DMAC.is_request_falling_or_low_LOEN = false;
        config_DMAC.request_direction_REQD = DMAC_DMAREQ_DESTINATION_1;

        /* Delegate to DMAC  */
        if ( IS_BIT_SET( config_flags, F_PFV_TRANSFER_INTERVAL_COUNT_OF_DMA ) )
        {
            config_DMAC.flags |= F_DMAC_INTERVAL_COUNT;
            config_DMAC.interval_count = in_out_config->interval_count_of_DMA;
        }
        if ( IS_BIT_SET( config_flags, F_PFV_TRANSFER_SOURCE_AXI_CACHE_ATTRIBUTE ) )
        {
            config_DMAC.flags |= F_DMAC_SOURCE_AXI_CACHE_ATTRIBUTE;
            config_DMAC.source_AXI_cache_attribute = in_out_config->source_AXI_cache_attribute;
        }
        if ( IS_BIT_SET( config_flags, F_PFV_TRANSFER_SOURCE_AXI_CACHE_ATTRIBUTE ) )
        {
            config_DMAC.flags |= F_DMAC_SOURCE_AXI_PROTECTION;
            config_DMAC.source_AXI_protection = in_out_config->source_AXI_protection;
        }

        /* Start */
        R_OSPL_ASYNC_CopyExceptAThread( async, async_DMAC );
        e= R_DMAC_RM_StartAsRegisterMode( self->channel_of_input_DMAC, &config_DMAC, async_DMAC );
        IF ( e != 0 )
        {
            goto  fin;
        }
    }


    /* Start output DMAC */
    if ( output_transfer_byte > 0u )
    {
        dmac_register_mode_config_t  config_DMAC;
        r_ospl_async_t* const        async_DMAC = &self->async_of_output_DMAC;

#if PFV_PARAMETER_CHECK
        IF ( in_out_config->output_buffer_size < output_transfer_byte )
        {
            e = E_OTHERS;
            goto  fin;
        }
#endif


        config_DMAC.flags =
            F_DMAC_NEXT0_SOURCE_PHYSICAL_ADDRESS |
            F_DMAC_NEXT0_DESTINATION_PHYSICAL_ADDRESS |
            F_DMAC_NEXT0_TRANSFER_BYTE |
            F_DMAC_IS_SOURCE_ADDRESS_FIXED |
            F_DMAC_SOURCE_BIT_COUNT |
            F_DMAC_DESTINATION_BIT_COUNT |

            F_DMAC_REQUEST_RESOURCE_RID |
            F_DMAC_REQUEST_RESOURCE_MID |
            F_DMAC_TRANSFER_MODE_TM |
            F_DMAC_IS_DMA_ACK_OUTPUT_MODE_AM |
            F_DMAC_REQUEST_EDGE_OR_LEVEL_LVL |
            F_DMAC_IS_REQUEST_RAISING_OR_HIGH_HIEN |
            F_DMAC_IS_REQUEST_FALLING_OR_LOW_LOEN |
            F_DMAC_REQUEST_DIRECTION_REQD;

        /* Set address */
        e= R_OSPL_ToPhysicalAddress( &reg->PFVOD0,
                                     &config_DMAC.next0_source_physical_address );
        IF(e!=0)
        {
            goto fin;
        }
        config_DMAC.next0_destination_physical_address = in_out_config->output_buffer_address;
        config_DMAC.next0_transfer_byte = (size_t) output_transfer_byte;
        config_DMAC.is_source_address_fixed = true;
        config_DMAC.source_bit_count      = bit_count_DMA.output_PFV_DMA_bit_count;
        config_DMAC.destination_bit_count = bit_count_DMA.output_buffer_DMA_bit_count;

        /* Set request resource */
        R_STATIC_ASSERT( (int_fast32_t) R_COUNT_OF( array_of_MID ) == (int_fast32_t) PFV_CHANNEL_MAX + 1, "" );
        config_DMAC.request_resource_MID = array_of_MID[ pfv_channel ];
        config_DMAC.request_resource_RID = num_2_RID;
        config_DMAC.transfer_mode_TM = DMAC_SINGLE_TRANSFER_0;
        config_DMAC.is_DMA_ack_output_mode_AM = DMAC_ACK_BUS_CYCLE_MODE_010;
        config_DMAC.request_edge_or_level_LVL = DMAC_LEVEL_1;
        config_DMAC.is_request_raising_or_high_HIEN = true;
        config_DMAC.is_request_falling_or_low_LOEN = false;
        config_DMAC.request_direction_REQD = DMAC_DMAREQ_SOURCE_0;

        /* Delegate to DMAC  */
        if ( IS_BIT_SET( config_flags, F_PFV_TRANSFER_INTERVAL_COUNT_OF_DMA ) )
        {
            config_DMAC.flags |= F_DMAC_INTERVAL_COUNT;
            config_DMAC.interval_count = in_out_config->interval_count_of_DMA;
        }
        if ( IS_BIT_SET( config_flags, F_PFV_TRANSFER_DESTINATION_AXI_CACHE_ATTRIBUTE ) )
        {
            config_DMAC.flags |= F_DMAC_DESTINATION_AXI_CACHE_ATTRIBUTE;
            config_DMAC.destination_AXI_cache_attribute = in_out_config->destination_AXI_cache_attribute;
        }
        if ( IS_BIT_SET( config_flags, F_PFV_TRANSFER_DESTINATION_AXI_CACHE_ATTRIBUTE ) )
        {
            config_DMAC.flags |= F_DMAC_DESTINATION_AXI_PROTECTION;
            config_DMAC.destination_AXI_protection      = in_out_config->destination_AXI_protection;
        }

        /* Start */
        R_OSPL_ASYNC_CopyExceptAThread( async, async_DMAC );
        e= R_DMAC_RM_StartAsRegisterMode( self->channel_of_output_DMAC, &config_DMAC, async_DMAC );
        IF ( e != 0 )
        {
            goto  fin;
        }
    }

    self->async_status.State = R_OSPL_RUNNING;

    e = 0;
fin:
    if ( IS( was_enabled ) )
    {
        /* ->QAC 3353 : "self" is always assigned, if "was_enabled" is true */
        IF_DQ( self == NULL ) {}  /* QAC 3353 raises SEC R3.2.2 */
        else
        {
            self->i_lock_v_table->Unlock( self->i_lock );
        }
        /* <-QAC 3353 */
    }
    return  e;
}


/***********************************************************************
* Function: R_PFV_DMAC_StopTransfer
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_PFV_DMAC_StopTransfer( int_fast32_t const  pfv_channel )
{
    errnum_t              e;
    errnum_t              ee;
    pfv_dmac_t*           self = NULL;
    bool_t                was_enabled = false;
    r_ospl_async_state_t  state;

#if PFV_PARAMETER_CHECK
    IF ( (pfv_channel < PFV_CHANNEL_MIN) || (pfv_channel > PFV_CHANNEL_MAX) )
    {
        e = E_OTHERS;
        goto  fin;
    }
#endif

    self = &gs_pfv_dmac_contexts.channel[ pfv_channel ];

    ASSERT_R( self->async_status.State != R_OSPL_UNINITIALIZED,  e=E_STATE; goto fin );

    was_enabled = self->i_lock_v_table->Lock( self->i_lock );


    e = 0;
    R_AVOID_UNSAFE_ALWAYS_WARNING( e );

    state = self->async_status.State;
    if ( (state == R_OSPL_RUNNING)  ||
            (state == R_OSPL_INTERRUPTING)  ||
            (state == R_OSPL_INTERRUPTED) )
    {
        if ( IS( self->is_used_input_DMAC ) )
        {
            ee= R_DMAC_RM_StopAsRegisterMode( self->channel_of_input_DMAC );
            IF ( (ee != 0) && (e == 0) )
            {
                e = ee;
            }
        }

        if ( IS( self->is_used_output_DMAC ) )
        {
            ee= R_DMAC_RM_StopAsRegisterMode( self->channel_of_output_DMAC );
            IF ( (ee != 0) && (e == 0) )
            {
                e = ee;
            }
        }

        R_PFV_DMAC_ExitRunning_Sub( pfv_channel, self );
    }

#if PFV_PARAMETER_CHECK
fin:
#endif
    if ( IS( was_enabled ) )
    {
        /* ->QAC 3353 : "self" is always assigned, if "was_enabled" is true */
        IF_DQ( self == NULL ) {}  /* QAC 3353 raises SEC R3.2.2 */
        else
        {
            self->i_lock_v_table->Unlock( self->i_lock );
        }
        /* <-QAC 3353 */
    }
    return  e;
}


/***********************************************************************
* Function: R_PFV_DMAC_OnInterrupting
*    This function is run in interrupt context.
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_PFV_DMAC_OnInterrupting( const r_ospl_interrupt_t* const  InterruptSource )
{
    errnum_t  e;
    pfv_dmac_interrupt_unit_t  unit;

    e= R_Userdef_PFV_DMAC_GetInterruptUnit( InterruptSource, &unit );
    IF ( e != 0 )
    {
        goto fin;
    }

    if ( unit == PFV_INTERRUPT_UNIT_PFV )
    {
        e= R_PFV_OnInterrupting( InterruptSource );
        IF ( e != 0 )
        {
            goto fin;
        }
    }
    else
    {
        ASSERT_D( unit == PFV_INTERRUPT_UNIT_DMAC,  e=E_OTHERS; goto fin );

        e= R_DMAC_RM_OnInterrupting( InterruptSource );
        IF ( e != 0 )
        {
            goto fin;
        }
    }

    e=0;
fin:
    return  e;
}


/***********************************************************************
* Function: R_PFV_DMAC_OnInterrupted
*    This function is run in task context.
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_PFV_DMAC_OnInterrupted( int_fast32_t const  pfv_channel )
{
    pfv_dmac_t*      self;
    errnum_t         e;
    errnum_t         ee;
    r_ospl_async_t*  nofity_async = NULL;

    self = &gs_pfv_dmac_contexts.channel[ pfv_channel ];

    e=0;
    R_AVOID_UNSAFE_ALWAYS_WARNING( e );

    if ( self->async_status.State == R_OSPL_INTERRUPTED )
    {
        ee= R_PFV_OnInterrupted( pfv_channel );
        e= R_OSPL_MergeErrNum( ee, e );

        if ( self->async_status_of_input_DMAC != NULL )
        {
            ee= R_DMAC_RM_StopAsRegisterMode( self->channel_of_input_DMAC );
            e= R_OSPL_MergeErrNum( ee, e );
        }

        if ( self->async_status_of_output_DMAC != NULL )
        {
            ee= R_DMAC_RM_StopAsRegisterMode( self->channel_of_output_DMAC );
            e= R_OSPL_MergeErrNum( ee, e );
        }
    }

    if ( (self->async_status_of_input_DMAC != NULL)  &&
            (self->async_status_of_input_DMAC->State == R_OSPL_INTERRUPTED) )
    {
        ee= R_DMAC_RM_OnInterrupted( self->channel_of_input_DMAC );
        e= R_OSPL_MergeErrNum( ee, e );

        if ( self->async_status_of_input_DMAC->State == R_OSPL_RUNNABLE )
        {
            if ( ! self->is_used_output_DMAC )
            {
                nofity_async = self->interrupt_callback_caller.Async;
                R_PFV_DMAC_ExitRunning_Sub( pfv_channel, self );
            }
            ee = self->async_of_input_DMAC.ReturnValue;
            e= R_OSPL_MergeErrNum( ee, e );
        }
    }

    if ( (self->async_status_of_output_DMAC != NULL)  &&
            (self->async_status_of_output_DMAC->State == R_OSPL_INTERRUPTED) )
    {
        ee= R_DMAC_RM_OnInterrupted( self->channel_of_output_DMAC );
        e= R_OSPL_MergeErrNum( ee, e );

        if ( self->async_status_of_output_DMAC->State == R_OSPL_RUNNABLE )
        {
            nofity_async = self->interrupt_callback_caller.Async;
            R_PFV_DMAC_ExitRunning_Sub( pfv_channel, self );
            ee = self->async_of_output_DMAC.ReturnValue;
            e= R_OSPL_MergeErrNum( ee, e );
        }
    }

    if ( nofity_async != NULL )
    {
        R_OSPL_EVENT_Set( nofity_async->A_Thread, nofity_async->A_EventValue );
    }
    return  e;
}


/***********************************************************************
* Function: R_PFV_DMAC_ExitRunning_Sub
*
* Return Value:
*    None
************************************************************************/
static void  R_PFV_DMAC_ExitRunning_Sub( int_fast32_t const  pfv_channel,  pfv_dmac_t* const  self )
{
    pfv_t* const  pfv = &gs_pfv_contexts.channel[ pfv_channel ];

    IF_DQ( self == NULL )
    {
        goto fin;
    }

    R_OSPL_FLAG32_Clear( &pfv->interrupt_enables, ~PFV_INTERRUPT_LINE_PFVEIn );
    self->async_status.State = R_OSPL_RUNNABLE;

fin:
    return;
}


/***********************************************************************
* Function: R_PFV_GetAsyncStatus
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_PFV_GetAsyncStatus( int_fast32_t const  pfv_channel,
                                const pfv_async_status_t** const  out_Status )
{
    errnum_t     e;
    pfv_dmac_t*  self; /* = NULL; */ /* QAC 3197 */


#if PFV_PARAMETER_CHECK
    IF ( (pfv_channel < PFV_CHANNEL_MIN) || (pfv_channel > PFV_CHANNEL_MAX) )
    {
        e = E_OTHERS;
        goto  fin;
    }
#endif

    IF_DQ( out_Status == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }

    self = &gs_pfv_dmac_contexts.channel[ pfv_channel ];

    *out_Status = &self->async_status;

    e=0;
fin:
    return  e;
}


/***********************************************************************
* Function: R_PFV_DMAC_I_LOCK_Reset
*
* Return Value:
*    None
************************************************************************/
static void   R_PFV_DMAC_I_LOCK_Reset( r_pfv_dmac_i_lock_t* const  self )
{
    bool_t  b;

    IF_DQ( self == NULL )
    {
        goto fin;
    }

    self->is_lock = false;
    self->enables_PFV = 0;
    self->enables_InputDMAC = 0;
    self->enables_OutputDMAC = 0;
    b= R_PFV_DisableInterrupt( self->channel_num_PFV );
    R_UNREFERENCED_VARIABLE( b );  /* QAC 3200 : This is not error information */
    b= R_DMAC_RM_DisableInterrupt( self->channel_num_input_DMAC );
    R_UNREFERENCED_VARIABLE( b );  /* QAC 3200 : This is not error information */
    b= R_DMAC_RM_DisableInterrupt( self->channel_num_output_DMAC );
    R_UNREFERENCED_VARIABLE( b );  /* QAC 3200 : This is not error information */
    self->is_requested_finalize = false;

fin:
    return;
}


/***********************************************************************
* Function: R_PFV_DMAC_I_LOCK_Finalize
*
* Return Value:
*    Was interrupt enabled
************************************************************************/
static void  R_PFV_DMAC_I_LOCK_Finalize( int_fast32_t const  pfv_channel )
{
    if ( (pfv_channel >= 0)  &&  (pfv_channel <= PFV_CHANNEL_MAX) )
    {
        pfv_dmac_t* const  self = &gs_pfv_dmac_contexts.channel[ pfv_channel ];
        bool_t             b;

        ASSERT_D( self->is_i_lock_master, R_NOOP() );

        R_PFV_DMAC_I_LOCK_Reset( self->i_lock );
        b= R_PFV_DMAC_I_LOCK_Replace( pfv_channel, NULL, NULL );
        R_UNREFERENCED_VARIABLE( b );  /* QAC 3200 : This is not error information */
        self->is_i_lock_master = false;
    }
}


/***********************************************************************
* Function: R_PFV_DMAC_I_LOCK_Lock
*
* Return Value:
*    Was interrupt enabled
************************************************************************/
static bool_t  R_PFV_DMAC_I_LOCK_Lock( void* const  self_ )
{
    bool_t        is_locked;
    bool_t        was_all_enabled;  /* = false; */ /* QAC 3197 */
    r_pfv_dmac_i_lock_t* const  self = (r_pfv_dmac_i_lock_t*) self_;

    IF_DQ( self == NULL )
    {
        is_locked = true;
        goto fin;
    }

    was_all_enabled = R_OSPL_DisableAllInterrupt();

    is_locked = self->is_lock;
    if ( ! is_locked )
    {
        bool_t  b;

        b= R_PFV_DisableInterrupt( self->channel_num_PFV );
        R_UNREFERENCED_VARIABLE( b );  /* QAC 3200 : This is not error information */

        b= R_DMAC_RM_DisableInterrupt( self->channel_num_input_DMAC );
        R_UNREFERENCED_VARIABLE( b );  /* QAC 3200 : This is not error information */

        b= R_DMAC_RM_DisableInterrupt( self->channel_num_output_DMAC );
        R_UNREFERENCED_VARIABLE( b );  /* QAC 3200 : This is not error information */

        self->is_lock = true;
    }

    if ( IS( was_all_enabled ) )
    {
        R_OSPL_EnableAllInterrupt();
    }

fin:
    return  ! is_locked;
}


/***********************************************************************
* Function: R_PFV_DMAC_I_LOCK_Unlock
*
* Return Value:
*    None
************************************************************************/
static void  R_PFV_DMAC_I_LOCK_Unlock( void* const  self_ )
{
    bool_t  was_all_enabled;  /* = false; */ /* QAC 3197 */
    r_pfv_dmac_i_lock_t* const  self = (r_pfv_dmac_i_lock_t*) self_;

    IF_DQ( self == NULL )
    {
        goto fin;
    }

    was_all_enabled = R_OSPL_DisableAllInterrupt();

    R_PFV_EnableInterrupt( self->channel_num_PFV );
    R_DMAC_RM_EnableInterrupt( self->channel_num_input_DMAC );
    R_DMAC_RM_EnableInterrupt( self->channel_num_output_DMAC );
    self->is_lock = false;

    if ( IS( was_all_enabled ) )
    {
        R_OSPL_EnableAllInterrupt();
    }

fin:
    return;
}


/***********************************************************************
* Function: R_PFV_DMAC_I_LOCK_RequestFinalize
*
* Return Value:
*    None
************************************************************************/
static void   R_PFV_DMAC_I_LOCK_RequestFinalize( void* const  self_ )
{
    r_pfv_dmac_i_lock_t* const  self = (r_pfv_dmac_i_lock_t*) self_;

    IF_DQ( self == NULL )
    {
        goto fin;
    }

    self->is_requested_finalize = true;

fin:
    return;
}


/***********************************************************************
* Function: R_PFV_DMAC_I_LOCK_GetRootChannelNum
*
* Return Value:
*    None
************************************************************************/
static int_fast32_t  R_PFV_DMAC_I_LOCK_GetRootChannelNum( const void* const  self_ )
{
    int_fast32_t  root_channel_num;
    r_pfv_dmac_i_lock_t* const  self = (r_pfv_dmac_i_lock_t*) self_;

    IF_DQ( self == NULL )
    {
        root_channel_num = 0;
        goto fin;
    }

    root_channel_num = self->channel_num_PFV;

fin:
    return  root_channel_num;
}


/***********************************************************************
* Function: R_PFV_Initialize
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_PFV_Initialize( int_fast32_t const  pfv_channel,
                            pfv_config_t*  in_out_config )
{
    errnum_t    e;
    pfv_t*      self;
    uint32_t    value;
    volatile struct st_pfv*  reg;  /* register */
    static pfv_config_t  null_config = { 0 };

    enum { /*int_fast32_t*/  num_0xFF = 0xFF };


    /*[gs_DefaultIOFormat]*/
    pfv_io_format_t  gs_DefaultIOFormat;

    gs_DefaultIOFormat.flags =
        F_PFV_INPUT_FORMAT |
        F_PFV_INPUT_SWAP |
        F_PFV_OUTPUT_FORMAT |
        F_PFV_OUTPUT_SWAP |
        F_PFV_OUTPUT_ALPHA |
        F_PFV_IS_DITHER |
        F_PFV_IMAGE_WIDTH |
        F_PFV_IMAGE_HEIGHT;
    gs_DefaultIOFormat.input_format  = PFV_ARGB8888;
    gs_DefaultIOFormat.input_swap    = PFV_SWAP_ARGB8888;
    gs_DefaultIOFormat.output_format = PFV_ARGB8888;
    gs_DefaultIOFormat.output_swap   = PFV_SWAP_ARGB8888;
    gs_DefaultIOFormat.output_alpha  = num_0xFF;
    gs_DefaultIOFormat.is_dither     = true;
    gs_DefaultIOFormat.image_width   = 0;
    gs_DefaultIOFormat.image_height  = 0;


    R_IT_WILL_BE_NOT_CONST( in_out_config );


    /* Set "in_out_config" : null_config */
    if ( in_out_config == NULL )
    {
        in_out_config = &null_config;
    }


    e= R_Userdef_PFV_SetDefaultConfig( in_out_config );
    IF ( e != 0 )
    {
        goto fin;
    }


    e= R_Userdef_PFV_OnInitialize( pfv_channel );
    IF ( e != 0 )
    {
        goto fin;
    }


    e= R_PFV_STATIC_GetRegisterSet( pfv_channel, &reg );
    IF(e!=0)
    {
        goto fin;
    }
    IF_DQ( reg == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }

    self = &gs_pfv_contexts.channel[ pfv_channel ];
    self->is_initialized = true;


    /* Set "reset_color_matrixes" */
    IF ( IS_BIT_NOT_SET( in_out_config->flags, F_PFV_RESET_COLOR_MATRIXES ) )
    {
        e = E_OTHERS;
        goto fin;
    }
    self->reset_color_matrixes = in_out_config->reset_color_matrixes;


    /* Write reset value for 2nd initialize */
    reg->PFVCR            = 0x00000000U;   /* Set "PFVE" bitfield = 0 */
    reg->PFVICR           = 0x00000000u;
    value = reg->PFVISR;  /* Read from the register for clear */
    R_UNREFERENCED_VARIABLE( value );
    reg->PFVISR           = 0x00000000u;  /* Clear the interrupt status */
    reg->PFVACR           = 0x00000000u;
    reg->PFV_MTX_MODE     = 0x00000000u;
    reg->PFV_MTX_YG_ADJ0  = 0x00000000u;
    reg->PFV_MTX_YG_ADJ1  = 0x00000000u;
    reg->PFV_MTX_CBB_ADJ0 = 0x00000000u;
    reg->PFV_MTX_CBB_ADJ1 = 0x00000000u;
    reg->PFV_MTX_CRR_ADJ0 = 0x00000000u;
    reg->PFV_MTX_CRR_ADJ1 = 0x00000000u;
    reg->PFVSZR           = 0x00000000u;
    R_PFV_SET_REGISTER_BIT_FIELD( &reg->PFVCR, PFVCR, PFVE, TO_UNSIGNED( true ) );


    /* Set default I/O format */
    e= R_PFV_SetIOFormat( pfv_channel, &gs_DefaultIOFormat );
    IF ( e != 0 )
    {
        goto fin;
    }


    self->is_enabled_interrupt = true;
    R_OSPL_FLAG32_InitConst( &self->interrupt_enables );

    e = 0;
fin:
    return  e;
}


/***********************************************************************
* Function: R_PFV_Terminate
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_PFV_Terminate( int_fast32_t const  pfv_channel )
{
    errnum_t    e;
    pfv_t*      self;
    volatile struct st_pfv*  reg;  /* register */

    e= R_PFV_STATIC_GetRegisterSet( pfv_channel, &reg );
    IF(e!=0)
    {
        goto fin;
    }
    IF_DQ( reg == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }

    self = &gs_pfv_contexts.channel[ pfv_channel ];

    if ( ! self->is_initialized )
    {
        e = 0;
        goto fin;
    }

    self->is_initialized = false;

    reg->PFVCR = 0x00000000;  /* Set "PFVE" bitfield = 0 */

    /* Call "R_Userdef_PFV_OnFinalize" */
    e= R_Userdef_PFV_OnFinalize( pfv_channel, e );
    IF ( e != 0 )
    {
        goto fin;
    }

fin:
    return  e;
}


/***********************************************************************
* Function: R_PFV_SetIOFormat
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_PFV_SetIOFormat( int_fast32_t const  pfv_channel,
                             pfv_io_format_t* const  in_out_io_format )
{
    errnum_t    e;
    uint32_t    value;
    uint32_t    sub_value;
    volatile struct st_pfv*  reg;  /* register */

#if PFV_PARAMETER_CHECK
    pfv_t*    self;

    e= R_PFV_STATIC_GetRegisterSet( pfv_channel, &reg );
    IF(e!=0)
    {
        goto fin;
    }
    IF_DQ( reg == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }

    self = &gs_pfv_contexts.channel[ pfv_channel ];

    IF ( ! self->is_initialized )
    {
        e = E_OTHERS;
        goto  fin;
    }

    IF ( in_out_io_format == NULL )
    {
        e = E_OTHERS;
        goto  fin;
    }
#endif


    /* Set default value of "in_out_io_format->input_swap" */
    if ( IS_BIT_SET( in_out_io_format->flags,  F_PFV_INPUT_FORMAT ) )
    {
        if ( IS_BIT_NOT_SET( in_out_io_format->flags,  F_PFV_INPUT_SWAP ) )
        {
            switch ( in_out_io_format->input_format )
            {
            case  PFV_RGB888:
                in_out_io_format->input_swap = PFV_SWAP_ARGB8888;
                break;

            case  PFV_RGB565:
                in_out_io_format->input_swap = PFV_SWAP_RGB565_PIXEL10;
                break;

            default:
#if PFV_PARAMETER_CHECK
                ASSERT_R( in_out_io_format->input_format == PFV_YCbCr422,
                          e=E_OTHERS; goto fin );
#endif
                in_out_io_format->input_swap = PFV_SWAP_YCbCr422_Y1CrY0Cb;
                break;
            }

            in_out_io_format->flags |= F_PFV_INPUT_SWAP;
        }
    }


    /* Set default value of "in_out_io_format->output_swap" */
    if ( IS_BIT_SET( in_out_io_format->flags, F_PFV_OUTPUT_FORMAT ) )
    {
        if ( IS_BIT_NOT_SET( in_out_io_format->flags, F_PFV_OUTPUT_SWAP ) )
        {
            switch ( in_out_io_format->output_format )
            {
            case  PFV_ARGB8888:
                in_out_io_format->output_swap = PFV_SWAP_ARGB8888;
                break;

            case  PFV_RGB565:
                in_out_io_format->output_swap = PFV_SWAP_RGB565_PIXEL10;
                break;

            default:
#if PFV_PARAMETER_CHECK
                ASSERT_R( in_out_io_format->output_format == PFV_YCbCr422,
                          e=E_OTHERS; goto fin );
#endif
                in_out_io_format->output_swap = PFV_SWAP_YCbCr422_Y1CrY0Cb;
                break;
            }

            in_out_io_format->flags |= F_PFV_OUTPUT_SWAP;
        }
    }


    /* Write "PFVCR" register */
    {
        /* Read "value" */
        value = reg->PFVCR;

        /* Set value of "DTH_ON" bitfield */
        if ( IS_BIT_SET( in_out_io_format->flags, F_PFV_IS_DITHER ) )
        {
            sub_value = (uint32_t) in_out_io_format->is_dither;
#if PFV_PARAMETER_CHECK
            IF ( IS( R_PFV_IS_OVERFLOW_BIT_FIELD( PFVCR, DTH_ON, sub_value ) ) )
            {
                e = E_OTHERS;
                goto  fin;
            }
#endif
            R_PFV_SET_REGISTER_BIT_FIELD( &value, PFVCR, DTH_ON, sub_value );
        }

        /* Set value of "IFMT" bitfield */
        if ( IS_BIT_SET( in_out_io_format->flags, F_PFV_INPUT_FORMAT ) )
        {
            sub_value = (uint32_t) in_out_io_format->input_format;
#if PFV_PARAMETER_CHECK
            IF ( IS( R_PFV_IS_OVERFLOW_BIT_FIELD( PFVCR, IFMT, sub_value ) ) )
            {
                e = E_OTHERS;
                goto  fin;
            }
#endif
            R_PFV_SET_REGISTER_BIT_FIELD( &value, PFVCR, IFMT, sub_value );
        }

        /* Set value of "OFMT" bitfield */
        if ( IS_BIT_SET( in_out_io_format->flags, F_PFV_OUTPUT_FORMAT ) )
        {
            sub_value = (uint32_t) in_out_io_format->output_format;
#if PFV_PARAMETER_CHECK
            IF ( IS( R_PFV_IS_OVERFLOW_BIT_FIELD( PFVCR, OFMT, sub_value ) ) )
            {
                e = E_OTHERS;
                goto  fin;
            }
#endif
            R_PFV_SET_REGISTER_BIT_FIELD( &value, PFVCR, OFMT, sub_value );
        }

        /* Set value of "DINSWAP32" bitfield */
        if ( IS_BIT_SET( in_out_io_format->flags, F_PFV_INPUT_SWAP ) )
        {
            sub_value = (uint32_t) in_out_io_format->input_swap;
#if PFV_PARAMETER_CHECK
            IF ( IS( R_PFV_IS_OVERFLOW_BIT_FIELD( PFVCR, DINSWAP32, sub_value ) ) )
            {
                e = E_OTHERS;
                goto  fin;
            }

            if ( in_out_io_format->input_format == PFV_RGB565 )
            {
                IF ( ( in_out_io_format->input_swap != PFV_SWAP_RGB565_PIXEL01 ) &&
                     ( in_out_io_format->input_swap != PFV_SWAP_RGB565_PIXEL10 ) )
                {
                    e = E_OTHERS;
                    goto  fin;
                }
            }
#endif
            R_PFV_SET_REGISTER_BIT_FIELD( &value, PFVCR, DINSWAP32, sub_value );
        }

        /* Set value of "DOUTSWAP32" bitfield */
        if ( IS_BIT_SET( in_out_io_format->flags, F_PFV_OUTPUT_SWAP ) )
        {
            sub_value = (uint32_t) in_out_io_format->output_swap;
#if PFV_PARAMETER_CHECK
            IF ( IS( R_PFV_IS_OVERFLOW_BIT_FIELD( PFVCR, DOUTSWAP32, sub_value ) ) )
            {
                e = E_OTHERS;
                goto  fin;
            }

            if ( in_out_io_format->output_format == PFV_RGB565 )
            {
                IF ( ( in_out_io_format->output_swap != PFV_SWAP_RGB565_PIXEL01 ) &&
                     ( in_out_io_format->output_swap != PFV_SWAP_RGB565_PIXEL10 ) )
                {
                    e = E_OTHERS;
                    goto  fin;
                }
            }
#endif
            R_PFV_SET_REGISTER_BIT_FIELD( &value, PFVCR, DOUTSWAP32, sub_value );
        }

        /* Write "PFVCR" register */
        reg->PFVCR = value;
    }

    /* Write "PFVACR" register */
    if ( IS_BIT_NOT_SET( in_out_io_format->flags, F_PFV_OUTPUT_ALPHA ) )
    {
        enum { /*int_fast32_t*/  num_0xFF_no_transparent = 0xFF };

        in_out_io_format->output_alpha = num_0xFF_no_transparent;
        in_out_io_format->flags |= F_PFV_OUTPUT_ALPHA;
    }
    sub_value = (uint32_t) in_out_io_format->output_alpha;
#if PFV_PARAMETER_CHECK
    IF ( IS( R_PFV_IS_OVERFLOW_BIT_FIELD( PFVACR, ALPHA, sub_value ) ) )
    {
        e = E_OTHERS;
        goto  fin;
    }
#endif
    reg->PFVACR = sub_value;

    /* Write "PFVSZR" register */
    {
        /* Read "value" */
        value = reg->PFVSZR;

        /* Set value of "PFVSZX" bitfield */
        if ( IS_BIT_SET( in_out_io_format->flags, F_PFV_IMAGE_WIDTH ) )
        {
            sub_value = (uint32_t) in_out_io_format->image_width;
#if PFV_PARAMETER_CHECK
            IF ( IS( R_PFV_IS_OVERFLOW_BIT_FIELD( PFVSZR, PFVSZX, sub_value ) ) )
            {
                e = E_OTHERS;
                goto  fin;
            }
#endif
            R_PFV_SET_REGISTER_BIT_FIELD( &value, PFVSZR, PFVSZX, sub_value );
        }

        /* Set value of "PFVSZY" bitfield */
        if ( IS_BIT_SET( in_out_io_format->flags, F_PFV_IMAGE_HEIGHT ) )
        {
            sub_value = (uint32_t) in_out_io_format->image_height;
#if PFV_PARAMETER_CHECK
            IF ( IS( R_PFV_IS_OVERFLOW_BIT_FIELD( PFVSZR, PFVSZY, sub_value ) ) )
            {
                e = E_OTHERS;
                goto  fin;
            }
#endif
            R_PFV_SET_REGISTER_BIT_FIELD( &value, PFVSZR, PFVSZY, sub_value );
        }

        /* Write "PFVSZR" register */
        reg->PFVSZR = value;
    }


    /* Call "R_PFV_ResetImageColorMatrix" */
    {
        pfv_color_matrix_mode_t  color_matrix_mode;

        if ( IS_ANY_BITS_NOT_SET( in_out_io_format->flags,
                                  F_PFV_INPUT_FORMAT | F_PFV_OUTPUT_FORMAT ) )
        {
            uint32_t const  value_of_PFVCR = reg->PFVCR;

            in_out_io_format->input_format = (pfv_format_t)
                                             R_PFV_GET_REGISTER_BIT_FIELD( value_of_PFVCR, PFVCR, IFMT );
            in_out_io_format->output_format = (pfv_format_t)
                                              R_PFV_GET_REGISTER_BIT_FIELD( value_of_PFVCR, PFVCR, OFMT );
        }

        e= R_PFV_STATIC_GetColorMatrixMode( in_out_io_format->input_format,
                                            in_out_io_format->output_format, &color_matrix_mode );
        IF ( e != 0 )
        {
            goto fin;
        }

        e= R_PFV_ResetImageColorMatrix( pfv_channel, color_matrix_mode );
        IF ( e != 0 )
        {
            goto fin;
        }
    }


    e = 0;
fin:
    return  e;
}


/***********************************************************************
* Function: R_PFV_GetIOFormat
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_PFV_GetIOFormat( int_fast32_t const  pfv_channel,
                             pfv_io_format_t* const  out_io_format )
{
    errnum_t    e;
    uint32_t    value_of_PFVCR;
    uint32_t    value_of_PFVSZR;
    volatile struct st_pfv*  reg;  /* register */

#if PFV_PARAMETER_CHECK
    pfv_t*    self;

    e= R_PFV_STATIC_GetRegisterSet( pfv_channel, &reg );
    IF(e!=0)
    {
        goto fin;
    }
    IF_DQ( reg == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }

    self = &gs_pfv_contexts.channel[ pfv_channel ];

    IF ( ! self->is_initialized )
    {
        e = E_OTHERS;
        goto  fin;
    }
#endif

    IF_DQ( out_io_format == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }


    /* Set part of "out_io_format->flags" */
    out_io_format->flags =
        F_PFV_IS_DITHER |
        F_PFV_INPUT_FORMAT |
        F_PFV_OUTPUT_FORMAT |
        F_PFV_INPUT_SWAP |
        F_PFV_OUTPUT_SWAP |
        F_PFV_OUTPUT_ALPHA |
        F_PFV_IMAGE_WIDTH |
        F_PFV_IMAGE_HEIGHT;

    /* Set part of "out_io_format" */
    value_of_PFVCR = reg->PFVCR;
    out_io_format->is_dither = (bool_t)
                               R_PFV_GET_REGISTER_BIT_FIELD( value_of_PFVCR, PFVCR, DTH_ON );
    out_io_format->input_format = (pfv_format_t)
                                  R_PFV_GET_REGISTER_BIT_FIELD( value_of_PFVCR, PFVCR, IFMT );
    out_io_format->output_format = (pfv_format_t)
                                   R_PFV_GET_REGISTER_BIT_FIELD( value_of_PFVCR, PFVCR, OFMT );

    /* Set "out_io_format->input_swap" */
    out_io_format->input_swap = (pfv_swap_t)
                                R_PFV_GET_REGISTER_BIT_FIELD( value_of_PFVCR, PFVCR, DINSWAP32 );

    /* Set "out_io_format->output_swap" */
    out_io_format->output_swap = (pfv_swap_t)
                                 R_PFV_GET_REGISTER_BIT_FIELD( value_of_PFVCR, PFVCR, DOUTSWAP32 );

    /* Set part of "out_io_format" */
    out_io_format->output_alpha = (int_fast32_t) reg->PFVACR;

    value_of_PFVSZR = reg->PFVSZR;
    out_io_format->image_width = (int_fast32_t)
                                 R_PFV_GET_REGISTER_BIT_FIELD( value_of_PFVSZR, PFVSZR, PFVSZX );
    out_io_format->image_height = (int_fast32_t)
                                  R_PFV_GET_REGISTER_BIT_FIELD( value_of_PFVSZR, PFVSZR, PFVSZY );

    e = 0;
#if PFV_PARAMETER_CHECK
fin:
#endif
    return  e;
}


/***********************************************************************
* Function: R_PFV_SetImageColorMatrix
*
* Return Value:
*    Error code, 0=No error
************************************************************************/

static errnum_t  PFV_SetImageColorMatrix_Sub( int_fast32_t  pfv_channel,  pfv_color_matrix_mode_t  mode,
        uint16_t const  dc_offset_plus_128_array[ PFV_DC_OFFSET_ELEMENT_COUNT ],
        int16_t const   matrix_multiply_256_array[ PFV_COLOR_MATRIX_ELEMENT_COUNT ] );

errnum_t  R_PFV_SetImageColorMatrix( int_fast32_t const  pfv_channel,
                                     const pfv_color_matrix_t* const  offset_and_matrix )
{
    errnum_t  e;

#if PFV_PARAMETER_CHECK
    pfv_t*    self;
#endif

#if PFV_PARAMETER_CHECK
    self = &gs_pfv_contexts.channel[ pfv_channel ];

    IF ( ! self->is_initialized )
    {
        e = E_OTHERS;
        goto  fin;
    }
#endif

    IF_DQ( offset_and_matrix == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }


    e= PFV_SetImageColorMatrix_Sub(
           pfv_channel,
           offset_and_matrix->mode,
           offset_and_matrix->dc_offset_plus_128,
           offset_and_matrix->matrix_multiply_256 );
    IF ( e != 0 )
    {
        goto  fin;
    }

    e = 0;
fin:
    return  e;
}


static errnum_t  PFV_SetImageColorMatrix_Sub( int_fast32_t const  pfv_channel,
        pfv_color_matrix_mode_t const  mode,
        const uint16_t  dc_offset_plus_128_array[ PFV_DC_OFFSET_ELEMENT_COUNT ],
        const int16_t   matrix_multiply_256_array[ PFV_COLOR_MATRIX_ELEMENT_COUNT ] )
{
    errnum_t    e;
    uint32_t    value;
    uint32_t    sub_value;
    int32_t     signed_value;
    volatile struct st_pfv*  reg;  /* register */

    IF_DQ( dc_offset_plus_128_array  == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }
    IF_DQ( matrix_multiply_256_array == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }


    e= R_PFV_STATIC_GetRegisterSet( pfv_channel, &reg );
    IF(e!=0)
    {
        goto fin;
    }
    IF_DQ( reg == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }

    /* Write "PFV_MTX_MODE" register */
    /* Check "offset_and_matrix->mode" == "PFV_MTX_MODE" register */
#ifdef __ICCARM__
#pragma diag_suppress=Pa089
#endif
#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wenum-compare"
#endif
    R_STATIC_ASSERT( PFV_GBR_TO_GBR     == PFV_PFV_MTX_MD_GBR_TO_GBR, "" );
    R_STATIC_ASSERT( PFV_GBR_TO_YCbCr   == PFV_PFV_MTX_MD_GBR_TO_YCBCR, "" );
    R_STATIC_ASSERT( PFV_YCbCr_TO_GBR   == PFV_PFV_MTX_MD_YCBCR_TO_GBR, "" );
    R_STATIC_ASSERT( PFV_YCbCr_TO_YCbCr == PFV_PFV_MTX_MD_YCBCR_TO_YCBCR, "" );
#ifdef __ICCARM__
#pragma diag_default=Pa089
#endif
#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif

    reg->PFV_MTX_MODE = mode;


    /* Write "PFV_MTX_YG_ADJ0" register */
    {
        value = 0;

        /* Set value of "PFV_MTX_YG" bitfield */
        sub_value = dc_offset_plus_128_array[ PFV_DC_OFFSET_Y_OR_GREEN ];
#if PFV_PARAMETER_CHECK
        IF ( /* sub_value < PFV_DC_OFFSET_UINT_MIN || */
            sub_value > PFV_DC_OFFSET_UINT_MAX )
        {
            e = E_OTHERS;
            goto  fin;
        }
#endif
        R_PFV_SET_REGISTER_BIT_FIELD( &value, PFV_MTX_YG_ADJ0, PFV_MTX_YG, sub_value );

        /* Set value of "PFV_MTX_GG" bitfield */
        signed_value = matrix_multiply_256_array[ PFV_COLOR_MATRIX_GG ];
#if PFV_PARAMETER_CHECK
        IF ( (signed_value < PFV_MATRIX_MULTIPLY_INT_MIN) ||
             (signed_value > PFV_MATRIX_MULTIPLY_INT_MAX) )
        {
            e = E_OTHERS;
            goto  fin;
        }
#endif
        R_PFV_SET_REGISTER_BIT_FIELD( &value, PFV_MTX_YG_ADJ0, PFV_MTX_GG, (uint32_t) signed_value );

        /* Write "PFV_MTX_YG_ADJ0" register */
        reg->PFV_MTX_YG_ADJ0 = value;
    }

    /* Write "PFV_MTX_YG_ADJ1" register */
    {
        value = 0;

        /* Set value of "PFV_MTX_GB" bitfield */
        signed_value = matrix_multiply_256_array[ PFV_COLOR_MATRIX_GB ];
#if PFV_PARAMETER_CHECK
        IF ( (signed_value < PFV_MATRIX_MULTIPLY_INT_MIN) ||
             (signed_value > PFV_MATRIX_MULTIPLY_INT_MAX) )
        {
            e = E_OTHERS;
            goto  fin;
        }
#endif
        R_PFV_SET_REGISTER_BIT_FIELD( &value, PFV_MTX_YG_ADJ1, PFV_MTX_GB, (uint32_t) signed_value );

        /* Set value of "PFV_MTX_GR" bitfield */
        signed_value = matrix_multiply_256_array[ PFV_COLOR_MATRIX_GR ];
#if PFV_PARAMETER_CHECK
        IF ( (signed_value < PFV_MATRIX_MULTIPLY_INT_MIN) ||
             (signed_value > PFV_MATRIX_MULTIPLY_INT_MAX) )
        {
            e = E_OTHERS;
            goto  fin;
        }
#endif
        R_PFV_SET_REGISTER_BIT_FIELD( &value, PFV_MTX_YG_ADJ1, PFV_MTX_GR, (uint32_t) signed_value );

        /* Write "PFV_MTX_YG_ADJ1" register */
        reg->PFV_MTX_YG_ADJ1 = value;
    }

    /* Write "PFV_MTX_CBB_ADJ0" register */
    {
        value = 0;

        /* Set value of "PFV_MTX_B" bitfield */
        sub_value = dc_offset_plus_128_array[ PFV_DC_OFFSET_BLUE ];
#if PFV_PARAMETER_CHECK
        IF ( /* sub_value < gc_PFV_DC_OFFSET_UINT_MIN || */
            sub_value > PFV_DC_OFFSET_UINT_MAX )
        {
            e = E_OTHERS;
            goto  fin;
        }
#endif
        R_PFV_SET_REGISTER_BIT_FIELD( &value, PFV_MTX_CBB_ADJ0, PFV_MTX_B, (uint32_t) sub_value );

        /* Set value of "PFV_MTX_BG" bitfield */
        signed_value = matrix_multiply_256_array[ PFV_COLOR_MATRIX_BG ];
#if PFV_PARAMETER_CHECK
        IF ( (signed_value < PFV_MATRIX_MULTIPLY_INT_MIN) ||
             (signed_value > PFV_MATRIX_MULTIPLY_INT_MAX) )
        {
            e = E_OTHERS;
            goto  fin;
        }
#endif
        R_PFV_SET_REGISTER_BIT_FIELD( &value, PFV_MTX_CBB_ADJ0, PFV_MTX_BG, (uint32_t) signed_value );

        /* Write "PFV_MTX_CBB_ADJ0" register */
        reg->PFV_MTX_CBB_ADJ0 = value;
    }

    /* Write "PFV_MTX_CBB_ADJ1" register */
    {
        value = 0;

        /* Set value of "PFV_MTX_BB" bitfield */
        signed_value = matrix_multiply_256_array[ PFV_COLOR_MATRIX_BB ];
#if PFV_PARAMETER_CHECK
        IF ( (signed_value < PFV_MATRIX_MULTIPLY_INT_MIN) ||
             (signed_value > PFV_MATRIX_MULTIPLY_INT_MAX) )
        {
            e = E_OTHERS;
            goto  fin;
        }
#endif
        R_PFV_SET_REGISTER_BIT_FIELD( &value, PFV_MTX_CBB_ADJ1, PFV_MTX_BB, (uint32_t) signed_value );

        /* Set value of "PFV_MTX_BR" bitfield */
        signed_value = matrix_multiply_256_array[ PFV_COLOR_MATRIX_BR ];
#if PFV_PARAMETER_CHECK
        IF ( (signed_value < PFV_MATRIX_MULTIPLY_INT_MIN) ||
             (signed_value > PFV_MATRIX_MULTIPLY_INT_MAX) )
        {
            e = E_OTHERS;
            goto  fin;
        }
#endif
        R_PFV_SET_REGISTER_BIT_FIELD( &value, PFV_MTX_CBB_ADJ1, PFV_MTX_BR, (uint32_t) signed_value );

        /* Write "PFV_MTX_CBB_ADJ1" register */
        reg->PFV_MTX_CBB_ADJ1 = value;
    }

    /* Write "PFV_MTX_CRR_ADJ0" register */
    {
        value = 0;

        /* Set value of "PFV_MTX_R" bitfield */
        sub_value = dc_offset_plus_128_array[ PFV_DC_OFFSET_RED ];
#if PFV_PARAMETER_CHECK
        IF ( /* sub_value < PFV_DC_OFFSET_UINT_MIN || */
            sub_value > PFV_DC_OFFSET_UINT_MAX )
        {
            e = E_OTHERS;
            goto  fin;
        }
#endif
        R_PFV_SET_REGISTER_BIT_FIELD( &value, PFV_MTX_CRR_ADJ0, PFV_MTX_R, sub_value );

        /* Set value of "PFV_MTX_RG" bitfield */
        signed_value = matrix_multiply_256_array[ PFV_COLOR_MATRIX_RG ];
#if PFV_PARAMETER_CHECK
        IF ( (signed_value < PFV_MATRIX_MULTIPLY_INT_MIN) ||
             (signed_value > PFV_MATRIX_MULTIPLY_INT_MAX) )
        {
            e = E_OTHERS;
            goto  fin;
        }
#endif
        R_PFV_SET_REGISTER_BIT_FIELD( &value, PFV_MTX_CRR_ADJ0, PFV_MTX_RG, (uint32_t) signed_value );

        /* Write "PFV_MTX_CRR_ADJ0" register */
        reg->PFV_MTX_CRR_ADJ0 = value;
    }

    /* Write "PFV_MTX_CRR_ADJ1" register */
    {
        value = 0;

        /* Set value of "PFV_MTX_RB" bitfield */
        signed_value = matrix_multiply_256_array[ PFV_COLOR_MATRIX_RB ];
#if PFV_PARAMETER_CHECK
        IF ( (signed_value < PFV_MATRIX_MULTIPLY_INT_MIN) ||
             (signed_value > PFV_MATRIX_MULTIPLY_INT_MAX) )
        {
            e = E_OTHERS;
            goto  fin;
        }
#endif
        R_PFV_SET_REGISTER_BIT_FIELD( &value, PFV_MTX_CRR_ADJ1, PFV_MTX_RB, (uint32_t) signed_value );

        /* Set value of "PFV_MTX_RR" bitfield */
        signed_value = matrix_multiply_256_array[ PFV_COLOR_MATRIX_RR ];
#if PFV_PARAMETER_CHECK
        IF ( (signed_value < PFV_MATRIX_MULTIPLY_INT_MIN) ||
             (signed_value > PFV_MATRIX_MULTIPLY_INT_MAX) )
        {
            e = E_OTHERS;
            goto  fin;
        }
#endif
        R_PFV_SET_REGISTER_BIT_FIELD( &value, PFV_MTX_CRR_ADJ1, PFV_MTX_RR, (uint32_t) signed_value );

        /* Write "PFV_MTX_CRR_ADJ1" register */
        reg->PFV_MTX_CRR_ADJ1 = value;
    }

    e = 0;
#if PFV_PARAMETER_CHECK
fin:
#endif
    return  e;
}


/***********************************************************************
* Function: R_PFV_GetImageColorMatrix
*
* Return Value:
*    Error code, 0=No error
************************************************************************/

static errnum_t  PFV_GetImageColorMatrix_Sub( int_fast32_t  pfv_channel,  pfv_color_matrix_mode_t* out_mode,
        uint16_t* out_dc_offset_plus_128_array,  int16_t* out_matrix_multiply_256_array );

errnum_t  R_PFV_GetImageColorMatrix( int_fast32_t const  pfv_channel,
                                     pfv_color_matrix_t* const  out_offset_and_matrix )
{
    errnum_t  e;

#if PFV_PARAMETER_CHECK
    pfv_t*    self;


    IF ( (pfv_channel < PFV_CHANNEL_MIN) || (pfv_channel > PFV_CHANNEL_MAX) )
    {
        e = E_OTHERS;
        goto  fin;
    }

    self = &gs_pfv_contexts.channel[ pfv_channel ];

    IF ( ! self->is_initialized )
    {
        e = E_OTHERS;
        goto  fin;
    }
#endif

    IF_DQ( out_offset_and_matrix == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }


    e= PFV_GetImageColorMatrix_Sub(
           pfv_channel,
           &out_offset_and_matrix->mode,
           out_offset_and_matrix->dc_offset_plus_128,
           out_offset_and_matrix->matrix_multiply_256 );
    IF ( e != 0 )
    {
        goto  fin;
    }

    e = 0;
fin:
    return  e;
}


STATIC_INLINE int_fast32_t  PFV_SetSignForColorMatrixMultiplyValue( uint_fast32_t const  uint11_value );
/* MISRA 8.1 Advice */
STATIC_INLINE int_fast32_t  PFV_SetSignForColorMatrixMultiplyValue( uint_fast32_t const  uint11_value )
{
    enum { /*int_fast32_t*/  num_21 = 21 };

    /* Copy bit10 to bit31..11 */
    /* ->MISRA 12.7 */
    return  ( ( (int_fast32_t) uint11_value ) << num_21 ) >> num_21;
    /* <-MISRA 12.7 */
}



static errnum_t  PFV_GetImageColorMatrix_Sub( int_fast32_t const  pfv_channel,
        pfv_color_matrix_mode_t* const  out_mode,
        uint16_t* const  out_dc_offset_plus_128_array,
        int16_t* const  out_matrix_multiply_256_array )
{
    errnum_t    e;
    uint32_t    value;
    uint32_t    sub_value;
    volatile struct st_pfv*  reg;  /* register */

    IF_DQ( out_mode                      == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }
    IF_DQ( out_dc_offset_plus_128_array  == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }
    IF_DQ( out_matrix_multiply_256_array == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }


    e= R_PFV_STATIC_GetRegisterSet( pfv_channel, &reg );
    IF(e!=0)
    {
        goto fin;
    }
    IF_DQ( reg == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }

    /* Set "*out_mode" */
    *out_mode = (pfv_color_matrix_mode_t) reg->PFV_MTX_MODE;


    /* Set "out_dc_offset_plus_128_array", "out_matrix_multiply_256_array" */

    value = reg->PFV_MTX_YG_ADJ0;

    out_dc_offset_plus_128_array[ PFV_DC_OFFSET_Y_OR_GREEN ] = (uint16_t)
            R_PFV_GET_REGISTER_BIT_FIELD( value, PFV_MTX_YG_ADJ0, PFV_MTX_YG );

    sub_value = R_PFV_GET_REGISTER_BIT_FIELD( value, PFV_MTX_YG_ADJ0, PFV_MTX_GG );
    out_matrix_multiply_256_array[ PFV_COLOR_MATRIX_GG ] = (int16_t)
            PFV_SetSignForColorMatrixMultiplyValue( sub_value );

    value = reg->PFV_MTX_YG_ADJ1;

    sub_value = R_PFV_GET_REGISTER_BIT_FIELD( value, PFV_MTX_YG_ADJ1, PFV_MTX_GB );
    out_matrix_multiply_256_array[ PFV_COLOR_MATRIX_GB ] = (int16_t)
            PFV_SetSignForColorMatrixMultiplyValue( sub_value );

    sub_value = R_PFV_GET_REGISTER_BIT_FIELD( value, PFV_MTX_YG_ADJ1, PFV_MTX_GR );
    out_matrix_multiply_256_array[ PFV_COLOR_MATRIX_GR ] = (int16_t)
            PFV_SetSignForColorMatrixMultiplyValue( sub_value );

    value = reg->PFV_MTX_CBB_ADJ0;

    out_dc_offset_plus_128_array[ PFV_DC_OFFSET_BLUE ] = (uint16_t)
            R_PFV_GET_REGISTER_BIT_FIELD( value, PFV_MTX_CBB_ADJ0, PFV_MTX_B );

    sub_value = R_PFV_GET_REGISTER_BIT_FIELD( value, PFV_MTX_CBB_ADJ0, PFV_MTX_BG );
    out_matrix_multiply_256_array[ PFV_COLOR_MATRIX_BG ] = (int16_t)
            PFV_SetSignForColorMatrixMultiplyValue( sub_value );

    value = reg->PFV_MTX_CBB_ADJ1;

    sub_value = R_PFV_GET_REGISTER_BIT_FIELD( value, PFV_MTX_CBB_ADJ1, PFV_MTX_BB );
    out_matrix_multiply_256_array[ PFV_COLOR_MATRIX_BB ] = (int16_t)
            PFV_SetSignForColorMatrixMultiplyValue( sub_value );

    sub_value = R_PFV_GET_REGISTER_BIT_FIELD( value, PFV_MTX_CBB_ADJ1, PFV_MTX_BR );
    out_matrix_multiply_256_array[ PFV_COLOR_MATRIX_BR ] = (int16_t)
            PFV_SetSignForColorMatrixMultiplyValue( sub_value );

    value = reg->PFV_MTX_CRR_ADJ0;

    out_dc_offset_plus_128_array[ PFV_DC_OFFSET_RED ] = (uint16_t)
            R_PFV_GET_REGISTER_BIT_FIELD( value, PFV_MTX_CRR_ADJ0, PFV_MTX_R );

    sub_value = R_PFV_GET_REGISTER_BIT_FIELD( value, PFV_MTX_CRR_ADJ0, PFV_MTX_RG );
    out_matrix_multiply_256_array[ PFV_COLOR_MATRIX_RG ] = (int16_t)
            PFV_SetSignForColorMatrixMultiplyValue( sub_value );

    value = reg->PFV_MTX_CRR_ADJ1;

    sub_value = R_PFV_GET_REGISTER_BIT_FIELD( value, PFV_MTX_CRR_ADJ1, PFV_MTX_RB );
    out_matrix_multiply_256_array[ PFV_COLOR_MATRIX_RB ] = (int16_t)
            PFV_SetSignForColorMatrixMultiplyValue( sub_value );

    sub_value = R_PFV_GET_REGISTER_BIT_FIELD( value, PFV_MTX_CRR_ADJ1, PFV_MTX_RR );
    out_matrix_multiply_256_array[ PFV_COLOR_MATRIX_RR ] = (int16_t)
            PFV_SetSignForColorMatrixMultiplyValue( sub_value );

    e = 0;
fin:
    return  e;
}


/***********************************************************************
* Function: R_PFV_ResetImageColorMatrix
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_PFV_ResetImageColorMatrix( int_fast32_t const  pfv_channel,
                                       pfv_color_matrix_mode_t const  mode )
{
    errnum_t    e;
    pfv_t*      self;

#if PFV_PARAMETER_CHECK
    IF ( (pfv_channel < PFV_CHANNEL_MIN) || (pfv_channel > PFV_CHANNEL_MAX) )
    {
        e = E_OTHERS;
        goto  fin;
    }
#endif

    self = &gs_pfv_contexts.channel[ pfv_channel ];

#if PFV_PARAMETER_CHECK
    IF ( ! self->is_initialized )
    {
        e = E_OTHERS;
        goto  fin;
    }

    IF ( mode >= PFV_MATRIX_MODE_COUNT )
    {
        e = E_OTHERS;
        goto  fin;
    }
#endif


    e= PFV_SetImageColorMatrix_Sub(
           pfv_channel,
           mode,
           self->reset_color_matrixes[ mode ].dc_offset_plus_128,
           self->reset_color_matrixes[ mode ].matrix_multiply_256 );
    IF ( e != 0 )
    {
        goto  fin;
    }

    e = 0;
fin:
    return  e;
}


/***********************************************************************
* Function: R_PFV_GetFilledByteInInputFIFO
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_PFV_GetFilledByteInInputFIFO( int_fast32_t const  pfv_channel,
        int_fast32_t* const  out_filled_byte )
{
    errnum_t  e;
    volatile struct st_pfv*  reg;  /* register */

    IF_DQ( out_filled_byte == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }


    e= R_PFV_STATIC_GetRegisterSet( pfv_channel, &reg );
    IF(e!=0)
    {
        goto fin;
    }
    IF_DQ( reg == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }

    *out_filled_byte = (int_fast32_t) reg->PFVIFSR;

    e = 0;
fin:
    return  e;
}


/***********************************************************************
* Function: R_PFV_WritePixelDataViaPIO
*
* Return Value:
*    Error code, 0=No error
************************************************************************/

#if PFV_PARAMETER_CHECK
static errnum_t  PFV_CheckConvertableForPIO( int_fast32_t  pfv_channel,
        int_fast32_t  read_bit_count,  int_fast32_t  write_bit_count );
#endif

errnum_t  R_PFV_WritePixelDataViaPIO( int_fast32_t const  pfv_channel,
                                      uint32_t const  pixel_data,  int_fast32_t const  bit_count )
{
    errnum_t  e;
    volatile struct st_pfv*  reg;  /* register */
    enum { /*int_fast32_t*/  num_16 = 16 };
    enum { /*int_fast32_t*/  num_32 = 32 };

#if PFV_PARAMETER_CHECK
    pfv_t*    self;
#endif

    e= R_PFV_STATIC_GetRegisterSet( pfv_channel, &reg );
    IF(e!=0)
    {
        goto fin;
    }
    IF_DQ( reg == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }

#if PFV_PARAMETER_CHECK
    self = &gs_pfv_contexts.channel[ pfv_channel ];

    IF ( ! self->is_initialized )
    {
        e = E_OTHERS;
        goto  fin;
    }

    e= PFV_CheckConvertableForPIO( pfv_channel, 0, bit_count );
    IF ( e != 0 )
    {
        goto  fin;
    }
#endif

    /* Check input FIFO is not full */
    IF ( reg->PFVIFSR >= (uint32_t)( PFV_INPUT_FIFO_FULL_BYTE - R_Mod_8s( bit_count ) ) )
    {
        e = E_OTHERS;
        goto  fin;
    }

    /* Write input pixel(s) data */
    if ( bit_count == num_16 )
    {
        /* ->MISRA 11.4 */ /* ->SEC R2.7.1 */
        R_PFV_SET_REGISTER_BIT_FIELD_WITH_REG_WIDTH( &reg->PFVID0, PFVID, ID, pixel_data, 16 );
        /* <-MISRA 11.4 */ /* <-SEC R2.7.1 */
    }
    else
    {
        IF ( bit_count != num_32 )
        {
            e = E_OTHERS;
            goto  fin;
        }

        reg->PFVID0 = pixel_data;
    }

    e = 0;
fin:
    return  e;
}


/***********************************************************************
* Function: R_PFV_GetFilledByteInOutputFIFO
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_PFV_GetFilledByteInOutputFIFO( int_fast32_t const  pfv_channel,
        int_fast32_t* const  out_filled_byte )
{
    errnum_t  e;
    volatile struct st_pfv*  reg;  /* register */

    IF_DQ( out_filled_byte == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }


    e= R_PFV_STATIC_GetRegisterSet( pfv_channel, &reg );
    IF(e!=0)
    {
        goto fin;
    }
    IF_DQ( reg == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }

    *out_filled_byte = (int_fast32_t) reg->PFVOFSR;

    e = 0;
fin:
    return  e;
}


/***********************************************************************
* Function: R_PFV_ReadPixelDataViaPIO
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_PFV_ReadPixelDataViaPIO( int_fast32_t const  pfv_channel,
                                     uint32_t* const  out_pixel_data,  int_fast32_t const  bit_count )
{
    errnum_t  e;
    volatile struct st_pfv*  reg;  /* register */
#if PFV_PARAMETER_CHECK
    pfv_t*    self;
#endif
    enum { /*int_fast32_t*/  num_16 = 16 };
    enum { /*int_fast32_t*/  num_32 = 32 };

    IF_DQ( out_pixel_data == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }


    e= R_PFV_STATIC_GetRegisterSet( pfv_channel, &reg );
    IF(e!=0)
    {
        goto fin;
    }
    IF_DQ( reg == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }

#if PFV_PARAMETER_CHECK
    self = &gs_pfv_contexts.channel[ pfv_channel ];

    IF ( ! self->is_initialized )
    {
        e = E_OTHERS;
        goto  fin;
    }

    e= PFV_CheckConvertableForPIO( pfv_channel, bit_count, 0 );
    IF ( e != 0 )
    {
        goto  fin;
    }
#endif

    /* Check output FIFO is not empty */
    IF ( reg->PFVOFSR < (uint32_t) R_Mod_8s( bit_count ) )
    {
        e = E_OTHERS;
        goto  fin;
    }

    /* Read output pixel(s) data */
    if ( bit_count == num_16 )
    {
        /* ->MISRA 11.4 */ /* ->SEC R2.7.1 */
        *out_pixel_data = (uint32_t) R_PFV_GET_REGISTER_BIT_FIELD_WITH_REG_WIDTH(
                              reg->PFVOD0, PFVOD, OD, 16 );
        /* <-MISRA 11.4 */ /* <-SEC R2.7.1 */
    }
    else
    {
        IF ( bit_count != num_32 )
        {
            e = E_OTHERS;
            goto  fin;
        }

        *out_pixel_data = reg->PFVOD0;
    }

    e = 0;
fin:
    return  e;
}


/***********************************************************************
* Function: PFV_CheckConvertableForPIO
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
#if PFV_PARAMETER_CHECK
static errnum_t  PFV_CheckConvertableForPIO( int_fast32_t const  pfv_channel,
        int_fast32_t const  read_bit_count,  int_fast32_t const  write_bit_count )
{
    errnum_t                 e;
    pfv_io_format_t          io_format;
    pfv_color_matrix_t       offset_and_matrix;
    pfv_color_matrix_mode_t  color_matrix_mode;


    /* Set "io_format", "offset_and_matrix" : Register values */
    e= R_PFV_GetIOFormat( pfv_channel, &io_format );
    IF ( e != 0 )
    {
        goto  fin;
    }

    e= R_PFV_GetImageColorMatrix( pfv_channel, &offset_and_matrix );
    IF ( e != 0 )
    {
        goto  fin;
    }


    /* Check "offset_and_matrix->mode" */
    e= R_PFV_STATIC_GetColorMatrixMode( io_format.input_format, io_format.output_format,
                                        &color_matrix_mode );
    IF ( e != 0 )
    {
        goto  fin;
    }
    IF ( color_matrix_mode != offset_and_matrix.mode )
    {
        e = E_OTHERS;
        goto  fin;
    }


    /* Check "read_bit_count" */
    if ( read_bit_count != 0 )
    {
        e= PFV_CheckIO_BitCount( io_format.output_format,  read_bit_count );
        IF ( e != 0 )
        {
            goto  fin;
        }
    }


    /* Check "write_bit_count" */
    if ( write_bit_count != 0 )
    {
        e= PFV_CheckIO_BitCount( io_format.input_format,  write_bit_count );
        {
            goto  fin;
        }
    }

    e = 0;
fin:
    return  e;
}
#endif


/***********************************************************************
* Function: R_PFV_STATIC_GetRegisterSet
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
/* ->MISRA 8.10 : External linkage is for Debug configuration *//* ->SEC M2.2.3 */
errnum_t  R_PFV_STATIC_GetRegisterSet( int_fast32_t const  pfv_channel,
                                       volatile struct st_pfv** const  out_PFV )
/* <-MISRA 8.10 */ /* <-SEC M2.2.3 */
{
    errnum_t  e;

    /* ->MISRA 11.3 */ /* ->SEC R2.7.1 */
#if IODEFINE_H_VERSION >= 100
    static struct st_pfv* const  pfv_array[ PFV_COUNT ] = PFV_ADDRESS_LIST;
#elif IODEFINE_H_VERSION >= 60
    static struct st_pfv* const  pfv_array[ PFV_COUNT ] = { &PFV0, &PFV1 };
#else
    static volatile struct st_pfv* const  pfv_array[ PFV_COUNT ] = { &PFV0, &PFV1 };
#endif
    /* <-MISRA 11.3 */ /* <-SEC R2.7.1 */

    IF_DQ( out_PFV == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }

    IF ( (pfv_channel < PFV_CHANNEL_MIN) || (pfv_channel > PFV_CHANNEL_MAX) )
    {
        e=E_OTHERS;
        goto fin;
    }

    *out_PFV = pfv_array[ pfv_channel - PFV_CHANNEL_MIN ];

    e=0;
fin:
    return  e;
}


/***********************************************************************
* Function: R_PFV_STATIC_GetColorMatrixMode
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_PFV_STATIC_GetColorMatrixMode( pfv_format_t const  input_format,
        pfv_format_t const  output_format,
        pfv_color_matrix_mode_t* const  out_color_matrix_mode )
{
    errnum_t    e;
    bool_t      is_input_YCbCr;
    bool_t      is_output_YCbCr;

    IF_DQ( out_color_matrix_mode == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }


    e= pfv_format_t_Is_YCbCr( input_format, &is_input_YCbCr );
    IF ( e != 0 )
    {
        goto  fin;
    }

    e= pfv_format_t_Is_YCbCr( output_format, &is_output_YCbCr );
    IF ( e != 0 )
    {
        goto  fin;
    }

    if ( IS( is_input_YCbCr ) )
    {
        if ( IS( is_output_YCbCr ) )
        {
            *out_color_matrix_mode = PFV_YCbCr_TO_YCbCr;
        }
        else
        {
            *out_color_matrix_mode = PFV_YCbCr_TO_GBR;
        }
    }
    else
    {
        if ( IS( is_output_YCbCr ) )
        {
            *out_color_matrix_mode = PFV_GBR_TO_YCbCr;
        }
        else
        {
            *out_color_matrix_mode = PFV_GBR_TO_GBR;
        }
    }

    e = 0;
fin:
    return  e;
}


/***********************************************************************
* Function: R_PFV_OnInterrupting
*    This function is run in interrupt context.
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_PFV_OnInterrupting( const r_ospl_interrupt_t* const  InterruptSource )
{
    errnum_t                 e;
    int_fast32_t             pfv_channel;
    uint32_t                 status;  /* Bitfield */
    uint32_t                 clear_status;  /* Bitfield */
    volatile struct st_pfv*  reg;  /* register */
    pfv_dmac_t*              self;  /* = NULL; */ /* QAC 3197 */
#ifdef __ICCARM__
#pragma diag_suppress=Pa089
#endif
    static const uint32_t    mask_of_PFVISR =
        PFV__MASK32__PFVISR__IFOVF |
        PFV__MASK32__PFVISR__OFUDF |
        PFV__MASK32__PFVISR__IFEMP |
        PFV__MASK32__PFVISR__OFFUL;
    static const uint32_t   acceptable_in_PFVISR =
        PFV__MASK32__PFVISR__IFOVF |
        PFV__MASK32__PFVISR__OFUDF;
#ifdef __ICCARM__
#pragma diag_default=Pa089
#endif

    IF_DQ( InterruptSource == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }


    pfv_channel = InterruptSource->ChannelNum;
    self = &gs_pfv_dmac_contexts.channel[ pfv_channel ];

    ASSERT_R( self->async_status.State == R_OSPL_INTERRUPTING,  e=E_STATE; goto fin );


    /* Get interrupt status */
    e= R_PFV_STATIC_GetRegisterSet( pfv_channel, &reg );
    IF(e!=0)
    {
        goto fin;
    }
    IF_DQ( reg == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }

    status = reg->PFVISR;

    status &= acceptable_in_PFVISR;
    R_OSPL_FLAG32_Set( &self->async_status.InterruptFlags, status );


    /* Clear interrupt status */
    clear_status = ~status & mask_of_PFVISR;
    reg->PFVISR = clear_status;


    /* Set state */
    self->async_status.State = R_OSPL_INTERRUPTED;


    e=0;
fin:
    return  e;
}


/***********************************************************************
* Function: R_PFV_OnInterrupted
*    This function is run in task context.
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_PFV_OnInterrupted( int_fast32_t const  pfv_channel )
{
    errnum_t  e;
    uint32_t  status;
    pfv_dmac_t*      self = NULL;
    r_ospl_async_t*  notify_async = NULL;
    bool_t           was_enabled = false;


#if PFV_PARAMETER_CHECK
    IF ( (pfv_channel < PFV_CHANNEL_MIN) || (pfv_channel > PFV_CHANNEL_MAX) )
    {
        e = E_OTHERS;
        goto  fin;
    }
#endif


    self = &gs_pfv_dmac_contexts.channel[ pfv_channel ];

    was_enabled = self->i_lock_v_table->Lock( self->i_lock );

    ASSERT_R( self->async_status.State != R_OSPL_UNINITIALIZED,  e=E_STATE; goto fin );
#ifdef __ICCARM__
#pragma diag_suppress=Pa089
#endif

    status = R_OSPL_FLAG32_GetAndClear( &self->async_status.InterruptFlags );
    IF ( IS_ANY_BITS_SET( status, PFV__MASK32__PFVISR__IFOVF | PFV__MASK32__PFVISR__OFUDF )  &&
         (self->interrupt_callback_caller.Async != NULL) )
    {
        notify_async = self->interrupt_callback_caller.Async;
        R_PFV_DMAC_ExitRunning_Sub( pfv_channel, self );
        e = E_PFV_HW_ERROR;
        goto fin;
    }
#ifdef __ICCARM__
#pragma diag_default=Pa089
#endif

    e=0;
fin:
    if ( IS( was_enabled ) )
    {
        IF_DQ( self == NULL ) { }
        else
        {
            self->i_lock_v_table->Unlock( self->i_lock );
        }
    }

    if ( notify_async != NULL )
    {
        R_OSPL_EVENT_Set( notify_async->A_Thread, notify_async->A_EventValue );
    }
    return  e;
}


/***********************************************************************
* Function: R_PFV_DMAC_I_LOCK_Replace
*    (i_lock != NULL) Get I-Lock, (i_lock == NULL) Release I-Lock
*
* Return Value:
*    (i_lock != NULL) Whether I-Lock got, (i_lock == NULL) "true"
************************************************************************/
bool_t  R_PFV_DMAC_I_LOCK_Replace( int_fast32_t const  pfv_channel,
                                   void* const  i_lock,  const r_ospl_i_lock_vtable_t* const  i_lock_v_table )
{
    bool_t  ret = false;

    if ( (pfv_channel >= PFV_CHANNEL_MIN) && (pfv_channel <= PFV_CHANNEL_MAX) )
    {
        pfv_dmac_t* const  self = &gs_pfv_dmac_contexts.channel[ pfv_channel ];

        ASSERT_R( self->async_status.State == R_OSPL_UNINITIALIZED,  ret = false; goto fin );

        if ( i_lock != NULL )
        {
            if ( self->i_lock == NULL )
            {
                self->i_lock = i_lock;
                self->i_lock_v_table = i_lock_v_table;
                ret = true;
            }
        }
        else
        {
            self->i_lock = NULL;
            ret = true;
        }
    }

fin:
    return  ret;
}


/***********************************************************************
* Function: R_PFV_EnableInterrupt
*
* Return Value:
*    Was interrupt enabled
************************************************************************/
void  R_PFV_EnableInterrupt( int_fast32_t pfv_channel )
{
    errnum_t  e;
    pfv_t*    self;

#if PFV_PARAMETER_CHECK
    IF ( (pfv_channel < PFV_CHANNEL_MIN) || (pfv_channel > PFV_CHANNEL_MAX) )
    {
        e = E_OTHERS;
        goto  fin;
    }
#endif

    self = &gs_pfv_contexts.channel[ pfv_channel ];

    self->is_enabled_interrupt = true;
    R_Userdef_PFV_OnEnableInterrupt( pfv_channel, self->interrupt_enables.Flags );

    e=0;
fin:
    R_UNREFERENCED_VARIABLE( e );
}


/***********************************************************************
* Function: R_PFV_DisableInterrupt
*
* Return Value:
*    Was interrupt enabled
************************************************************************/
bool_t  R_PFV_DisableInterrupt( int_fast32_t pfv_channel )
{
    errnum_t  e;
    pfv_t*    self;
    bool_t    was_interrupted = false;

#if PFV_PARAMETER_CHECK
    IF ( (pfv_channel < PFV_CHANNEL_MIN) || (pfv_channel > PFV_CHANNEL_MAX) )
    {
        e = E_OTHERS;
        goto  fin;
    }
#endif

    self = &gs_pfv_contexts.channel[ pfv_channel ];

    was_interrupted = self->is_enabled_interrupt;

    R_Userdef_PFV_OnDisableInterrupt( pfv_channel, PFV_INTERRUPT_LINE_ALL );
    self->is_enabled_interrupt = false;

    e=0;
fin:
    R_UNREFERENCED_VARIABLE( e );
    return  was_interrupted;
}


