/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2015 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* File: r_dmac_rm_user.c
*
* - $Version: $ (=PFV_VERSION)
* - $ModuleRevision: {Masters}\PFV_DMAC\PFV_DMAC\23_RStyled $
* - $Rev: $
* - $Date: 2017-08-24T18:32:46+09:00 $
*******************************************************************************/


/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include  "r_ospl.h"
#include  "r_dmac_rm_internal.h"
#include  "r_dmac_rm_inline.h"
#include  "iobitmasks/dmac_rm_iobitmask.h"
#ifndef R_OSPL_NDEBUG
#include  <stdio.h>
#endif


/******************************************************************************
Typedef definitions
******************************************************************************/

/******************************************************************************
Macro definitions
******************************************************************************/

/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/

/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/

/******************************************************************************
Private global variables and functions
******************************************************************************/


/******************************************************************************
* Function Name: [R_DMAC_RM_STATIC_PrintRegisters]
* Description  :
* Arguments    :
* Return Value : Error code, 0=No error
******************************************************************************/
#ifndef R_OSPL_NDEBUG
errnum_t  R_DMAC_RM_STATIC_PrintRegisters( int_fast32_t const  dmac_channel )
{
    errnum_t  e;
    uint32_t  value;
    struct st_dmac_n*          regs_DMAC;  /* C4017W */
    struct st_dmars_mm*        regs_DMAC_01;
    const dmars_mask_shift_t*  mask_shift_DMARS;


    /* Set "regs_DMAC", "regs_DMAC_01" */
    e= DMAC_RM_STATIC_GetRegisterSet( dmac_channel, &regs_DMAC );
    IF ( e != 0 )
    {
        e = E_OTHERS;
        goto  fin;
    }

    e= DMAC_RM_STATIC_GetResouceRegisterSet( dmac_channel, &regs_DMAC_01, &mask_shift_DMARS );
    IF ( e != 0 )
    {
        e = E_OTHERS;
        goto  fin;
    }


    /* Print all register's values */
    printf( "\n" );
    printf( "DMAC[%d]:\n", dmac_channel );
    printf( "    N0SA_n = 0x%08X   \"Next0 Source Address Register\"\n", regs_DMAC->N0SA_n );
    printf( "    N0DA_n = 0x%08X   \"Next0 Destination Address Register\"\n", regs_DMAC->N0DA_n );
    printf( "    N0TB_n = 0x%08X   \"Next0 Transaction Byte Register\"\n", regs_DMAC->N0TB_n );
    printf( "    N1SA_n = 0x%08X   \"Next1 Source Address Register\"\n", regs_DMAC->N1SA_n );
    printf( "    N1DA_n = 0x%08X   \"Next1 Destination Address Register\"\n", regs_DMAC->N1DA_n );
    printf( "    N1TB_n = 0x%08X   \"Next1 Transaction Byte Register\"\n", regs_DMAC->N1TB_n );
    printf( "    CRSA_n = 0x%08X   \"Current Source Address Register\"\n", regs_DMAC->CRSA_n );
    printf( "    CRDA_n = 0x%08X   \"Current Destination Address Register\"\n", regs_DMAC->CRDA_n );
    printf( "    CRTB_n = 0x%08X   \"Current Transaction Byte Register\"\n", regs_DMAC->CRTB_n );
    printf( "\n" );
    value = regs_DMAC->CHSTAT_n;
    printf( "    CHSTAT_n = 0x%08X   \"Channel Status Register\"\n", value );
    printf( "        .INTMSK = %d   \"Interrupt mask\"\n",
            R_DMAC_RM_GET_REGISTER_BIT_FIELD( value, CHSTAT_n, INTMSK ) );
    printf( "        .MODE = %d     \"DMA mode\"\n",
            R_DMAC_RM_GET_REGISTER_BIT_FIELD( value, CHSTAT_n, MODE ) );
    printf( "        .DER = %d      \"Descriptor Error\"\n",
            R_DMAC_RM_GET_REGISTER_BIT_FIELD( value, CHSTAT_n, DER ) );
    printf( "        .DW = %d       \"Descriptor WriteBack\"\n",
            R_DMAC_RM_GET_REGISTER_BIT_FIELD( value, CHSTAT_n, DW ) );
    printf( "        .DL = %d       \"Descriptor Load\"\n",
            R_DMAC_RM_GET_REGISTER_BIT_FIELD( value, CHSTAT_n, DL ) );
    printf( "        .SR = %d       \"Selected Register Set\"\n",
            R_DMAC_RM_GET_REGISTER_BIT_FIELD( value, CHSTAT_n, SR ) );
    printf( "        .TC = %d       \"Terminal Count\"\n",
            R_DMAC_RM_GET_REGISTER_BIT_FIELD( value, CHSTAT_n, TC ) );
    printf( "        .END = %d      \"DMA END Interrupted\"\n",
            R_DMAC_RM_GET_REGISTER_BIT_FIELD( value, CHSTAT_n, END ) );
    printf( "        .ER = %d       \"Error bit\"\n",
            R_DMAC_RM_GET_REGISTER_BIT_FIELD( value, CHSTAT_n, ER ) );
    printf( "        .SUS = %d      \"Suspend\"\n",
            R_DMAC_RM_GET_REGISTER_BIT_FIELD( value, CHSTAT_n, SUS ) );
    printf( "        .TACT = %d     \"Transaction Active\"\n",
            R_DMAC_RM_GET_REGISTER_BIT_FIELD( value, CHSTAT_n, TACT ) );
    printf( "        .RQST = %d     \"Request\"\n",
            R_DMAC_RM_GET_REGISTER_BIT_FIELD( value, CHSTAT_n, RQST ) );
    printf( "        .EN = %d       \"Enable\"\n",
            R_DMAC_RM_GET_REGISTER_BIT_FIELD( value, CHSTAT_n, EN ) );
    printf( "\n" );
    value = regs_DMAC->CHCTRL_n;
    printf( "    CHCTRL_n = 0x%08X   \"Channel Control Register\"\n", value );
    printf( "        .CLRINTMSK = %d   \"Clear interrupt mask\"\n",
            R_DMAC_RM_GET_REGISTER_BIT_FIELD( value, CHCTRL_n, CLRINTMSK ) );
    printf( "        .SETINTMSK = %d   \"Set interrupt mask\"\n",
            R_DMAC_RM_GET_REGISTER_BIT_FIELD( value, CHCTRL_n, SETINTMSK ) );
    printf( "        .CLRSUS = %d      \"Clear Suspend\"\n",
            R_DMAC_RM_GET_REGISTER_BIT_FIELD( value, CHCTRL_n, CLRSUS ) );
    printf( "        .SETSUS = %d      \"Set Suspend\"\n",
            R_DMAC_RM_GET_REGISTER_BIT_FIELD( value, CHCTRL_n, SETSUS ) );
    printf( "        .CLRTC = %d       \"Clear TC bit\"\n",
            R_DMAC_RM_GET_REGISTER_BIT_FIELD( value, CHCTRL_n, CLRTC ) );
    printf( "        .CLREND = %d      \"Clear End bit\"\n",
            R_DMAC_RM_GET_REGISTER_BIT_FIELD( value, CHCTRL_n, CLREND ) );
    printf( "        .CLRRQ = %d       \"Clear Request bit\"\n",
            R_DMAC_RM_GET_REGISTER_BIT_FIELD( value, CHCTRL_n, CLRRQ ) );
    printf( "        .SWRST = %d       \"Software Reset\"\n",
            R_DMAC_RM_GET_REGISTER_BIT_FIELD( value, CHCTRL_n, SWRST ) );
    printf( "        .STG = %d         \"Software Trigger\"\n",
            R_DMAC_RM_GET_REGISTER_BIT_FIELD( value, CHCTRL_n, STG ) );
    printf( "        .CLREN = %d       \"Clear Enable\"\n",
            R_DMAC_RM_GET_REGISTER_BIT_FIELD( value, CHCTRL_n, CLREN ) );
    printf( "        .SETEN = %d       \"Set Enable\"\n",
            R_DMAC_RM_GET_REGISTER_BIT_FIELD( value, CHCTRL_n, SETEN ) );
    printf( "\n" );
    value = regs_DMAC->CHCFG_n;
    printf( "    CHCFG_n = 0x%08X   \"Channel Configuration Register\"\n", value );
    printf( "        .DMS = %d    \"DMA Mode Select\"\n",
            R_DMAC_RM_GET_REGISTER_BIT_FIELD( value, CHCFG_n, DMS ) );
    printf( "        .REN = %d    \"Register Set Enable\"\n",
            R_DMAC_RM_GET_REGISTER_BIT_FIELD( value, CHCFG_n, REN ) );
    printf( "        .RSW = %d    \"Register Select Switch\"\n",
            R_DMAC_RM_GET_REGISTER_BIT_FIELD( value, CHCFG_n, RSW ) );
    printf( "        .RSEL = %d   \"Register Set Select\"\n",
            R_DMAC_RM_GET_REGISTER_BIT_FIELD( value, CHCFG_n, RSEL ) );
    printf( "        .SBE = %d    \"Sweep Buffer Enable\"\n",
            R_DMAC_RM_GET_REGISTER_BIT_FIELD( value, CHCFG_n, SBE ) );
    printf( "        .DEM = %d    \"DMA interrupt enable\"\n",
            R_DMAC_RM_GET_REGISTER_BIT_FIELD( value, CHCFG_n, DEM ) );
    printf( "        .TM = %d     \"Transfer Mode\"\n",
            R_DMAC_RM_GET_REGISTER_BIT_FIELD( value, CHCFG_n, TM ) );
    printf( "        .DAD = %d    \"Destination not add\"\n",
            R_DMAC_RM_GET_REGISTER_BIT_FIELD( value, CHCFG_n, DAD ) );
    printf( "        .SAD = %d    \"Source not add\"\n",
            R_DMAC_RM_GET_REGISTER_BIT_FIELD( value, CHCFG_n, SAD ) );
    printf( "        .DDS = %d    \"Destination Data Size\"\n",
            R_DMAC_RM_GET_REGISTER_BIT_FIELD( value, CHCFG_n, DDS ) );
    printf( "        .SDS = %d    \"Source Data Size\"\n",
            R_DMAC_RM_GET_REGISTER_BIT_FIELD( value, CHCFG_n, SDS ) );
    printf( "        .AM = %d     \"ACK Mode\"\n",
            R_DMAC_RM_GET_REGISTER_BIT_FIELD( value, CHCFG_n, AM ) );
    printf( "        .LVL = %d    \"Level\"\n",
            R_DMAC_RM_GET_REGISTER_BIT_FIELD( value, CHCFG_n, LVL ) );
    printf( "        .HIEN = %d   \"High Enable\"\n",
            R_DMAC_RM_GET_REGISTER_BIT_FIELD( value, CHCFG_n, HIEN ) );
    printf( "        .LOEN = %d   \"Low Enable\"\n",
            R_DMAC_RM_GET_REGISTER_BIT_FIELD( value, CHCFG_n, LOEN ) );
    printf( "        .REQD = %d   \"Request Direction\"\n",
            R_DMAC_RM_GET_REGISTER_BIT_FIELD( value, CHCFG_n, REQD ) );
    printf( "        .SEL = %d    \"\"\n",
            R_DMAC_RM_GET_REGISTER_BIT_FIELD( value, CHCFG_n, SEL ) );
    printf( "\n" );
    printf( "    CHITVL_n      = 0x%08X   \"Channel Interval Register\n", regs_DMAC->CHITVL_n );
    printf( "    CHEXT_n       = 0x%08X   \"Channel Extension Register\n", regs_DMAC->CHEXT_n );
    printf( "    NXLA_n        = 0x%08X   \"Next Link Address Resister\n", regs_DMAC->NXLA_n );
    printf( "    CRLA_n        = 0x%08X   \"Current Link Address Resister\n", regs_DMAC->CRLA_n );
    printf( "\n" );

    printf( "    DMARSn = 0x%08X   \"Resource selector\"\n",  regs_DMAC_01->DMARS );
    value = R_OSPL_GET_FROM_32_BIT_REGISTER( &regs_DMAC_01->DMARS,
            mask_shift_DMARS->mask_MID, mask_shift_DMARS->shift_MID );
    printf( "        .MID = 0x%X,  .RID = 0x%X\n",
            value,
            R_OSPL_GET_FROM_32_BIT_REGISTER( &regs_DMAC_01->DMARS,
                    mask_shift_DMARS->mask_RID, mask_shift_DMARS->shift_RID ) );

    e = 0;
fin:
    return  e;
}
#endif

