/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2015 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* File Name    : main.c
* $Rev: 1320 $
* $Date:: 2015-01-29 15:54:51 +0900#$
* Device(s)    : RZ/A1H
* Tool-Chain   : DS-5 Ver 5.16
*              : ARM Complier
* OS           : None
* H/W Platform : R7S72100 CPU Board
* Description  : RZ/A1H Sample Program - Main
* Operation    : 
* Limitations  : 
*******************************************************************************/


/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include <stdio.h>
#include "r_typedefs.h"
#include "main.h"

#include "graphics.h"
#include "r_jcu_api.h"
#include "jcu_sample.h"

#include "r_pfv.h"
#include "r_pfv_sample.h"


/******************************************************************************
Typedef definitions
******************************************************************************/


/******************************************************************************
Macro definitions
******************************************************************************/

/***********************************************************************
* Macro: GS_STACK_SIZE
*    Size of stack area
*
* Description:
*    Case of OS less for RZ/A1 with ARMCC, this should set less than
*    "ARM_LIB_STACK" in "scatter.scat" file.
*
*    Case of RTX, this should set less than "OS_MAINSTKSIZE" or "OS_STKSIZE".
*
*    See "GS_INTERRUPT_STACK_SIZE" in "r_ospl.c" file.
************************************************************************/
#define  GS_STACK_SIZE  0x2000


/***********************************************************************
* Macro: GS_STACK_BUFFER_SIZE
*    Larger than used stack size at calling "R_OSPL_SET_END_OF_STACK".
************************************************************************/
#define  GS_STACK_BUFFER_SIZE  0x100



/***********************************************************************
* Define: GS_TARGET_IS
*    Selects example program.
*
*    : GS_TARGET_IS_JCU - Run example of JCU
*    : GS_TARGET_IS_PFV - Run example of PFV
************************************************************************/
#define  GS_TARGET_IS  GS_TARGET_IS_JCU

#define  GS_TARGET_IS_JCU  1
#define  GS_TARGET_IS_PFV  2


/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/


/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/


/******************************************************************************
Private global variables and functions
******************************************************************************/
#if defined(__ICCARM__) || defined(__GNUC__)
void SystemInit(void);
#endif


/******************************************************************************
* Function Name: main
* Description  : Displays the sample program information on the terminal 
*              : connected with the CPU board by the UART.
* Arguments    : none
* Return Value : 0
******************************************************************************/
int_t main(void)
{
#if  R_OSPL_STACK_CHECK_CODE
    size_t  stack_free_size_at_fist;
#endif

#if defined(__ICCARM__) || defined(__GNUC__)
    SystemInit();
#endif

    /* ==== Setup the I/O without buffering ==== */
#ifdef __ICCARM__
    setvbuf( stdout, NULL, _IONBF, 0 );
    setvbuf( stdin,  NULL, _IONBF, 0 );
#endif

    #if  R_OSPL_STACK_CHECK_CODE
    {
        uint8_t*  sp_min = R_OSPL_GET_STACK_POINTER() - ( GS_STACK_SIZE - GS_STACK_BUFFER_SIZE );

        R_OSPL_SET_END_OF_STACK( sp_min );
        R_OSPL_RESET_MIN_FREE_STACK_SIZE();
        stack_free_size_at_fist = R_OSPL_GET_MIN_FREE_STACK_SIZE();
        printf( "\nsp_min = 0x%08X\n", sp_min );
    }
    #endif

#ifdef __CC_ARM
    printf("\nRZ/A1H CPU Board Sample Program. ");
#endif
#ifdef __ICCARM__
    printf("\nRZ/A1H CPU Board Sample Program for ICCARM. ");
#endif
#ifdef __GNUC__
    printf("\nRZ/A1H CPU Board Sample Program for KPIT GCC. ");
#endif
    printf("Ver.1.04\n");

    printf("Copyright (C) 2015 Renesas Electronics Corporation. All rights reserved.\n");
    printf("\n");

#if  GS_TARGET_IS == GS_TARGET_IS_JCU

    printf("RZ/A1H CPU Board JCU Sample Program. Ver.%s\n", JCU_VERSION_STRING);
    printf("Copyright (C) 2016 Renesas Electronics Corporation. All rights reserved.\n");

    printf("\n<< JPEG decode only sample >>\n");
    R_JCU_SampleDecode(0,0);

    printf("\n<< JPEG decode and encode sample >>\n");
    R_JCU_SampleDecodeEncode(0,0);

    printf("\n<< Motion JPEG decode and show sample >>\n");
    R_JCU_SampleDecodeAndShow(0,0);

    printf("\n<< Video encode and decode sample >>\n");
    R_JCU_SampleVideoEncodeAndShowI(0,0);

    printf("\n<< Video encode and decode sample >>\n");
    R_JCU_SampleVideoEncodeAndShowP(0,0);

    printf("\nFinish.\n");

    #if  R_OSPL_STACK_CHECK_CODE
    {
        printf( "\nUsed stack size = %d\n",
            stack_free_size_at_fist - R_OSPL_GET_MIN_FREE_STACK_SIZE() );
        R_OSPL_SET_END_OF_STACK( NULL );
    }
    #endif


#endif

#if  GS_TARGET_IS == GS_TARGET_IS_PFV

    printf("\nRZ/A1H CPU Board PFV Sample Program. Ver.%s\n", PFV_VERSION_STRING);
    printf("Copyright (C) 2016 Renesas Electronics Corporation. All rights reserved.\n");
    printf("\n");

    printf("\n<< PFV by PIO sample >>\n");
    R_Sample_PFV_PIO(0,0);

    printf("\n<< PFV by DMAC sample >>\n");
    R_Sample_PFV_DMAC(0,0);

    printf("\n<< Image converting sample by PFV >>\n");
    R_Sample_PFV_DMAC_Image(0,0);

    printf("\nFinish.\n");

#endif

    return 0;
}

/* End of File */

