/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2015 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/******************************************************************************
* File: frame_buffer_typedef.h
*    Frame Buffer and Color Format
*
* - $Version: $ (=RGA_VERSION)
* - $ModuleRevision: {Masters}\RGA\FrameBuffer\25_RStyled $
* - $Rev: $
* - $Date: 2017-01-17T19:28:25+09:00 $
*
* Data structure:
* - <frame_buffer_t>
* - | <pixel_format_t>
* - | <byte_per_pixel_t>
* - | <frame_buffer_delegate_t>
******************************************************************************/

#ifndef  FRAME_BUFFER_TYPEDEF_H
#define  FRAME_BUFFER_TYPEDEF_H

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include  <stdint.h>
#include  "clib_drivers_config.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/******************************************************************************
Macro definitions
******************************************************************************/

/***********************************************************************
* Enumeration: E_CATEGORY_PIXEL_FORMAT
*    E_CATEGORY_PIXEL_FORMAT
*
*    : E_NOT_SUPPORTED_PIXEL_FORMAT - E_CATEGORY_PIXEL_FORMAT | 0x01
************************************************************************/
#ifndef  E_CATEGORY_PIXEL_FORMAT
#define  E_CATEGORY_PIXEL_FORMAT  E_CATEGORY_PIXEL_FORMAT
enum {
    E_CATEGORY_PIXEL_FORMAT = 0x00009400
};
#endif
enum { E_NOT_SUPPORTED_PIXEL_FORMAT  = E_CATEGORY_PIXEL_FORMAT | 0x01 }; /* 0x9401 = 37889 */


/******************************************************************************
Typedef definitions
******************************************************************************/

/***********************************************************************
* Type: byte_per_pixel_t
*    This can have shift value, if byte per pixel is less than 1
************************************************************************/
typedef int  byte_per_pixel_t;


/***********************************************************************
* Constant:
*    Related with <byte_per_pixel_t>.
*
*    : R_BYTE_PER_PIXEL_MASK  - 0xFF
*    : R_BYTE_PER_PIXEL_SHIFT - 8
************************************************************************/
#define  R_BYTE_PER_PIXEL_MASK  0xFFu
enum
{
    R_BYTE_PER_PIXEL_SHIFT = 8
};


/***********************************************************************
* Type: frame_buffer_delegate_t
*    This is user defined variable. Library does not access it.
************************************************************************/
typedef void  frame_buffer_delegate_t;
enum { /* int_fast32_t */  frame_buffer_t_max_buffer_count = 3 };


/***********************************************************************
* Constant: pixel_format_t
*    pixel_format_t
*
*    : PIXEL_FORMAT_UNKNOWN  -  0,              0x00
*    : PIXEL_FORMAT_ARGB8888 -  1,              0x01
*    : PIXEL_FORMAT_RGB565   -  3,              0x03
*    : PIXEL_FORMAT_ARGB4444 -  5,              0x05
*    : PIXEL_FORMAT_A8       - 11,              0x0B
*    : PIXEL_FORMAT_A4       - 14,              0x0E
*    : PIXEL_FORMAT_A1       - 13,              0x0D
*    : PIXEL_FORMAT_RGB888   - 15,              0x0F
*    : PIXEL_FORMAT_R8G8B8A8 -  6 | (1 << 4),   0x16
*    : PIXEL_FORMAT_XRGB8888 -  0 | (1 << 6),   0x40
*    : PIXEL_FORMAT_ARGB1555 -  4 | (1 << 6),   0x10002
*    : PIXEL_FORMAT_YCbCr422 -  2 | (1 << 16),  0x10002
*    : PIXEL_FORMAT_YUV422   -  2 | (1 << 16),  0x10002
*    : PIXEL_FORMAT_YUV422_GRAY_SCALE_IS_0x80 - 2 | (1 << 16),  0x10002
*    : PIXEL_FORMAT_JPEG     - 12 | (2 << 8),    0x20C
*    : PIXEL_FORMAT_PNG      - 12 | (3 << 8),    0x30C
*    : PIXEL_FORMAT_GIF      - 12 | (4 << 8),    0x40C
*    : PIXEL_FORMAT_CLUT1    - 12 | (1 << 12),  0x100C
*    : PIXEL_FORMAT_CLUT4    - 12 | (4 << 12),  0x400C
*    : PIXEL_FORMAT_CLUT8    - 12 | (8 << 12)   0x800C
************************************************************************/
enum _pixel_format_t
{
    PIXEL_FORMAT_UNKNOWN  =  0,
    PIXEL_FORMAT_ARGB8888 =  1,
    PIXEL_FORMAT_RGB565   =  3,
    PIXEL_FORMAT_ARGB4444 =  5,
    PIXEL_FORMAT_A8       = 11,
    PIXEL_FORMAT_A4       = 14,
    PIXEL_FORMAT_A1       = 13,
    PIXEL_FORMAT_RGB888   = 15,
    PIXEL_FORMAT_R8G8B8A8 =  6 | (1 << 4),
    PIXEL_FORMAT_XRGB8888 =  0 | (1 << 6),
    PIXEL_FORMAT_ARGB1555 =  4 | (1 << 6),
    PIXEL_FORMAT_YCbCr422 =  2 | (1 << 16),
    PIXEL_FORMAT_YUV422   =  2 | (1 << 16),
    PIXEL_FORMAT_YUV422_GRAY_SCALE_IS_0x80  = 2 | (1 << 16),
    PIXEL_FORMAT_JPEG     = 12 | (2 << 8),
    PIXEL_FORMAT_PNG      = 12 | (3 << 8),
    PIXEL_FORMAT_GIF      = 12 | (4 << 8),
    PIXEL_FORMAT_CLUT1    = 12 | (1 << 12),
    PIXEL_FORMAT_CLUT4    = 12 | (4 << 12),
    PIXEL_FORMAT_CLUT8    = 12 | (8 << 12)
};
typedef enum _pixel_format_t  pixel_format_t;


/***********************************************************************
* Structure: frame_buffer_t
*    frame_buffer_t
************************************************************************/
typedef struct st_frame_buffer_t  frame_buffer_t;
struct st_frame_buffer_t
{

    /* Variable: buffer_address
    	Cached address, [2] is Reserved */
    uint8_t*          buffer_address[ frame_buffer_t_max_buffer_count ];

    /* Variable: buffer_count
    	max is 2 */
    int_fast32_t      buffer_count;

    /* Variable: show_buffer_index
    	Index of "buffer_address" */
    int_fast32_t      show_buffer_index;

    /* Variable: draw_buffer_index
    	Index of "buffer_address" */
    int_fast32_t      draw_buffer_index;

    /* Variable: width */
    int_fast32_t      width;

    /* Variable: byte_per_pixel
    	check Stride and PixelFormat */
    byte_per_pixel_t  byte_per_pixel;

    /* Variable: stride
    	byte to next bottom line */
    int_fast32_t      stride;

    /* Variable: height */
    int_fast32_t      height;

    /* Variable: pixel_format */
    pixel_format_t    pixel_format;

    /* Variable: delegate */
    frame_buffer_delegate_t*  delegate;
};


/***********************************************************************
* Structure: r8g8b8a8_t
*    One of color type
************************************************************************/
#define  r8g8b8a8_t  r8g8b8a8_t
typedef union st_r8g8b8a8_t  r8g8b8a8_t;
union st_r8g8b8a8_t
{
    struct
    {
        /* Variable: u.Red */
        uint8_t  Red;

        /* Variable: u.Green */
        uint8_t  Green;

        /* Variable: u.Blue */
        uint8_t  Blue;

        /* Variable: u.Alpha */
        uint8_t  Alpha;
    } u;  /* 1st member "u" can be used by initializer */

    /* Variable: Value */
    uint32_t  Value;  /* This is depend on CPU endian. Use only copy or compare */
};


/***********************************************************************
* Structure: argb8888_t
*    One of color type
************************************************************************/
#define        argb8888_t  argb8888_t
typedef union _ARGB8888Type  argb8888_t;
union _ARGB8888Type
{
    uint32_t  Value;  /* CPU Endian. Not frame buffer endian */
    struct
    {
#ifndef  BYTE_ENDIAN
#error
#endif
#if BYTE_ENDIAN == BYTE_LITTLE_ENDIAN

        /* Variable: u.Blue
        	Blue is Value & 0x000000FF */
        uint8_t  Blue;

        /* Variable: u.Green */
        uint8_t  Green;

        /* Variable: u.Red */
        uint8_t  Red;

        /* Variable: u.Alpha */
        uint8_t  Alpha;
#else
        uint8_t  Alpha;
        uint8_t  Red;
        uint8_t  Green;
        uint8_t  Blue;   /* Blue is Value & 0x000000FF */
#endif
    } u;
};


/***********************************************************************
* Structure: xrgb8888_t
*    One of color type
************************************************************************/
#define        xrgb8888_t  xrgb8888_t
typedef union _XRGB8888Type  xrgb8888_t;
union _XRGB8888Type
{
    uint32_t  Value;  /* CPU Endian. Not frame buffer endian */
    struct
    {
#if BYTE_ENDIAN == BYTE_LITTLE_ENDIAN

        /* Variable: u.Blue
        	Blue is Value & 0x000000FF */
        uint8_t  Blue;

        /* Variable: u.Green */
        uint8_t  Green;

        /* Variable: u.Red */
        uint8_t  Red;

        /* Variable: u.X */
        uint8_t  X;      /* zero */
#else
        uint8_t  X;      /* zero */
        uint8_t  Red;
        uint8_t  Green;
        uint8_t  Blue;   /* Blue is Value & 0x000000FF */
#endif
    } u;
};


/***********************************************************************
* Structure: rgb565_t
*    One of color type
************************************************************************/
#define        rgb565_t  rgb565_t
typedef union _RGB565Type  rgb565_t;
union _RGB565Type
{
    uint16_t  Value;  /* CPU Endian. Not frame buffer endian */
    struct
    {
#if  BIT_FIELD_ENDIAN == BIT_FIELD_LITTLE_ENDIAN

        /* Variable: u.Blue
        	Blue is at LSB */
        unsigned  Blue  : 5;

        /* Variable: u.Green */
        unsigned  Green : 6;

        /* Variable: u.Red */
        unsigned  Red   : 5;
#else
        unsigned  Red   : 5;
        unsigned  Green : 6;
        unsigned  Blue  : 5;  /* Blue is at LSB */
#endif
    } u;
};


/***********************************************************************
* Structure: argb1555_t
*    One of color type
************************************************************************/
#define        argb1555_t  argb1555_t
typedef union _ARGB1555Type  argb1555_t;
union _ARGB1555Type
{
    uint16_t  Value;  /* CPU Endian. Not frame buffer endian */
    struct
    {
#if  BIT_FIELD_ENDIAN == BIT_FIELD_LITTLE_ENDIAN

        /* Variable: u.Blue
        	Blue is at LSB */
        unsigned  Blue  : 5;

        /* Variable: u.Green */
        unsigned  Green : 5;

        /* Variable: u.Red */
        unsigned  Red   : 5;

        /* Variable: u.Alpha */
        unsigned  Alpha : 1;
#else
        unsigned  Alpha : 1;
        unsigned  Red   : 5;
        unsigned  Green : 5;
        unsigned  Blue  : 5;  /* Blue is at LSB */
#endif
    } u;
};


/***********************************************************************
* Structure: argb4444_t
*    One of color type
************************************************************************/
#define        argb4444_t  argb4444_t
typedef union _ARGB4444Type  argb4444_t;
union _ARGB4444Type
{
    uint16_t  Value;  /* CPU Endian. Not frame buffer endian */
    struct
    {
#if  BIT_FIELD_ENDIAN == BIT_FIELD_LITTLE_ENDIAN

        /* Variable: u.Blue
        	Blue is at LSB */
        unsigned  Blue  : 4;

        /* Variable: u.Green */
        unsigned  Green : 4;

        /* Variable: u.Red */
        unsigned  Red   : 4;

        /* Variable: u.Alpha */
        unsigned  Alpha : 4;
#else
        unsigned  Alpha : 4;
        unsigned  Red   : 4;
        unsigned  Green : 4;
        unsigned  Blue  : 4;  /* Blue is at LSB */
#endif
    } u;
};


/***********************************************************************
* Structure: rgba8888_t
*    One of color type
************************************************************************/
#define        rgba8888_t  rgba8888_t
typedef union _RGBA8888Type  rgba8888_t;
union _RGBA8888Type
{
    uint32_t  Value;  /* CPU Endian. Not frame buffer endian */
    struct
    {
#if  BIT_FIELD_ENDIAN == BIT_FIELD_LITTLE_ENDIAN

        /* Variable: u.Alpha
        	Alpha is at LSB */
        unsigned  Alpha : 8;

        /* Variable: u.Blue */
        unsigned  Blue  : 8;

        /* Variable: u.Green */
        unsigned  Green : 8;

        /* Variable: u.Red */
        unsigned  Red   : 8;
#else
        unsigned  Red   : 8;
        unsigned  Green : 8;
        unsigned  Blue  : 8;
        unsigned  Alpha : 8;  /* Alpha is at LSB */
#endif
    } u;
};


/***********************************************************************
* Structure: y_cb_cr_422_t
*    Same as <yuv422_t>
************************************************************************/
/***********************************************************************
* Structure: yuv422_t
*    One of color type
************************************************************************/
#define          y_cb_cr_422_t  y_cb_cr_422_t
typedef union st_y_cb_cr_422_t  y_cb_cr_422_t;
#define          yuv422_t       yuv422_t
typedef union st_y_cb_cr_422_t  yuv422_t;
union st_y_cb_cr_422_t
{
    struct
    {
        /* Variable: u.Cb_01
        	U */
        uint8_t  Cb_01;

        /* Variable: u.Y_0 */
        uint8_t  Y_0;

        /* Variable: u.Cr_01
        	V */
        uint8_t  Cr_01;

        /* Variable: u.Y_1 */
        uint8_t  Y_1;

    } u;  /* 1st member "u" can be used by initializer */

    struct
    {
        /* Variable: uu.U_01 */
        uint8_t  U_01;

        /* Variable: uu.Y_0 */
        uint8_t  Y_0;

        /* Variable: uu.V_01 */
        uint8_t  V_01;

        /* Variable: uu.Y_1 */
        uint8_t  Y_1;
    } uu;

    uint32_t  Value;
};


/******************************************************************************
Variable Externs
******************************************************************************/

/******************************************************************************
Functions Prototypes
******************************************************************************/
/* In "frame_buffer.h" */

#ifdef __cplusplus
}  /* extern "C" */
#endif /* __cplusplus */

#endif
