/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2015 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/******************************************************************************
* File: video_input_typedef.h
*    Video Input API
*
* - $Version: $ (=WINDOW_SURFACES_VERSION)
* - $ModuleRevision: {Masters}\WindowSurfaces\RZ_NoEx\10_RStyle $
* - $Rev: $
* - $Date: 2017-01-20T18:55:08+09:00 $
*
* Data structure:
* - <video_input_t>
* - <video_input_config_t>
******************************************************************************/

#ifndef  VIDEO_INPUT_TYPEDEF_H
#define  VIDEO_INPUT_TYPEDEF_H

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include  "r_vdc5.h"
#include  "r_vdec.h"
#include  "frame_buffer_typedef.h"
#include  "r_ospl_typedef.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/******************************************************************************
Macro definitions
******************************************************************************/

/******************************************************************************
Typedef definitions
******************************************************************************/

/***********************************************************************
* Enumeration: video_input_channel_enum_t
*    Special integer value
*
*    : VIDEO_INPUT_NOT_DISPLAY - 0xFF
************************************************************************/
enum st_video_input_channel_enum_t {
    VIDEO_INPUT_NOT_DISPLAY = 0xFF
};
typedef enum st_video_input_channel_enum_t  video_input_channel_enum_t;
/* "typedef" must be after "enum". Because GCC_ARM raises an error */


/***********************************************************************
* Structure: video_input_t
*    video_input_t
*
* Description:
*    Member variables can not be accessed by user.
************************************************************************/
typedef struct st_video_input_t  video_input_t;
struct st_video_input_t
{

    /*-----------------------------------------------------------*/
    /* Group: Cannel */

    /* Variable: video_input_channel */
    vdc5_channel_t      video_input_channel;

    /* Variable: video_input_select */
    vdec_adc_vinsel_t   video_input_select;

    /* Variable: is_data_control */
    bool_t              is_data_control;

    /* Variable: data_control_ID */
    vdc5_layer_id_t     data_control_ID;


    /*-----------------------------------------------------------*/
    /* Group: Interrupt */

    /* Variable: is_vsync_interrupt_registered */
    bool_t              is_vsync_interrupt_registered[3];

    /* Variable: interrupt_for_VDC5 */
    vdc5_int_t          interrupt_for_VDC5;

    /* Variable: interrupt_ID */
    bsp_int_src_t       interrupt_ID;

    /* Variable: interrupt_vector */
#if  R_OSPL_INTERRUPT_HANDLER_IS != R_OSPL_INTERRUPT_HANDLER_IS_WITH_INT_SENSE
    bsp_int_cb_t        interrupt_vector;
#endif


    /*-----------------------------------------------------------*/
    /* Group: Buffer */

    /* Variable: frame_buffer */
    frame_buffer_t      frame_buffer;

    /* Variable: captured_buffer_index */
    int_fast32_t        captured_buffer_index;

    /* Variable: captured_count */
    int_fast32_t        captured_count;

    /* Variable: captured_async */
    r_ospl_async_t*     captured_async;
};


/***********************************************************************
* Structure: video_input_config_t
*    Parameters of <R_VIDEO_INPUT_Initialize>
************************************************************************/
typedef struct st_video_input_config_t  video_input_config_t;
struct st_video_input_config_t
{

    /* Variable: flags
    	0 | VIDEO_INPUT_CONFIG_T_layer_ID | ... */
    bit_flags_fast32_t  flags;


    /* Variable: video_input_channel_num
    	0=ch0.VIN1, 1=ch1.VIN1, 2=ch0.VIN2, 3=ch1.VIN2 */
    int_fast32_t        video_input_channel_num;

    /* Variable: display_channel_num
    	0, 1 or VIDEO_INPUT_NOT_DISPLAY */
    int_fast32_t        display_channel_num;

    /* Variable: display_layer_num
    	-1 or VIDEO_INPUT_NOT_DISPLAY */
    int_fast32_t        display_layer_num;

    /* Variable: frame_buffer */
    frame_buffer_t*     frame_buffer;

    /* Variable: captured_async */
    r_ospl_async_t*     captured_async;
};
/* video_input_config_t::flags */
#define  VIDEO_INPUT_CONFIG_T_VIDEO_INPUT_CHANNEL_NUM  0x0001
#define  VIDEO_INPUT_CONFIG_T_DISPLAY_CHANNEL_NUM      0x0002
#define  VIDEO_INPUT_CONFIG_T_DISPLAY_LAYER_NUM        0x0004
#define  VIDEO_INPUT_CONFIG_T_FRAME_BUFFER             0x0008
#define  VIDEO_INPUT_CONFIG_T_CAPTURED_ASYNC           0x0010


/******************************************************************************
Variable Externs
******************************************************************************/

/******************************************************************************
Functions Prototypes
******************************************************************************/
/* In "video_input.h" */

#ifdef __cplusplus
}  /* extern "C" */
#endif /* __cplusplus */

#endif
