/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2015 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/******************************************************************************
* File: vsync_pl.h
*    Porting Layer of V-Sync
*
* - $Version: $ (=WINDOW_SURFACES_VERSION)
* - $ModuleRevision: {Masters}\WindowSurfaces\RZ_NoEx\10_RStyle $
* - $Rev: $
* - $Date: 2015-10-27T16:11:57+09:00 $
******************************************************************************/

#ifndef  WINDOW_SURFACES_PL_H
#define  WINDOW_SURFACES_PL_H

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include  "r_ospl_typedef.h"
#include  "vsync_typedef.h"

#ifdef __cplusplus
extern "C"
{
#endif /* __cplusplus */


    /******************************************************************************
    Typedef definitions
    ******************************************************************************/

    /******************************************************************************
    Macro definitions
    ******************************************************************************/

    /******************************************************************************
    Variable Externs
    ******************************************************************************/

    /******************************************************************************
    Functions Prototypes
    ******************************************************************************/


    /***********************************************************************
    * Function: R_V_SYNC_SetDefaultAsync
    *    Set default <r_ospl_async_t>
    *
    * Arguments:
    *    in_out_Async - Async parameters
    *    AsyncType    - r_ospl_async_type_t
    *
    * Return Value:
    *    None
    ************************************************************************/
    void  R_V_SYNC_SetDefaultAsync( r_ospl_async_t* const  in_out_Async,  r_ospl_async_type_t AsyncType );


    /***********************************************************************
    * Function: R_V_SYNC_OnInitialize
    *    Called from <R_V_SYNC_Initialize>
    *
    * Arguments:
    *    ChannelNum - ChannelNum parameters
    *
    * Return Value:
    *    Error Code. 0=No Error.
    ************************************************************************/
    errnum_t  R_V_SYNC_OnInitialize( int_fast32_t const  ChannelNum );


    /***********************************************************************
    * Function: R_V_SYNC_OnFinalize
    *    Called from <R_V_SYNC_Finalize>
    *
    * Arguments:
    *    ChannelNum - ChannelNum parameters
    *
    * Return Value:
    *    Error Code. 0=No Error.
    ************************************************************************/
    errnum_t  R_V_SYNC_OnFinalize( int_fast32_t const  ChannelNum,  errnum_t e );


    /***********************************************************************
    * Function: R_V_SYNC_SetInterruptCallbackCaller
    *    Send <r_ospl_caller_t> to porting layer
    *
    * Arguments:
    *    ChannelNum - ChannelNum parameters
    *    Caller     - r_ospl_caller_t
    *
    * Return Value:
    *    Error Code. 0=No Error.
    ************************************************************************/
    errnum_t  R_V_SYNC_SetInterruptCallbackCaller( int_fast32_t const  ChannelNum,
            const r_ospl_caller_t* const  Caller );


    /***********************************************************************
    * Function: R_V_SYNC_OnEnableInterrupt
    *    Enable interrupt lines
    *
    * Arguments:
    *    ChannelNum - ChannelNum parameters
    *    Enables    - <r_v_sync_interrupt_lines_t>. 1=Enable, 0=NotChange
    *
    * Return Value:
    *    Error Code. 0=No Error.
    ************************************************************************/
    void   R_V_SYNC_OnEnableInterrupt( int_fast32_t const  ChannelNum,
                                       r_v_sync_interrupt_lines_t const  Enables );


    /***********************************************************************
    * Function: R_V_SYNC_OnDisableInterrupt
    *    Disable interrupt lines
    *
    * Arguments:
    *    ChannelNum - ChannelNum parameters
    *    Disables   - <r_v_sync_interrupt_lines_t>. 1=Disable, 0=NotChange
    *
    * Return Value:
    *    Error Code. 0=No Error.
    ************************************************************************/
    void  R_V_SYNC_OnDisableInterrupt( int_fast32_t const  ChannelNum,
                                       r_v_sync_interrupt_lines_t const  Disables );


    /***********************************************************************
    * Function: R_V_SYNC_OnInterruptDefault
    *    Default interrupt callback function
    *
    * Arguments:
    *    InterruptSource - r_ospl_interrupt_t
    *    Caller          - r_ospl_caller_t
    *
    * Return Value:
    *    Error Code. 0=No Error.
    ************************************************************************/
    errnum_t  R_V_SYNC_OnInterruptDefault( const r_ospl_interrupt_t* const  InterruptSource,
                                           const r_ospl_caller_t* const  Caller );


#ifdef __cplusplus
}  /* extern "C" */
#endif /* __cplusplus */

#endif
