/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2015 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/******************************************************************************
* File: window_surfaces_ex_typedef.h
*    Window Surfaces API
*
* - $Version: $ (=WINDOW_SURFACES_VERSION)
* - $ModuleRevision: {Masters}\WindowSurfaces\RZ_NoEx\10_RStyle $
* - $Rev: $
* - $Date: 2017-01-17T19:19:51+09:00 $
*
* Data structure:
* - <layer_attributes_t>
* - | <access_t> .Access
* - <window_surfaces_attribute_t>
******************************************************************************/

#ifndef  WINDOW_SURFACES_EX_TYPEDEF_H
#define  WINDOW_SURFACES_EX_TYPEDEF_H

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/******************************************************************************
Typedef definitions
******************************************************************************/


/***********************************************************************
* Structure: window_surfaces_attribute_t
*    Type of <window_surfaces_t>::attributes.
************************************************************************/
typedef  struct st_window_surfaces_attribute_t  window_surfaces_attribute_t;
struct st_window_surfaces_attribute_t
{

    /* Variable: X */
    int_fast32_t  X;

    /* Variable: Y */
    int_fast32_t  Y;

    /* Variable: Width */
    int_fast32_t  Width;

    /* Variable: Height */
    int_fast32_t  Height;

    /* Variable: OffsetX */
    int_fast32_t  OffsetX;

    /* Variable: OffsetY */
    int_fast32_t  OffsetY;

    /* Variable: OffsetByte */
    int_fast32_t  OffsetByte;
};


/* Section: Global */
/***********************************************************************
* Constant: access_t
*
*    : ACCESS_READ       - 1
*    : ACCESS_WRITE      - 2
*    : ACCESS_READ_WRITE - 3
*    : ACCESS_PLUS       - 4
*    : ACCESS_ALL_MASK   - All bits
************************************************************************/
#define  ACCESS_READ        1u
#define  ACCESS_WRITE       2u
#define  ACCESS_READ_WRITE  ( 1u | 2u )
#define  ACCESS_PLUS        4u

#define  ACCESS_ALL_MASK  ( ACCESS_READ | ACCESS_WRITE | ACCESS_READ_WRITE | ACCESS_PLUS )

typedef  bit_flags_fast32_t  access_t;


/***********************************************************************
* Structure: layer_attributes_t
*    Parameters of <R_WINDOW_SURFACES_AccessLayerAttributes>.
************************************************************************/
typedef  struct st_layer_attributes_t  layer_attributes_t;
struct st_layer_attributes_t
{

    /* Variable: access */
    access_t      access;

    /* Variable: flags
    	0 | F_LAYER_IS_SHOW | ... */
    bit_flags_fast32_t  flags;

    /* Variable: id */
    int_fast32_t  id;

    /* Variable: priority */
    int_fast32_t  priority;

    /* Variable: is_show */
    bool_t        is_show;

    /* Variable: is_color_key */
    bool_t        is_color_key;

    /* Variable: color_key */
    r8g8b8a8_t    color_key;

    /* Variable: layer_color */
    r8g8b8a8_t    layer_color;

    /* Variable: x */
    int_fast32_t  x;

    /* Variable: y */
    int_fast32_t  y;

    /* Variable: width */
    int_fast32_t  width;

    /* Variable: height */
    int_fast32_t  height;

    /* Variable: offset_x */
    int_fast32_t  offset_x;

    /* Variable: offset_y */
    int_fast32_t  offset_y;

    /* Variable: CLUT
    	CLUT (Color Look Up Table) */
    uint32_t*     CLUT;

    /* Variable: CLUT_count
    	CLUT (Color Look Up Table) count */
    int_fast32_t  CLUT_count;
};

/* bit_flags_fast32_t */
#define  F_LAYER_ID            0x0001u
#define  F_LAYER_PRIORITY      0x0002u
#define  F_LAYER_IS_SHOW       0x0004u
#define  F_LAYER_IS_COLOR_KEY  0x0008u
#define  F_LAYER_COLOR_KEY     0x0010u
#define  F_LAYER_LAYER_COLOR   0x0020u
#define  F_LAYER_X             0x0040u
#define  F_LAYER_Y             0x0080u
#define  F_LAYER_WIDTH         0x0100u
#define  F_LAYER_HEIGHT        0x0200u
#define  F_LAYER_OFFSET_X      0x0400u
#define  F_LAYER_OFFSET_Y      0x0800u
#define  F_LAYER_CLUT          0x1000u
#define  F_LAYER_CLUT_COUNT    0x2000u


/******************************************************************************
Macro definitions
******************************************************************************/

/******************************************************************************
Variable Externs
******************************************************************************/

/******************************************************************************
Functions Prototypes
******************************************************************************/
/* In "window_surfaces_ex.h" */


#ifdef __cplusplus
}  /* extern "C" */
#endif /* __cplusplus */

#endif
