/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2015 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/******************************************************************************
* File: window_surfaces_typedef.h
*    Window Surfaces API
*
* - $Version: $ (=WINDOW_SURFACES_VERSION)
* - $ModuleRevision: {Masters}\WindowSurfaces\RZ_NoEx\10_RStyle $
* - $Rev: $
* - $Date: 2016-07-04T12:19:50+09:00 $
*
* Data structure:
* - <window_surfaces_t>
* - | <window_surfaces_vdc5_layer_t> .layers
* - <window_surfaces_config_t>
******************************************************************************/

#ifndef  WINDOW_SURFACES_TYPEDEF_H
#define  WINDOW_SURFACES_TYPEDEF_H

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include  "window_surfaces_config.h"
#include  "r_vdc5.h"
#include  "frame_buffer_typedef.h"
#include  "r_ospl_typedef.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/******************************************************************************
Macro definitions
******************************************************************************/

/***********************************************************************
* Constant: WINDOW_SURFACES_DEFAULT_CHANNEL
*     0 or 1
************************************************************************/
#define  WINDOW_SURFACES_DEFAULT_CHANNEL  0


/******************************************************************************
Typedef definitions
******************************************************************************/

/***********************************************************************
* Enumeration: background_format_t
*    background_format_t
*
*    : BACKGROUND_FORMAT_SOLID_COLOR     - 0
*    : BACKGROUND_FORMAT_VIDEO_INTERLACE - 1
************************************************************************/
enum st_background_format_t {
    BACKGROUND_FORMAT_SOLID_COLOR,
    BACKGROUND_FORMAT_VIDEO_INTERLACE
};
typedef enum st_background_format_t  background_format_t;
/* "typedef" must be after "enum". Because GCC_ARM raises an error */


/***********************************************************************
* Structure: vram_stack_t
************************************************************************/
typedef struct st_vram_stack_t  vram_stack_t;
struct st_vram_stack_t
{

    /* Variable: start_of_VRAM */
    byte_t*  start_of_VRAM;

    /* Variable: over_of_VRAM */
    byte_t*  over_of_VRAM;

    /* Variable: stack_pointer_of_VRAM */
    byte_t*  stack_pointer_of_VRAM;
};


/***********************************************************************
* Structure: vram_stack_config_t
************************************************************************/
typedef struct st_vram_stack_config_t  vram_stack_config_t;
struct st_vram_stack_config_t
{

    /* Variable: flags
    	0 | F_VRAM_STACK_ADDRESS | F_VRAM_STACK_SIZE */
    bit_flags_fast32_t  flags;

    /* Variable: address */
    byte_t*  address;

    /* Variable: size */
    size_t  size;
};


/***********************************************************************
* Constants:
*    Value of "vram_stack_config_t::flags" enabling member variables.
*
*    : F_VRAM_STACK_ADDRESS - address
*    : F_VRAM_STACK_SIZE    - size
************************************************************************/
#define  F_VRAM_STACK_ADDRESS  0x01u
#define  F_VRAM_STACK_SIZE     0x02u


/***********************************************************************
* Structure: window_surfaces_vdc5_layer_t
*    VDC5 layer
************************************************************************/
typedef struct st_window_surfaces_vdc5_layer_t  window_surfaces_vdc5_layer_t;
struct st_window_surfaces_vdc5_layer_t
{

    /*-----------------------------------------------------------*/
    /* Group: Layer */

    /* Variable: layer_num */
    int8_t  layer_num;

    /* Variable: graphics_layer_ID */
    vdc5_graphics_type_t  graphics_layer_ID;


    /*-----------------------------------------------------------*/
    /* Group: Data Control */

    /* Variable: is_data_control */
    bool_t  is_data_control;

    /* Variable: data_control_ID */
    vdc5_layer_id_t  data_control_ID;


    /*-----------------------------------------------------------*/
    /* Group: Color Format */

    /* Variable: vdc5_format */
    vdc5_gr_format_t  vdc5_format;

    /* Variable: layer_color */
    r8g8b8a8_t  layer_color;
};


/***********************************************************************
* Structure: window_surfaces_t
*    window_surfaces_t
************************************************************************/
typedef struct st_window_surfaces_t  window_surfaces_t;
#define   WINDOW_SURFACES_T_MAX_LAYERS_COUNT   3

struct st_window_surfaces_t
{
    /* Variable: frame_buffers
    	Index is 0 .. (layer_num_max + background_frame_count) */
    frame_buffer_t  frame_buffers[ WINDOW_SURFACES_T_MAX_LAYERS_COUNT ];

    /* Variable: screen_channel */
    vdc5_channel_t  screen_channel;

    /* Variable: layer_num_min */
    int8_t  layer_num_min;

    /* Variable: layer_num_max */
    int8_t  layer_num_max;

    /* Variable: layers */
    window_surfaces_vdc5_layer_t  layers[ WINDOW_SURFACES_T_MAX_LAYERS_COUNT ];

    /* Variable: vdc5_disp_sel_array */
    vdc5_gr_disp_sel_t   vdc5_disp_sel_array[ VDC5_GR_TYPE_NUM ];

    /* Variable: background_format */
    background_format_t  background_format;

    /* Variable: background_frame_count */
    int_fast32_t   background_frame_count;

    /* Variable: stack_in_VRAM
    	This links to "stack_in_VRAM_body" or other. */
    vram_stack_t*  stack_in_VRAM;

    /* Variable: stack_in_VRAM_body */
    vram_stack_t  stack_in_VRAM_body;


    /*-----------------------------------------------------------*/
    /* Group: State */

    /* Variable: is_initialized */
    bool_t  is_initialized;  /* For reduce loading */
};


/***********************************************************************
* Structure: window_surfaces_config_t
*    window_surfaces_config_t
************************************************************************/
typedef struct st_window_surfaces_config_t  window_surfaces_config_t;
struct st_window_surfaces_config_t
{
    /* Variable: flags
    	0 | F_WINDOW_SURFACES_PIXEL_FORMAT | ... */
    bit_flags_fast32_t  flags;

    /* Variable: pixel_format */
    pixel_format_t  pixel_format;

    /* Variable: layer_count */
    int_fast32_t  layer_count;

    /* Variable: buffer_height */
    int_fast32_t  buffer_height;

    /* Variable: background_color */
    r8g8b8a8_t  background_color;

    /* Variable: background_format */
    background_format_t  background_format;

    /* Variable: buffer_count_max */
    int_fast32_t  buffer_count_max;

    /* Variable: shared_stack_in_VRAM */
    vram_stack_t*  shared_stack_in_VRAM;
};


/***********************************************************************
* Constants:
*    Value of "window_surfaces_config_t::flags" enabling member variables.
*
*    : F_WINDOW_SURFACES_PIXEL_FORMAT          - pixel_format
*    : F_WINDOW_SURFACES_LAYER_COUNT           - layer_count
*    : F_WINDOW_SURFACES_BUFFER_HEIGHT         - buffer_height
*    : F_WINDOW_SURFACES_BACKGROUND_COLOR      - background_color
*    : F_WINDOW_SURFACES_BACKGROUND_FORMAT     - background_format
*    : F_WINDOW_SURFACES_BUFFER_COUNT_MAX      - buffer_count_max
*    : F_WINDOW_SURFACES_SHARED_STACK_IN_VRAM  - shared_stack_in_VRAM
************************************************************************/
#define  F_WINDOW_SURFACES_PIXEL_FORMAT           0x08u
#define  F_WINDOW_SURFACES_LAYER_COUNT            0x20u
#define  F_WINDOW_SURFACES_BUFFER_HEIGHT          0x80u
#define  F_WINDOW_SURFACES_BACKGROUND_COLOR       0x40u
#define  F_WINDOW_SURFACES_BACKGROUND_FORMAT     0x100u
#define  F_WINDOW_SURFACES_BUFFER_COUNT_MAX      0x200u
#define  F_WINDOW_SURFACES_SHARED_STACK_IN_VRAM  0x400u


/***********************************************************************
* Section: Global
************************************************************************/

/******************************************************************************
Variable Externs
******************************************************************************/

/******************************************************************************
Functions Prototypes
******************************************************************************/
/* In "frame_buffer.h" */

#ifdef __cplusplus
}  /* extern "C" */
#endif

#endif  /* WINDOW_SURFACES_TYPEDEF_H */
