/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2014 - 2015 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/******************************************************************************
* File: window_surfaces_cpp.c
*    Window Surfaces for mbed style C++ API
*
* - $Version: $ (=WINDOW_SURFACES_VERSION)
* - $ModuleRevision: {Masters}\WindowSurfaces\RZ_NoEx\10_RStyle $
* - $Rev: $
* - $Date: 2017-06-09T14:05:30+09:00 $
*******************************************************************************/


/******************************************************************************
Includes   <System Includes> , "Project Includes"
*******************************************************************************/
#include  "use_list.h"
#ifdef  USE_LIST_WINDOW_SURFACES
#if  defined( USE_LIST_RTX ) && defined( USE_LIST_CORE_CA9_PORT )
#include  "core_ca9_port.h"
#endif
#include  "r_ospl.h"
#include  "window_surfaces.hpp"
#include  "window_surfaces.h"


/******************************************************************************
Typedef definitions
******************************************************************************/

/******************************************************************************
Macro definitions
******************************************************************************/
#define  GS_DEFAULT_INT_VALUE  0x7FFFFFFF
#define  GS_DEFAULT_HEIGHT  -1
#define  GS_DEFAULT_CLEAR_COLOR  0x00000100


/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/

/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/

/******************************************************************************
Private global variables and functions
******************************************************************************/


/***********************************************************************
* ClassImplement: WindowSurfacesClass
************************************************************************/

/***********************************************************************
* Implement: WindowSurfacesClass
************************************************************************/
WindowSurfacesClass::WindowSurfacesClass()
{
    this->_self = nullptr;
}


/***********************************************************************
* Implement: ~WindowSurfacesClass
************************************************************************/
WindowSurfacesClass::~WindowSurfacesClass()
{
    this->destroy();
}


/***********************************************************************
* Implement: initialize
************************************************************************/
errnum_t  WindowSurfacesClass::initialize( WindowSurfacesConfigClass& in_out_config )
{
    this->_self = new  window_surfaces_t;
    ASSERT_R( this->_self != nullptr,  return  E_FEW_MEMORY );

    in_out_config.flags =
        F_WINDOW_SURFACES_PIXEL_FORMAT |
        F_WINDOW_SURFACES_LAYER_COUNT |
        F_WINDOW_SURFACES_BACKGROUND_COLOR |
        F_WINDOW_SURFACES_BUFFER_HEIGHT;

    if ( in_out_config.buffer_height == GS_DEFAULT_HEIGHT )
    {
        in_out_config.flags &= ~F_WINDOW_SURFACES_BUFFER_HEIGHT;
    }
    if ( in_out_config.background_color.Value == GS_DEFAULT_CLEAR_COLOR )
    {
        in_out_config.flags &= ~F_WINDOW_SURFACES_BACKGROUND_COLOR;
    }

    R_WINDOW_SURFACES_InitConst( this->_self );

    return  R_WINDOW_SURFACES_Initialize( this->_self,
                                          (window_surfaces_config_t*) &in_out_config );
}


/***********************************************************************
* Implement: destroy
************************************************************************/
void  WindowSurfacesClass::destroy()
{
    if ( this->_self != nullptr )
    {
        errnum_t  e = R_WINDOW_SURFACES_Finalize( this->_self,  0 );
        ASSERT_R( e == 0,  R_NOOP() );
    }
}


/***********************************************************************
* Implement: get_layer_frame_buffer
************************************************************************/
errnum_t  WindowSurfacesClass::get_layer_frame_buffer( int_fast32_t const  layer_num,  frame_buffer_t** const  out_frame_buffer )
{
    return  R_WINDOW_SURFACES_GetLayerFrameBuffer( this->_self,  layer_num,  out_frame_buffer );
}


/***********************************************************************
* Implement: swap_buffers
************************************************************************/
errnum_t  WindowSurfacesClass::swap_buffers( int_fast32_t const  layer_num,  const void* const  null_context )
{
    return  R_WINDOW_SURFACES_SwapBuffers( this->_self,  layer_num,  null_context );
}


/***********************************************************************
* Implement: swap_buffers_start
************************************************************************/
errnum_t  WindowSurfacesClass::swap_buffers_start( int_fast32_t const  layer_num,  r_ospl_async_t* async )
{
    return  R_WINDOW_SURFACES_SwapBuffersStart( this->_self,  layer_num,  async );
}


/***********************************************************************
* Implement: wait_for_v_sync
************************************************************************/
errnum_t  WindowSurfacesClass::wait_for_v_sync( int_fast32_t const  swap_interval,  bool_t const  is_1_v_sync_at_minimum )
{
    return  R_WINDOW_SURFACES_WaitForVSync( this->_self,  swap_interval,  is_1_v_sync_at_minimum );
}


/***********************************************************************
* Implement: alloc_offscreen_stack
************************************************************************/
errnum_t  WindowSurfacesClass::alloc_offscreen_stack( frame_buffer_t* const  in_out_frame_buffer )
{
    return  R_WINDOW_SURFACES_AllocOffscreenStack( this->_self,  in_out_frame_buffer );
}


/***********************************************************************
* Implement: free_offscreen_stack
************************************************************************/
errnum_t  WindowSurfacesClass::free_offscreen_stack( const frame_buffer_t* const  frame_buffer )
{
    return  R_WINDOW_SURFACES_FreeOffscreenStack( this->_self,  frame_buffer );
}


/***********************************************************************
* ClassImplement: WindowSurfacesConfigClass
************************************************************************/

/***********************************************************************
* Implement: WindowSurfacesConfigClass
************************************************************************/
WindowSurfacesConfigClass::WindowSurfacesConfigClass()
{
    this->flags = 0;
    this->pixel_format = PIXEL_FORMAT_ARGB8888;
    this->layer_count = 1;
    this->buffer_height = GS_DEFAULT_HEIGHT;
    this->background_color.Value = GS_DEFAULT_CLEAR_COLOR;
}


#endif  /* USE_LIST_WINDOW_SURFACES */
