/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2017 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/******************************************************************************
* File: decode_show_sample.c
*     Decode a motion JPEG AVI by JCU (JPEG Codec Unit)
*
* - $Rev: 35 $
* - $Date: 2017-08-04T14:08:09+09:00 $
* - Device(s)    : RZ/A1H, RZ/A1M, RZ/A1LU
* - Tool-Chain   : -
* - OS           : -
* - H/W Platform : Genmai board
* - Description  : -
* - Operation    : None.
* - Limitations  : None.
******************************************************************************/
/******************************************************************************
* Areas
*   Includes   <System Includes> , "Project Includes"
*   Macro definitions
*   Typedef definitions
*   Imported global variables and functions (from other files)
*   Exported global variables and functions (to be accessed by other files)
*   Private global variables and functions
******************************************************************************/
/******************************************************************************
* Topic: Main Call Tree
* - <R_JCU_SampleDecodeAndShow>
* - - <T_RIFF_1_Sample>
* - - - <R_MJPEG_AVI_MEMORY_R_Initialize>
* - - - <R_MJPEG_AVI_MEMORY_R_Next>
******************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include  <string.h>
#include  "r_typedefs.h"
#include  "frame_buffer.h"
#include  "clib_drivers.h"
#include  "window_surfaces.h"
#include  "r_jcu_api.h"
#include  "jcu_sample.h"
#include  "Image/JCU_SampleImage.h"
#include  "Movie/MotionJPEG_Sample.h"
#ifndef  R_OSPL_NDEBUG
#include  <stdio.h>  /* printf */
#endif


/******************************************************************************
Macro definitions
******************************************************************************/

#define  GS_OUTPUT_PIXEL_FORMAT_JCU  JCU_OUTPUT_ARGB8888
/* JCU_OUTPUT_RGB565(=2), JCU_OUTPUT_ARGB8888(=1), JCU_OUTPUT_YCbCr422(=0) */

#define  GS_OUTPUT_PIXEL_FORMAT    PIXEL_FORMAT_ARGB8888
/* PIXEL_FORMAT_RGB565, PIXEL_FORMAT_ARGB8888, PIXEL_FORMAT_YCbCr422 */

#define  GS_BENCHMARK                  0  /* 0 or 1 */

#define  GS_JPEG_BUFFER_SIZE           (100 * 1024)
#define  GS_SLOW_INTERVAL_MSEC         100
#define  GS_BENCHMARK_COUNT_OF_SLOW    3
#define  GS_BENCHMARK_COUNT_OF_NORMAL  3
#define  GS_BENCHMARK_COUNT_OF_FAST    3
#define  GS_BENCHMARK_INTERVAL_MSEC    1000

#define  GS_BENCHMARK_ALL_COUNT   \
	( GS_BENCHMARK_COUNT_OF_SLOW + GS_BENCHMARK_COUNT_OF_NORMAL + GS_BENCHMARK_COUNT_OF_FAST )
#define  GS_JCU_OUTPUT_ALIGNMENT  8
#define  GS_JPEG_HEADER_LETTER_1  0xFF
#define  GS_JPEG_HEADER_LETTER_2  0xD8
#define  GS_NUM_255_ALPHA_MAX     255


/***********************************************************************
* Macro: R_MAKEFOURCC
*   Macro returned FourCC type value
*
* Arguments:
*   Character_0 - Character at 1st byte
*   Character_1 - Character at 2nd byte
*   Character_2 - Character at 3rd byte
*   Character_3 - Character at 4th byte
************************************************************************/
#define  R_MAKEFOURCC( Character_0, Character_1, Character_2, Character_3 ) \
	(r_fourcc_t) \
	( (Character_0) | \
	( (Character_1) << 8 ) | \
	( (Character_2) << 16 ) | \
	( (Character_3) << 24 ) )


/***********************************************************************
* Macro: R_NOT_AVAILABLE
*    Not available value. Not available array index. The value is -1.
************************************************************************/
enum { R_NOT_AVAILABLE = -1 };


/******************************************************************************
Typedef definitions
******************************************************************************/


/***********************************************************************
* Type: r_fourcc_t
*   FourCC type. 32bit integer value made by 4 letters Ascii characters.
************************************************************************/
typedef uint32_t  r_fourcc_t;


/***********************************************************************
* Type: r_riff_list_header_t
*    RIFF's LIST chunk header.
*
* Member Variables:
*    ChunkID - ChunkID
*    Size    - Size
*    TypeID  - TypeID
************************************************************************/
typedef struct st_r_riff_list_header_t  r_riff_list_header_t;
struct st_r_riff_list_header_t
{
    r_fourcc_t  ChunkID;
    uint32_t    Size;
    r_fourcc_t  TypeID;
};


/***********************************************************************
* Type: r_riff_chunk_header_t
*    RIFF's not list chunk header.
*
* Member Variables:
*    ChunkID - ChunkID
*    Size    - Size
************************************************************************/
typedef struct st_r_riff_chunk_header_t  r_riff_chunk_header_t;
struct st_r_riff_chunk_header_t
{
    r_fourcc_t  ChunkID;
    uint32_t    Size;
};


/***********************************************************************
* Type: r_riff_memory_range_t
*    RIFF's range of sub chunk or range of data in the chunk.
*
* Member Variables:
*    Start - If 'RIFF' or 'LIST', next of 'TypeID'. Others are next of "Size"
*    Next  - Next chunk
************************************************************************/
typedef struct st_r_riff_memory_range_t  r_riff_memory_range_t;
struct st_r_riff_memory_range_t
{
    const uint8_t*  Header;
    const uint8_t*  Start;
    const uint8_t*  Next;
};


/***********************************************************************
* Type: r_riff_memory_list_t
*    RIFF's List chunk in the memory.
************************************************************************/
typedef struct st_r_riff_memory_list_t  r_riff_memory_list_t;
struct st_r_riff_memory_list_t
{
    r_riff_list_header_t   Header;
    r_riff_memory_range_t  Range;
    int32_t                CurrentListIndex;
};

errnum_t  R_RIFF_ReadLIST( const void* AddressInRIFF, r_riff_memory_list_t* out_LIST,
                           r_riff_memory_range_t* RangeOfRIFF );


/***********************************************************************
* Type: r_riff_memory_chunk_t
*    RIFF's Not list chunk in the memory.
************************************************************************/
typedef struct st_r_riff_memory_chunk_t  r_riff_memory_chunk_t;
struct st_r_riff_memory_chunk_t
{
    r_riff_chunk_header_t  Header;
    r_riff_memory_range_t  Range;
};

errnum_t  R_RIFF_ReadChunk( const void* AddressInRIFF, r_riff_memory_chunk_t* out_Chunk,
                            r_riff_memory_range_t* RangeOfRIFF );


/***********************************************************************
* Type: r_riff_memory_r_t
*    Parser (read) of RIFF in the memory.
*
* Member Variables:
*    CurrentChunk     - CurrentChunk.
*                       Available when "CurrentListIndex" != R_NOT_AVAILABLE
*    CurrentListIndex - Index or R_NOT_AVAILABLE
*    NextPointer      - Next parsing position
*    ParentArray      - Array of parent 'LIST' or 'RIFF' chunk
*    ParentCount      - ParentCount
*    ParentCountMax   - ParentCountMax
************************************************************************/
typedef struct st_r_riff_memory_r_t  r_riff_memory_r_t;
struct st_r_riff_memory_r_t
{
    r_riff_memory_chunk_t  CurrentChunk;      /*  */
    const uint8_t*         NextPointer;
    r_riff_memory_list_t*  ParentArray;  /* [0]: 'RIFF' chunk */
    uint_fast32_t          ParentCount;  /* 1 or more */
    uint_fast32_t          ParentCountMax;
};

errnum_t  R_RIFF_MEMORY_R_Initialize( r_riff_memory_r_t* self,
                                      const void* RIFF_Data,  size_t RIFF_DataSize,
                                      r_riff_memory_list_t* WorkParentArray,  size_t WorkParentSize );
errnum_t  R_RIFF_MEMORY_R_Next( r_riff_memory_r_t* self,  bool_t* out_IsNextExist );


/***********************************************************************
* Type: AVIMAINHEADER
*   Data structure in AVI format file.
************************************************************************/
typedef struct st_avimainheader
{
    r_fourcc_t  fcc;
    uint32_t    cb;
    uint32_t    dwMicroSecPerFrame;
    uint32_t    dwMaxBytesPerSec;
    uint32_t    dwPaddingGranularity;
    uint32_t    dwFlags;
    uint32_t    dwTotalFrames;
    uint32_t    dwInitialFrames;
    uint32_t    dwStreams;
    uint32_t    dwSuggestedBufferSize;
    uint32_t    dwWidth;
    uint32_t    dwHeight;
    uint32_t    dwReserved[4];
} AVIMAINHEADER;


/***********************************************************************
* Type: AVISTREAMHEADER
*   Data structure in AVI format file.
************************************************************************/
typedef struct st_avistreamheader
{
    r_fourcc_t  fcc;
    uint32_t    cb;
    r_fourcc_t  fccType;
    r_fourcc_t  fccHandler;
    uint32_t    dwFlags;
    uint16_t    wPriority;
    uint16_t    wLanguage;
    uint32_t    dwInitialFrames;
    uint32_t    dwScale;
    uint32_t    dwRate;
    uint32_t    dwStart;
    uint32_t    dwLength;
    uint32_t    dwSuggestedBufferSize;
    uint32_t    dwQuality;
    uint32_t    dwSampleSize;
    struct
    {
        int16_t  left;
        int16_t  top;
        int16_t  right;
        int16_t  bottom;
    } rcFrame;
} AVISTREAMHEADER;


/***********************************************************************
* Type: r_mjpeg_avi_memory_r_t
*    Parser (read) of motion JPEG AVI in the memory.
*
* Member Variables:
*    RIFF              - RIFF parser
*    TotalFrames       - TotalFrames
*    Width             - Video width
*    Height            - Video height
*    VideoStreamFourCC - VideoStreamFourCC
*    LIST_movi         - LIST_movi chunk
*    IsFirstVideoData  - IsFirstVideoData
************************************************************************/
typedef struct st_r_mjpeg_avi_memory_r_t  r_mjpeg_avi_memory_r_t;
struct st_r_mjpeg_avi_memory_r_t
{
    r_riff_memory_r_t       RIFF;
    int32_t                 MicroSecPerFrame;
    int32_t                 TotalFrames;
    int32_t                 Width;
    int32_t                 Height;
    r_fourcc_t              VideoStreamFourCC;
    r_riff_memory_list_t    LIST_movi;
    bool_t                  IsFirstVideoData;
};

errnum_t  R_MJPEG_AVI_MEMORY_R_Initialize( r_mjpeg_avi_memory_r_t* self,
        const uint8_t* AVI_Data,  size_t AVI_DataSize,
        r_riff_memory_list_t* WorkParentArray,  size_t WorkParentSize );
errnum_t  R_MJPEG_AVI_MEMORY_R_Next( r_mjpeg_avi_memory_r_t* self,  bool_t* out_IsNextExist );


/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/

/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/

/******************************************************************************
Private global variables and functions
******************************************************************************/
uint32_t  R_MEMORY_ALIGN2_ReadUInt32( const void* Address );


/******************************************************************************
* Variable: gs_JPEG_Buffer
*    Buffer for adjusting JPEG alignment of JCU.
******************************************************************************/
R_OSPL_ALIGNMENT( GS_JCU_OUTPUT_ALIGNMENT,
                  static char_t  gs_JPEG_Buffer[ GS_JPEG_BUFFER_SIZE ]; )


/******************************************************************************
* Variable: gs_gets_Buffer
******************************************************************************/

/******************************************************************************
* Function: R_JCU_SampleDecodeAndShow
*   Main of motion JPEG example.
*
* Arguments:
*    argc - Ignored
*    argv - Ignored
*
* Returns:
*    Error Code. 0=No Error.
******************************************************************************/
errnum_t  R_JCU_SampleDecodeAndShow( int32_t const  argc, char_t * const* const  argv )
{
    enum { loop_count_max = 20 };

    errnum_t                  e;
    jcu_errorcode_t           jcu_error;
    jcu_decode_param_t        decode;
    jcu_buffer_param_t        buffer;
    bool_t                    is_JCU_initialized = false;
    uintptr_t                 physical_address_of_frame;
    uintptr_t                 physical_address_of_JPEG;
    window_surfaces_t         window;
    int_fast32_t              loop_count;
    int_fast32_t              loop_count_plus_value;
    frame_buffer_t*           frame;
    int_fast32_t              frame_num;
#if GS_BENCHMARK
    int_fast32_t              benchmark_count;
    r_ospl_ftimer_spec_t      timer_spec;
    int32_t                   next_tick;
    uint32_t                  slow_interval_count;
    uint32_t                  normal_interval_count;
    uint32_t                  benchmark_end_tick;
    int32_t                   benchmark_decode_count;
#endif
    bool_t                    is_next_exist;
    r_mjpeg_avi_memory_r_t    mjpeg;
    r_riff_memory_list_t      mjpeg_parents[8];

    R_UNREFERENCED_VARIABLE_2( argc, argv );

    R_WINDOW_SURFACES_InitConst( &window );


    /* Create a surface */
    {
        window_surfaces_config_t  config;

        config.flags = F_WINDOW_SURFACES_PIXEL_FORMAT;
        config.pixel_format = GS_OUTPUT_PIXEL_FORMAT;

        e= R_WINDOW_SURFACES_Initialize( &window, &config );
        IF(e!=0)
        {
            goto fin;
        }
    }

    e= R_WINDOW_SURFACES_GetLayerFrameBuffer( &window, 0, &frame );
    IF(e!=0)
    {
        goto fin;
    }
    IF_DQ( frame == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }

    e= R_OSPL_ToPhysicalAddress( frame->buffer_address[0], &physical_address_of_frame );
    IF(e!=0)
    {
        goto fin;
    }


    /* Clear the frame buffer */
#if  GS_OUTPUT_PIXEL_FORMAT == PIXEL_FORMAT_YCbCr422
    {
        size_t    i;
        size_t    frame_size = (size_t)( frame->stride * frame->height );
        uint8_t*  buffer = frame->buffer_address[0];

        for ( i = 0;  i < frame_size;  i += 2 )
        {
            buffer[ i + 0 ] = 0x80;
            buffer[ i + 1 ] = 0x00;
        }
    }
#else
    memset( frame->buffer_address[0], 0x00, frame->stride * frame->height );
#endif
    R_OSPL_MEMORY_Flush( R_OSPL_FLUSH_WRITEBACK_INVALIDATE );


    /* Set up JCU ( JPEG Codec Unit ) */
    jcu_error = R_JCU_Initialize( NULL );
    IF ( jcu_error != JCU_ERROR_OK )
    {
        e=E_OTHERS;
        goto fin;
    }
    is_JCU_initialized = true;

    jcu_error = R_JCU_SelectCodec( JCU_DECODE );
    IF ( jcu_error != JCU_ERROR_OK )
    {
        e=E_OTHERS;
        goto fin;
    }

    buffer.source.swapSetting       = JCU_SWAP_JPEG_NONE;
    /* ->MISRA 11.3 */ /* ->MISRA 11.4 */ /* ->SEC R2.7.1 */
    buffer.source.address           = NULL;  /* This will be set after here */
    buffer.destination.address      = (uint32_t*) physical_address_of_frame;
    /* <-MISRA 11.3 */ /* <-MISRA 11.4 */ /* <-SEC R2.7.1 */
    buffer.lineOffset               = (int16_t)(frame->width);
    decode.decodeFormat             = GS_OUTPUT_PIXEL_FORMAT_JCU;

    if ( decode.decodeFormat == JCU_OUTPUT_YCbCr422 )
    {
        buffer.destination.swapSetting  = JCU_SWAP_CB_Y0_CR_Y1_PIXEL0123;
        decode.outputCbCrOffset = JCU_CBCR_OFFSET_128;
    }
    else if ( GS_OUTPUT_PIXEL_FORMAT_JCU == JCU_OUTPUT_RGB565 )
    {
        buffer.destination.swapSetting  = JCU_SWAP_RGB565_PIXEL0123;
        decode.outputCbCrOffset = JCU_CBCR_OFFSET_0;
    }
    else
    {
        ASSERT_R( GS_OUTPUT_PIXEL_FORMAT_JCU == JCU_OUTPUT_ARGB8888, e=E_OTHERS; goto fin );
        buffer.destination.swapSetting  = JCU_SWAP_ARGB8888_PIXEL01;
        decode.outputCbCrOffset = JCU_CBCR_OFFSET_0;
    }
    decode.alpha                    = GS_NUM_255_ALPHA_MAX;
    decode.horizontalSubSampling    = JCU_SUB_SAMPLING_1_1;
    decode.verticalSubSampling      = JCU_SUB_SAMPLING_1_1;


    /* Set up timer */
#if GS_BENCHMARK
    e= R_OSPL_FTIMER_InitializeIfNot( &timer_spec );
    IF(e!=0)
    {
        goto fin;
    }
    slow_interval_count = R_OSPL_FTIMER_TimeToCount( &timer_spec, GS_SLOW_INTERVAL_MSEC );
    next_tick = R_OSPL_FTIMER_Get();
    benchmark_end_tick = R_OSPL_FTIMER_Get() + R_OSPL_FTIMER_TimeToCount( &timer_spec,
                         GS_BENCHMARK_INTERVAL_MSEC );
    benchmark_count = 0;
    benchmark_decode_count = 0;
#endif


    if ( argc == 0 )
    {
        loop_count_plus_value = 1;
    }
    else
    {
        loop_count_plus_value = 0;
    }

    for ( loop_count = 0;  loop_count < loop_count_max;  loop_count += loop_count_plus_value )
    {

        /* Parse motion JPEG AVI data */
        e= R_MJPEG_AVI_MEMORY_R_Initialize( &mjpeg,
                                            g_MotionJPEG_Sample, sizeof( g_MotionJPEG_Sample ),
                                            mjpeg_parents, sizeof( mjpeg_parents ) );
        IF(e)
        {
            goto fin;
        }

#if GS_BENCHMARK
        normal_interval_count = R_OSPL_FTIMER_TimeToCount( &timer_spec, mjpeg.MicroSecPerFrame / 1000 );
#endif

        for ( frame_num = 0;  ;  frame_num += 1 )
        {
            e= R_MJPEG_AVI_MEMORY_R_Next( &mjpeg, &is_next_exist );
            IF(e)
            {
                goto fin;
            }
            if ( ! is_next_exist )
            {
                break;
            }

            ASSERT_R( mjpeg.RIFF.CurrentChunk.Range.Start[0] == GS_JPEG_HEADER_LETTER_1,
                      e=E_OTHERS; goto fin );
            ASSERT_R( mjpeg.RIFF.CurrentChunk.Range.Start[1] == GS_JPEG_HEADER_LETTER_2,
                      e=E_OTHERS; goto fin );


            /* Adjust to JPEG alignment of JCU */
            if ( (uintptr_t) mjpeg.RIFF.CurrentChunk.Range.Start % GS_JCU_OUTPUT_ALIGNMENT == 0 )
            {
                e= R_OSPL_ToPhysicalAddress( mjpeg.RIFF.CurrentChunk.Range.Start,
                                             &physical_address_of_JPEG );
                IF(e!=0)
                {
                    goto fin;
                }
            }
            else
            {
                memcpy( gs_JPEG_Buffer,
                        mjpeg.RIFF.CurrentChunk.Range.Start,
                        mjpeg.RIFF.CurrentChunk.Header.Size );

                R_OSPL_MEMORY_Flush( R_OSPL_FLUSH_WRITEBACK_INVALIDATE );

                e= R_OSPL_ToPhysicalAddress( gs_JPEG_Buffer,
                                             &physical_address_of_JPEG );
                IF(e!=0)
                {
                    goto fin;
                }
            }


            /* Wait for slow interval */
#if GS_BENCHMARK
            {
                int32_t   wait_count;
                uint32_t  current_tick = R_OSPL_FTIMER_Get();

                wait_count = next_tick - current_tick;

                if ( wait_count > 0 )
                {
                    e= R_OSPL_Delay( R_OSPL_FTIMER_CountToTime( &timer_spec, wait_count ) );
                    IF(e!=0)
                    {
                        goto fin;
                    }
                }

                if ( benchmark_count < GS_BENCHMARK_COUNT_OF_SLOW )
                {
                    next_tick += slow_interval_count;
                }
                else if ( benchmark_count < GS_BENCHMARK_COUNT_OF_SLOW + GS_BENCHMARK_COUNT_OF_NORMAL )
                {
                    next_tick += normal_interval_count;
                }
                else
                {
                    next_tick = current_tick;
                }

                if ( next_tick < current_tick )
                {
                    next_tick = current_tick;
                }
            }
#endif


            /* Run JCU */
            buffer.source.address = (uint32_t*) physical_address_of_JPEG;

            jcu_error = R_JCU_SetDecodeParam( &decode, &buffer );
            IF ( jcu_error != JCU_ERROR_OK )
            {
                e=E_OTHERS;
                goto fin;
            }

            jcu_error = R_JCU_Start();
            IF ( jcu_error != JCU_ERROR_OK )
            {
                e=E_OTHERS;
                goto fin;
            }

            e= R_WINDOW_SURFACES_SwapBuffers( &window, 0, NULL );
            IF(e!=0)
            {
                goto fin;
            }


            /* Print benchmark */
#if GS_BENCHMARK
            {
                bool_t  is_past;

                benchmark_decode_count += 1;

                e= R_OSPL_FTIMER_IsPast( &timer_spec, R_OSPL_FTIMER_Get(),
                                         benchmark_end_tick, &is_past );
                IF(e!=0)
                {
                    goto fin;
                }

                if ( is_past )
                {
                    printf( "%dfps\n", benchmark_decode_count );

                    benchmark_count += 1;
                    if ( benchmark_count >= GS_BENCHMARK_ALL_COUNT )
                    {
                        benchmark_count = 0;

                        if ( argc == 0 )
                        {
                            goto exit_for;
                        }
                    }

                    benchmark_decode_count = 0;

                    benchmark_end_tick += R_OSPL_FTIMER_TimeToCount( &timer_spec,
                                          GS_BENCHMARK_INTERVAL_MSEC );
                }
            }
#endif
        }
    }

    e = 0;
fin:
    if ( is_JCU_initialized != false )
    {
        jcu_error = R_JCU_Terminate();
        IF ( (jcu_error != JCU_ERROR_OK) && (e == 0) )
        {
            e = E_OTHERS;
        }
    }

    e= R_WINDOW_SURFACES_Finalize( &window, e );
    e= R_WINDOW_SURFACES_Dispose( &window, e );
    R_DEBUG_BREAK_IF_ERROR();
    return  e;
}


/***********************************************************************
* Function: R_MJPEG_AVI_MEMORY_R_Initialize
*    Parse memory mapped motion JPEG .avi file.
*
* Arguments:
*    self            - Object.
*    AVI_Data        - First address of data read from .avi file
*    AVI_DataSize    - Byte size of .avi file
*    WorkParentArray - Work area in "self"
*    WorkParentSize  - Byte size of "WorkParentArray"
*
* Returns:
*    Error Code. 0=No Error.
************************************************************************/
errnum_t  R_MJPEG_AVI_MEMORY_R_Initialize( r_mjpeg_avi_memory_r_t* self,
        const uint8_t* AVI_Data,  size_t AVI_DataSize,
        r_riff_memory_list_t* WorkParentArray,  size_t WorkParentSize )
{
    errnum_t  e;
    bool_t    is_next_exist;
    r_riff_memory_r_t*  riff = &self->RIFF;


    self->VideoStreamFourCC = 0;
    self->IsFirstVideoData = false;


    e= R_RIFF_MEMORY_R_Initialize( riff, AVI_Data, AVI_DataSize,
                                   WorkParentArray, WorkParentSize );
    IF(e)
    {
        goto fin;
    }

    ASSERT_R( riff->ParentArray[0].Header.TypeID == R_MAKEFOURCC( 'A','V','I',' ' ),
              e=E_BAD_COMMAND_ID; goto fin );

    for (;;)
    {
        e= R_RIFF_MEMORY_R_Next( riff, &is_next_exist );
        IF(e)
        {
            goto fin;
        }

        if ( ! is_next_exist )
        {
            break;
        }

        switch ( riff->CurrentChunk.Header.ChunkID )
        {
        case  R_MAKEFOURCC( 'a','v','i','h' ):
        {
            const AVIMAINHEADER*  data = (const AVIMAINHEADER*) riff->CurrentChunk.Range.Header;

            self->MicroSecPerFrame = (int32_t) R_MEMORY_ALIGN2_ReadUInt32( &data->dwMicroSecPerFrame );
            self->TotalFrames = (int32_t) R_MEMORY_ALIGN2_ReadUInt32( &data->dwTotalFrames );
            self->Width  = (int32_t) R_MEMORY_ALIGN2_ReadUInt32( &data->dwWidth );
            self->Height = (int32_t) R_MEMORY_ALIGN2_ReadUInt32( &data->dwHeight );

            break;
        }

        case  R_MAKEFOURCC( 's','t','r','h' ):
        {
            const AVISTREAMHEADER*  data = (const AVISTREAMHEADER*) riff->CurrentChunk.Range.Header;

            if ( R_MEMORY_ALIGN2_ReadUInt32( &data->fccType ) ==
                    R_MAKEFOURCC( 'v','i','d','s' ) )
            {
                if ( self->VideoStreamFourCC == 0 )
                {
                    int_fast32_t  index = riff->ParentArray[
                                              riff->ParentCount - 1 ].CurrentListIndex;

                    self->VideoStreamFourCC = R_MAKEFOURCC( '0', index + '0', 'd', 'c' );
                }
            }
            break;
        }

        default:
            /* Do Nothing: Ignore other chunks */
            break;
        }

        if ( ! self->IsFirstVideoData )
        {
            switch ( riff->ParentArray[ riff->ParentCount - 1 ].Header.TypeID )
            {
            case  R_MAKEFOURCC( 'm','o','v','i' ):
            {
                self->LIST_movi = riff->ParentArray[ riff->ParentCount - 1 ];
                self->IsFirstVideoData = true;
                break;
            }

            default:
                /* Do Nothing: Ignore other chunks */
                break;
            }

            if ( self->IsFirstVideoData )
            {
                break;
            }
        }
    }

    e=0;
fin:
    return  e;
}


/***********************************************************************
* Function: R_MJPEG_AVI_MEMORY_R_Next
*    Get next JPEG data.
*
* Arguments:
*    self            - Object.
*    out_IsNextExist - (out) Whether next chunk exists.
*
* Returns:
*    Error Code. 0=No Error.
*
* Description:
*    Output JPEG data is in "self->RIFF.CurrentChunk.Range.Start".
************************************************************************/
errnum_t  R_MJPEG_AVI_MEMORY_R_Next( r_mjpeg_avi_memory_r_t* self,  bool_t* out_IsNextExist )
{
    errnum_t  e;
    bool_t    is_next_exist = false;
    r_riff_memory_r_t*  riff = &self->RIFF;


    if ( ! self->IsFirstVideoData  ||  riff->CurrentChunk.Header.ChunkID != self->VideoStreamFourCC )
    {
        for (;;)
        {
            e= R_RIFF_MEMORY_R_Next( riff, &is_next_exist );
            IF(e)
            {
                goto fin;
            }

            if ( ! is_next_exist )
            {
                break;
            }

            if ( riff->CurrentChunk.Header.ChunkID == self->VideoStreamFourCC )
            {
                break;
            }
        }
    }
    else
    {
        self->IsFirstVideoData = false;
        is_next_exist = true;
    }

    e=0;
fin:
    *out_IsNextExist = is_next_exist;
    return  e;
}


/***********************************************************************
* Function: R_RIFF_MEMORY_R_Initialize
*    Parse memory mapped RIFF file.
*
* Arguments:
*    self            - Object.
*    RIFF_Data       - First address of data read from .avi file
*    RIFF_DataSize   - Byte size of .avi file
*    WorkParentArray - Work area in "self"
*    WorkParentSize  - Byte size of "WorkParentArray"
*
* Returns:
*    Error Code. 0=No Error.
************************************************************************/
errnum_t  R_RIFF_MEMORY_R_Initialize( r_riff_memory_r_t* self,
                                      const void* RIFF_Data,  size_t RIFF_DataSize,
                                      r_riff_memory_list_t* WorkParentArray,  size_t WorkParentSize )
{
    errnum_t  e;
    r_riff_memory_range_t  range;

    self->ParentArray = WorkParentArray;
    self->ParentCount = 1;
    self->ParentCountMax = WorkParentSize / sizeof( r_riff_memory_list_t );
    ASSERT_R( self->ParentCountMax >= 1,  e=E_FEW_ARRAY; goto fin );

    range.Start = RIFF_Data;
    range.Next = (const uint8_t*) RIFF_Data + RIFF_DataSize;

    e= R_RIFF_ReadLIST( RIFF_Data, &self->ParentArray[0], &range );
    IF(e)
    {
        goto fin;
    }
    ASSERT_R( self->ParentArray[0].Header.ChunkID == R_MAKEFOURCC( 'R','I','F','F' ),
              e=E_BAD_COMMAND_ID; goto fin );
    self->ParentArray[0].CurrentListIndex = R_NOT_AVAILABLE;

    self->NextPointer = self->ParentArray[0].Range.Start;

    e=0;
fin:
    return  e;
}


/***********************************************************************
* Function: R_RIFF_MEMORY_R_Next
*    Get next chunk. 'RIFF' and 'LIST' chunk is skipped.
*
* Arguments:
*    self            - Object.
*    out_IsNextExist - (out) Whether next chunk exists.
*
* Returns:
*    Error Code. 0=No Error.
*
* Description:
*    Current chunk is in "self->CurrentChunk".
*    'LIST' chunk is in "self->ParentArray[ self->ParentCount - 1 ]".
************************************************************************/
errnum_t  R_RIFF_MEMORY_R_Next( r_riff_memory_r_t* self,  bool_t* out_IsNextExist )
{
    errnum_t  e;
    bool_t    is_next_exist = false;
    r_riff_memory_list_t*  parent;


    /* Operation of end of 'LIST' */
    {
        int_fast32_t   index;

        for ( index = (int_fast32_t) self->ParentCount - 1;  index >= 0;  index -= 1 )
        {
            parent = &self->ParentArray[ index ];

            if ( self->NextPointer < parent->Range.Next )
            {
                break;
            }
        }

        if ( index < 0 )    /* End of file */
        {
            e=0;
            goto fin;
        }

        self->ParentCount = (uint_fast32_t)( index + 1 );

        parent = &self->ParentArray[ self->ParentCount - 1 ];
        parent->CurrentListIndex += 1;
    }


    /* Parse chunks */
    for (;;)
    {
        r_fourcc_t  chunk_ID = R_MEMORY_ALIGN2_ReadUInt32( self->NextPointer );

        if ( chunk_ID == R_MAKEFOURCC( 'L','I','S','T' ) )
        {
            ASSERT_R( self->ParentCount < self->ParentCountMax,  e=E_FEW_ARRAY; goto fin );

            parent = &self->ParentArray[ self->ParentCount ];

            e= R_RIFF_ReadLIST( self->NextPointer, parent, &self->ParentArray[0].Range );
            IF(e)
            {
                goto fin;
            }
            parent->CurrentListIndex = R_NOT_AVAILABLE;

            self->ParentCount += 1;
            self->NextPointer = parent->Range.Start;
        }
        else
        {
            e= R_RIFF_ReadChunk( self->NextPointer, &self->CurrentChunk,
                                 &self->ParentArray[0].Range );
            IF(e)
            {
                goto fin;
            }

            parent = &self->ParentArray[ self->ParentCount - 1 ];
            parent->CurrentListIndex += 1;

            self->NextPointer = self->CurrentChunk.Range.Next;

            is_next_exist = true;
            break;
        }
    }

    e=0;
fin:
    *out_IsNextExist = is_next_exist;
    return  e;
}


/***********************************************************************
* Function: R_RIFF_ReadLIST
*    Read 'LIST' or 'RIFF' chunk.
*
* Arguments:
*    AddressInRIFF - Address of reading in "RangeOfRIFF".
*    out_LIST      - (out) 'LIST' or 'RIFF' chunk.
*    RangeOfRIFF   - Range of RIFF data.
*
* Returns:
*    Error Code. 0=No Error.
************************************************************************/
errnum_t  R_RIFF_ReadLIST( const void* AddressInRIFF, r_riff_memory_list_t* out_LIST,
                           r_riff_memory_range_t* RangeOfRIFF )
{
    enum
    {
        num_4_byte = 4,
        num_8_byte = 8,
        num_12_byte = 12,
    };
    errnum_t        e;
    const uint8_t*  pointer = AddressInRIFF;
    int_fast32_t    padding_size;


    ASSERT_R( (const uint8_t*) AddressInRIFF >= RangeOfRIFF->Start  &&
              (const uint8_t*) AddressInRIFF <= RangeOfRIFF->Next - num_12_byte,
              e=E_BAD_COMMAND_ID;  goto fin );


    out_LIST->Header.ChunkID = R_MEMORY_ALIGN2_ReadUInt32( pointer );
    out_LIST->Header.Size    = R_MEMORY_ALIGN2_ReadUInt32( pointer + num_4_byte );
    out_LIST->Header.TypeID  = R_MEMORY_ALIGN2_ReadUInt32( pointer + num_8_byte );

    if ( out_LIST->Header.Size % 2u == 0 )
    {
        padding_size = 0;
    }
    else
    {
        padding_size = 1;
    }

    out_LIST->Range.Header = pointer;
    out_LIST->Range.Start = pointer + num_12_byte;
    out_LIST->Range.Next = pointer + num_8_byte + out_LIST->Header.Size + padding_size;


    ASSERT_R( out_LIST->Range.Next >= RangeOfRIFF->Start  &&
              out_LIST->Range.Next <= RangeOfRIFF->Next,
              e=E_BAD_COMMAND_ID;  goto fin );


    e=0;
fin:
    return  e;
}


/***********************************************************************
* Function: R_RIFF_ReadChunk
*    Read not 'LIST' and 'RIFF' chunk.
*
* Arguments:
*    AddressInRIFF - Address of reading in "RangeOfRIFF".
*    out_Chunk     - (out) A chunk.
*    RangeOfRIFF   - Range of RIFF data.
*
* Returns:
*    Error Code. 0=No Error.
************************************************************************/
errnum_t  R_RIFF_ReadChunk( const void* AddressInRIFF, r_riff_memory_chunk_t* out_Chunk,
                            r_riff_memory_range_t* RangeOfRIFF )
{
    enum
    {
        num_4_byte = 4,
        num_8_byte = 8,
    };
    errnum_t        e;
    const uint8_t*  pointer = AddressInRIFF;
    int_fast32_t    padding_size;


    ASSERT_R( (const uint8_t*) AddressInRIFF >= RangeOfRIFF->Start  &&
              (const uint8_t*) AddressInRIFF <= RangeOfRIFF->Next - num_8_byte,
              e=E_BAD_COMMAND_ID;  goto fin );


    out_Chunk->Header.ChunkID = R_MEMORY_ALIGN2_ReadUInt32( pointer );
    out_Chunk->Header.Size    = R_MEMORY_ALIGN2_ReadUInt32( pointer + num_4_byte );

    if ( out_Chunk->Header.Size % 2u == 0 )
    {
        padding_size = 0;
    }
    else
    {
        padding_size = 1;
    }

    out_Chunk->Range.Header = pointer;
    out_Chunk->Range.Start = pointer + num_8_byte;
    out_Chunk->Range.Next = pointer + num_8_byte + out_Chunk->Header.Size + padding_size;

    e=0;
fin:
    return  e;
}


/***********************************************************************
* Function: R_MEMORY_ALIGN2_ReadUInt32
*    Read uint32_t type value from 2 byte alingned memory address.
*
* Arguments:
*    Address - Address of reading from memory.
*
* Returns:
*    uint32_t type value.
*
* Description:
*    If not 2 byte aligned address, return value is not defined.
************************************************************************/
uint32_t  R_MEMORY_ALIGN2_ReadUInt32( const void* Address )
{
    const uint16_t*  adr = (const uint16_t*) Address;
    return  (uint32_t)( adr[0] | ( adr[1] << 16 ) );
}


