/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2015 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/******************************************************************************
* File: pfv_sample.c
*     Example of Pixel Format Converter.
*
* - $Rev: 35 $
* - $Date: 2017-08-04T18:56:50+09:00 $
* - Device(s)    : RZ/A1H, RZ/A1M
* - Tool-Chain   : ARMCC, ICCARM, KPITGCC
* - OS           : -
* - H/W Platform : Genmai board
* - Description  : -
* - Operation    : None.
* - Limitations  : None.
******************************************************************************/


#define  SAMPLE_PFV_DEFAULT_PFV_CHANNEL          0
#define  SAMPLE_PFV_DEFAULT_INPUT_DMAC_CHANNEL   0
#define  SAMPLE_PFV_DEFAULT_OUTPUT_DMAC_CHANNEL  1


/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include  "use_list.h"
#include  "r_ospl_config.h"
#ifndef R_OSPL_NDEBUG
#include  <stdio.h>
#endif
#include  <string.h>
#include  "r_typedefs.h"
#include  "dev_drv.h"
#include  "clib_drivers.h"
#include  "r_pfv.h"
#include  "VRAM.h"
#include  "RGA_raw_image.h"
#include  "frame_buffer.h"
#ifdef  USE_LIST_WINDOW_SURFACES
#include  "r_vdc5.h"
#include  "window_surfaces.h"
#endif
#include  "./inc/r_pfv_sample.h"
#include  "./Image/PFV_SampleImage.h"


/******************************************************************************
Typedef definitions
******************************************************************************/

/******************************************************************************
Macro definitions
******************************************************************************/


/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/

/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/

/******************************************************************************
Private global variables and functions
******************************************************************************/

/***********************************************************************
* Function: Get_ARGB8888
*    Gets ARGB8888 format 32bit integer.
*
* Arguments:
*    Alpha_ - 0-255
*    Red_   - 0-255
*    Green_ - 0-255
*    Blue_  - 0-255
*
* Returns:
*    ARGB8888 format 32bit integer.
************************************************************************/
#ifdef  USE_LIST_WINDOW_SURFACES
STATIC_INLINE uint32_t  Get_ARGB8888( int_fast32_t const  Alpha_,  int_fast32_t const  Red_,
                                      int_fast32_t const  Green_,  int_fast32_t const  Blue_ );  /* QAC-3450 */
STATIC_INLINE uint32_t  Get_ARGB8888( int_fast32_t const  Alpha_,  int_fast32_t const  Red_,
                                      int_fast32_t const  Green_,  int_fast32_t const  Blue_ )
{
    argb8888_t  color_;  /* _ is for MISRA 5.6 */

    /* ->QAC 3198 */
    color_.Value = DUMMY_INITIAL_VALUE;  /* for avoid ARMCC warning C4017W */
    /* <-QAC 3198 */

    color_.u.Alpha = (uint8_t) Alpha_;  /* _ is for MISRA 5.6 */
    color_.u.Red   = (uint8_t) Red_;
    color_.u.Green = (uint8_t) Green_;
    color_.u.Blue  = (uint8_t) Blue_;

    return  color_.Value;
}
#endif


/***********************************************************************
* Function: Get_YCbCr422_Y0Y1CbCr
*    Gets YCbCr422 format 32bit integer.
*
* Arguments:
*    Y0_ - 0-255
*    Y1_ - 0-255
*    Cb_ - 0-255
*    Cr_ - 0-255
*
* Returns:
*    YCbCr422 format 32bit integer.
*
* Description:
*    Return value is [Cb][Y0][Cr][Y1] as byte array.
************************************************************************/
STATIC_INLINE uint32_t  Get_YCbCr422_Y0Y1CbCr( int_fast32_t const  Y0_,  int_fast32_t const  Y1_,
        int_fast32_t const  Cb_,  int_fast32_t const  Cr_ );  /* QAC-3450 */
STATIC_INLINE uint32_t  Get_YCbCr422_Y0Y1CbCr( int_fast32_t const  Y0_,  int_fast32_t const  Y1_,
        int_fast32_t const  Cb_,  int_fast32_t const  Cr_ )
{
    y_cb_cr_422_t  color_;  /* _ is for MISRA 5.6 */

    /* ->QAC 3198 */
    color_.Value = DUMMY_INITIAL_VALUE;  /* for avoid ARMCC warning C4017W */
    /* <-QAC 3198 */

    color_.u.Cb_01 = (uint8_t) Cb_;  /* _ is for MISRA 5.6 */
    color_.u.Y_0   = (uint8_t) Y0_;
    color_.u.Cr_01 = (uint8_t) Cr_;
    color_.u.Y_1   = (uint8_t) Y1_;

    return  color_.Value;
}


/***********************************************************************
* Function: R_GRAPHICS_IMAGE_GetAddress
*    Gets address of image.
*
* Arguments:
*    self        - graphics_image_t
*    out_Address - (out) address of image
*
* Returns:
*    Error Code. 0=No Error.
************************************************************************/
STATIC_INLINE errnum_t  R_GRAPHICS_IMAGE_GetAddress( const graphics_image_t* const  self,
        void* const  out_Address ); /* QAC-3450 */
STATIC_INLINE errnum_t  R_GRAPHICS_IMAGE_GetAddress( const graphics_image_t* const  self,
        void* const  out_Address )
{
    errnum_t  e;

    IF_DQ( self == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }
    IF_DQ( out_Address == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }

    /* ->MISRA 11.4 */ /* ->SEC R2.7.1 */
    *(const uint8_t**) out_Address = &( (const uint8_t*) self )[ self->offset_to_image ];
    /* <-MISRA 11.4 */ /* <-SEC R2.7.1 */

    e=0;
fin:
    return  e;
}


/***********************************************************************
* Function: R_Sample_PFV_PIO
*    Example code of PFV by PIO.
*
* Arguments:
*    argc - The argument of main function. Ignored.
*    argv - The argument of main function. Ignored.
*
* Returns:
*    Error Code. 0=No Error.
************************************************************************/
errnum_t  R_Sample_PFV_PIO( int32_t const  argc,
                            char_t * const* const  argv )
{
    errnum_t            e;
    errnum_t            ee;
    int_fast32_t const  channel_of_PFV = SAMPLE_PFV_DEFAULT_PFV_CHANNEL;

#ifndef R_OSPL_NDEBUG
    printf( "\nRZ/A1H - PFV sample version %s - Sample_PFV_PIO\n", PFV_VERSION_STRING );
#endif

    R_UNREFERENCED_VARIABLE_2( argc, argv );


    e= R_PFV_Initialize( channel_of_PFV, NULL );
    IF ( e != 0 )
    {
        goto  fin;
    }


    /* Call "R_PFV_SetIOFormat" */
    {
        pfv_io_format_t  format;

        format.flags = F_PFV_INPUT_FORMAT | F_PFV_OUTPUT_FORMAT;
        format.input_format  = PFV_ARGB8888;
        format.output_format = PFV_YCbCr422;

        e= R_PFV_SetIOFormat( channel_of_PFV, &format );
        IF ( e != 0 )
        {
            goto  fin;
        }
    }


    /* Call "R_PFV_WritePixelDataViaPIO" */
    {
        enum
        {
            count_of_out_data = 2,
            num_32_bit_access = 32,  /* QAC 3120 */
            num_8 = 8
        };
        uint32_t  pixel_data;

        for (;;)
        {
            int_fast32_t  filled_byte;

            e= R_PFV_GetFilledByteInInputFIFO( channel_of_PFV,  &filled_byte );
            IF ( e != 0 )
            {
                goto  fin;
            }

            if ( filled_byte <= (PFV_INPUT_FIFO_FULL_BYTE -
                                 ((count_of_out_data * num_32_bit_access) / num_8)) )
            {
                break;
            }
        }

        pixel_data = Get_ARGB8888( U8_255, U8_255, U8_255, 0x00 );
        e= R_PFV_WritePixelDataViaPIO( channel_of_PFV, pixel_data, num_32_bit_access );
        IF ( e != 0 )
        {
            goto  fin;
        }

        pixel_data = Get_ARGB8888( U8_255, U8_255, U8_255, 0x00 );
        e= R_PFV_WritePixelDataViaPIO( channel_of_PFV, pixel_data, num_32_bit_access );
        IF ( e != 0 )
        {
            goto  fin;
        }
        /* ARGB8888 x 2 pixel => YCbCr422 x 1 */
    }


    /* Call "R_PFV_ReadPixelDataViaPIO" */
    {
        enum
        {
            num_32_bit_access = 32,  /* QAC 3120 */
            num_0xE2_answer = 0xE2,
            num_0x95_answer = 0x95,
            num_8 = 8
        };

        int_fast32_t  filled_byte;
        uint32_t      pixel_data;
        uint32_t      pixel_data_answer;

        for (;;)
        {

            e= R_PFV_GetFilledByteInOutputFIFO( channel_of_PFV,  &filled_byte );
            IF ( e != 0 )
            {
                goto  fin;
            }

            if ( filled_byte >= (num_32_bit_access / num_8) )
            {
                break;
            }
        }

        e= R_PFV_ReadPixelDataViaPIO( channel_of_PFV, &pixel_data, num_32_bit_access );
        IF ( e != 0 )
        {
            goto  fin;
        }

        pixel_data_answer = Get_YCbCr422_Y0Y1CbCr(
                                num_0xE2_answer, num_0xE2_answer, 0x00, num_0x95_answer );
        IF ( pixel_data != pixel_data_answer )
        {
            e = E_OTHERS;
            goto  fin;
        }
    }

    e = 0;
fin:
    ee= R_PFV_Terminate( channel_of_PFV );
    IF ( (ee != 0) && (e == 0) )
    {
        e = ee;
    }

    R_DEBUG_BREAK_IF_ERROR();
    return  e;
}


/***********************************************************************
* Function: R_Sample_PFV_DMAC
*    Example code of PFV by DMA.
*
* Arguments:
*    argc - The argument of main function. Ignored.
*    argv - The argument of main function. Ignored.
*
* Returns:
*    Error Code. 0=No Error.
************************************************************************/
errnum_t  R_Sample_PFV_DMAC( int32_t const  argc,
                             char_t * const* const  argv )
{
    errnum_t              e;
    errnum_t              ee;
    int_fast32_t const    channel_of_PFV = SAMPLE_PFV_DEFAULT_PFV_CHANNEL;
    uintptr_t             big_memory_physical_address;
    uint8_t*              big_memory_address;
    size_t                big_memory_size;
    uintptr_t             input_physical_address;
    uint8_t*              input_address;
    uintptr_t             output_physical_address;
    uint8_t*              output_address;
    int32_t               output_max_size;
    void*                 p;
    bool_t                is_cached;

    enum { input_data_n = 6 };  /* Count is by initializer. SEC R3.1.1 (1) */
    const static uint32_t  input_data[ input_data_n ] =
    {
        /* ->QAC 3120 */
        /* See "input_format" */
        0xFFFE0000u, 0xFFFE0000u, 0xFF00FF00u, 0xFF00FF00u, 0xFF0000FEu, 0xFF0000FEu
    }; /* <-QAC 3120 */

    enum { output_answer_data_n = 3 };  /* Count is by initializer. SEC R3.1.1 (1) */
    const static uint32_t  output_answer_data[ output_answer_data_n ] =
    {
        /* ->QAC 3120 */
        /* See "output_format" */
        0x4CFF4C55u, 0x9515952Bu, 0x1D6B1DFFu
    }; /* <-QAC 3120 */


#ifndef R_OSPL_NDEBUG
    printf( "\nRZ/A1H - PFV sample version %s - Sample_PFV_DMAC\n", PFV_VERSION_STRING );
#endif

    R_UNREFERENCED_VARIABLE_2( argc, argv );


    /* Make input data and output address */
    e= R_SAMPLE_GetVRAM( /*Set*/ &big_memory_address,  &big_memory_size );
    IF(e!=0)
    {
        goto fin;
    }
    IF_DQ( big_memory_address == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }
    e= R_OSPL_ToPhysicalAddress( big_memory_address,  /*Set*/ &big_memory_physical_address );
    IF(e!=0)
    {
        goto fin;
    }
    e= R_OSPL_ToUncachedAddress( big_memory_address,  /*Set*/ &big_memory_address );
    if ( e == E_ACCESS_DENIED )
    {
        R_OSPL_CLEAR_ERROR();
        e= R_OSPL_ToCachedAddress( big_memory_address,  /*Set*/ &big_memory_address );
        is_cached = true;
    }
    else
    {
        is_cached = false;
    }
    IF(e!=0)
    {
        goto fin;
    }
    IF_DQ( big_memory_address == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }

    input_physical_address  = big_memory_physical_address;
    input_address           = big_memory_address;
    output_physical_address =  big_memory_physical_address + sizeof(input_data);
    output_address          = &big_memory_address[  sizeof(input_data) ];
    output_max_size = (int32_t) big_memory_size - (int32_t) sizeof(input_data);
    ASSERT_R( output_max_size > 0,  e=E_OTHERS; goto fin );

    p= memset( output_address, 0, (size_t) output_max_size );
    R_UNREFERENCED_VARIABLE( p ); /* MISRA 16.10 */
    p= memcpy( input_address,  input_data,  sizeof(input_data) );
    R_UNREFERENCED_VARIABLE( p ); /* MISRA 16.10 */
    if ( is_cached )
    {
        R_OSPL_MEMORY_Flush( R_OSPL_FLUSH_WRITEBACK_INVALIDATE );
    }


    /* Call "R_PFV_DMAC_Initialize" */
    {
        pfv_dmac_config_t  config;

        config.flags = F_PFV_DMAC_INPUT_DMAC_CHANNEL | F_PFV_DMAC_OUTPUT_DMAC_CHANNEL;
        config.input_DMAC_channel  = SAMPLE_PFV_DEFAULT_INPUT_DMAC_CHANNEL;
        config.output_DMAC_channel = SAMPLE_PFV_DEFAULT_OUTPUT_DMAC_CHANNEL;

        e= R_PFV_DMAC_Initialize( channel_of_PFV, &config );
        IF ( e != 0 )
        {
            goto  fin;
        }
    }


    /* Call "R_PFV_DMAC_SetIOFormat" */
    {
        pfv_io_format_t  format;

        format.flags = F_PFV_INPUT_FORMAT | F_PFV_OUTPUT_FORMAT;
        format.input_format  = PFV_ARGB8888;
        format.output_format = PFV_YCbCr422;

        e= R_PFV_DMAC_SetIOFormat( channel_of_PFV, &format );
        IF ( e != 0 )
        {
            goto  fin;
        }
    }


    /* Call "R_PFV_DMAC_Transfer" */
    {
        pfv_transfer_config_t  config;

        config.flags =
            F_PFV_TRANSFER_INPUT_BUFFER_ADDRESS |
            F_PFV_TRANSFER_INPUT_BUFFER_SIZE |
            F_PFV_TRANSFER_OUTPUT_BUFFER_ADDRESS |
            F_PFV_TRANSFER_OUTPUT_BUFFER_SIZE;
        config.input_buffer_address = input_physical_address;
        config.input_buffer_size = (uint32_t) sizeof(input_data);
        config.output_buffer_address = output_physical_address;
        config.output_buffer_size = (uint32_t) sizeof(output_answer_data);

        e= R_PFV_DMAC_Transfer( channel_of_PFV, &config );
        IF ( e != 0 )
        {
            goto  fin;
        }


        /* Check output data */
        IF ( memcmp( output_address,
                     output_answer_data, sizeof(output_answer_data) ) != 0 )
        {
            e = E_OTHERS;
            goto  fin;
        }
    }

    e = 0;
fin:
    ee= R_PFV_DMAC_Terminate( channel_of_PFV );
    IF ( (ee != 0) && (e == 0) )
    {
        e = ee;
    }

    R_DEBUG_BREAK_IF_ERROR();
    return  e;
}


/***********************************************************************
* Function: R_Sample_PFV_DMAC_Image
*    Example code of image converting by PFV.
*
* Arguments:
*    argc - 0 = returns after few seconds, 1 = infinite loop
*    argv - The argument of main function. Ignored.
*
* Returns:
*    Error Code. 0=No Error.
************************************************************************/
#ifdef  USE_LIST_WINDOW_SURFACES
errnum_t  R_Sample_PFV_DMAC_Image( int32_t const  argc,
                                   char_t * const* const  argv )
{
    enum {        frame_num_count = 5 * 60 };
    bool_t const  is_infinite_loop = ( argc != 0 );  /* True if from command prompt */

    const graphics_image_t* const   image = &g_pfv_sample_bmp_argb8888;

    errnum_t              e;
    errnum_t              ee;
    int_fast32_t const    channel_of_PFV = SAMPLE_PFV_DEFAULT_PFV_CHANNEL;
    uintptr_t             input_physical_address;
    uint8_t*              input_cached_address;  /* C4017W */
    size_t                input_size;
    uintptr_t             output_physical_address;
    uint8_t*              output_address;
    size_t                output_max_size;
    size_t                output_size;
    size_t                image_size;
    window_surfaces_t     window;
    frame_buffer_t*       frame_buffer;
    int_fast32_t          frame_num;
    uintptr_t             frame_physical_address;
    uint8_t*              frame_address;
    frame_buffer_t        offscreen;
    int_fast32_t          i;
    pfv_color_matrix_t    matrix;
    pfv_color_matrix_t    reset_matrix;
    void*                 p;
    bool_t                is_cached;
    enum { num_400 = 400,  num_800 = 800,  num_40 = 40,  num_2 = 2 };
    enum { num_4_bpp = 4 };


#ifndef R_OSPL_NDEBUG
    printf( "\nRZ/A1H - PFV sample version %s - Sample_PFV_DMAC_Image\n", PFV_VERSION_STRING );
#endif

    R_UNREFERENCED_VARIABLE_2( argc, argv );


    R_WINDOW_SURFACES_InitConst( &window );
    offscreen.buffer_count = 0;


    /* Set "frame_buffer" and show frame buffer to a screen */
    {
        window_surfaces_config_t  config;

        config.flags = F_WINDOW_SURFACES_PIXEL_FORMAT | F_WINDOW_SURFACES_BUFFER_COUNT_MAX;
        config.pixel_format = PIXEL_FORMAT_YCbCr422;
        config.buffer_count_max = 1;

        e= R_WINDOW_SURFACES_Initialize( &window, &config );
        IF ( e != 0 )
        {
            goto  fin;
        }

        e= R_WINDOW_SURFACES_GetLayerFrameBuffer( &window, 0, &frame_buffer );
        IF ( e != 0 )
        {
            goto  fin;
        }
        IF_DQ( frame_buffer == NULL )
        {
            e=E_OTHERS;
            goto fin;
        }

        offscreen.width  = num_400;
        offscreen.stride = num_800;
        offscreen.height =  num_40;
        offscreen.byte_per_pixel = num_2;
        offscreen.buffer_count = 1;
        offscreen.show_buffer_index = 0;
        offscreen.draw_buffer_index = 0;
        e= R_WINDOW_SURFACES_AllocOffscreenStack( &window, &offscreen );
        IF ( e != 0 )
        {
            goto  fin;
        }
    }


    /* Make input data and output address */
    image_size = (size_t) image->width * num_4_bpp * (size_t) image->height;
    e= R_GRAPHICS_IMAGE_GetAddress( image, &input_cached_address );
    IF(e!=0)
    {
        goto fin;
    }
    e= R_OSPL_ToPhysicalAddress( input_cached_address, &input_physical_address );
    IF(e!=0)
    {
        goto fin;
    }
    input_size = image_size;

    /* Address of offscreen buffer */
    e= R_OSPL_ToPhysicalAddress( offscreen.buffer_address[ offscreen.show_buffer_index ],
                                 /*Set*/ &output_physical_address );
    IF(e!=0)
    {
        goto fin;
    }
    e= R_OSPL_ToUncachedAddress( offscreen.buffer_address[ offscreen.show_buffer_index ],
                                 /*Set*/ &output_address );
    if ( e == E_ACCESS_DENIED )
    {
        R_OSPL_CLEAR_ERROR();
        e= R_OSPL_ToCachedAddress( offscreen.buffer_address[ offscreen.show_buffer_index ],
                                   /*Set*/ &output_address );
        is_cached = true;
    }
    else
    {
        is_cached = false;
    }
    IF(e!=0)
    {
        goto fin;
    }
    IF_DQ( output_address == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }
    output_max_size = (size_t)( offscreen.stride * offscreen.height );
    p= memset( output_address, 0, output_max_size );
    R_UNREFERENCED_VARIABLE( p ); /* QAC 3200 : This is not error information */

    /* Address of visible buffer */
    e= R_OSPL_ToPhysicalAddress( frame_buffer->buffer_address[ frame_buffer->show_buffer_index ],
                                 /*Set*/ &frame_physical_address );
    IF(e!=0)
    {
        goto fin;
    }
    e= R_OSPL_ToUncachedAddress( frame_buffer->buffer_address[ frame_buffer->show_buffer_index ],
                                 /*Set*/ &frame_address );
    if ( e == E_ACCESS_DENIED )
    {
        R_OSPL_CLEAR_ERROR();
        e= R_OSPL_ToCachedAddress( frame_buffer->buffer_address[ frame_buffer->show_buffer_index ],
                                   /*Set*/ &frame_address );
        IF( ! is_cached )
        {
            e=E_OTHERS;
            goto fin;
        }
    }
    else
    {
        IF( is_cached )
        {
            e=E_OTHERS;
            goto fin;
        }
    }
    IF(e!=0)
    {
        goto fin;
    }
    IF_DQ( frame_address == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }
    p= memset( frame_address, 0, (size_t)( frame_buffer->stride * frame_buffer->height ) );
    R_UNREFERENCED_VARIABLE( p ); /* QAC 3200 : This is not error information */
    if ( is_cached )
    {
        R_OSPL_MEMORY_Flush( R_OSPL_FLUSH_WRITEBACK_INVALIDATE );
    }


    /* Call "R_PFV_DMAC_Initialize" */
    {
        pfv_dmac_config_t  config;

        config.flags = F_PFV_DMAC_INPUT_DMAC_CHANNEL | F_PFV_DMAC_OUTPUT_DMAC_CHANNEL;
        config.input_DMAC_channel  = SAMPLE_PFV_DEFAULT_INPUT_DMAC_CHANNEL;
        config.output_DMAC_channel = SAMPLE_PFV_DEFAULT_OUTPUT_DMAC_CHANNEL;

        e= R_PFV_DMAC_Initialize( channel_of_PFV, &config );
        IF ( e != 0 )
        {
            goto  fin;
        }
    }


    /* Call "R_PFV_DMAC_SetIOFormat" */
    {
        pfv_io_format_t  format;

        format.flags = F_PFV_INPUT_FORMAT | F_PFV_OUTPUT_FORMAT;
        format.input_format  = PFV_ARGB8888;
        format.output_format = PFV_YCbCr422;

        e= R_PFV_DMAC_SetIOFormat( channel_of_PFV, &format );
        IF ( e != 0 )
        {
            goto  fin;
        }
    }


    /* Set "reset_matrix" */
    e= R_PFV_DMAC_GetImageColorMatrix( channel_of_PFV, &reset_matrix );
    IF ( e != 0 )
    {
        goto  fin;
    }


    for ( frame_num = 0;
            (frame_num < frame_num_count) || (is_infinite_loop);
            frame_num += 1 )
    {
        int_fast32_t  gain_;  /* _ is for MISRA 5.6 */
        enum {        divide = 128 };
        /* Deviding count of FullColor -> Black  or  Black -> FullColor */
        enum {        divide_of_all_repeat = divide * 2 };


        /* Set "gain_" : Triangle wave */
        gain_ = frame_num % divide_of_all_repeat;
        if ( gain_ >= divide )
        {
            gain_ = divide_of_all_repeat - gain_;
        }


        /* Call "R_PFV_DMAC_SetImageColorMatrix" : Use "gain_" */
        matrix.mode = reset_matrix.mode;
        for ( i = 0;  i < (int_fast32_t) R_COUNT_OF( matrix.dc_offset_plus_128 );  i += 1 )
        {
            matrix.dc_offset_plus_128[i] = reset_matrix.dc_offset_plus_128[i];
        }
        for ( i = 0;  i < (int_fast32_t) R_COUNT_OF( matrix.matrix_multiply_256 );  i += 1 )
        {
            matrix.matrix_multiply_256[i] = (int16_t)(
                                                ( reset_matrix.matrix_multiply_256[i] * gain_ ) / divide );
        }
        e= R_PFV_DMAC_SetImageColorMatrix( channel_of_PFV, &matrix );
        IF ( e != 0 )
        {
            goto  fin;
        }


        /* Call "R_PFV_DMAC_Transfer" */
        {
            pfv_transfer_config_t  config;

            config.flags =
                F_PFV_TRANSFER_INPUT_BUFFER_ADDRESS |
                F_PFV_TRANSFER_INPUT_BUFFER_SIZE |
                F_PFV_TRANSFER_OUTPUT_BUFFER_ADDRESS |
                F_PFV_TRANSFER_OUTPUT_BUFFER_SIZE |
                F_PFV_TRANSFER_BUFFER_WIDTH |
                F_PFV_TRANSFER_BUFFER_HEIGHT;
            config.input_buffer_address = input_physical_address;
            config.input_buffer_size = input_size;
            config.output_buffer_address = output_physical_address;
            config.output_buffer_size = output_max_size;
            config.buffer_width  = (int_fast32_t) image->width;
            config.buffer_height = (int_fast32_t) image->height;

            e= R_PFV_DMAC_Transfer( channel_of_PFV, &config );
            IF ( e != 0 )
            {
                goto  fin;
            }

            output_size = config.output_buffer_size;
        }


        /* Move each pixel lines by stride */
        {
            int_fast32_t  y_;  /* _ is for MISRA 5.6 */
            uint8_t*      source_address;
            uint8_t*      destination_address;
            int32_t       image_stride = (int32_t) image->width * offscreen.byte_per_pixel;
            enum {        show_x = 32 };
            enum {        show_y = 32 };

            for ( y_ = (int_fast32_t) image->height - 1;  y_ >= 0;  y_ -= 1 )
            {
                source_address = &output_address[ y_ * image_stride ];
                destination_address = &frame_address[
                                          ( (y_ + show_y) * frame_buffer->stride ) +
                                          ( show_x * frame_buffer->byte_per_pixel ) ];

                p= memmove( destination_address, source_address, (size_t) image_stride );
                R_UNREFERENCED_VARIABLE( p );
                /* QAC 3200 : This is not error information */

                p= memset( source_address, 0, (size_t) image_stride );
                R_UNREFERENCED_VARIABLE( p );
                /* QAC 3200 : This is not error information */

                if ( is_cached )
                {
                    R_OSPL_MEMORY_Flush( R_OSPL_FLUSH_WRITEBACK_INVALIDATE );
                }
            }
        }


        /* Check output data size */
        /* "/ 2" is bit count of "PFV_ARGB8888" / "PFV_YCbCr422" */
        IF ( output_size != (input_size / num_2) )
        {
            e = E_OTHERS;
            goto  fin;
        }


        e= R_WINDOW_SURFACES_SwapBuffers( &window, 0, NULL );
        IF(e!=0)
        {
            goto fin;
        }
    }

    e = 0;
fin:
    ee= R_PFV_DMAC_Terminate( channel_of_PFV );
    IF ( (ee != 0) && (e == 0) )
    {
        e = ee;
    }

    ee= R_WINDOW_SURFACES_FreeOffscreenStack( &window, &offscreen );
    IF ( (ee != 0) && (e == 0) )
    {
        e = ee;
    }

    e= R_WINDOW_SURFACES_Finalize( &window, e );

    R_DEBUG_BREAK_IF_ERROR();
    return  e;
}
#endif


