/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2015 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* File Name   : retarget.c
* $Rev: 1320 $
* $Date:: 2015-01-29 15:54:51 +0900#$
* Description : Retarget
*******************************************************************************/


        #ifdef        __STANDALONE__            /* Semihosting disabled    */

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include <stdio.h>
#include "r_typedefs.h"
#include "sio_char.h"

#pragma import(__use_no_semihosting)

/******************************************************************************
Typedef definitions
******************************************************************************/
struct __FILE
{
  int_t handle;

  /* Whatever you require here. If the only file you are using is   */
  /* standard output using printf() for debugging, no file handling */
  /* is required.                                                   */
};

/******************************************************************************
Macro definitions
******************************************************************************/
/* File descriptor */
#define STDIN       (0)
#define STDOUT      (1)
#define STDERR      (2)

#define IOSTREAM    (1)
#define BUFF_SIZE   (256)

/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/


/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/
FILE __stdout;
FILE __stdin;

/******************************************************************************
Private global variables and functions
******************************************************************************/
/*****************************************************************************/
/* Definition of static variables used in low-level interface routines       */
/*****************************************************************************/
static int_t lowsrc_io_buff[IOSTREAM][BUFF_SIZE];


/******************************************************************************
* Function Name: fgetc
* Description  : Obtains the next character as an unsigned char converted to
*              : an int, from the input stream pointed to by stream.
* Arguments    : FILE * file_p
* Return Value : int_t
******************************************************************************/
int_t fgetc(FILE * file_p)
{
    static int32_t buff_num = 0;
    static int32_t read_pos = 0;
    int_t          get_data;


    if (0 == buff_num)
    {
        buff_num = SioRead(STDIN, &lowsrc_io_buff[0][0], BUFF_SIZE);
        read_pos = 0;
    }

    if (buff_num > 0)
    {
        get_data = lowsrc_io_buff[0][read_pos++];
        buff_num--;

        return get_data;
    }

    /* no character to read */
    return EOF;
}

/******************************************************************************
* Function Name: fputc
* Description  : writes the character tothe output stream pointed to by stream,
*              : at the position indicated by the asociated file position indicator,
*              : and advances the indicator appropriately.
* Arguments    : int_t character
*              : FILE * file_p
* Return Value : int_t
******************************************************************************/
int_t fputc(int_t character, FILE * file_p)
{
    int_t temp_data;

    temp_data = character;

    SioWrite(STDOUT, &temp_data, 1);

    return character;
}

/******************************************************************************
* Function Name: ferror
* Description  : Tests the error indicator for the stream pointed to by stream.
* Arguments    : FILE * file_p
* Return Value : int_t
******************************************************************************/
int_t ferror(FILE * file_p)
{
    return 0;
}

/******************************************************************************
* Function Name: __backspace
* Description  : This function must only be called after reading a character 
*              : from the stream. You must not call it after a write, a seek, 
*              : or immediately after opening the file, for example. It returns 
*              : to the stream the last character that was read from the stream, 
*              : so that the same character can be read from the stream again 
*              : by the next read operation. 
* Arguments    : FILE * file_p
* Return Value : int_t
******************************************************************************/
int_t __backspace(FILE * file_p)
{
    return 0;
}

/******************************************************************************
* Function Name: _ttywrch
* Description  : Write a character to the output channel. This function is used
*              : for last-resort error message output.
* Arguments    : int_t character
* Return Value : none
******************************************************************************/
void _ttywrch(int_t character)
{
    /* Do Nothing */
}

/******************************************************************************
* Function Name: _sys_exit
* Description  : Terminate the program, passing a return code back to the user.
* Arguments    : int_t returncode
* Return Value : none
******************************************************************************/
void _sys_exit(int_t returncode)
{
    while (1)
    {
        /* Do Nothing */
    }
}

/******************************************************************************
* Function Name: _sys_command_string
* Description  : Return a pointer to the command line used to invoke the program.
*              : The supplied buffer may be used to store the string, but need
*              : not be.
* Arguments    : char_t * cmd
*              : int_t len
* Return Value : char_t *
******************************************************************************/
char_t * _sys_command_string(char_t * cmd, int_t len)
{
    return cmd;
}

    #endif        /* __STANDALONE__    Semihosting disabled    */


/* End of File */

