/*******************
 *
 * Copyright 1998-2010 IAR Systems AB.
 *
 * This is a template implementation of the "__read" function used by
 * the standard library.  Replace it with a system-specific
 * implementation.
 *
 * The "__read" function reads a number of bytes, at most "size" into
 * the memory area pointed to by "buffer".  It returns the number of
 * bytes read, 0 at the end of the file, or _LLIO_ERROR if failure
 * occurs.
 *
 * The template implementation below assumes that the application
 * provides the function "MyLowLevelGetchar".  It should return a
 * character value, or -1 on failure.
 *
 ********************/

#include <yfuns.h>
#include "r_typedefs.h"
#include "sio_char.h"

_STD_BEGIN

#pragma module_name = "?__read"

#define BUFF_SIZE	(256)

static int_t lowsrc_io_buff[BUFF_SIZE];

size_t __read(int handle, unsigned char * buffer, size_t size)
{
	/* Remove the #if #endif pair to enable the implementation */
#if 1
#if 0
	int nChars = 0;

	/* This template only reads from "standard in", for all other file
	 * handles it returns failure. */
	if (handle != _LLIO_STDIN)
	{
		return _LLIO_ERROR;
	}

	for (/* Empty */; size > 0; --size)
	{
		int c = MyLowLevelGetchar();
		if (c < 0)
			break;

		*buffer++ = c;
		++nChars;
	}
#else
	static int32_t buff_num = 0;
	static int32_t read_pos = 0;
	int32_t        nChars = 0;
	int32_t        idx;

	/* This template only reads from "standard in", for all other file
	 * handles it returns failure. */
	if (handle != _LLIO_STDIN)
	{
		return _LLIO_ERROR;
	}

	if( size < 1 )
	{
		nChars = 0;
	}
	else
	{
		if (0 == buff_num)
		{
			buff_num = SioRead(handle, &lowsrc_io_buff[0], BUFF_SIZE);
			read_pos = 0;
		}

		if (buff_num > 0)
		{
			if (buff_num < size)
			{
				nChars = buff_num;
			}
			else
			{
				nChars = size;
			}
			for (idx = 0; idx < nChars; ++idx)
			{
				buffer[idx] = (unsigned char)(lowsrc_io_buff[read_pos++]);
				buff_num--;
			}
		}
		else
		{
			nChars = buff_num;
		}
	}

	if (nChars < 0)
	{
		return _LLIO_ERROR;
	}
#endif

	return nChars;

#else

	/* Always return error code when implementation is disabled. */
	return _LLIO_ERROR;

#endif
}

_STD_END
/*EOF*/
