/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2014 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/******************************************************************************
* File Name    : pl310.c
* $Rev:  $
* $Date: 2015-09-17T11:52:15+09:00 $
* Description  : PL310
******************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include "r_typedefs.h"
#include "core_ca9_io.h"
#include "pl310.h"


/******************************************************************************
Typedef definitions
******************************************************************************/

/******************************************************************************
Macro definitions
******************************************************************************/
#define Renesas_RZ_A1_PL310_BASE                   (0x3ffff000UL)

/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/

/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/

/******************************************************************************
Private global variables and functions
******************************************************************************/


/******************************************************************************
* Function Name: init_PL310
* Description  : Initialize 2nd cache
* Arguments    : None
* Return Value : None
******************************************************************************/
void  init_PL310(void)
{
	unsigned int l2_id;

	l2_id = PL310_GetID();
	if (l2_id)
	{
		PL310_InvAllByWay();
		PL310_Enable();
	}
}


/**************************************************************************//**
 * @file     pl310.c
 * @brief    Implementation of pl310 functions
 * @version
 * @date     11 June 2013
 *
 * @note
 *
 ******************************************************************************/
/* Copyright (c) 2011 - 2013 ARM LIMITED

   All rights reserved.
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:
   - Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
   - Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
   - Neither the name of ARM nor the names of its contributors may be used
     to endorse or promote products derived from this software without
     specific prior written permission.
   *
   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL COPYRIGHT HOLDERS AND CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
   ---------------------------------------------------------------------------*/

//Cache Sync operation
void PL310_Sync(void)
{
    PL310->CACHE_SYNC = 0x0;
}

//return Cache controller cache ID
int PL310_GetID (void)
{
    return PL310->CACHE_ID;
}

//return Cache controller cache Type
int PL310_GetType (void)
{
    return PL310->CACHE_TYPE;
}

//Invalidate all cache by way
void PL310_InvAllByWay (void)
{
    unsigned int assoc;

    if (PL310->AUX_CNT & (1<<16))
        assoc = 16;
    else
        assoc =  8;

    PL310->INV_WAY = (1 << assoc) - 1;
    while(PL310->INV_WAY && ((1 << assoc) - 1)); //poll invalidate

    PL310_Sync();
}

//Clean and Invalidate all cache by way
void PL310_CleanInvAllByWay (void)
{
    unsigned int assoc;

    if (PL310->AUX_CNT & (1<<16))
        assoc = 16;
    else
        assoc =  8;

    PL310->CLEAN_INV_WAY = (1 << assoc) - 1;
    while(PL310->CLEAN_INV_WAY && ((1 << assoc) - 1)); //poll invalidate

    PL310_Sync();
}

//Enable Cache
void PL310_Enable(void)
{
    PL310->CONTROL = 0;
    PL310->INTERRUPT_CLEAR = 0;
    PL310->DEBUG_CONTROL = 0;
    PL310->DATA_LOCK_0_WAY = 0;
    PL310->CACHE_SYNC = 0;

    PL310->CONTROL = 0x01;
    PL310_Sync();
}
//Disable Cache
void PL310_Disable(void)
{
    PL310->CONTROL = 0x00;
    PL310_Sync();
}

//Invalidate cache by physical address
void PL310_InvPa (void *pa)
{
    PL310->INV_LINE_PA = (unsigned int)pa;
    PL310_Sync();
}

//Clean cache by physical address
void PL310_CleanPa (void *pa)
{
    PL310->CLEAN_LINE_PA = (unsigned int)pa;
    PL310_Sync();
}

//Clean and invalidate cache by physical address
void PL310_CleanInvPa (void *pa)
{
    PL310->CLEAN_INV_LINE_PA = (unsigned int)pa;
    PL310_Sync();
}


