/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2015 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/******************************************************************************
* File: frame_buffer.h
*    Frame Buffer and Color Format
*
* - $Version: $ (=RGA_VERSION)
* - $ModuleRevision: {Masters}\RGA\FrameBuffer\25_RStyled $
* - $Rev: $
* - $Date: 2017-01-16T14:39:17+09:00 $
******************************************************************************/

#ifndef  FRAME_BUFFER_H
#define  FRAME_BUFFER_H

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include  "frame_buffer_typedef.h"
#include  "r_multi_compiler_typedef.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/******************************************************************************
Typedef definitions
******************************************************************************/
/* In "frame_buffer_typedef.h" */

/******************************************************************************
Macro definitions
******************************************************************************/
/* In "frame_buffer_typedef.h" */

/******************************************************************************
Variable Externs
******************************************************************************/
/* In "frame_buffer_typedef.h" */

/******************************************************************************
Functions Prototypes
******************************************************************************/

/***********************************************************************
* Function: R_RGA_Get_R8G8B8A8
*    Get R8G8B8A8 type color
*
* Arguments:
*    Red - Red.
*    Green - Green.
*    Blue - Blue.
*    Alpha - Alpha.
*
* Return Value:
*    Color code as "r8g8b8a8_t"
************************************************************************/
INLINE r8g8b8a8_t  R_RGA_Get_R8G8B8A8(
    int_fast32_t const  in_Red,
    int_fast32_t const  in_Green,
    int_fast32_t const  in_Blue,
    int_fast32_t const  in_Alpha );


/***********************************************************************
* Function: argb8888_t_to_rgb565_t
*    Converts color format of one pixel.
************************************************************************/
INLINE  rgb565_t  argb8888_t_to_rgb565_t( argb8888_t in_Color );


/***********************************************************************
* Function: argb8888_t_to_argb1555_t
*    Converts color format of one pixel.
************************************************************************/
INLINE  argb1555_t  argb8888_t_to_argb1555_t( argb8888_t in_Color );


/***********************************************************************
* Function: argb8888_t_to_argb4444_t
*    Converts color format of one pixel.
************************************************************************/
INLINE  argb4444_t  argb8888_t_to_argb4444_t( argb8888_t in_Color );


/***********************************************************************
* Function: argb8888_t_to_r8g8b8a8_t
*    Converts color format of one pixel.
************************************************************************/
INLINE  r8g8b8a8_t  argb8888_t_to_r8g8b8a8_t( argb8888_t in_Color );


/***********************************************************************
* Function: rgb565_t_to_argb8888_t
*    Converts color format of one pixel.
************************************************************************/
INLINE  argb8888_t  rgb565_t_to_argb8888_t( rgb565_t in_Color );


/***********************************************************************
* Function: argb1555_t_to_argb8888_t
*    Converts color format of one pixel.
************************************************************************/
INLINE  argb8888_t  argb1555_t_to_argb8888_t( argb1555_t in_Color );


/***********************************************************************
* Function: argb4444_t_to_argb8888_t
*    Converts color format of one pixel.
************************************************************************/
INLINE  argb8888_t  argb4444_t_to_argb8888_t( argb4444_t in_Color );


/***********************************************************************
* Function: rgba8888_t_to_argb8888_t
*    Converts color format of one pixel.
************************************************************************/
INLINE argb8888_t  rgba8888_t_to_argb8888_t( rgba8888_t in_Color );


/***********************************************************************
* Function: rgba8888_t_to_rgb565_t
*    Converts color format of one pixel.
************************************************************************/
INLINE rgb565_t  rgba8888_t_to_rgb565_t( rgba8888_t in_Color );


/***********************************************************************
* Function: rgba8888_t_to_argb1555_t
*    Converts color format of one pixel.
************************************************************************/
INLINE argb1555_t  rgba8888_t_to_argb1555_t( rgba8888_t in_Color );


/***********************************************************************
* Function: rgba8888_t_to_argb4444_t
*    Converts color format of one pixel.
************************************************************************/
INLINE argb4444_t  rgba8888_t_to_argb4444_t( rgba8888_t in_Color );


/***********************************************************************
* Function: r8g8b8a8_t_to_argb8888_t
*    Converts color format of one pixel.
************************************************************************/
INLINE  argb8888_t  r8g8b8a8_t_to_argb8888_t( r8g8b8a8_t in_Color );


#ifdef __cplusplus
}  /* extern "C" */
#endif /* __cplusplus */

/* Inline Functions */
#include  "frame_buffer_inline.h"

#endif
