/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2016 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/***********************************************************************
* File: locking_user.h
*    Lock related FIT BSP. User defined.
*
* - $Version: $ (=R_OSPL_VERSION)
* - $ModuleRevision: {Masters}\OSPL\ALL_Driver_parent\27_RStyled $
* - $Rev: $
* - $Date: 2017-08-29T10:34:42+09:00 $
************************************************************************/

#ifndef LOCKING_USER_H
#define LOCKING_USER_H


/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include  "r_ospl_config.h"
#include  "r_typedefs.h"
#include  "locking_typedef.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/******************************************************************************
Typedef definitions
******************************************************************************/

/***********************************************************************
* Structure: r_ospl_user_lock_t
*    Example of user defined lock type
************************************************************************/
#if  R_OSPL_C_LOCK_CODE
typedef struct st_r_ospl_user_lock_t  r_ospl_user_lock_t;
struct st_r_ospl_user_lock_t
{
    int  UserLockMember;
};
#endif


/******************************************************************************
Macro definitions
******************************************************************************/

/***********************************************************************
* Macro: BSP_CFG_USER_LOCKING_SW_LOCK_FUNCTION
*    Start C-lock for software module
*
* Description:
*    The value can be changed.
*    This is enabled, when "BSP_CFG_USER_LOCKING_ENABLED == 1".
************************************************************************/
#if  R_OSPL_C_LOCK_CODE
#define  BSP_CFG_USER_LOCKING_SW_LOCK_FUNCTION    R_OSPL_USER_LOCK_SoftwareLock
#endif


/***********************************************************************
* Macro: BSP_CFG_USER_LOCKING_SW_UNLOCK_FUNCTION
*    End C-lock for software module
*
* Description:
*    The value can be changed.
*    This is enabled, when "BSP_CFG_USER_LOCKING_ENABLED == 1".
************************************************************************/
#if  R_OSPL_C_LOCK_CODE
#define  BSP_CFG_USER_LOCKING_SW_UNLOCK_FUNCTION  R_OSPL_USER_LOCK_SoftwareUnlock
#endif


/***********************************************************************
* Macro: BSP_CFG_USER_LOCKING_HW_LOCK_FUNCTION
*    Start C-lock for hardware module
*
* Description:
*    The value can be changed.
*    This is enabled, when "BSP_CFG_USER_LOCKING_ENABLED == 1".
************************************************************************/
#if  R_OSPL_C_LOCK_CODE
#define  BSP_CFG_USER_LOCKING_HW_LOCK_FUNCTION    R_OSPL_USER_LOCK_HardwareLock
#endif


/***********************************************************************
* Macro: BSP_CFG_USER_LOCKING_HW_UNLOCK_FUNCTION
*    End C-lock for hardware module
*
* Description:
*    The value can be changed.
*    This is enabled, when "BSP_CFG_USER_LOCKING_ENABLED == 1".
************************************************************************/
#if  R_OSPL_C_LOCK_CODE
#define  BSP_CFG_USER_LOCKING_HW_UNLOCK_FUNCTION  R_OSPL_USER_LOCK_HardwareUnlock
#endif


/******************************************************************************
Variable Externs
******************************************************************************/

/******************************************************************************
Functions Prototypes
******************************************************************************/

/***********************************************************************
* Function: R_OSPL_USER_LOCK_SoftwareLock
*    Example of BSP_CFG_USER_LOCKING_SW_LOCK_FUNCTION
************************************************************************/
#if  R_OSPL_C_LOCK_CODE
#if BSP_CFG_USER_LOCKING_ENABLED == 1
bool_t  R_OSPL_USER_LOCK_SoftwareLock( r_ospl_user_lock_t* LockObject );
#endif
#endif


/***********************************************************************
* Function: R_OSPL_USER_LOCK_SoftwareUnlock
*    Example of BSP_CFG_USER_LOCKING_SW_UNLOCK_FUNCTION
************************************************************************/
#if  R_OSPL_C_LOCK_CODE
#if BSP_CFG_USER_LOCKING_ENABLED == 1
bool_t  R_OSPL_USER_LOCK_SoftwareUnlock( r_ospl_user_lock_t* LockObject );
#endif
#endif


/***********************************************************************
* Function: R_OSPL_USER_LOCK_HardwareLock
*    Example of BSP_CFG_USER_LOCKING_HW_LOCK_FUNCTION
************************************************************************/
#if  R_OSPL_C_LOCK_CODE
#if BSP_CFG_USER_LOCKING_ENABLED == 1
bool_t  R_OSPL_USER_LOCK_HardwareLock( mcu_lock_t  HardwareIndex );
#endif
#endif


/***********************************************************************
* Function: R_OSPL_USER_LOCK_HardwareUnlock
*    Example of BSP_CFG_USER_LOCKING_HW_UNLOCK_FUNCTION
************************************************************************/
#if  R_OSPL_C_LOCK_CODE
#if BSP_CFG_USER_LOCKING_ENABLED == 1
bool_t  R_OSPL_USER_LOCK_HardwareUnlock( mcu_lock_t  HardwareIndex );
#endif
#endif


/***********************************************************************
* Function: R_OSPL_GetHardwareLockObjectForTest
*    For Test
************************************************************************/
#if  R_OSPL_C_LOCK_CODE
#if BSP_CFG_USER_LOCKING_ENABLED == 1
r_ospl_user_lock_t*  R_OSPL_GetHardwareLockObjectForTest( mcu_lock_t  HardwareIndex );
#endif
#endif


/***********************************************************************
* End of File:
************************************************************************/
#ifdef __cplusplus
}  /* extern "C" */
#endif /* __cplusplus */

#endif /* LOCKING_USER_H */

