/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2016 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/***********************************************************************
* File: ospl_os_less.h
*    OS Porting Layer API for OS less Compatibility
*
* - $Version: $ (=R_OSPL_VERSION)
* - $ModuleRevision: {Masters}\OSPL\ALL_Driver_parent\27_RStyled $
* - $Rev: $
* - $Date: 2017-08-29T10:34:42+09:00 $
************************************************************************/

#ifndef OSPL_OS_LESS_H
#define OSPL_OS_LESS_H

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include  "r_ospl_config.h"
#include  "r_ospl_os_less_typedef.h"
#include  "./r_ospl.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/******************************************************************************
Typedef definitions
******************************************************************************/
/* In "ospl_os_less_typedef.h" */

/******************************************************************************
Macro definitions
******************************************************************************/
/* In "ospl_os_less_typedef.h" */

/******************************************************************************
Variable Externs
******************************************************************************/
/* In "ospl_os_less_typedef.h" */

/******************************************************************************
Functions Prototypes
******************************************************************************/

/***********************************************************************
* Function: R_OSPL_THREAD_Create
*    Creates a pseudo thread (for OS less)
*
* Arguments:
*    ThreadDef    - Work area of the thread and initial value returned by <R_OSPL_THREAD> function.
*    Argument     - The parameter passing to <R_OSPL_THREAD_GetArgument> function.
*    out_ThreadId - Output: Thread ID
*
* Return Value:
*    Error code.  If there is no error, the return value is 0.
*
* Description:
*    There is only when "R_OSPL_IS_PREEMPTION" = 0.
*    Usually, Use it for creating more than one event.
*
* Example:
*    > r_ospl_thread_id_t  g_ThreadA_Id;
*    > R_OSPL_THREAD_DEF( g_ThreadA_Def );
*    >
*    > void  main()
*    > {
*    >     errnum_t  e;
*    >
*    >     e= R_OSPL_THREAD_Create( R_OSPL_THREAD( g_ThreadA_Def ),
*    >         NULL, &g_ThreadA_Id );
*    >         IF(e){goto fin;}
*    > }
************************************************************************/
#if  R_OSPL_THREAD_CODE
#if  ! R_OSPL_IS_PREEMPTION
errnum_t  R_OSPL_THREAD_Create( r_ospl_thread_def_t* const  ThreadDef, void* const  Argument,
                                r_ospl_thread_id_t* const  out_ThreadId );
#endif
#endif


/***********************************************************************
* Function: R_OSPL_THREAD_Destroy
*    Destroy a pseudo thread (for OS less)
************************************************************************/
#if  R_OSPL_THREAD_CODE
#if  ! R_OSPL_IS_PREEMPTION
errnum_t  R_OSPL_THREAD_Destroy( r_ospl_thread_id_t* const  in_out_ThreadId );
#endif
#endif


/***********************************************************************
* Function: R_OSPL_THREAD_GetArgument
*    Returns parameters passed when the running thread was created (for OS less)
*
* Arguments:
*    None
*
* Return Value:
*    "Argument" argument of <R_OSPL_THREAD_Create> function
*
* Description:
*    There is only when "R_OSPL_IS_PREEMPTION" = 0.
*    Call "R_OSPL_THREAD_SetCurrentId" function from the message loop
*    before calling this function.
************************************************************************/
#if  R_OSPL_THREAD_CODE
#if  ! R_OSPL_IS_PREEMPTION
void*  R_OSPL_THREAD_GetArgument(void);
#endif
#endif


/***********************************************************************
* Function: R_OSPL_THREAD_SetCurrentId
*    Set running thread ID (for OS less)
*
* Arguments:
*    ThreadId - Thread ID setting to current running state.
*
* Return Value:
*    None
*
* Description:
*    There is only when "R_OSPL_IS_PREEMPTION" = 0.
*    Call this function from the message loop on the pseudo multi-thread environment.
*
* Example:
*    > R_OSPL_THREAD_SetCurrentId( ChildThreadId );
*    > ChildThreadFunction();
*    > R_OSPL_THREAD_SetCurrentId( MainThreadId );
************************************************************************/
#if  R_OSPL_THREAD_CODE
#if  ! R_OSPL_IS_PREEMPTION
void  R_OSPL_THREAD_SetCurrentId( r_ospl_thread_id_t const  ThreadId );
#endif
#endif


/***********************************************************************
* Function: R_OSPL_THREAD_GetMainId
*    Returns main thread ID (for OS less)
*
* Arguments:
*    None
*
* Return Value:
*    Main thread ID
*
* Description:
*    There is only when <R_OSPL_IS_PREEMPTION> = 0.
*    Call <R_OSPL_THREAD_SetCurrentId> function passed main thread ID at
*    the first of "main" function.
************************************************************************/
#if  R_OSPL_THREAD_CODE
#if  ! R_OSPL_IS_PREEMPTION
r_ospl_thread_id_t  R_OSPL_THREAD_GetMainId(void);
#endif
#endif


/***********************************************************************
* Function: R_OSPL_THREAD_SetOnWait
*    Set waiting behavior of current thread.
*
* Arguments:
*    OnWait - Behavior on waiting
*
* Return Value:
*    Error code.  If there is no error, the return value is 0.
*
* Description:
*    - Case of <R_OSPL_IS_PREEMPTION> = 0:
*
*    Initial value is <R_OSPL_WAIT_POLLING>.
*    If this function was called from the interrupt context, E_STATE error is raised.
*
*    - Case of <R_OSPL_IS_PREEMPTION> = 1:
*
*    This function is for compatibility only.
*    Arguments are ignored.
*    This function returns 0.
*
*    Refer to: <R_OSPL_THREAD_GetIsWaiting>
************************************************************************/
#if  R_OSPL_THREAD_CODE
#if  ! R_OSPL_IS_PREEMPTION
errnum_t  R_OSPL_THREAD_SetOnWait( r_ospl_wait_t const  OnWait );
#else
#ifdef _SH
#pragma inline  R_OSPL_THREAD_SetOnWait
#endif
INLINE errnum_t  R_OSPL_THREAD_SetOnWait( r_ospl_wait_t const  OnWait )
{
    return  0;
}
#endif
#endif


/***********************************************************************
* Function: R_OSPL_THREAD_GetOnWait
*    Get waiting behavior of current thread.
*
* Arguments:
*    None
*
* Return Value:
*    Behavior on waiting
*
* Description:
*    - Case of <R_OSPL_IS_PREEMPTION> = 0:
*
*    Initial value is <R_OSPL_WAIT_POLLING>.
*    If this function was called from the interrupt context,
*    this function returns <R_OSPL_WAIT_POLLING>.
*
*    - Case of <R_OSPL_IS_PREEMPTION> = 1:
*
*    This function is for compatibility only.
*    This function returns R_OSPL_WAIT_POLLING.
*    But it does not polling on waiting
*
*    Refer to: <R_OSPL_THREAD_SetOnWait>
************************************************************************/
#if  R_OSPL_THREAD_CODE
#if  ! R_OSPL_IS_PREEMPTION
r_ospl_wait_t  R_OSPL_THREAD_GetOnWait(void);
#else
#ifdef _SH
#pragma inline  R_OSPL_THREAD_GetOnWait
#endif
INLINE r_ospl_wait_t  R_OSPL_THREAD_GetOnWait(void)
{
    return  R_OSPL_WAIT_POLLING;
}
#endif
#endif


/***********************************************************************
* Function: R_OSPL_THREAD_GetIsWaiting
*    Get whether the current thread is waiting or not.
*
* Arguments:
*    None
*
* Return Value:
*    Whether the current thread is waiting or not
*
* Description:
*    - Case of <R_OSPL_IS_PREEMPTION> = 0:
*
*    If <R_OSPL_WAIT_PM_THREAD> was set by <R_OSPL_THREAD_SetOnWait> function,
*    some waiting functions return soon even if the state is waiting and
*    <R_OSPL_THREAD_GetIsWaiting> function returns true.
*
*    If time out was set to 0, <R_OSPL_THREAD_GetIsWaiting> function returns
*    true at time out, even if any value was passed to <R_OSPL_THREAD_SetOnWait>
*    function,
*
*    If this function was called from the interrupt context, this function
*    returns false and <ASSERT_D> in this function notifies in debug configuration.
*
*    - Case of <R_OSPL_IS_PREEMPTION> = 1:
*
*    This function is for compatibility only.
*    This function returns false.
*
* Example:
*    > e= R_OSPL_Delay( 100 ); IF(e){goto fin;}
*    > if ( R_OSPL_THREAD_GetIsWaiting() ) { e=0; goto fin; }
************************************************************************/
#if  R_OSPL_THREAD_CODE
#if  ! R_OSPL_IS_PREEMPTION
bool_t  R_OSPL_THREAD_GetIsWaiting(void);
#else
#ifdef _SH
#pragma inline  R_OSPL_THREAD_GetIsWaiting
#endif
INLINE bool_t  R_OSPL_THREAD_GetIsWaiting(void)
{
    return  false;
}
#endif
#endif


/***********************************************************************
* Function: R_OSPL_THREAD_ExitWaiting
*    Exit waiting state, if current thread was waiting state.
*
* Arguments:
*    None
*
* Return Value:
*    None
*
* Description:
*    - Case of <R_OSPL_IS_PREEMPTION> = 0:
*
*    The thread returned true from <R_OSPL_THREAD_GetIsWaiting> function
*    after a waiting function must call the waiting function again.
*    After exiting waiting state, other operation and other waiting can
*    be done. If time out was set to 0, exiting does not have to do.
*
*    If it was detected in OSPL API that necessary exiting was not done,
*    E_STATE error is raised. However sometimes the state can not be detected.
*    In this case, time out will be not correct.
*
*    This function does not do anything called from the interrupt context.
*    "ASSERT_D" in this function notifies in debug configuration.
*
*    - Case of <R_OSPL_IS_PREEMPTION> = 1:
*
*    This function is for compatibility only.
*    This function does not do anything.
************************************************************************/
#if  R_OSPL_THREAD_CODE
#if  ! R_OSPL_IS_PREEMPTION
void  R_OSPL_THREAD_ExitWaiting(void);
#else
#ifdef _SH
#pragma inline  R_OSPL_THREAD_ExitWaiting
#endif
INLINE void  R_OSPL_THREAD_ExitWaiting(void)
{
}
#endif
#endif


/***********************************************************************
* Implement: R_OSPL_THREAD_SetDelegate
************************************************************************/
#if  R_OSPL_THREAD_CODE
void  R_OSPL_THREAD_SetDelegate( r_ospl_thread_id_t const  in_ThreadId,  void*  in_Delegate );
#endif


/***********************************************************************
* Function: R_OSPL_THREAD_GetDelegate
************************************************************************/
#if  R_OSPL_THREAD_CODE
void*  R_OSPL_THREAD_GetDelegate( r_ospl_thread_id_t const  in_ThreadId );
#endif


/***********************************************************************
* Function: R_OSPL_EVENT_OBJECT_Create
************************************************************************/
#if  R_OSPL_EVENT_OBJECT_CODE
errnum_t  R_OSPL_EVENT_OBJECT_Create( r_ospl_event_object_id_t* const  out_EventObject,
                                      r_ospl_event_object_def_t* const  in_EventObjectDef );
#endif


/***********************************************************************
* Function: R_OSPL_OnIdleDefault
*    The default callback function on idle state (for OS less)
*
* Arguments:
*    IdleEvent - The kind about idle state
*
* Return Value:
*    Error code.  If there is no error, the return value is 0.
*
* Description:
*    Callbacks many times at idle state.
*
*    This function is <r_ospl_idle_callback_t> type.
*
*    This function shows CPU load by calling <R_OSPL_IDLE_Print_CPU_Load>
*    function during periodic intervals, if <R_OSPL_CPU_LOAD> was set to be 1
*    and <R_OSPL_IDLE_Start_CPU_Load> function was called.
*    However, this function does not show during periodic intervals,
*    if waiting API was not called.
*
*    This function can be customized by application developer.
************************************************************************/
#if  R_OSPL_THREAD_CODE
#if  ! R_OSPL_IS_PREEMPTION
errnum_t  R_OSPL_OnIdleDefault( r_ospl_idle_event_t const  IdleEvent );
#endif
#endif


/***********************************************************************
* Function: R_OSPL_IDLE_Start_CPU_Load
*    Starts to measure CPU load (for OS less)
*
* Arguments:
*    Interval_msec - The periodic intervals to show (msec), 0=Not show
*
* Return Value:
*    None
*
* Description:
*    This function sets OSPL to call printf with CPU load from
*    <R_OSPL_OnIdleDefault> function during periodic intervals.
*
*    This function is enabled, if OSPL was compiled with setting
*    <R_OSPL_CPU_LOAD> to 1.
*    Call <R_OSPL_IDLE_Print_CPU_Load> function, when CPU load is shown soon.
*    <R_OSPL_WAIT_POLLING> must be set.
************************************************************************/
#if  R_OSPL_THREAD_CODE
#if  ! R_OSPL_IS_PREEMPTION
void  R_OSPL_IDLE_Start_CPU_Load( uint32_t const  Interval_msec );
#endif
#endif


/***********************************************************************
* Function: R_OSPL_IDLE_Print_CPU_Load
*    Show the CPU load by printf (for OS less)
*
* Arguments:
*    IsPrintNow - Whether shows immediately unless the timing of periodic to show
*
* Return Value:
*    None
*
* Description:
*    This function is enabled, if OSPL was compiled with setting <R_OSPL_CPU_LOAD> to 1.
*    This function shows CPU load from calling <R_OSPL_IDLE_Start_CPU_Load> function
*    or the time of "Interval_msec" argument of its function until calling
*    <R_OSPL_IDLE_Print_CPU_Load> function. However, it is necessary to call
*    <R_OSPL_IDLE_Start_CPU_Load> function once at least.
*
*    If "IsPrintNow = false", this function shows, when the time of "Interval_msec"
*    argument was passed only.
*    This function shows CPU load from previous showing to now, if "Interval_msec"
*    argument of <R_OSPL_IDLE_Start_CPU_Load> function was 0.
*
*    Developer of application can customize this function.
************************************************************************/
#if  R_OSPL_THREAD_CODE
#if  ! R_OSPL_IS_PREEMPTION
void  R_OSPL_IDLE_Print_CPU_Load( bool_t const  IsPrintNow );
#endif
#endif


/***********************************************************************
* End of File:
************************************************************************/
#ifdef __cplusplus
}  /* extern "C" */
#endif /* __cplusplus */

#endif /* OSPL_OS_LESS_H */

