/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2016 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/***********************************************************************
* File: ospl_os_less_typedef.h
*    OS Porting Layer API for OS less Compatibility. Data types.
*
* - $Version: $ (=R_OSPL_VERSION)
* - $ModuleRevision: {Masters}\OSPL\ALL_Driver_parent\27_RStyled $
* - $Rev: $
* - $Date: 2017-08-29T10:34:43+09:00 $
************************************************************************/

#ifndef OSPL_OS_LESS_TYPEDEF_H
#define OSPL_OS_LESS_TYPEDEF_H

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include  "r_ospl_config.h"
#include  "r_typedefs.h"
#include  "r_ospl_typedef_1.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/******************************************************************************
Macro definitions
******************************************************************************/

/***********************************************************************
* Macro: R_OSPL_THREAD_DEF
*    Defines the work area and initial value of the thread (for OS less)
*
* Arguments:
*    Name - Thread name. Do not bracket by ""
*
* Description:
*    There is only when "R_OSPL_IS_PREEMPTION" = 0.
*
*    It is not possible to use this macro in the library.
*    If implement of OSPL was changed, the library must be recompiled.
************************************************************************/
#if ! R_OSPL_IS_PREEMPTION  &&  ! R_OSPL_LIBRARY_MAKING
/* ->QAC 3410 : Misplaced warning */ /* ->QAC 3413 */ /* ->SEC M1.8.2 */
#define  R_OSPL_THREAD_DEF( Name )  r_ospl_thread_def_t  Name##_OSPL_DEF
/* <-QAC 3410 */ /* <-QAC 3413 */ /* <-SEC M1.8.2 */
#endif  /* ! R_OSPL_IS_PREEMPTION */


/***********************************************************************
* Macro: R_OSPL_THREAD
*    Returns the work area of the thread (for OS less)
*
* Arguments:
*    Name - Thread name defined by "R_OSPL_THREAD_DEF". Do not bracket by ""
*
* Description:
*    There is only when "R_OSPL_IS_PREEMPTION" = 0.
*
*    It is not possible to use this macro in the library.
*    If implement of OSPL was changed, the library must be recompiled.
************************************************************************/
#if ! R_OSPL_IS_PREEMPTION  &&  ! R_OSPL_LIBRARY_MAKING
/* ->MISRA 19.7 : Can not replace to funcion */ /* ->SEC M5.1.3 */
#define  R_OSPL_THREAD( Name )  (&(Name##_OSPL_DEF))
/* <-MISRA 19.7 */ /* <-SEC M5.1.3 */
#endif  /* ! R_OSPL_IS_PREEMPTION */


/***********************************************************************
* Macro: R_OSPL_EVENT_OBJECT_DEF
*    Defines the work area and initial value of the event object (for OS less)
*
* Arguments:
*    Name - Event object name. Do not bracket by ""
*
* Description:
*    There is only when "R_OSPL_IS_PREEMPTION" = 0.
*
*    It is not possible to use this macro in the library.
*    If implement of OSPL was changed, the library must be recompiled.
************************************************************************/
#if ! R_OSPL_IS_PREEMPTION  &&  ! R_OSPL_LIBRARY_MAKING
/* ->QAC 3410 : Misplaced warning */ /* ->QAC 3413 */ /* ->SEC M1.8.2 */
#define  R_OSPL_EVENT_OBJECT_DEF( Name )  r_ospl_event_object_def_t  Name
/* <-QAC 3410 */ /* <-QAC 3413 */ /* <-SEC M1.8.2 */
#endif  /* ! R_OSPL_IS_PREEMPTION */


/***********************************************************************
* Macro: R_OSPL_EVENT_OBJECT
*    Returns the work area of the event object (for OS less)
*
* Arguments:
*    Name - Event object name defined by "R_OSPL_EVENT_OBJECT_DEF". Do not bracket by ""
*
* Description:
*    There is only when "R_OSPL_IS_PREEMPTION" = 0.
*
*    It is not possible to use this macro in the library.
*    If implement of OSPL was changed, the library must be recompiled.
************************************************************************/
#if ! R_OSPL_IS_PREEMPTION  &&  ! R_OSPL_LIBRARY_MAKING
/* ->MISRA 19.7 : Can not replace to funcion */ /* ->SEC M5.1.3 */
#define  R_OSPL_EVENT_OBJECT( Name )  (&(Name))
/* <-MISRA 19.7 */ /* <-SEC M5.1.3 */
#endif  /* ! R_OSPL_IS_PREEMPTION */


/***********************************************************************
* Enumeration: r_ospl_idle_event_t
*    Events related to idle state (for OS less)
*
*    : R_OSPL_IDLING          - 0
*    : R_OSPL_IDLE_START      - 1
*    : R_OSPL_IDLE_END        - 2
*    : R_OSPL_INTERRUPT_START - 3
*    : R_OSPL_INTERRUPT_END   - 4
*    : R_OSPL_BUSY            - 5
************************************************************************/
#if ! R_OSPL_IS_PREEMPTION
typedef enum {
    R_OSPL_IDLING          = 0,
    R_OSPL_IDLE_START      = 1,
    R_OSPL_IDLE_END        = 2,
    R_OSPL_INTERRUPT_START = 3,
    R_OSPL_INTERRUPT_END   = 4,
    R_OSPL_BUSY            = 5
}
                         r_ospl_idle_event_t;
#endif  /* ! R_OSPL_IS_PREEMPTION */


/***********************************************************************
* Enumeration: r_ospl_thread_state_t
*    r_ospl_thread_state_t
*
*    : R_OSPL_IS_THREAD_USED       - 0x0001
*    : R_OSPL_IS_EXIT_IF_WAIT      - 0x0010
*    : R_OSPL_IS_WAITING           - 0x0020
*    : R_OSPL_IS_WAITING_BY_0      - 0x0040
*    : R_OSPL_IS_OVER_MAX_TIME_OUT - 0x0080
*    : R_OSPL_IS_CASCADE_TIME_OUT  - 0x0100
*    : R_OSPL_IS_TIME_OUT_INFINITE - 0x0200
************************************************************************/
#if ! R_OSPL_IS_PREEMPTION
typedef enum
{
    R_OSPL_IS_THREAD_USED       = 0x0001,
    R_OSPL_IS_EXIT_IF_WAIT      = 0x0010,
    R_OSPL_IS_WAITING           = 0x0020,
    R_OSPL_IS_WAITING_BY_0      = 0x0040,
    R_OSPL_IS_OVER_MAX_TIME_OUT = 0x0080,
    R_OSPL_IS_CASCADE_TIME_OUT  = 0x0100,
    R_OSPL_IS_TIME_OUT_INFINITE = 0x0200
} r_ospl_thread_state_t;
#endif  /* ! R_OSPL_IS_PREEMPTION */


/***********************************************************************
* Enumeration: r_ospl_wait_t
*    Waitnig mode of pseudo multi thread
*
*    : R_OSPL_WAIT_POLLING   - 0
*    : R_OSPL_WAIT_PM_THREAD - 1
************************************************************************/
#if  R_OSPL_THREAD_CODE
typedef enum
{
    R_OSPL_WAIT_POLLING   = 0,
    R_OSPL_WAIT_PM_THREAD = 1
} r_ospl_wait_t;
#endif


/******************************************************************************
Typedef definitions
******************************************************************************/

/***********************************************************************
* Type: r_ospl_thread_status_t
*    Bit flags of <r_ospl_thread_state_t>
************************************************************************/
#if ! R_OSPL_IS_PREEMPTION
typedef bit_flags_fast32_t  r_ospl_thread_status_t;
#endif  /* ! R_OSPL_IS_PREEMPTION */


/***********************************************************************
* Structure: r_ospl_thread_def_t
*    Thread.
*
* Description:
*    Member variables should not be accessed.
************************************************************************/
typedef struct st_r_ospl_thread_def_t  r_ospl_thread_def_t;
#if ! R_OSPL_IS_PREEMPTION  &&  ! R_OSPL_LIBRARY_MAKING
#ifdef __ICCARM__
#pragma diag_suppress=Pe301  /* Warning of typedef name has already been declared */
#endif
#ifdef __ICCARM__
#pragma diag_default=Pe301
#endif
struct st_r_ospl_thread_def_t
{
    volatile uint32_t       Flags;  /* For "R_OSPL_EVENT_*" */
    void*                   Argument;
    r_ospl_thread_status_t  Status;  /* Initial Value is 0 */
    uint32_t                Timeout_msec;
    uint32_t                TimeoutCount;
    void*                   Delegate;
};
#endif  /* ! R_OSPL_IS_PREEMPTION */


/***********************************************************************
* Structure: r_ospl_event_object_def_t
*    Event Object.
*
* Description:
*    Member variables should not be accessed.
************************************************************************/
typedef struct st_r_ospl_event_object_def_t   r_ospl_event_object_def_t;
#if ! R_OSPL_IS_PREEMPTION  &&  ! R_OSPL_LIBRARY_MAKING
struct st_r_ospl_event_object_def_t
{
    volatile bool_t  IsSignaled;
};
#endif  /* ! R_OSPL_IS_PREEMPTION */


/***********************************************************************
* Function: r_ospl_idle_callback_t
*    Callback function on idle
*
* Arguments:
*    IdleEvent      - Input
*
* Return Value:
*    Error code.  If there is no error, the return value is 0.
************************************************************************/
#if ! R_OSPL_IS_PREEMPTION
typedef errnum_t  (* r_ospl_idle_callback_t )( r_ospl_idle_event_t IdleEvent );
#endif  /* ! R_OSPL_IS_PREEMPTION */


/***********************************************************************
* Constant: R_OSPL_PERCENT_ERROR
*    Error value for percent
************************************************************************/
#if ! R_OSPL_IS_PREEMPTION
enum { R_OSPL_PERCENT_ERROR = -100 };
#endif  /* ! R_OSPL_IS_PREEMPTION */


/******************************************************************************
Variable Externs
******************************************************************************/

/******************************************************************************
Functions Prototypes
******************************************************************************/
/* In "r_ospl_os_less.h" */


/***********************************************************************
* End of File:
************************************************************************/
#ifdef __cplusplus
}  /* extern "C" */
#endif /* __cplusplus */

#endif /* OSPL_OS_LESS_TYPEDEF_H */
