/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2016 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/***********************************************************************
* File: ospl_idle.c
*    OS Porting Layer API
*
* - $Version: $ (=R_OSPL_VERSION)
* - $ModuleRevision: {Masters}\OSPL\ALL_Driver_parent\27_RStyled $
* - $Rev: $
* - $Date: 2017-08-29T10:34:43+09:00 $
************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include  "r_typedefs.h"
#include  "r_ospl_os_less.h"
#if R_OSPL_CPU_LOAD
#include  <stdio.h>
#endif

#ifdef _SH
#pragma section     _OSPL
#endif

#if  ! R_OSPL_IS_PREEMPTION


/******************************************************************************
Typedef definitions
******************************************************************************/

/******************************************************************************
Macro definitions
******************************************************************************/

/***********************************************************************
* Macro: GS_CPU_LOAD_DEBUG
*    0 or 1
************************************************************************/
#define  GS_CPU_LOAD_DEBUG  0


/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/

/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/

/******************************************************************************
Private global variables and functions
******************************************************************************/
#if R_OSPL_CPU_LOAD

static r_ospl_ftimer_spec_t  gs_FTimerSpec;

static volatile uint32_t  gs_IdleTime = 0;
static volatile bool_t    gs_IsIdleTimeOverFlow = false;
static volatile uint32_t  gs_IdleStartClock = 0;

static volatile uint32_t  gs_IntervalClocks = 0;
static volatile uint32_t  gs_IntervalTargetClock = 0;
static volatile uint32_t  gs_BeforeIdleTime = 0;

static volatile uint32_t  gs_IdleStartCount = 0;
static volatile uint32_t  gs_IdleEndCount = 0;
static volatile bool_t    gs_IsIdleStartCountOverFlow = false;
static volatile uint32_t  gs_InterruptStartCount = 0;
static volatile uint32_t  gs_InterruptEndCount = 0;

#endif


/***********************************************************************
* Implement: R_OSPL_OnIdleDefault
************************************************************************/
errnum_t  R_OSPL_OnIdleDefault( r_ospl_idle_event_t const  in_IdleEvent )
{
#if R_OSPL_CPU_LOAD
    errnum_t  e = 0;
    uint32_t  old;
    r_ospl_idle_event_t  idle_event_2;


    /* Set "idle_event_2" */
    if ( in_IdleEvent == R_OSPL_IDLE_START  ||  in_IdleEvent == R_OSPL_IDLE_END )
    {
        idle_event_2 = in_IdleEvent;
    }
    else if ( in_IdleEvent == R_OSPL_INTERRUPT_START )
    {
        int32_t  v;

#if  GS_CPU_LOAD_DEBUG
        R_D_AddToIntLog( 0x4051 );
        R_D_AddToIntLog( (int_fast32_t) R_OSPL_FTIMER_Get() );
#endif

        v = (int32_t) gs_IdleStartCount;  /* "v" avoids warning of volatile order */
        if ( v - (int32_t) gs_IdleEndCount > 0 )
        {
            idle_event_2 = R_OSPL_IDLE_END;
            gs_InterruptStartCount += 1;
        }
        else
        {
            idle_event_2 = R_OSPL_BUSY;
        }
    }
    else if ( in_IdleEvent == R_OSPL_INTERRUPT_END )
    {
        int32_t  v;

#if  GS_CPU_LOAD_DEBUG
        R_D_AddToIntLog( 0x4052 );
        R_D_AddToIntLog( (int_fast32_t) R_OSPL_FTIMER_Get() );
#endif

        v = (int32_t) gs_InterruptStartCount;  /* "v" avoids warning of volatile order */
        if ( v - (int32_t) gs_InterruptEndCount > 0 )
        {
            idle_event_2 = R_OSPL_IDLE_START;
            gs_InterruptEndCount += 1;
        }
        else
        {
            idle_event_2 = R_OSPL_BUSY;
        }
    }
    else
    {
        idle_event_2 = R_OSPL_IDLING;
    }


    /* Set current clock */
    if ( idle_event_2 == R_OSPL_IDLE_START )
    {
        uint32_t  v;

#if  GS_CPU_LOAD_DEBUG
        R_D_AddToIntLog( 0x4041 );
        R_D_AddToIntLog( (int_fast32_t) R_OSPL_FTIMER_Get() );
#endif

        old = gs_IdleStartCount;
        gs_IdleStartCount += 1;
        if ( gs_IdleStartCount < old )
        {
            gs_IsIdleStartCountOverFlow = true;
        }
        /* else */
        R_UNREFERENCED_VARIABLE( gs_IsIdleStartCountOverFlow );  /* Avoid warning of "set but never used" */

        v = gs_IdleStartCount;  /* "v" avoids warning of volatile order */
        if ( v - 1 == gs_IdleEndCount )
        {
            R_OSPL_FTIMER_InitializeIfNot( &gs_FTimerSpec );
            gs_IdleStartClock = R_OSPL_FTIMER_Get();
        }
    }
    else if ( idle_event_2 == R_OSPL_IDLE_END )
    {
        uint32_t  v;

#if  GS_CPU_LOAD_DEBUG
        R_D_AddToIntLog( 0x4042 );
        R_D_AddToIntLog( (int_fast32_t) R_OSPL_FTIMER_Get() );
#endif

        gs_IdleEndCount += 1;
        v = gs_IdleStartCount;  /* "v" avoids warning of volatile order */
        if ( v == gs_IdleEndCount )
        {
            old = gs_IdleTime;
            v = gs_IdleStartClock;  /* "v" avoids warning of volatile order */
            gs_IdleTime += R_OSPL_FTIMER_Get() - v;
            if ( gs_IdleTime < old )
            {
                gs_IsIdleTimeOverFlow = true;
            }
            /* else */
            R_UNREFERENCED_VARIABLE( gs_IsIdleTimeOverFlow );  /* Avoid warning of "set but never used" */
        }
        R_OSPL_IDLE_Print_CPU_Load( false );
    }
    else   /* idle_event_2 == R_OSPL_IDLING  || idle_event_2 == R_OSPL_BUSY */
    {
        R_OSPL_IDLE_Print_CPU_Load( false );
    }

#else
    errnum_t const  e = 0;
    R_UNREFERENCED_VARIABLE( in_IdleEvent );
#endif
    return  (errnum_t)  e;  /* Cast is for avoiding warning "Conversion in return". */
}


/***********************************************************************
* Implement: R_OSPL_IDLE_Start_CPU_Load
************************************************************************/
void  R_OSPL_IDLE_Start_CPU_Load( uint32_t const  in_Interval_msec )
{
#if R_OSPL_CPU_LOAD
    uint32_t  now;

    R_OSPL_FTIMER_InitializeIfNot( &gs_FTimerSpec );
    now = R_OSPL_FTIMER_Get();

    gs_IntervalClocks = in_Interval_msec *
                        gs_FTimerSpec.msec_Denominator / gs_FTimerSpec.msec_Numerator;
    gs_IntervalTargetClock = now + gs_IntervalClocks;
    gs_BeforeIdleTime = gs_IdleTime;
#else
    R_UNREFERENCED_VARIABLE( in_Interval_msec );
    R_DEBUG_BREAK();  /* Set R_OSPL_CPU_LOAD = 1 */
#endif
}


/***********************************************************************
* Function: R_OSPL_IDLE_CalculatePercent_Sub
*    Sub routine
*
* Arguments:
*    ElapsedClocks        - ElapsedClocks
*    IdleClocks           - IdleClocks
*    out_Percent          - out_Percent
*    out_PercentDecimal_1 - out_PercentDecimal_1
*
* Return Value:
*    None
************************************************************************/
#if R_OSPL_CPU_LOAD
static void  R_OSPL_IDLE_CalculatePercent_Sub( uint32_t const  in_ElapsedClocks,  uint32_t const  in_IdleClocks,
        int_fast32_t* const  out_Percent,  int_fast32_t* const  out_PercentDecimal_1 );  /* QAC-3450 */
static void  R_OSPL_IDLE_CalculatePercent_Sub( uint32_t const  in_ElapsedClocks,  uint32_t const  in_IdleClocks,
        int_fast32_t* const  out_Percent,  int_fast32_t* const  out_PercentDecimal_1 )
{
    if ( in_IdleClocks > in_ElapsedClocks )
    {
        *out_Percent = R_OSPL_PERCENT_ERROR;
        *out_PercentDecimal_1 = 0;
    }
    else if ( gs_IdleTime == 0 )
    {
        *out_Percent = 0;
        *out_PercentDecimal_1 = 0;
    }
    else
    {
        enum {        shift_1000 = 1000 };  /* Shift level */
        static const  uint32_t  divided_max_uint = 0xFFFFFFFFu / shift_1000;
        int_fast32_t  load;
        int_fast32_t  load_x10;

        if ( in_ElapsedClocks < divided_max_uint )
        {
            load_x10 = shift_1000 - ( in_IdleClocks * shift_1000 ) / in_ElapsedClocks;
        }
        else
        {
            load_x10 = shift_1000 - ( in_IdleClocks / ( in_ElapsedClocks / shift_1000 ) );
        }

#if  GS_CPU_LOAD_DEBUG
        R_D_AddToIntLog( 0x4090 );
        R_D_AddToIntLog( (int_fast32_t) in_ElapsedClocks );
        R_D_AddToIntLog( (int_fast32_t) in_IdleClocks );
#endif

        load = load_x10 / 10;
        *out_Percent = load;
        *out_PercentDecimal_1 = load_x10 - ( load * 10 );
    }
}
#endif


/***********************************************************************
* Implement: R_OSPL_IDLE_Print_CPU_Load
************************************************************************/
void  R_OSPL_IDLE_Print_CPU_Load( bool_t const  in_IsPrintNow )
{
#if R_OSPL_CPU_LOAD
    int_fast32_t  load = 0;  /* 0 is avoid for warning of "Used before set symbol" */
    int_fast32_t  load_decimal_1 = 0;  /* 0 is avoid for warning of "Used before set symbol" */
    uint32_t      is_print;
    uint32_t      now = R_OSPL_FTIMER_Get();

    if ( in_IsPrintNow )
    {
        uint32_t  v = gs_IntervalTargetClock;  /* "v" avoids warning of volatile order */
        uint32_t  elapsed = now - ( v - gs_IntervalClocks );
        uint32_t  idle_clocks;

        v = gs_IdleTime;  /* "v" avoids warning of volatile order */
        idle_clocks = v - gs_BeforeIdleTime;

        R_OSPL_IDLE_CalculatePercent_Sub( elapsed, idle_clocks, &load, &load_decimal_1 );
        is_print = true;

        if ( gs_IntervalClocks == 0 )
        {
            gs_BeforeIdleTime = gs_IdleTime;
            gs_IntervalTargetClock = now;
        }
    }
    else if ( gs_IntervalClocks != 0 )
    {
        errnum_t  e;
        bool_t    is_past;
        r_ospl_ftimer_spec_t  ts;

        is_print = false;
        R_OSPL_FTIMER_GetSpecification( &ts );
        e= R_OSPL_FTIMER_IsPast( &ts, now, gs_IntervalTargetClock, &is_past );
        if ( e == 0  &&  is_past )
        {
            if ( gs_IntervalClocks != 0 )
            {
                uint32_t  v = gs_IdleTime;  /* "v" avoids warning of volatile order */
                uint32_t  idle_clocks = v - gs_BeforeIdleTime;

                R_OSPL_IDLE_CalculatePercent_Sub( gs_IntervalClocks, idle_clocks, &load, &load_decimal_1 );
                is_print = true;

                gs_BeforeIdleTime = gs_IdleTime;
                v = gs_IntervalClocks;  /* "v" avoids warning of volatile order */
                gs_IntervalTargetClock += v;
            }
        }
    }
    else
    {
        is_print = false;
    }

    if ( is_print )
    {
        if ( load == R_OSPL_PERCENT_ERROR )
        {
            printf( "CPU Load: (Overflow)\n" );
        }
        else
        {
            printf( "CPU Load: %d.%d%%\n", load, load_decimal_1 );
        }


        /* Warn by "R_OSPL_THREAD_GetOnWait". */
#ifndef  R_OSPL_NDEBUG
        {
            static bool_t  was_warned = false;

            if ( ! was_warned )
            {
                if ( R_OSPL_THREAD_GetOnWait() != R_OSPL_WAIT_POLLING )
                {
                    printf( "CPU load is not correct, "
                            "if R_OSPL_WAIT_POLLING was not set.\n\n" );
                    was_warned = true;
                }
            }
        }
#endif
    }
#else
    R_UNREFERENCED_VARIABLE( in_IsPrintNow );
#endif
}


#endif  /* R_OSPL_IS_PREEMPTION */
