/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2015 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/***********************************************************************
* File: r_dmac_lm_typedef.h
*    Direct Memory Access Controller (DMAC) - Link Mode Sample Driver.
*
* - $Version: $ (=PFV_VERSION)
* - $ModuleRevision: {Masters}\PFV_DMAC\PFV_DMAC\23_RStyled $
* - $Rev: $
* - $Date: 2017-08-24T18:32:45+09:00 $
************************************************************************/

#ifndef R_DMAC_LM_TYPEDEF_H
#define R_DMAC_LM_TYPEDEF_H

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include  "r_dmac_rm.h"


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/******************************************************************************
Macro definitions
******************************************************************************/

/*[dmac_lm_header_t]*/
#define /*<uint_fast32_t>*/  F_DMAC_LM_DESCRIPTOR_INTERRUPT_MASK  0x00000008u
#define /*<uint_fast32_t>*/  F_DMAC_LM_WRITE_BACK_DISABLE         0x00000004u
#define /*<uint_fast32_t>*/  F_DMAC_LM_LINK_END                   0x00000002u
#define /*<uint_fast32_t>*/  F_DMAC_LM_LINK_VALID                 0x00000001u


/******************************************************************************
Typedef definitions
******************************************************************************/

/***********************************************************************
* Structure: dmac_lm_descriptor_t
*    lm = Link Mode. This must be at 4byte (32bit) aligned address.
*
* Description:
*    This structure is read by DMAC.
************************************************************************/
typedef struct st_dmac_lm_descriptor_t  dmac_lm_descriptor_t;
struct st_dmac_lm_descriptor_t
{
    bit_flags32_t /*<dmac_lm_header_t>*/  header;
    uintptr_t  source_physical_address;
    uintptr_t  destination_physical_address;
    uint32_t   transfer_byte;
    uint32_t   CHCFG_n;
    uint32_t   CHITVL_n;
    uint32_t   CHEXT_n;
    uintptr_t  next_descriptor_link_address;
};


/***********************************************************************
* Structure: dmac_lm_descriptor_config_t
************************************************************************/
typedef struct st_dmac_lm_descriptor_config_t  dmac_lm_descriptor_config_t;
struct st_dmac_lm_descriptor_config_t
{
    bit_flags_fast32_t  flags;   /* F_DMAC_AXI_CACHE_ATTRIBUTE | ... */
    r_ospl_axi_cache_attribute_t  AXI_cache_attribute;  /* CHEXT_n register - SCA, DCA bitfield */
    r_ospl_axi_protection_t       AXI_protection;       /* CHEXT_n register - SPR, DPR bitfield */
};

/*[dmac_lm_descriptor_config_t::flags]*/
#define  F_DMAC_AXI_CACHE_ATTRIBUTE                 0x00000001u
/* Default is R_OSPL_AXI_CACHE_ZERO */
#define  F_DMAC_AXI_PROTECTION                      0x00000002u
/* Default is R_OSPL_AXI_SECURE */


/******************************************************************************
Variable Externs
******************************************************************************/

/******************************************************************************
Functions Prototypes
******************************************************************************/
/* In "r_dmac_lm.h" */


#ifdef __cplusplus
}  /* End of extern "C" */
#endif /* __cplusplus */

#endif
