/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2015 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/***********************************************************************
* File: r_dmac_rm_typedef.h
*    Direct Memory Access Controller (DMAC) - Register Mode Sample Driver.
*
* - $Version: $ (=PFV_VERSION)
* - $ModuleRevision: {Masters}\PFV_DMAC\PFV_DMAC\23_RStyled $
* - $Rev: $
* - $Date: 2017-08-24T18:32:46+09:00 $
************************************************************************/

#ifndef R_DMAC_RM_TYPEDEF_H
#define R_DMAC_RM_TYPEDEF_H

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include  "r_ospl_typedef.h"
#include  "clib_drivers_typedef.h"


/******************************************************************************
Macro definitions
******************************************************************************/

/* DMAC channel */
enum { DMAC_CHANNEL_MIN =  0 };  /*[DMAC_CHANNEL_MIN]*/
enum { DMAC_CHANNEL_MAX = 15 };  /*[DMAC_CHANNEL_MAX]*/
enum { R_DMAC_RM_CHANNEL_COUNT = 16 };  /*[R_DMAC_RM_CHANNEL_COUNT]*/


/******************************************************************************
Typedef definitions
******************************************************************************/

/***********************************************************************
* Structure: dmac_rm_async_status_t
************************************************************************/
typedef r_ospl_async_status_t  dmac_rm_async_status_t;


/***********************************************************************
* Structure: dmac_pulse_or_level_t
************************************************************************/
typedef enum
{
    DMAC_PULSE = 0,
    DMAC_LEVEL = 1
} dmac_pulse_or_level_t;


/***********************************************************************
* Structure: dmac_priority_mode_t
************************************************************************/
typedef enum
{
    DMAC_PRIORITY_FIXED      = 0,
    DMAC_PRIORITY_ROUNDROBIN = 1
} dmac_priority_mode_t;


/***********************************************************************
* Structure: dmac_shared_config_t
************************************************************************/
typedef struct st_dmac_shared_config_t  dmac_shared_config_t;
struct st_dmac_shared_config_t
{
    bit_flags_fast32_t  flags;   /* F_LINK_WRITE_BACK_CACHE | ... */

    r_ospl_axi_cache_attribute_t  link_write_back_cache;       /* DCTRL_nn register - LWCA bitfield */
    r_ospl_axi_protection_t       link_write_back_protection;  /* DCTRL_nn register - LWPR bitfield */
    r_ospl_axi_cache_attribute_t  link_descriptor_cache;       /* DCTRL_nn register - LDCA bitfield */
    r_ospl_axi_protection_t       link_descriptor_protection;  /* DCTRL_nn register - LDPR bitfield */
    dmac_pulse_or_level_t  output_interrupt_pulse_or_level;  /* DCTRL_nn register - LVINT bitfield */
    dmac_priority_mode_t   priority_mode;                    /* DCTRL_nn register - PR bitfield */
};
enum  /*[dmac_shared_config_t::flags]*/
{
    F_DMAC_LINK_WRITE_BACK_CACHE           = 0x00000001,  /* Default is not change value */
    F_DMAC_LINK_WRITE_BACK_PROTECTION      = 0x00000010,  /* Default is not change value */
    F_DMAC_LINK_DESCRIPTOR_CACHE           = 0x00000002,  /* Default is not change value */
    F_DMAC_LINK_DESCRIPTOR_PROTECTION      = 0x00000020,  /* Default is not change value */
#define	F_DMAC_OUTPUT_INTERRUPT_PULSE_OR_LEVEL   0x00000004   /* Default is not change value */
    F_DMAC_PRIORITY_MODE                   = 0x00000008   /* Default is not change value */
};


/***********************************************************************
* Structure: dmac_bit_count_t
************************************************************************/
typedef enum
{
    DMAC_8_BIT    = 0x0,
    DMAC_16_BIT   = 0x1,
    DMAC_32_BIT   = 0x2,
    DMAC_64_BIT   = 0x3,
    DMAC_128_BIT  = 0x4,
    DMAC_256_BIT  = 0x5,
    DMAC_512_BIT  = 0x6,
    DMAC_1024_BIT = 0x7
} dmac_bit_count_t;
#define  DMAC_BIT_COUNT_T_DEFINED


/***********************************************************************
* Structure: dmac_transfer_mode_t
************************************************************************/
typedef enum
{
    DMAC_SINGLE_TRANSFER_0 = 0,
    DMAC_BLOCK_TRANSFER_1  = 1
} dmac_transfer_mode_t;


/***********************************************************************
* Structure: dmac_ack_mode_t
************************************************************************/
typedef enum
{
    DMAC_ACK_MODE_RESET_VALUE   = 0,
    DMAC_ACK_LEVEL_MODE_001     = 1,
    DMAC_ACK_BUS_CYCLE_MODE_010 = 2,
    DMAC_ACK_NOT_OUTPUT_100     = 4
} dmac_ack_mode_t;


/***********************************************************************
* Structure: dmac_edge_or_level_t
************************************************************************/
typedef enum
{
    DMAC_EDGE_0  = 0,
    DMAC_LEVEL_1 = 1
} dmac_edge_or_level_t;


/***********************************************************************
* Structure: dmac_request_direction_t
************************************************************************/
typedef enum
{
    DMAC_DMAREQ_SOURCE_0 = 0,
    DMAC_DMAREQ_DESTINATION_1 = 1
} dmac_request_direction_t;


/***********************************************************************
* Constants: dmac_interrupt_line_t
************************************************************************/
#define /*<uint_fast32_t>*/  DMAC_INTERRUPT_LINE_TRANSFER_END  0x00000001u
#define /*<uint_fast32_t>*/  DMAC_INTERRUPT_LINE_ERROR         0x00000002u


/***********************************************************************
* Structure: dmac_interrupt_lines_t
************************************************************************/
typedef bit_flags_fast32_t /*<dmac_interrupt_line_t>*/  dmac_interrupt_lines_t;


/***********************************************************************
* Constant: DMAC_INTERRUPT_LINE_ALL
************************************************************************/
#define /*<uint_fast32_t>*/  DMAC_INTERRUPT_LINE_ALL  \
	( DMAC_INTERRUPT_LINE_TRANSFER_END | DMAC_INTERRUPT_LINE_ERROR )


/***********************************************************************
* Structure: dmac_register_mode_config_t
************************************************************************/
typedef struct st_dmac_register_mode_config_t  dmac_register_mode_config_t;
struct st_dmac_register_mode_config_t
{
    bit_flags32_t  flags;   /* F_DMAC_NEXT0_SOURCE_PHYSICAL_ADDRESS | ... */

    uintptr_t  next0_source_physical_address;       /* N0SA_n register */
    uintptr_t  next0_destination_physical_address;  /* N0DA_n register */
    size_t     next0_transfer_byte;                 /* N0TB_n register */

    uintptr_t  next1_source_physical_address;       /* N1SA_n register */
    uintptr_t  next1_destination_physical_address;  /* N1DA_n register */
    size_t     next1_transfer_byte;                 /* N1TB_n register */

    bool8_t  is_source_address_fixed;          /* CHCFG_n register - SAD bit, false:address+=1 */
    dmac_bit_count_t  source_bit_count;        /* CHCFG_n register - SDS bitfield */

    bool8_t  is_destination_address_fixed;     /* CHCFG_n register - DAD bit, false:address+=1 */
    dmac_bit_count_t  destination_bit_count;   /* CHCFG_n register - DDS bitfield */

    bool8_t  is_reload_next_register_set;      /* CHCFG_n register - REN bit */
    bool8_t  is_automatic_switch_next_num;     /* CHCFG_n register - RSW bit */
    int8_t   next_register_set_number;         /* CHCFG_n register - DMS bitfield, 0(Next0) or 1(Next1) */
    bool8_t  is_sweep_buffer;                  /* CHCFG_n register - SBE bit */
    bool8_t  is_disable_next_end_interrupt;    /* CHCFG_n register - DEM bit */

    dmac_transfer_mode_t  transfer_mode_TM;    /* CHCFG_n register - TM bit */
    uint8_t      is_DMA_ack_output_mode_AM;    /* CHCFG_n register - AM bit */

    int_fast32_t  request_resource_MID;               /* DMARSn register - MID bitfield */
    int_fast32_t  request_resource_RID;               /* DMARSn register - RID bitfield */

    dmac_edge_or_level_t  request_edge_or_level_LVL;   /* CHCFG_n register - LVL bit */
    bool8_t  is_request_raising_or_high_HIEN;          /* CHCFG_n register - HIEN bit */
    bool8_t  is_request_falling_or_low_LOEN;           /* CHCFG_n register - LOEN bit */
    dmac_request_direction_t  request_direction_REQD;  /* CHCFG_n register - REQD bit */

    int_fast32_t  interval_count;              /* CHITVL_n register - ITVL bitfield */

    r_ospl_axi_cache_attribute_t  source_AXI_cache_attribute;       /* CHEXT_n register - SCA bitfield */
    r_ospl_axi_protection_t       source_AXI_protection;            /* CHEXT_n register - SPR bitfield */
    r_ospl_axi_cache_attribute_t  destination_AXI_cache_attribute;  /* CHEXT_n register - DCA bitfield */
    r_ospl_axi_protection_t       destination_AXI_protection;       /* CHEXT_n register - DPR bitfield */
};

/*[dmac_register_mode_config_t::flags]*/
#define  F_DMAC_NEXT0_SOURCE_PHYSICAL_ADDRESS       0x00000001u  /* Must */
#define  F_DMAC_NEXT0_DESTINATION_PHYSICAL_ADDRESS  0x00000002u  /* Must */
#define  F_DMAC_NEXT0_TRANSFER_BYTE                 0x00000004u  /* Must */

#define  F_DMAC_NEXT1_SOURCE_PHYSICAL_ADDRESS       0x00000008u
/* Default is not used */
#define  F_DMAC_NEXT1_DESTINATION_PHYSICAL_ADDRESS  0x00000010u
/* Must if F_DMAC_NEXT1_SOURCE_PHYSICAL_ADDRESS was set */
#define  F_DMAC_NEXT1_TRANSFER_BYTE                 0x00000020u
/* Must if F_DMAC_NEXT1_SOURCE_PHYSICAL_ADDRESS was set */

#define  F_DMAC_IS_SOURCE_ADDRESS_FIXED             0x00000040u
/* Default is false */
#define  F_DMAC_SOURCE_BIT_COUNT                    0x00000080u
/* Default is DMAC_8_BIT */

#define  F_DMAC_IS_DESTINATION_ADDRESS_FIXED        0x00000100u
/* Default is false */
#define  F_DMAC_DESTINATION_BIT_COUNT               0x00000200u
/* Default is DMAC_8_BIT */

#define  F_DMAC_IS_RELOAD_NEXT_REGISTER_SET         0x00000400u
/* Default is false */
#define  F_DMAC_IS_AUTOMATIC_SWITCH_NEXT_NUM        0x00000800u
/* Default is false */
#define  F_DMAC_NEXT_REGISTER_SET_NUMBER            0x00001000u
/* Default is 0 */
#define  F_DMAC_IS_SWEEP_BUFFER                     0x00004000u
/* Default is false */
#define  F_DMAC_IS_DISABLE_NEXT_END_INTERRUPT       0x00008000u
/* Default is false */

#define  F_DMAC_TRANSFER_MODE_TM                    0x00010000u
/* Default is DMAC_SINGLE_TRANSFER. */
/* Must if F_DMAC_REQUEST_RESOURCE_MID was set */
#define  F_DMAC_IS_DMA_ACK_OUTPUT_MODE_AM           0x00020000u
/* Default is 0. Must if F_DMAC_REQUEST_RESOURCE_MID was set */
/* 4. If F_DMAC_REQUEST_RESOURCE_MID was not set */

#define  F_DMAC_REQUEST_RESOURCE_MID                0x00200000u
/* Default is that any resouece selector is not used */
#define  F_DMAC_REQUEST_RESOURCE_RID                0x00100000u
/* Must if F_DMAC_REQUEST_RESOURCE_MID was set */

#define  F_DMAC_REQUEST_EDGE_OR_LEVEL_LVL           0x00400000u
/* Must if F_DMAC_REQUEST_RESOURCE_MID was set */
#define  F_DMAC_IS_REQUEST_RAISING_OR_HIGH_HIEN     0x00800000u
/* Must if F_DMAC_REQUEST_RESOURCE_MID was set */
#define  F_DMAC_IS_REQUEST_FALLING_OR_LOW_LOEN      0x01000000u
/* Must if F_DMAC_REQUEST_RESOURCE_MID was set */
#define  F_DMAC_REQUEST_DIRECTION_REQD              0x02000000u
/* Must if F_DMAC_REQUEST_RESOURCE_MID was set */

#define  F_DMAC_INTERVAL_COUNT                      0x00040000u
/* Default is 0 */

#define  F_DMAC_SOURCE_AXI_CACHE_ATTRIBUTE                 0x04000000u
/* Default is R_OSPL_AXI_CACHE_ZERO */
#define  F_DMAC_SOURCE_AXI_PROTECTION                      0x08000000u
/* Default is R_OSPL_AXI_SECURE */
#define  F_DMAC_DESTINATION_AXI_CACHE_ATTRIBUTE            0x10000000u
/* Default is R_OSPL_AXI_CACHE_ZERO */
#define  F_DMAC_DESTINATION_AXI_PROTECTION                 0x20000000u
/* Default is R_OSPL_AXI_SECURE */

/***********************************************************************
* Structure: dmac_rm_config_t
************************************************************************/
typedef struct st_dmac_rm_config_t  dmac_rm_config_t;
struct st_dmac_rm_config_t
{
    bit_flags32_t  flags;   /* F_DMAC_CHANNEL_NUM | ... */

    int_fast32_t  channel_num;
};
/*[dmac_rm_config_t::flags]*/
#define  F_DMAC_CHANNEL_NUM       0x00000001u


/******************************************************************************
Variable Externs
******************************************************************************/

/******************************************************************************
Functions Prototypes
******************************************************************************/

#endif /* R_DMAC_RM_TYPEDEF_H */

