/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2015 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/***********************************************************************
* File: r_pfv_pl.c
*    Pixel Format Converter (PFV) Sample Driver. OS Porting Layer.
*
* - $Version: $ (=PFV_VERSION)
* - $ModuleRevision: {Masters}\PFV_DMAC\PFV_DMAC\23_RStyled $
* - $Rev: $
* - $Date: 2017-08-24T18:32:46+09:00 $
************************************************************************/


/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include  "r_ospl.h"
#include  "iodefine.h"
#include  "clib_registers.h"
#include  "r_pfv.h"
#include  "r_pfv_pl.h"
#include  "r_dmac_rm_pl.h"
#include  "iobitmasks/pfv_iobitmask.h"


/******************************************************************************
Typedef definitions
******************************************************************************/

/***********************************************************************
* Structure: r_pfv_pl_channel_t
************************************************************************/
typedef struct st_r_pfv_pl_channel_t  r_pfv_pl_channel_t;
struct st_r_pfv_pl_channel_t
{
    const r_ospl_caller_t*           InterruptCallbackCaller;
    const pfv_async_status_t*        Status;
};


/***********************************************************************
* Variable: gs_pfv_pl_channel
************************************************************************/
static r_pfv_pl_channel_t  gs_pfv_pl_channel[ PFV_CHANNEL_COUNT ];


/******************************************************************************
Macro definitions
******************************************************************************/
enum { GS_MINUS_N = 2048 };  /* if ( matrix_multiply_256 < 0 )  matrix_multiply_256 = register_value - 2048 */
#define  GS_INTERRUPT_LEVEL_OF_OUTPUT_DMAC  19  /* QAC 4800 */
#define  GS_INTERRUPT_LEVEL_OF_INPUT_DMAC   20
#define  GS_INTERRUPT_LEVEL_OF_OUTPUT_FULL  23
#define  GS_INTERRUPT_LEVEL_OF_INPUT_EMPTY  24
enum {   GS_INTERRUPT_LEVEL_OF_ERROR      = 25 };

/* "STBCR13" register for "R_PFV_SET_REGISTER_BIT_FIELD()" */
enum { STBCR13_MSTP131_PFV0 = 0x02 };  /* "CMSTP11" bitfield */
enum { STBCR13_MSTP132_PFV1 = 0x04 };

R_STATIC_ASSERT_GLOBAL( PFV_CHANNEL_COUNT == PFV_COUNT, "" );


/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/

/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/

/******************************************************************************
Private global variables and functions
******************************************************************************/
static void  R_PFV_Error_IRQ_HandlerN( int_fast32_t const  in_PFV_Channel );
static void  R_PFV_Error_IRQ_Handler0( R_OSPL_INTERRUPT_ARGUMENTS );
static void  R_PFV_Error_IRQ_Handler1( R_OSPL_INTERRUPT_ARGUMENTS );


/***********************************************************************
* Variable: gs_interrupt_context_pfv_error
*    gs_InterruptContext_PFV_Error
************************************************************************/
static const r_ospl_interrupt_t  gs_interrupt_context_pfv_error[ PFV_CHANNEL_COUNT ] =
{
    { BSP_INT_SRC_PFVEI0, 0 },
    { BSP_INT_SRC_PFVEI1, 1 }
};


/***********************************************************************
* Variable: gs_reset_color_matrixes
************************************************************************/
static const pfv_reset_color_matrix_t  gs_reset_color_matrixes[ PFV_MATRIX_MODE_COUNT ] =
{
    /* ->QAC 3120 */
    {   /* GBR => GBR */
        { 128, 128, 128 },
        { 256, 0, 0, 0, 256, 0, 0, 0, 256 }
    },
    {   /* GBR => YCBCR, SMPTE 293M */
        { 128, 128, 128 },
        { 150, 29, 77, 1963-GS_MINUS_N, 128, 2005-GS_MINUS_N, 1941-GS_MINUS_N, 2027-GS_MINUS_N, 128 }
    },
    {   /* YCBCR => GBR, SMPTE 293M */
        { 128, 128, 128 },
        { 256, 1960-GS_MINUS_N, 1865-GS_MINUS_N, 256, 454, 0, 256, 0, 359 }
    },
    {   /* YCBCR => YCBCR */
        { 128, 128, 128 },
        { 256, 0, 0, 0, 256, 0, 0, 0, 256 }
    }
    /* <-QAC 3120 */
};


/***********************************************************************
* Function: R_Userdef_PFV_SetDefaultConfig
*    This function is run in task context.
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_Userdef_PFV_SetDefaultConfig( pfv_config_t* const  in_out_Config )
{
    errnum_t  e;

    IF_DQ( in_out_Config == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }


    /* Set "reset_color_matrixes" */
    if ( IS_BIT_NOT_SET( in_out_Config->flags, F_PFV_RESET_COLOR_MATRIXES ) )
    {
        in_out_Config->reset_color_matrixes = gs_reset_color_matrixes;
        in_out_Config->flags |= F_PFV_RESET_COLOR_MATRIXES;
    }


    e = 0;
fin:
    return  e;
}


/***********************************************************************
* Function: R_Userdef_PFV_DMAC_SetDefaultConfig
*    This function is run in task context.
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_Userdef_PFV_DMAC_SetDefaultConfig( pfv_dmac_config_t* const  in_out_Config )
{
    errnum_t  e;

    IF_DQ( in_out_Config == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }


    /* Set "reset_color_matrixes" */
    if ( IS_BIT_NOT_SET( in_out_Config->flags, F_PFV_DMAC_RESET_COLOR_MATRIXES ) )
    {
        in_out_Config->reset_color_matrixes = gs_reset_color_matrixes;
        in_out_Config->flags |= F_PFV_DMAC_RESET_COLOR_MATRIXES;
    }


    /* Set "get_dma_bit_count_func" */
    if ( IS_BIT_NOT_SET( in_out_Config->flags, F_PFV_DMAC_GET_DMA_BIT_COUNT ) )
    {
        in_out_Config->get_dma_bit_count_func = &( R_Userdef_PFV_GetDMA_BitCount );
        in_out_Config->flags |= F_PFV_DMAC_GET_DMA_BIT_COUNT;
    }


    e = 0;
fin:
    return  e;
}


/***********************************************************************
* Function: R_Userdef_PFV_GetDMA_BitCount
*    This function is run in task context.
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_Userdef_PFV_GetDMA_BitCount(
    uintptr_t const  in_InputBufferStartAddress,
    size_t    const  in_InputBufferSize,
    uintptr_t const  in_OutputBufferStartAddress,
    size_t    const  in_OutputBufferSize,
    pfv_dma_bit_count_t* const  out )
{
    errnum_t  e;

    uintptr_t  input_buffer_start;
    uintptr_t  input_buffer_over;
    uintptr_t  output_buffer_start;
    uintptr_t  output_buffer_over;
    uintptr_t  mask;
    bool_t     or;   /* for MISRA 12.4 */
    bool_t     and;  /* for MISRA 12.4 */

    enum { num_0x03 = 0x03 };
    enum { num_0x7F = 0x7F };
    enum { num_0x1F = 0x1F };


    IF_DQ( out == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }

    if ( in_InputBufferSize != 0u )
    {
        input_buffer_start = in_InputBufferStartAddress;
        input_buffer_over  = in_InputBufferStartAddress +
                             (uintptr_t) in_InputBufferSize;
    }
    else
    {
        input_buffer_start = 0;
        input_buffer_over  = 0;
    }


    if ( in_OutputBufferSize != 0u )
    {
        output_buffer_start = in_OutputBufferStartAddress;
        output_buffer_over  = in_OutputBufferStartAddress +
                              (uintptr_t) in_OutputBufferSize;
    }
    else
    {
        output_buffer_start = 0;
        output_buffer_over  = 0;
    }


#if PFV_PARAMETER_CHECK

    mask = 0x01;  /* 8bit = 1byte alignment, not 2 byte alignment */
    or = IS_ANY_BITS_SET( input_buffer_start, mask );
    if (!or)
    {
        or = IS_ANY_BITS_SET( input_buffer_over, mask );
    }
    if (!or)
    {
        or = IS_ANY_BITS_SET( output_buffer_start, mask );
    }
    if (!or)
    {
        or = IS_ANY_BITS_SET( output_buffer_over, mask );
    }
    IF (IS( or ))
    {
        e = E_OTHERS;
        goto  fin;
    }

#endif


    mask = num_0x03;  /* 2 byte alignment, not 4 byte alignment */
    or = IS_ANY_BITS_SET( input_buffer_start, mask );
    if (!or)
    {
        or = IS_ANY_BITS_SET( input_buffer_over, mask );
    }
    if (!or)
    {
        or = IS_ANY_BITS_SET( output_buffer_start, mask );
    }
    if (!or)
    {
        or = IS_ANY_BITS_SET( output_buffer_over, mask );
    }
    if (IS( or ))
    {
        out->input_buffer_DMA_bit_count  = DMAC_16_BIT;
        out->input_PFV_DMA_bit_count     = DMAC_16_BIT;
        out->input_trigger_byte_count    = PFV_IDTRG_SPACED_2_BYTE;

        out->output_PFV_DMA_bit_count    = DMAC_16_BIT;
        out->output_buffer_DMA_bit_count = DMAC_16_BIT;
        out->output_trigger_byte_count   = PFV_ODTRG_FILLED_2_BYTE;
    }
    else
    {
        mask = num_0x7F;  /* 128 byte alignment */
        and = IS_ALL_BITS_NOT_SET( input_buffer_start, mask );
        if (IS(and))
        {
            and = IS_ALL_BITS_NOT_SET( input_buffer_over, mask );
        }
        if (IS( and ))
        {
            out->input_buffer_DMA_bit_count  = DMAC_1024_BIT;
        }
        else
        {
            out->input_buffer_DMA_bit_count  = DMAC_32_BIT;
        }


        mask = num_0x1F;  /* 32 byte alignment */
        and = IS_ALL_BITS_NOT_SET( input_buffer_start, mask );
        if (IS(and))
        {
            and = IS_ALL_BITS_NOT_SET( input_buffer_over, mask );
        }
        if (IS( and ))
        {
            out->input_PFV_DMA_bit_count  = DMAC_256_BIT;
            out->input_trigger_byte_count = PFV_IDTRG_SPACED_32_BYTE;
        }
        else
        {
            out->input_PFV_DMA_bit_count  = DMAC_32_BIT;
            out->input_trigger_byte_count = PFV_IDTRG_SPACED_4_BYTE;
        }


        mask = num_0x7F;  /* 128 byte alignment */
        and = IS_ALL_BITS_NOT_SET( output_buffer_start, mask );
        if (IS(and))
        {
            and = IS_ALL_BITS_NOT_SET( output_buffer_over, mask );
        }
        if (IS( and ))
        {
            out->output_buffer_DMA_bit_count  = DMAC_1024_BIT;
        }
        else
        {
            out->output_buffer_DMA_bit_count  = DMAC_32_BIT;
        }


        mask = num_0x1F;  /* 32 byte alignment */
        and = IS_ALL_BITS_NOT_SET( output_buffer_start, mask );
        if (IS(and))
        {
            and = IS_ALL_BITS_NOT_SET( output_buffer_over, mask );
        }
        if (IS( and ))
        {
            out->output_PFV_DMA_bit_count  = DMAC_256_BIT;
            out->output_trigger_byte_count = PFV_ODTRG_FILLED_32_BYTE;
        }
        else
        {
            out->output_PFV_DMA_bit_count  = DMAC_32_BIT;
            out->output_trigger_byte_count = PFV_ODTRG_FILLED_4_BYTE;
        }
    }

    e = 0;
#if PFV_PARAMETER_CHECK
fin:
#endif
    return  e;
}


/***********************************************************************
* Function: R_Userdef_PFV_SetDefaultAsync
*
* Return Value:
*    None
************************************************************************/
void  R_Userdef_PFV_SetDefaultAsync( r_ospl_async_t*  ref_Async,  r_ospl_async_type_t  in_AsyncType )
{
    IF_DQ( ref_Async == NULL )
    {
        goto fin;
    }

    R_OSPL_ASYNC_SetDefaultPreset( ref_Async );

    if ( IS_BIT_NOT_SET( ref_Async->Flags, R_F_OSPL_A_Thread ) )
    {
        ref_Async->A_Thread = R_OSPL_THREAD_NULL;
    }

    if ( IS_BIT_NOT_SET( ref_Async->Flags, R_F_OSPL_A_EventValue ) )
    {
        if ( in_AsyncType == R_OSPL_ASYNC_TYPE_NORMAL )
        {
            ref_Async->A_EventValue = R_OSPL_A_FLAG;
        }
        else
        {
            ref_Async->A_EventValue = R_OSPL_FINAL_A_FLAG;
        }
    }
    else
    {
        ASSERT_D( IS_BIT_SET( ref_Async->Flags, R_F_OSPL_A_Thread ), R_NOOP() );
    }

    if ( IS_BIT_NOT_SET( ref_Async->Flags, R_F_OSPL_I_Thread ) )
    {
        ref_Async->I_Thread = R_OSPL_THREAD_NULL;
    }

    if ( IS_BIT_NOT_SET( ref_Async->Flags, R_F_OSPL_I_EventValue ) )
    {
        if ( ref_Async->I_Thread == R_OSPL_THREAD_NULL )
        {
            ref_Async->I_EventValue = 0;
        }
        else
        {
            ref_Async->I_EventValue = R_OSPL_I_FLAG;
        }
    }
    else
    {
        ASSERT_D( IS_BIT_SET( ref_Async->Flags, R_F_OSPL_I_Thread ), R_NOOP() );
    }

    if ( IS_BIT_NOT_SET( ref_Async->Flags, R_F_OSPL_InterruptCallback ) )
    {
        ref_Async->InterruptCallback = &( R_Userdef_PFV_OnInterruptDefault );    /* MISRA 16.9 */
    }

    ref_Async->Flags = R_F_OSPL_A_Thread | R_F_OSPL_A_EventValue |
                       R_F_OSPL_I_Thread | R_F_OSPL_I_EventValue |
                       R_F_OSPL_InterruptCallback | R_F_OSPL_Delegate;
fin:
    return;
}


/***********************************************************************
* Function: R_Userdef_PFV_OnInitialize
*    This function is run in task context.
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_Userdef_PFV_OnInitialize( int_fast32_t const  in_PFV_Channel )
{
    errnum_t        e;
    errnum_t        ee;
    bsp_int_err_t   eb;
    bsp_int_src_t const   num_of_IRQ = gs_interrupt_context_pfv_error[ in_PFV_Channel ].IRQ_Num;
    struct st_cpg* const  reg_CPG = R_Get_CPG_Base();
#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wunused-but-set-variable"
#endif
    volatile uint8_t      dummy_read;
#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif

    /* Variable: gs_Error_IRQHandlers */
    static bsp_int_cb_t  gs_Error_IRQHandlers[ PFV_CHANNEL_COUNT ] =
    {
        &( R_PFV_Error_IRQ_Handler0 ), /* MISRA 16.9 */
        &( R_PFV_Error_IRQ_Handler1 )
    };


    IF_DQ( reg_CPG == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }


    IF ( (in_PFV_Channel < PFV_CHANNEL_MIN) || (in_PFV_Channel > PFV_CHANNEL_MAX) )
    {
        e = E_OTHERS;
        goto  fin;
    }


    /* Supply clock to PFV */
    if ( in_PFV_Channel == 0 )
    {
        R_PFV_SET_REGISTER_BIT_FIELD( &reg_CPG->STBCR13, STBCR13, MSTP131, false );
    }
    else
    {
        R_PFV_SET_REGISTER_BIT_FIELD( &reg_CPG->STBCR13, STBCR13, MSTP132, false );
    }
    dummy_read = reg_CPG->STBCR13;


    /* Enable PFV interrupt */
    {
        eb= R_BSP_InterruptWrite( num_of_IRQ, gs_Error_IRQHandlers[ in_PFV_Channel ] );
        IF(eb!=0)
        {
            e=E_OTHERS;
            goto fin;
        }

        e= R_OSPL_SetInterruptPriority( num_of_IRQ, GS_INTERRUPT_LEVEL_OF_ERROR );
        IF(e!=0)
        {
            goto fin;
        }
    }


    e = 0;
fin:
    if ( e != 0 )
    {
        ee= R_Userdef_PFV_OnFinalize( in_PFV_Channel, e );
        R_UNREFERENCED_VARIABLE( ee );
    }
    return  e;
}


/***********************************************************************
* Function: R_Userdef_PFV_OnFinalize
*    This function is run in task context.
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_Userdef_PFV_OnFinalize( int_fast32_t const  in_PFV_Channel,  errnum_t  e )
{
    struct st_cpg* const  reg_CPG = R_Get_CPG_Base();
#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wunused-but-set-variable"
#endif
    volatile uint8_t      dummy_read;
#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif

    IF_DQ( reg_CPG == NULL )
    {
        if ( e == 0 )
        {
            e = E_OTHERS;
        }
        goto fin;
    }

    IF ( (in_PFV_Channel < PFV_CHANNEL_MIN) || (in_PFV_Channel > PFV_CHANNEL_MAX) )
    {
        if ( e == 0 )
        {
            e = E_OTHERS;
        }
        goto  fin;
    }


    /* Not supply clock to PFV */
    if ( in_PFV_Channel == 0 )
    {
        R_PFV_SET_REGISTER_BIT_FIELD( &reg_CPG->STBCR13, STBCR13, MSTP131, true );
    }
    else
    {
        R_PFV_SET_REGISTER_BIT_FIELD( &reg_CPG->STBCR13, STBCR13, MSTP132, true );
    }
    dummy_read = reg_CPG->STBCR13;


fin:
    return  e;
}


/***********************************************************************
* Function: R_Userdef_PFV_SetInterruptCallbackCaller
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_Userdef_PFV_SetInterruptCallbackCaller( int_fast32_t const  in_PFV_Channel,
        const r_ospl_caller_t* const  in_Caller )
{
    r_pfv_pl_channel_t* const  self = &gs_pfv_pl_channel[ in_PFV_Channel ];

    self->InterruptCallbackCaller = in_Caller;

    return  0;
}


/***********************************************************************
* Function: R_Userdef_PFV_OnEnableInterrupt
*
* Return Value:
*    None
************************************************************************/
void  R_Userdef_PFV_OnEnableInterrupt( int_fast32_t const  in_PFV_Channel,
                                       pfv_interrupt_lines_t const  in_Enables )
{
    bsp_int_err_t  eb;

    if ( IS_BIT_SET( in_Enables, PFV_INTERRUPT_LINE_PFVEIn ) )
    {
        bsp_int_src_t const  num_of_IRQ = gs_interrupt_context_pfv_error[ in_PFV_Channel ].IRQ_Num;

        eb= R_BSP_InterruptControl( num_of_IRQ, BSP_INT_CMD_INTERRUPT_ENABLE, FIT_NO_PTR );
        ASSERT_D( eb == 0,  R_NOOP() );
        R_UNREFERENCED_VARIABLE( eb ); /* for Release configuration */
    }
}


/***********************************************************************
* Function: R_Userdef_PFV_OnDisableInterrupt
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
void  R_Userdef_PFV_OnDisableInterrupt( int_fast32_t const  in_PFV_Channel,
                                        pfv_interrupt_lines_t const  in_Disables1 )
{
    bsp_int_err_t  eb;

    if ( IS_BIT_SET( in_Disables1, PFV_INTERRUPT_LINE_PFVEIn ) )
    {
        bsp_int_src_t const  num_of_IRQ = gs_interrupt_context_pfv_error[ in_PFV_Channel ].IRQ_Num;

        eb= R_BSP_InterruptControl( num_of_IRQ, BSP_INT_CMD_INTERRUPT_DISABLE, FIT_NO_PTR );
        ASSERT_D( eb == 0,  R_NOOP() );
        R_UNREFERENCED_VARIABLE( eb ); /* for Release configuration */
    }
}


/***********************************************************************
* Function: R_Userdef_PFV_OnInterruptDefault
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_Userdef_PFV_OnInterruptDefault( const r_ospl_interrupt_t* const  in_InterruptSource,
        const r_ospl_caller_t* const  in_Caller )
{
    errnum_t  e;
    errnum_t  ee;
    r_ospl_async_t*  async;

    IF_DQ( in_Caller == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }

    async = in_Caller->Async;

    e = 0;
    R_AVOID_UNSAFE_ALWAYS_WARNING( e );

    ee= R_PFV_DMAC_OnInterrupting( in_InterruptSource );
    IF ( (ee != 0) && (e == 0) )
    {
        e = ee;
    }

    if ( async->I_Thread == R_OSPL_THREAD_NULL )
    {
        int_fast32_t  argument;  /* SEC R3.6.2 */

        argument = R_OSPL_CALLER_GetRootChannelNum( in_Caller );
        ee= R_PFV_DMAC_OnInterrupted( argument );
        IF ( (ee != 0) && (e == 0) )
        {
            e = ee;
        }
    }
    else
    {
        R_OSPL_EVENT_Set( async->I_Thread, async->I_EventValue );
    }

fin:
    return  e;
}


/***********************************************************************
* Function: R_Userdef_PFV_DMAC_GetInterruptUnit
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_Userdef_PFV_DMAC_GetInterruptUnit( const r_ospl_interrupt_t* const  in_InterruptSource,
        pfv_dmac_interrupt_unit_t* const  out_Unit )
{
    errnum_t       e;
    bsp_int_src_t  num_of_IRQ;

    IF_DQ( in_InterruptSource == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }
    IF_DQ( out_Unit == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }

    num_of_IRQ = in_InterruptSource->IRQ_Num;

    if ( num_of_IRQ >= BSP_INT_SRC_PFVEI0 )    /* PFV */
    {
        ASSERT_D( num_of_IRQ >= BSP_INT_SRC_PFVEI0  &&  num_of_IRQ <= BSP_INT_SRC_PFVEI1, R_NOOP() );
        *out_Unit = PFV_INTERRUPT_UNIT_PFV;
    }
    else    /* DMAC */
    {
        ASSERT_D( num_of_IRQ >= BSP_INT_SRC_DMAINT0  &&  num_of_IRQ <= BSP_INT_SRC_DMAINT15, R_NOOP() );
        *out_Unit = PFV_INTERRUPT_UNIT_DMAC;
    }

    e=0;
fin:
    return  e;
}


/***********************************************************************
* Function: R_PFV_Error_IRQ_HandlerN
*    This function is run in interrupt context.
*
* Return Value:
*    None
************************************************************************/
static void  R_PFV_Error_IRQ_HandlerN( int_fast32_t const  in_PFV_Channel )
{
    r_pfv_pl_channel_t* const  self = &gs_pfv_pl_channel[ in_PFV_Channel ];
    const r_ospl_interrupt_t* const  i_context = &gs_interrupt_context_pfv_error[ in_PFV_Channel ];

    R_OSPL_CallInterruptCallback( self->InterruptCallbackCaller, i_context );
}


/***********************************************************************
* Function: R_PFV_Error_IRQ_Handler0
*    This function is run in interrupt context.
*
* Return Value:
*    None
************************************************************************/
static void  R_PFV_Error_IRQ_Handler0( R_OSPL_INTERRUPT_ARGUMENTS )
{
    R_PFV_Error_IRQ_HandlerN( 0 );
    R_OSPL_END_OF_LEVEL_INTERRUPT();
}


/***********************************************************************
* Function: R_PFV_Error_IRQ_Handler1
*    This function is run in interrupt context.
*
* Return Value:
*    None
************************************************************************/
static void  R_PFV_Error_IRQ_Handler1( R_OSPL_INTERRUPT_ARGUMENTS )
{
    R_PFV_Error_IRQ_HandlerN( 1 );
    R_OSPL_END_OF_LEVEL_INTERRUPT();
}


