/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2015 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/***********************************************************************
* File: r_dmac_rm.c
*    Direct Memory Access Controller (DMAC) - Register Mode Sample Driver.
*
* - $Version: $ (=PFV_VERSION)
* - $ModuleRevision: {Masters}\PFV_DMAC\PFV_DMAC\23_RStyled $
* - $Rev: $
* - $Date: 2017-08-24T18:32:46+09:00 $
************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include  "r_typedefs.h"
#include  "dev_drv.h"                /* Device Driver common header */
#include  "iodefine.h"
#include  "clib_drivers.h"
#include  "iobitmasks/dmac_rm_iobitmask.h"
#include  "r_dmac_rm.h"
#include  "r_dmac_rm_internal.h"
#include  "r_dmac_rm_inline.h"
#include  "r_dmac_rm_pl.h"
#ifndef R_OSPL_NDEBUG
#include  <stdio.h>
#endif


/******************************************************************************
Typedef definitions
******************************************************************************/

/******************************************************************************
Macro definitions
******************************************************************************/

/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/

/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/

/******************************************************************************
Private global variables and functions
******************************************************************************/

/*[gs_dmac_rm_channel]*/
static r_dmac_rm_channel_t  gs_dmac_rm_channel[ R_DMAC_RM_CHANNEL_COUNT ];

/*[gs_dmac_rm_i_lock]*/
static r_dmac_rm_i_lock_t  gs_dmac_rm_i_lock[ R_DMAC_RM_CHANNEL_COUNT ];


static void   R_DMAC_RM_I_LOCK_Reset( r_dmac_rm_i_lock_t* const  self_ );
static void   R_DMAC_RM_I_LOCK_Finalize( int_fast32_t const  dmac_channel );
static bool_t R_DMAC_RM_I_LOCK_Lock( void* const  self_ );
static void   R_DMAC_RM_I_LOCK_Unlock( void* const  self_ );
static void   R_DMAC_RM_I_LOCK_RequestFinalize( void* const  self_ );
static int_fast32_t  R_DMAC_RM_I_LOCK_GetRootChannelNum( const void* const  self_ );


/***********************************************************************
* Function: R_DMAC_RM_STATIC_GetChanelContext
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
r_dmac_rm_channel_t*  R_DMAC_RM_STATIC_GetChanelContexts()
{
    return  gs_dmac_rm_channel;
}


/***********************************************************************
* Function: DMAC_RM_STATIC_GetResouceRegisterSet
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
/* ->MISRA 8.10 : External linkage is for Debug configuration */ /* ->SEC M2.2.3 */
#if IODEFINE_H_VERSION >= 60
errnum_t  DMAC_RM_STATIC_GetResouceRegisterSet( int_fast32_t const  dmac_channel,
        struct st_dmars_mm** const  out_reg_set,
        const dmars_mask_shift_t** const  out_mask_shift )
#else
errnum_t  DMAC_RM_STATIC_GetResouceRegisterSet( int_fast32_t const  dmac_channel,
        volatile struct st_dmars_mm** const  out_reg_set,
        const dmars_mask_shift_t** const  out_mask_shift )
#endif
/* <-MISRA 8.10 */ /* <-SEC M2.2.3 */
{
    errnum_t  e;

    /* ->MISRA 11.3 */ /* ->MISRA 11.4 */ /* ->MISRA 11.5 */ /* ->SEC R2.7.1 */ /* ->SEC R2.7.2 */
    /* ->QAC 0312 */
#if IODEFINE_H_VERSION >= 100
    static struct st_dmars_mm*  resource_reg_set_array[ DMACmm_COUNT ] =
                DMACmm_ADDRESS_LIST;
#elif IODEFINE_H_VERSION >= 60
    static struct st_dmars_mm*  resource_reg_set_array[ DMACmm_COUNT ] =
    {
        &DMAC01, &DMAC23, &DMAC45, &DMAC67, &DMAC89, &DMAC1011, &DMAC1213, &DMAC1415
    };
#else
    static volatile struct st_dmars_mm*  resource_reg_set_array[ DMACmm_COUNT ] =
    {
        &DMAC01, &DMAC23, &DMAC45, &DMAC67, &DMAC89, &DMAC1011, &DMAC1213, &DMAC1415
    };
#endif
    /* <-MISRA 11.3 */ /* <-MISRA 11.4 */ /* <-MISRA 11.5 */ /* <-SEC R2.7.1 */ /* <-SEC R2.7.2 */
    /* <-QAC 0312 */

    static const dmars_mask_shift_t  mask_shift[ DMACn_COUNT / DMACmm_COUNT ] =
{
        {
            DMAC_T__MASK32__B_MID0,  DMAC_T__SHIFT__B_MID0,
            DMAC_T__MASK32__B_RID0,  DMAC_T__SHIFT__B_RID0
        },
        {
            DMAC_T__MASK32__B_MID1,  DMAC_T__SHIFT__B_MID1,
            DMAC_T__MASK32__B_RID1,  DMAC_T__SHIFT__B_RID1
        }
    };


    IF_DQ( out_reg_set    == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }
    IF_DQ( out_mask_shift == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }


#if DMAC_PARAMETER_CHECK
    IF ( dmac_channel < DMAC_CHANNEL_MIN )
    {
        e = E_OTHERS;
        goto fin;
    }
    IF ( dmac_channel > DMAC_CHANNEL_MAX )
    {
        e = E_OTHERS;
        goto fin;
    }
#endif

    {
        enum { count_in_register = ( DMACn_COUNT / DMACmm_COUNT ) };

        *out_reg_set = resource_reg_set_array[ dmac_channel / count_in_register ];
        *out_mask_shift = &mask_shift[ R_Mod_2s( dmac_channel ) ];
    }

    e = 0;
#if DMAC_PARAMETER_CHECK
fin:
#endif
    return  e;
}


/***********************************************************************
* Function: R_DMAC_RM_STATIC_GetID_FromInterruptID
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_DMAC_RM_STATIC_GetID_FromInterruptID( bsp_int_src_t const  interrupt_id,
        int_fast32_t* const  out_dmac_channel )
{
    errnum_t  e;

    IF_DQ( out_dmac_channel == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }

    if ( (interrupt_id >= BSP_INT_SRC_DMAINT0) && (interrupt_id <= BSP_INT_SRC_DMAINT15) )
    {
        *out_dmac_channel = (int_fast32_t)( interrupt_id - (int_fast32_t)BSP_INT_SRC_DMAINT0 );
    }
    else
    {
        /* Check "ER" bit in "CHSTAT_n" register */
        int_fast32_t    dmac_channel;
        uint32_t        value;
        bool_t          is_found;
#if IODEFINE_H_VERSION >= 60
        struct st_dmac_n*  regs_DMAC;
#else
        volatile struct st_dmac_n*  regs_DMAC;
#endif



        IF ( interrupt_id != BSP_INT_SRC_DMAERR )
        {
            e = E_OTHERS;
            goto fin;
        }


        is_found = false;
        for ( dmac_channel = (int_fast32_t) DMAC_CHANNEL_MIN;
                dmac_channel <= (int_fast32_t) DMAC_CHANNEL_MAX;
                dmac_channel = (int_fast32_t)( (int_fast32_t) dmac_channel + 1 ) )
        {
            e= DMAC_RM_STATIC_GetRegisterSet( dmac_channel, &regs_DMAC );
            IF ( e != 0 )
            {
                goto fin;
            }
            IF_DQ( regs_DMAC == NULL )
            {
                e=E_OTHERS;
                goto fin;
            }

            value = regs_DMAC->CHSTAT_n;
            if ( IS( (bool_t) R_DMAC_RM_GET_REGISTER_BIT_FIELD( value, CHSTAT_n, ER ) ) )
            {
                *out_dmac_channel = dmac_channel;
                is_found = true;
            }
        }
        IF ( ! is_found )
        {
            e = E_OTHERS;
            goto fin;
        }
    }

    e = 0;
fin:
    return  e;
}


/***********************************************************************
* Function: R_DMAC_RM_STATIC_GetInterruptID
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_DMAC_RM_STATIC_GetInterruptID( int_fast32_t const  dmac_channel,
        bsp_int_src_t* const  out_interrupt_id )
{
    errnum_t  e;
    IF_DQ( out_interrupt_id == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }

    IF ( dmac_channel < DMAC_CHANNEL_MIN )
    {
        e = E_OTHERS;
        goto fin;
    }
    IF ( dmac_channel > DMAC_CHANNEL_MAX )
    {
        e = E_OTHERS;
        goto fin;
    }

    *out_interrupt_id = (bsp_int_src_t)( BSP_INT_SRC_DMAINT0 + dmac_channel );

    e = 0;
fin:
    return  e;
}


/***********************************************************************
* Function: R_DMAC_RM_Initialize
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_DMAC_RM_Initialize( int_fast32_t const  dmac_channel )
{
    return  R_DMAC_RM_InitializeEx( dmac_channel, NULL );
}


/***********************************************************************
* Function: R_DMAC_RM_InitializeEx
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_DMAC_RM_InitializeEx( int_fast32_t  dmac_channel, dmac_rm_config_t* in_out_Config )
{
    errnum_t  e;
    dmac_rm_config_t  config_body;
#if IODEFINE_H_VERSION >= 60
    struct st_dmac_n*  regs_DMAC;
#else
    volatile struct st_dmac_n*  regs_DMAC;
#endif
    r_dmac_rm_channel_t*  self;

    /*[gs_I_LockVTable]*/
    static const r_ospl_i_lock_vtable_t  gs_I_LockVTable =
    {
        &( R_DMAC_RM_I_LOCK_Lock ),  /* MISRA 16.9 */
        &( R_DMAC_RM_I_LOCK_Unlock ),
        &( R_DMAC_RM_I_LOCK_RequestFinalize ),
        &( R_DMAC_RM_I_LOCK_GetRootChannelNum )
    };

    if ( in_out_Config == NULL )
    {
        in_out_Config = &config_body;
        in_out_Config->flags = 0;
    }

    e= R_DMAC_RM_LockChannel( dmac_channel, &dmac_channel );
    IF(e!=0)
    {
        goto fin;
    }
    in_out_Config->channel_num = dmac_channel;

    self = &gs_dmac_rm_channel[ dmac_channel ];

    IF ( (dmac_channel < 0)  ||  (dmac_channel >= R_DMAC_RM_CHANNEL_COUNT) )
    {
        e = E_OTHERS;
        goto fin;
    }

    if ( self->i_lock == NULL )
    {
        r_dmac_rm_i_lock_t* const  i_lock_ = &gs_dmac_rm_i_lock[ dmac_channel ];

        i_lock_->channel_num = dmac_channel;

        self->is_i_lock_master = R_DMAC_RM_I_LOCK_Replace(
                                     dmac_channel,
                                     i_lock_,  &gs_I_LockVTable  );
    }

    if ( IS( self->is_i_lock_master ) )
    {
        R_DMAC_RM_I_LOCK_Reset( self->i_lock );
    }
    ASSERT_R( self->async_status.State == R_OSPL_UNINITIALIZED,  e=E_STATE; goto fin );
    R_OSPL_FLAG32_InitConst( &self->async_status.InterruptFlags );
    R_OSPL_FLAG32_InitConst( &self->async_status.CancelFlags );
#if  R_OSPL_IS_PREEMPTION
    self->async_status.LockOwner.Thread = R_OSPL_THREAD_GetCurrentId();
    R_OSPL_CHANGE_THREAD_LOCKED_COUNT( self->async_status.LockOwner.Thread, +1 );
#endif

    e= R_DMAC_RM_OnInitialize( dmac_channel );
    IF(e!=0)
    {
        goto fin;
    }

    e= DMAC_RM_STATIC_GetRegisterSet( dmac_channel, &regs_DMAC );
    IF ( e != 0 )
    {
        goto fin;
    }
    IF_DQ( regs_DMAC == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }

    /* Write "SWRST" bit in "CHCTRL_n" register */
    R_DMAC_RM_SET_REGISTER_BIT_FIELD( &regs_DMAC->CHCTRL_n, CHCTRL_n, SWRST, TO_UNSIGNED(true) );

    R_OSPL_FLAG32_InitConst( &self->async_status.InterruptEnables );
    self->async_status.State = R_OSPL_RUNNABLE;

    e = 0;
fin:
    return  e;
}


/***********************************************************************
* Function: R_DMAC_RM_Finalize
*
* Return Value:
*    Error code, 0=No error and e=0
************************************************************************/
errnum_t  R_DMAC_RM_Finalize( int_fast32_t const  dmac_channel, errnum_t e )
{
    errnum_t        ee;
    bool_t          was_enabled = false;
    uint32_t        value;
    r_dmac_rm_channel_t* const  self = &gs_dmac_rm_channel[ dmac_channel ];

    if ( ( (dmac_channel >= 0)  &&  (dmac_channel < R_DMAC_RM_CHANNEL_COUNT) )  &&
            self->async_status.State != R_OSPL_UNINITIALIZED
#if  R_OSPL_IS_PREEMPTION
            &&  self->async_status.LockOwner.Thread == R_OSPL_THREAD_GetCurrentId()
#endif
       )
    {
        was_enabled = self->i_lock_v_table->Lock( self->i_lock );

        R_OSPL_FLAG32_Set( &self->async_status.CancelFlags,
                           R_OSPL_CANNEL_REQUEST | R_OSPL_FINALIZE_REQUEST );

        ee=  R_DMAC_RM_StopAsRegisterMode( dmac_channel );
        IF ( (ee != 0)  &&  (e == 0) )
        {
            e = ee;
        }

        value = R_OSPL_FLAG32_Get( &self->async_status.CancelFlags ); /* QAC 3441 */
        ASSERT_R( IS_ALL_BITS_SET( value,
                                   R_OSPL_CANNEL_REQUEST | R_OSPL_CANNELING | R_OSPL_CANNELED ),  ee=E_OTHERS );
        IF ( (ee != 0)  &&  (e == 0) )
        {
            e = ee;
        }

        R_OSPL_FLAG32_Set( &self->async_status.CancelFlags, R_OSPL_FINALIZING );

        e= R_DMAC_RM_OnFinalize( dmac_channel, e );

        R_OSPL_FLAG32_Set( &self->async_status.CancelFlags, R_OSPL_FINALIZED );
        R_OSPL_FLAG32_Clear( &self->async_status.CancelFlags, 0 );
        self->async_status.State = R_OSPL_UNINITIALIZED;
        self->i_lock_v_table->RequestFinalize( self->i_lock );

#if  R_OSPL_IS_PREEMPTION
        R_OSPL_CHANGE_THREAD_LOCKED_COUNT( self->async_status.LockOwner.Thread, -1 );
        self->async_status.LockOwner.Thread = NULL;
#endif
        e= R_DMAC_RM_UnlockChannel( dmac_channel, e );
    }

    if ( IS( was_enabled ) )
    {
        self->i_lock_v_table->Unlock( self->i_lock );

        if ( IS( self->is_i_lock_master ) )
        {
            IF_DQ ( ( (dmac_channel < 0) || (dmac_channel >= R_DMAC_RM_CHANNEL_COUNT) ) ) {}
            else   /* QAC 3689 */
            {
                if ( IS( gs_dmac_rm_i_lock[ dmac_channel ].is_requested_finalize ) )
                {
                    R_DMAC_RM_I_LOCK_Finalize( dmac_channel );
                }
            }
        }
    }

    return  e;
}


/***********************************************************************
* Function: R_DMAC_RM_SetSharedConfiguration
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_DMAC_RM_SetSharedConfiguration( int_fast32_t const  dmac_channel,
        dmac_shared_config_t* const  config )
{
    errnum_t            e;
    uint32_t            value;
    uint32_t            sub_value;
    bit_flags_fast32_t  config_flags;
#if IODEFINE_H_VERSION >= 60
#ifdef __GNUC__
    struct st_dmaccommon_n*  reg_sh_DMAC = NULL;
    /* NULL is avoid for warning that "-Wmaybe-uninitialized". */
#else
    struct st_dmaccommon_n*  reg_sh_DMAC;
#endif
#else
    volatile struct st_dmaccommon_n*  reg_sh_DMAC;
#endif


    R_IT_WILL_BE_NOT_CONST( config );
    IF_DQ( config == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }

    config_flags = config->flags;


    /* Set "reg_sh_DMAC" */
    e= DMAC_RM_STATIC_GetSharedRegisterSet( dmac_channel, &reg_sh_DMAC );
    IF ( e != 0 )
    {
        e = E_OTHERS;
        goto  fin;
    }
    IF_DQ( reg_sh_DMAC == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }


    /* Read from "DCTRL_nn" register */
    value = reg_sh_DMAC->DCTRL_0_7;


    /* Set to "LWCA" bitfield */
    if ( IS_BIT_SET( config_flags, F_DMAC_LINK_WRITE_BACK_CACHE ) )
    {
        sub_value = (uint32_t) config->link_write_back_cache;
#if DMAC_PARAMETER_CHECK
        IF ( IS( R_DMAC_RM_IS_OVERFLOW_BIT_FIELD( DCTRL_nn, LWCA, sub_value ) ) )
        {
            e = E_OTHERS;
            goto  fin;
        }
#endif
        R_DMAC_RM_SET_REGISTER_BIT_FIELD( &value, DCTRL_nn, LWCA, sub_value );
    }

    /* Set to "LWPR" bitfield */
    if ( IS_BIT_SET( config_flags, F_DMAC_LINK_WRITE_BACK_PROTECTION ) )
    {
        sub_value = (uint32_t) config->link_write_back_protection;
#if DMAC_PARAMETER_CHECK
        IF ( IS( R_DMAC_RM_IS_OVERFLOW_BIT_FIELD( DCTRL_nn, LWPR, sub_value ) ) )
        {
            e = E_OTHERS;
            goto  fin;
        }
#endif
        R_DMAC_RM_SET_REGISTER_BIT_FIELD( &value, DCTRL_nn, LWPR, sub_value );
    }

    /* Set to "LDCA" bitfield */
    if ( IS_BIT_SET( config_flags, F_DMAC_LINK_DESCRIPTOR_CACHE ) )
    {
        sub_value = (uint32_t) config->link_descriptor_cache;
#if DMAC_PARAMETER_CHECK
        IF ( IS( R_DMAC_RM_IS_OVERFLOW_BIT_FIELD( DCTRL_nn, LDCA, sub_value ) ) )
        {
            e = E_OTHERS;
            goto  fin;
        }
#endif
        R_DMAC_RM_SET_REGISTER_BIT_FIELD( &value, DCTRL_nn, LDCA, sub_value );
    }

    /* Set to "LDPR" bitfield */
    if ( IS_BIT_SET( config_flags, F_DMAC_LINK_DESCRIPTOR_PROTECTION ) )
    {
        sub_value = (uint32_t) config->link_descriptor_protection;
#if DMAC_PARAMETER_CHECK
        IF ( IS( R_DMAC_RM_IS_OVERFLOW_BIT_FIELD( DCTRL_nn, LDPR, sub_value ) ) )
        {
            e = E_OTHERS;
            goto  fin;
        }
#endif
        R_DMAC_RM_SET_REGISTER_BIT_FIELD( &value, DCTRL_nn, LDPR, sub_value );
    }

    /* Set to "LVINT" bitfield */
    if ( IS_BIT_SET( config_flags, F_DMAC_OUTPUT_INTERRUPT_PULSE_OR_LEVEL ) )
    {
        sub_value = (uint32_t) config->output_interrupt_pulse_or_level;
#if DMAC_PARAMETER_CHECK
        IF_D ( IS( R_DMAC_RM_IS_OVERFLOW_BIT_FIELD( DCTRL_nn, LVINT, sub_value ) ) )
        {
            e = E_OTHERS;
            goto  fin;
        }
#endif
        R_DMAC_RM_SET_REGISTER_BIT_FIELD( &value, DCTRL_nn, LVINT, sub_value );
    }

    /* Set to "PR" bitfield */
    if ( IS_BIT_SET( config_flags, F_DMAC_PRIORITY_MODE ) )
    {
        sub_value = (uint32_t) config->priority_mode;
#if DMAC_PARAMETER_CHECK
        IF_D ( IS( R_DMAC_RM_IS_OVERFLOW_BIT_FIELD( DCTRL_nn, PR, sub_value ) ) )
        {
            e = E_OTHERS;
            goto  fin;
        }
#endif
        R_DMAC_RM_SET_REGISTER_BIT_FIELD( &value, DCTRL_nn, PR, sub_value );
    }


    /* Write to "DCTRL_nn" register */
    reg_sh_DMAC->DCTRL_0_7 = value;

    e = 0;
fin:
    return  e;
}


/***********************************************************************
* Function: R_DMAC_RM_TransferAsRegisterMode
*    See "R_DMAC_RM_TransferAsLinkModeStart"
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_DMAC_RM_TransferAsRegisterMode( int_fast32_t const  dmac_channel,
        dmac_register_mode_config_t* const  config )
{
    errnum_t         e;
    errnum_t         ee;
    r_ospl_async_t   async;
    bit_flags32_t    got_flags;


    async.A_EventValue = 0;  /* For fin block */

    async.Flags = R_F_OSPL_A_Thread | R_F_OSPL_A_EventValue;
    e= R_OSPL_EVENT_Allocate(
           &async.A_Thread,  R_OSPL_THREAD_GetCurrentId(),
           &async.A_EventValue,  R_OSPL_UNUSED_FLAG );
    IF(e)
    {
        goto fin;
    }

    e= R_DMAC_RM_StartAsRegisterMode( dmac_channel, config,  &async );
    IF(e)
    {
        goto fin;
    }

    e= R_OSPL_EVENT_Wait( async.A_EventValue, &got_flags, R_OSPL_INFINITE );
    IF(e)
    {
        goto fin;
    }

    e=0;
fin:
    ee= R_OSPL_EVENT_Free( &async.A_Thread, &async.A_EventValue );
    e= R_OSPL_MergeErrNum( e, ee );
    return  e;
}


/***********************************************************************
* Function: R_DMAC_RM_StartAsRegisterMode
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_DMAC_RM_StartAsRegisterMode( int_fast32_t const  dmac_channel,
        dmac_register_mode_config_t* const  config, r_ospl_async_t* const  Async )
{
    errnum_t            e;
    uint32_t            value;
    bool_t              was_enabled = false;
    bit_flags_fast32_t  config_flags;
#if IODEFINE_H_VERSION >= 60
    struct st_dmac_n*                  regs_DMAC;
    struct st_dmars_mm*                regs_res_DMAC;
#else
    volatile struct st_dmac_n*         regs_DMAC;
    volatile struct st_dmars_mm*       regs_res_DMAC;
#endif
    const dmars_mask_shift_t*          mask_shift_DMARS;
    r_dmac_rm_channel_t* const  self = &gs_dmac_rm_channel[ dmac_channel ];


    IF_DQ( config == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }
    IF_DQ( Async == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }

    ASSERT_R( self->async_status.State == R_OSPL_RUNNABLE,  e=E_STATE; goto fin );

    config_flags = config->flags;


    R_DMAC_RM_SetDefaultAsync( Async, R_OSPL_ASYNC_TYPE_NORMAL );
    Async->ReturnValue = 0;

    was_enabled = self->i_lock_v_table->Lock( self->i_lock );


    /* Set to memory */
    /* Call "R_OSPL_MEMORY_Flush( R_OSPL_FLUSH_WRITEBACK_INVALIDATE );" at application */
    R_OSPL_MEMORY_Barrier();


    /* Set "regs_DMAC" */
    e= DMAC_RM_STATIC_GetRegisterSet( dmac_channel, &regs_DMAC );
    IF(e!=0)
    {
        goto fin;
    }
    IF_DQ( regs_DMAC == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }

    /* Set "regs_res_DMAC" */
    e= DMAC_RM_STATIC_GetResouceRegisterSet( dmac_channel, &regs_res_DMAC, &mask_shift_DMARS );
    IF(e!=0)
    {
        goto fin;
    }
    IF_DQ( regs_res_DMAC == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }
    IF_DQ( mask_shift_DMARS == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }


    /* Check must flags */
    {
        static const bit_flags_fast32_t  must_flags =
            F_DMAC_NEXT0_SOURCE_PHYSICAL_ADDRESS |
            F_DMAC_NEXT0_DESTINATION_PHYSICAL_ADDRESS |
            F_DMAC_NEXT0_TRANSFER_BYTE;

        IF ( IS_ANY_BITS_NOT_SET( config_flags, must_flags ) )
        {
            e = E_OTHERS;
            goto  fin;
        }
    }

    if ( IS_BIT_SET( config_flags, F_DMAC_REQUEST_RESOURCE_MID ) )
    {
        static const bit_flags_fast32_t  must_flags_rid =
            F_DMAC_REQUEST_RESOURCE_MID |
            F_DMAC_REQUEST_RESOURCE_RID |
            F_DMAC_TRANSFER_MODE_TM |
            F_DMAC_IS_DMA_ACK_OUTPUT_MODE_AM |
            F_DMAC_REQUEST_EDGE_OR_LEVEL_LVL |
            F_DMAC_IS_REQUEST_RAISING_OR_HIGH_HIEN |
            F_DMAC_IS_REQUEST_FALLING_OR_LOW_LOEN |
            F_DMAC_REQUEST_DIRECTION_REQD;

        IF ( IS_ANY_BITS_NOT_SET( config_flags, must_flags_rid ) )
        {
            e = E_OTHERS;
            goto  fin;
        }
    }


    /* Set default config */
    R_DMAC_RM_SetDefaultConfig_Sub( config );


    /* Write to "N0SA_n", "N0DA_n", "N0TB_n" register */
    regs_DMAC->N0SA_n = (uintptr_t) config->next0_source_physical_address;
    regs_DMAC->N0DA_n = (uintptr_t) config->next0_destination_physical_address;
    regs_DMAC->N0TB_n = config->next0_transfer_byte;


    /* Write to "N1SA_n", "N1DA_n", "N1TB_n" register */
    if ( IS_BIT_SET( config_flags, F_DMAC_NEXT1_SOURCE_PHYSICAL_ADDRESS ) )
    {
        const bit_flags_fast32_t  must_flags_n1 =
            F_DMAC_NEXT1_SOURCE_PHYSICAL_ADDRESS |
            F_DMAC_NEXT1_DESTINATION_PHYSICAL_ADDRESS |
            F_DMAC_NEXT1_TRANSFER_BYTE;

        IF ( IS_ANY_BITS_NOT_SET( config_flags, must_flags_n1 ) )
        {
            e = E_OTHERS;
            goto  fin;
        }

        regs_DMAC->N1SA_n = (uintptr_t) config->next1_source_physical_address;
        regs_DMAC->N1DA_n = (uintptr_t) config->next1_destination_physical_address;
        regs_DMAC->N1TB_n = config->next1_transfer_byte;
    }


    /* Write to "CHCFG_n", "CHITVL_n", "CHEXT_n" register */
    {
        uint32_t  value_of_CHCFG_n;
        uint32_t  value_of_CHITVL_n;
        uint32_t  value_of_CHEXT_n;

        e= R_DMAC_RM_GetRegisterValues_Sub( config, dmac_channel,
                                            &value_of_CHCFG_n, &value_of_CHITVL_n, &value_of_CHEXT_n );
        IF(e)
        {
            goto fin;
        }

        regs_DMAC->CHCFG_n  = value_of_CHCFG_n;
        regs_DMAC->CHITVL_n = value_of_CHITVL_n;
        regs_DMAC->CHEXT_n  = value_of_CHEXT_n;
    }


    /* Write to "DMARSn" register */
    if ( IS_BIT_SET( config_flags, F_DMAC_REQUEST_RESOURCE_MID ) )
    {
        value = regs_res_DMAC->DMARS;
        e= R_DMAC_RM_GetDMARS_Values_Sub( config, mask_shift_DMARS, &value );
        IF(e)
        {
            goto fin;
        }
        regs_res_DMAC->DMARS = value;
    }


    /* Attach "Async" to interrupt */
    /* ->MISRA 11.4 : Not too big "enum" is same bit count as "int_t" */ /* ->SEC R2.7.1 */
    R_OSPL_CALLER_Initialize( &self->interrupt_callback_caller,
                              Async, (volatile int_t*) &self->async_status.State, R_OSPL_INTERRUPTING,
                              self->i_lock, self->i_lock_v_table );
    /* <-MISRA 11.4 */ /* <-SEC R2.7.1 */
    e= R_DMAC_RM_SetInterruptCallbackCaller( dmac_channel,
            &self->interrupt_callback_caller );
    IF(e!=0)
    {
        goto fin;
    }
    R_OSPL_FLAG32_Set( &self->async_status.InterruptEnables, DMAC_INTERRUPT_LINE_TRANSFER_END );


    /* Clear event flags */
    R_OSPL_EVENT_Clear( Async->A_Thread, Async->A_EventValue );
    R_OSPL_EVENT_Clear( Async->I_Thread, Async->I_EventValue );


    self->async_status.State = R_OSPL_RUNNING;


    /* Software reset DMA */
    R_DMAC_RM_SET_REGISTER_BIT_FIELD( &regs_DMAC->CHCTRL_n, CHCTRL_n, SWRST, TO_UNSIGNED( true ) );
#if  DMAC_PARAMETER_RESET_DELAY
    R_OSPL_Delay( 1 );
#endif


    /* Start DMA transfer */
    R_DMAC_RM_SET_REGISTER_BIT_FIELD( &regs_DMAC->CHCTRL_n, CHCTRL_n, SETEN, TO_UNSIGNED( true ) );


    /* Start DMA transfer by auto request mode */
    if ( IS_BIT_NOT_SET( config_flags, F_DMAC_REQUEST_RESOURCE_MID ) )
    {
        R_DMAC_RM_SET_REGISTER_BIT_FIELD( &regs_DMAC->CHCTRL_n, CHCTRL_n, STG, TO_UNSIGNED( true ) );
    }


    e = 0;
fin:
    if ( IS( was_enabled ) )
    {
        self->i_lock_v_table->Unlock( self->i_lock );
    }
    return  e;
}


/***********************************************************************
* Function: R_DMAC_RM_SetDefaultConfig_Sub
*
* Return Value:
*    None
************************************************************************/
void  R_DMAC_RM_SetDefaultConfig_Sub( dmac_register_mode_config_t* const  config )
{
    bit_flags_fast32_t const  config_flags = config->flags;

    if ( IS_BIT_NOT_SET( config_flags, F_DMAC_IS_SOURCE_ADDRESS_FIXED ) )
    {
        config->is_source_address_fixed = false;
    }

    if ( IS_BIT_NOT_SET( config_flags, F_DMAC_SOURCE_BIT_COUNT ) )
    {
        config->source_bit_count = DMAC_8_BIT;
    }

    if ( IS_BIT_NOT_SET( config_flags, F_DMAC_IS_DESTINATION_ADDRESS_FIXED ) )
    {
        config->is_destination_address_fixed = false;
    }

    if ( IS_BIT_NOT_SET( config_flags, F_DMAC_DESTINATION_BIT_COUNT ) )
    {
        config->destination_bit_count = DMAC_8_BIT;
    }

    if ( IS_BIT_NOT_SET( config_flags, F_DMAC_IS_RELOAD_NEXT_REGISTER_SET ) )
    {
        config->is_reload_next_register_set = false;
    }

    if ( IS_BIT_NOT_SET( config_flags, F_DMAC_IS_AUTOMATIC_SWITCH_NEXT_NUM ) )
    {
        config->is_automatic_switch_next_num = false;
    }

    if ( IS_BIT_NOT_SET( config_flags, F_DMAC_NEXT_REGISTER_SET_NUMBER ) )
    {
        config->next_register_set_number = 0;
    }

    if ( IS_BIT_NOT_SET( config_flags, F_DMAC_IS_SWEEP_BUFFER ) )
    {
        config->is_sweep_buffer = false;
    }

    if ( IS_BIT_NOT_SET( config_flags, F_DMAC_IS_DISABLE_NEXT_END_INTERRUPT ) )
    {
        config->is_disable_next_end_interrupt = false;
    }

    if ( IS_BIT_NOT_SET( config_flags, F_DMAC_TRANSFER_MODE_TM ) )
    {
        if ( IS_BIT_SET( config_flags, F_DMAC_REQUEST_RESOURCE_MID ) )
        {
            config->transfer_mode_TM = DMAC_SINGLE_TRANSFER_0;
        }
        else
        {
            config->transfer_mode_TM = DMAC_BLOCK_TRANSFER_1;
        }
    }

    if ( IS_BIT_NOT_SET( config_flags, F_DMAC_IS_DMA_ACK_OUTPUT_MODE_AM ) )
    {
        if ( IS_BIT_SET( config_flags, F_DMAC_REQUEST_RESOURCE_MID ) )
        {
            config->is_DMA_ack_output_mode_AM = DMAC_ACK_MODE_RESET_VALUE;
        }
        else
        {
            config->is_DMA_ack_output_mode_AM = DMAC_ACK_NOT_OUTPUT_100;
        }
    }

    if ( IS_BIT_NOT_SET( config_flags, F_DMAC_REQUEST_EDGE_OR_LEVEL_LVL ) )
    {
        config->request_edge_or_level_LVL = DMAC_EDGE_0;  /* This is reset value */
    }

    if ( IS_BIT_NOT_SET( config_flags, F_DMAC_IS_REQUEST_RAISING_OR_HIGH_HIEN ) )
    {
        config->is_request_raising_or_high_HIEN = false;  /* This is reset value */
    }

    if ( IS_BIT_NOT_SET( config_flags, F_DMAC_IS_REQUEST_FALLING_OR_LOW_LOEN ) )
    {
        config->is_request_falling_or_low_LOEN = false;  /* This is reset value */
    }

    if ( IS_BIT_NOT_SET( config_flags, F_DMAC_REQUEST_DIRECTION_REQD ) )
    {
        config->request_direction_REQD = DMAC_DMAREQ_SOURCE_0;  /* This is reset value */
    }

    if ( IS_BIT_NOT_SET( config_flags, F_DMAC_INTERVAL_COUNT ) )
    {
        config->interval_count = 0;
    }

    if ( IS_BIT_NOT_SET( config_flags, F_DMAC_SOURCE_AXI_CACHE_ATTRIBUTE ) )
    {
        config->source_AXI_cache_attribute = R_OSPL_AXI_CACHE_ZERO;
    }

    if ( IS_BIT_NOT_SET( config_flags, F_DMAC_SOURCE_AXI_PROTECTION ) )
    {
        config->source_AXI_protection = R_OSPL_AXI_PROTECTION_ZERO;
    }

    if ( IS_BIT_NOT_SET( config_flags, F_DMAC_DESTINATION_AXI_CACHE_ATTRIBUTE ) )
    {
        config->destination_AXI_cache_attribute = R_OSPL_AXI_CACHE_ZERO;
    }

    if ( IS_BIT_NOT_SET( config_flags, F_DMAC_DESTINATION_AXI_PROTECTION ) )
    {
        config->destination_AXI_protection = R_OSPL_AXI_PROTECTION_ZERO;
    }
}


/***********************************************************************
* Function: R_DMAC_RM_GetRegisterValues_Sub
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_DMAC_RM_GetRegisterValues_Sub( dmac_register_mode_config_t* const  config,
        int_fast32_t const  dmac_channel,
        uint32_t*  out_CHCFG_n,  uint32_t*  out_CHITVL_n,  uint32_t*  out_CHEXT_n )
{
    errnum_t  e;
    uint32_t  value;
    uint32_t  sub_value;


    /* Set related with "CHCFG_n" register */
    {
        value = 0;

        /* Set to "DMS" bit */
        R_DMAC_RM_SET_REGISTER_BIT_FIELD( &value, CHCFG_n, DMS,
                                          DMAC_T__DMS_REGISTER_MODE );


        /* Set to "REN" bit */
        sub_value = (uint32_t) config->is_reload_next_register_set;
#if DMAC_PARAMETER_CHECK
        IF ( IS( R_DMAC_RM_IS_OVERFLOW_BIT_FIELD( CHCFG_n, REN, sub_value ) ) )
        {
            e = E_OTHERS;
            goto  fin;
        }
#endif
        R_DMAC_RM_SET_REGISTER_BIT_FIELD( &value, CHCFG_n, REN, sub_value );


        /* Set to "RSW" bit */
        sub_value = (uint32_t) config->is_automatic_switch_next_num;
#if DMAC_PARAMETER_CHECK
        IF ( IS( R_DMAC_RM_IS_OVERFLOW_BIT_FIELD( CHCFG_n, RSW, sub_value ) ) )
        {
            e = E_OTHERS;
            goto  fin;
        }
#endif
        R_DMAC_RM_SET_REGISTER_BIT_FIELD( &value, CHCFG_n, RSW, sub_value );


        /* Set to "RSEL" bit */
        sub_value = (uint32_t) config->next_register_set_number;
#if DMAC_PARAMETER_CHECK
        IF ( IS( R_DMAC_RM_IS_OVERFLOW_BIT_FIELD( CHCFG_n, RSEL, sub_value ) ) )
        {
            e = E_OTHERS;
            goto  fin;
        }
        IF ( ((int_fast32_t) sub_value < DMAC_T__NEXT_REGISTER_SET_MIN_NUM) ||
             ((int_fast32_t) sub_value > DMAC_T__NEXT_REGISTER_SET_MAX_NUM) )
        {
            e = E_OTHERS;
            goto  fin;
        }
#endif
        R_DMAC_RM_SET_REGISTER_BIT_FIELD( &value, CHCFG_n, RSEL, sub_value );


        /* Set to "SBE" bit */
        sub_value = (uint32_t) config->is_sweep_buffer;
#if DMAC_PARAMETER_CHECK
        IF ( IS( R_DMAC_RM_IS_OVERFLOW_BIT_FIELD( CHCFG_n, SBE, sub_value ) ) )
        {
            e = E_OTHERS;
            goto  fin;
        }
#endif
        R_DMAC_RM_SET_REGISTER_BIT_FIELD( &value, CHCFG_n, SBE, sub_value );


        /* Set to "DEM" bit */
        sub_value = (uint32_t) config->is_disable_next_end_interrupt;
#if DMAC_PARAMETER_CHECK
        IF ( IS( R_DMAC_RM_IS_OVERFLOW_BIT_FIELD( CHCFG_n, DEM, sub_value ) ) )
        {
            e = E_OTHERS;
            goto  fin;
        }
#endif
        R_DMAC_RM_SET_REGISTER_BIT_FIELD( &value, CHCFG_n, DEM, sub_value );


        /* Set to "TM" bit */
        sub_value = (uint32_t) config->transfer_mode_TM;
#if DMAC_PARAMETER_CHECK
        IF_D ( IS( R_DMAC_RM_IS_OVERFLOW_BIT_FIELD( CHCFG_n, TM, sub_value ) ) )
        {
            e = E_OTHERS;
            goto  fin;
        }
        IF_DS ( (sub_value != DMAC_SINGLE_TRANSFER_0) &&
                (sub_value != DMAC_BLOCK_TRANSFER_1) )
        {
            e = E_OTHERS;
            goto  fin;
        }
#endif
        R_DMAC_RM_SET_REGISTER_BIT_FIELD( &value, CHCFG_n, TM, sub_value );


        /* Set to "DAD" bit */
        sub_value = (uint32_t) config->is_destination_address_fixed;
#if DMAC_PARAMETER_CHECK
        IF ( IS( R_DMAC_RM_IS_OVERFLOW_BIT_FIELD( CHCFG_n, DAD, sub_value ) ) )
        {
            e = E_OTHERS;
            goto  fin;
        }
#endif
        R_DMAC_RM_SET_REGISTER_BIT_FIELD( &value, CHCFG_n, DAD, sub_value );


        /* Set to "SAD" bit */
        sub_value = (uint32_t) config->is_source_address_fixed;
#if DMAC_PARAMETER_CHECK
        IF ( IS( R_DMAC_RM_IS_OVERFLOW_BIT_FIELD( CHCFG_n, SAD, sub_value ) ) )
        {
            e = E_OTHERS;
            goto  fin;
        }
#endif
        R_DMAC_RM_SET_REGISTER_BIT_FIELD( &value, CHCFG_n, SAD, sub_value );


        /* Set to "DDS" bitfield */
        sub_value = (uint32_t) config->destination_bit_count;
#if DMAC_PARAMETER_CHECK
        IF_D ( IS( R_DMAC_RM_IS_OVERFLOW_BIT_FIELD( CHCFG_n, DDS, sub_value ) ) )
        {
            e = E_OTHERS;
            goto  fin;
        }
#endif
        R_DMAC_RM_SET_REGISTER_BIT_FIELD( &value, CHCFG_n, DDS, sub_value );


        /* Set to "SDS" bitfield */
        sub_value = (uint32_t) config->source_bit_count;
#if DMAC_PARAMETER_CHECK
        IF_D ( IS( R_DMAC_RM_IS_OVERFLOW_BIT_FIELD( CHCFG_n, SDS, sub_value ) ) )
        {
            e = E_OTHERS;
            goto  fin;
        }
#endif
        R_DMAC_RM_SET_REGISTER_BIT_FIELD( &value, CHCFG_n, SDS, sub_value );


        /* Set to "AM" bit */
        sub_value = (uint32_t) config->is_DMA_ack_output_mode_AM;
#if DMAC_PARAMETER_CHECK
        IF ( IS( R_DMAC_RM_IS_OVERFLOW_BIT_FIELD( CHCFG_n, AM, sub_value ) ) )
        {
            e = E_OTHERS;
            goto  fin;
        }
#endif
        R_DMAC_RM_SET_REGISTER_BIT_FIELD( &value, CHCFG_n, AM, sub_value );


        /* Set to "LVL" bit */
        sub_value = (uint32_t) config->request_edge_or_level_LVL;
#if DMAC_PARAMETER_CHECK
        IF_D ( IS( R_DMAC_RM_IS_OVERFLOW_BIT_FIELD( CHCFG_n, LVL, sub_value ) ) )
        {
            e = E_OTHERS;
            goto  fin;
        }
#endif
        R_DMAC_RM_SET_REGISTER_BIT_FIELD( &value, CHCFG_n, LVL, sub_value );


        /* Set to "HIEN" bit */
        sub_value = (uint32_t) config->is_request_raising_or_high_HIEN;
#if DMAC_PARAMETER_CHECK
        IF ( IS( R_DMAC_RM_IS_OVERFLOW_BIT_FIELD( CHCFG_n, HIEN, sub_value ) ) )
        {
            e = E_OTHERS;
            goto  fin;
        }
#endif
        R_DMAC_RM_SET_REGISTER_BIT_FIELD( &value, CHCFG_n, HIEN, sub_value );


        /* Set to "LOEN" bit */
        sub_value = (uint32_t) config->is_request_falling_or_low_LOEN;
#if DMAC_PARAMETER_CHECK
        IF ( IS( R_DMAC_RM_IS_OVERFLOW_BIT_FIELD( CHCFG_n, LOEN, sub_value ) ) )
        {
            e = E_OTHERS;
            goto  fin;
        }
#endif
        R_DMAC_RM_SET_REGISTER_BIT_FIELD( &value, CHCFG_n, LOEN, sub_value );


        /* Set to "REQD" bit */
        sub_value = (uint32_t) config->request_direction_REQD;
#if DMAC_PARAMETER_CHECK
        IF_D ( IS( R_DMAC_RM_IS_OVERFLOW_BIT_FIELD( CHCFG_n, REQD, sub_value ) ) )
        {
            e = E_OTHERS;
            goto  fin;
        }
#endif
        R_DMAC_RM_SET_REGISTER_BIT_FIELD( &value, CHCFG_n, REQD, sub_value );


        /* Set to "SEL" bitfield */
        {
            enum { max_value_of_SEL = 7 };

            sub_value = (uint32_t) dmac_channel % ( max_value_of_SEL + 1 );
#if DMAC_PARAMETER_CHECK
            IF ( IS( R_DMAC_RM_IS_OVERFLOW_BIT_FIELD( CHCFG_n, SEL, sub_value ) ) )
            {
                e = E_OTHERS;
                goto  fin;
            }
#endif
            R_DMAC_RM_SET_REGISTER_BIT_FIELD( &value, CHCFG_n, SEL, sub_value );
        }


        /* Write to "CHCFG_n" register */
        *out_CHCFG_n = value;
    }


    /* Set related with "CHITVL_n" register */
    {
        value = (uint32_t) config->interval_count;
#if DMAC_PARAMETER_CHECK
        IF ( IS( R_DMAC_RM_IS_OVERFLOW_BIT_FIELD( CHITVL_n, ITVL, value ) ) )
        {
            e = E_OTHERS;
            goto  fin;
        }
#endif

        /* Write to "CHITVL_n" register */
        *out_CHITVL_n = value;
    }


    /* Set related with "CHEXT_n" register */
    {
        value = 0;

        /* Set to "DCA" bit */
        sub_value = (uint32_t) config->destination_AXI_cache_attribute;
#if DMAC_PARAMETER_CHECK
        IF ( IS( R_DMAC_RM_IS_OVERFLOW_BIT_FIELD( CHEXT_n, DCA, sub_value ) ) )
        {
            e = E_OTHERS;
            goto  fin;
        }
#endif
        R_DMAC_RM_SET_REGISTER_BIT_FIELD( &value, CHEXT_n, DCA, sub_value );


        /* Set to "DPR" bit */
        sub_value = (uint32_t) config->destination_AXI_protection;
#if DMAC_PARAMETER_CHECK
        IF ( IS( R_DMAC_RM_IS_OVERFLOW_BIT_FIELD( CHEXT_n, DPR, sub_value ) ) )
        {
            e = E_OTHERS;
            goto  fin;
        }
#endif
        R_DMAC_RM_SET_REGISTER_BIT_FIELD( &value, CHEXT_n, DPR, sub_value );


        /* Set to "SCA" bit */
        sub_value = (uint32_t) config->source_AXI_cache_attribute;
#if DMAC_PARAMETER_CHECK
        IF ( IS( R_DMAC_RM_IS_OVERFLOW_BIT_FIELD( CHEXT_n, SCA, sub_value ) ) )
        {
            e = E_OTHERS;
            goto  fin;
        }
#endif
        R_DMAC_RM_SET_REGISTER_BIT_FIELD( &value, CHEXT_n, SCA, sub_value );


        /* Set to "SPR" bit */
        sub_value = (uint32_t) config->source_AXI_protection;
#if DMAC_PARAMETER_CHECK
        IF ( IS( R_DMAC_RM_IS_OVERFLOW_BIT_FIELD( CHEXT_n, SPR, sub_value ) ) )
        {
            e = E_OTHERS;
            goto  fin;
        }
#endif
        R_DMAC_RM_SET_REGISTER_BIT_FIELD( &value, CHEXT_n, SPR, sub_value );


        /* Write to "CHEXT_n" register */
        *out_CHEXT_n = value;
    }

    e=0;
fin:
    return  e;
}


/***********************************************************************
* Function: R_DMAC_RM_GetDMARS_Values_Sub
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_DMAC_RM_GetDMARS_Values_Sub( dmac_register_mode_config_t* const  config,
        const dmars_mask_shift_t*  mask_shift_DMARS,
        uint32_t*  in_out_DMARS )
{
    errnum_t  e;
    uint32_t  value;
    uint32_t  sub_value;


    {
        /* Read from "DMARSn" register */
        value = *in_out_DMARS;


        /* Set "MID" bitfield */
        sub_value = (uint32_t) config->request_resource_MID;
#if DMAC_PARAMETER_CHECK
        IF ( IS( R_DMAC_RM_IS_OVERFLOW_BIT_FIELD( DMARSn, CH0__MID, sub_value ) ) )
        {
            e = E_OTHERS;
            goto  fin;
        }
#endif
        R_OSPL_SET_TO_32_BIT_REGISTER( &value, mask_shift_DMARS->mask_MID,
                                       mask_shift_DMARS->shift_MID, sub_value );


        /* Set "RID" bitfield */
        sub_value = (uint32_t) config->request_resource_RID;
#if DMAC_PARAMETER_CHECK
        IF ( IS( R_DMAC_RM_IS_OVERFLOW_BIT_FIELD( DMARSn, CH0__RID, sub_value ) ) )
        {
            e = E_OTHERS;
            goto  fin;
        }
#endif
        R_OSPL_SET_TO_32_BIT_REGISTER( &value, mask_shift_DMARS->mask_RID,
                                       mask_shift_DMARS->shift_RID, sub_value );


        /* Write to "DMARSn" register */
        *in_out_DMARS = value;
    }

    e=0;
fin:
    return  e;
}


/***********************************************************************
* Function: R_DMAC_RM_StopAsRegisterMode
*
* Return Value:
*    Error code, 0=No error
************************************************************************/

#define  R_DMAC_MEASURE_CLREN_TIME  0

errnum_t  R_DMAC_RM_StopAsRegisterMode( int_fast32_t const  dmac_channel )
{
    errnum_t  e;
#if IODEFINE_H_VERSION >= 60
    struct st_dmac_n*  regs_DMAC;
#else
    volatile struct st_dmac_n*  regs_DMAC;
#endif
    bool_t             was_enabled = false;
    r_dmac_rm_channel_t* const  self = &gs_dmac_rm_channel[ dmac_channel ];


    ASSERT_R( self->async_status.State != R_OSPL_UNINITIALIZED,  e=E_STATE; goto fin );

    was_enabled = self->i_lock_v_table->Lock( self->i_lock );

    ASSERT_R( self->async_status.State == R_OSPL_RUNNABLE ||
              self->async_status.State == R_OSPL_RUNNING ||
              self->async_status.State == R_OSPL_INTERRUPTING ||
              self->async_status.State == R_OSPL_INTERRUPTED,  e=E_STATE; goto fin );


    R_OSPL_FLAG32_Set( &self->async_status.CancelFlags, R_OSPL_CANNELING );


    /* Set "regs_DMAC" */
    e= DMAC_RM_STATIC_GetRegisterSet( dmac_channel, &regs_DMAC );
    IF ( e != 0 )
    {
        e = E_OTHERS;
        goto  fin;
    }
    IF_DQ( regs_DMAC == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }


    /* Clear "EN" bit */
    R_DMAC_RM_SET_REGISTER_BIT_FIELD( &regs_DMAC->CHCTRL_n, CHCTRL_n, CLREN, TO_UNSIGNED( true ) );


    /* Wait for "TACT" bit == 0 */
#if  R_DMAC_MEASURE_CLREN_TIME
    {
        r_ospl_ftimer_spec_t  ts;
        uint32_t              start;
        uint32_t              end;

        e= R_OSPL_FTIMER_InitializeIfNot( &ts );
        IF(e!=0)
        {
            goto fin;
        }

        start = R_OSPL_FTIMER_Get();
        end = R_OSPL_FTIMER_Get();
        printf( "%dmsec (%d)\n", ( end - start ) * ts.msec_Numerator / ts.msec_Denominator, ( end - start ) );
        start = R_OSPL_FTIMER_Get();
#endif


        while ( R_DMAC_RM_GET_REGISTER_BIT_FIELD( regs_DMAC->CHSTAT_n, CHSTAT_n, TACT ) == 1u )
        {
            /* This loop will be exited soon. */
        }


#if  R_DMAC_MEASURE_CLREN_TIME
        end = R_OSPL_FTIMER_Get();
        printf( "%dmsec (%d)\n", ( end - start ) * ts.msec_Numerator / ts.msec_Denominator, ( end - start ) );
    }
#endif

    self->async_status.State = R_OSPL_RUNNABLE;

    e = 0;
fin:
    R_OSPL_FLAG32_Set( &self->async_status.CancelFlags, R_OSPL_CANNELED );
    if ( IS( was_enabled ) )
    {
        self->i_lock_v_table->Unlock( self->i_lock );
    }
    return  e;
}


/***********************************************************************
* Function: R_DMAC_RM_STATIC_GetDMA_BitCountSymbol
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_DMAC_RM_STATIC_GetDMA_BitCountSymbol( int_fast32_t const  bit_count,
        dmac_bit_count_t* const  out_symbol )
{
    errnum_t  e;

    static const int_fast32_t  num_1024 = 1024;  /* This is not magic number for QAC */
    static const int_fast32_t  num_512  =  512;
    static const int_fast32_t  num_256  =  256;
    static const int_fast32_t  num_128  =  128;
    static const int_fast32_t  num_64   =   64;
    static const int_fast32_t  num_32   =   32;
    static const int_fast32_t  num_16   =   16;
    static const int_fast32_t  num_8    =    8;


    IF_DQ( out_symbol == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }


    if ( bit_count == num_1024 )
    {
        *out_symbol = DMAC_1024_BIT;
    }
    else if ( bit_count == num_512 )
    {
        *out_symbol = DMAC_512_BIT;
    }
    else if ( bit_count == num_256 )
    {
        *out_symbol = DMAC_256_BIT;
    }
    else if ( bit_count == num_128 )
    {
        *out_symbol = DMAC_128_BIT;
    }
    else if ( bit_count == num_64 )
    {
        *out_symbol = DMAC_64_BIT;
    }
    else if ( bit_count == num_32 )
    {
        *out_symbol = DMAC_32_BIT;
    }
    else if ( bit_count == num_16 )
    {
        *out_symbol = DMAC_16_BIT;
    }
    else
    {
#if DMAC_PARAMETER_CHECK
        IF ( bit_count != num_8 )
        {
            e = E_OTHERS;
            goto  fin;
        }
#endif

        *out_symbol = DMAC_8_BIT;
    }

    e=0;
#if DMAC_PARAMETER_CHECK
fin:
#endif
    return  e;
}


/***********************************************************************
* Function: R_DMAC_RM_OnInterrupting
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_DMAC_RM_OnInterrupting( const r_ospl_interrupt_t* const  Interrupt )
{
    errnum_t      e;
    int_fast32_t  dmac_channel;
    uint32_t      value;
    r_dmac_rm_channel_t*  self;
#if IODEFINE_H_VERSION >= 60
    struct st_dmac_n*  regs_DMAC;
#else
    volatile struct st_dmac_n*  regs_DMAC;
#endif

    IF_DQ( Interrupt == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }

    dmac_channel = Interrupt->ChannelNum;
    self = &gs_dmac_rm_channel[ dmac_channel ];


    /* Set "regs_DMAC" */
    e= DMAC_RM_STATIC_GetRegisterSet( dmac_channel, &regs_DMAC );
    IF(e!=0)
    {
        goto fin;
    }
    IF_DQ( regs_DMAC == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }

    /* Get interrupt status */
    value = regs_DMAC->CHSTAT_n;  /* QAC 3345 */
    R_OSPL_FLAG32_Set( &self->async_status.InterruptFlags, value );

    /* Clear interrupt status */
    R_DMAC_RM_SET_REGISTER_BIT_FIELD( &regs_DMAC->CHCTRL_n, CHCTRL_n, CLREND, TO_UNSIGNED( true ) );
    R_DMAC_RM_SET_REGISTER_BIT_FIELD( &regs_DMAC->CHCTRL_n, CHCTRL_n, SWRST,  TO_UNSIGNED( true ) );

    /* Set state */
    self->async_status.State = R_OSPL_INTERRUPTED;

    e=0;
fin:
    return  e;
}


/***********************************************************************
* Function: R_DMAC_RM_OnInterrupted
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_DMAC_RM_OnInterrupted( int_fast32_t const  dmac_channel )
{
    errnum_t    e;
    uint32_t    interrupt_flag;
    bool_t      was_enabled = false;
    uint32_t    value;
    r_dmac_rm_channel_t* const  self = &gs_dmac_rm_channel[ dmac_channel ];
#if IODEFINE_H_VERSION >= 60
    struct st_dmac_n*                  regs_DMAC;
#else
    volatile struct st_dmac_n*         regs_DMAC;
#endif
    r_ospl_async_t*                    notify_async = NULL;

    IF ( (dmac_channel < 0)  ||  (dmac_channel >= R_DMAC_RM_CHANNEL_COUNT) )
    {
        e=E_OTHERS;
        goto fin;
    }

    /* Set "regs_DMAC" */
    e= DMAC_RM_STATIC_GetRegisterSet( dmac_channel, &regs_DMAC );
    IF(e!=0)
    {
        goto fin;
    }
    IF_DQ( regs_DMAC == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }

    was_enabled = self->i_lock_v_table->Lock( self->i_lock );


    interrupt_flag = R_OSPL_FLAG32_GetAndClear( &self->async_status.InterruptFlags );
    if ( interrupt_flag != 0u )
    {
        R_OSPL_FLAG32_Clear( &self->async_status.InterruptEnables, ~DMAC_INTERRUPT_LINE_TRANSFER_END );
        self->async_status.State = R_OSPL_RUNNABLE;
        notify_async = self->interrupt_callback_caller.Async;
    }


    /* Error check */
    value = R_DMAC_RM_GET_REGISTER_BIT_FIELD( interrupt_flag, CHSTAT_n, ER );
    IF ( IS( R_ToSigned( value ) ) )
    {
        e=E_OTHERS;
        goto fin;
    }

    e=0;
fin:
    if ( IS( was_enabled ) )
    {
        self->i_lock_v_table->Unlock( self->i_lock );
    }

    if ( notify_async != NULL )
    {
        R_OSPL_EVENT_Set( notify_async->A_Thread, notify_async->A_EventValue );
    }

    return  e;
}


/***********************************************************************
* Function: R_DMAC_RM_GetAsyncStatus
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
errnum_t  R_DMAC_RM_GetAsyncStatus( int_fast32_t const  dmac_channel,
                                    const dmac_rm_async_status_t** const  out_Status )
{
    errnum_t  e;
    r_dmac_rm_channel_t* const  self = &gs_dmac_rm_channel[ dmac_channel ];

    IF ( (dmac_channel < 0)  ||  (dmac_channel >= R_DMAC_RM_CHANNEL_COUNT) )
    {
        e=E_OTHERS;
        goto fin;
    }
    IF_DQ( out_Status == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }

    *out_Status = &self->async_status;

    e=0;
fin:
    return  e;
}


/***********************************************************************
* Function: R_DMAC_RM_EnableInterrupt
*
* Return Value:
*    Was interrupt enabled
************************************************************************/
void  R_DMAC_RM_EnableInterrupt( int_fast32_t const  dmac_channel )
{
    errnum_t  e;
    r_dmac_rm_channel_t* const  self = &gs_dmac_rm_channel[ dmac_channel ];

    IF ( (dmac_channel < 0)  ||  (dmac_channel >= R_DMAC_RM_CHANNEL_COUNT) )
    {
        e=E_OTHERS;
        goto fin;
    }

    self->async_status.IsEnabledInterrupt = true;
    R_DMAC_RM_OnEnableInterrupt( dmac_channel, self->async_status.InterruptEnables.Flags );

    e=0;
fin:
    R_UNREFERENCED_VARIABLE( e );
}


/***********************************************************************
* Function: R_DMAC_RM_DisableInterrupt
*
* Return Value:
*    Was interrupt enabled
************************************************************************/
bool_t  R_DMAC_RM_DisableInterrupt( int_fast32_t const  dmac_channel )
{
    errnum_t  e;
    bool_t    was_interrupted = false;
    r_dmac_rm_channel_t* const  self = &gs_dmac_rm_channel[ dmac_channel ];

    IF ( (dmac_channel < 0)  ||  (dmac_channel >= R_DMAC_RM_CHANNEL_COUNT) )
    {
        e=E_OTHERS;
        goto fin;
    }

    was_interrupted = self->async_status.IsEnabledInterrupt;

    R_DMAC_RM_OnDisableInterrupt( dmac_channel, DMAC_INTERRUPT_LINE_ALL );
    self->async_status.IsEnabledInterrupt = false;

    e=0;
fin:
    R_UNREFERENCED_VARIABLE( e );
    return  was_interrupted;
}


/***********************************************************************
* Function: R_DMAC_RM_I_LOCK_Replace
*
* Return Value:
*    None
************************************************************************/
bool_t  R_DMAC_RM_I_LOCK_Replace( int_fast32_t const  dmac_channel,
                                  void* const  i_lock,  const r_ospl_i_lock_vtable_t* const  i_lock_v_table )
{
    bool_t  ret = false;

    if ( (dmac_channel >= 0)  &&  (dmac_channel <= R_DMAC_RM_CHANNEL_COUNT) )
    {
        r_dmac_rm_channel_t* const  self = &gs_dmac_rm_channel[ dmac_channel ];

        if ( i_lock != NULL )
        {
            if ( self->i_lock == NULL )
            {
                self->i_lock = i_lock;
                self->i_lock_v_table = i_lock_v_table;
                ret = true;
            }
        }
        else
        {
            self->i_lock = NULL;
        }
    }

    return  ret;
}


/***********************************************************************
* Function: R_DMAC_RM_I_LOCK_Reset
*
* Return Value:
*    None
************************************************************************/
static void  R_DMAC_RM_I_LOCK_Reset( r_dmac_rm_i_lock_t* const  self_ )
{
    IF_DQ( self_ == NULL )
    {
        goto fin;
    }

    self_->is_lock = false;
    R_DMAC_RM_OnDisableInterrupt( self_->channel_num, DMAC_INTERRUPT_LINE_ALL );
    self_->is_requested_finalize = false;

fin:
    return;
}


/***********************************************************************
* Function: R_DMAC_RM_I_LOCK_Finalize
*
* Return Value:
*    Was interrupt enabled
************************************************************************/
static void  R_DMAC_RM_I_LOCK_Finalize( int_fast32_t const  dmac_channel )
{
    if ( (dmac_channel >= 0)  &&  (dmac_channel <= R_DMAC_RM_CHANNEL_COUNT) )
    {
        r_dmac_rm_channel_t* const  self = &gs_dmac_rm_channel[ dmac_channel ];
        bool_t                             b;

        ASSERT_D( self->is_i_lock_master, R_NOOP() );

        R_DMAC_RM_I_LOCK_Reset( self->i_lock );
        b= R_DMAC_RM_I_LOCK_Replace( dmac_channel, NULL, NULL );
        R_UNREFERENCED_VARIABLE( b );  /* QAC 3200 : This is not error information */
        self->is_i_lock_master = false;
    }
}


/***********************************************************************
* Function: R_DMAC_RM_I_LOCK_Lock
*
* Return Value:
*    Was interrupt enabled
************************************************************************/
static bool_t  R_DMAC_RM_I_LOCK_Lock( void* const  self_ )
{
    bool_t  is_locked;
    bool_t  was_all_enabled; /* = false; */ /* QAC 3197 */
    bool_t  b;
    r_dmac_rm_i_lock_t* const  self = (r_dmac_rm_i_lock_t*) self_;

    IF_DQ( self == NULL )
    {
        is_locked = true;
        goto fin;
    }

    was_all_enabled = R_OSPL_DisableAllInterrupt();

    is_locked = self->is_lock;
    if ( ! is_locked )
    {
        b= R_DMAC_RM_DisableInterrupt( self->channel_num );
        R_UNREFERENCED_VARIABLE( b );  /* QAC 3200 : This is not error information */
        self->is_lock = true;
    }

    if ( IS( was_all_enabled ) )
    {
        R_OSPL_EnableAllInterrupt();
    }

fin:
    return  ! is_locked;
}


/***********************************************************************
* Function: R_DMAC_RM_I_LOCK_Unlock
*
* Return Value:
*    None
************************************************************************/
static void  R_DMAC_RM_I_LOCK_Unlock( void* const  self_ )
{
    bool_t  was_all_enabled; /* = false; */ /* QAC 3197 */
    r_dmac_rm_i_lock_t* const  self = (r_dmac_rm_i_lock_t*) self_;

    IF_DQ( self == NULL )
    {
        goto fin;
    }

    was_all_enabled = R_OSPL_DisableAllInterrupt();

    ASSERT_D( self->channel_num >= 0  &&  self->channel_num < R_DMAC_RM_CHANNEL_COUNT,  R_NOOP() );

    R_DMAC_RM_EnableInterrupt( self->channel_num );
    self->is_lock = false;

    if ( IS( was_all_enabled ) )
    {
        R_OSPL_EnableAllInterrupt();
    }

fin:
    return;
}


/***********************************************************************
* Function: R_OSPL_EXAMPLE_I_LOCK_Unlock
*
* Return Value:
*    None
************************************************************************/
static void   R_DMAC_RM_I_LOCK_RequestFinalize( void* const  self_ )
{
    r_dmac_rm_i_lock_t* const  self = (r_dmac_rm_i_lock_t*) self_;

    IF_DQ( self == NULL )
    {
        goto fin;
    }

    self->is_requested_finalize = true;

fin:
    return;
}


/***********************************************************************
* Function: R_DMAC_RM_I_LOCK_GetRootChannelNum
*
* Return Value:
*    None
************************************************************************/
static int_fast32_t  R_DMAC_RM_I_LOCK_GetRootChannelNum( const void* const  self_ )
{
    int_fast32_t  root_channel_num;
    r_dmac_rm_i_lock_t* const  self = (r_dmac_rm_i_lock_t*) self_;

    IF_DQ( self == NULL )
    {
        root_channel_num = 0;
        goto fin;
    }

    root_channel_num = self->channel_num;

fin:
    return  root_channel_num;
}


