/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2015 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* File: r_pfv_user.c
*    Pixel Format Converter (PFV) - Porting Layer.
*
* - $Version: $ (=PFV_VERSION)
* - $ModuleRevision: {Masters}\PFV_DMAC\PFV_DMAC\23_RStyled $
* - $Rev: $
* - $Date: 2017-08-24T18:32:46+09:00 $
*******************************************************************************/


/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include  "r_ospl.h"
#include  "r_pfv.h"
#include  "r_pfv_internal.h"
#ifndef R_OSPL_NDEBUG
#include  <stdio.h>
#include  <inttypes.h>
#endif


/******************************************************************************
Typedef definitions
******************************************************************************/

/******************************************************************************
Macro definitions
******************************************************************************/

/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/

/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/

/******************************************************************************
Private global variables and functions
******************************************************************************/

/******************************************************************************
* Function Name: [R_PFV_STATIC_PrintRegisters]
* Description  :
* Arguments    :
* Return Value : Error code, 0=No error
******************************************************************************/
#ifndef R_OSPL_NDEBUG
errnum_t  R_PFV_STATIC_PrintRegisters( int_fast32_t  pfv_channel )
{
    errnum_t  e;
    volatile struct st_pfv*  reg;  /* register */

    e= R_PFV_STATIC_GetRegisterSet( pfv_channel, &reg );
    IF(e!=0)
    {
        goto fin;
    }
    IF_DQ( reg == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }


    /* Print all register's values */
    printf( "\n" );
    printf( "PFV[%d]:\n", pfv_channel );
    printf( "    PFVCR        = 0x%08" PRIX32 "   \"Conrtol Register\"\n", reg->PFVCR );
    printf( "    PFVICR       = 0x%08" PRIX32 "   \"Interrupt Control Register\"\n", reg->PFVICR );
    printf( "    PFVISR       = 0x%08" PRIX32 "   \"Interrupt Status Register\"\n", reg->PFVISR );
    printf( "    PFVIFSR      = 0x%08" PRIX32 "   \"Input FIFO Status Register\"\n", reg->PFVIFSR );
    printf( "    PFVOFSR      = 0x%08" PRIX32 "   \"Output FIFO Status Register\"\n", reg->PFVOFSR );
    printf( "    PFVACR       = 0x%08" PRIX32 "   \"Alpha Color Register\"\n", reg->PFVACR );
    printf( "    PFV_MTX_MODE = 0x%08" PRIX32 "   \"Matrix Mode Register\"\n", reg->PFV_MTX_MODE );
    printf( "    PFV_MTX_YG_ADJ0 = 0x%08" PRIX32 "\n", reg->PFV_MTX_YG_ADJ0 );
    printf( "    PFV_MTX_YG_ADJ1 = 0x%08" PRIX32 "\n", reg->PFV_MTX_YG_ADJ1 );
    printf( "    PFV_MTX_CBB_ADJ0 = 0x%08" PRIX32 "\n", reg->PFV_MTX_CBB_ADJ0 );
    printf( "    PFV_MTX_CBB_ADJ1 = 0x%08" PRIX32 "\n", reg->PFV_MTX_CBB_ADJ1 );
    printf( "    PFV_MTX_CRR_ADJ0 = 0x%08" PRIX32 "\n", reg->PFV_MTX_CRR_ADJ0 );
    printf( "    PFV_MTX_CRR_ADJ1 = 0x%08" PRIX32 "\n", reg->PFV_MTX_CRR_ADJ1 );
    printf( "    PFVSZR       = 0x%08" PRIX32 "\n", reg->PFVSZR );

    e = 0;
fin:
    return  e;
}
#endif


