/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2014 - 2015 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/******************************************************************************
* File: window_surfaces.hpp
*    Window Surfaces API for mbed style C++ API
*
* - $Version: $ (=WINDOW_SURFACES_VERSION)
* - $ModuleRevision: {Masters}\WindowSurfaces\RZ_NoEx\10_RStyle $
* - $Rev: $
* - $Date: 2017-06-23T14:44:17+09:00 $
******************************************************************************/

#ifndef  WINDOW_SURFACES_HPP
#define  WINDOW_SURFACES_HPP

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include  "r_ospl_typedef.h"
#include  "window_surfaces_typedef.h"


/******************************************************************************
Macro definitions
******************************************************************************/

/******************************************************************************
Typedef definitions
******************************************************************************/

class  WindowSurfacesConfigClass;


/***********************************************************************
* Class: WindowSurfacesClass
*    mbed style C++ version of <window_surfaces_t>
************************************************************************/
class  WindowSurfacesClass {
public:
	/* Function: WindowSurfacesClass */
	WindowSurfacesClass();
	~WindowSurfacesClass();

	/* Function: initialize */
	errnum_t  initialize( WindowSurfacesConfigClass& in_out_config );

	/* Function: destroy */
	void  destroy();

	/* Function: get_layer_frame_buffer */
	errnum_t  get_layer_frame_buffer( int_fast32_t const  layer_num,  frame_buffer_t** const  out_frame_buffer );

	/* Function: swap_buffers */
	errnum_t  swap_buffers( int_fast32_t const  layer_num,  const void* const  null_context );

	/* Function: swap_buffers_start */
	errnum_t  swap_buffers_start( int_fast32_t const  layer_num,  r_ospl_async_t* async );

	/* Function: wait_for_v_sync */
	errnum_t  wait_for_v_sync( int_fast32_t const  swap_interval,  bool_t const  is_1_v_sync_at_minimum );

	/* Function: alloc_offscreen_stack */
	errnum_t  alloc_offscreen_stack( frame_buffer_t* const  in_out_frame_buffer );

	/* Function: free_offscreen_stack */
	errnum_t  free_offscreen_stack( const frame_buffer_t* const  frame_buffer );

private:
	window_surfaces_t*  _self;
};


/***********************************************************************
* Class: WindowSurfacesConfigClass
*    mbed style C++ version of <window_surfaces_config_t>
************************************************************************/
class  WindowSurfacesConfigClass : public window_surfaces_config_t {
public:
	/* Function: WindowSurfacesConfigClass */
	WindowSurfacesConfigClass();
};


/******************************************************************************
Variable Externs
******************************************************************************/

/******************************************************************************
Functions Prototypes
******************************************************************************/

#endif  /* WINDOW_SURFACES_HPP */
