/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2017 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/******************************************************************************
* File: window_surfaces.c
*    Window Surfaces API
*
* - $Version: $ (=WINDOW_SURFACES_VERSION)
* - $ModuleRevision: {Masters}\WindowSurfaces\RZ_NoEx\10_RStyle $
* - $Rev: $
* - $Date: 2017-07-21T19:28:37+09:00 $
******************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
*******************************************************************************/
#include  "use_list.h"
#ifdef  USE_LIST_WINDOW_SURFACES
#include  "mcu_board_select.h"
#include  "r_typedefs.h"
#include  "lcd_panel.h"
#ifdef  USE_LIST_RZA1H_BSP
#include  "posix_types.h"
#endif
#include  "r_ospl.h"
#include  "clib_registers.h"
#include  "clib_drivers.h"
#include  "window_surfaces.h"
#include  "frame_buffer.h"
#include  "vsync.h"
#include  "VRAM.h"
#ifndef R_OSPL_NDEBUG
#include  <stdio.h>
#endif
#if  USE_LCD_BACK_LIGHT
#include  "lcd_if.h"
#endif


/******************************************************************************
Typedef definitions
******************************************************************************/

/******************************************************************************
Macro definitions
******************************************************************************/

/* GS_ : global static */
/* VDC5_CHANNEL_0 = LCD_VDC5_CH0_PANEL = ... */
/* VDC5_CHANNEL_1 = LCD_VDC5_CH1_PANEL = ... */
#if WINDOW_SURFACES_DEFAULT_CHANNEL
#define   GS_CHANNEL_n              VDC5_CHANNEL_1
#define   GS_LCD_CHn_SIG_FV         LCD_CH1_SIG_FV
#define   GS_LCD_CHn_SIG_FH         LCD_CH1_SIG_FH
#define   GS_LCD_CHn_DISP_VS        LCD_CH1_DISP_VS
#define   GS_LCD_CHn_DISP_VW        LCD_CH1_DISP_VW
#define   GS_LCD_CHn_DISP_HS        LCD_CH1_DISP_HS
#define   GS_LCD_CHn_DISP_HW        LCD_CH1_DISP_HW
#define   GS_LCD_CHn_TCON_HALF      LCD_CH1_TCON_HALF
#define   GS_LCD_CHn_TCON_OFFSET    LCD_CH1_TCON_OFFSET
#define   GS_LCD_CHn_OUT_EDGE       LCD_CH1_OUT_EDGE
#define   GS_LCD_CHn_OUT_FORMAT     LCD_CH1_OUT_FORMAT
#define   GS_LCD_CHn_PANEL_CLK      LCD_CH1_PANEL_CLK
#define   GS_LCD_CHn_PANEL_CLK_DIV  LCD_CH1_PANEL_CLK_DIV
#define   GS_DISPLAY_CH             1   /* 1 or 0 */
#else
#define   GS_CHANNEL_n              VDC5_CHANNEL_0
#define   GS_LCD_CHn_SIG_FV         LCD_CH0_SIG_FV
#define   GS_LCD_CHn_SIG_FH         LCD_CH0_SIG_FH
#define   GS_LCD_CHn_DISP_VS        LCD_CH0_DISP_VS
#define   GS_LCD_CHn_DISP_VW        LCD_CH0_DISP_VW
#define   GS_LCD_CHn_DISP_HS        LCD_CH0_DISP_HS
#define   GS_LCD_CHn_DISP_HW        LCD_CH0_DISP_HW
#define   GS_LCD_CHn_TCON_HALF      LCD_CH0_TCON_HALF
#define   GS_LCD_CHn_TCON_OFFSET    LCD_CH0_TCON_OFFSET
#define   GS_LCD_CHn_OUT_EDGE       LCD_CH0_OUT_EDGE
#define   GS_LCD_CHn_OUT_FORMAT     LCD_CH0_OUT_FORMAT
#define   GS_LCD_CHn_PANEL_CLK      LCD_CH0_PANEL_CLK
#define   GS_LCD_CHn_PANEL_CLK_DIV  LCD_CH0_PANEL_CLK_DIV
#define   GS_DISPLAY_CH             0   /* 1 or 0 */
#endif

#if  1
#define   GS_BUFFER_WIDTH           800
#define   GS_BUFFER_HEIGHT          480
#else
#define   GS_BUFFER_WIDTH           GS_LCD_CHn_DISP_HW
#define   GS_BUFFER_HEIGHT          GS_LCD_CHn_DISP_VW
#endif

#define   GS_MEASURE_GPU_LOAD       0

#define  GS_DEFAULT_CLEAR_COLOR  R_RGA_Get_R8G8B8A8( 0, 0, 0, 0 )

#if  TARGET_RZA1 == TARGET_RZA1L  ||  TARGET_RZA1 == TARGET_RZA1LU  ||  TARGET_RZA1 == TARGET_RZA1LC
#define  GRAPHICS_GetLvdsParam( ch )                 NULL
#define  GRAPHICS_SetLcdTconSettings( ch, setting )  GRAPHICS_L_SetLcdTconSettings( setting )
#define  GRAPHICS_SetLcdPanel( ch )                  GRAPHICS_L_SetLcdPanel()
#endif

/***********************************************************************
* Constant: GS_OFFSET_BYTE_NOT_SHOW
************************************************************************/
enum { GS_OFFSET_BYTE_NOT_SHOW = -1 };


/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/

/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/

/******************************************************************************
Private global variables and functions
******************************************************************************/

static errnum_t  R_WINDOW_SURFACES_SwapBuffers_Sub( window_surfaces_t* const  self,
        int_fast32_t const  layer_num );


/***********************************************************************
* Variable: gs_window_surfaces_vdc5_is_init
*
* Description:
*    This is for GS_CHANNEL_n only.
************************************************************************/
static bool_t  gs_window_surfaces_vdc5_is_init = false;


/***********************************************************************
* Function: alloc_VRAM_stack_sub
*    alloc_VRAM_stack_sub
*
* Arguments:
*    in_out_StackPointer - in_out_StackPointer
*    OverOfVRAM          - OverOfVRAM
*    in_out_FrameBuffer  - in_out_FrameBuffer
*
* Return Value:
*    Error code, 0=No error
*
* Description:
*    (input) ->stride, ->height, ->buffer_count
*    (output) ->buffer_address[(all)]
************************************************************************/
static errnum_t  alloc_VRAM_stack_sub( uint8_t** const  in_out_StackPointer,
                                       const uint8_t* const  OverOfVRAM,
                                       frame_buffer_t* const  in_out_FrameBuffer ); /* QAC-3450 */
static errnum_t  alloc_VRAM_stack_sub( uint8_t** const  in_out_StackPointer,
                                       const uint8_t* const  OverOfVRAM,
                                       frame_buffer_t* const  in_out_FrameBuffer )
{
    errnum_t      e;
    int_fast32_t  size_1;
    int_fast32_t  size_all;
    int_fast32_t  buffer_num;
    uint8_t*      stack_pointer;


    IF_DQ( in_out_StackPointer == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }
    IF_DQ( in_out_FrameBuffer == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }


    stack_pointer = *in_out_StackPointer;


    /* Set "size_1" */
    size_1 = in_out_FrameBuffer->stride * in_out_FrameBuffer->height;
    size_1 = ( R_Ceil_64s( size_1 ) );


    /* Set "size_all" */
    size_all = in_out_FrameBuffer->buffer_count * size_1;


    /* Check */
    /* ->MISRA 17.4 */ /* ->SEC R1.3.1 (1) */
    IF ( (stack_pointer + size_all) > OverOfVRAM )
    {
        in_out_FrameBuffer->buffer_count = 0;
        e=E_FEW_ARRAY;
        goto fin;
    }
    /* <-MISRA 17.4 */ /* <-SEC R1.3.1 (1) */

    IF ( in_out_FrameBuffer->buffer_count >
         (int_fast32_t) R_COUNT_OF( in_out_FrameBuffer->buffer_address ) )
    {
        e=E_OTHERS;
        goto fin;
    }


    /* Set "in_out_FrameBuffer->buffer_address" */
    for ( buffer_num = 0;
            buffer_num < in_out_FrameBuffer->buffer_count;
            buffer_num += 1 )
    {
        in_out_FrameBuffer->buffer_address[ buffer_num ] = stack_pointer;

        /* ->MISRA 17.4 */ /* ->SEC R1.3.1 (1) */
        stack_pointer += size_1;  /* MISRA 17.4: Bound check is done by "OverOfVRAM" */
        /* <-MISRA 17.4 */ /* <-SEC R1.3.1 (1) */
    }
    for ( /* buffer_num */;
                          buffer_num < (int_fast32_t) R_COUNT_OF( in_out_FrameBuffer->buffer_address );
                          buffer_num += 1 )
    {
        in_out_FrameBuffer->buffer_address[ buffer_num ] = NULL;
    }

    *in_out_StackPointer = stack_pointer;

    e=0;
fin:
    return  e;
}


/***********************************************************************
* Function: free_VRAM_stack_sub
*    free_VRAM_stack_sub
*
* Arguments:
*    in_out_StackPointer - in_out_StackPointer
*    frame_buffer        - frame_buffer
*    StartOfVRAM         - StartOfVRAM
*
* Return Value:
*    Error code, 0=No error
*
* Description:
*    If frame_buffer->buffer_count == 0, do nothing.
************************************************************************/
static errnum_t  free_VRAM_stack_sub( uint8_t** const  in_out_StackPointer,
                                      const frame_buffer_t* const  frame_buffer,
                                      const uint8_t* const  StartOfVRAM ); /* QAC-3450 */
static errnum_t  free_VRAM_stack_sub( uint8_t** const  in_out_StackPointer,
                                      const frame_buffer_t* const  frame_buffer,
                                      const uint8_t* const  StartOfVRAM )
{
    errnum_t      e;
    int_fast32_t  size_1;
    int_fast32_t  size_all;
    int_fast32_t  buffer_num;
    uint8_t*      next_stack_pointer;


    IF_DQ( frame_buffer == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }
    IF_DQ( in_out_StackPointer == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }


    /* Set "size_1" */
    size_1 = frame_buffer->stride * frame_buffer->height;
    size_1 = ( R_Ceil_64s( size_1 ) );


    /* Set "size_all" */
    size_all = frame_buffer->buffer_count * size_1;


    /* Set "next_stack_pointer" */
    /* ->MISRA 17.4 */ /* ->SEC R1.3.1 (1) */
    next_stack_pointer = *in_out_StackPointer - size_all;
    /* MISRA 17.4: Bound check is done by "StartOfVRAM" */
    /* <-MISRA 17.4 */ /* <-SEC R1.3.1 (1) */


    /* Check */
    IF ( next_stack_pointer < StartOfVRAM )
    {
        e=E_OTHERS;
        goto fin;
    }

    IF ( frame_buffer->buffer_count > (int_fast32_t) R_COUNT_OF( frame_buffer->buffer_address ) )
    {
        e=E_OTHERS;
        goto fin;
    }

    for ( buffer_num = frame_buffer->buffer_count - 1;  buffer_num >= 0;  buffer_num -= 1 )
    {
        /* ->MISRA 17.4 */ /* ->SEC R1.3.1 (1) */
        IF ( frame_buffer->buffer_address[ buffer_num ] !=
             (next_stack_pointer + (size_1 * buffer_num)) )
        {
            e=E_ACCESS_DENIED;
            goto fin;
        }
        /* MISRA 17.4: Bound check is done by "size_all" */
        /* <-MISRA 17.4 */ /* <-SEC R1.3.1 (1) */
    }


    /* Set "*in_out_StackPointer" */
    *in_out_StackPointer = next_stack_pointer;

    e=0;
fin:
    return  e;
}


/*-------------------------------------------------------------------------*/
/* <<<< ### (window_surfaces_vdc5_layer_t) Class implement >>>> */
/*-------------------------------------------------------------------------*/

/***********************************************************************
* Function: window_surfaces_vdc5_layer_t__initialize
*   Initialize <window_surfaces_vdc5_layer_t>
*
* Arguments:
*   self              - window_surfaces_vdc5_layer_t
*   layer_num         - layer_num
*   graphics_layer_ID - graphics_layer_ID
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
static errnum_t  window_surfaces_vdc5_layer_t__initialize(
    window_surfaces_vdc5_layer_t* const  self,
    int_fast32_t const  layer_num,
    vdc5_graphics_type_t const  graphics_layer_ID ); /* QAC-3450 */

static errnum_t  window_surfaces_vdc5_layer_t__initialize(
    window_surfaces_vdc5_layer_t* const  self,
    int_fast32_t const  layer_num,
    vdc5_graphics_type_t const  graphics_layer_ID )
{
    errnum_t  e;

    IF_DQ( self == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }

    e= R_int_t_to_int8_t( layer_num, &self->layer_num );
    IF(e!=0)
    {
        goto fin;
    }

    self->graphics_layer_ID = graphics_layer_ID;

    self->is_data_control = false;

    self->layer_color = GS_DEFAULT_CLEAR_COLOR;

    switch ( graphics_layer_ID )
    {
    case  VDC5_GR_TYPE_GR0:
        self->data_control_ID = VDC5_LAYER_ID_0_RD;
        break;
#if  TARGET_RZA1 == TARGET_RZA1H  ||  TARGET_RZA1 == TARGET_RZA1M
    case  VDC5_GR_TYPE_GR1:
        self->data_control_ID = VDC5_LAYER_ID_1_RD;
        break;
#endif
    case  VDC5_GR_TYPE_GR2:
        self->data_control_ID = VDC5_LAYER_ID_2_RD;
        break;
    case  VDC5_GR_TYPE_GR3:
        self->data_control_ID = VDC5_LAYER_ID_3_RD;
        break;
    default:
        e=E_OTHERS;
        goto fin; /* Bad "graphics_layer_ID" */
    }

    e=0;
fin:
    return  e;
}



/*-------------------------------------------------------------------------*/
/* <<<< ### (window_surfaces_t) Class implement >>>> */
/*-------------------------------------------------------------------------*/

/***********************************************************************
* Function: window_surfaces_t__get_layer
*   Get <window_surfaces_vdc5_layer_t>
*
* Arguments:
*   self        - window_surfaces_t
*   in_LayerNum - in_LayerNum
*   out_Layer   - out_Layer
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
STATIC_INLINE errnum_t  window_surfaces_t__get_layer( window_surfaces_t* const  self,
        int_fast32_t const  in_LayerNum,
        window_surfaces_vdc5_layer_t** const  out_Layer );  /* QAC-3450 */
/* ->MISRA 16.7 : "self" can not "const", because "out_Layer" is not "const" */
/* ->SEC M1.11.1 */
STATIC_INLINE errnum_t  window_surfaces_t__get_layer( window_surfaces_t* const  self,
        int_fast32_t const  in_LayerNum,
        window_surfaces_vdc5_layer_t** const  out_Layer )
/* <-MISRA 16.7 */ /* <-SEC M1.11.1 */
{
    errnum_t       e;
    uint_fast32_t  index;

    IF_DQ( self == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }
    IF_DQ( out_Layer == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }

    index = (uint_fast32_t)( in_LayerNum - self->layer_num_min );

    IF ( index >= R_COUNT_OF( self->layers ) )
    {
        e = E_OTHERS;
        goto fin;
    }

    *out_Layer = &self->layers[ index ];

    e=0;
fin:
    return  e;
}


/***********************************************************************
* Class: window_surfaces_t
************************************************************************/

/***********************************************************************
* Implement: R_WINDOW_SURFACES_InitConst
************************************************************************/
void  R_WINDOW_SURFACES_InitConst( window_surfaces_t* const  self )
{
    int_fast32_t  i;

    IF_DQ( self == NULL )
    {
        goto fin;
    }

    for ( i = 0;  i < (ssize_t) R_COUNT_OF( self->frame_buffers );  i += 1 )
    {
        self->frame_buffers[i].buffer_address[0] = NULL;
    }
    self->is_initialized = false;

fin:
    return;
}


/***********************************************************************
* Implement: R_WINDOW_SURFACES_Initialize
************************************************************************/

static void  vdc5_init_func( uint32_t user_num );

errnum_t  R_WINDOW_SURFACES_Initialize( window_surfaces_t* const  self,
                                        window_surfaces_config_t*  in_out_config )
{
    errnum_t                  e;
    errnum_t                  ee;
    vdc5_error_t              error_vdc;
    window_surfaces_config_t  default_config;
    int_fast32_t              layer_num;
    window_surfaces_vdc5_layer_t*  main_layer[ R_COUNT_OF( self->frame_buffers ) ];
    window_surfaces_vdc5_layer_t*  back_layer;

    enum { num_2 = 2,  num_4 = 4 };  /* This avoids warning of magic number */


    IF_DQ( self == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }

    self->screen_channel = GS_CHANNEL_n;


    /* Set default configuration */
    if ( in_out_config == NULL )
    {
        in_out_config = &default_config;
        in_out_config->flags = 0;
    }

    if ( IS_BIT_NOT_SET( in_out_config->flags, F_WINDOW_SURFACES_PIXEL_FORMAT ) )
    {
        in_out_config->pixel_format = PIXEL_FORMAT_ARGB8888;
        in_out_config->flags |= F_WINDOW_SURFACES_PIXEL_FORMAT;
    }


    if ( IS_BIT_NOT_SET( in_out_config->flags, F_WINDOW_SURFACES_LAYER_COUNT ) )
    {
        in_out_config->layer_count = 1;
        in_out_config->flags |= F_WINDOW_SURFACES_LAYER_COUNT;
    }
    ASSERT_D( in_out_config->layer_count >= 1  &&  in_out_config->layer_count <= 2,
              e=E_OTHERS; goto fin );


    if ( IS_BIT_NOT_SET( in_out_config->flags, F_WINDOW_SURFACES_BUFFER_HEIGHT ) )
    {
        in_out_config->buffer_height = R_ToSigned( GS_BUFFER_HEIGHT );
        in_out_config->flags |= F_WINDOW_SURFACES_BUFFER_HEIGHT;
    }


    if ( IS_BIT_NOT_SET( in_out_config->flags, F_WINDOW_SURFACES_BACKGROUND_COLOR ) )
    {
        in_out_config->background_color = GS_DEFAULT_CLEAR_COLOR;
        in_out_config->flags |= F_WINDOW_SURFACES_BACKGROUND_COLOR;
    }


    if ( IS_BIT_NOT_SET( in_out_config->flags, F_WINDOW_SURFACES_BACKGROUND_FORMAT ) )
    {
        in_out_config->background_format = BACKGROUND_FORMAT_SOLID_COLOR;
        in_out_config->flags |= F_WINDOW_SURFACES_BACKGROUND_FORMAT;
    }


    if ( IS_BIT_NOT_SET( in_out_config->flags, F_WINDOW_SURFACES_BUFFER_COUNT_MAX ) )
    {
        in_out_config->buffer_count_max = num_2;
        in_out_config->flags |= F_WINDOW_SURFACES_BUFFER_COUNT_MAX;
    }
    ASSERT_D( in_out_config->buffer_count_max >= 1  &&  in_out_config->buffer_count_max <= num_2,
              e=E_OTHERS; goto fin );


    if ( IS_BIT_NOT_SET( in_out_config->flags, F_WINDOW_SURFACES_SHARED_STACK_IN_VRAM ) )
    {
        in_out_config->shared_stack_in_VRAM = &self->stack_in_VRAM_body;
        in_out_config->flags |= F_WINDOW_SURFACES_SHARED_STACK_IN_VRAM;
    }
    self->stack_in_VRAM = in_out_config->shared_stack_in_VRAM;


    /* Set layers */
#if  TARGET_RZA1 == TARGET_RZA1H  ||  TARGET_RZA1 == TARGET_RZA1M
    self->layer_num_min = -1;
    self->layer_num_max =  (int8_t)( in_out_config->layer_count - 1 );
    ASSERT_D( (ssize_t) R_COUNT_OF( self->layers ) >= self->layer_num_max - self->layer_num_min + 1,
              e=E_OTHERS; goto fin );
    e= window_surfaces_vdc5_layer_t__initialize( &self->layers[0], -1, VDC5_GR_TYPE_GR0 );
    IF(e!=0)
    {
        goto fin;
    }
    e= window_surfaces_vdc5_layer_t__initialize( &self->layers[1],  0, VDC5_GR_TYPE_GR1 );
    IF(e!=0)
    {
        goto fin;
    }
    e= window_surfaces_vdc5_layer_t__initialize( &self->layers[2],  1, VDC5_GR_TYPE_GR2 );
    IF(e!=0)
    {
        goto fin;
    }
#else
    if ( in_out_config->pixel_format != PIXEL_FORMAT_YUV422 )
    {
        self->layer_num_min = -1;
        self->layer_num_max =  in_out_config->layer_count - 1;
        ASSERT_D( R_COUNT_OF( self->layers ) >= self->layer_num_max - self->layer_num_min + 1,
                  e=E_OTHERS; goto fin );
        e= window_surfaces_vdc5_layer_t__initialize( &self->layers[0], -1, VDC5_GR_TYPE_GR0 );
        IF(e!=0)
        {
            goto fin;
        }
        e= window_surfaces_vdc5_layer_t__initialize( &self->layers[1],  0, VDC5_GR_TYPE_GR2 );
        IF(e!=0)
        {
            goto fin;
        }
        e= window_surfaces_vdc5_layer_t__initialize( &self->layers[2],  1, VDC5_GR_TYPE_GR3 );
        IF(e!=0)
        {
            goto fin;
        }
    }
    else   /* PIXEL_FORMAT_YUV422 */
    {
        self->layer_num_min = 0;
        self->layer_num_max = in_out_config->layer_count - 1;
        ASSERT_D( R_COUNT_OF( self->layers ) >= self->layer_num_max - self->layer_num_min + 1,
                  e=E_OTHERS; goto fin );
        e= window_surfaces_vdc5_layer_t__initialize( &self->layers[0],  0, VDC5_GR_TYPE_GR0 );
        IF(e!=0)
        {
            goto fin;
        }
        e= window_surfaces_vdc5_layer_t__initialize( &self->layers[1],  1, VDC5_GR_TYPE_GR2 );
        IF(e!=0)
        {
            goto fin;
        }
    }
#endif


    /* Set "main_layer", "back_layer" */
    for ( layer_num = 0;  layer_num <= self->layer_num_max;  layer_num += 1 )
    {
        e= window_surfaces_t__get_layer( self,  layer_num,  &main_layer[ layer_num ] );
        IF(e!=0)
        {
            goto fin;
        }
        IF_DQ( main_layer[ layer_num ] == NULL )
        {
            e=E_OTHERS;
            goto fin;
        }
    }

    self->background_format = in_out_config->background_format;
    if ( self->layer_num_min >= 0 )
    {
        back_layer = NULL;
        self->background_frame_count = 0;
    }
    else
    {
        back_layer = NULL;
        /* NULL is for avoiding warning C417W of mbed cloud compiler */
        e= window_surfaces_t__get_layer( self, -1, &back_layer );
        IF(e!=0)
        {
            goto fin;
        }
        IF_DQ( back_layer == NULL )
        {
            e=E_OTHERS;
            goto fin;
        }

        if ( in_out_config->background_format != BACKGROUND_FORMAT_SOLID_COLOR )
        {
            self->background_frame_count = 1;
        }
        else
        {
            self->background_frame_count = 0;
        }
    }


    /* Set "self->stack_in_VRAM_body" */
    if ( self->stack_in_VRAM == &self->stack_in_VRAM_body )
    {
        uint8_t*  memory_address;
        size_t    memory_size;

        vram_stack_config_t  config;


        /* Set "physical_address", ... */
        e= R_SAMPLE_GetVRAM( &memory_address,  &memory_size );
        IF(e)
        {
            goto fin;
        }

        /* 64byte alignment */
        ASSERT_R( R_Mod_64u( (uintptr_t) memory_address ) == 0,  e=E_OTHERS; goto fin );

        /* Set "self->stack_in_VRAM_body" */
        config.flags = F_VRAM_STACK_ADDRESS | F_VRAM_STACK_SIZE;
        config.address = memory_address;
        config.size = memory_size;

        e= R_VRAM_STACK_Initialize( &self->stack_in_VRAM_body,  &config );
        IF(e)
        {
            goto fin;
        }
    }


    for ( layer_num = -self->background_frame_count;  layer_num <= self->layer_num_max;  layer_num += 1 )
    {
        frame_buffer_t*  frame;
        window_surfaces_vdc5_layer_t*  layer = main_layer[ layer_num ];
        pixel_format_t  pixel_format;

        e= R_WINDOW_SURFACES_GetLayerFrameBuffer( self, layer_num, &frame );
        IF(e)
        {
            goto fin;
        }

        if ( layer_num <= -1  &&  in_out_config->background_format != BACKGROUND_FORMAT_SOLID_COLOR )
        {
            pixel_format = PIXEL_FORMAT_YUV422;
        }
        else if ( layer_num >= 1  &&  in_out_config->pixel_format == PIXEL_FORMAT_YUV422 )
        {
            pixel_format = PIXEL_FORMAT_ARGB4444;
        }
        else
        {
            pixel_format = in_out_config->pixel_format;
        }


        /* Set "self->frame_buffers" */
        frame->buffer_count      = num_2;
        frame->show_buffer_index = 0;
        frame->draw_buffer_index = 0;
        frame->width             = R_ToSigned( GS_BUFFER_WIDTH );
        frame->height            = in_out_config->buffer_height;
        frame->pixel_format      = pixel_format;
        frame->delegate          = NULL;

        switch ( frame->pixel_format )
        {

        case PIXEL_FORMAT_ARGB8888:
            frame->buffer_count      = 1;
            frame->draw_buffer_index = 0;
            frame->byte_per_pixel    = num_4;
            layer->vdc5_format = VDC5_GR_FORMAT_ARGB8888;
            break;

        case PIXEL_FORMAT_XRGB8888:
            frame->buffer_count      = 1;
            frame->draw_buffer_index = 0;
            frame->byte_per_pixel    = num_4;
            layer->vdc5_format = VDC5_GR_FORMAT_RGB888;
            break;

        case PIXEL_FORMAT_RGB565:
            frame->buffer_count      = num_2;
            frame->draw_buffer_index = 1;
            frame->byte_per_pixel    = num_2;
            layer->vdc5_format = VDC5_GR_FORMAT_RGB565;
            break;

        case PIXEL_FORMAT_ARGB1555:
            frame->buffer_count      = num_2;
            frame->draw_buffer_index = 1;
            frame->byte_per_pixel    = num_2;
            layer->vdc5_format = VDC5_GR_FORMAT_ARGB1555;
            break;

        case PIXEL_FORMAT_ARGB4444:
            frame->buffer_count      = num_2;
            frame->draw_buffer_index = 1;
            frame->byte_per_pixel    = num_2;
            layer->vdc5_format = VDC5_GR_FORMAT_ARGB4444;
            break;

        case PIXEL_FORMAT_YUV422:
            frame->buffer_count      = num_2;
            frame->draw_buffer_index = 1;
            frame->byte_per_pixel    = num_2;
            layer->vdc5_format = VDC5_GR_FORMAT_YCBCR422;
            break;

        default:
            layer->vdc5_format = VDC5_GR_FORMAT_NUM;  /* dummy data */
            break;
        }
        if ( frame->buffer_count > in_out_config->buffer_count_max )
        {
            ASSERT_D( in_out_config->buffer_count_max == 1,  e=E_OTHERS; goto fin );
            frame->buffer_count      = 1;
            frame->draw_buffer_index = 0;
        }
        frame->stride = frame->width * frame->byte_per_pixel;
        e= R_WINDOW_SURFACES_AllocOffscreenStack( self, frame );
        IF(e!=0)
        {
            goto fin;
        }

#ifndef R_OSPL_NDEBUG
        printf( "Screen %dx%dx%dx%d vdc5_format=%d stride=%d \n address[0]=0x%08X address[1]=0x%08X\n",
                frame->buffer_count, frame->width, frame->height, frame->byte_per_pixel,
                layer->vdc5_format, frame->stride,
                (uintptr_t) frame->buffer_address[0], (uintptr_t) frame->buffer_address[1] );
        /* Cast of "uintptr_t" is for avoiding "format" warning of GNU_ARM */
#endif
    }


    if ( ! gs_window_surfaces_vdc5_is_init )
    {

        /* Call "R_VDC5_Initialize" */
        {
            vdc5_init_t  init;

            init.panel_icksel = GS_LCD_CHn_PANEL_CLK;
            init.panel_dcdr   = GS_LCD_CHn_PANEL_CLK_DIV;
            init.lvds         = GRAPHICS_GetLvdsParam( self->screen_channel );

            error_vdc = R_VDC5_Initialize( self->screen_channel, &init,
                                           &vdc5_init_func, (uint32_t) self->screen_channel );
            IF ( error_vdc != VDC5_OK )
            {
                e = E_OTHERS;
                goto fin;
            }
        }


        /* Call "R_VDC5_SyncControl" */
        {
            vdc5_sync_ctrl_t  sync_ctrl;

            sync_ctrl.res_vs_sel    = VDC5_ON;                      /* Free-running Vsync ON/OFF */
            sync_ctrl.res_vs_in_sel = VDC5_RES_VS_IN_SEL_SC0;       /* SC_RES_VS_IN_SEL */
            sync_ctrl.res_fv        = (uint16_t)GS_LCD_CHn_SIG_FV;     /* Free-running Vsync period setting */
            sync_ctrl.res_fh        = (uint16_t)GS_LCD_CHn_SIG_FH;     /* Hsync period setting */
            sync_ctrl.res_vsdly     = (uint16_t)0u;                 /* Vsync signal delay control */
            /* Full-screen enable control */
            sync_ctrl.res_f.vs      = (uint16_t)GS_LCD_CHn_DISP_VS;
            sync_ctrl.res_f.vw      = (uint16_t)GS_LCD_CHn_DISP_VW;
            sync_ctrl.res_f.hs      = (uint16_t)GS_LCD_CHn_DISP_HS;
            sync_ctrl.res_f.hw      = (uint16_t)GS_LCD_CHn_DISP_HW;
            sync_ctrl.vsync_cpmpe   = NULL;                         /* Vsync signal compensation */

            error_vdc = R_VDC5_SyncControl( self->screen_channel, &sync_ctrl );
            IF ( error_vdc != VDC5_OK )
            {
                e = E_OTHERS;
                goto fin;
            }
        }


        /* Call "R_VDC5_DisplayOutput" */
        {
            vdc5_output_t  output;
            argb8888_t     background_color_;  /* _ is for MISRA 5.6 */

            /* ->QAC 3198 */
            background_color_.Value = DUMMY_INITIAL_VALUE;  /* for avoid ARMCC warning C4017W */
            /* <-QAC 3198 */
            background_color_.u.Alpha = 0;
            background_color_.u.Red   = in_out_config->background_color.u.Red;
            background_color_.u.Green = in_out_config->background_color.u.Green;
            background_color_.u.Blue  = in_out_config->background_color.u.Blue;


            output.tcon_half        = (uint16_t)GS_LCD_CHn_TCON_HALF;  /* TCON reference timing, 1/2fH timing */
            output.tcon_offset      = (uint16_t)GS_LCD_CHn_TCON_OFFSET;/* TCON reference timing, offset Hsync signal timing */
            /* LCD TCON timing setting */
            GRAPHICS_SetLcdTconSettings( self->screen_channel, output.outctrl );
            output.outcnt_lcd_edge  = GS_LCD_CHn_OUT_EDGE;          /* Output phase control of LCD_DATA23 to LCD_DATA0 pin */
            output.out_endian_on    = VDC5_OFF;                     /* Bit endian change ON/OFF control */
            output.out_swap_on      = VDC5_OFF;                     /* B/R signal swap ON/OFF control */
            output.out_format       = GS_LCD_CHn_OUT_FORMAT;        /* LCD output format select */
            output.out_frq_sel      = VDC5_LCD_PARALLEL_CLKFRQ_1;   /* Clock frequency control */
            output.out_dir_sel      = VDC5_LCD_SERIAL_SCAN_FORWARD; /* Scan direction select */
            output.out_phase        = VDC5_LCD_SERIAL_CLKPHASE_0;   /* Clock phase adjustment */
            output.bg_color         = background_color_.Value;

            error_vdc = R_VDC5_DisplayOutput( self->screen_channel, &output );
            IF ( error_vdc != VDC5_OK )
            {
                e = E_OTHERS;
                goto fin;
            }

            if ( back_layer != NULL )
            {
                back_layer->layer_color = in_out_config->background_color;
            }
        }

        gs_window_surfaces_vdc5_is_init = true;
    }
    self->is_initialized = true;


    /* Call "R_VDC5_ReadDataControl" */
    {
        vdc5_read_t           read;
        vdc5_width_read_fb_t  read_area;
        uintptr_t             physical_address;


        for ( layer_num = 0;  layer_num <= self->layer_num_max;  layer_num += 1 )
        {
            int_fast32_t     bit_per_pixel;
            frame_buffer_t*  frame;
            window_surfaces_vdc5_layer_t*  layer = main_layer[ layer_num ];

            e= R_WINDOW_SURFACES_GetLayerFrameBuffer( self, layer_num, &frame );
            IF(e)
            {
                goto fin;
            }


            /* Read data parameter */
            read.gr_ln_off_dir  = VDC5_GR_LN_OFF_DIR_INC;   /* Line offset address direction of the frame buffer */
            read.gr_flm_sel = VDC5_GR_FLM_SEL_FLM_NUM;  /* Selects a frame buffer address setting signal */
            read.gr_imr_flm_inv = VDC5_OFF;                 /* Sets the frame buffer number for distortion correction */
            read.gr_bst_md      = VDC5_BST_MD_32BYTE;       /* Frame buffer burst transfer mode */
            e= R_OSPL_ToPhysicalAddress( frame->buffer_address[ frame->show_buffer_index ],
                                         &physical_address );
            IF(e)
            {
                goto fin;
            }
            /* ->QAC 0306 */
            read.gr_base        = (void*) physical_address; /* Frame buffer base address */
            /* <-QAC 0306 */
            read.gr_ln_off      = (uint32_t) frame->stride; /* Frame buffer line offset address */
            read.width_read_fb = NULL;             /* width of the image read from frame buffer */
            read.adj_sel = VDC5_OFF;                /* Measures to decrease the influence
		                                                   by folding pixels/lines (ON/OFF) */
            read.gr_format      = layer->vdc5_format;       /* Graphics format of the frame buffer read signal */
            read.gr_ycc_swap = VDC5_GR_YCCSWAP_Y1CRY0CB;
            /* Controls swapping of data read from buffer in the YCbCr422 format */

            bit_per_pixel = frame->byte_per_pixel * 8;
            if ( (bit_per_pixel == 32) || (frame->pixel_format == PIXEL_FORMAT_YCbCr422) )
            {
                read.gr_rdswa = VDC5_WR_RD_WRSWA_32BIT;
            }
            else if ( bit_per_pixel == 16 )
            {
                read.gr_rdswa = VDC5_WR_RD_WRSWA_32_16BIT;
            }
            else if ( bit_per_pixel <= 8 )
            {
                read.gr_rdswa = VDC5_WR_RD_WRSWA_32_16_8BIT;
            }
            /* Display area */
            read.gr_grc.hs      = (uint16_t) GS_LCD_CHn_DISP_HS;
            read.gr_grc.hw      = (uint16_t) GS_LCD_CHn_DISP_HW;
            read.gr_grc.vs      = (uint16_t) GS_LCD_CHn_DISP_VS;
            if ( GS_BUFFER_HEIGHT < GS_LCD_CHn_DISP_VW )
            {
                read.gr_grc.vw      = (uint16_t) GS_BUFFER_HEIGHT;
            }
            else
            {
                read.gr_grc.vw      = (uint16_t) GS_LCD_CHn_DISP_VW;
            }

            error_vdc = R_VDC5_ReadDataControl( self->screen_channel, layer->data_control_ID, &read );
            IF ( error_vdc != VDC5_OK )
            {
                e = E_OTHERS;
                goto fin;
            }
            layer->is_data_control = true;
        }

        if ( back_layer != NULL )
        {
            if ( self->background_format == BACKGROUND_FORMAT_SOLID_COLOR )
            {

                /* Set display area for back ground */
                /* "read.gr_base", ... are ignored */

                error_vdc = R_VDC5_ReadDataControl( self->screen_channel, back_layer->data_control_ID, &read );
                IF ( error_vdc != VDC5_OK )
                {
                    e = E_OTHERS;
                    goto fin;
                }
            }
            else
            {
                frame_buffer_t*                frame;
                window_surfaces_vdc5_layer_t*  layer = back_layer;

                e= R_WINDOW_SURFACES_GetLayerFrameBuffer( self, -1, &frame );
                IF(e)
                {
                    goto fin;
                }


                /* Read data parameter */
                read.gr_ln_off_dir  = VDC5_GR_LN_OFF_DIR_INC;   /* Line offset address direction of the frame buffer */
                read.gr_flm_sel     = VDC5_GR_FLM_SEL_FLM_NUM;  /* Selects a frame buffer address setting signal */
                read.gr_imr_flm_inv = VDC5_OFF;                 /* Sets the frame buffer number for distortion correction */
                read.gr_bst_md      = VDC5_BST_MD_32BYTE;       /* Frame buffer burst transfer mode */
                e= R_OSPL_ToPhysicalAddress( frame->buffer_address[ frame->show_buffer_index ],
                                             &physical_address );
                IF(e)
                {
                    goto fin;
                }
                /* ->QAC 0306 */
                read.gr_base        = (void*) physical_address; /* Frame buffer base address */
                /* <-QAC 0306 */
                read.gr_ln_off      = (uint32_t) frame->stride; /* Frame buffer line offset address */
                read_area.in_hw     = (uint16_t) frame->width;
                read_area.in_vw     = (uint16_t)( frame->height / 2 );
                read.width_read_fb  = &read_area;
                read.adj_sel        = VDC5_ON;
                read.gr_format      = layer->vdc5_format;       /* Graphics format of the frame buffer read signal */
                read.gr_ycc_swap    = VDC5_GR_YCCSWAP_CBY0CRY1;
                /* Controls swapping of data read from buffer in the YCbCr422 format */
                read.gr_rdswa = VDC5_WR_RD_WRSWA_16BIT;

                /* Display area */
                read.gr_grc.hs      = (uint16_t) GS_LCD_CHn_DISP_HS;
                read.gr_grc.hw      = (uint16_t) GS_LCD_CHn_DISP_HW;
                read.gr_grc.vs      = (uint16_t) GS_LCD_CHn_DISP_VS;
                if ( GS_BUFFER_HEIGHT < GS_LCD_CHn_DISP_VW )
                {
                    read.gr_grc.vw = (uint16_t) GS_BUFFER_HEIGHT;
                }
                else
                {
                    read.gr_grc.vw = (uint16_t) GS_LCD_CHn_DISP_VW;
                }

                error_vdc = R_VDC5_ReadDataControl( self->screen_channel, layer->data_control_ID, &read );
                IF ( error_vdc != VDC5_OK )
                {
                    e = E_OTHERS;
                    goto fin;
                }
            }
            back_layer->is_data_control = true;
        }
    }


    /* Call "R_VDC5_AlphaBlending" for ARGB1555 */
    for ( layer_num = 0;  layer_num <= self->layer_num_max;  layer_num += 1 )
    {
        frame_buffer_t*  frame;

        e= R_WINDOW_SURFACES_GetLayerFrameBuffer( self, layer_num, &frame );
        IF(e)
        {
            goto fin;
        }

        if ( frame->pixel_format == PIXEL_FORMAT_ARGB1555 )
        {
            static vdc5_alpha_argb1555_t  alpha_for_argb1555 = { 0x00, U8_255 };
            static vdc5_alpha_blending_t  blend = { &alpha_for_argb1555, NULL };

            error_vdc = R_VDC5_AlphaBlending( self->screen_channel,
                                              main_layer[ layer_num ]->data_control_ID,
                                              &blend );
            IF( error_vdc != VDC5_OK )
            {
                e=E_OTHERS;
                goto fin;
            }
        }
    }


    /* Call "R_V_SYNC_Initialize" */
    e= R_V_SYNC_Initialize( (int_fast32_t) self->screen_channel );
    IF ( e != 0 )
    {
        goto fin;
    }


    /* Wait for avoiding LCD filled white (LCD-KIT-B01) */
    e= R_OSPL_Delay( 80 );
    IF(e)
    {
        goto fin;
    }


    /* Set graphics display mode */
    {
        vdc5_start_t  start;
        int_fast32_t  i;

        for ( i = 0;  i < (int_fast32_t) R_COUNT_OF( self->vdc5_disp_sel_array );  i += 1 )
        {
            self->vdc5_disp_sel_array[ i ] = VDC5_DISPSEL_LOWER;  /* Reset */
        }
        self->vdc5_disp_sel_array[ VDC5_GR_TYPE_GR0 ] = VDC5_DISPSEL_BACK;

        start.gr_disp_sel = self->vdc5_disp_sel_array;
        error_vdc = R_VDC5_StartProcess( self->screen_channel, VDC5_LAYER_ID_ALL, &start );
        IF ( error_vdc != VDC5_OK )
        {
            e = E_OTHERS;
            goto fin;
        }
    }

#if  USE_LCD_BACK_LIGHT
    R_OSPL_Delay( 2*17 );  /* Synchronize to LCD */
    LCD_SetBacklight( 100 );
#endif


    e=0;
fin:
    if ( e != 0 )
    {
        if ( self != NULL )
        {
            ee= R_WINDOW_SURFACES_Finalize( self, e );
            R_UNREFERENCED_VARIABLE( ee );
        }
    }
    return  e;
}


/***********************************************************************
* Function: vdc5_init_func
*    vdc5_init_func
*
* Arguments:
*    user_num - channel
*
* Return Value:
*    None
************************************************************************/
static void  vdc5_init_func( uint32_t const  user_num )
{
    vdc5_channel_t        channel;
    struct st_cpg* const  reg_CPG = R_Get_CPG_Base();

    IF_DQ( reg_CPG == NULL )
    {
        goto fin;
    }

    channel = (vdc5_channel_t)user_num;
    if (channel == VDC5_CHANNEL_0)
    {
        /* Standby control register 9 (STBCR9)
        	b1      ------0-;  MSTP91 : 0 : Video display controller channel 0 & LVDS enable */
        R_DRV_SET_REGISTER_BIT_FIELD( &reg_CPG->STBCR9, STBCR9, MSTP91, false );
    }
    else
    {
        /* Standby control register 9 (STBCR9)
        	b1      ------0-;  MSTP91 : 0 : Video display controller channel 0 & LVDS enable
        	b0      -------0;  MSTP90 : 0 : Video display controller channel 1 enable */
        R_DRV_SET_REGISTER_BIT_FIELD( &reg_CPG->STBCR9, STBCR9, MSTP91, false );
        R_DRV_SET_REGISTER_BIT_FIELD( &reg_CPG->STBCR9, STBCR9, MSTP90, false );
    }
    GRAPHICS_SetLcdPanel(channel);

fin:
    return;
}


/***********************************************************************
* Implement: R_WINDOW_SURFACES_Finalize
************************************************************************/

static void  vdc5_quit_func( uint32_t const  user_num );

errnum_t  R_WINDOW_SURFACES_Finalize( window_surfaces_t* const  self,  errnum_t e )
{
    vdc5_error_t  error_vdc;

    IF_DQ( self == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }

    e= R_V_SYNC_Finalize( (int_fast32_t) self->screen_channel, e );


    if ( self->is_initialized )
    {
        int_fast32_t  i;

        for ( i = self->layer_num_max;  i >= -self->background_frame_count;  i -= 1 )
        {
            frame_buffer_t*  frame;
            errnum_t         ee;

            ee= R_WINDOW_SURFACES_GetLayerFrameBuffer( self, i, &frame );
            e= R_OSPL_MergeErrNum( e, ee );

            ee= R_WINDOW_SURFACES_FreeOffscreenStack( self, frame );
            e= R_OSPL_MergeErrNum( e, ee );

            frame->buffer_address[0] = NULL;
        }

        for ( i = self->layer_num_min;  i <=  self->layer_num_max;  i += 1 )
        {
            window_surfaces_vdc5_layer_t*  layer = NULL;
            /* NULL is for avoiding warning C417W of mbed cloud compiler */
            errnum_t  ee = window_surfaces_t__get_layer( self, i, &layer );

            IF_DQ ( layer == NULL )
            {
                e=E_OTHERS;    /* Same check as "ee" */
                goto fin;
            }

            if ( (ee == 0) && (layer->is_data_control) )
            {
                error_vdc = R_VDC5_StopProcess( self->screen_channel,
                                                layer->data_control_ID );
                IF ( (error_vdc != VDC5_OK) && (e == 0) )
                {
                    e = E_OTHERS;
                }
                error_vdc = R_VDC5_ReleaseDataControl( self->screen_channel,
                                                       layer->data_control_ID );
                layer->is_data_control = false;
                IF ( (error_vdc != VDC5_OK) && (e == 0) )
                {
                    e = E_OTHERS;
                }
            }
        }
        self->is_initialized = false;


#define  R_WINDOW_SURFACES_TERMINATE_VDC5  0  /* 0 or 1 */  /* For on/off test PC display safety */
#if      R_WINDOW_SURFACES_TERMINATE_VDC5
        e= R_WINDOW_SURFACES_Dispose( self, e );
#endif
    }

fin:
    return  e;
}


/***********************************************************************
* Implement: R_WINDOW_SURFACES_Dispose
************************************************************************/
errnum_t  R_WINDOW_SURFACES_Dispose( window_surfaces_t* const  self,  errnum_t e )
{
#if ! R_WINDOW_SURFACES_TERMINATE_VDC5
    if ( gs_window_surfaces_vdc5_is_init )
    {
        vdc5_error_t  error_vdc;

        error_vdc = R_VDC5_Terminate( self->screen_channel, &vdc5_quit_func,
                                      (uint32_t) self->screen_channel );
        IF ( (error_vdc != VDC5_OK) )
        {
            e= R_OSPL_MergeErrNum( e, E_OTHERS );
        }
        gs_window_surfaces_vdc5_is_init = false;
    }
#endif

    return  e;
}


/***********************************************************************
* Function: vdc5_quit_func
*    vdc5_quit_func
*
* Arguments:
*    user_num - channel
*
* Return Value:
*    None
************************************************************************/
static void  vdc5_quit_func( uint32_t const  user_num )
{
    vdc5_channel_t const  channel = (vdc5_channel_t) user_num;
    struct st_cpg* const  reg_CPG = R_Get_CPG_Base();

    IF_DQ( reg_CPG == NULL )
    {
        goto fin;
    }

    if (channel == VDC5_CHANNEL_0)
    {
        /* If LVDS should remain enabled, the following code should be removed. */
        R_DRV_SET_REGISTER_BIT_FIELD( &reg_CPG->STBCR9, STBCR9, MSTP91, true );
    }
    else
    {
        R_DRV_SET_REGISTER_BIT_FIELD( &reg_CPG->STBCR9, STBCR9, MSTP90, true );
    }

fin:
    return;
}


/***********************************************************************
* Implement: R_WINDOW_SURFACES_GetLayerFrameBuffer
************************************************************************/
errnum_t  R_WINDOW_SURFACES_GetLayerFrameBuffer( const window_surfaces_t* const  self,
        int_fast32_t const  layer_num,  frame_buffer_t** const  out_frame_buffer )
{
    errnum_t       e;
    uint_fast32_t  index;

    IF_DQ( out_frame_buffer == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }
    IF_DQ( self == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }

    index = (uint_fast32_t)( layer_num + self->background_frame_count );
    IF ( index >= R_COUNT_OF( self->frame_buffers ) )
    {
        e = E_OTHERS;
        goto fin;
    }

    *out_frame_buffer = (frame_buffer_t*) &self->frame_buffers[ index ];
    /* Cast is for const */

    e=0;
fin:
    return  e;
}


/***********************************************************************
* Implement: R_WINDOW_SURFACES_SwapBuffers
************************************************************************/
errnum_t  R_WINDOW_SURFACES_SwapBuffers( window_surfaces_t* const  self,
        int_fast32_t const  layer_num,  const void* const  null_context )
{
    errnum_t  e;
#if GS_MEASURE_GPU_LOAD
    enum {                interval = 60 };
    uint32_t              idle_start_time;
    uint32_t              idle_end_time;
    static  uint32_t      skip_count;
    static  uint32_t      previous_idle_end_time;
    r_ospl_ftimer_spec_t  ts;
#endif

    R_UNREFERENCED_VARIABLE( null_context );

    IF_DQ( self == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }


#if GS_MEASURE_GPU_LOAD
    e= R_OSPL_FTIMER_InitializeIfNot( &ts );
    IF(e)
    {
        goto fin;
    }
    idle_start_time = R_OSPL_FTIMER_Get();
#endif


    e= R_WINDOW_SURFACES_SwapBuffers_Sub( self, layer_num );
    IF(e!=0)
    {
        goto fin;
    }


    e= R_V_SYNC_Wait( self->screen_channel, 1, true );
    IF(e!=0)
    {
        goto fin;
    }


#if GS_MEASURE_GPU_LOAD
    idle_end_time = R_OSPL_FTIMER_Get();
    if ( previous_idle_end_time != 0 )    /* Skip at first */
    {
        skip_count += 1;
        if ( skip_count >= interval )
        {
            printf( "GPU %d%% in %dmsec\n",
                    ( idle_start_time - previous_idle_end_time ) * 100 /
                    ( idle_end_time   - previous_idle_end_time ),
                    R_OSPL_FTIMER_CountToTime( &ts,
                                               idle_end_time - previous_idle_end_time ) );
            skip_count = 0;

            /* __heapstats( (__heapprt) fprintf, stdout ); */

            /* R_DEBUG_BREAK(); */ /* This is for measuring CPU load */
        }
    }
    previous_idle_end_time = idle_end_time;
#endif


    e=0;
fin:
    return  e;
}


/***********************************************************************
* Implement: R_WINDOW_SURFACES_SwapBuffersStart
************************************************************************/
errnum_t  R_WINDOW_SURFACES_SwapBuffersStart( window_surfaces_t* const  self,
        int_fast32_t const  layer_num,  r_ospl_async_t* async )
{
    errnum_t  e;

    e= R_WINDOW_SURFACES_SwapBuffers_Sub( self, layer_num );
    IF(e!=0)
    {
        goto fin;
    }

    e= R_V_SYNC_WaitStart( self->screen_channel, 1, true, async );
    IF(e!=0)
    {
        goto fin;
    }

    e=0;
fin:
    return  e;
}


/***********************************************************************
* Function: R_WINDOW_SURFACES_SwapBuffers_Sub
*    Sub routine of <R_WINDOW_SURFACES_SwapBuffers>
*
* Arguments:
*    self      - window_surfaces_t
*    layer_num - layer_num
*
* Return Value:
*    Error code, 0=No error
************************************************************************/
static errnum_t  R_WINDOW_SURFACES_SwapBuffers_Sub( window_surfaces_t* const  self,
        int_fast32_t const  layer_num )
{
    errnum_t         e;
    vdc5_error_t     error_vdc;
    frame_buffer_t*  frame;
    window_surfaces_vdc5_layer_t*  layer = NULL;
    /* NULL is for avoiding warning C417W of mbed cloud compiler */


    IF_DQ( self == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }

    e= window_surfaces_t__get_layer( self, layer_num, &layer );
    IF(e!=0)
    {
        goto fin;
    }
    IF_DQ( layer == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }


    /* Swap buffer index */
    e= R_WINDOW_SURFACES_GetLayerFrameBuffer( self, layer_num, &frame );
    IF(e)
    {
        goto fin;
    }
    frame->show_buffer_index = frame->draw_buffer_index;
    frame->draw_buffer_index += 1;
    if ( frame->draw_buffer_index >= (int_t) frame->buffer_count )
    {
        frame->draw_buffer_index = 0;
    }


    /* Show the frame buffer */
    {
        vdc5_read_chg_t  config;
        uintptr_t        physical_address;

        e= R_OSPL_ToPhysicalAddress( frame->buffer_address[ frame->show_buffer_index ],
                                     &physical_address );
        IF(e!=0)
        {
            goto fin;
        }

        config.gr_base        = (void*) physical_address;
        config.width_read_fb  = NULL;
        config.gr_grc         = NULL;
        config.gr_disp_sel    = NULL;
        error_vdc = R_VDC5_ChangeReadProcess( self->screen_channel, layer->data_control_ID, &config );
        IF ( error_vdc != VDC5_OK )
        {
            e=E_OTHERS;
            goto  fin;
        }
    }


    /* Show the layer */
    {
        vdc5_gr_disp_sel_t  new_value;

        if ( layer_num == 1 )
        {
            new_value = VDC5_DISPSEL_BLEND;
        }
        else if ( layer_num == 0 )
        {
            frame_buffer_t*  frame;

            e= R_WINDOW_SURFACES_GetLayerFrameBuffer( self, layer_num, &frame );
            IF(e)
            {
                goto fin;
            }

            if ( frame->pixel_format == PIXEL_FORMAT_YUV422 )
            {
                new_value = VDC5_DISPSEL_CURRENT;
            }
            else
            {
                new_value = VDC5_DISPSEL_BLEND;
            }
        }
        else
        {
            ASSERT_D( layer_num == -1,  e=E_OTHERS; goto fin );

            if ( self->background_format == BACKGROUND_FORMAT_SOLID_COLOR )
            {
                new_value = VDC5_DISPSEL_CURRENT;
            }
            else
            {
                new_value = VDC5_DISPSEL_LOWER;  /* "*_LOWER" is "*_CURRENT" for resize */
            }
        }

        if ( self->vdc5_disp_sel_array[ layer->graphics_layer_ID ] != new_value )
        {
            vdc5_start_t  start;

            self->vdc5_disp_sel_array[ layer->graphics_layer_ID ] = new_value;

            start.gr_disp_sel = self->vdc5_disp_sel_array;
            error_vdc = R_VDC5_StartProcess( self->screen_channel, VDC5_LAYER_ID_ALL, &start );
            IF ( error_vdc != VDC5_OK )
            {
                e = E_OTHERS;
                goto fin;
            }
        }
    }

    e=0;
fin:
    return  e;
}


/***********************************************************************
* Implement: R_WINDOW_SURFACES_WaitForVSync
************************************************************************/
errnum_t  R_WINDOW_SURFACES_WaitForVSync( window_surfaces_t* const  self,
        int_fast32_t const  swap_interval,  bool_t const  is_1_v_sync_at_minimum )
{
    return  R_V_SYNC_Wait( self->screen_channel, swap_interval, is_1_v_sync_at_minimum );
}


/***********************************************************************
* Implement: R_WINDOW_SURFACES_AllocOffscreenStack
************************************************************************/
errnum_t  R_WINDOW_SURFACES_AllocOffscreenStack( window_surfaces_t* const  self,
        frame_buffer_t* const  in_out_frame_buffer )
{
    errnum_t  e;

    IF_DQ( self == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }

    e= alloc_VRAM_stack_sub(
           &self->stack_in_VRAM->stack_pointer_of_VRAM,
           self->stack_in_VRAM->over_of_VRAM,
           in_out_frame_buffer );
    IF(e!=0)
    {
        goto fin;
    }

    e=0;
fin:
    return  e;
}


/***********************************************************************
* Implement: R_WINDOW_SURFACES_FreeOffscreenStack
************************************************************************/
errnum_t  R_WINDOW_SURFACES_FreeOffscreenStack( window_surfaces_t* const  self,
        const frame_buffer_t* const  frame_buffer )
{
    errnum_t  e;

    IF_DQ( self == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }

    e= free_VRAM_stack_sub( &self->stack_in_VRAM->stack_pointer_of_VRAM,
                            frame_buffer,
                            self->stack_in_VRAM->start_of_VRAM );
    IF(e!=0)
    {
        goto fin;
    }

    e=0;
fin:
    return  e;
}


/***********************************************************************
* Class: vram_stack_t
************************************************************************/

/***********************************************************************
* Implement: R_VRAM_STACK_Initialize
************************************************************************/
errnum_t  R_VRAM_STACK_Initialize( vram_stack_t* self,  vram_stack_config_t* in_out_Config )
{
    static const bit_flags_fast32_t  must_flags = F_VRAM_STACK_ADDRESS | F_VRAM_STACK_SIZE;

    errnum_t  e;

    ASSERT_R( IS_ALL_BITS_SET( in_out_Config->flags,  must_flags ),  e=E_OTHERS; goto fin );

    self->start_of_VRAM = in_out_Config->address;
    /* ->MISRA 17.4 */ /* ->SEC R1.3.1 (1) */
    self->over_of_VRAM  = self->start_of_VRAM + in_out_Config->size;
    /* MISRA 17.4: This is a bound */
    /* <-MISRA 17.4 */ /* <-SEC R1.3.1 (1) */
    self->stack_pointer_of_VRAM = self->start_of_VRAM;

    e=0;
fin:
    return  e;
}


/***********************************************************************
* Implement: R_VRAM_STACK_AllocateMemory
************************************************************************/
errnum_t  R_VRAM_STACK_AllocateMemory( vram_stack_t*  self,
                                       void* /* byte** */  out_Address,  size_t  in_AllocatingSize )
{
    errnum_t      e;
    uint8_t*      stack_pointer;


    IF_DQ( self == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }
    IF_DQ( out_Address == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }


    stack_pointer = self->stack_pointer_of_VRAM;


    /* Check */
    /* ->MISRA 17.4 */ /* ->SEC R1.3.1 (1) */
    IF ( (stack_pointer + in_AllocatingSize) > self->over_of_VRAM )
    {
        *(byte_t**) out_Address = NULL;
        e=E_FEW_ARRAY;
        goto fin;
    }
    /* <-MISRA 17.4 */ /* <-SEC R1.3.1 (1) */

    *(byte_t**) out_Address = stack_pointer;

    /* ->MISRA 17.4 */ /* ->SEC R1.3.1 (1) */
    stack_pointer += in_AllocatingSize;  /* MISRA 17.4: Bound check is done by "self->over_of_VRAM" */
    /* <-MISRA 17.4 */ /* <-SEC R1.3.1 (1) */


    self->stack_pointer_of_VRAM = stack_pointer;

    e=0;
fin:
    return  e;
}


/***********************************************************************
* Implement: R_VRAM_STACK_FreeMemory
************************************************************************/
errnum_t  R_VRAM_STACK_FreeMemory( vram_stack_t* self,
                                   void* /* byte** */  in_out_Address,  size_t  in_AllocatedSize )
{
    errnum_t      e;
    uint8_t*      stack_pointer;


    IF_DQ( self == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }
    IF_DQ( in_out_Address == NULL )
    {
        e=E_OTHERS;
        goto fin;
    }


    stack_pointer = self->stack_pointer_of_VRAM;


    /* Check */
    /* ->MISRA 17.4 */ /* ->SEC R1.3.1 (1) */
    IF ( (stack_pointer - in_AllocatedSize) < self->start_of_VRAM )
    {
        e=E_ACCESS_DENIED;
        goto fin;
    }
    /* <-MISRA 17.4 */ /* <-SEC R1.3.1 (1) */


    stack_pointer -= in_AllocatedSize;
    *(byte_t**) in_out_Address = NULL;


    self->stack_pointer_of_VRAM = stack_pointer;

    e=0;
fin:
    return  e;
}


#endif  /* USE_LIST_WINDOW_SURFACES */
