
MEMORY {
    ROM              (rx)  : ORIGIN = 0x00000000, LENGTH = 128M
    SYSTEM_RAM       (rwx) : ORIGIN = 0x20020000, LENGTH = 0x04000
    CACHED_RAM       (rw)  : ORIGIN = 0x20024000, LENGTH = 0x0071C000
    UNCACHED_RAM     (rw)  : ORIGIN = 0x60740000, LENGTH = 0x00040000
    READ_ONLY_RAM    (rx)  : ORIGIN = 0x60780000, LENGTH = 0x00280000  /* For RAM_only */ /* 0x00240000 is needed by RGA Test */
    SDRAM            (rw)  : ORIGIN = 0x08000000, LENGTH = 0x04000000
    SDRAM2           (rw)  : ORIGIN = 0x0C000000, LENGTH = 0x04000000
}

SECTIONS
{
    .text : {

        . = 0x00000000;
        KEEP( * (VECTOR_TABLE) )    /*asm*/

        . = 0x00000200;
        * (RESET_HANDLER)           /*asm*/
        * (INIT_TTB)                /*asm*/
        * (INITCA9CACHE)            /*asm*/

        * (CODE_BASIC_SETUP)
        */peripheral_init_basic.o (.text .text.*)
        */bsc.o (.text .text.*)
        */bsc_userdef.o (.text .text.*)

        * (CONST_BASIC_SETUP)
        */peripheral_init_basic.o (.rodata .rodata.*)
        */bsc.o (.rodata .rodata.*)
        */bsc_userdef.o (.rodata .rodata.*)

        * (CODE_IRQ_HANDLER)        /*asm*/

        * (InRoot$$Sections)

        * (CODE_FPU_INIT)           /*nothing*/
        * (FPU_INIT)                /*asm*/

        * (CONST_FPU_INIT)          /*nothing*/

        * (CODE_RESET)              /*asm*/
        */initsect.o (.rodata .rodata.*)
        */port_init.o (.text .text.*)
        */stb_init.o (.text .text.*)
        */resetprg.o (.text .text.*)
        */l2_cache_init.o (.text .text.*)
        * (INIT_VBAR)               /*asm*/

        * (CONST_RESET)             /*asm*/
        */sections.o (.rodata .rodata.*)
        */port_init.o (.rodata .rodata.*)
        */stb_init.o (.rodata .rodata.*)
        */resetprg.o (.rodata .rodata.*)
        */l2_cache_init.o (.rodata .rodata.*)

        /* Start of RGA : const data*/
        /* SAMPLE_DATA_REGION */
        . = ALIGN( 0x40 );
        */BinaryImage_RZ_A1H.o (.rodata .rodata.*)
        /* TEST_DATA_REGION */
        */TestBinaryImage_RZ_A1.o (.rodata .rodata.*)
        . = ALIGN( 0x4 );
        /* End of RGA */
    } > ROM   /* Comment out for RAM_only */
/*  } > READ_ONLY_RAM  /* For RAM_only */

    .ARM.exidx : {
        PROVIDE_HIDDEN(__exidx_start = .);
        *(.ARM.exidx*)
        PROVIDE_HIDDEN(__exidx_end = .);
    } > ROM   /* Comment out for RAM_only */
/*  } > READ_ONLY_RAM  /* For RAM_only */

    .ARM.extab : {
        *(.ARM.extab* .gnu.linkonce.armextab.*)
    } > ROM   /* Comment out for RAM_only */
/*  } > READ_ONLY_RAM  /* For RAM_only */

    /*
        The vector mirror table and the irq handler must include in the .data section.
        Because the addresses on ROM are not calculated if the two put in each sections.
    */
    .data : {
        . = 0x00000000;
        __vect_load  = .;
        __vect_start = LOADADDR(.data) + ( __vect_load - ADDR(.data) );

        Image$$VECTOR_MIRROR_TABLE$$Base = .;
        KEEP( * (VECTOR_MIRROR_TABLE) )  /*asm*/

        __vect_end =  LOADADDR(.data) + ( . - ADDR(.data) );

        /********************************************************************/

        . = 0x00000100;
        __fastcode_load  = .;
        __fastcode_start = LOADADDR(.data) + ( __fastcode_load - ADDR(.data) );

        * (CODE_HANDLER_JMPTBL)
        */intc_userdef.o (.text .text.*)

        * (CONST_HANDLER_JMPTBL)
        */intc_userdef.o (.rodata .rodata.*)

        * (IRQ_FIQ_HANDLER)         /*asm*/
        * (CODE_HANDLER)
        */intc_handler.o (.text .text.*)

        * (CONST_HANDLER)
        */intc_handler.o (.rodata .rodata.*)

        * (CODE_IO_REGRW)
        */rza_io_regrw.o (.text .text.*)

        * (CONST_IO_REGRW)
        */rza_io_regrw.o (.rodata .rodata.*)

        __fastcode_end = LOADADDR(.data) + ( . - ADDR(.data) );

        /********************************************************************/

        __data_load  = .;
        __data_start = LOADADDR(.data) + ( __data_load - ADDR(.data) );

        * (DATA_HANDLER_JMPTBL)
        */intc_userdef.o (.data)

        * (DATA_BASIC_SETUP)
        */peripheral_init_basic.o (.data)
        */bsc.o (.data)
        */bsc_userdef.o (.data)

        * (DATA_HANDLER)
        */intc_handler.o (.data)

        * (DATA_FPU_INIT)           /*nothing*/

        * (DATA_RESET)
        */port_init.o (.data)
        */stb_init.o (.data)
        */resetprg.o (.data)
        */l2_cache_init.o (.data)

        * (DATA_IO_REGRW)
        */rza_io_regrw.o (.data)

        * (.data .data.*)

        . = ALIGN( 0x4 );
        __data_end = LOADADDR(.data) + ( . - ADDR(.data) );

    } > SYSTEM_RAM AT > ROM  /* Comment out for RAM_only */
/*  } > SYSTEM_RAM AT > READ_ONLY_RAM  /* For RAM_only */


    .stack : {
        . = ALIGN( 0x10 );
        end = .;                            /* referenced by syscalls.c _sbrk() for heap management */
        Image$$ARM_LIB_STACK$$ZI$$Base = .;
        . += 0x00008000;                    /* This area is shared with stack and heap. */
        Image$$ARM_LIB_STACK$$ZI$$Limit = .;

        . = ALIGN( 0x10 );
        Image$$IRQ_STACK$$ZI$$Base = .;
        . += 0x00002000;
        Image$$IRQ_STACK$$ZI$$Limit = .;

        . = ALIGN( 0x10 );
        Image$$FIQ_STACK$$ZI$$Base = .;
        . += 0x00002000;
        Image$$FIQ_STACK$$ZI$$Limit = .;

        . = ALIGN( 0x10 );
        Image$$SVC_STACK$$ZI$$Base = .;
        . += 0x00002000;
        Image$$SVC_STACK$$ZI$$Limit = .;

        . = ALIGN( 0x10 );
        Image$$ABT_STACK$$ZI$$Base = .;
        . += 0x00002000;
        Image$$ABT_STACK$$ZI$$Limit = .;

        . = ALIGN( 0x4000 );
        Image$$TTB$$ZI$$Base = .;
        . += 0x00004000;
        Image$$TTB$$ZI$$Limit = .;
    } > CACHED_RAM


    .bss : {
        __bss_start = .;

        * (BSS_HANDLER_JMPTBL)
        */intc_userdef.o (.bss)

        * (BSS_BASIC_SETUP)
        */peripheral_init_basic.o (.bss)
        */bsc.o (.bss)
        */bsc_userdef.o (.bss)

        * (BSS_HANDLER)
        */intc_handler.o (.bss)

        * (BSS_FPU_INIT)            /*nothing*/

        * (BSS_RESET)
        */port_init.o (.bss)
        */stb_init.o (.bss)
        */resetprg.o (.bss)
        */l2_cache_init.o (.bss)

        * (BSS_IO_REGRW)
        */rza_io_regrw.o (.bss)

        __bss_end = .;
    } > CACHED_RAM


    .uncached_RAM (NOLOAD) : {
        * (.WORK_SECTION1)
    } > UNCACHED_RAM


    .uncached_RAM2 : {
        __cache_operation_load  = .;
        __cache_operation_start = LOADADDR(.uncached_RAM2) + ( __cache_operation_load - ADDR(.uncached_RAM2) );

        * (L1_CACHE_OPERATION)
        * (CODE_CACHE_OPERATION)
        */cache.o (.text .text.*)

        * (CONST_CACHE_OPERATION)
        */cache.o (.rodata .rodata.*)

        * (DATA_CACHE_OPERATION)
        */cache.o (.data)

        * (BSS_CACHE_OPERATION)
        */cache.o (.bss)

        __cache_operation_end = LOADADDR(.uncached_RAM2) + ( . - ADDR(.uncached_RAM2) );
    } > UNCACHED_RAM  AT > ROM   /* Comment out for RAM_only */
/*  } > UNCACHED_RAM  AT > READ_ONLY_RAM  /* For RAM_only */


    .text2 : {
        * (.text .text.*)
        * (.rodata .rodata.*)
    } > ROM   /* Comment out for RAM_only */
/*  } > READ_ONLY_RAM  /* For RAM_only */


    .data2 : {
        __data2_load  = .;
        __data2_start = LOADADDR(.data2) + ( __data2_load - ADDR(.data2) );

        * (.data .data.*)

        __data2_end = LOADADDR(.data2) + ( . - ADDR(.data2) );
    } > CACHED_RAM AT > ROM   /* Comment out for RAM_only */
/*  } > CACHED_RAM AT > READ_ONLY_RAM  /* For RAM_only */


    .bss2 : {
        __bss2_start = .;

        * (.bss .bss.*)
        * (COMMON)

        __bss2_end = .;
    } > CACHED_RAM


    .VRAM (NOLOAD) : {
        . = ALIGN( 0x40 );
        * (VRAM)
        * (.WORK_SECTION2)

    } > CACHED_RAM


    .SDRAM (NOLOAD) : {
        * (SDRAM1)
    } > SDRAM


    .SDRAM2 (NOLOAD) : {
        __SDRAM2_start = .;
        * (SDRAM_NOLOAD)
        * (SDRAM2)
        __SDRAM2_end = .;
    } > SDRAM2


    .dummy : {
        . = 0x0100;
    } > ROM = 0x00
}
