/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* Copyright (C) 2012 - 2017 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/***********************************************************************
* File: from_Cpp.cpp
*    Callback function from C++ runtime library.
*
* - $ModuleRevision: {Masters}\ToolChain\newlib_helper\GNUC\04 $
* - $Date: 2017-07-11T15:24:23+09:00 $
************************************************************************/

extern "C" {
#include  <stdlib.h>
}
#include "use_list.h"
#ifdef  USE_LIST_OSPL
#include "r_ospl.h"
#endif


/***********************************************************************
* Define: GS_USE_INFINITE_LOOP
*    0 or 1
************************************************************************/
#define  GS_USE_INFINITE_LOOP  0


/***********************************************************************
* Function: operator new
************************************************************************/
void*  operator new( size_t  in_Size )
{
	void*  address;

	#if  R_OSPL_STACK_CHECK_CODE
		R_OSPL_CHECK_STACK_OVERFLOW();
	#endif

	address = malloc( in_Size );

	#if  GS_USE_INFINITE_LOOP
		while ( address == NULL ) {
			/* Error: Few heap memory. */
			/* abort(); */
		}
	#endif

	return  address;
		/* Return address is passed to constructor, even if it was NULL. */
}


/***********************************************************************
* Function: operator delete
************************************************************************/
void  operator delete( void*  in_Pointer )
{
	#if  R_OSPL_STACK_CHECK_CODE
		R_OSPL_CHECK_STACK_OVERFLOW();
	#endif

	if ( in_Pointer != NULL ) {
		free( in_Pointer );
	}
}


/***********************************************************************
* Function: operator new []
************************************************************************/
void*  operator new[]( size_t  in_Size )
{
	void*  address;

	#if  R_OSPL_STACK_CHECK_CODE
		R_OSPL_CHECK_STACK_OVERFLOW();
	#endif

	address = malloc( in_Size );

	#if  GS_USE_INFINITE_LOOP
		while ( address == NULL ) {
			/* Error: Few heap memory. */
			/* abort(); */
		}
	#endif

	return  address;
		/* Return addresses (+offset) are passed to each constructors, even if address was NULL. */
}


/***********************************************************************
* Function: operator delete []
************************************************************************/
void  operator delete[]( void*  in_Pointer )
{
	#if  R_OSPL_STACK_CHECK_CODE
		R_OSPL_CHECK_STACK_OVERFLOW();
	#endif

	if ( in_Pointer != NULL ) {
		free( in_Pointer );
	}
}


/***********************************************************************
* Variable: __dso_handle
*    C++ ABI - DSO Object Destruction API.
************************************************************************/
unsigned /* uint32_t*/  __dso_handle;


